/* Generated by Cython 3.2.3 */

#ifndef PY_SSIZE_T_CLEAN
#define PY_SSIZE_T_CLEAN
#endif /* PY_SSIZE_T_CLEAN */
/* InitLimitedAPI */
#if defined(Py_LIMITED_API)
  #if !defined(CYTHON_LIMITED_API)
  #define CYTHON_LIMITED_API 1
  #endif
#elif defined(CYTHON_LIMITED_API)
  #ifdef _MSC_VER
  #pragma message ("Limited API usage is enabled with 'CYTHON_LIMITED_API' but 'Py_LIMITED_API' does not define a Python target version. Consider setting 'Py_LIMITED_API' instead.")
  #else
  #warning Limited API usage is enabled with 'CYTHON_LIMITED_API' but 'Py_LIMITED_API' does not define a Python target version. Consider setting 'Py_LIMITED_API' instead.
  #endif
#endif

#include "Python.h"
#ifndef Py_PYTHON_H
    #error Python headers needed to compile C extensions, please install development version of Python.
#elif PY_VERSION_HEX < 0x03080000
    #error Cython requires Python 3.8+.
#else
#define __PYX_ABI_VERSION "3_2_3"
#define CYTHON_HEX_VERSION 0x030203F0
#define CYTHON_FUTURE_DIVISION 1
/* CModulePreamble */
#include <stddef.h>
#ifndef offsetof
  #define offsetof(type, member) ( (size_t) & ((type*)0) -> member )
#endif
#if !defined(_WIN32) && !defined(WIN32) && !defined(MS_WINDOWS)
  #ifndef __stdcall
    #define __stdcall
  #endif
  #ifndef __cdecl
    #define __cdecl
  #endif
  #ifndef __fastcall
    #define __fastcall
  #endif
#endif
#ifndef DL_IMPORT
  #define DL_IMPORT(t) t
#endif
#ifndef DL_EXPORT
  #define DL_EXPORT(t) t
#endif
#define __PYX_COMMA ,
#ifndef PY_LONG_LONG
  #define PY_LONG_LONG LONG_LONG
#endif
#ifndef Py_HUGE_VAL
  #define Py_HUGE_VAL HUGE_VAL
#endif
#define __PYX_LIMITED_VERSION_HEX PY_VERSION_HEX
#if defined(GRAALVM_PYTHON)
  /* For very preliminary testing purposes. Most variables are set the same as PyPy.
     The existence of this section does not imply that anything works or is even tested */
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #define CYTHON_COMPILING_IN_LIMITED_API 0
  #define CYTHON_COMPILING_IN_GRAAL 1
  #define CYTHON_COMPILING_IN_CPYTHON_FREETHREADING 0
  #undef CYTHON_USE_TYPE_SLOTS
  #define CYTHON_USE_TYPE_SLOTS 0
  #undef CYTHON_USE_TYPE_SPECS
  #define CYTHON_USE_TYPE_SPECS 0
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #undef CYTHON_USE_UNICODE_INTERNALS
  #define CYTHON_USE_UNICODE_INTERNALS 0
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #undef CYTHON_AVOID_BORROWED_REFS
  #define CYTHON_AVOID_BORROWED_REFS 1
  #undef CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS
  #define CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS 0
  #undef CYTHON_ASSUME_SAFE_MACROS
  #define CYTHON_ASSUME_SAFE_MACROS 0
  #undef CYTHON_ASSUME_SAFE_SIZE
  #define CYTHON_ASSUME_SAFE_SIZE 0
  #undef CYTHON_UNPACK_METHODS
  #define CYTHON_UNPACK_METHODS 0
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_GIL
  #define CYTHON_FAST_GIL 0
  #undef CYTHON_METH_FASTCALL
  #define CYTHON_METH_FASTCALL 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #ifndef CYTHON_PEP487_INIT_SUBCLASS
    #define CYTHON_PEP487_INIT_SUBCLASS 1
  #endif
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 1
  #undef CYTHON_USE_MODULE_STATE
  #define CYTHON_USE_MODULE_STATE 0
  #undef CYTHON_USE_SYS_MONITORING
  #define CYTHON_USE_SYS_MONITORING 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
  #undef CYTHON_USE_AM_SEND
  #define CYTHON_USE_AM_SEND 0
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 1
  #ifndef CYTHON_UPDATE_DESCRIPTOR_DOC
    #define CYTHON_UPDATE_DESCRIPTOR_DOC 0
  #endif
  #undef CYTHON_USE_FREELISTS
  #define CYTHON_USE_FREELISTS 0
  #undef CYTHON_IMMORTAL_CONSTANTS
  #define CYTHON_IMMORTAL_CONSTANTS 0
#elif defined(PYPY_VERSION)
  #define CYTHON_COMPILING_IN_PYPY 1
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #define CYTHON_COMPILING_IN_LIMITED_API 0
  #define CYTHON_COMPILING_IN_GRAAL 0
  #define CYTHON_COMPILING_IN_CPYTHON_FREETHREADING 0
  #undef CYTHON_USE_TYPE_SLOTS
  #define CYTHON_USE_TYPE_SLOTS 1
  #ifndef CYTHON_USE_TYPE_SPECS
    #define CYTHON_USE_TYPE_SPECS 0
  #endif
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #undef CYTHON_USE_UNICODE_INTERNALS
  #define CYTHON_USE_UNICODE_INTERNALS 0
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #undef CYTHON_AVOID_BORROWED_REFS
  #define CYTHON_AVOID_BORROWED_REFS 1
  #undef CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS
  #define CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS 1
  #undef CYTHON_ASSUME_SAFE_MACROS
  #define CYTHON_ASSUME_SAFE_MACROS 0
  #ifndef CYTHON_ASSUME_SAFE_SIZE
    #define CYTHON_ASSUME_SAFE_SIZE 1
  #endif
  #undef CYTHON_UNPACK_METHODS
  #define CYTHON_UNPACK_METHODS 0
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_GIL
  #define CYTHON_FAST_GIL 0
  #undef CYTHON_METH_FASTCALL
  #define CYTHON_METH_FASTCALL 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #ifndef CYTHON_PEP487_INIT_SUBCLASS
    #define CYTHON_PEP487_INIT_SUBCLASS 1
  #endif
  #if PY_VERSION_HEX < 0x03090000
    #undef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #elif !defined(CYTHON_PEP489_MULTI_PHASE_INIT)
    #define CYTHON_PEP489_MULTI_PHASE_INIT 1
  #endif
  #undef CYTHON_USE_MODULE_STATE
  #define CYTHON_USE_MODULE_STATE 0
  #undef CYTHON_USE_SYS_MONITORING
  #define CYTHON_USE_SYS_MONITORING 0
  #ifndef CYTHON_USE_TP_FINALIZE
    #define CYTHON_USE_TP_FINALIZE (PYPY_VERSION_NUM >= 0x07030C00)
  #endif
  #undef CYTHON_USE_AM_SEND
  #define CYTHON_USE_AM_SEND 0
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
  #ifndef CYTHON_UPDATE_DESCRIPTOR_DOC
    #define CYTHON_UPDATE_DESCRIPTOR_DOC (PYPY_VERSION_NUM >= 0x07031100)
  #endif
  #undef CYTHON_USE_FREELISTS
  #define CYTHON_USE_FREELISTS 0
  #undef CYTHON_IMMORTAL_CONSTANTS
  #define CYTHON_IMMORTAL_CONSTANTS 0
#elif defined(CYTHON_LIMITED_API)
  #ifdef Py_LIMITED_API
    #undef __PYX_LIMITED_VERSION_HEX
    #define __PYX_LIMITED_VERSION_HEX Py_LIMITED_API
  #endif
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #define CYTHON_COMPILING_IN_LIMITED_API 1
  #define CYTHON_COMPILING_IN_GRAAL 0
  #define CYTHON_COMPILING_IN_CPYTHON_FREETHREADING 0
  #undef CYTHON_USE_TYPE_SLOTS
  #define CYTHON_USE_TYPE_SLOTS 0
  #undef CYTHON_USE_TYPE_SPECS
  #define CYTHON_USE_TYPE_SPECS 1
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #undef CYTHON_USE_UNICODE_INTERNALS
  #define CYTHON_USE_UNICODE_INTERNALS 0
  #ifndef CYTHON_USE_UNICODE_WRITER
    #define CYTHON_USE_UNICODE_WRITER 0
  #endif
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS
    #define CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS 0
  #endif
  #undef CYTHON_ASSUME_SAFE_MACROS
  #define CYTHON_ASSUME_SAFE_MACROS 0
  #undef CYTHON_ASSUME_SAFE_SIZE
  #define CYTHON_ASSUME_SAFE_SIZE 0
  #undef CYTHON_UNPACK_METHODS
  #define CYTHON_UNPACK_METHODS 0
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_GIL
  #define CYTHON_FAST_GIL 0
  #undef CYTHON_METH_FASTCALL
  #define CYTHON_METH_FASTCALL (__PYX_LIMITED_VERSION_HEX >= 0x030C0000)
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #ifndef CYTHON_PEP487_INIT_SUBCLASS
    #define CYTHON_PEP487_INIT_SUBCLASS 1
  #endif
  #ifndef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT 1
  #endif
  #ifndef CYTHON_USE_MODULE_STATE
    #define CYTHON_USE_MODULE_STATE 0
  #endif
  #undef CYTHON_USE_SYS_MONITORING
  #define CYTHON_USE_SYS_MONITORING 0
  #ifndef CYTHON_USE_TP_FINALIZE
    #define CYTHON_USE_TP_FINALIZE 0
  #endif
  #ifndef CYTHON_USE_AM_SEND
    #define CYTHON_USE_AM_SEND (__PYX_LIMITED_VERSION_HEX >= 0x030A0000)
  #endif
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
  #ifndef CYTHON_UPDATE_DESCRIPTOR_DOC
    #define CYTHON_UPDATE_DESCRIPTOR_DOC 0
  #endif
  #ifndef CYTHON_USE_FREELISTS
  #define CYTHON_USE_FREELISTS 1
  #endif
  #undef CYTHON_IMMORTAL_CONSTANTS
  #define CYTHON_IMMORTAL_CONSTANTS 0
#else
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_CPYTHON 1
  #define CYTHON_COMPILING_IN_LIMITED_API 0
  #define CYTHON_COMPILING_IN_GRAAL 0
  #ifdef Py_GIL_DISABLED
    #define CYTHON_COMPILING_IN_CPYTHON_FREETHREADING 1
  #else
    #define CYTHON_COMPILING_IN_CPYTHON_FREETHREADING 0
  #endif
  #if PY_VERSION_HEX < 0x030A0000
    #undef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #elif !defined(CYTHON_USE_TYPE_SLOTS)
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #ifndef CYTHON_USE_TYPE_SPECS
    #define CYTHON_USE_TYPE_SPECS 0
  #endif
  #ifndef CYTHON_USE_PYTYPE_LOOKUP
    #define CYTHON_USE_PYTYPE_LOOKUP 1
  #endif
  #ifndef CYTHON_USE_PYLONG_INTERNALS
    #define CYTHON_USE_PYLONG_INTERNALS 1
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    #undef CYTHON_USE_PYLIST_INTERNALS
    #define CYTHON_USE_PYLIST_INTERNALS 0
  #elif !defined(CYTHON_USE_PYLIST_INTERNALS)
    #define CYTHON_USE_PYLIST_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING || PY_VERSION_HEX >= 0x030B00A2
    #undef CYTHON_USE_UNICODE_WRITER
    #define CYTHON_USE_UNICODE_WRITER 0
  #elif !defined(CYTHON_USE_UNICODE_WRITER)
    #define CYTHON_USE_UNICODE_WRITER 1
  #endif
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    #undef CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS
    #define CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS 1
  #elif !defined(CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS)
    #define CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_ASSUME_SAFE_SIZE
    #define CYTHON_ASSUME_SAFE_SIZE 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #ifndef CYTHON_FAST_THREAD_STATE
    #define CYTHON_FAST_THREAD_STATE 1
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    #undef CYTHON_FAST_GIL
    #define CYTHON_FAST_GIL 0
  #elif !defined(CYTHON_FAST_GIL)
    #define CYTHON_FAST_GIL (PY_VERSION_HEX < 0x030C00A6)
  #endif
  #ifndef CYTHON_METH_FASTCALL
    #define CYTHON_METH_FASTCALL 1
  #endif
  #ifndef CYTHON_FAST_PYCALL
    #define CYTHON_FAST_PYCALL 1
  #endif
  #ifndef CYTHON_PEP487_INIT_SUBCLASS
    #define CYTHON_PEP487_INIT_SUBCLASS 1
  #endif
  #ifndef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT 1
  #endif
  #ifndef CYTHON_USE_MODULE_STATE
    #define CYTHON_USE_MODULE_STATE 0
  #endif
  #ifndef CYTHON_USE_SYS_MONITORING
    #define CYTHON_USE_SYS_MONITORING (PY_VERSION_HEX >= 0x030d00B1)
  #endif
  #ifndef CYTHON_USE_TP_FINALIZE
    #define CYTHON_USE_TP_FINALIZE 1
  #endif
  #ifndef CYTHON_USE_AM_SEND
    #define CYTHON_USE_AM_SEND 1
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    #undef CYTHON_USE_DICT_VERSIONS
    #define CYTHON_USE_DICT_VERSIONS 0
  #elif !defined(CYTHON_USE_DICT_VERSIONS)
    #define CYTHON_USE_DICT_VERSIONS  (PY_VERSION_HEX < 0x030C00A5 && !CYTHON_USE_MODULE_STATE)
  #endif
  #ifndef CYTHON_USE_EXC_INFO_STACK
    #define CYTHON_USE_EXC_INFO_STACK 1
  #endif
  #ifndef CYTHON_UPDATE_DESCRIPTOR_DOC
    #define CYTHON_UPDATE_DESCRIPTOR_DOC 1
  #endif
  #ifndef CYTHON_USE_FREELISTS
    #define CYTHON_USE_FREELISTS (!CYTHON_COMPILING_IN_CPYTHON_FREETHREADING)
  #endif
  #if defined(CYTHON_IMMORTAL_CONSTANTS) && PY_VERSION_HEX < 0x030C0000
    #undef CYTHON_IMMORTAL_CONSTANTS
    #define CYTHON_IMMORTAL_CONSTANTS 0  // definitely won't work
  #elif !defined(CYTHON_IMMORTAL_CONSTANTS)
    #define CYTHON_IMMORTAL_CONSTANTS (PY_VERSION_HEX >= 0x030C0000 && !CYTHON_USE_MODULE_STATE && CYTHON_COMPILING_IN_CPYTHON_FREETHREADING)
  #endif
#endif
#ifndef CYTHON_COMPRESS_STRINGS
  #define CYTHON_COMPRESS_STRINGS 1
#endif
#ifndef CYTHON_FAST_PYCCALL
#define CYTHON_FAST_PYCCALL  CYTHON_FAST_PYCALL
#endif
#ifndef CYTHON_VECTORCALL
#if CYTHON_COMPILING_IN_LIMITED_API
#define CYTHON_VECTORCALL  (__PYX_LIMITED_VERSION_HEX >= 0x030C0000)
#else
#define CYTHON_VECTORCALL  (CYTHON_FAST_PYCCALL)
#endif
#endif
#if CYTHON_USE_PYLONG_INTERNALS
  #undef SHIFT
  #undef BASE
  #undef MASK
  #ifdef SIZEOF_VOID_P
    enum { __pyx_check_sizeof_voidp = 1 / (int)(SIZEOF_VOID_P == sizeof(void*)) };
  #endif
#endif
#ifndef __has_attribute
  #define __has_attribute(x) 0
#endif
#ifndef __has_cpp_attribute
  #define __has_cpp_attribute(x) 0
#endif
#ifndef CYTHON_RESTRICT
  #if defined(__GNUC__)
    #define CYTHON_RESTRICT __restrict__
  #elif defined(_MSC_VER) && _MSC_VER >= 1400
    #define CYTHON_RESTRICT __restrict
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_RESTRICT restrict
  #else
    #define CYTHON_RESTRICT
  #endif
#endif
#ifndef CYTHON_UNUSED
  #if defined(__cplusplus)
    /* for clang __has_cpp_attribute(maybe_unused) is true even before C++17
     * but leads to warnings with -pedantic, since it is a C++17 feature */
    #if ((defined(_MSVC_LANG) && _MSVC_LANG >= 201703L) || __cplusplus >= 201703L)
      #if __has_cpp_attribute(maybe_unused)
        #define CYTHON_UNUSED [[maybe_unused]]
      #endif
    #endif
  #endif
#endif
#ifndef CYTHON_UNUSED
# if defined(__GNUC__)
#   if !(defined(__cplusplus)) || (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4))
#     define CYTHON_UNUSED __attribute__ ((__unused__))
#   else
#     define CYTHON_UNUSED
#   endif
# elif defined(__ICC) || (defined(__INTEL_COMPILER) && !defined(_MSC_VER))
#   define CYTHON_UNUSED __attribute__ ((__unused__))
# else
#   define CYTHON_UNUSED
# endif
#endif
#ifndef CYTHON_UNUSED_VAR
#  if defined(__cplusplus)
     template<class T> void CYTHON_UNUSED_VAR( const T& ) { }
#  else
#    define CYTHON_UNUSED_VAR(x) (void)(x)
#  endif
#endif
#ifndef CYTHON_MAYBE_UNUSED_VAR
  #define CYTHON_MAYBE_UNUSED_VAR(x) CYTHON_UNUSED_VAR(x)
#endif
#ifndef CYTHON_NCP_UNUSED
# if CYTHON_COMPILING_IN_CPYTHON && !CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
#  define CYTHON_NCP_UNUSED
# else
#  define CYTHON_NCP_UNUSED CYTHON_UNUSED
# endif
#endif
#ifndef CYTHON_USE_CPP_STD_MOVE
  #if defined(__cplusplus) && (\
    __cplusplus >= 201103L || (defined(_MSC_VER) && _MSC_VER >= 1600))
    #define CYTHON_USE_CPP_STD_MOVE 1
  #else
    #define CYTHON_USE_CPP_STD_MOVE 0
  #endif
#endif
#define __Pyx_void_to_None(void_result) ((void)(void_result), Py_INCREF(Py_None), Py_None)
#include <stdint.h>
typedef uintptr_t  __pyx_uintptr_t;
#ifndef CYTHON_FALLTHROUGH
  #if defined(__cplusplus)
    /* for clang __has_cpp_attribute(fallthrough) is true even before C++17
     * but leads to warnings with -pedantic, since it is a C++17 feature */
    #if ((defined(_MSVC_LANG) && _MSVC_LANG >= 201703L) || __cplusplus >= 201703L)
      #if __has_cpp_attribute(fallthrough)
        #define CYTHON_FALLTHROUGH [[fallthrough]]
      #endif
    #endif
    #ifndef CYTHON_FALLTHROUGH
      #if __has_cpp_attribute(clang::fallthrough)
        #define CYTHON_FALLTHROUGH [[clang::fallthrough]]
      #elif __has_cpp_attribute(gnu::fallthrough)
        #define CYTHON_FALLTHROUGH [[gnu::fallthrough]]
      #endif
    #endif
  #endif
  #ifndef CYTHON_FALLTHROUGH
    #if __has_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH __attribute__((fallthrough))
    #else
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
  #if defined(__clang__) && defined(__apple_build_version__)
    #if __apple_build_version__ < 7000000
      #undef  CYTHON_FALLTHROUGH
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
#endif
#ifndef Py_UNREACHABLE
  #define Py_UNREACHABLE()  assert(0); abort()
#endif
#ifdef __cplusplus
  template <typename T>
  struct __PYX_IS_UNSIGNED_IMPL {static const bool value = T(0) < T(-1);};
  #define __PYX_IS_UNSIGNED(type) (__PYX_IS_UNSIGNED_IMPL<type>::value)
#else
  #define __PYX_IS_UNSIGNED(type) (((type)-1) > 0)
#endif
#if CYTHON_COMPILING_IN_PYPY == 1
  #define __PYX_NEED_TP_PRINT_SLOT  (PY_VERSION_HEX < 0x030A0000)
#else
  #define __PYX_NEED_TP_PRINT_SLOT  (PY_VERSION_HEX < 0x03090000)
#endif
#define __PYX_REINTERPRET_FUNCION(func_pointer, other_pointer) ((func_pointer)(void(*)(void))(other_pointer))

/* CppInitCode */
#ifndef __cplusplus
  #error "Cython files generated with the C++ option must be compiled with a C++ compiler."
#endif
#ifndef CYTHON_INLINE
  #if defined(__clang__)
    #define CYTHON_INLINE __inline__ __attribute__ ((__unused__))
  #else
    #define CYTHON_INLINE inline
  #endif
#endif
template<typename T>
void __Pyx_call_destructor(T& x) {
    x.~T();
}
template<typename T>
class __Pyx_FakeReference {
  public:
    __Pyx_FakeReference() : ptr(NULL) { }
    __Pyx_FakeReference(const T& ref) : ptr(const_cast<T*>(&ref)) { }
    T *operator->() { return ptr; }
    T *operator&() { return ptr; }
    operator T&() { return *ptr; }
    template<typename U> bool operator ==(const U& other) const { return *ptr == other; }
    template<typename U> bool operator !=(const U& other) const { return *ptr != other; }
    template<typename U> bool operator==(const __Pyx_FakeReference<U>& other) const { return *ptr == *other.ptr; }
    template<typename U> bool operator!=(const __Pyx_FakeReference<U>& other) const { return *ptr != *other.ptr; }
  private:
    T *ptr;
};

/* PythonCompatibility */
#define __PYX_BUILD_PY_SSIZE_T "n"
#define CYTHON_FORMAT_SSIZE_T "z"
#define __Pyx_BUILTIN_MODULE_NAME "builtins"
#define __Pyx_DefaultClassType PyType_Type
#if CYTHON_COMPILING_IN_LIMITED_API
    #ifndef CO_OPTIMIZED
    static int CO_OPTIMIZED;
    #endif
    #ifndef CO_NEWLOCALS
    static int CO_NEWLOCALS;
    #endif
    #ifndef CO_VARARGS
    static int CO_VARARGS;
    #endif
    #ifndef CO_VARKEYWORDS
    static int CO_VARKEYWORDS;
    #endif
    #ifndef CO_ASYNC_GENERATOR
    static int CO_ASYNC_GENERATOR;
    #endif
    #ifndef CO_GENERATOR
    static int CO_GENERATOR;
    #endif
    #ifndef CO_COROUTINE
    static int CO_COROUTINE;
    #endif
#else
    #ifndef CO_COROUTINE
      #define CO_COROUTINE 0x80
    #endif
    #ifndef CO_ASYNC_GENERATOR
      #define CO_ASYNC_GENERATOR 0x200
    #endif
#endif
static int __Pyx_init_co_variables(void);
#if PY_VERSION_HEX >= 0x030900A4 || defined(Py_IS_TYPE)
  #define __Pyx_IS_TYPE(ob, type) Py_IS_TYPE(ob, type)
#else
  #define __Pyx_IS_TYPE(ob, type) (((const PyObject*)ob)->ob_type == (type))
#endif
#if PY_VERSION_HEX >= 0x030A00B1 || defined(Py_Is)
  #define __Pyx_Py_Is(x, y)  Py_Is(x, y)
#else
  #define __Pyx_Py_Is(x, y) ((x) == (y))
#endif
#if PY_VERSION_HEX >= 0x030A00B1 || defined(Py_IsNone)
  #define __Pyx_Py_IsNone(ob) Py_IsNone(ob)
#else
  #define __Pyx_Py_IsNone(ob) __Pyx_Py_Is((ob), Py_None)
#endif
#if PY_VERSION_HEX >= 0x030A00B1 || defined(Py_IsTrue)
  #define __Pyx_Py_IsTrue(ob) Py_IsTrue(ob)
#else
  #define __Pyx_Py_IsTrue(ob) __Pyx_Py_Is((ob), Py_True)
#endif
#if PY_VERSION_HEX >= 0x030A00B1 || defined(Py_IsFalse)
  #define __Pyx_Py_IsFalse(ob) Py_IsFalse(ob)
#else
  #define __Pyx_Py_IsFalse(ob) __Pyx_Py_Is((ob), Py_False)
#endif
#define __Pyx_NoneAsNull(obj)  (__Pyx_Py_IsNone(obj) ? NULL : (obj))
#if PY_VERSION_HEX >= 0x030900F0 && !CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyObject_GC_IsFinalized(o) PyObject_GC_IsFinalized(o)
#else
  #define __Pyx_PyObject_GC_IsFinalized(o) _PyGC_FINALIZED(o)
#endif
#ifndef Py_TPFLAGS_CHECKTYPES
  #define Py_TPFLAGS_CHECKTYPES 0
#endif
#ifndef Py_TPFLAGS_HAVE_INDEX
  #define Py_TPFLAGS_HAVE_INDEX 0
#endif
#ifndef Py_TPFLAGS_HAVE_NEWBUFFER
  #define Py_TPFLAGS_HAVE_NEWBUFFER 0
#endif
#ifndef Py_TPFLAGS_HAVE_FINALIZE
  #define Py_TPFLAGS_HAVE_FINALIZE 0
#endif
#ifndef Py_TPFLAGS_SEQUENCE
  #define Py_TPFLAGS_SEQUENCE 0
#endif
#ifndef Py_TPFLAGS_MAPPING
  #define Py_TPFLAGS_MAPPING 0
#endif
#ifndef Py_TPFLAGS_IMMUTABLETYPE
  #define Py_TPFLAGS_IMMUTABLETYPE (1UL << 8)
#endif
#ifndef Py_TPFLAGS_DISALLOW_INSTANTIATION
  #define Py_TPFLAGS_DISALLOW_INSTANTIATION (1UL << 7)
#endif
#ifndef METH_STACKLESS
  #define METH_STACKLESS 0
#endif
#ifndef METH_FASTCALL
  #ifndef METH_FASTCALL
     #define METH_FASTCALL 0x80
  #endif
  typedef PyObject *(*__Pyx_PyCFunctionFast) (PyObject *self, PyObject *const *args, Py_ssize_t nargs);
  typedef PyObject *(*__Pyx_PyCFunctionFastWithKeywords) (PyObject *self, PyObject *const *args,
                                                          Py_ssize_t nargs, PyObject *kwnames);
#else
  #if PY_VERSION_HEX >= 0x030d00A4
  #  define __Pyx_PyCFunctionFast PyCFunctionFast
  #  define __Pyx_PyCFunctionFastWithKeywords PyCFunctionFastWithKeywords
  #else
  #  define __Pyx_PyCFunctionFast _PyCFunctionFast
  #  define __Pyx_PyCFunctionFastWithKeywords _PyCFunctionFastWithKeywords
  #endif
#endif
#if CYTHON_METH_FASTCALL
  #define __Pyx_METH_FASTCALL METH_FASTCALL
  #define __Pyx_PyCFunction_FastCall __Pyx_PyCFunctionFast
  #define __Pyx_PyCFunction_FastCallWithKeywords __Pyx_PyCFunctionFastWithKeywords
#else
  #define __Pyx_METH_FASTCALL METH_VARARGS
  #define __Pyx_PyCFunction_FastCall PyCFunction
  #define __Pyx_PyCFunction_FastCallWithKeywords PyCFunctionWithKeywords
#endif
#if CYTHON_VECTORCALL
  #define __pyx_vectorcallfunc vectorcallfunc
  #define __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET  PY_VECTORCALL_ARGUMENTS_OFFSET
  #define __Pyx_PyVectorcall_NARGS(n)  PyVectorcall_NARGS((size_t)(n))
#else
  #define __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET  0
  #define __Pyx_PyVectorcall_NARGS(n)  ((Py_ssize_t)(n))
#endif
#if PY_VERSION_HEX >= 0x030900B1
#define __Pyx_PyCFunction_CheckExact(func)  PyCFunction_CheckExact(func)
#else
#define __Pyx_PyCFunction_CheckExact(func)  PyCFunction_Check(func)
#endif
#define __Pyx_CyOrPyCFunction_Check(func)  PyCFunction_Check(func)
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_CyOrPyCFunction_GET_FUNCTION(func)  (((PyCFunctionObject*)(func))->m_ml->ml_meth)
#elif !CYTHON_COMPILING_IN_LIMITED_API
#define __Pyx_CyOrPyCFunction_GET_FUNCTION(func)  PyCFunction_GET_FUNCTION(func)
#endif
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_CyOrPyCFunction_GET_FLAGS(func)  (((PyCFunctionObject*)(func))->m_ml->ml_flags)
static CYTHON_INLINE PyObject* __Pyx_CyOrPyCFunction_GET_SELF(PyObject *func) {
    return (__Pyx_CyOrPyCFunction_GET_FLAGS(func) & METH_STATIC) ? NULL : ((PyCFunctionObject*)func)->m_self;
}
#endif
static CYTHON_INLINE int __Pyx__IsSameCFunction(PyObject *func, void (*cfunc)(void)) {
#if CYTHON_COMPILING_IN_LIMITED_API
    return PyCFunction_Check(func) && PyCFunction_GetFunction(func) == (PyCFunction) cfunc;
#else
    return PyCFunction_Check(func) && PyCFunction_GET_FUNCTION(func) == (PyCFunction) cfunc;
#endif
}
#define __Pyx_IsSameCFunction(func, cfunc)   __Pyx__IsSameCFunction(func, cfunc)
#if PY_VERSION_HEX < 0x03090000 || (CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030A0000)
  #define __Pyx_PyType_FromModuleAndSpec(m, s, b)  ((void)m, PyType_FromSpecWithBases(s, b))
  typedef PyObject *(*__Pyx_PyCMethod)(PyObject *, PyTypeObject *, PyObject *const *, size_t, PyObject *);
#else
  #define __Pyx_PyType_FromModuleAndSpec(m, s, b)  PyType_FromModuleAndSpec(m, s, b)
  #define __Pyx_PyCMethod  PyCMethod
#endif
#ifndef METH_METHOD
  #define METH_METHOD 0x200
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Malloc)
  #define PyObject_Malloc(s)   PyMem_Malloc(s)
  #define PyObject_Free(p)     PyMem_Free(p)
  #define PyObject_Realloc(p)  PyMem_Realloc(p)
#endif
#if CYTHON_COMPILING_IN_LIMITED_API
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno)
#elif CYTHON_COMPILING_IN_GRAAL && defined(GRAALPY_VERSION_NUM) && GRAALPY_VERSION_NUM > 0x19000000
  #define __Pyx_PyCode_HasFreeVars(co)  (PyCode_GetNumFree(co) > 0)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno) GraalPyFrame_SetLineNumber((frame), (lineno))
#elif CYTHON_COMPILING_IN_GRAAL
  #define __Pyx_PyCode_HasFreeVars(co)  (PyCode_GetNumFree(co) > 0)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno) _PyFrame_SetLineNumber((frame), (lineno))
#else
  #define __Pyx_PyCode_HasFreeVars(co)  (PyCode_GetNumFree(co) > 0)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno)  (frame)->f_lineno = (lineno)
#endif
#if CYTHON_COMPILING_IN_LIMITED_API
  #define __Pyx_PyThreadState_Current PyThreadState_Get()
#elif !CYTHON_FAST_THREAD_STATE
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#elif PY_VERSION_HEX >= 0x030d00A1
  #define __Pyx_PyThreadState_Current PyThreadState_GetUnchecked()
#else
  #define __Pyx_PyThreadState_Current _PyThreadState_UncheckedGet()
#endif
#if CYTHON_USE_MODULE_STATE
static CYTHON_INLINE void *__Pyx__PyModule_GetState(PyObject *op)
{
    void *result;
    result = PyModule_GetState(op);
    if (!result)
        Py_FatalError("Couldn't find the module state");
    return result;
}
#define __Pyx_PyModule_GetState(o) (__pyx_mstatetype *)__Pyx__PyModule_GetState(o)
#else
#define __Pyx_PyModule_GetState(op) ((void)op,__pyx_mstate_global)
#endif
#define __Pyx_PyObject_GetSlot(obj, name, func_ctype)  __Pyx_PyType_GetSlot(Py_TYPE((PyObject *) obj), name, func_ctype)
#define __Pyx_PyObject_TryGetSlot(obj, name, func_ctype) __Pyx_PyType_TryGetSlot(Py_TYPE(obj), name, func_ctype)
#define __Pyx_PyObject_GetSubSlot(obj, sub, name, func_ctype) __Pyx_PyType_GetSubSlot(Py_TYPE(obj), sub, name, func_ctype)
#define __Pyx_PyObject_TryGetSubSlot(obj, sub, name, func_ctype) __Pyx_PyType_TryGetSubSlot(Py_TYPE(obj), sub, name, func_ctype)
#if CYTHON_USE_TYPE_SLOTS
  #define __Pyx_PyType_GetSlot(type, name, func_ctype)  ((type)->name)
  #define __Pyx_PyType_TryGetSlot(type, name, func_ctype) __Pyx_PyType_GetSlot(type, name, func_ctype)
  #define __Pyx_PyType_GetSubSlot(type, sub, name, func_ctype) (((type)->sub) ? ((type)->sub->name) : NULL)
  #define __Pyx_PyType_TryGetSubSlot(type, sub, name, func_ctype) __Pyx_PyType_GetSubSlot(type, sub, name, func_ctype)
#else
  #define __Pyx_PyType_GetSlot(type, name, func_ctype)  ((func_ctype) PyType_GetSlot((type), Py_##name))
  #define __Pyx_PyType_TryGetSlot(type, name, func_ctype)\
    ((__PYX_LIMITED_VERSION_HEX >= 0x030A0000 ||\
     (PyType_GetFlags(type) & Py_TPFLAGS_HEAPTYPE) || __Pyx_get_runtime_version() >= 0x030A0000) ?\
     __Pyx_PyType_GetSlot(type, name, func_ctype) : NULL)
  #define __Pyx_PyType_GetSubSlot(obj, sub, name, func_ctype) __Pyx_PyType_GetSlot(obj, name, func_ctype)
  #define __Pyx_PyType_TryGetSubSlot(obj, sub, name, func_ctype) __Pyx_PyType_TryGetSlot(obj, name, func_ctype)
#endif
#if CYTHON_COMPILING_IN_CPYTHON || defined(_PyDict_NewPresized)
#define __Pyx_PyDict_NewPresized(n)  ((n <= 8) ? PyDict_New() : _PyDict_NewPresized(n))
#else
#define __Pyx_PyDict_NewPresized(n)  PyDict_New()
#endif
#define __Pyx_PyNumber_Divide(x,y)         PyNumber_TrueDivide(x,y)
#define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceTrueDivide(x,y)
#if CYTHON_COMPILING_IN_CPYTHON && CYTHON_USE_UNICODE_INTERNALS
#define __Pyx_PyDict_GetItemStrWithError(dict, name)  _PyDict_GetItem_KnownHash(dict, name, ((PyASCIIObject *) name)->hash)
static CYTHON_INLINE PyObject * __Pyx_PyDict_GetItemStr(PyObject *dict, PyObject *name) {
    PyObject *res = __Pyx_PyDict_GetItemStrWithError(dict, name);
    if (res == NULL) PyErr_Clear();
    return res;
}
#elif !CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07020000
#define __Pyx_PyDict_GetItemStrWithError  PyDict_GetItemWithError
#define __Pyx_PyDict_GetItemStr           PyDict_GetItem
#else
static CYTHON_INLINE PyObject * __Pyx_PyDict_GetItemStrWithError(PyObject *dict, PyObject *name) {
#if CYTHON_COMPILING_IN_PYPY
    return PyDict_GetItem(dict, name);
#else
    PyDictEntry *ep;
    PyDictObject *mp = (PyDictObject*) dict;
    long hash = ((PyStringObject *) name)->ob_shash;
    assert(hash != -1);
    ep = (mp->ma_lookup)(mp, name, hash);
    if (ep == NULL) {
        return NULL;
    }
    return ep->me_value;
#endif
}
#define __Pyx_PyDict_GetItemStr           PyDict_GetItem
#endif
#if CYTHON_USE_TYPE_SLOTS
  #define __Pyx_PyType_GetFlags(tp)   (((PyTypeObject *)tp)->tp_flags)
  #define __Pyx_PyType_HasFeature(type, feature)  ((__Pyx_PyType_GetFlags(type) & (feature)) != 0)
#else
  #define __Pyx_PyType_GetFlags(tp)   (PyType_GetFlags((PyTypeObject *)tp))
  #define __Pyx_PyType_HasFeature(type, feature)  PyType_HasFeature(type, feature)
#endif
#define __Pyx_PyObject_GetIterNextFunc(iterator)  __Pyx_PyObject_GetSlot(iterator, tp_iternext, iternextfunc)
#if CYTHON_USE_TYPE_SPECS
#define __Pyx_PyHeapTypeObject_GC_Del(obj)  {\
    PyTypeObject *type = Py_TYPE((PyObject*)obj);\
    assert(__Pyx_PyType_HasFeature(type, Py_TPFLAGS_HEAPTYPE));\
    PyObject_GC_Del(obj);\
    Py_DECREF(type);\
}
#else
#define __Pyx_PyHeapTypeObject_GC_Del(obj)  PyObject_GC_Del(obj)
#endif
#if CYTHON_COMPILING_IN_LIMITED_API
  #define __Pyx_PyUnicode_READY(op)       (0)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) PyUnicode_ReadChar(u, i)
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   ((void)u, 1114111U)
  #define __Pyx_PyUnicode_KIND(u)         ((void)u, (0))
  #define __Pyx_PyUnicode_DATA(u)         ((void*)u)
  #define __Pyx_PyUnicode_READ(k, d, i)   ((void)k, PyUnicode_ReadChar((PyObject*)(d), i))
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GetLength(u))
#else
  #if PY_VERSION_HEX >= 0x030C0000
    #define __Pyx_PyUnicode_READY(op)       (0)
  #else
    #define __Pyx_PyUnicode_READY(op)       (likely(PyUnicode_IS_READY(op)) ?\
                                                0 : _PyUnicode_Ready((PyObject *)(op)))
  #endif
  #define __Pyx_PyUnicode_READ_CHAR(u, i) PyUnicode_READ_CHAR(u, i)
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   PyUnicode_MAX_CHAR_VALUE(u)
  #define __Pyx_PyUnicode_KIND(u)         ((int)PyUnicode_KIND(u))
  #define __Pyx_PyUnicode_DATA(u)         PyUnicode_DATA(u)
  #define __Pyx_PyUnicode_READ(k, d, i)   PyUnicode_READ(k, d, i)
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  PyUnicode_WRITE(k, d, i, (Py_UCS4) ch)
  #if PY_VERSION_HEX >= 0x030C0000
    #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GET_LENGTH(u))
  #else
    #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x03090000
    #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : ((PyCompactUnicodeObject *)(u))->wstr_length))
    #else
    #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : PyUnicode_GET_SIZE(u)))
    #endif
  #endif
#endif
#if CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyUnicode_Concat(a, b)      PyNumber_Add(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  PyNumber_Add(a, b)
#else
  #define __Pyx_PyUnicode_Concat(a, b)      PyUnicode_Concat(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  ((unlikely((a) == Py_None) || unlikely((b) == Py_None)) ?\
      PyNumber_Add(a, b) : __Pyx_PyUnicode_Concat(a, b))
#endif
#if CYTHON_COMPILING_IN_PYPY
  #if !defined(PyUnicode_DecodeUnicodeEscape)
    #define PyUnicode_DecodeUnicodeEscape(s, size, errors)  PyUnicode_Decode(s, size, "unicode_escape", errors)
  #endif
  #if !defined(PyUnicode_Contains)
    #define PyUnicode_Contains(u, s)  PySequence_Contains(u, s)
  #endif
  #if !defined(PyByteArray_Check)
    #define PyByteArray_Check(obj)  PyObject_TypeCheck(obj, &PyByteArray_Type)
  #endif
  #if !defined(PyObject_Format)
    #define PyObject_Format(obj, fmt)  PyObject_CallMethod(obj, "__format__", "O", fmt)
  #endif
#endif
#define __Pyx_PyUnicode_FormatSafe(a, b)  ((unlikely((a) == Py_None || (PyUnicode_Check(b) && !PyUnicode_CheckExact(b)))) ? PyNumber_Remainder(a, b) : PyUnicode_Format(a, b))
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  #define __Pyx_PySequence_ListKeepNew(obj)\
    (likely(PyList_CheckExact(obj) && PyUnstable_Object_IsUniquelyReferenced(obj)) ? __Pyx_NewRef(obj) : PySequence_List(obj))
#elif CYTHON_COMPILING_IN_CPYTHON
  #define __Pyx_PySequence_ListKeepNew(obj)\
    (likely(PyList_CheckExact(obj) && Py_REFCNT(obj) == 1) ? __Pyx_NewRef(obj) : PySequence_List(obj))
#else
  #define __Pyx_PySequence_ListKeepNew(obj)  PySequence_List(obj)
#endif
#ifndef PySet_CheckExact
  #define PySet_CheckExact(obj)        __Pyx_IS_TYPE(obj, &PySet_Type)
#endif
#if PY_VERSION_HEX >= 0x030900A4
  #define __Pyx_SET_REFCNT(obj, refcnt) Py_SET_REFCNT(obj, refcnt)
  #define __Pyx_SET_SIZE(obj, size) Py_SET_SIZE(obj, size)
#else
  #define __Pyx_SET_REFCNT(obj, refcnt) Py_REFCNT(obj) = (refcnt)
  #define __Pyx_SET_SIZE(obj, size) Py_SIZE(obj) = (size)
#endif
enum __Pyx_ReferenceSharing {
  __Pyx_ReferenceSharing_DefinitelyUnique, // We created it so we know it's unshared - no need to check
  __Pyx_ReferenceSharing_OwnStrongReference,
  __Pyx_ReferenceSharing_FunctionArgument,
  __Pyx_ReferenceSharing_SharedReference, // Never trust it to be unshared because it's a global or similar
};
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING && PY_VERSION_HEX >= 0x030E0000
#define __Pyx_IS_UNIQUELY_REFERENCED(o, sharing)\
    (sharing == __Pyx_ReferenceSharing_DefinitelyUnique ? 1 :\
      (sharing == __Pyx_ReferenceSharing_FunctionArgument ? PyUnstable_Object_IsUniqueReferencedTemporary(o) :\
      (sharing == __Pyx_ReferenceSharing_OwnStrongReference ? PyUnstable_Object_IsUniquelyReferenced(o) : 0)))
#elif (CYTHON_COMPILING_IN_CPYTHON && !CYTHON_COMPILING_IN_CPYTHON_FREETHREADING) || CYTHON_COMPILING_IN_LIMITED_API
#define __Pyx_IS_UNIQUELY_REFERENCED(o, sharing) (((void)sharing), Py_REFCNT(o) == 1)
#else
#define __Pyx_IS_UNIQUELY_REFERENCED(o, sharing) (((void)o), ((void)sharing), 0)
#endif
#if CYTHON_AVOID_BORROWED_REFS || CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS
  #if __PYX_LIMITED_VERSION_HEX >= 0x030d0000
    #define __Pyx_PyList_GetItemRef(o, i) PyList_GetItemRef(o, i)
  #elif CYTHON_COMPILING_IN_LIMITED_API || !CYTHON_ASSUME_SAFE_MACROS
    #define __Pyx_PyList_GetItemRef(o, i) (likely((i) >= 0) ? PySequence_GetItem(o, i) : (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL))
  #else
    #define __Pyx_PyList_GetItemRef(o, i) PySequence_ITEM(o, i)
  #endif
#elif CYTHON_COMPILING_IN_LIMITED_API || !CYTHON_ASSUME_SAFE_MACROS
  #if __PYX_LIMITED_VERSION_HEX >= 0x030d0000
    #define __Pyx_PyList_GetItemRef(o, i) PyList_GetItemRef(o, i)
  #else
    #define __Pyx_PyList_GetItemRef(o, i) __Pyx_XNewRef(PyList_GetItem(o, i))
  #endif
#else
  #define __Pyx_PyList_GetItemRef(o, i) __Pyx_NewRef(PyList_GET_ITEM(o, i))
#endif
#if CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS && !CYTHON_COMPILING_IN_LIMITED_API && CYTHON_ASSUME_SAFE_MACROS
  #define __Pyx_PyList_GetItemRefFast(o, i, unsafe_shared) (__Pyx_IS_UNIQUELY_REFERENCED(o, unsafe_shared) ?\
    __Pyx_NewRef(PyList_GET_ITEM(o, i)) : __Pyx_PyList_GetItemRef(o, i))
#else
  #define __Pyx_PyList_GetItemRefFast(o, i, unsafe_shared) __Pyx_PyList_GetItemRef(o, i)
#endif
#if __PYX_LIMITED_VERSION_HEX >= 0x030d0000
#define __Pyx_PyDict_GetItemRef(dict, key, result) PyDict_GetItemRef(dict, key, result)
#elif CYTHON_AVOID_BORROWED_REFS || CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS
static CYTHON_INLINE int __Pyx_PyDict_GetItemRef(PyObject *dict, PyObject *key, PyObject **result) {
  *result = PyObject_GetItem(dict, key);
  if (*result == NULL) {
    if (PyErr_ExceptionMatches(PyExc_KeyError)) {
      PyErr_Clear();
      return 0;
    }
    return -1;
  }
  return 1;
}
#else
static CYTHON_INLINE int __Pyx_PyDict_GetItemRef(PyObject *dict, PyObject *key, PyObject **result) {
  *result = PyDict_GetItemWithError(dict, key);
  if (*result == NULL) {
    return PyErr_Occurred() ? -1 : 0;
  }
  Py_INCREF(*result);
  return 1;
}
#endif
#if defined(CYTHON_DEBUG_VISIT_CONST) && CYTHON_DEBUG_VISIT_CONST
  #define __Pyx_VISIT_CONST(obj)  Py_VISIT(obj)
#else
  #define __Pyx_VISIT_CONST(obj)
#endif
#if CYTHON_ASSUME_SAFE_MACROS
  #define __Pyx_PySequence_ITEM(o, i) PySequence_ITEM(o, i)
  #define __Pyx_PySequence_SIZE(seq)  Py_SIZE(seq)
  #define __Pyx_PyTuple_SET_ITEM(o, i, v) (PyTuple_SET_ITEM(o, i, v), (0))
  #define __Pyx_PyTuple_GET_ITEM(o, i) PyTuple_GET_ITEM(o, i)
  #define __Pyx_PyList_SET_ITEM(o, i, v) (PyList_SET_ITEM(o, i, v), (0))
  #define __Pyx_PyList_GET_ITEM(o, i) PyList_GET_ITEM(o, i)
#else
  #define __Pyx_PySequence_ITEM(o, i) PySequence_GetItem(o, i)
  #define __Pyx_PySequence_SIZE(seq)  PySequence_Size(seq)
  #define __Pyx_PyTuple_SET_ITEM(o, i, v) PyTuple_SetItem(o, i, v)
  #define __Pyx_PyTuple_GET_ITEM(o, i) PyTuple_GetItem(o, i)
  #define __Pyx_PyList_SET_ITEM(o, i, v) PyList_SetItem(o, i, v)
  #define __Pyx_PyList_GET_ITEM(o, i) PyList_GetItem(o, i)
#endif
#if CYTHON_ASSUME_SAFE_SIZE
  #define __Pyx_PyTuple_GET_SIZE(o) PyTuple_GET_SIZE(o)
  #define __Pyx_PyList_GET_SIZE(o) PyList_GET_SIZE(o)
  #define __Pyx_PySet_GET_SIZE(o) PySet_GET_SIZE(o)
  #define __Pyx_PyBytes_GET_SIZE(o) PyBytes_GET_SIZE(o)
  #define __Pyx_PyByteArray_GET_SIZE(o) PyByteArray_GET_SIZE(o)
  #define __Pyx_PyUnicode_GET_LENGTH(o) PyUnicode_GET_LENGTH(o)
#else
  #define __Pyx_PyTuple_GET_SIZE(o) PyTuple_Size(o)
  #define __Pyx_PyList_GET_SIZE(o) PyList_Size(o)
  #define __Pyx_PySet_GET_SIZE(o) PySet_Size(o)
  #define __Pyx_PyBytes_GET_SIZE(o) PyBytes_Size(o)
  #define __Pyx_PyByteArray_GET_SIZE(o) PyByteArray_Size(o)
  #define __Pyx_PyUnicode_GET_LENGTH(o) PyUnicode_GetLength(o)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyUnicode_InternFromString)
  #define PyUnicode_InternFromString(s) PyUnicode_FromString(s)
#endif
#define __Pyx_PyLong_FromHash_t PyLong_FromSsize_t
#define __Pyx_PyLong_AsHash_t   __Pyx_PyIndex_AsSsize_t
#if __PYX_LIMITED_VERSION_HEX >= 0x030A0000
    #define __Pyx_PySendResult PySendResult
#else
    typedef enum {
        PYGEN_RETURN = 0,
        PYGEN_ERROR = -1,
        PYGEN_NEXT = 1,
    } __Pyx_PySendResult;
#endif
#if CYTHON_COMPILING_IN_LIMITED_API || PY_VERSION_HEX < 0x030A00A3
  typedef __Pyx_PySendResult (*__Pyx_pyiter_sendfunc)(PyObject *iter, PyObject *value, PyObject **result);
#else
  #define __Pyx_pyiter_sendfunc sendfunc
#endif
#if !CYTHON_USE_AM_SEND
#define __PYX_HAS_PY_AM_SEND 0
#elif __PYX_LIMITED_VERSION_HEX >= 0x030A0000
#define __PYX_HAS_PY_AM_SEND 1
#else
#define __PYX_HAS_PY_AM_SEND 2  // our own backported implementation
#endif
#if __PYX_HAS_PY_AM_SEND < 2
    #define __Pyx_PyAsyncMethodsStruct PyAsyncMethods
#else
    typedef struct {
        unaryfunc am_await;
        unaryfunc am_aiter;
        unaryfunc am_anext;
        __Pyx_pyiter_sendfunc am_send;
    } __Pyx_PyAsyncMethodsStruct;
    #define __Pyx_SlotTpAsAsync(s) ((PyAsyncMethods*)(s))
#endif
#if CYTHON_USE_AM_SEND && PY_VERSION_HEX < 0x030A00F0
    #define __Pyx_TPFLAGS_HAVE_AM_SEND (1UL << 21)
#else
    #define __Pyx_TPFLAGS_HAVE_AM_SEND (0)
#endif
#if PY_VERSION_HEX >= 0x03090000
#define __Pyx_PyInterpreterState_Get() PyInterpreterState_Get()
#else
#define __Pyx_PyInterpreterState_Get() PyThreadState_Get()->interp
#endif
#if CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030A0000
#ifdef __cplusplus
extern "C"
#endif
PyAPI_FUNC(void *) PyMem_Calloc(size_t nelem, size_t elsize);
#endif
#if CYTHON_COMPILING_IN_LIMITED_API
static int __Pyx_init_co_variable(PyObject *inspect, const char* name, int *write_to) {
    int value;
    PyObject *py_value = PyObject_GetAttrString(inspect, name);
    if (!py_value) return 0;
    value = (int) PyLong_AsLong(py_value);
    Py_DECREF(py_value);
    *write_to = value;
    return value != -1 || !PyErr_Occurred();
}
static int __Pyx_init_co_variables(void) {
    PyObject *inspect;
    int result;
    inspect = PyImport_ImportModule("inspect");
    result =
#if !defined(CO_OPTIMIZED)
        __Pyx_init_co_variable(inspect, "CO_OPTIMIZED", &CO_OPTIMIZED) &&
#endif
#if !defined(CO_NEWLOCALS)
        __Pyx_init_co_variable(inspect, "CO_NEWLOCALS", &CO_NEWLOCALS) &&
#endif
#if !defined(CO_VARARGS)
        __Pyx_init_co_variable(inspect, "CO_VARARGS", &CO_VARARGS) &&
#endif
#if !defined(CO_VARKEYWORDS)
        __Pyx_init_co_variable(inspect, "CO_VARKEYWORDS", &CO_VARKEYWORDS) &&
#endif
#if !defined(CO_ASYNC_GENERATOR)
        __Pyx_init_co_variable(inspect, "CO_ASYNC_GENERATOR", &CO_ASYNC_GENERATOR) &&
#endif
#if !defined(CO_GENERATOR)
        __Pyx_init_co_variable(inspect, "CO_GENERATOR", &CO_GENERATOR) &&
#endif
#if !defined(CO_COROUTINE)
        __Pyx_init_co_variable(inspect, "CO_COROUTINE", &CO_COROUTINE) &&
#endif
        1;
    Py_DECREF(inspect);
    return result ? 0 : -1;
}
#else
static int __Pyx_init_co_variables(void) {
    return 0;  // It's a limited API-only feature
}
#endif

/* MathInitCode */
#if defined(_WIN32) || defined(WIN32) || defined(MS_WINDOWS)
  #ifndef _USE_MATH_DEFINES
    #define _USE_MATH_DEFINES
  #endif
#endif
#include <math.h>
#if defined(__CYGWIN__) && defined(_LDBL_EQ_DBL)
#define __Pyx_truncl trunc
#else
#define __Pyx_truncl truncl
#endif

#ifndef CYTHON_CLINE_IN_TRACEBACK_RUNTIME
#define CYTHON_CLINE_IN_TRACEBACK_RUNTIME 0
#endif
#ifndef CYTHON_CLINE_IN_TRACEBACK
#define CYTHON_CLINE_IN_TRACEBACK CYTHON_CLINE_IN_TRACEBACK_RUNTIME
#endif
#if CYTHON_CLINE_IN_TRACEBACK
#define __PYX_MARK_ERR_POS(f_index, lineno)  { __pyx_filename = __pyx_f[f_index]; (void) __pyx_filename; __pyx_lineno = lineno; (void) __pyx_lineno; __pyx_clineno = __LINE__; (void) __pyx_clineno; }
#else
#define __PYX_MARK_ERR_POS(f_index, lineno)  { __pyx_filename = __pyx_f[f_index]; (void) __pyx_filename; __pyx_lineno = lineno; (void) __pyx_lineno; (void) __pyx_clineno; }
#endif
#define __PYX_ERR(f_index, lineno, Ln_error) \
    { __PYX_MARK_ERR_POS(f_index, lineno) goto Ln_error; }

#ifdef CYTHON_EXTERN_C
    #undef __PYX_EXTERN_C
    #define __PYX_EXTERN_C CYTHON_EXTERN_C
#elif defined(__PYX_EXTERN_C)
    #ifdef _MSC_VER
    #pragma message ("Please do not define the '__PYX_EXTERN_C' macro externally. Use 'CYTHON_EXTERN_C' instead.")
    #else
    #warning Please do not define the '__PYX_EXTERN_C' macro externally. Use 'CYTHON_EXTERN_C' instead.
    #endif
#else
    #define __PYX_EXTERN_C extern "C++"
#endif

#define __PYX_HAVE__plyvel___plyvel
#define __PYX_HAVE_API__plyvel___plyvel
/* Early includes */
#include <string.h>
#include <stdio.h>

    #if __PYX_LIMITED_VERSION_HEX < 0x030d0000
    static CYTHON_INLINE PyObject *
    __Pyx_CAPI_PyList_GetItemRef(PyObject *list, Py_ssize_t index)
    {
        PyObject *item = PyList_GetItem(list, index);
        Py_XINCREF(item);
        return item;
    }
    #else
    #define __Pyx_CAPI_PyList_GetItemRef PyList_GetItemRef
    #endif

    #if CYTHON_COMPILING_IN_LIMITED_API || PY_VERSION_HEX < 0x030d0000
    static CYTHON_INLINE int
    __Pyx_CAPI_PyList_Extend(PyObject *list, PyObject *iterable)
    {
        return PyList_SetSlice(list, PY_SSIZE_T_MAX, PY_SSIZE_T_MAX, iterable);
    }

    static CYTHON_INLINE int
    __Pyx_CAPI_PyList_Clear(PyObject *list)
    {
        return PyList_SetSlice(list, 0, PY_SSIZE_T_MAX, NULL);
    }
    #else
    #define __Pyx_CAPI_PyList_Extend PyList_Extend
    #define __Pyx_CAPI_PyList_Clear PyList_Clear
    #endif
    

    #if PY_MAJOR_VERSION >= 3
      #define __Pyx_PyFloat_FromString(obj)  PyFloat_FromString(obj)
    #else
      #define __Pyx_PyFloat_FromString(obj)  PyFloat_FromString(obj, NULL)
    #endif
    
#include <stddef.h>

    #if PY_MAJOR_VERSION <= 2
    #define PyDict_GetItemWithError _PyDict_GetItemWithError
    #endif

    #if __PYX_LIMITED_VERSION_HEX < 0x030d0000
    static CYTHON_INLINE int
    __Pyx_CAPI_PyDict_GetItemStringRef(PyObject *mp, const char *key, PyObject **result)
    {
        int res;
        PyObject *key_obj = PyUnicode_FromString(key);
        if (key_obj == NULL) {
            *result = NULL;
            return -1;
        }
        res = __Pyx_PyDict_GetItemRef(mp, key_obj, result);
        Py_DECREF(key_obj);
        return res;
    }
    #else
    #define __Pyx_CAPI_PyDict_GetItemStringRef PyDict_GetItemStringRef
    #endif
    #if PY_VERSION_HEX < 0x030d0000 || (CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030F0000)
    static CYTHON_INLINE int
    __Pyx_CAPI_PyDict_SetDefaultRef(PyObject *d, PyObject *key, PyObject *default_value,
                        PyObject **result)
    {
        PyObject *value;
        if (__Pyx_PyDict_GetItemRef(d, key, &value) < 0) {
            // get error
            if (result) {
                *result = NULL;
            }
            return -1;
        }
        if (value != NULL) {
            // present
            if (result) {
                *result = value;
            }
            else {
                Py_DECREF(value);
            }
            return 1;
        }

        // missing: set the item
        if (PyDict_SetItem(d, key, default_value) < 0) {
            // set error
            if (result) {
                *result = NULL;
            }
            return -1;
        }
        if (result) {
            Py_INCREF(default_value);
            *result = default_value;
        }
        return 0;
    }
    #else
    #define __Pyx_CAPI_PyDict_SetDefaultRef PyDict_SetDefaultRef
    #endif
    

    #if PY_VERSION_HEX < 0x030d0000
    static CYTHON_INLINE int __Pyx_PyWeakref_GetRef(PyObject *ref, PyObject **pobj)
    {
        PyObject *obj = PyWeakref_GetObject(ref);
        if (obj == NULL) {
            // SystemError if ref is NULL
            *pobj = NULL;
            return -1;
        }
        if (obj == Py_None) {
            *pobj = NULL;
            return 0;
        }
        Py_INCREF(obj);
        *pobj = obj;
        return 1;
    }
    #else
    #define __Pyx_PyWeakref_GetRef PyWeakref_GetRef
    #endif
    
#include "pythread.h"

    #if (CYTHON_COMPILING_IN_PYPY && PYPY_VERSION_NUM < 0x07030600) && !defined(PyContextVar_Get)
    #define PyContextVar_Get(var, d, v)         ((d) ?             ((void)(var), Py_INCREF(d), (v)[0] = (d), 0) :             ((v)[0] = NULL, 0)         )
    #endif
    
#include <stdint.h>
#include <stdlib.h>
#include <string_view>
#include "ios"
#include "new"
#include "stdexcept"
#include "typeinfo"
#include <string>
#include "leveldb/db.h"
#include "leveldb/status.h"
#include "leveldb/options.h"
#include "leveldb/slice.h"
#include "leveldb/write_batch.h"
#include "leveldb/iterator.h"
#include "leveldb/comparator.h"
#include "leveldb/filter_policy.h"
#include "leveldb/cache.h"
#include "comparator.h"
#ifdef _OPENMP
#include <omp.h>
#endif /* _OPENMP */

#if defined(PYREX_WITHOUT_ASSERTIONS) && !defined(CYTHON_WITHOUT_ASSERTIONS)
#define CYTHON_WITHOUT_ASSERTIONS
#endif

#ifdef CYTHON_FREETHREADING_COMPATIBLE
#if CYTHON_FREETHREADING_COMPATIBLE
#define __Pyx_FREETHREADING_COMPATIBLE Py_MOD_GIL_NOT_USED
#else
#define __Pyx_FREETHREADING_COMPATIBLE Py_MOD_GIL_USED
#endif
#else
#define __Pyx_FREETHREADING_COMPATIBLE Py_MOD_GIL_USED
#endif
#define __PYX_DEFAULT_STRING_ENCODING_IS_ASCII 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_UTF8 0
#define __PYX_DEFAULT_STRING_ENCODING ""
#define __Pyx_PyObject_FromString __Pyx_PyBytes_FromString
#define __Pyx_PyObject_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#define __Pyx_uchar_cast(c) ((unsigned char)c)
#define __Pyx_long_cast(x) ((long)x)
#define __Pyx_fits_Py_ssize_t(v, type, is_signed)  (\
    (sizeof(type) < sizeof(Py_ssize_t))  ||\
    (sizeof(type) > sizeof(Py_ssize_t) &&\
          likely(v < (type)PY_SSIZE_T_MAX ||\
                 v == (type)PY_SSIZE_T_MAX)  &&\
          (!is_signed || likely(v > (type)PY_SSIZE_T_MIN ||\
                                v == (type)PY_SSIZE_T_MIN)))  ||\
    (sizeof(type) == sizeof(Py_ssize_t) &&\
          (is_signed || likely(v < (type)PY_SSIZE_T_MAX ||\
                               v == (type)PY_SSIZE_T_MAX)))  )
static CYTHON_INLINE int __Pyx_is_valid_index(Py_ssize_t i, Py_ssize_t limit) {
    return (size_t) i < (size_t) limit;
}
#if defined (__cplusplus) && __cplusplus >= 201103L
    #include <cstdlib>
    #define __Pyx_sst_abs(value) std::abs(value)
#elif SIZEOF_INT >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) abs(value)
#elif SIZEOF_LONG >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) labs(value)
#elif defined (_MSC_VER)
    #define __Pyx_sst_abs(value) ((Py_ssize_t)_abs64(value))
#elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define __Pyx_sst_abs(value) llabs(value)
#elif defined (__GNUC__)
    #define __Pyx_sst_abs(value) __builtin_llabs(value)
#else
    #define __Pyx_sst_abs(value) ((value<0) ? -value : value)
#endif
static CYTHON_INLINE Py_ssize_t __Pyx_ssize_strlen(const char *s);
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject*);
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject*, Py_ssize_t* length);
static CYTHON_INLINE PyObject* __Pyx_PyByteArray_FromString(const char*);
#define __Pyx_PyByteArray_FromStringAndSize(s, l) PyByteArray_FromStringAndSize((const char*)s, l)
#define __Pyx_PyBytes_FromString        PyBytes_FromString
#define __Pyx_PyBytes_FromStringAndSize PyBytes_FromStringAndSize
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char*);
#if CYTHON_ASSUME_SAFE_MACROS
    #define __Pyx_PyBytes_AsWritableString(s)     ((char*) PyBytes_AS_STRING(s))
    #define __Pyx_PyBytes_AsWritableSString(s)    ((signed char*) PyBytes_AS_STRING(s))
    #define __Pyx_PyBytes_AsWritableUString(s)    ((unsigned char*) PyBytes_AS_STRING(s))
    #define __Pyx_PyBytes_AsString(s)     ((const char*) PyBytes_AS_STRING(s))
    #define __Pyx_PyBytes_AsSString(s)    ((const signed char*) PyBytes_AS_STRING(s))
    #define __Pyx_PyBytes_AsUString(s)    ((const unsigned char*) PyBytes_AS_STRING(s))
    #define __Pyx_PyByteArray_AsString(s) PyByteArray_AS_STRING(s)
#else
    #define __Pyx_PyBytes_AsWritableString(s)     ((char*) PyBytes_AsString(s))
    #define __Pyx_PyBytes_AsWritableSString(s)    ((signed char*) PyBytes_AsString(s))
    #define __Pyx_PyBytes_AsWritableUString(s)    ((unsigned char*) PyBytes_AsString(s))
    #define __Pyx_PyBytes_AsString(s)     ((const char*) PyBytes_AsString(s))
    #define __Pyx_PyBytes_AsSString(s)    ((const signed char*) PyBytes_AsString(s))
    #define __Pyx_PyBytes_AsUString(s)    ((const unsigned char*) PyBytes_AsString(s))
    #define __Pyx_PyByteArray_AsString(s) PyByteArray_AsString(s)
#endif
#define __Pyx_PyObject_AsWritableString(s)    ((char*)(__pyx_uintptr_t) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableSString(s)    ((signed char*)(__pyx_uintptr_t) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableUString(s)    ((unsigned char*)(__pyx_uintptr_t) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsSString(s)    ((const signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsUString(s)    ((const unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_FromCString(s)  __Pyx_PyObject_FromString((const char*)s)
#define __Pyx_PyBytes_FromCString(s)   __Pyx_PyBytes_FromString((const char*)s)
#define __Pyx_PyByteArray_FromCString(s)   __Pyx_PyByteArray_FromString((const char*)s)
#define __Pyx_PyUnicode_FromCString(s) __Pyx_PyUnicode_FromString((const char*)s)
#define __Pyx_PyUnicode_FromOrdinal(o)       PyUnicode_FromOrdinal((int)o)
#define __Pyx_PyUnicode_AsUnicode            PyUnicode_AsUnicode
static CYTHON_INLINE PyObject *__Pyx_NewRef(PyObject *obj) {
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030a0000 || defined(Py_NewRef)
    return Py_NewRef(obj);
#else
    Py_INCREF(obj);
    return obj;
#endif
}
static CYTHON_INLINE PyObject *__Pyx_XNewRef(PyObject *obj) {
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030a0000 || defined(Py_XNewRef)
    return Py_XNewRef(obj);
#else
    Py_XINCREF(obj);
    return obj;
#endif
}
static CYTHON_INLINE PyObject *__Pyx_Owned_Py_None(int b);
static CYTHON_INLINE PyObject * __Pyx_PyBool_FromLong(long b);
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject*);
static CYTHON_INLINE int __Pyx_PyObject_IsTrueAndDecref(PyObject*);
static CYTHON_INLINE PyObject* __Pyx_PyNumber_Long(PyObject* x);
#define __Pyx_PySequence_Tuple(obj)\
    (likely(PyTuple_CheckExact(obj)) ? __Pyx_NewRef(obj) : PySequence_Tuple(obj))
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject*);
static CYTHON_INLINE PyObject * __Pyx_PyLong_FromSize_t(size_t);
static CYTHON_INLINE Py_hash_t __Pyx_PyIndex_AsHash_t(PyObject*);
#if CYTHON_ASSUME_SAFE_MACROS
#define __Pyx_PyFloat_AsDouble(x) (PyFloat_CheckExact(x) ? PyFloat_AS_DOUBLE(x) : PyFloat_AsDouble(x))
#define __Pyx_PyFloat_AS_DOUBLE(x) PyFloat_AS_DOUBLE(x)
#else
#define __Pyx_PyFloat_AsDouble(x) PyFloat_AsDouble(x)
#define __Pyx_PyFloat_AS_DOUBLE(x) PyFloat_AsDouble(x)
#endif
#define __Pyx_PyFloat_AsFloat(x) ((float) __Pyx_PyFloat_AsDouble(x))
#define __Pyx_PyNumber_Int(x) (PyLong_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Long(x))
#if CYTHON_USE_PYLONG_INTERNALS
  #if PY_VERSION_HEX >= 0x030C00A7
  #ifndef _PyLong_SIGN_MASK
    #define _PyLong_SIGN_MASK 3
  #endif
  #ifndef _PyLong_NON_SIZE_BITS
    #define _PyLong_NON_SIZE_BITS 3
  #endif
  #define __Pyx_PyLong_Sign(x)  (((PyLongObject*)x)->long_value.lv_tag & _PyLong_SIGN_MASK)
  #define __Pyx_PyLong_IsNeg(x)  ((__Pyx_PyLong_Sign(x) & 2) != 0)
  #define __Pyx_PyLong_IsNonNeg(x)  (!__Pyx_PyLong_IsNeg(x))
  #define __Pyx_PyLong_IsZero(x)  (__Pyx_PyLong_Sign(x) & 1)
  #define __Pyx_PyLong_IsPos(x)  (__Pyx_PyLong_Sign(x) == 0)
  #define __Pyx_PyLong_CompactValueUnsigned(x)  (__Pyx_PyLong_Digits(x)[0])
  #define __Pyx_PyLong_DigitCount(x)  ((Py_ssize_t) (((PyLongObject*)x)->long_value.lv_tag >> _PyLong_NON_SIZE_BITS))
  #define __Pyx_PyLong_SignedDigitCount(x)\
        ((1 - (Py_ssize_t) __Pyx_PyLong_Sign(x)) * __Pyx_PyLong_DigitCount(x))
  #if defined(PyUnstable_Long_IsCompact) && defined(PyUnstable_Long_CompactValue)
    #define __Pyx_PyLong_IsCompact(x)     PyUnstable_Long_IsCompact((PyLongObject*) x)
    #define __Pyx_PyLong_CompactValue(x)  PyUnstable_Long_CompactValue((PyLongObject*) x)
  #else
    #define __Pyx_PyLong_IsCompact(x)     (((PyLongObject*)x)->long_value.lv_tag < (2 << _PyLong_NON_SIZE_BITS))
    #define __Pyx_PyLong_CompactValue(x)  ((1 - (Py_ssize_t) __Pyx_PyLong_Sign(x)) * (Py_ssize_t) __Pyx_PyLong_Digits(x)[0])
  #endif
  typedef Py_ssize_t  __Pyx_compact_pylong;
  typedef size_t  __Pyx_compact_upylong;
  #else
  #define __Pyx_PyLong_IsNeg(x)  (Py_SIZE(x) < 0)
  #define __Pyx_PyLong_IsNonNeg(x)  (Py_SIZE(x) >= 0)
  #define __Pyx_PyLong_IsZero(x)  (Py_SIZE(x) == 0)
  #define __Pyx_PyLong_IsPos(x)  (Py_SIZE(x) > 0)
  #define __Pyx_PyLong_CompactValueUnsigned(x)  ((Py_SIZE(x) == 0) ? 0 : __Pyx_PyLong_Digits(x)[0])
  #define __Pyx_PyLong_DigitCount(x)  __Pyx_sst_abs(Py_SIZE(x))
  #define __Pyx_PyLong_SignedDigitCount(x)  Py_SIZE(x)
  #define __Pyx_PyLong_IsCompact(x)  (Py_SIZE(x) == 0 || Py_SIZE(x) == 1 || Py_SIZE(x) == -1)
  #define __Pyx_PyLong_CompactValue(x)\
        ((Py_SIZE(x) == 0) ? (sdigit) 0 : ((Py_SIZE(x) < 0) ? -(sdigit)__Pyx_PyLong_Digits(x)[0] : (sdigit)__Pyx_PyLong_Digits(x)[0]))
  typedef sdigit  __Pyx_compact_pylong;
  typedef digit  __Pyx_compact_upylong;
  #endif
  #if PY_VERSION_HEX >= 0x030C00A5
  #define __Pyx_PyLong_Digits(x)  (((PyLongObject*)x)->long_value.ob_digit)
  #else
  #define __Pyx_PyLong_Digits(x)  (((PyLongObject*)x)->ob_digit)
  #endif
#endif
#if __PYX_DEFAULT_STRING_ENCODING_IS_UTF8
  #define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_DecodeUTF8(c_str, size, NULL)
#elif __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
  #define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_DecodeASCII(c_str, size, NULL)
#else
  #define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_Decode(c_str, size, __PYX_DEFAULT_STRING_ENCODING, NULL)
#endif


/* Test for GCC > 2.95 */
#if defined(__GNUC__)     && (__GNUC__ > 2 || (__GNUC__ == 2 && (__GNUC_MINOR__ > 95)))
  #define likely(x)   __builtin_expect(!!(x), 1)
  #define unlikely(x) __builtin_expect(!!(x), 0)
#else /* !__GNUC__ or GCC < 2.95 */
  #define likely(x)   (x)
  #define unlikely(x) (x)
#endif /* __GNUC__ */
/* PretendToInitialize */
#ifdef __cplusplus
#if __cplusplus > 201103L
#include <type_traits>
#endif
template <typename T>
static void __Pyx_pretend_to_initialize(T* ptr) {
#if __cplusplus > 201103L
    if ((std::is_trivially_default_constructible<T>::value))
#endif
        *ptr = T();
    (void)ptr;
}
#else
static CYTHON_INLINE void __Pyx_pretend_to_initialize(void* ptr) { (void)ptr; }
#endif


#if !CYTHON_USE_MODULE_STATE
static PyObject *__pyx_m = NULL;
#endif
static int __pyx_lineno;
static int __pyx_clineno = 0;
static const char * const __pyx_cfilenm = __FILE__;
static const char *__pyx_filename;

/* #### Code section: filename_table ### */

static const char* const __pyx_f[] = {
  "plyvel/_plyvel.pyx",
  "<stringsource>",
  "cpython/contextvars.pxd",
  "cpython/type.pxd",
  "cpython/bool.pxd",
  "cpython/complex.pxd",
};
/* #### Code section: utility_code_proto_before_types ### */
/* NoFastGil.proto */
#define __Pyx_PyGILState_Ensure PyGILState_Ensure
#define __Pyx_PyGILState_Release PyGILState_Release
#define __Pyx_FastGIL_Remember()
#define __Pyx_FastGIL_Forget()
#define __Pyx_FastGilFuncInit()

/* CriticalSectionsDefinition.proto (used by CriticalSections) */
#if !CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
#define __Pyx_PyCriticalSection void*
#define __Pyx_PyCriticalSection2 void*
#define __Pyx_PyCriticalSection_End(cs)
#define __Pyx_PyCriticalSection2_End(cs)
#else
#define __Pyx_PyCriticalSection PyCriticalSection
#define __Pyx_PyCriticalSection2 PyCriticalSection2
#define __Pyx_PyCriticalSection_End PyCriticalSection_End
#define __Pyx_PyCriticalSection2_End PyCriticalSection2_End
#endif

/* CriticalSections.proto (used by GetItemIntByteArray) */
#if !CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
#define __Pyx_PyCriticalSection_Begin(cs, arg) (void)(cs)
#define __Pyx_PyCriticalSection2_Begin(cs, arg1, arg2) (void)(cs)
#else
#define __Pyx_PyCriticalSection_Begin PyCriticalSection_Begin
#define __Pyx_PyCriticalSection2_Begin PyCriticalSection2_Begin
#endif
#if PY_VERSION_HEX < 0x030d0000 || CYTHON_COMPILING_IN_LIMITED_API
#define __Pyx_BEGIN_CRITICAL_SECTION(o) {
#define __Pyx_END_CRITICAL_SECTION() }
#else
#define __Pyx_BEGIN_CRITICAL_SECTION Py_BEGIN_CRITICAL_SECTION
#define __Pyx_END_CRITICAL_SECTION Py_END_CRITICAL_SECTION
#endif

/* Atomics.proto (used by UnpackUnboundCMethod) */
#include <pythread.h>
#ifndef CYTHON_ATOMICS
    #define CYTHON_ATOMICS 1
#endif
#define __PYX_CYTHON_ATOMICS_ENABLED() CYTHON_ATOMICS
#define __PYX_GET_CYTHON_COMPILING_IN_CPYTHON_FREETHREADING() CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
#define __pyx_atomic_int_type int
#define __pyx_nonatomic_int_type int
#if CYTHON_ATOMICS && (defined(__STDC_VERSION__) &&\
                        (__STDC_VERSION__ >= 201112L) &&\
                        !defined(__STDC_NO_ATOMICS__))
    #include <stdatomic.h>
#elif CYTHON_ATOMICS && (defined(__cplusplus) && (\
                    (__cplusplus >= 201103L) ||\
                    (defined(_MSC_VER) && _MSC_VER >= 1700)))
    #include <atomic>
#endif
#if CYTHON_ATOMICS && (defined(__STDC_VERSION__) &&\
                        (__STDC_VERSION__ >= 201112L) &&\
                        !defined(__STDC_NO_ATOMICS__) &&\
                       ATOMIC_INT_LOCK_FREE == 2)
    #undef __pyx_atomic_int_type
    #define __pyx_atomic_int_type atomic_int
    #define __pyx_atomic_ptr_type atomic_uintptr_t
    #define __pyx_nonatomic_ptr_type uintptr_t
    #define __pyx_atomic_incr_relaxed(value) atomic_fetch_add_explicit(value, 1, memory_order_relaxed)
    #define __pyx_atomic_incr_acq_rel(value) atomic_fetch_add_explicit(value, 1, memory_order_acq_rel)
    #define __pyx_atomic_decr_acq_rel(value) atomic_fetch_sub_explicit(value, 1, memory_order_acq_rel)
    #define __pyx_atomic_sub(value, arg) atomic_fetch_sub(value, arg)
    #define __pyx_atomic_int_cmp_exchange(value, expected, desired) atomic_compare_exchange_strong(value, expected, desired)
    #define __pyx_atomic_load(value) atomic_load(value)
    #define __pyx_atomic_store(value, new_value) atomic_store(value, new_value)
    #define __pyx_atomic_pointer_load_relaxed(value) atomic_load_explicit(value, memory_order_relaxed)
    #define __pyx_atomic_pointer_load_acquire(value) atomic_load_explicit(value, memory_order_acquire)
    #define __pyx_atomic_pointer_exchange(value, new_value) atomic_exchange(value, (__pyx_nonatomic_ptr_type)new_value)
    #define __pyx_atomic_pointer_cmp_exchange(value, expected, desired) atomic_compare_exchange_strong(value, expected, desired)
    #if defined(__PYX_DEBUG_ATOMICS) && defined(_MSC_VER)
        #pragma message ("Using standard C atomics")
    #elif defined(__PYX_DEBUG_ATOMICS)
        #warning "Using standard C atomics"
    #endif
#elif CYTHON_ATOMICS && (defined(__cplusplus) && (\
                    (__cplusplus >= 201103L) ||\
\
                    (defined(_MSC_VER) && _MSC_VER >= 1700)) &&\
                    ATOMIC_INT_LOCK_FREE == 2)
    #undef __pyx_atomic_int_type
    #define __pyx_atomic_int_type std::atomic_int
    #define __pyx_atomic_ptr_type std::atomic_uintptr_t
    #define __pyx_nonatomic_ptr_type uintptr_t
    #define __pyx_atomic_incr_relaxed(value) std::atomic_fetch_add_explicit(value, 1, std::memory_order_relaxed)
    #define __pyx_atomic_incr_acq_rel(value) std::atomic_fetch_add_explicit(value, 1, std::memory_order_acq_rel)
    #define __pyx_atomic_decr_acq_rel(value) std::atomic_fetch_sub_explicit(value, 1, std::memory_order_acq_rel)
    #define __pyx_atomic_sub(value, arg) std::atomic_fetch_sub(value, arg)
    #define __pyx_atomic_int_cmp_exchange(value, expected, desired) std::atomic_compare_exchange_strong(value, expected, desired)
    #define __pyx_atomic_load(value) std::atomic_load(value)
    #define __pyx_atomic_store(value, new_value) std::atomic_store(value, new_value)
    #define __pyx_atomic_pointer_load_relaxed(value) std::atomic_load_explicit(value, std::memory_order_relaxed)
    #define __pyx_atomic_pointer_load_acquire(value) std::atomic_load_explicit(value, std::memory_order_acquire)
    #define __pyx_atomic_pointer_exchange(value, new_value) std::atomic_exchange(value, (__pyx_nonatomic_ptr_type)new_value)
    #define __pyx_atomic_pointer_cmp_exchange(value, expected, desired) std::atomic_compare_exchange_strong(value, expected, desired)
    #if defined(__PYX_DEBUG_ATOMICS) && defined(_MSC_VER)
        #pragma message ("Using standard C++ atomics")
    #elif defined(__PYX_DEBUG_ATOMICS)
        #warning "Using standard C++ atomics"
    #endif
#elif CYTHON_ATOMICS && (__GNUC__ >= 5 || (__GNUC__ == 4 &&\
                    (__GNUC_MINOR__ > 1 ||\
                    (__GNUC_MINOR__ == 1 && __GNUC_PATCHLEVEL__ >= 2))))
    #define __pyx_atomic_ptr_type void*
    #define __pyx_nonatomic_ptr_type void*
    #define __pyx_atomic_incr_relaxed(value) __sync_fetch_and_add(value, 1)
    #define __pyx_atomic_incr_acq_rel(value) __sync_fetch_and_add(value, 1)
    #define __pyx_atomic_decr_acq_rel(value) __sync_fetch_and_sub(value, 1)
    #define __pyx_atomic_sub(value, arg) __sync_fetch_and_sub(value, arg)
    static CYTHON_INLINE int __pyx_atomic_int_cmp_exchange(__pyx_atomic_int_type* value, __pyx_nonatomic_int_type* expected, __pyx_nonatomic_int_type desired) {
        __pyx_nonatomic_int_type old = __sync_val_compare_and_swap(value, *expected, desired);
        int result = old == *expected;
        *expected = old;
        return result;
    }
    #define __pyx_atomic_load(value) __sync_fetch_and_add(value, 0)
    #define __pyx_atomic_store(value, new_value) __sync_lock_test_and_set(value, new_value)
    #define __pyx_atomic_pointer_load_relaxed(value) __sync_fetch_and_add(value, 0)
    #define __pyx_atomic_pointer_load_acquire(value) __sync_fetch_and_add(value, 0)
    #define __pyx_atomic_pointer_exchange(value, new_value) __sync_lock_test_and_set(value, (__pyx_atomic_ptr_type)new_value)
    static CYTHON_INLINE int __pyx_atomic_pointer_cmp_exchange(__pyx_atomic_ptr_type* value, __pyx_nonatomic_ptr_type* expected, __pyx_nonatomic_ptr_type desired) {
        __pyx_nonatomic_ptr_type old = __sync_val_compare_and_swap(value, *expected, desired);
        int result = old == *expected;
        *expected = old;
        return result;
    }
    #ifdef __PYX_DEBUG_ATOMICS
        #warning "Using GNU atomics"
    #endif
#elif CYTHON_ATOMICS && defined(_MSC_VER)
    #include <intrin.h>
    #undef __pyx_atomic_int_type
    #define __pyx_atomic_int_type long
    #define __pyx_atomic_ptr_type void*
    #undef __pyx_nonatomic_int_type
    #define __pyx_nonatomic_int_type long
    #define __pyx_nonatomic_ptr_type void*
    #pragma intrinsic (_InterlockedExchangeAdd, _InterlockedExchange, _InterlockedCompareExchange, _InterlockedCompareExchangePointer, _InterlockedExchangePointer)
    #define __pyx_atomic_incr_relaxed(value) _InterlockedExchangeAdd(value, 1)
    #define __pyx_atomic_incr_acq_rel(value) _InterlockedExchangeAdd(value, 1)
    #define __pyx_atomic_decr_acq_rel(value) _InterlockedExchangeAdd(value, -1)
    #define __pyx_atomic_sub(value, arg) _InterlockedExchangeAdd(value, -arg)
    static CYTHON_INLINE int __pyx_atomic_int_cmp_exchange(__pyx_atomic_int_type* value, __pyx_nonatomic_int_type* expected, __pyx_nonatomic_int_type desired) {
        __pyx_nonatomic_int_type old = _InterlockedCompareExchange(value, desired, *expected);
        int result = old == *expected;
        *expected = old;
        return result;
    }
    #define __pyx_atomic_load(value) _InterlockedExchangeAdd(value, 0)
    #define __pyx_atomic_store(value, new_value) _InterlockedExchange(value, new_value)
    #define __pyx_atomic_pointer_load_relaxed(value) *(void * volatile *)value
    #define __pyx_atomic_pointer_load_acquire(value) _InterlockedCompareExchangePointer(value, 0, 0)
    #define __pyx_atomic_pointer_exchange(value, new_value) _InterlockedExchangePointer(value, (__pyx_atomic_ptr_type)new_value)
    static CYTHON_INLINE int __pyx_atomic_pointer_cmp_exchange(__pyx_atomic_ptr_type* value, __pyx_nonatomic_ptr_type* expected, __pyx_nonatomic_ptr_type desired) {
        __pyx_atomic_ptr_type old = _InterlockedCompareExchangePointer(value, desired, *expected);
        int result = old == *expected;
        *expected = old;
        return result;
    }
    #ifdef __PYX_DEBUG_ATOMICS
        #pragma message ("Using MSVC atomics")
    #endif
#else
    #undef CYTHON_ATOMICS
    #define CYTHON_ATOMICS 0
    #ifdef __PYX_DEBUG_ATOMICS
        #warning "Not using atomics"
    #endif
#endif

/* IncludeStructmemberH.proto (used by FixUpExtensionType) */
#include <structmember.h>

/* ForceInitThreads.proto */
#ifndef __PYX_FORCE_INIT_THREADS
  #define __PYX_FORCE_INIT_THREADS 0
#endif

/* #### Code section: numeric_typedefs ### */
/* #### Code section: complex_type_declarations ### */
/* #### Code section: type_declarations ### */

/*--- Type declarations ---*/
struct __pyx_obj_6plyvel_7_plyvel_DB;
struct __pyx_obj_6plyvel_7_plyvel_PrefixedDB;
struct __pyx_obj_6plyvel_7_plyvel_WriteBatch;
struct __pyx_obj_6plyvel_7_plyvel_BaseIterator;
struct __pyx_obj_6plyvel_7_plyvel_Iterator;
struct __pyx_obj_6plyvel_7_plyvel_RawIterator;
struct __pyx_obj_6plyvel_7_plyvel_Snapshot;
struct __pyx_obj_6plyvel_7_plyvel___pyx_scope_struct____init__;
struct __pyx_opt_args_7cpython_11contextvars_get_value;
struct __pyx_opt_args_7cpython_11contextvars_get_value_no_default;

/* "cpython/contextvars.pxd":116
 * 
 * @_cython.c_compile_guard("!CYTHON_COMPILING_IN_LIMITED_API")
 * cdef inline object get_value(var, default_value=None):             # <<<<<<<<<<<<<<
 *     """Return a new reference to the value of the context variable,
 *     or the default value of the context variable,
*/
struct __pyx_opt_args_7cpython_11contextvars_get_value {
  int __pyx_n;
  PyObject *default_value;
};

/* "cpython/contextvars.pxd":134
 * 
 * @_cython.c_compile_guard("!CYTHON_COMPILING_IN_LIMITED_API")
 * cdef inline object get_value_no_default(var, default_value=None):             # <<<<<<<<<<<<<<
 *     """Return a new reference to the value of the context variable,
 *     or the provided default value if no such value was found.
*/
struct __pyx_opt_args_7cpython_11contextvars_get_value_no_default {
  int __pyx_n;
  PyObject *default_value;
};

/* "plyvel/_plyvel.pyx":660
 * #
 * 
 * cdef enum IteratorState:             # <<<<<<<<<<<<<<
 *     BEFORE_START
 *     AFTER_STOP
*/
enum __pyx_t_6plyvel_7_plyvel_IteratorState {
  __pyx_e_6plyvel_7_plyvel_BEFORE_START,
  __pyx_e_6plyvel_7_plyvel_AFTER_STOP,
  __pyx_e_6plyvel_7_plyvel_IN_BETWEEN,
  __pyx_e_6plyvel_7_plyvel_IN_BETWEEN_ALREADY_POSITIONED
};

/* "plyvel/_plyvel.pyx":667
 * 
 * 
 * cdef enum IteratorDirection:             # <<<<<<<<<<<<<<
 *     FORWARD
 *     REVERSE
*/
enum __pyx_t_6plyvel_7_plyvel_IteratorDirection {
  __pyx_e_6plyvel_7_plyvel_FORWARD,
  __pyx_e_6plyvel_7_plyvel_REVERSE
};

/* "plyvel/_plyvel.pyx":220
 * #
 * 
 * @cython.final             # <<<<<<<<<<<<<<
 * cdef class DB:
 *     cdef leveldb.DB* _db
*/
struct __pyx_obj_6plyvel_7_plyvel_DB {
  PyObject_HEAD
  struct __pyx_vtabstruct_6plyvel_7_plyvel_DB *__pyx_vtab;
  leveldb::DB *_db;
  leveldb::Options options;
  PyObject *name;
  PyObject *lock;
  PyObject *iterators;
};


/* "plyvel/_plyvel.pyx":459
 * 
 * 
 * cdef class PrefixedDB:             # <<<<<<<<<<<<<<
 *     cdef readonly DB db
 *     cdef readonly bytes prefix
*/
struct __pyx_obj_6plyvel_7_plyvel_PrefixedDB {
  PyObject_HEAD
  struct __pyx_obj_6plyvel_7_plyvel_DB *db;
  PyObject *prefix;
};


/* "plyvel/_plyvel.pyx":558
 * #
 * 
 * @cython.final             # <<<<<<<<<<<<<<
 * cdef class WriteBatch:
 *     cdef leveldb.WriteBatch* _write_batch
*/
struct __pyx_obj_6plyvel_7_plyvel_WriteBatch {
  PyObject_HEAD
  leveldb::WriteBatch *_write_batch;
  leveldb::WriteOptions write_options;
  struct __pyx_obj_6plyvel_7_plyvel_DB *db;
  PyObject *prefix;
  bool transaction;
};


/* "plyvel/_plyvel.pyx":672
 * 
 * 
 * cdef class BaseIterator:             # <<<<<<<<<<<<<<
 *     cdef DB db
 *     cdef leveldb.Iterator* _iter
*/
struct __pyx_obj_6plyvel_7_plyvel_BaseIterator {
  PyObject_HEAD
  struct __pyx_vtabstruct_6plyvel_7_plyvel_BaseIterator *__pyx_vtab;
  struct __pyx_obj_6plyvel_7_plyvel_DB *db;
  leveldb::Iterator *_iter;
  PyObject *__weakref__;
};


/* "plyvel/_plyvel.pyx":720
 * 
 * 
 * @cython.final             # <<<<<<<<<<<<<<
 * cdef class Iterator(BaseIterator):
 *     cdef IteratorDirection direction
*/
struct __pyx_obj_6plyvel_7_plyvel_Iterator {
  struct __pyx_obj_6plyvel_7_plyvel_BaseIterator __pyx_base;
  enum __pyx_t_6plyvel_7_plyvel_IteratorDirection direction;
  enum __pyx_t_6plyvel_7_plyvel_IteratorState state;
  leveldb::Comparator *comparator;
  PyObject *start;
  PyObject *stop;
  leveldb::Slice start_slice;
  leveldb::Slice stop_slice;
  bool include_start;
  bool include_stop;
  bool include_key;
  bool include_value;
  PyObject *db_prefix;
  size_t db_prefix_len;
};


/* "plyvel/_plyvel.pyx":1027
 * 
 * 
 * @cython.final             # <<<<<<<<<<<<<<
 * cdef class RawIterator(BaseIterator):
 *     def valid(self):
*/
struct __pyx_obj_6plyvel_7_plyvel_RawIterator {
  struct __pyx_obj_6plyvel_7_plyvel_BaseIterator __pyx_base;
};


/* "plyvel/_plyvel.pyx":1117
 * #
 * 
 * @cython.final             # <<<<<<<<<<<<<<
 * cdef class Snapshot:
 *     cdef leveldb.Snapshot* _snapshot
*/
struct __pyx_obj_6plyvel_7_plyvel_Snapshot {
  PyObject_HEAD
  struct __pyx_vtabstruct_6plyvel_7_plyvel_Snapshot *__pyx_vtab;
  leveldb::Snapshot *_snapshot;
  struct __pyx_obj_6plyvel_7_plyvel_DB *db;
  PyObject *prefix;
};


/* "plyvel/_plyvel.pyx":680
 *     cdef object __weakref__
 * 
 *     def __init__(self, DB db, bool verify_checksums, bool fill_cache,             # <<<<<<<<<<<<<<
 *                  Snapshot snapshot):
 *         if db._db is NULL:
*/
struct __pyx_obj_6plyvel_7_plyvel___pyx_scope_struct____init__ {
  PyObject_HEAD
  PyObject *__pyx_v_iterator_id;
  PyObject *__pyx_v_ref_dict;
};



/* "plyvel/_plyvel.pyx":220
 * #
 * 
 * @cython.final             # <<<<<<<<<<<<<<
 * cdef class DB:
 *     cdef leveldb.DB* _db
*/

struct __pyx_vtabstruct_6plyvel_7_plyvel_DB {
  PyObject *(*close)(struct __pyx_obj_6plyvel_7_plyvel_DB *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_6plyvel_7_plyvel_DB *__pyx_vtabptr_6plyvel_7_plyvel_DB;
static PyObject *__pyx_f_6plyvel_7_plyvel_2DB_close(struct __pyx_obj_6plyvel_7_plyvel_DB *, int __pyx_skip_dispatch);


/* "plyvel/_plyvel.pyx":672
 * 
 * 
 * cdef class BaseIterator:             # <<<<<<<<<<<<<<
 *     cdef DB db
 *     cdef leveldb.Iterator* _iter
*/

struct __pyx_vtabstruct_6plyvel_7_plyvel_BaseIterator {
  PyObject *(*close)(struct __pyx_obj_6plyvel_7_plyvel_BaseIterator *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_6plyvel_7_plyvel_BaseIterator *__pyx_vtabptr_6plyvel_7_plyvel_BaseIterator;


/* "plyvel/_plyvel.pyx":720
 * 
 * 
 * @cython.final             # <<<<<<<<<<<<<<
 * cdef class Iterator(BaseIterator):
 *     cdef IteratorDirection direction
*/

struct __pyx_vtabstruct_6plyvel_7_plyvel_Iterator {
  struct __pyx_vtabstruct_6plyvel_7_plyvel_BaseIterator __pyx_base;
  PyObject *(*current)(struct __pyx_obj_6plyvel_7_plyvel_Iterator *);
  PyObject *(*real_next)(struct __pyx_obj_6plyvel_7_plyvel_Iterator *);
  PyObject *(*real_prev)(struct __pyx_obj_6plyvel_7_plyvel_Iterator *);
};
static struct __pyx_vtabstruct_6plyvel_7_plyvel_Iterator *__pyx_vtabptr_6plyvel_7_plyvel_Iterator;
static PyObject *__pyx_f_6plyvel_7_plyvel_8Iterator_current(struct __pyx_obj_6plyvel_7_plyvel_Iterator *);
static PyObject *__pyx_f_6plyvel_7_plyvel_8Iterator_real_next(struct __pyx_obj_6plyvel_7_plyvel_Iterator *);
static PyObject *__pyx_f_6plyvel_7_plyvel_8Iterator_real_prev(struct __pyx_obj_6plyvel_7_plyvel_Iterator *);


/* "plyvel/_plyvel.pyx":1027
 * 
 * 
 * @cython.final             # <<<<<<<<<<<<<<
 * cdef class RawIterator(BaseIterator):
 *     def valid(self):
*/

struct __pyx_vtabstruct_6plyvel_7_plyvel_RawIterator {
  struct __pyx_vtabstruct_6plyvel_7_plyvel_BaseIterator __pyx_base;
  PyObject *(*key)(struct __pyx_obj_6plyvel_7_plyvel_RawIterator *, int __pyx_skip_dispatch);
  PyObject *(*value)(struct __pyx_obj_6plyvel_7_plyvel_RawIterator *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_6plyvel_7_plyvel_RawIterator *__pyx_vtabptr_6plyvel_7_plyvel_RawIterator;
static PyObject *__pyx_f_6plyvel_7_plyvel_11RawIterator_key(struct __pyx_obj_6plyvel_7_plyvel_RawIterator *, int __pyx_skip_dispatch);
static PyObject *__pyx_f_6plyvel_7_plyvel_11RawIterator_value(struct __pyx_obj_6plyvel_7_plyvel_RawIterator *, int __pyx_skip_dispatch);


/* "plyvel/_plyvel.pyx":1117
 * #
 * 
 * @cython.final             # <<<<<<<<<<<<<<
 * cdef class Snapshot:
 *     cdef leveldb.Snapshot* _snapshot
*/

struct __pyx_vtabstruct_6plyvel_7_plyvel_Snapshot {
  PyObject *(*close)(struct __pyx_obj_6plyvel_7_plyvel_Snapshot *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_6plyvel_7_plyvel_Snapshot *__pyx_vtabptr_6plyvel_7_plyvel_Snapshot;
static PyObject *__pyx_f_6plyvel_7_plyvel_8Snapshot_close(struct __pyx_obj_6plyvel_7_plyvel_Snapshot *, int __pyx_skip_dispatch);
/* #### Code section: utility_code_proto ### */

/* --- Runtime support code (head) --- */
/* Refnanny.proto */
#ifndef CYTHON_REFNANNY
  #define CYTHON_REFNANNY 0
#endif
#if CYTHON_REFNANNY
  typedef struct {
    void (*INCREF)(void*, PyObject*, Py_ssize_t);
    void (*DECREF)(void*, PyObject*, Py_ssize_t);
    void (*GOTREF)(void*, PyObject*, Py_ssize_t);
    void (*GIVEREF)(void*, PyObject*, Py_ssize_t);
    void* (*SetupContext)(const char*, Py_ssize_t, const char*);
    void (*FinishContext)(void**);
  } __Pyx_RefNannyAPIStruct;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNanny = NULL;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname);
  #define __Pyx_RefNannyDeclarations void *__pyx_refnanny = NULL;
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          if (acquire_gil) {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), (__LINE__), (__FILE__));\
              PyGILState_Release(__pyx_gilstate_save);\
          } else {\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), (__LINE__), (__FILE__));\
          }
  #define __Pyx_RefNannyFinishContextNogil() {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __Pyx_RefNannyFinishContext();\
              PyGILState_Release(__pyx_gilstate_save);\
          }
  #define __Pyx_RefNannyFinishContextNogil() {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __Pyx_RefNannyFinishContext();\
              PyGILState_Release(__pyx_gilstate_save);\
          }
  #define __Pyx_RefNannyFinishContext()\
          __Pyx_RefNanny->FinishContext(&__pyx_refnanny)
  #define __Pyx_INCREF(r)  __Pyx_RefNanny->INCREF(__pyx_refnanny, (PyObject *)(r), (__LINE__))
  #define __Pyx_DECREF(r)  __Pyx_RefNanny->DECREF(__pyx_refnanny, (PyObject *)(r), (__LINE__))
  #define __Pyx_GOTREF(r)  __Pyx_RefNanny->GOTREF(__pyx_refnanny, (PyObject *)(r), (__LINE__))
  #define __Pyx_GIVEREF(r) __Pyx_RefNanny->GIVEREF(__pyx_refnanny, (PyObject *)(r), (__LINE__))
  #define __Pyx_XINCREF(r)  do { if((r) == NULL); else {__Pyx_INCREF(r); }} while(0)
  #define __Pyx_XDECREF(r)  do { if((r) == NULL); else {__Pyx_DECREF(r); }} while(0)
  #define __Pyx_XGOTREF(r)  do { if((r) == NULL); else {__Pyx_GOTREF(r); }} while(0)
  #define __Pyx_XGIVEREF(r) do { if((r) == NULL); else {__Pyx_GIVEREF(r);}} while(0)
#else
  #define __Pyx_RefNannyDeclarations
  #define __Pyx_RefNannySetupContext(name, acquire_gil)
  #define __Pyx_RefNannyFinishContextNogil()
  #define __Pyx_RefNannyFinishContext()
  #define __Pyx_INCREF(r) Py_INCREF(r)
  #define __Pyx_DECREF(r) Py_DECREF(r)
  #define __Pyx_GOTREF(r)
  #define __Pyx_GIVEREF(r)
  #define __Pyx_XINCREF(r) Py_XINCREF(r)
  #define __Pyx_XDECREF(r) Py_XDECREF(r)
  #define __Pyx_XGOTREF(r)
  #define __Pyx_XGIVEREF(r)
#endif
#define __Pyx_Py_XDECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; Py_XDECREF(tmp);\
    } while (0)
#define __Pyx_XDECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_XDECREF(tmp);\
    } while (0)
#define __Pyx_DECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_DECREF(tmp);\
    } while (0)
#define __Pyx_CLEAR(r)    do { PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);} while(0)
#define __Pyx_XCLEAR(r)   do { if((r) != NULL) {PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);}} while(0)

/* PyErrExceptionMatches.proto (used by PyObjectGetAttrStrNoError) */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_ExceptionMatches(err) __Pyx_PyErr_ExceptionMatchesInState(__pyx_tstate, err)
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err);
#else
#define __Pyx_PyErr_ExceptionMatches(err)  PyErr_ExceptionMatches(err)
#endif

/* PyThreadStateGet.proto (used by PyErrFetchRestore) */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyThreadState_declare  PyThreadState *__pyx_tstate;
#define __Pyx_PyThreadState_assign  __pyx_tstate = __Pyx_PyThreadState_Current;
#if PY_VERSION_HEX >= 0x030C00A6
#define __Pyx_PyErr_Occurred()  (__pyx_tstate->current_exception != NULL)
#define __Pyx_PyErr_CurrentExceptionType()  (__pyx_tstate->current_exception ? (PyObject*) Py_TYPE(__pyx_tstate->current_exception) : (PyObject*) NULL)
#else
#define __Pyx_PyErr_Occurred()  (__pyx_tstate->curexc_type != NULL)
#define __Pyx_PyErr_CurrentExceptionType()  (__pyx_tstate->curexc_type)
#endif
#else
#define __Pyx_PyThreadState_declare
#define __Pyx_PyThreadState_assign
#define __Pyx_PyErr_Occurred()  (PyErr_Occurred() != NULL)
#define __Pyx_PyErr_CurrentExceptionType()  PyErr_Occurred()
#endif

/* PyErrFetchRestore.proto (used by PyObjectGetAttrStrNoError) */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_Clear() __Pyx_ErrRestore(NULL, NULL, NULL)
#define __Pyx_ErrRestoreWithState(type, value, tb)  __Pyx_ErrRestoreInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)    __Pyx_ErrFetchInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  __Pyx_ErrRestoreInState(__pyx_tstate, type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)    __Pyx_ErrFetchInState(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030C00A6
#define __Pyx_PyErr_SetNone(exc) (Py_INCREF(exc), __Pyx_ErrRestore((exc), NULL, NULL))
#else
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#endif
#else
#define __Pyx_PyErr_Clear() PyErr_Clear()
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#define __Pyx_ErrRestoreWithState(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestoreInState(tstate, type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchInState(tstate, type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)  PyErr_Fetch(type, value, tb)
#endif

/* PyObjectGetAttrStr.proto (used by PyObjectGetAttrStrNoError) */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GetAttrStr(o,n) PyObject_GetAttr(o,n)
#endif

/* PyObjectGetAttrStrNoError.proto (used by GetBuiltinName) */
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStrNoError(PyObject* obj, PyObject* attr_name);

/* GetBuiltinName.proto */
static PyObject *__Pyx_GetBuiltinName(PyObject *name);

/* PyDictVersioning.proto (used by GetModuleGlobalName) */
#if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_TYPE_SLOTS
#define __PYX_DICT_VERSION_INIT  ((PY_UINT64_T) -1)
#define __PYX_GET_DICT_VERSION(dict)  (((PyDictObject*)(dict))->ma_version_tag)
#define __PYX_UPDATE_DICT_CACHE(dict, value, cache_var, version_var)\
    (version_var) = __PYX_GET_DICT_VERSION(dict);\
    (cache_var) = (value);
#define __PYX_PY_DICT_LOOKUP_IF_MODIFIED(VAR, DICT, LOOKUP) {\
    static PY_UINT64_T __pyx_dict_version = 0;\
    static PyObject *__pyx_dict_cached_value = NULL;\
    if (likely(__PYX_GET_DICT_VERSION(DICT) == __pyx_dict_version)) {\
        (VAR) = __Pyx_XNewRef(__pyx_dict_cached_value);\
    } else {\
        (VAR) = __pyx_dict_cached_value = (LOOKUP);\
        __pyx_dict_version = __PYX_GET_DICT_VERSION(DICT);\
    }\
}
static CYTHON_INLINE PY_UINT64_T __Pyx_get_tp_dict_version(PyObject *obj);
static CYTHON_INLINE PY_UINT64_T __Pyx_get_object_dict_version(PyObject *obj);
static CYTHON_INLINE int __Pyx_object_dict_version_matches(PyObject* obj, PY_UINT64_T tp_dict_version, PY_UINT64_T obj_dict_version);
#else
#define __PYX_GET_DICT_VERSION(dict)  (0)
#define __PYX_UPDATE_DICT_CACHE(dict, value, cache_var, version_var)
#define __PYX_PY_DICT_LOOKUP_IF_MODIFIED(VAR, DICT, LOOKUP)  (VAR) = (LOOKUP);
#endif

/* GetModuleGlobalName.proto */
#if CYTHON_USE_DICT_VERSIONS
#define __Pyx_GetModuleGlobalName(var, name)  do {\
    static PY_UINT64_T __pyx_dict_version = 0;\
    static PyObject *__pyx_dict_cached_value = NULL;\
    (var) = (likely(__pyx_dict_version == __PYX_GET_DICT_VERSION(__pyx_mstate_global->__pyx_d))) ?\
        (likely(__pyx_dict_cached_value) ? __Pyx_NewRef(__pyx_dict_cached_value) : __Pyx_GetBuiltinName(name)) :\
        __Pyx__GetModuleGlobalName(name, &__pyx_dict_version, &__pyx_dict_cached_value);\
} while(0)
#define __Pyx_GetModuleGlobalNameUncached(var, name)  do {\
    PY_UINT64_T __pyx_dict_version;\
    PyObject *__pyx_dict_cached_value;\
    (var) = __Pyx__GetModuleGlobalName(name, &__pyx_dict_version, &__pyx_dict_cached_value);\
} while(0)
static PyObject *__Pyx__GetModuleGlobalName(PyObject *name, PY_UINT64_T *dict_version, PyObject **dict_cached_value);
#else
#define __Pyx_GetModuleGlobalName(var, name)  (var) = __Pyx__GetModuleGlobalName(name)
#define __Pyx_GetModuleGlobalNameUncached(var, name)  (var) = __Pyx__GetModuleGlobalName(name)
static CYTHON_INLINE PyObject *__Pyx__GetModuleGlobalName(PyObject *name);
#endif

/* PyObjectCall.proto (used by PyObjectFastCall) */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw);
#else
#define __Pyx_PyObject_Call(func, arg, kw) PyObject_Call(func, arg, kw)
#endif

/* PyObjectCallMethO.proto (used by PyObjectFastCall) */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg);
#endif

/* PyObjectFastCall.proto */
#define __Pyx_PyObject_FastCall(func, args, nargs)  __Pyx_PyObject_FastCallDict(func, args, (size_t)(nargs), NULL)
static CYTHON_INLINE PyObject* __Pyx_PyObject_FastCallDict(PyObject *func, PyObject * const*args, size_t nargs, PyObject *kwargs);

/* RaiseException.export */
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause);

/* PyUnicodeEncodeError_Check.proto */
#define __Pyx_PyExc_UnicodeEncodeError_Check(obj)  __Pyx_TypeCheck(obj, PyExc_UnicodeEncodeError)

/* RaiseUnexpectedTypeError.proto */
static int __Pyx_RaiseUnexpectedTypeError(const char *expected, PyObject *obj);

/* PyTypeError_Check.proto */
#define __Pyx_PyExc_TypeError_Check(obj)  __Pyx_TypeCheck(obj, PyExc_TypeError)

/* PyObjectFastCallMethod.proto */
#if CYTHON_VECTORCALL && PY_VERSION_HEX >= 0x03090000
#define __Pyx_PyObject_FastCallMethod(name, args, nargsf) PyObject_VectorcallMethod(name, args, nargsf, NULL)
#else
static PyObject *__Pyx_PyObject_FastCallMethod(PyObject *name, PyObject *const *args, size_t nargsf);
#endif

/* GetTopmostException.proto (used by SaveResetException) */
#if CYTHON_USE_EXC_INFO_STACK && CYTHON_FAST_THREAD_STATE
static _PyErr_StackItem * __Pyx_PyErr_GetTopmostException(PyThreadState *tstate);
#endif

/* SaveResetException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_ExceptionSave(type, value, tb)  __Pyx__ExceptionSave(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionSave(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#define __Pyx_ExceptionReset(type, value, tb)  __Pyx__ExceptionReset(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionReset(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
#else
#define __Pyx_ExceptionSave(type, value, tb)   PyErr_GetExcInfo(type, value, tb)
#define __Pyx_ExceptionReset(type, value, tb)  PyErr_SetExcInfo(type, value, tb)
#endif

/* GetException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_GetException(type, value, tb)  __Pyx__GetException(__pyx_tstate, type, value, tb)
static int __Pyx__GetException(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#else
static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb);
#endif

/* PyValueError_Check.proto */
#define __Pyx_PyExc_ValueError_Check(obj)  __Pyx_TypeCheck(obj, PyExc_ValueError)

/* SwapException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_ExceptionSwap(type, value, tb)  __Pyx__ExceptionSwap(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionSwap(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#else
static CYTHON_INLINE void __Pyx_ExceptionSwap(PyObject **type, PyObject **value, PyObject **tb);
#endif

/* SetStringIndexingError.proto (used by GetItemIntByteArray) */
static void __Pyx_SetStringIndexingError(const char* message, int has_gil);

/* GetItemIntByteArray.proto */
#define __Pyx_GetItemInt_ByteArray(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck, has_gil, unsafe_shared)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_ByteArray_Fast(o, (Py_ssize_t)i, wraparound, boundscheck, has_gil, unsafe_shared) :\
    (__Pyx_SetStringIndexingError("bytearray index out of range", has_gil), -1))
static CYTHON_INLINE int __Pyx_GetItemInt_ByteArray_Fast(PyObject* string, Py_ssize_t i,
                                                         int wraparound, int boundscheck, int has_gil, int unsafe_shared);

/* SetItemIntByteArray.proto */
#define __Pyx_SetItemInt_ByteArray(o, i, v, type, is_signed, to_py_func, is_list, wraparound, boundscheck, has_gil, unsafe_shared)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_SetItemInt_ByteArray_Fast(o, (Py_ssize_t)i, v, wraparound, boundscheck, has_gil, unsafe_shared) :\
    (__Pyx_SetStringIndexingError("bytearray index out of range", has_gil), -1))
static CYTHON_INLINE int __Pyx_SetItemInt_ByteArray_Fast(PyObject* string, Py_ssize_t i, unsigned char v,
                                                         int wraparound, int boundscheck, int has_gil, int unsafe_shared);

/* IncludeStringH.proto (used by BytesEquals) */
#include <string.h>

/* BytesEquals.proto (used by UnicodeEquals) */
static CYTHON_INLINE int __Pyx_PyBytes_Equals(PyObject* s1, PyObject* s2, int equals);

/* UnicodeEquals.proto */
static CYTHON_INLINE int __Pyx_PyUnicode_Equals(PyObject* s1, PyObject* s2, int equals);

/* CallableCheck.proto */
#if CYTHON_USE_TYPE_SLOTS && !CYTHON_COMPILING_IN_PYPY
#define __Pyx_PyCallable_Check(obj)   (Py_TYPE(obj)->tp_call != NULL)
#else
#define __Pyx_PyCallable_Check(obj)   PyCallable_Check(obj)
#endif

/* TupleAndListFromArray.proto (used by fastcall) */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyList_FromArray(PyObject *const *src, Py_ssize_t n);
#endif
#if CYTHON_COMPILING_IN_CPYTHON || CYTHON_METH_FASTCALL
static CYTHON_INLINE PyObject* __Pyx_PyTuple_FromArray(PyObject *const *src, Py_ssize_t n);
#endif

/* fastcall.proto */
#if CYTHON_AVOID_BORROWED_REFS
    #define __Pyx_ArgRef_VARARGS(args, i) __Pyx_PySequence_ITEM(args, i)
#elif CYTHON_ASSUME_SAFE_MACROS
    #define __Pyx_ArgRef_VARARGS(args, i) __Pyx_NewRef(__Pyx_PyTuple_GET_ITEM(args, i))
#else
    #define __Pyx_ArgRef_VARARGS(args, i) __Pyx_XNewRef(PyTuple_GetItem(args, i))
#endif
#define __Pyx_NumKwargs_VARARGS(kwds) PyDict_Size(kwds)
#define __Pyx_KwValues_VARARGS(args, nargs) NULL
#define __Pyx_GetKwValue_VARARGS(kw, kwvalues, s) __Pyx_PyDict_GetItemStrWithError(kw, s)
#define __Pyx_KwargsAsDict_VARARGS(kw, kwvalues) PyDict_Copy(kw)
#if CYTHON_METH_FASTCALL
    #define __Pyx_ArgRef_FASTCALL(args, i) __Pyx_NewRef(args[i])
    #define __Pyx_NumKwargs_FASTCALL(kwds) __Pyx_PyTuple_GET_SIZE(kwds)
    #define __Pyx_KwValues_FASTCALL(args, nargs) ((args) + (nargs))
    static CYTHON_INLINE PyObject * __Pyx_GetKwValue_FASTCALL(PyObject *kwnames, PyObject *const *kwvalues, PyObject *s);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030d0000 || CYTHON_COMPILING_IN_LIMITED_API
    CYTHON_UNUSED static PyObject *__Pyx_KwargsAsDict_FASTCALL(PyObject *kwnames, PyObject *const *kwvalues);
  #else
    #define __Pyx_KwargsAsDict_FASTCALL(kw, kwvalues) _PyStack_AsDict(kwvalues, kw)
  #endif
#else
    #define __Pyx_ArgRef_FASTCALL __Pyx_ArgRef_VARARGS
    #define __Pyx_NumKwargs_FASTCALL __Pyx_NumKwargs_VARARGS
    #define __Pyx_KwValues_FASTCALL __Pyx_KwValues_VARARGS
    #define __Pyx_GetKwValue_FASTCALL __Pyx_GetKwValue_VARARGS
    #define __Pyx_KwargsAsDict_FASTCALL __Pyx_KwargsAsDict_VARARGS
#endif
#define __Pyx_ArgsSlice_VARARGS(args, start, stop) PyTuple_GetSlice(args, start, stop)
#if CYTHON_METH_FASTCALL || (CYTHON_COMPILING_IN_CPYTHON && CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS)
#define __Pyx_ArgsSlice_FASTCALL(args, start, stop) __Pyx_PyTuple_FromArray(args + start, stop - start)
#else
#define __Pyx_ArgsSlice_FASTCALL(args, start, stop) PyTuple_GetSlice(args, start, stop)
#endif

/* py_dict_items.proto (used by OwnedDictNext) */
static CYTHON_INLINE PyObject* __Pyx_PyDict_Items(PyObject* d);

/* CallCFunction.proto (used by CallUnboundCMethod0) */
#define __Pyx_CallCFunction(cfunc, self, args)\
    ((PyCFunction)(void(*)(void))(cfunc)->func)(self, args)
#define __Pyx_CallCFunctionWithKeywords(cfunc, self, args, kwargs)\
    ((PyCFunctionWithKeywords)(void(*)(void))(cfunc)->func)(self, args, kwargs)
#define __Pyx_CallCFunctionFast(cfunc, self, args, nargs)\
    ((__Pyx_PyCFunctionFast)(void(*)(void))(PyCFunction)(cfunc)->func)(self, args, nargs)
#define __Pyx_CallCFunctionFastWithKeywords(cfunc, self, args, nargs, kwnames)\
    ((__Pyx_PyCFunctionFastWithKeywords)(void(*)(void))(PyCFunction)(cfunc)->func)(self, args, nargs, kwnames)

/* PyObjectCallOneArg.proto (used by CallUnboundCMethod0) */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg);

/* UnpackUnboundCMethod.proto (used by CallUnboundCMethod0) */
typedef struct {
    PyObject *type;
    PyObject **method_name;
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING && CYTHON_ATOMICS
    __pyx_atomic_int_type initialized;
#endif
    PyCFunction func;
    PyObject *method;
    int flag;
} __Pyx_CachedCFunction;
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
static CYTHON_INLINE int __Pyx_CachedCFunction_GetAndSetInitializing(__Pyx_CachedCFunction *cfunc) {
#if !CYTHON_ATOMICS
    return 1;
#else
    __pyx_nonatomic_int_type expected = 0;
    if (__pyx_atomic_int_cmp_exchange(&cfunc->initialized, &expected, 1)) {
        return 0;
    }
    return expected;
#endif
}
static CYTHON_INLINE void __Pyx_CachedCFunction_SetFinishedInitializing(__Pyx_CachedCFunction *cfunc) {
#if CYTHON_ATOMICS
    __pyx_atomic_store(&cfunc->initialized, 2);
#endif
}
#else
#define __Pyx_CachedCFunction_GetAndSetInitializing(cfunc) 2
#define __Pyx_CachedCFunction_SetFinishedInitializing(cfunc)
#endif

/* CallUnboundCMethod0.proto */
CYTHON_UNUSED
static PyObject* __Pyx__CallUnboundCMethod0(__Pyx_CachedCFunction* cfunc, PyObject* self);
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_CallUnboundCMethod0(__Pyx_CachedCFunction* cfunc, PyObject* self);
#else
#define __Pyx_CallUnboundCMethod0(cfunc, self)  __Pyx__CallUnboundCMethod0(cfunc, self)
#endif

/* py_dict_values.proto (used by OwnedDictNext) */
static CYTHON_INLINE PyObject* __Pyx_PyDict_Values(PyObject* d);

/* OwnedDictNext.proto (used by ParseKeywordsImpl) */
#if CYTHON_AVOID_BORROWED_REFS
static int __Pyx_PyDict_NextRef(PyObject *p, PyObject **ppos, PyObject **pkey, PyObject **pvalue);
#else
CYTHON_INLINE
static int __Pyx_PyDict_NextRef(PyObject *p, Py_ssize_t *ppos, PyObject **pkey, PyObject **pvalue);
#endif

/* RaiseDoubleKeywords.proto (used by ParseKeywordsImpl) */
static void __Pyx_RaiseDoubleKeywordsError(const char* func_name, PyObject* kw_name);

/* ParseKeywordsImpl.export */
static int __Pyx_ParseKeywordsTuple(
    PyObject *kwds,
    PyObject * const *kwvalues,
    PyObject ** const argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    Py_ssize_t num_kwargs,
    const char* function_name,
    int ignore_unknown_kwargs
);
static int __Pyx_ParseKeywordDictToDict(
    PyObject *kwds,
    PyObject ** const argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    const char* function_name
);
static int __Pyx_ParseKeywordDict(
    PyObject *kwds,
    PyObject ** const argnames[],
    PyObject *values[],
    Py_ssize_t num_pos_args,
    Py_ssize_t num_kwargs,
    const char* function_name,
    int ignore_unknown_kwargs
);

/* CallUnboundCMethod2.proto */
CYTHON_UNUSED
static PyObject* __Pyx__CallUnboundCMethod2(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg1, PyObject* arg2);
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject *__Pyx_CallUnboundCMethod2(__Pyx_CachedCFunction *cfunc, PyObject *self, PyObject *arg1, PyObject *arg2);
#else
#define __Pyx_CallUnboundCMethod2(cfunc, self, arg1, arg2)  __Pyx__CallUnboundCMethod2(cfunc, self, arg1, arg2)
#endif

/* ParseKeywords.proto */
static CYTHON_INLINE int __Pyx_ParseKeywords(
    PyObject *kwds, PyObject *const *kwvalues, PyObject ** const argnames[],
    PyObject *kwds2, PyObject *values[],
    Py_ssize_t num_pos_args, Py_ssize_t num_kwargs,
    const char* function_name,
    int ignore_unknown_kwargs
);

/* RaiseArgTupleInvalid.proto */
static void __Pyx_RaiseArgtupleInvalid(const char* func_name, int exact,
    Py_ssize_t num_min, Py_ssize_t num_max, Py_ssize_t num_found);

/* ArgTypeTestFunc.export */
static int __Pyx__ArgTypeTest(PyObject *obj, PyTypeObject *type, const char *name, int exact);

/* ArgTypeTest.proto */
#define __Pyx_ArgTypeTest(obj, type, none_allowed, name, exact)\
    ((likely(__Pyx_IS_TYPE(obj, type) | (none_allowed && (obj == Py_None)))) ? 1 :\
        __Pyx__ArgTypeTest(obj, type, name, exact))

/* MoveIfSupported.proto */
#if CYTHON_USE_CPP_STD_MOVE
  #include <utility>
  #define __PYX_STD_MOVE_IF_SUPPORTED(x) std::move(x)
#else
  #define __PYX_STD_MOVE_IF_SUPPORTED(x) x
#endif

/* PyObjectLookupSpecial.proto */
#if CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
#define __Pyx_PyObject_LookupSpecialNoError(obj, attr_name)  __Pyx__PyObject_LookupSpecial(obj, attr_name, 0)
#define __Pyx_PyObject_LookupSpecial(obj, attr_name)  __Pyx__PyObject_LookupSpecial(obj, attr_name, 1)
static CYTHON_INLINE PyObject* __Pyx__PyObject_LookupSpecial(PyObject* obj, PyObject* attr_name, int with_error);
#else
#define __Pyx_PyObject_LookupSpecialNoError(o,n) __Pyx_PyObject_GetAttrStrNoError(o,n)
#define __Pyx_PyObject_LookupSpecial(o,n) __Pyx_PyObject_GetAttrStr(o,n)
#endif

/* GetItemInt.proto */
#define __Pyx_GetItemInt(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck, has_gil, unsafe_shared)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Fast(o, (Py_ssize_t)i, is_list, wraparound, boundscheck, unsafe_shared) :\
    (is_list ? (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL) :\
               __Pyx_GetItemInt_Generic(o, to_py_func(i))))
#define __Pyx_GetItemInt_List(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck, has_gil, unsafe_shared)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_List_Fast(o, (Py_ssize_t)i, wraparound, boundscheck, unsafe_shared) :\
    (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck, int unsafe_shared);
#define __Pyx_GetItemInt_Tuple(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck, has_gil, unsafe_shared)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Tuple_Fast(o, (Py_ssize_t)i, wraparound, boundscheck, unsafe_shared) :\
    (PyErr_SetString(PyExc_IndexError, "tuple index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck, int unsafe_shared);
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j);
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i,
                                                     int is_list, int wraparound, int boundscheck, int unsafe_shared);

/* ExtTypeTest.proto */
static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type);

/* RejectKeywords.export */
static void __Pyx_RejectKeywords(const char* function_name, PyObject *kwds);

/* WriteUnraisableException.proto */
static void __Pyx_WriteUnraisable(const char *name, int clineno,
                                  int lineno, const char *filename,
                                  int full_traceback, int nogil);

/* PyObjectFormatAndDecref.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_FormatSimpleAndDecref(PyObject* s, PyObject* f);
static CYTHON_INLINE PyObject* __Pyx_PyObject_FormatAndDecref(PyObject* s, PyObject* f);

/* PyUnicode_Unicode.proto */
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_Unicode(PyObject *obj);

/* PyNumber_Hex.proto */
#define __Pyx_PyNumber_Hex(obj) PyNumber_ToBase((obj), 16)

/* JoinPyUnicode.export */
static PyObject* __Pyx_PyUnicode_Join(PyObject** values, Py_ssize_t value_count, Py_ssize_t result_ulength,
                                      Py_UCS4 max_char);

/* PyRuntimeError_Check.proto */
#define __Pyx_PyExc_RuntimeError_Check(obj)  __Pyx_TypeCheck(obj, PyExc_RuntimeError)

/* PyObjectVectorCallKwBuilder.proto */
CYTHON_UNUSED static int __Pyx_VectorcallBuilder_AddArg_Check(PyObject *key, PyObject *value, PyObject *builder, PyObject **args, int n);
#if CYTHON_VECTORCALL
#if PY_VERSION_HEX >= 0x03090000
#define __Pyx_Object_Vectorcall_CallFromBuilder PyObject_Vectorcall
#else
#define __Pyx_Object_Vectorcall_CallFromBuilder _PyObject_Vectorcall
#endif
#define __Pyx_MakeVectorcallBuilderKwds(n) PyTuple_New(n)
static int __Pyx_VectorcallBuilder_AddArg(PyObject *key, PyObject *value, PyObject *builder, PyObject **args, int n);
static int __Pyx_VectorcallBuilder_AddArgStr(const char *key, PyObject *value, PyObject *builder, PyObject **args, int n);
#else
#define __Pyx_Object_Vectorcall_CallFromBuilder __Pyx_PyObject_FastCallDict
#define __Pyx_MakeVectorcallBuilderKwds(n) __Pyx_PyDict_NewPresized(n)
#define __Pyx_VectorcallBuilder_AddArg(key, value, builder, args, n) PyDict_SetItem(builder, key, value)
#define __Pyx_VectorcallBuilder_AddArgStr(key, value, builder, args, n) PyDict_SetItemString(builder, key, value)
#endif

/* RaiseTooManyValuesToUnpack.proto */
static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected);

/* RaiseNeedMoreValuesToUnpack.proto */
static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index);

/* IterFinish.proto */
static CYTHON_INLINE int __Pyx_IterFinish(void);

/* UnpackItemEndCheck.proto */
static int __Pyx_IternextUnpackEndCheck(PyObject *retval, Py_ssize_t expected);

/* ListCompAppend.proto */
#if CYTHON_USE_PYLIST_INTERNALS && CYTHON_ASSUME_SAFE_MACROS
static CYTHON_INLINE int __Pyx_ListComp_Append(PyObject* list, PyObject* x) {
    PyListObject* L = (PyListObject*) list;
    Py_ssize_t len = Py_SIZE(list);
    if (likely(L->allocated > len)) {
        Py_INCREF(x);
        #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030d0000
        L->ob_item[len] = x;
        #else
        PyList_SET_ITEM(list, len, x);
        #endif
        __Pyx_SET_SIZE(list, len + 1);
        return 0;
    }
    return PyList_Append(list, x);
}
#else
#define __Pyx_ListComp_Append(L,x) PyList_Append(L,x)
#endif

/* RaiseKeywordRequired.proto */
static void __Pyx_RaiseKeywordRequired(const char* func_name, PyObject* kw_name);

/* PyObjectVectorCallMethodKwBuilder.proto */
#if CYTHON_VECTORCALL && PY_VERSION_HEX >= 0x03090000
#define __Pyx_Object_VectorcallMethod_CallFromBuilder PyObject_VectorcallMethod
#else
static PyObject *__Pyx_Object_VectorcallMethod_CallFromBuilder(PyObject *name, PyObject *const *args, size_t nargsf, PyObject *kwnames);
#endif

/* GetAttr3.proto */
static CYTHON_INLINE PyObject *__Pyx_GetAttr3(PyObject *, PyObject *, PyObject *);

/* py_dict_pop.proto */
static CYTHON_INLINE PyObject *__Pyx_PyDict_Pop(PyObject *d, PyObject *key, PyObject *default_value);

/* PyObjectCall2Args.proto (used by CallUnboundCMethod1) */
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call2Args(PyObject* function, PyObject* arg1, PyObject* arg2);

/* CallUnboundCMethod1.proto */
CYTHON_UNUSED
static PyObject* __Pyx__CallUnboundCMethod1(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg);
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_CallUnboundCMethod1(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg);
#else
#define __Pyx_CallUnboundCMethod1(cfunc, self, arg)  __Pyx__CallUnboundCMethod1(cfunc, self, arg)
#endif

/* RaiseClosureNameError.proto */
static void __Pyx_RaiseClosureNameError(const char *varname);

/* dict_setdefault.proto (used by FetchCommonType) */
static CYTHON_INLINE PyObject *__Pyx_PyDict_SetDefault(PyObject *d, PyObject *key, PyObject *default_value);

/* LimitedApiGetTypeDict.proto (used by SetItemOnTypeDict) */
#if CYTHON_COMPILING_IN_LIMITED_API
static PyObject *__Pyx_GetTypeDict(PyTypeObject *tp);
#endif

/* SetItemOnTypeDict.proto (used by FixUpExtensionType) */
static int __Pyx__SetItemOnTypeDict(PyTypeObject *tp, PyObject *k, PyObject *v);
#define __Pyx_SetItemOnTypeDict(tp, k, v) __Pyx__SetItemOnTypeDict((PyTypeObject*)tp, k, v)

/* FixUpExtensionType.proto (used by FetchCommonType) */
static CYTHON_INLINE int __Pyx_fix_up_extension_type_from_spec(PyType_Spec *spec, PyTypeObject *type);

/* AddModuleRef.proto (used by FetchSharedCythonModule) */
#if ((CYTHON_COMPILING_IN_CPYTHON_FREETHREADING ) ||\
     __PYX_LIMITED_VERSION_HEX < 0x030d0000)
  static PyObject *__Pyx_PyImport_AddModuleRef(const char *name);
#else
  #define __Pyx_PyImport_AddModuleRef(name) PyImport_AddModuleRef(name)
#endif

/* FetchSharedCythonModule.proto (used by FetchCommonType) */
static PyObject *__Pyx_FetchSharedCythonABIModule(void);

/* FetchCommonType.proto (used by CommonTypesMetaclass) */
static PyTypeObject* __Pyx_FetchCommonTypeFromSpec(PyTypeObject *metaclass, PyObject *module, PyType_Spec *spec, PyObject *bases);

/* CommonTypesMetaclass.proto (used by CythonFunctionShared) */
static int __pyx_CommonTypesMetaclass_init(PyObject *module);
#define __Pyx_CommonTypesMetaclass_USED

/* CallTypeTraverse.proto (used by CythonFunctionShared) */
#if !CYTHON_USE_TYPE_SPECS || (!CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x03090000)
#define __Pyx_call_type_traverse(o, always_call, visit, arg) 0
#else
static int __Pyx_call_type_traverse(PyObject *o, int always_call, visitproc visit, void *arg);
#endif

/* PyMethodNew.proto (used by CythonFunctionShared) */
static PyObject *__Pyx_PyMethod_New(PyObject *func, PyObject *self, PyObject *typ);

/* PyVectorcallFastCallDict.proto (used by CythonFunctionShared) */
#if CYTHON_METH_FASTCALL && CYTHON_VECTORCALL
static CYTHON_INLINE PyObject *__Pyx_PyVectorcall_FastCallDict(PyObject *func, __pyx_vectorcallfunc vc, PyObject *const *args, size_t nargs, PyObject *kw);
#endif

/* CythonFunctionShared.proto (used by CythonFunction) */
#define __Pyx_CyFunction_USED
#define __Pyx_CYFUNCTION_STATICMETHOD  0x01
#define __Pyx_CYFUNCTION_CLASSMETHOD   0x02
#define __Pyx_CYFUNCTION_CCLASS        0x04
#define __Pyx_CYFUNCTION_COROUTINE     0x08
#define __Pyx_CyFunction_GetClosure(f)\
    (((__pyx_CyFunctionObject *) (f))->func_closure)
#if PY_VERSION_HEX < 0x030900B1 || CYTHON_COMPILING_IN_LIMITED_API
  #define __Pyx_CyFunction_GetClassObj(f)\
      (((__pyx_CyFunctionObject *) (f))->func_classobj)
#else
  #define __Pyx_CyFunction_GetClassObj(f)\
      ((PyObject*) ((PyCMethodObject *) (f))->mm_class)
#endif
#define __Pyx_CyFunction_SetClassObj(f, classobj)\
    __Pyx__CyFunction_SetClassObj((__pyx_CyFunctionObject *) (f), (classobj))
#define __Pyx_CyFunction_Defaults(type, f)\
    ((type *)(((__pyx_CyFunctionObject *) (f))->defaults))
#define __Pyx_CyFunction_SetDefaultsGetter(f, g)\
    ((__pyx_CyFunctionObject *) (f))->defaults_getter = (g)
typedef struct {
#if CYTHON_COMPILING_IN_LIMITED_API
    PyObject_HEAD
    PyObject *func;
#elif PY_VERSION_HEX < 0x030900B1
    PyCFunctionObject func;
#else
    PyCMethodObject func;
#endif
#if CYTHON_COMPILING_IN_LIMITED_API && CYTHON_METH_FASTCALL
    __pyx_vectorcallfunc func_vectorcall;
#endif
#if CYTHON_COMPILING_IN_LIMITED_API
    PyObject *func_weakreflist;
#endif
#if PY_VERSION_HEX < 0x030C0000 || CYTHON_COMPILING_IN_LIMITED_API
    PyObject *func_dict;
#endif
    PyObject *func_name;
    PyObject *func_qualname;
    PyObject *func_doc;
    PyObject *func_globals;
    PyObject *func_code;
    PyObject *func_closure;
#if PY_VERSION_HEX < 0x030900B1 || CYTHON_COMPILING_IN_LIMITED_API
    PyObject *func_classobj;
#endif
    PyObject *defaults;
    int flags;
    PyObject *defaults_tuple;
    PyObject *defaults_kwdict;
    PyObject *(*defaults_getter)(PyObject *);
    PyObject *func_annotations;
    PyObject *func_is_coroutine;
} __pyx_CyFunctionObject;
#undef __Pyx_CyOrPyCFunction_Check
#define __Pyx_CyFunction_Check(obj)  __Pyx_TypeCheck(obj, __pyx_mstate_global->__pyx_CyFunctionType)
#define __Pyx_CyOrPyCFunction_Check(obj)  __Pyx_TypeCheck2(obj, __pyx_mstate_global->__pyx_CyFunctionType, &PyCFunction_Type)
#define __Pyx_CyFunction_CheckExact(obj)  __Pyx_IS_TYPE(obj, __pyx_mstate_global->__pyx_CyFunctionType)
static CYTHON_INLINE int __Pyx__IsSameCyOrCFunction(PyObject *func, void (*cfunc)(void));
#undef __Pyx_IsSameCFunction
#define __Pyx_IsSameCFunction(func, cfunc)   __Pyx__IsSameCyOrCFunction(func, cfunc)
static PyObject *__Pyx_CyFunction_Init(__pyx_CyFunctionObject* op, PyMethodDef *ml,
                                      int flags, PyObject* qualname,
                                      PyObject *closure,
                                      PyObject *module, PyObject *globals,
                                      PyObject* code);
static CYTHON_INLINE void __Pyx__CyFunction_SetClassObj(__pyx_CyFunctionObject* f, PyObject* classobj);
static CYTHON_INLINE PyObject *__Pyx_CyFunction_InitDefaults(PyObject *func,
                                                         PyTypeObject *defaults_type);
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsTuple(PyObject *m,
                                                            PyObject *tuple);
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsKwDict(PyObject *m,
                                                             PyObject *dict);
static CYTHON_INLINE void __Pyx_CyFunction_SetAnnotationsDict(PyObject *m,
                                                              PyObject *dict);
static int __pyx_CyFunction_init(PyObject *module);
#if CYTHON_METH_FASTCALL
static PyObject * __Pyx_CyFunction_Vectorcall_NOARGS(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames);
static PyObject * __Pyx_CyFunction_Vectorcall_O(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames);
static PyObject * __Pyx_CyFunction_Vectorcall_FASTCALL_KEYWORDS(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames);
static PyObject * __Pyx_CyFunction_Vectorcall_FASTCALL_KEYWORDS_METHOD(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames);
#if CYTHON_COMPILING_IN_LIMITED_API
#define __Pyx_CyFunction_func_vectorcall(f) (((__pyx_CyFunctionObject*)f)->func_vectorcall)
#else
#define __Pyx_CyFunction_func_vectorcall(f) (((PyCFunctionObject*)f)->vectorcall)
#endif
#endif

/* CythonFunction.proto */
static PyObject *__Pyx_CyFunction_New(PyMethodDef *ml,
                                      int flags, PyObject* qualname,
                                      PyObject *closure,
                                      PyObject *module, PyObject *globals,
                                      PyObject* code);

/* PyStopIteration_Check.proto */
#define __Pyx_PyExc_StopIteration_Check(obj)  __Pyx_TypeCheck(obj, PyExc_StopIteration)

/* AssertionsEnabled.proto */
#if CYTHON_COMPILING_IN_LIMITED_API  ||  PY_VERSION_HEX >= 0x030C0000
  static int __pyx_assertions_enabled_flag;
  #define __pyx_assertions_enabled() (__pyx_assertions_enabled_flag)
  #if __clang__ || __GNUC__
  __attribute__((no_sanitize("thread")))
  #endif
  static int __Pyx_init_assertions_enabled(void) {
    PyObject *builtins, *debug, *debug_str;
    int flag;
    builtins = PyEval_GetBuiltins();
    if (!builtins) goto bad;
    debug_str = PyUnicode_FromStringAndSize("__debug__", 9);
    if (!debug_str) goto bad;
    debug = PyObject_GetItem(builtins, debug_str);
    Py_DECREF(debug_str);
    if (!debug) goto bad;
    flag = PyObject_IsTrue(debug);
    Py_DECREF(debug);
    if (flag == -1) goto bad;
    __pyx_assertions_enabled_flag = flag;
    return 0;
  bad:
    __pyx_assertions_enabled_flag = 1;
    return -1;
  }
#else
  #define __Pyx_init_assertions_enabled()  (0)
  #define __pyx_assertions_enabled()  (!Py_OptimizeFlag)
#endif

/* PyAssertionError_Check.proto */
#define __Pyx_PyExc_AssertionError_Check(obj)  __Pyx_TypeCheck(obj, PyExc_AssertionError)

/* AllocateExtensionType.proto */
static PyObject *__Pyx_AllocateExtensionType(PyTypeObject *t, int is_final);

/* DefaultPlacementNew.proto */
#include <new>
template<typename T>
void __Pyx_default_placement_construct(T* x) {
    new (static_cast<void*>(x)) T();
}

/* CheckTypeForFreelists.proto */
#if CYTHON_USE_FREELISTS
#if CYTHON_USE_TYPE_SPECS
#define __PYX_CHECK_FINAL_TYPE_FOR_FREELISTS(t, expected_tp, expected_size) ((int) ((t) == (expected_tp)))
#define __PYX_CHECK_TYPE_FOR_FREELIST_FLAGS  Py_TPFLAGS_IS_ABSTRACT
#else
#define __PYX_CHECK_FINAL_TYPE_FOR_FREELISTS(t, expected_tp, expected_size) ((int) ((t)->tp_basicsize == (expected_size)))
#define __PYX_CHECK_TYPE_FOR_FREELIST_FLAGS  (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)
#endif
#define __PYX_CHECK_TYPE_FOR_FREELISTS(t, expected_tp, expected_size)\
    (__PYX_CHECK_FINAL_TYPE_FOR_FREELISTS((t), (expected_tp), (expected_size)) &\
     (int) (!__Pyx_PyType_HasFeature((t), __PYX_CHECK_TYPE_FOR_FREELIST_FLAGS)))
#endif

/* PyObjectCallNoArg.proto (used by PyObjectCallMethod0) */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func);

/* PyObjectGetMethod.proto (used by PyObjectCallMethod0) */
#if !(CYTHON_VECTORCALL && (__PYX_LIMITED_VERSION_HEX >= 0x030C0000 || (!CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x03090000)))
static int __Pyx_PyObject_GetMethod(PyObject *obj, PyObject *name, PyObject **method);
#endif

/* PyObjectCallMethod0.proto (used by PyType_Ready) */
static PyObject* __Pyx_PyObject_CallMethod0(PyObject* obj, PyObject* method_name);

/* ValidateBasesTuple.proto (used by PyType_Ready) */
#if CYTHON_COMPILING_IN_CPYTHON || CYTHON_COMPILING_IN_LIMITED_API || CYTHON_USE_TYPE_SPECS
static int __Pyx_validate_bases_tuple(const char *type_name, Py_ssize_t dictoffset, PyObject *bases);
#endif

/* PyType_Ready.proto */
CYTHON_UNUSED static int __Pyx_PyType_Ready(PyTypeObject *t);

/* SetVTable.proto */
static int __Pyx_SetVtable(PyTypeObject* typeptr , void* vtable);

/* GetVTable.proto (used by MergeVTables) */
static void* __Pyx_GetVtable(PyTypeObject *type);

/* MergeVTables.proto */
static int __Pyx_MergeVtables(PyTypeObject *type);

/* DelItemOnTypeDict.proto (used by SetupReduce) */
static int __Pyx__DelItemOnTypeDict(PyTypeObject *tp, PyObject *k);
#define __Pyx_DelItemOnTypeDict(tp, k) __Pyx__DelItemOnTypeDict((PyTypeObject*)tp, k)

/* SetupReduce.proto */
static int __Pyx_setup_reduce(PyObject* type_obj);

/* TypeImport.proto */
#ifndef __PYX_HAVE_RT_ImportType_proto_3_2_3
#define __PYX_HAVE_RT_ImportType_proto_3_2_3
#if defined (__STDC_VERSION__) && __STDC_VERSION__ >= 201112L
#include <stdalign.h>
#endif
#if (defined (__STDC_VERSION__) && __STDC_VERSION__ >= 201112L) || __cplusplus >= 201103L
#define __PYX_GET_STRUCT_ALIGNMENT_3_2_3(s) alignof(s)
#else
#define __PYX_GET_STRUCT_ALIGNMENT_3_2_3(s) sizeof(void*)
#endif
enum __Pyx_ImportType_CheckSize_3_2_3 {
   __Pyx_ImportType_CheckSize_Error_3_2_3 = 0,
   __Pyx_ImportType_CheckSize_Warn_3_2_3 = 1,
   __Pyx_ImportType_CheckSize_Ignore_3_2_3 = 2
};
static PyTypeObject *__Pyx_ImportType_3_2_3(PyObject* module, const char *module_name, const char *class_name, size_t size, size_t alignment, enum __Pyx_ImportType_CheckSize_3_2_3 check_size);
#endif

/* HasAttr.proto (used by ImportImpl) */
#if __PYX_LIMITED_VERSION_HEX >= 0x030d0000
#define __Pyx_HasAttr(o, n)  PyObject_HasAttrWithError(o, n)
#else
static CYTHON_INLINE int __Pyx_HasAttr(PyObject *, PyObject *);
#endif

/* ImportImpl.export */
static PyObject *__Pyx__Import(PyObject *name, PyObject *const *imported_names, Py_ssize_t len_imported_names, PyObject *qualname, PyObject *moddict, int level);

/* Import.proto */
static CYTHON_INLINE PyObject *__Pyx_Import(PyObject *name, PyObject *const *imported_names, Py_ssize_t len_imported_names, PyObject *qualname, int level);

/* ImportFrom.proto */
static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name);

/* BuildPyUnicode.proto (used by COrdinalToPyUnicode) */
static PyObject* __Pyx_PyUnicode_BuildFromAscii(Py_ssize_t ulength, const char* chars, int clength,
                                                int prepend_sign, char padding_char);

/* COrdinalToPyUnicode.proto (used by CIntToPyUnicode) */
static CYTHON_INLINE int __Pyx_CheckUnicodeValue(int value);
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromOrdinal_Padded(int value, Py_ssize_t width, char padding_char);

/* GCCDiagnostics.proto (used by CIntToPyUnicode) */
#if !defined(__INTEL_COMPILER) && defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 6))
#define __Pyx_HAS_GCC_DIAGNOSTIC
#endif

/* IncludeStdlibH.proto (used by CIntToPyUnicode) */
#include <stdlib.h>

/* CIntToPyUnicode.proto */
#define __Pyx_PyUnicode_From_int(value, width, padding_char, format_char) (\
    ((format_char) == ('c')) ?\
        __Pyx_uchar___Pyx_PyUnicode_From_int(value, width, padding_char) :\
        __Pyx____Pyx_PyUnicode_From_int(value, width, padding_char, format_char)\
    )
static CYTHON_INLINE PyObject* __Pyx_uchar___Pyx_PyUnicode_From_int(int value, Py_ssize_t width, char padding_char);
static CYTHON_INLINE PyObject* __Pyx____Pyx_PyUnicode_From_int(int value, Py_ssize_t width, char padding_char, char format_char);

/* Py3UpdateBases.proto */
static PyObject* __Pyx_PEP560_update_bases(PyObject *bases);

/* CalculateMetaclass.proto */
static PyObject *__Pyx_CalculateMetaclass(PyTypeObject *metaclass, PyObject *bases);

/* Py3ClassCreate.proto */
static PyObject *__Pyx_Py3MetaclassPrepare(PyObject *metaclass, PyObject *bases, PyObject *name, PyObject *qualname,
                                           PyObject *mkw, PyObject *modname, PyObject *doc);
static PyObject *__Pyx_Py3ClassCreate(PyObject *metaclass, PyObject *name, PyObject *bases, PyObject *dict,
                                      PyObject *mkw, int calculate_metaclass, int allow_py2_metaclass);

/* CLineInTraceback.proto (used by AddTraceback) */
#if CYTHON_CLINE_IN_TRACEBACK && CYTHON_CLINE_IN_TRACEBACK_RUNTIME
static int __Pyx_CLineForTraceback(PyThreadState *tstate, int c_line);
#else
#define __Pyx_CLineForTraceback(tstate, c_line)  (((CYTHON_CLINE_IN_TRACEBACK)) ? c_line : 0)
#endif

/* CodeObjectCache.proto (used by AddTraceback) */
#if CYTHON_COMPILING_IN_LIMITED_API
typedef PyObject __Pyx_CachedCodeObjectType;
#else
typedef PyCodeObject __Pyx_CachedCodeObjectType;
#endif
typedef struct {
    __Pyx_CachedCodeObjectType* code_object;
    int code_line;
} __Pyx_CodeObjectCacheEntry;
struct __Pyx_CodeObjectCache {
    int count;
    int max_count;
    __Pyx_CodeObjectCacheEntry* entries;
  #if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    __pyx_atomic_int_type accessor_count;
  #endif
};
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line);
static __Pyx_CachedCodeObjectType *__pyx_find_code_object(int code_line);
static void __pyx_insert_code_object(int code_line, __Pyx_CachedCodeObjectType* code_object);

/* AddTraceback.proto */
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename);

/* CheckUnpickleChecksum.proto */
static CYTHON_INLINE int __Pyx_CheckUnpickleChecksum(long checksum, long checksum1, long checksum2, long checksum3, const char *members);

/* CIntFromPy.proto */
static CYTHON_INLINE int __Pyx_PyLong_As_int(PyObject *);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyLong_From_int(int value);

/* CIntFromPy.proto */
static CYTHON_INLINE long __Pyx_PyLong_As_long(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE size_t __Pyx_PyLong_As_size_t(PyObject *);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyLong_From_long(long value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyLong_From_uint64_t(uint64_t value);

/* PyObjectCallMethod1.proto */
static PyObject* __Pyx_PyObject_CallMethod1(PyObject* obj, PyObject* method_name, PyObject* arg);

/* UpdateUnpickledDict.proto */
static int __Pyx_UpdateUnpickledDict(PyObject *obj, PyObject *state, Py_ssize_t index);

/* CppExceptionConversion.proto */
#ifndef __Pyx_CppExn2PyErr
#include <new>
#include <typeinfo>
#include <stdexcept>
#include <ios>
static void __Pyx_CppExn2PyErr() {
  try {
    if (PyErr_Occurred())
      ; // let the latest Python exn pass through and ignore the current one
    else
      throw;
  } catch (const std::bad_alloc& exn) {
    PyErr_SetString(PyExc_MemoryError, exn.what());
  } catch (const std::bad_cast& exn) {
    PyErr_SetString(PyExc_TypeError, exn.what());
  } catch (const std::bad_typeid& exn) {
    PyErr_SetString(PyExc_TypeError, exn.what());
  } catch (const std::domain_error& exn) {
    PyErr_SetString(PyExc_ValueError, exn.what());
  } catch (const std::invalid_argument& exn) {
    PyErr_SetString(PyExc_ValueError, exn.what());
  } catch (const std::ios_base::failure& exn) {
    PyErr_SetString(PyExc_IOError, exn.what());
  } catch (const std::out_of_range& exn) {
    PyErr_SetString(PyExc_IndexError, exn.what());
  } catch (const std::overflow_error& exn) {
    PyErr_SetString(PyExc_OverflowError, exn.what());
  } catch (const std::range_error& exn) {
    PyErr_SetString(PyExc_ArithmeticError, exn.what());
  } catch (const std::underflow_error& exn) {
    PyErr_SetString(PyExc_ArithmeticError, exn.what());
  } catch (const std::exception& exn) {
    PyErr_SetString(PyExc_RuntimeError, exn.what());
  }
  catch (...)
  {
    PyErr_SetString(PyExc_RuntimeError, "Unknown exception");
  }
}
#endif

/* FormatTypeName.proto */
#if CYTHON_COMPILING_IN_LIMITED_API
typedef PyObject *__Pyx_TypeName;
#define __Pyx_FMT_TYPENAME "%U"
#define __Pyx_DECREF_TypeName(obj) Py_XDECREF(obj)
#if __PYX_LIMITED_VERSION_HEX >= 0x030d0000
#define __Pyx_PyType_GetFullyQualifiedName PyType_GetFullyQualifiedName
#else
static __Pyx_TypeName __Pyx_PyType_GetFullyQualifiedName(PyTypeObject* tp);
#endif
#else  // !LIMITED_API
typedef const char *__Pyx_TypeName;
#define __Pyx_FMT_TYPENAME "%.200s"
#define __Pyx_PyType_GetFullyQualifiedName(tp) ((tp)->tp_name)
#define __Pyx_DECREF_TypeName(obj)
#endif

/* FastTypeChecks.proto */
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_TypeCheck(obj, type) __Pyx_IsSubtype(Py_TYPE(obj), (PyTypeObject *)type)
#define __Pyx_TypeCheck2(obj, type1, type2) __Pyx_IsAnySubtype2(Py_TYPE(obj), (PyTypeObject *)type1, (PyTypeObject *)type2)
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b);
static CYTHON_INLINE int __Pyx_IsAnySubtype2(PyTypeObject *cls, PyTypeObject *a, PyTypeObject *b);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject *type);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *type1, PyObject *type2);
#else
#define __Pyx_TypeCheck(obj, type) PyObject_TypeCheck(obj, (PyTypeObject *)type)
#define __Pyx_TypeCheck2(obj, type1, type2) (PyObject_TypeCheck(obj, (PyTypeObject *)type1) || PyObject_TypeCheck(obj, (PyTypeObject *)type2))
#define __Pyx_PyErr_GivenExceptionMatches(err, type) PyErr_GivenExceptionMatches(err, type)
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *type1, PyObject *type2) {
    return PyErr_GivenExceptionMatches(err, type1) || PyErr_GivenExceptionMatches(err, type2);
}
#endif
#define __Pyx_PyErr_ExceptionMatches2(err1, err2)  __Pyx_PyErr_GivenExceptionMatches2(__Pyx_PyErr_CurrentExceptionType(), err1, err2)
#define __Pyx_PyException_Check(obj) __Pyx_TypeCheck(obj, PyExc_Exception)
#ifdef PyExceptionInstance_Check
  #define __Pyx_PyBaseException_Check(obj) PyExceptionInstance_Check(obj)
#else
  #define __Pyx_PyBaseException_Check(obj) __Pyx_TypeCheck(obj, PyExc_BaseException)
#endif

/* GetRuntimeVersion.proto */
#if __PYX_LIMITED_VERSION_HEX < 0x030b0000
static unsigned long __Pyx_cached_runtime_version = 0;
static void __Pyx_init_runtime_version(void);
#else
#define __Pyx_init_runtime_version()
#endif
static unsigned long __Pyx_get_runtime_version(void);

/* CheckBinaryVersion.proto */
static int __Pyx_check_binary_version(unsigned long ct_version, unsigned long rt_version, int allow_newer);

/* DecompressString.proto */
static PyObject *__Pyx_DecompressString(const char *s, Py_ssize_t length, int algo);

/* MultiPhaseInitModuleState.proto */
#if CYTHON_PEP489_MULTI_PHASE_INIT && CYTHON_USE_MODULE_STATE
static PyObject *__Pyx_State_FindModule(void*);
static int __Pyx_State_AddModule(PyObject* module, void*);
static int __Pyx_State_RemoveModule(void*);
#elif CYTHON_USE_MODULE_STATE
#define __Pyx_State_FindModule PyState_FindModule
#define __Pyx_State_AddModule PyState_AddModule
#define __Pyx_State_RemoveModule PyState_RemoveModule
#endif

/* #### Code section: module_declarations ### */
/* CythonABIVersion.proto */
#if CYTHON_COMPILING_IN_LIMITED_API
    #if CYTHON_METH_FASTCALL
        #define __PYX_FASTCALL_ABI_SUFFIX  "_fastcall"
    #else
        #define __PYX_FASTCALL_ABI_SUFFIX
    #endif
    #define __PYX_LIMITED_ABI_SUFFIX "limited" __PYX_FASTCALL_ABI_SUFFIX __PYX_AM_SEND_ABI_SUFFIX
#else
    #define __PYX_LIMITED_ABI_SUFFIX
#endif
#if __PYX_HAS_PY_AM_SEND == 1
    #define __PYX_AM_SEND_ABI_SUFFIX
#elif __PYX_HAS_PY_AM_SEND == 2
    #define __PYX_AM_SEND_ABI_SUFFIX "amsendbackport"
#else
    #define __PYX_AM_SEND_ABI_SUFFIX "noamsend"
#endif
#ifndef __PYX_MONITORING_ABI_SUFFIX
    #define __PYX_MONITORING_ABI_SUFFIX
#endif
#if CYTHON_USE_TP_FINALIZE
    #define __PYX_TP_FINALIZE_ABI_SUFFIX
#else
    #define __PYX_TP_FINALIZE_ABI_SUFFIX "nofinalize"
#endif
#if CYTHON_USE_FREELISTS || !defined(__Pyx_AsyncGen_USED)
    #define __PYX_FREELISTS_ABI_SUFFIX
#else
    #define __PYX_FREELISTS_ABI_SUFFIX "nofreelists"
#endif
#define CYTHON_ABI  __PYX_ABI_VERSION __PYX_LIMITED_ABI_SUFFIX __PYX_MONITORING_ABI_SUFFIX __PYX_TP_FINALIZE_ABI_SUFFIX __PYX_FREELISTS_ABI_SUFFIX __PYX_AM_SEND_ABI_SUFFIX
#define __PYX_ABI_MODULE_NAME "_cython_" CYTHON_ABI
#define __PYX_TYPE_MODULE_PREFIX __PYX_ABI_MODULE_NAME "."

#if !CYTHON_COMPILING_IN_LIMITED_API
static CYTHON_INLINE double __pyx_f_7cpython_7complex_7complex_4real_real(PyComplexObject *__pyx_v_self); /* proto*/
#endif
#if !CYTHON_COMPILING_IN_LIMITED_API
static CYTHON_INLINE double __pyx_f_7cpython_7complex_7complex_4imag_imag(PyComplexObject *__pyx_v_self); /* proto*/
#endif
static PyObject *__pyx_f_6plyvel_7_plyvel_2DB_close(struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_self, CYTHON_UNUSED int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_6plyvel_7_plyvel_12BaseIterator_close(struct __pyx_obj_6plyvel_7_plyvel_BaseIterator *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_6plyvel_7_plyvel_8Iterator_current(struct __pyx_obj_6plyvel_7_plyvel_Iterator *__pyx_v_self); /* proto*/
static PyObject *__pyx_f_6plyvel_7_plyvel_8Iterator_real_next(struct __pyx_obj_6plyvel_7_plyvel_Iterator *__pyx_v_self); /* proto*/
static PyObject *__pyx_f_6plyvel_7_plyvel_8Iterator_real_prev(struct __pyx_obj_6plyvel_7_plyvel_Iterator *__pyx_v_self); /* proto*/
static PyObject *__pyx_f_6plyvel_7_plyvel_11RawIterator_key(struct __pyx_obj_6plyvel_7_plyvel_RawIterator *__pyx_v_self, CYTHON_UNUSED int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_6plyvel_7_plyvel_11RawIterator_value(struct __pyx_obj_6plyvel_7_plyvel_RawIterator *__pyx_v_self, CYTHON_UNUSED int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_6plyvel_7_plyvel_8Snapshot_close(struct __pyx_obj_6plyvel_7_plyvel_Snapshot *__pyx_v_self, CYTHON_UNUSED int __pyx_skip_dispatch); /* proto*/

/* Module declarations from "cython" */

/* Module declarations from "cpython.version" */

/* Module declarations from "__builtin__" */

/* Module declarations from "cpython.type" */

/* Module declarations from "libc.string" */

/* Module declarations from "libc.stdio" */

/* Module declarations from "cpython.object" */

/* Module declarations from "cpython.ref" */

/* Module declarations from "cpython.exc" */

/* Module declarations from "cpython.module" */

/* Module declarations from "cpython.mem" */

/* Module declarations from "cpython.tuple" */

/* Module declarations from "cpython.list" */

/* Module declarations from "cpython.sequence" */

/* Module declarations from "cpython.mapping" */

/* Module declarations from "cpython.iterator" */

/* Module declarations from "cpython.number" */

/* Module declarations from "__builtin__" */

/* Module declarations from "cpython.bool" */

/* Module declarations from "cpython.long" */

/* Module declarations from "cpython.float" */

/* Module declarations from "__builtin__" */

/* Module declarations from "cpython.complex" */

/* Module declarations from "libc.stddef" */

/* Module declarations from "cpython.unicode" */

/* Module declarations from "cpython.pyport" */

/* Module declarations from "cpython.dict" */

/* Module declarations from "cpython.instance" */

/* Module declarations from "cpython.function" */

/* Module declarations from "cpython.method" */

/* Module declarations from "cpython.weakref" */

/* Module declarations from "cpython.getargs" */

/* Module declarations from "cpython.pythread" */

/* Module declarations from "cpython.pystate" */

/* Module declarations from "cpython.set" */

/* Module declarations from "cpython.buffer" */

/* Module declarations from "cpython.bytes" */

/* Module declarations from "cpython.pycapsule" */

/* Module declarations from "cpython.contextvars" */

/* Module declarations from "cpython" */

/* Module declarations from "libc.stdint" */

/* Module declarations from "libc.stdlib" */

/* Module declarations from "libcpp.string_view" */

/* Module declarations from "libcpp.string" */

/* Module declarations from "libcpp" */

/* Module declarations from "plyvel.leveldb" */

/* Module declarations from "plyvel.comparator" */

/* Module declarations from "plyvel._plyvel" */
static int __pyx_f_6plyvel_7_plyvel_raise_for_status(leveldb::Status); /*proto*/
static CYTHON_INLINE PyObject *__pyx_f_6plyvel_7_plyvel_db_get(struct __pyx_obj_6plyvel_7_plyvel_DB *, PyObject *, PyObject *, leveldb::ReadOptions); /*proto*/
static PyObject *__pyx_f_6plyvel_7_plyvel_to_file_system_name(PyObject *); /*proto*/
static PyObject *__pyx_f_6plyvel_7_plyvel_bytes_increment(PyObject *); /*proto*/
static int __pyx_f_6plyvel_7_plyvel_parse_options(leveldb::Options *, bool, bool, PyObject *, PyObject *, PyObject *, PyObject *, PyObject *, PyObject *, PyObject *, PyObject *, int, PyObject *, PyObject *); /*proto*/
static PyObject *__pyx_f_6plyvel_7_plyvel___pyx_unpickle_PrefixedDB__set_state(struct __pyx_obj_6plyvel_7_plyvel_PrefixedDB *, PyObject *); /*proto*/
static CYTHON_INLINE PyObject *__pyx_convert_PyObject_string_to_py_6libcpp_6string_std__in_string(std::string const &); /*proto*/
static CYTHON_INLINE PyObject *__pyx_convert_PyUnicode_string_to_py_6libcpp_6string_std__in_string(std::string const &); /*proto*/
static CYTHON_INLINE PyObject *__pyx_convert_PyBytes_string_to_py_6libcpp_6string_std__in_string(std::string const &); /*proto*/
static CYTHON_INLINE PyObject *__pyx_convert_PyByteArray_string_to_py_6libcpp_6string_std__in_string(std::string const &); /*proto*/
static std::string __pyx_convert_string_from_py_6libcpp_6string_std__in_string(PyObject *); /*proto*/
/* #### Code section: typeinfo ### */
/* #### Code section: before_global_var ### */
#define __Pyx_MODULE_NAME "plyvel._plyvel"
extern int __pyx_module_is_main_plyvel___plyvel;
int __pyx_module_is_main_plyvel___plyvel = 0;

/* Implementation of "plyvel._plyvel" */
/* #### Code section: global_var ### */
static PyObject *__pyx_builtin_id;
static PyObject *__pyx_builtin_super;
/* #### Code section: string_decls ### */
static const char __pyx_k_db_prefix_2[] = "db, prefix";
static const char __pyx_k_Plyvel_a_Python_LevelDB_interfa[] = "\nPlyvel, a Python LevelDB interface.\n\nUse plyvel.DB() to create or open a database.\n";
/* #### Code section: decls ### */
static int __pyx_pf_6plyvel_7_plyvel_2DB___init__(struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_self, PyObject *__pyx_v_name, PyLongObject *__pyx_v_create_if_missing, PyLongObject *__pyx_v_error_if_exists, PyObject *__pyx_v_paranoid_checks, PyObject *__pyx_v_write_buffer_size, PyObject *__pyx_v_max_open_files, PyObject *__pyx_v_lru_cache_size, PyObject *__pyx_v_block_size, PyObject *__pyx_v_block_restart_interval, PyObject *__pyx_v_max_file_size, PyObject *__pyx_v_compression, int __pyx_v_bloom_filter_bits, PyObject *__pyx_v_comparator, PyObject *__pyx_v_comparator_name); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_2DB_2close(struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_2DB_6closed___get__(struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_self); /* proto */
static void __pyx_pf_6plyvel_7_plyvel_2DB_4__dealloc__(struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_2DB_6__repr__(struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_2DB_8get(struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_self, PyObject *__pyx_v_key, PyObject *__pyx_v_default, PyLongObject *__pyx_v_verify_checksums, PyLongObject *__pyx_v_fill_cache); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_2DB_10put(struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_self, PyObject *__pyx_v_key, PyObject *__pyx_v_value, PyLongObject *__pyx_v_sync); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_2DB_12delete(struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_self, PyObject *__pyx_v_key, PyLongObject *__pyx_v_sync); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_2DB_14write_batch(struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_self, PyLongObject *__pyx_v_transaction, PyLongObject *__pyx_v_sync); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_2DB_16__iter__(struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_2DB_18iterator(struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_self, PyObject *__pyx_v_reverse, PyObject *__pyx_v_start, PyObject *__pyx_v_stop, PyObject *__pyx_v_include_start, PyObject *__pyx_v_include_stop, PyObject *__pyx_v_prefix, PyObject *__pyx_v_include_key, PyObject *__pyx_v_include_value, PyLongObject *__pyx_v_verify_checksums, PyLongObject *__pyx_v_fill_cache); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_2DB_20raw_iterator(struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_self, PyLongObject *__pyx_v_verify_checksums, PyLongObject *__pyx_v_fill_cache); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_2DB_22snapshot(struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_2DB_24get_property(struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_self, PyObject *__pyx_v_name); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_2DB_26compact_range(struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_self, PyObject *__pyx_v_start, PyObject *__pyx_v_stop); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_2DB_28approximate_size(struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_self, PyObject *__pyx_v_start, PyObject *__pyx_v_stop); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_2DB_30approximate_sizes(struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_self, PyObject *__pyx_v_ranges); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_2DB_32prefixed_db(struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_self, PyObject *__pyx_v_prefix); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_2DB_34__enter__(struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_2DB_36__exit__(struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v_exc_type, CYTHON_UNUSED PyObject *__pyx_v_exc_val, CYTHON_UNUSED PyObject *__pyx_v_exc_tb); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_2DB_4name___get__(struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_2DB_38__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_2DB_40__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_6plyvel_7_plyvel_10PrefixedDB___init__(struct __pyx_obj_6plyvel_7_plyvel_PrefixedDB *__pyx_v_self, struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_db, PyObject *__pyx_v_prefix); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_10PrefixedDB_2__repr__(struct __pyx_obj_6plyvel_7_plyvel_PrefixedDB *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_10PrefixedDB_4get(struct __pyx_obj_6plyvel_7_plyvel_PrefixedDB *__pyx_v_self, PyObject *__pyx_v_key, PyObject *__pyx_v_default, PyLongObject *__pyx_v_verify_checksums, PyLongObject *__pyx_v_fill_cache); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_10PrefixedDB_6put(struct __pyx_obj_6plyvel_7_plyvel_PrefixedDB *__pyx_v_self, PyObject *__pyx_v_key, PyObject *__pyx_v_value, PyLongObject *__pyx_v_sync); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_10PrefixedDB_8delete(struct __pyx_obj_6plyvel_7_plyvel_PrefixedDB *__pyx_v_self, PyObject *__pyx_v_key, PyLongObject *__pyx_v_sync); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_10PrefixedDB_10write_batch(struct __pyx_obj_6plyvel_7_plyvel_PrefixedDB *__pyx_v_self, PyObject *__pyx_v_transaction, PyLongObject *__pyx_v_sync); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_10PrefixedDB_12__iter__(struct __pyx_obj_6plyvel_7_plyvel_PrefixedDB *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_10PrefixedDB_14iterator(struct __pyx_obj_6plyvel_7_plyvel_PrefixedDB *__pyx_v_self, PyObject *__pyx_v_reverse, PyObject *__pyx_v_start, PyObject *__pyx_v_stop, PyObject *__pyx_v_include_start, PyObject *__pyx_v_include_stop, PyObject *__pyx_v_prefix, PyObject *__pyx_v_include_key, PyObject *__pyx_v_include_value, PyLongObject *__pyx_v_verify_checksums, PyLongObject *__pyx_v_fill_cache); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_10PrefixedDB_16snapshot(struct __pyx_obj_6plyvel_7_plyvel_PrefixedDB *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_10PrefixedDB_18prefixed_db(struct __pyx_obj_6plyvel_7_plyvel_PrefixedDB *__pyx_v_self, PyObject *__pyx_v_prefix); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_10PrefixedDB_2db___get__(struct __pyx_obj_6plyvel_7_plyvel_PrefixedDB *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_10PrefixedDB_6prefix___get__(struct __pyx_obj_6plyvel_7_plyvel_PrefixedDB *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_10PrefixedDB_20__reduce_cython__(struct __pyx_obj_6plyvel_7_plyvel_PrefixedDB *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_10PrefixedDB_22__setstate_cython__(struct __pyx_obj_6plyvel_7_plyvel_PrefixedDB *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_repair_db(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_name, PyObject *__pyx_v_paranoid_checks, PyObject *__pyx_v_write_buffer_size, PyObject *__pyx_v_max_open_files, PyObject *__pyx_v_lru_cache_size, PyObject *__pyx_v_block_size, PyObject *__pyx_v_block_restart_interval, PyObject *__pyx_v_max_file_size, PyObject *__pyx_v_compression, int __pyx_v_bloom_filter_bits, PyObject *__pyx_v_comparator, PyObject *__pyx_v_comparator_name); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_2destroy_db(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_name); /* proto */
static int __pyx_pf_6plyvel_7_plyvel_10WriteBatch___init__(struct __pyx_obj_6plyvel_7_plyvel_WriteBatch *__pyx_v_self, struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_db, PyObject *__pyx_v_prefix, PyLongObject *__pyx_v_transaction, PyObject *__pyx_v_sync); /* proto */
static void __pyx_pf_6plyvel_7_plyvel_10WriteBatch_2__dealloc__(struct __pyx_obj_6plyvel_7_plyvel_WriteBatch *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_10WriteBatch_4put(struct __pyx_obj_6plyvel_7_plyvel_WriteBatch *__pyx_v_self, PyObject *__pyx_v_key, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_10WriteBatch_6delete(struct __pyx_obj_6plyvel_7_plyvel_WriteBatch *__pyx_v_self, PyObject *__pyx_v_key); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_10WriteBatch_8clear(struct __pyx_obj_6plyvel_7_plyvel_WriteBatch *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_10WriteBatch_10write(struct __pyx_obj_6plyvel_7_plyvel_WriteBatch *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_10WriteBatch_12approximate_size(struct __pyx_obj_6plyvel_7_plyvel_WriteBatch *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_10WriteBatch_14append(struct __pyx_obj_6plyvel_7_plyvel_WriteBatch *__pyx_v_self, struct __pyx_obj_6plyvel_7_plyvel_WriteBatch *__pyx_v_source); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_10WriteBatch_16__enter__(struct __pyx_obj_6plyvel_7_plyvel_WriteBatch *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_10WriteBatch_18__exit__(struct __pyx_obj_6plyvel_7_plyvel_WriteBatch *__pyx_v_self, PyObject *__pyx_v_exc_type, CYTHON_UNUSED PyObject *__pyx_v_exc_val, CYTHON_UNUSED PyObject *__pyx_v_exc_tb); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_10WriteBatch_20__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_6plyvel_7_plyvel_WriteBatch *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_10WriteBatch_22__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_6plyvel_7_plyvel_WriteBatch *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_lambda_funcdef_lambda(PyObject *__pyx_self, CYTHON_UNUSED PyObject *__pyx_v_wr); /* proto */
static int __pyx_pf_6plyvel_7_plyvel_12BaseIterator___init__(struct __pyx_obj_6plyvel_7_plyvel_BaseIterator *__pyx_v_self, struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_db, PyLongObject *__pyx_v_verify_checksums, PyLongObject *__pyx_v_fill_cache, struct __pyx_obj_6plyvel_7_plyvel_Snapshot *__pyx_v_snapshot); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_12BaseIterator_2close(struct __pyx_obj_6plyvel_7_plyvel_BaseIterator *__pyx_v_self); /* proto */
static void __pyx_pf_6plyvel_7_plyvel_12BaseIterator_4__dealloc__(struct __pyx_obj_6plyvel_7_plyvel_BaseIterator *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_12BaseIterator_6__enter__(struct __pyx_obj_6plyvel_7_plyvel_BaseIterator *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_12BaseIterator_8__exit__(struct __pyx_obj_6plyvel_7_plyvel_BaseIterator *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v_exc_type, CYTHON_UNUSED PyObject *__pyx_v_exc_val, CYTHON_UNUSED PyObject *__pyx_v_exc_tb); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_12BaseIterator_10__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_6plyvel_7_plyvel_BaseIterator *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_12BaseIterator_12__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_6plyvel_7_plyvel_BaseIterator *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_6plyvel_7_plyvel_8Iterator___init__(struct __pyx_obj_6plyvel_7_plyvel_Iterator *__pyx_v_self, struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_db, PyObject *__pyx_v_db_prefix, PyLongObject *__pyx_v_reverse, PyObject *__pyx_v_start, PyObject *__pyx_v_stop, PyLongObject *__pyx_v_include_start, PyLongObject *__pyx_v_include_stop, PyObject *__pyx_v_prefix, PyLongObject *__pyx_v_include_key, PyLongObject *__pyx_v_include_value, PyLongObject *__pyx_v_verify_checksums, PyLongObject *__pyx_v_fill_cache, struct __pyx_obj_6plyvel_7_plyvel_Snapshot *__pyx_v_snapshot); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_8Iterator_2__iter__(struct __pyx_obj_6plyvel_7_plyvel_Iterator *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_8Iterator_4__next__(struct __pyx_obj_6plyvel_7_plyvel_Iterator *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_8Iterator_6prev(struct __pyx_obj_6plyvel_7_plyvel_Iterator *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_8Iterator_8seek_to_start(struct __pyx_obj_6plyvel_7_plyvel_Iterator *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_8Iterator_10seek_to_stop(struct __pyx_obj_6plyvel_7_plyvel_Iterator *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_8Iterator_12seek(struct __pyx_obj_6plyvel_7_plyvel_Iterator *__pyx_v_self, PyObject *__pyx_v_target); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_8Iterator_14__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_6plyvel_7_plyvel_Iterator *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_8Iterator_16__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_6plyvel_7_plyvel_Iterator *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_11RawIterator_valid(struct __pyx_obj_6plyvel_7_plyvel_RawIterator *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_11RawIterator_2seek_to_first(struct __pyx_obj_6plyvel_7_plyvel_RawIterator *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_11RawIterator_4seek_to_last(struct __pyx_obj_6plyvel_7_plyvel_RawIterator *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_11RawIterator_6seek(struct __pyx_obj_6plyvel_7_plyvel_RawIterator *__pyx_v_self, PyObject *__pyx_v_target); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_11RawIterator_8next(struct __pyx_obj_6plyvel_7_plyvel_RawIterator *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_11RawIterator_10prev(struct __pyx_obj_6plyvel_7_plyvel_RawIterator *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_11RawIterator_12key(struct __pyx_obj_6plyvel_7_plyvel_RawIterator *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_11RawIterator_14value(struct __pyx_obj_6plyvel_7_plyvel_RawIterator *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_11RawIterator_16item(struct __pyx_obj_6plyvel_7_plyvel_RawIterator *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_11RawIterator_18__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_6plyvel_7_plyvel_RawIterator *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_11RawIterator_20__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_6plyvel_7_plyvel_RawIterator *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_6plyvel_7_plyvel_8Snapshot___init__(struct __pyx_obj_6plyvel_7_plyvel_Snapshot *__pyx_v_self, struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_db, PyObject *__pyx_v_prefix); /* proto */
static void __pyx_pf_6plyvel_7_plyvel_8Snapshot_2__dealloc__(struct __pyx_obj_6plyvel_7_plyvel_Snapshot *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_8Snapshot_4close(struct __pyx_obj_6plyvel_7_plyvel_Snapshot *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_8Snapshot_6release(struct __pyx_obj_6plyvel_7_plyvel_Snapshot *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_8Snapshot_8__enter__(struct __pyx_obj_6plyvel_7_plyvel_Snapshot *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_8Snapshot_10__exit__(struct __pyx_obj_6plyvel_7_plyvel_Snapshot *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v_exc_type, CYTHON_UNUSED PyObject *__pyx_v_exc_val, CYTHON_UNUSED PyObject *__pyx_v_exc_tb); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_8Snapshot_12get(struct __pyx_obj_6plyvel_7_plyvel_Snapshot *__pyx_v_self, PyObject *__pyx_v_key, PyObject *__pyx_v_default, PyLongObject *__pyx_v_verify_checksums, PyLongObject *__pyx_v_fill_cache); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_8Snapshot_14__iter__(struct __pyx_obj_6plyvel_7_plyvel_Snapshot *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_8Snapshot_16iterator(struct __pyx_obj_6plyvel_7_plyvel_Snapshot *__pyx_v_self, PyObject *__pyx_v_reverse, PyObject *__pyx_v_start, PyObject *__pyx_v_stop, PyObject *__pyx_v_include_start, PyObject *__pyx_v_include_stop, PyObject *__pyx_v_prefix, PyObject *__pyx_v_include_key, PyObject *__pyx_v_include_value, PyLongObject *__pyx_v_verify_checksums, PyLongObject *__pyx_v_fill_cache); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_8Snapshot_18raw_iterator(struct __pyx_obj_6plyvel_7_plyvel_Snapshot *__pyx_v_self, PyLongObject *__pyx_v_verify_checksums, PyLongObject *__pyx_v_fill_cache); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_8Snapshot_20__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_6plyvel_7_plyvel_Snapshot *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_8Snapshot_22__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_6plyvel_7_plyvel_Snapshot *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_6plyvel_7_plyvel_4__pyx_unpickle_PrefixedDB(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_tp_new_6plyvel_7_plyvel_DB(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_6plyvel_7_plyvel_PrefixedDB(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_6plyvel_7_plyvel_WriteBatch(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_6plyvel_7_plyvel_BaseIterator(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_6plyvel_7_plyvel_Iterator(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_6plyvel_7_plyvel_RawIterator(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_6plyvel_7_plyvel_Snapshot(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_6plyvel_7_plyvel___pyx_scope_struct____init__(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
/* #### Code section: late_includes ### */
/* #### Code section: module_state ### */
/* SmallCodeConfig */
#ifndef CYTHON_SMALL_CODE
#if defined(__clang__)
    #define CYTHON_SMALL_CODE
#elif defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 3))
    #define CYTHON_SMALL_CODE __attribute__((cold))
#else
    #define CYTHON_SMALL_CODE
#endif
#endif

typedef struct {
  PyObject *__pyx_d;
  PyObject *__pyx_b;
  PyObject *__pyx_cython_runtime;
  PyObject *__pyx_empty_tuple;
  PyObject *__pyx_empty_bytes;
  PyObject *__pyx_empty_unicode;
  PyTypeObject *__pyx_ptype_7cpython_4type_type;
  PyTypeObject *__pyx_ptype_7cpython_4bool_bool;
  PyTypeObject *__pyx_ptype_7cpython_7complex_complex;
  PyObject *__pyx_type_6plyvel_7_plyvel_DB;
  PyObject *__pyx_type_6plyvel_7_plyvel_PrefixedDB;
  PyObject *__pyx_type_6plyvel_7_plyvel_WriteBatch;
  PyObject *__pyx_type_6plyvel_7_plyvel_BaseIterator;
  PyObject *__pyx_type_6plyvel_7_plyvel_Iterator;
  PyObject *__pyx_type_6plyvel_7_plyvel_RawIterator;
  PyObject *__pyx_type_6plyvel_7_plyvel_Snapshot;
  PyObject *__pyx_type_6plyvel_7_plyvel___pyx_scope_struct____init__;
  PyTypeObject *__pyx_ptype_6plyvel_7_plyvel_DB;
  PyTypeObject *__pyx_ptype_6plyvel_7_plyvel_PrefixedDB;
  PyTypeObject *__pyx_ptype_6plyvel_7_plyvel_WriteBatch;
  PyTypeObject *__pyx_ptype_6plyvel_7_plyvel_BaseIterator;
  PyTypeObject *__pyx_ptype_6plyvel_7_plyvel_Iterator;
  PyTypeObject *__pyx_ptype_6plyvel_7_plyvel_RawIterator;
  PyTypeObject *__pyx_ptype_6plyvel_7_plyvel_Snapshot;
  PyTypeObject *__pyx_ptype_6plyvel_7_plyvel___pyx_scope_struct____init__;
  __Pyx_CachedCFunction __pyx_umethod_PyDict_Type_items;
  __Pyx_CachedCFunction __pyx_umethod_PyDict_Type_pop;
  __Pyx_CachedCFunction __pyx_umethod_PyDict_Type_values;
  PyObject *__pyx_tuple[3];
  PyObject *__pyx_codeobj_tab[71];
  PyObject *__pyx_string_tab[311];
  PyObject *__pyx_number_tab[1];
/* #### Code section: module_state_contents ### */
/* CommonTypesMetaclass.module_state_decls */
PyTypeObject *__pyx_CommonTypesMetaclassType;

/* CachedMethodType.module_state_decls */
#if CYTHON_COMPILING_IN_LIMITED_API
PyObject *__Pyx_CachedMethodType;
#endif

/* CythonFunctionShared.module_state_decls */
PyTypeObject *__pyx_CyFunctionType;


#if CYTHON_USE_FREELISTS
struct __pyx_obj_6plyvel_7_plyvel___pyx_scope_struct____init__ *__pyx_freelist_6plyvel_7_plyvel___pyx_scope_struct____init__[8];
int __pyx_freecount_6plyvel_7_plyvel___pyx_scope_struct____init__;
#endif
/* CodeObjectCache.module_state_decls */
struct __Pyx_CodeObjectCache __pyx_code_cache;

/* #### Code section: module_state_end ### */
} __pyx_mstatetype;

#if CYTHON_USE_MODULE_STATE
#ifdef __cplusplus
namespace {
extern struct PyModuleDef __pyx_moduledef;
} /* anonymous namespace */
#else
static struct PyModuleDef __pyx_moduledef;
#endif

#define __pyx_mstate_global (__Pyx_PyModule_GetState(__Pyx_State_FindModule(&__pyx_moduledef)))

#define __pyx_m (__Pyx_State_FindModule(&__pyx_moduledef))
#else
static __pyx_mstatetype __pyx_mstate_global_static =
#ifdef __cplusplus
    {};
#else
    {0};
#endif
static __pyx_mstatetype * const __pyx_mstate_global = &__pyx_mstate_global_static;
#endif
/* #### Code section: constant_name_defines ### */
#define __pyx_kp_u_ __pyx_string_tab[0]
#define __pyx_kp_u_Cannot_convert_unicode_name_to_a __pyx_string_tab[1]
#define __pyx_kp_u_Cannot_operate_on_closed_LevelDB __pyx_string_tab[2]
#define __pyx_kp_u_Database_is_closed __pyx_string_tab[3]
#define __pyx_kp_u_Database_or_iterator_is_closed __pyx_string_tab[4]
#define __pyx_kp_u_Database_or_snapshot_is_closed __pyx_string_tab[5]
#define __pyx_kp_u_None __pyx_string_tab[6]
#define __pyx_kp_u_Note_that_Cython_is_deliberately __pyx_string_tab[7]
#define __pyx_kp_u_Start_and_stop_of_range_must_be __pyx_string_tab[8]
#define __pyx_kp_u_UTF_8 __pyx_string_tab[9]
#define __pyx_kp_u__2 __pyx_string_tab[10]
#define __pyx_kp_u__3 __pyx_string_tab[11]
#define __pyx_kp_u__4 __pyx_string_tab[12]
#define __pyx_kp_u_add_note __pyx_string_tab[13]
#define __pyx_kp_u_at_0x __pyx_string_tab[14]
#define __pyx_kp_u_closed_2 __pyx_string_tab[15]
#define __pyx_kp_u_comparator_and_comparator_name __pyx_string_tab[16]
#define __pyx_kp_u_compression_must_be_None_or_a_s __pyx_string_tab[17]
#define __pyx_kp_u_compression_must_be_None_or_sna __pyx_string_tab[18]
#define __pyx_kp_u_custom_comparator_object_must_be __pyx_string_tab[19]
#define __pyx_kp_u_disable __pyx_string_tab[20]
#define __pyx_kp_u_enable __pyx_string_tab[21]
#define __pyx_kp_u_gc __pyx_string_tab[22]
#define __pyx_kp_u_isenabled __pyx_string_tab[23]
#define __pyx_kp_u_name_arg_must_be_a_byte_string __pyx_string_tab[24]
#define __pyx_kp_u_plyvel_DB_with_name __pyx_string_tab[25]
#define __pyx_kp_u_plyvel_PrefixedDB_with_prefix __pyx_string_tab[26]
#define __pyx_kp_u_plyvel__plyvel_pyx __pyx_string_tab[27]
#define __pyx_kp_u_prefix_cannot_be_used_together __pyx_string_tab[28]
#define __pyx_kp_u_self__db_self_options_cannot_be __pyx_string_tab[29]
#define __pyx_kp_u_self__iter_cannot_be_converted_t __pyx_string_tab[30]
#define __pyx_kp_u_self__iter_self_comparator_self __pyx_string_tab[31]
#define __pyx_kp_u_self__snapshot_cannot_be_convert __pyx_string_tab[32]
#define __pyx_kp_u_self__write_batch_self_write_opt __pyx_string_tab[33]
#define __pyx_kp_u_self_db_is_not_None_or_self_pref __pyx_string_tab[34]
#define __pyx_kp_u_stringsource __pyx_string_tab[35]
#define __pyx_n_u_BaseIterator __pyx_string_tab[36]
#define __pyx_n_u_BaseIterator___enter __pyx_string_tab[37]
#define __pyx_n_u_BaseIterator___exit __pyx_string_tab[38]
#define __pyx_n_u_BaseIterator___reduce_cython __pyx_string_tab[39]
#define __pyx_n_u_BaseIterator___setstate_cython __pyx_string_tab[40]
#define __pyx_n_u_BaseIterator_close __pyx_string_tab[41]
#define __pyx_n_u_CorruptionError __pyx_string_tab[42]
#define __pyx_n_u_DB __pyx_string_tab[43]
#define __pyx_n_u_DB___enter __pyx_string_tab[44]
#define __pyx_n_u_DB___exit __pyx_string_tab[45]
#define __pyx_n_u_DB___reduce_cython __pyx_string_tab[46]
#define __pyx_n_u_DB___setstate_cython __pyx_string_tab[47]
#define __pyx_n_u_DB_approximate_size __pyx_string_tab[48]
#define __pyx_n_u_DB_approximate_sizes __pyx_string_tab[49]
#define __pyx_n_u_DB_close __pyx_string_tab[50]
#define __pyx_n_u_DB_compact_range __pyx_string_tab[51]
#define __pyx_n_u_DB_delete __pyx_string_tab[52]
#define __pyx_n_u_DB_get __pyx_string_tab[53]
#define __pyx_n_u_DB_get_property __pyx_string_tab[54]
#define __pyx_n_u_DB_iterator __pyx_string_tab[55]
#define __pyx_n_u_DB_prefixed_db __pyx_string_tab[56]
#define __pyx_n_u_DB_put __pyx_string_tab[57]
#define __pyx_n_u_DB_raw_iterator __pyx_string_tab[58]
#define __pyx_n_u_DB_snapshot __pyx_string_tab[59]
#define __pyx_n_u_DB_write_batch __pyx_string_tab[60]
#define __pyx_n_u_Error __pyx_string_tab[61]
#define __pyx_n_u_IOError __pyx_string_tab[62]
#define __pyx_n_u_Iterator __pyx_string_tab[63]
#define __pyx_n_u_IteratorInvalidError __pyx_string_tab[64]
#define __pyx_n_u_Iterator___reduce_cython __pyx_string_tab[65]
#define __pyx_n_u_Iterator___setstate_cython __pyx_string_tab[66]
#define __pyx_n_u_Iterator_prev __pyx_string_tab[67]
#define __pyx_n_u_Iterator_seek __pyx_string_tab[68]
#define __pyx_n_u_Iterator_seek_to_start __pyx_string_tab[69]
#define __pyx_n_u_Iterator_seek_to_stop __pyx_string_tab[70]
#define __pyx_n_u_Lock __pyx_string_tab[71]
#define __pyx_n_u_PrefixedDB __pyx_string_tab[72]
#define __pyx_n_u_PrefixedDB___reduce_cython __pyx_string_tab[73]
#define __pyx_n_u_PrefixedDB___setstate_cython __pyx_string_tab[74]
#define __pyx_n_u_PrefixedDB_delete __pyx_string_tab[75]
#define __pyx_n_u_PrefixedDB_get __pyx_string_tab[76]
#define __pyx_n_u_PrefixedDB_iterator __pyx_string_tab[77]
#define __pyx_n_u_PrefixedDB_prefixed_db __pyx_string_tab[78]
#define __pyx_n_u_PrefixedDB_put __pyx_string_tab[79]
#define __pyx_n_u_PrefixedDB_snapshot __pyx_string_tab[80]
#define __pyx_n_u_PrefixedDB_write_batch __pyx_string_tab[81]
#define __pyx_n_u_Pyx_PyDict_NextRef __pyx_string_tab[82]
#define __pyx_n_u_RawIterator __pyx_string_tab[83]
#define __pyx_n_u_RawIterator___reduce_cython __pyx_string_tab[84]
#define __pyx_n_u_RawIterator___setstate_cython __pyx_string_tab[85]
#define __pyx_n_u_RawIterator_item __pyx_string_tab[86]
#define __pyx_n_u_RawIterator_key __pyx_string_tab[87]
#define __pyx_n_u_RawIterator_next __pyx_string_tab[88]
#define __pyx_n_u_RawIterator_prev __pyx_string_tab[89]
#define __pyx_n_u_RawIterator_seek __pyx_string_tab[90]
#define __pyx_n_u_RawIterator_seek_to_first __pyx_string_tab[91]
#define __pyx_n_u_RawIterator_seek_to_last __pyx_string_tab[92]
#define __pyx_n_u_RawIterator_valid __pyx_string_tab[93]
#define __pyx_n_u_RawIterator_value __pyx_string_tab[94]
#define __pyx_n_u_Snapshot __pyx_string_tab[95]
#define __pyx_n_u_Snapshot___enter __pyx_string_tab[96]
#define __pyx_n_u_Snapshot___exit __pyx_string_tab[97]
#define __pyx_n_u_Snapshot___reduce_cython __pyx_string_tab[98]
#define __pyx_n_u_Snapshot___setstate_cython __pyx_string_tab[99]
#define __pyx_n_u_Snapshot_close __pyx_string_tab[100]
#define __pyx_n_u_Snapshot_get __pyx_string_tab[101]
#define __pyx_n_u_Snapshot_iterator __pyx_string_tab[102]
#define __pyx_n_u_Snapshot_raw_iterator __pyx_string_tab[103]
#define __pyx_n_u_Snapshot_release __pyx_string_tab[104]
#define __pyx_n_u_WriteBatch __pyx_string_tab[105]
#define __pyx_n_u_WriteBatch___enter __pyx_string_tab[106]
#define __pyx_n_u_WriteBatch___exit __pyx_string_tab[107]
#define __pyx_n_u_WriteBatch___reduce_cython __pyx_string_tab[108]
#define __pyx_n_u_WriteBatch___setstate_cython __pyx_string_tab[109]
#define __pyx_n_u_WriteBatch_append __pyx_string_tab[110]
#define __pyx_n_u_WriteBatch_approximate_size __pyx_string_tab[111]
#define __pyx_n_u_WriteBatch_clear __pyx_string_tab[112]
#define __pyx_n_u_WriteBatch_delete __pyx_string_tab[113]
#define __pyx_n_u_WriteBatch_put __pyx_string_tab[114]
#define __pyx_n_u_WriteBatch_write __pyx_string_tab[115]
#define __pyx_n_u_append __pyx_string_tab[116]
#define __pyx_n_u_approximate_size __pyx_string_tab[117]
#define __pyx_n_u_approximate_sizes __pyx_string_tab[118]
#define __pyx_n_u_ascii __pyx_string_tab[119]
#define __pyx_n_u_asyncio_coroutines __pyx_string_tab[120]
#define __pyx_n_u_block_restart_interval __pyx_string_tab[121]
#define __pyx_n_u_block_size __pyx_string_tab[122]
#define __pyx_n_u_bloom_filter_bits __pyx_string_tab[123]
#define __pyx_n_u_c_ranges __pyx_string_tab[124]
#define __pyx_n_u_clear __pyx_string_tab[125]
#define __pyx_n_u_cline_in_traceback __pyx_string_tab[126]
#define __pyx_n_u_close __pyx_string_tab[127]
#define __pyx_n_u_closed __pyx_string_tab[128]
#define __pyx_n_u_compact_range __pyx_string_tab[129]
#define __pyx_n_u_comparator __pyx_string_tab[130]
#define __pyx_n_u_comparator_name __pyx_string_tab[131]
#define __pyx_n_u_compression __pyx_string_tab[132]
#define __pyx_n_u_create_if_missing __pyx_string_tab[133]
#define __pyx_n_u_d __pyx_string_tab[134]
#define __pyx_n_u_db __pyx_string_tab[135]
#define __pyx_n_u_db_prefix __pyx_string_tab[136]
#define __pyx_n_u_decode __pyx_string_tab[137]
#define __pyx_n_u_default __pyx_string_tab[138]
#define __pyx_n_u_delete __pyx_string_tab[139]
#define __pyx_n_u_destroy_db __pyx_string_tab[140]
#define __pyx_n_u_dict __pyx_string_tab[141]
#define __pyx_n_u_dict_2 __pyx_string_tab[142]
#define __pyx_n_u_doc __pyx_string_tab[143]
#define __pyx_n_u_encode __pyx_string_tab[144]
#define __pyx_n_u_enter __pyx_string_tab[145]
#define __pyx_n_u_error_if_exists __pyx_string_tab[146]
#define __pyx_n_u_exc_tb __pyx_string_tab[147]
#define __pyx_n_u_exc_type __pyx_string_tab[148]
#define __pyx_n_u_exc_val __pyx_string_tab[149]
#define __pyx_n_u_exit __pyx_string_tab[150]
#define __pyx_n_u_fill_cache __pyx_string_tab[151]
#define __pyx_n_u_fsname __pyx_string_tab[152]
#define __pyx_n_u_func __pyx_string_tab[153]
#define __pyx_n_u_get __pyx_string_tab[154]
#define __pyx_n_u_get_property __pyx_string_tab[155]
#define __pyx_n_u_getfilesystemencoding __pyx_string_tab[156]
#define __pyx_n_u_getstate __pyx_string_tab[157]
#define __pyx_n_u_i __pyx_string_tab[158]
#define __pyx_n_u_id __pyx_string_tab[159]
#define __pyx_n_u_include_key __pyx_string_tab[160]
#define __pyx_n_u_include_start __pyx_string_tab[161]
#define __pyx_n_u_include_stop __pyx_string_tab[162]
#define __pyx_n_u_include_value __pyx_string_tab[163]
#define __pyx_n_u_init __pyx_string_tab[164]
#define __pyx_n_u_init___locals_lambda __pyx_string_tab[165]
#define __pyx_n_u_is_coroutine __pyx_string_tab[166]
#define __pyx_n_u_item __pyx_string_tab[167]
#define __pyx_n_u_items __pyx_string_tab[168]
#define __pyx_n_u_iterator __pyx_string_tab[169]
#define __pyx_n_u_key __pyx_string_tab[170]
#define __pyx_n_u_key_slice __pyx_string_tab[171]
#define __pyx_n_u_lambda __pyx_string_tab[172]
#define __pyx_n_u_leveldb_version __pyx_string_tab[173]
#define __pyx_n_u_lru_cache_size __pyx_string_tab[174]
#define __pyx_n_u_main __pyx_string_tab[175]
#define __pyx_n_u_max_file_size __pyx_string_tab[176]
#define __pyx_n_u_max_open_files __pyx_string_tab[177]
#define __pyx_n_u_metaclass __pyx_string_tab[178]
#define __pyx_n_u_module __pyx_string_tab[179]
#define __pyx_n_u_mro_entries __pyx_string_tab[180]
#define __pyx_n_u_n_ranges __pyx_string_tab[181]
#define __pyx_n_u_name __pyx_string_tab[182]
#define __pyx_n_u_name_2 __pyx_string_tab[183]
#define __pyx_n_u_new __pyx_string_tab[184]
#define __pyx_n_u_next __pyx_string_tab[185]
#define __pyx_n_u_options __pyx_string_tab[186]
#define __pyx_n_u_paranoid_checks __pyx_string_tab[187]
#define __pyx_n_u_plyvel__plyvel __pyx_string_tab[188]
#define __pyx_n_u_pop __pyx_string_tab[189]
#define __pyx_n_u_popitem __pyx_string_tab[190]
#define __pyx_n_u_prefix __pyx_string_tab[191]
#define __pyx_n_u_prefixed_db __pyx_string_tab[192]
#define __pyx_n_u_prepare __pyx_string_tab[193]
#define __pyx_n_u_prev __pyx_string_tab[194]
#define __pyx_n_u_put __pyx_string_tab[195]
#define __pyx_n_u_pyx_checksum __pyx_string_tab[196]
#define __pyx_n_u_pyx_result __pyx_string_tab[197]
#define __pyx_n_u_pyx_state __pyx_string_tab[198]
#define __pyx_n_u_pyx_type __pyx_string_tab[199]
#define __pyx_n_u_pyx_unpickle_PrefixedDB __pyx_string_tab[200]
#define __pyx_n_u_pyx_vtable __pyx_string_tab[201]
#define __pyx_n_u_qualname __pyx_string_tab[202]
#define __pyx_n_u_ranges __pyx_string_tab[203]
#define __pyx_n_u_raw_iterator __pyx_string_tab[204]
#define __pyx_n_u_read_options __pyx_string_tab[205]
#define __pyx_n_u_reduce __pyx_string_tab[206]
#define __pyx_n_u_reduce_cython __pyx_string_tab[207]
#define __pyx_n_u_reduce_ex __pyx_string_tab[208]
#define __pyx_n_u_ref __pyx_string_tab[209]
#define __pyx_n_u_release __pyx_string_tab[210]
#define __pyx_n_u_repair_db __pyx_string_tab[211]
#define __pyx_n_u_result __pyx_string_tab[212]
#define __pyx_n_u_reverse __pyx_string_tab[213]
#define __pyx_n_u_seek __pyx_string_tab[214]
#define __pyx_n_u_seek_to_first __pyx_string_tab[215]
#define __pyx_n_u_seek_to_last __pyx_string_tab[216]
#define __pyx_n_u_seek_to_start __pyx_string_tab[217]
#define __pyx_n_u_seek_to_stop __pyx_string_tab[218]
#define __pyx_n_u_self __pyx_string_tab[219]
#define __pyx_n_u_set_name __pyx_string_tab[220]
#define __pyx_n_u_setdefault __pyx_string_tab[221]
#define __pyx_n_u_setstate __pyx_string_tab[222]
#define __pyx_n_u_setstate_cython __pyx_string_tab[223]
#define __pyx_n_u_sizes __pyx_string_tab[224]
#define __pyx_n_u_sl __pyx_string_tab[225]
#define __pyx_n_u_snappy __pyx_string_tab[226]
#define __pyx_n_u_snapshot __pyx_string_tab[227]
#define __pyx_n_u_source __pyx_string_tab[228]
#define __pyx_n_u_st __pyx_string_tab[229]
#define __pyx_n_u_start __pyx_string_tab[230]
#define __pyx_n_u_start_slice __pyx_string_tab[231]
#define __pyx_n_u_state __pyx_string_tab[232]
#define __pyx_n_u_stop __pyx_string_tab[233]
#define __pyx_n_u_stop_slice __pyx_string_tab[234]
#define __pyx_n_u_super __pyx_string_tab[235]
#define __pyx_n_u_sync __pyx_string_tab[236]
#define __pyx_n_u_sys __pyx_string_tab[237]
#define __pyx_n_u_target __pyx_string_tab[238]
#define __pyx_n_u_target_slice __pyx_string_tab[239]
#define __pyx_n_u_test __pyx_string_tab[240]
#define __pyx_n_u_threading __pyx_string_tab[241]
#define __pyx_n_u_transaction __pyx_string_tab[242]
#define __pyx_n_u_update __pyx_string_tab[243]
#define __pyx_n_u_use_setstate __pyx_string_tab[244]
#define __pyx_n_u_valid __pyx_string_tab[245]
#define __pyx_n_u_value __pyx_string_tab[246]
#define __pyx_n_u_value_buffer __pyx_string_tab[247]
#define __pyx_n_u_values __pyx_string_tab[248]
#define __pyx_n_u_verify_checksums __pyx_string_tab[249]
#define __pyx_n_u_weakref __pyx_string_tab[250]
#define __pyx_n_u_weakref_ref __pyx_string_tab[251]
#define __pyx_n_u_wr __pyx_string_tab[252]
#define __pyx_n_u_write __pyx_string_tab[253]
#define __pyx_n_u_write_batch __pyx_string_tab[254]
#define __pyx_n_u_write_buffer_size __pyx_string_tab[255]
#define __pyx_n_u_write_options __pyx_string_tab[256]
#define __pyx_kp_b_iso88591_2_1_a_4s_s_s_k_A_aq_xq_t5_ixy_a __pyx_string_tab[257]
#define __pyx_kp_b_iso88591_2_1_a_xq __pyx_string_tab[258]
#define __pyx_kp_b_iso88591_2_1_a_xq_2 __pyx_string_tab[259]
#define __pyx_kp_b_iso88591_44K1 __pyx_string_tab[260]
#define __pyx_kp_b_iso88591_4A_4s_s_s_k_A_aq_t1_Q_q_Q __pyx_string_tab[261]
#define __pyx_kp_b_iso88591_6a_4uCq_aq_XQ_uAU_Qa_T __pyx_string_tab[262]
#define __pyx_kp_b_iso88591_6a_t3gQd_E_a __pyx_string_tab[263]
#define __pyx_kp_b_iso88591_7_Yaxq_AQ __pyx_string_tab[264]
#define __pyx_kp_b_iso88591_A_4_Q_4z_4z __pyx_string_tab[265]
#define __pyx_kp_b_iso88591_A_4_Q_d_t_XRq_y_q_a_4uG1_A_q_4x __pyx_string_tab[266]
#define __pyx_kp_b_iso88591_A_4s_s_aq_4_D_a_F_F __pyx_string_tab[267]
#define __pyx_kp_b_iso88591_A_4s_s_aq_4xwa_hb_uAU_Qa_7_M_Q_F __pyx_string_tab[268]
#define __pyx_kp_b_iso88591_A_4s_s_aq_4xwa_hb_uAU_Qa_WAQ __pyx_string_tab[269]
#define __pyx_kp_b_iso88591_A_4s_s_aq_M_AQ __pyx_string_tab[270]
#define __pyx_kp_b_iso88591_A_4s_s_aq_S_F_4_7t1 __pyx_string_tab[271]
#define __pyx_kp_b_iso88591_A_4s_s_aq_V1 __pyx_string_tab[272]
#define __pyx_kp_b_iso88591_A_4s_s_aq_q __pyx_string_tab[273]
#define __pyx_kp_b_iso88591_A_4s_s_aq_t_0 __pyx_string_tab[274]
#define __pyx_kp_b_iso88591_A_4s_s_s_k_A_4_q_A_Q __pyx_string_tab[275]
#define __pyx_kp_b_iso88591_A_4uCq_aq_6_q_s_1_5_q_avS_M __pyx_string_tab[276]
#define __pyx_kp_b_iso88591_A_4uCq_aq_Cq_y_ay_6_2Q_V1A_wfAQ __pyx_string_tab[277]
#define __pyx_kp_b_iso88591_A_4uCq_aq_QfCq_T_aq_y_A __pyx_string_tab[278]
#define __pyx_kp_b_iso88591_A_4uCq_aq_t_RwfAQ __pyx_string_tab[279]
#define __pyx_kp_b_iso88591_A_4wc_aq_4_T_Ba_q_1A_4wgU_d_XQ_d __pyx_string_tab[280]
#define __pyx_kp_b_iso88591_A_4wc_aq_4t6_q_a_D_d_y_Rr_q __pyx_string_tab[281]
#define __pyx_kp_b_iso88591_A_4wc_aq_4t6_q_a_d_a_r_e1 __pyx_string_tab[282]
#define __pyx_kp_b_iso88591_A_4wc_aq_4t6_q_a_e1_V7 __pyx_string_tab[283]
#define __pyx_kp_b_iso88591_A_4wc_aq_IQ __pyx_string_tab[284]
#define __pyx_kp_b_iso88591_A_4wc_aq_k_V7 __pyx_string_tab[285]
#define __pyx_kp_b_iso88591_A_4wc_aq_l_V7 __pyx_string_tab[286]
#define __pyx_kp_b_iso88591_A_4wc_aq_q_1A_e1A_V7 __pyx_string_tab[287]
#define __pyx_kp_b_iso88591_A_4wc_aq_t6_q __pyx_string_tab[288]
#define __pyx_kp_b_iso88591_A_4wgQ_A __pyx_string_tab[289]
#define __pyx_kp_b_iso88591_A_F __pyx_string_tab[290]
#define __pyx_kp_b_iso88591_A_F_q __pyx_string_tab[291]
#define __pyx_kp_b_iso88591_A_q __pyx_string_tab[292]
#define __pyx_kp_b_iso88591_A_t3d_4xr_gU __pyx_string_tab[293]
#define __pyx_kp_b_iso88591_A_t4t4vQ __pyx_string_tab[294]
#define __pyx_kp_b_iso88591_A_xq_1 __pyx_string_tab[295]
#define __pyx_kp_b_iso88591_A_xq_4uG4q __pyx_string_tab[296]
#define __pyx_kp_b_iso88591_A_z_T_gT_1 __pyx_string_tab[297]
#define __pyx_kp_b_iso88591_A_z_V7 __pyx_string_tab[298]
#define __pyx_kp_b_iso88591_C1_4uCq_aq_a_XQ_uAU_Qa_7_T_T_F __pyx_string_tab[299]
#define __pyx_kp_b_iso88591_Kxt1A __pyx_string_tab[300]
#define __pyx_kp_b_iso88591_Q __pyx_string_tab[301]
#define __pyx_kp_b_iso88591_Q_8_1_Q_7_a_6a_1_A_A_XQha_AQ __pyx_string_tab[302]
#define __pyx_kp_b_iso88591_T_d_G1F_a_vWE_Q_q_q_4q_4q __pyx_string_tab[303]
#define __pyx_kp_b_iso88591_a_9_4s_s_s_k_A_aq_N_L_A_4xwa_hb __pyx_string_tab[304]
#define __pyx_kp_b_iso88591_a_9_4uCq_aq_N_vQfE __pyx_string_tab[305]
#define __pyx_kp_b_iso88591_a_9_t3d_A_Q_q __pyx_string_tab[306]
#define __pyx_kp_b_iso88591_q __pyx_string_tab[307]
#define __pyx_kp_b_iso88591_q_0_kQR_XQa_7_A_1 __pyx_string_tab[308]
#define __pyx_kp_b_iso88591_q_4uCq_aq_z_m1 __pyx_string_tab[309]
#define __pyx_kp_b_iso88591_z_e4y_Q __pyx_string_tab[310]
#define __pyx_int_245107514 __pyx_number_tab[0]
/* #### Code section: module_state_clear ### */
#if CYTHON_USE_MODULE_STATE
static CYTHON_SMALL_CODE int __pyx_m_clear(PyObject *m) {
  __pyx_mstatetype *clear_module_state = __Pyx_PyModule_GetState(m);
  if (!clear_module_state) return 0;
  Py_CLEAR(clear_module_state->__pyx_d);
  Py_CLEAR(clear_module_state->__pyx_b);
  Py_CLEAR(clear_module_state->__pyx_cython_runtime);
  Py_CLEAR(clear_module_state->__pyx_empty_tuple);
  Py_CLEAR(clear_module_state->__pyx_empty_bytes);
  Py_CLEAR(clear_module_state->__pyx_empty_unicode);
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  __Pyx_State_RemoveModule(NULL);
  #endif
  Py_CLEAR(clear_module_state->__pyx_ptype_7cpython_4type_type);
  Py_CLEAR(clear_module_state->__pyx_ptype_7cpython_4bool_bool);
  Py_CLEAR(clear_module_state->__pyx_ptype_7cpython_7complex_complex);
  Py_CLEAR(clear_module_state->__pyx_ptype_6plyvel_7_plyvel_DB);
  Py_CLEAR(clear_module_state->__pyx_type_6plyvel_7_plyvel_DB);
  Py_CLEAR(clear_module_state->__pyx_ptype_6plyvel_7_plyvel_PrefixedDB);
  Py_CLEAR(clear_module_state->__pyx_type_6plyvel_7_plyvel_PrefixedDB);
  Py_CLEAR(clear_module_state->__pyx_ptype_6plyvel_7_plyvel_WriteBatch);
  Py_CLEAR(clear_module_state->__pyx_type_6plyvel_7_plyvel_WriteBatch);
  Py_CLEAR(clear_module_state->__pyx_ptype_6plyvel_7_plyvel_BaseIterator);
  Py_CLEAR(clear_module_state->__pyx_type_6plyvel_7_plyvel_BaseIterator);
  Py_CLEAR(clear_module_state->__pyx_ptype_6plyvel_7_plyvel_Iterator);
  Py_CLEAR(clear_module_state->__pyx_type_6plyvel_7_plyvel_Iterator);
  Py_CLEAR(clear_module_state->__pyx_ptype_6plyvel_7_plyvel_RawIterator);
  Py_CLEAR(clear_module_state->__pyx_type_6plyvel_7_plyvel_RawIterator);
  Py_CLEAR(clear_module_state->__pyx_ptype_6plyvel_7_plyvel_Snapshot);
  Py_CLEAR(clear_module_state->__pyx_type_6plyvel_7_plyvel_Snapshot);
  Py_CLEAR(clear_module_state->__pyx_ptype_6plyvel_7_plyvel___pyx_scope_struct____init__);
  Py_CLEAR(clear_module_state->__pyx_type_6plyvel_7_plyvel___pyx_scope_struct____init__);
  for (int i=0; i<3; ++i) { Py_CLEAR(clear_module_state->__pyx_tuple[i]); }
  for (int i=0; i<71; ++i) { Py_CLEAR(clear_module_state->__pyx_codeobj_tab[i]); }
  for (int i=0; i<311; ++i) { Py_CLEAR(clear_module_state->__pyx_string_tab[i]); }
  for (int i=0; i<1; ++i) { Py_CLEAR(clear_module_state->__pyx_number_tab[i]); }
/* #### Code section: module_state_clear_contents ### */
/* CommonTypesMetaclass.module_state_clear */
Py_CLEAR(clear_module_state->__pyx_CommonTypesMetaclassType);

/* CythonFunctionShared.module_state_clear */
Py_CLEAR(clear_module_state->__pyx_CyFunctionType);

/* #### Code section: module_state_clear_end ### */
return 0;
}
#endif
/* #### Code section: module_state_traverse ### */
#if CYTHON_USE_MODULE_STATE
static CYTHON_SMALL_CODE int __pyx_m_traverse(PyObject *m, visitproc visit, void *arg) {
  __pyx_mstatetype *traverse_module_state = __Pyx_PyModule_GetState(m);
  if (!traverse_module_state) return 0;
  Py_VISIT(traverse_module_state->__pyx_d);
  Py_VISIT(traverse_module_state->__pyx_b);
  Py_VISIT(traverse_module_state->__pyx_cython_runtime);
  __Pyx_VISIT_CONST(traverse_module_state->__pyx_empty_tuple);
  __Pyx_VISIT_CONST(traverse_module_state->__pyx_empty_bytes);
  __Pyx_VISIT_CONST(traverse_module_state->__pyx_empty_unicode);
  Py_VISIT(traverse_module_state->__pyx_ptype_7cpython_4type_type);
  Py_VISIT(traverse_module_state->__pyx_ptype_7cpython_4bool_bool);
  Py_VISIT(traverse_module_state->__pyx_ptype_7cpython_7complex_complex);
  Py_VISIT(traverse_module_state->__pyx_ptype_6plyvel_7_plyvel_DB);
  Py_VISIT(traverse_module_state->__pyx_type_6plyvel_7_plyvel_DB);
  Py_VISIT(traverse_module_state->__pyx_ptype_6plyvel_7_plyvel_PrefixedDB);
  Py_VISIT(traverse_module_state->__pyx_type_6plyvel_7_plyvel_PrefixedDB);
  Py_VISIT(traverse_module_state->__pyx_ptype_6plyvel_7_plyvel_WriteBatch);
  Py_VISIT(traverse_module_state->__pyx_type_6plyvel_7_plyvel_WriteBatch);
  Py_VISIT(traverse_module_state->__pyx_ptype_6plyvel_7_plyvel_BaseIterator);
  Py_VISIT(traverse_module_state->__pyx_type_6plyvel_7_plyvel_BaseIterator);
  Py_VISIT(traverse_module_state->__pyx_ptype_6plyvel_7_plyvel_Iterator);
  Py_VISIT(traverse_module_state->__pyx_type_6plyvel_7_plyvel_Iterator);
  Py_VISIT(traverse_module_state->__pyx_ptype_6plyvel_7_plyvel_RawIterator);
  Py_VISIT(traverse_module_state->__pyx_type_6plyvel_7_plyvel_RawIterator);
  Py_VISIT(traverse_module_state->__pyx_ptype_6plyvel_7_plyvel_Snapshot);
  Py_VISIT(traverse_module_state->__pyx_type_6plyvel_7_plyvel_Snapshot);
  Py_VISIT(traverse_module_state->__pyx_ptype_6plyvel_7_plyvel___pyx_scope_struct____init__);
  Py_VISIT(traverse_module_state->__pyx_type_6plyvel_7_plyvel___pyx_scope_struct____init__);
  for (int i=0; i<3; ++i) { __Pyx_VISIT_CONST(traverse_module_state->__pyx_tuple[i]); }
  for (int i=0; i<71; ++i) { __Pyx_VISIT_CONST(traverse_module_state->__pyx_codeobj_tab[i]); }
  for (int i=0; i<311; ++i) { __Pyx_VISIT_CONST(traverse_module_state->__pyx_string_tab[i]); }
  for (int i=0; i<1; ++i) { __Pyx_VISIT_CONST(traverse_module_state->__pyx_number_tab[i]); }
/* #### Code section: module_state_traverse_contents ### */
/* CommonTypesMetaclass.module_state_traverse */
Py_VISIT(traverse_module_state->__pyx_CommonTypesMetaclassType);

/* CythonFunctionShared.module_state_traverse */
Py_VISIT(traverse_module_state->__pyx_CyFunctionType);

/* #### Code section: module_state_traverse_end ### */
return 0;
}
#endif
/* #### Code section: module_code ### */

/* "string.to_py":31
 *     cdef object __Pyx_PyObject_FromStringAndSize(const char*, size_t)
 * 
 * @cname("__pyx_convert_PyObject_string_to_py_6libcpp_6string_std__in_string")             # <<<<<<<<<<<<<<
 * cdef inline object __pyx_convert_PyObject_string_to_py_6libcpp_6string_std__in_string(const string& s):
 *     if s.size() > <size_t> PY_SSIZE_T_MAX:
*/

static CYTHON_INLINE PyObject *__pyx_convert_PyObject_string_to_py_6libcpp_6string_std__in_string(std::string const &__pyx_v_s) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_convert_PyObject_string_to_py_6libcpp_6string_std__in_string", 0);

  /* "string.to_py":33
 * @cname("__pyx_convert_PyObject_string_to_py_6libcpp_6string_std__in_string")
 * cdef inline object __pyx_convert_PyObject_string_to_py_6libcpp_6string_std__in_string(const string& s):
 *     if s.size() > <size_t> PY_SSIZE_T_MAX:             # <<<<<<<<<<<<<<
 *         raise MemoryError()
 *     return __Pyx_PyObject_FromStringAndSize(s.data(), <Py_ssize_t> s.size())
*/
  __pyx_t_1 = (__pyx_v_s.size() > ((size_t)PY_SSIZE_T_MAX));
  if (unlikely(__pyx_t_1)) {

    /* "string.to_py":34
 * cdef inline object __pyx_convert_PyObject_string_to_py_6libcpp_6string_std__in_string(const string& s):
 *     if s.size() > <size_t> PY_SSIZE_T_MAX:
 *         raise MemoryError()             # <<<<<<<<<<<<<<
 *     return __Pyx_PyObject_FromStringAndSize(s.data(), <Py_ssize_t> s.size())
 * cdef extern from *:
*/
    PyErr_NoMemory(); __PYX_ERR(1, 34, __pyx_L1_error)

    /* "string.to_py":33
 * @cname("__pyx_convert_PyObject_string_to_py_6libcpp_6string_std__in_string")
 * cdef inline object __pyx_convert_PyObject_string_to_py_6libcpp_6string_std__in_string(const string& s):
 *     if s.size() > <size_t> PY_SSIZE_T_MAX:             # <<<<<<<<<<<<<<
 *         raise MemoryError()
 *     return __Pyx_PyObject_FromStringAndSize(s.data(), <Py_ssize_t> s.size())
*/
  }

  /* "string.to_py":35
 *     if s.size() > <size_t> PY_SSIZE_T_MAX:
 *         raise MemoryError()
 *     return __Pyx_PyObject_FromStringAndSize(s.data(), <Py_ssize_t> s.size())             # <<<<<<<<<<<<<<
 * cdef extern from *:
 *     cdef object __Pyx_PyUnicode_FromStringAndSize(const char*, size_t)
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyObject_FromStringAndSize(__pyx_v_s.data(), ((Py_ssize_t)__pyx_v_s.size())); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 35, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "string.to_py":31
 *     cdef object __Pyx_PyObject_FromStringAndSize(const char*, size_t)
 * 
 * @cname("__pyx_convert_PyObject_string_to_py_6libcpp_6string_std__in_string")             # <<<<<<<<<<<<<<
 * cdef inline object __pyx_convert_PyObject_string_to_py_6libcpp_6string_std__in_string(const string& s):
 *     if s.size() > <size_t> PY_SSIZE_T_MAX:
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("string.to_py.__pyx_convert_PyObject_string_to_py_6libcpp_6string_std__in_string", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "string.to_py":39
 *     cdef object __Pyx_PyUnicode_FromStringAndSize(const char*, size_t)
 * 
 * @cname("__pyx_convert_PyUnicode_string_to_py_6libcpp_6string_std__in_string")             # <<<<<<<<<<<<<<
 * cdef inline object __pyx_convert_PyUnicode_string_to_py_6libcpp_6string_std__in_string(const string& s):
 *     if s.size() > <size_t> PY_SSIZE_T_MAX:
*/

static CYTHON_INLINE PyObject *__pyx_convert_PyUnicode_string_to_py_6libcpp_6string_std__in_string(std::string const &__pyx_v_s) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_convert_PyUnicode_string_to_py_6libcpp_6string_std__in_string", 0);

  /* "string.to_py":41
 * @cname("__pyx_convert_PyUnicode_string_to_py_6libcpp_6string_std__in_string")
 * cdef inline object __pyx_convert_PyUnicode_string_to_py_6libcpp_6string_std__in_string(const string& s):
 *     if s.size() > <size_t> PY_SSIZE_T_MAX:             # <<<<<<<<<<<<<<
 *         raise MemoryError()
 *     return __Pyx_PyUnicode_FromStringAndSize(s.data(), <Py_ssize_t> s.size())
*/
  __pyx_t_1 = (__pyx_v_s.size() > ((size_t)PY_SSIZE_T_MAX));
  if (unlikely(__pyx_t_1)) {

    /* "string.to_py":42
 * cdef inline object __pyx_convert_PyUnicode_string_to_py_6libcpp_6string_std__in_string(const string& s):
 *     if s.size() > <size_t> PY_SSIZE_T_MAX:
 *         raise MemoryError()             # <<<<<<<<<<<<<<
 *     return __Pyx_PyUnicode_FromStringAndSize(s.data(), <Py_ssize_t> s.size())
 * cdef extern from *:
*/
    PyErr_NoMemory(); __PYX_ERR(1, 42, __pyx_L1_error)

    /* "string.to_py":41
 * @cname("__pyx_convert_PyUnicode_string_to_py_6libcpp_6string_std__in_string")
 * cdef inline object __pyx_convert_PyUnicode_string_to_py_6libcpp_6string_std__in_string(const string& s):
 *     if s.size() > <size_t> PY_SSIZE_T_MAX:             # <<<<<<<<<<<<<<
 *         raise MemoryError()
 *     return __Pyx_PyUnicode_FromStringAndSize(s.data(), <Py_ssize_t> s.size())
*/
  }

  /* "string.to_py":43
 *     if s.size() > <size_t> PY_SSIZE_T_MAX:
 *         raise MemoryError()
 *     return __Pyx_PyUnicode_FromStringAndSize(s.data(), <Py_ssize_t> s.size())             # <<<<<<<<<<<<<<
 * cdef extern from *:
 *     cdef object __Pyx_PyBytes_FromStringAndSize(const char*, size_t)
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyUnicode_FromStringAndSize(__pyx_v_s.data(), ((Py_ssize_t)__pyx_v_s.size())); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 43, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "string.to_py":39
 *     cdef object __Pyx_PyUnicode_FromStringAndSize(const char*, size_t)
 * 
 * @cname("__pyx_convert_PyUnicode_string_to_py_6libcpp_6string_std__in_string")             # <<<<<<<<<<<<<<
 * cdef inline object __pyx_convert_PyUnicode_string_to_py_6libcpp_6string_std__in_string(const string& s):
 *     if s.size() > <size_t> PY_SSIZE_T_MAX:
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("string.to_py.__pyx_convert_PyUnicode_string_to_py_6libcpp_6string_std__in_string", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "string.to_py":47
 *     cdef object __Pyx_PyBytes_FromStringAndSize(const char*, size_t)
 * 
 * @cname("__pyx_convert_PyBytes_string_to_py_6libcpp_6string_std__in_string")             # <<<<<<<<<<<<<<
 * cdef inline object __pyx_convert_PyBytes_string_to_py_6libcpp_6string_std__in_string(const string& s):
 *     if s.size() > <size_t> PY_SSIZE_T_MAX:
*/

static CYTHON_INLINE PyObject *__pyx_convert_PyBytes_string_to_py_6libcpp_6string_std__in_string(std::string const &__pyx_v_s) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_convert_PyBytes_string_to_py_6libcpp_6string_std__in_string", 0);

  /* "string.to_py":49
 * @cname("__pyx_convert_PyBytes_string_to_py_6libcpp_6string_std__in_string")
 * cdef inline object __pyx_convert_PyBytes_string_to_py_6libcpp_6string_std__in_string(const string& s):
 *     if s.size() > <size_t> PY_SSIZE_T_MAX:             # <<<<<<<<<<<<<<
 *         raise MemoryError()
 *     return __Pyx_PyBytes_FromStringAndSize(s.data(), <Py_ssize_t> s.size())
*/
  __pyx_t_1 = (__pyx_v_s.size() > ((size_t)PY_SSIZE_T_MAX));
  if (unlikely(__pyx_t_1)) {

    /* "string.to_py":50
 * cdef inline object __pyx_convert_PyBytes_string_to_py_6libcpp_6string_std__in_string(const string& s):
 *     if s.size() > <size_t> PY_SSIZE_T_MAX:
 *         raise MemoryError()             # <<<<<<<<<<<<<<
 *     return __Pyx_PyBytes_FromStringAndSize(s.data(), <Py_ssize_t> s.size())
 * cdef extern from *:
*/
    PyErr_NoMemory(); __PYX_ERR(1, 50, __pyx_L1_error)

    /* "string.to_py":49
 * @cname("__pyx_convert_PyBytes_string_to_py_6libcpp_6string_std__in_string")
 * cdef inline object __pyx_convert_PyBytes_string_to_py_6libcpp_6string_std__in_string(const string& s):
 *     if s.size() > <size_t> PY_SSIZE_T_MAX:             # <<<<<<<<<<<<<<
 *         raise MemoryError()
 *     return __Pyx_PyBytes_FromStringAndSize(s.data(), <Py_ssize_t> s.size())
*/
  }

  /* "string.to_py":51
 *     if s.size() > <size_t> PY_SSIZE_T_MAX:
 *         raise MemoryError()
 *     return __Pyx_PyBytes_FromStringAndSize(s.data(), <Py_ssize_t> s.size())             # <<<<<<<<<<<<<<
 * cdef extern from *:
 *     cdef object __Pyx_PyByteArray_FromStringAndSize(const char*, size_t)
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyBytes_FromStringAndSize(__pyx_v_s.data(), ((Py_ssize_t)__pyx_v_s.size())); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 51, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "string.to_py":47
 *     cdef object __Pyx_PyBytes_FromStringAndSize(const char*, size_t)
 * 
 * @cname("__pyx_convert_PyBytes_string_to_py_6libcpp_6string_std__in_string")             # <<<<<<<<<<<<<<
 * cdef inline object __pyx_convert_PyBytes_string_to_py_6libcpp_6string_std__in_string(const string& s):
 *     if s.size() > <size_t> PY_SSIZE_T_MAX:
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("string.to_py.__pyx_convert_PyBytes_string_to_py_6libcpp_6string_std__in_string", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "string.to_py":55
 *     cdef object __Pyx_PyByteArray_FromStringAndSize(const char*, size_t)
 * 
 * @cname("__pyx_convert_PyByteArray_string_to_py_6libcpp_6string_std__in_string")             # <<<<<<<<<<<<<<
 * cdef inline object __pyx_convert_PyByteArray_string_to_py_6libcpp_6string_std__in_string(const string& s):
 *     if s.size() > <size_t> PY_SSIZE_T_MAX:
*/

static CYTHON_INLINE PyObject *__pyx_convert_PyByteArray_string_to_py_6libcpp_6string_std__in_string(std::string const &__pyx_v_s) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_convert_PyByteArray_string_to_py_6libcpp_6string_std__in_string", 0);

  /* "string.to_py":57
 * @cname("__pyx_convert_PyByteArray_string_to_py_6libcpp_6string_std__in_string")
 * cdef inline object __pyx_convert_PyByteArray_string_to_py_6libcpp_6string_std__in_string(const string& s):
 *     if s.size() > <size_t> PY_SSIZE_T_MAX:             # <<<<<<<<<<<<<<
 *         raise MemoryError()
 *     return __Pyx_PyByteArray_FromStringAndSize(s.data(), <Py_ssize_t> s.size())
*/
  __pyx_t_1 = (__pyx_v_s.size() > ((size_t)PY_SSIZE_T_MAX));
  if (unlikely(__pyx_t_1)) {

    /* "string.to_py":58
 * cdef inline object __pyx_convert_PyByteArray_string_to_py_6libcpp_6string_std__in_string(const string& s):
 *     if s.size() > <size_t> PY_SSIZE_T_MAX:
 *         raise MemoryError()             # <<<<<<<<<<<<<<
 *     return __Pyx_PyByteArray_FromStringAndSize(s.data(), <Py_ssize_t> s.size())
*/
    PyErr_NoMemory(); __PYX_ERR(1, 58, __pyx_L1_error)

    /* "string.to_py":57
 * @cname("__pyx_convert_PyByteArray_string_to_py_6libcpp_6string_std__in_string")
 * cdef inline object __pyx_convert_PyByteArray_string_to_py_6libcpp_6string_std__in_string(const string& s):
 *     if s.size() > <size_t> PY_SSIZE_T_MAX:             # <<<<<<<<<<<<<<
 *         raise MemoryError()
 *     return __Pyx_PyByteArray_FromStringAndSize(s.data(), <Py_ssize_t> s.size())
*/
  }

  /* "string.to_py":59
 *     if s.size() > <size_t> PY_SSIZE_T_MAX:
 *         raise MemoryError()
 *     return __Pyx_PyByteArray_FromStringAndSize(s.data(), <Py_ssize_t> s.size())             # <<<<<<<<<<<<<<
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyByteArray_FromStringAndSize(__pyx_v_s.data(), ((Py_ssize_t)__pyx_v_s.size())); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 59, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "string.to_py":55
 *     cdef object __Pyx_PyByteArray_FromStringAndSize(const char*, size_t)
 * 
 * @cname("__pyx_convert_PyByteArray_string_to_py_6libcpp_6string_std__in_string")             # <<<<<<<<<<<<<<
 * cdef inline object __pyx_convert_PyByteArray_string_to_py_6libcpp_6string_std__in_string(const string& s):
 *     if s.size() > <size_t> PY_SSIZE_T_MAX:
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("string.to_py.__pyx_convert_PyByteArray_string_to_py_6libcpp_6string_std__in_string", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "string.from_py":12
 *     cdef const char* __Pyx_PyObject_AsStringAndSize(object, Py_ssize_t*) except NULL
 * 
 * @cname("__pyx_convert_string_from_py_6libcpp_6string_std__in_string")             # <<<<<<<<<<<<<<
 * cdef string __pyx_convert_string_from_py_6libcpp_6string_std__in_string(object o) except *:
 *     cdef Py_ssize_t length = 0
*/

static std::string __pyx_convert_string_from_py_6libcpp_6string_std__in_string(PyObject *__pyx_v_o) {
  Py_ssize_t __pyx_v_length;
  char const *__pyx_v_data;
  std::string __pyx_r;
  char const *__pyx_t_1;
  std::string __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;

  /* "string.from_py":14
 * @cname("__pyx_convert_string_from_py_6libcpp_6string_std__in_string")
 * cdef string __pyx_convert_string_from_py_6libcpp_6string_std__in_string(object o) except *:
 *     cdef Py_ssize_t length = 0             # <<<<<<<<<<<<<<
 *     cdef const char* data = __Pyx_PyObject_AsStringAndSize(o, &length)
 *     return string(data, <size_t> length)
*/
  __pyx_v_length = 0;

  /* "string.from_py":15
 * cdef string __pyx_convert_string_from_py_6libcpp_6string_std__in_string(object o) except *:
 *     cdef Py_ssize_t length = 0
 *     cdef const char* data = __Pyx_PyObject_AsStringAndSize(o, &length)             # <<<<<<<<<<<<<<
 *     return string(data, <size_t> length)
 * 
*/
  __pyx_t_1 = __Pyx_PyObject_AsStringAndSize(__pyx_v_o, (&__pyx_v_length)); if (unlikely(__pyx_t_1 == ((void *)NULL))) __PYX_ERR(1, 15, __pyx_L1_error)
  __pyx_v_data = __pyx_t_1;

  /* "string.from_py":16
 *     cdef Py_ssize_t length = 0
 *     cdef const char* data = __Pyx_PyObject_AsStringAndSize(o, &length)
 *     return string(data, <size_t> length)             # <<<<<<<<<<<<<<
 * 
*/
  try {
    __pyx_t_2 = std::string(__pyx_v_data, ((size_t)__pyx_v_length));
  } catch(...) {
    __Pyx_CppExn2PyErr();
    __PYX_ERR(1, 16, __pyx_L1_error)
  }
  __pyx_r = __pyx_t_2;
  goto __pyx_L0;

  /* "string.from_py":12
 *     cdef const char* __Pyx_PyObject_AsStringAndSize(object, Py_ssize_t*) except NULL
 * 
 * @cname("__pyx_convert_string_from_py_6libcpp_6string_std__in_string")             # <<<<<<<<<<<<<<
 * cdef string __pyx_convert_string_from_py_6libcpp_6string_std__in_string(object o) except *:
 *     cdef Py_ssize_t length = 0
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("string.from_py.__pyx_convert_string_from_py_6libcpp_6string_std__in_string", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_pretend_to_initialize(&__pyx_r);
  __pyx_L0:;
  return __pyx_r;
}

/* "cpython/complex.pxd":20
 * 
 *         # unavailable in limited API
 *         @property             # <<<<<<<<<<<<<<
 *         @_cython.c_compile_guard("!CYTHON_COMPILING_IN_LIMITED_API")
 *         cdef inline double real(self) noexcept:
*/

#if !CYTHON_COMPILING_IN_LIMITED_API
static CYTHON_INLINE double __pyx_f_7cpython_7complex_7complex_4real_real(PyComplexObject *__pyx_v_self) {
  double __pyx_r;

  /* "cpython/complex.pxd":23
 *         @_cython.c_compile_guard("!CYTHON_COMPILING_IN_LIMITED_API")
 *         cdef inline double real(self) noexcept:
 *             return self.cval.real             # <<<<<<<<<<<<<<
 * 
 *         # unavailable in limited API
*/
  __pyx_r = __pyx_v_self->cval.real;
  goto __pyx_L0;

  /* "cpython/complex.pxd":20
 * 
 *         # unavailable in limited API
 *         @property             # <<<<<<<<<<<<<<
 *         @_cython.c_compile_guard("!CYTHON_COMPILING_IN_LIMITED_API")
 *         cdef inline double real(self) noexcept:
*/

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}
#endif /*!(#if !CYTHON_COMPILING_IN_LIMITED_API)*/

/* "cpython/complex.pxd":26
 * 
 *         # unavailable in limited API
 *         @property             # <<<<<<<<<<<<<<
 *         @_cython.c_compile_guard("!CYTHON_COMPILING_IN_LIMITED_API")
 *         cdef inline double imag(self) noexcept:
*/

#if !CYTHON_COMPILING_IN_LIMITED_API
static CYTHON_INLINE double __pyx_f_7cpython_7complex_7complex_4imag_imag(PyComplexObject *__pyx_v_self) {
  double __pyx_r;

  /* "cpython/complex.pxd":29
 *         @_cython.c_compile_guard("!CYTHON_COMPILING_IN_LIMITED_API")
 *         cdef inline double imag(self) noexcept:
 *             return self.cval.imag             # <<<<<<<<<<<<<<
 * 
 *     # PyTypeObject PyComplex_Type
*/
  __pyx_r = __pyx_v_self->cval.imag;
  goto __pyx_L0;

  /* "cpython/complex.pxd":26
 * 
 *         # unavailable in limited API
 *         @property             # <<<<<<<<<<<<<<
 *         @_cython.c_compile_guard("!CYTHON_COMPILING_IN_LIMITED_API")
 *         cdef inline double imag(self) noexcept:
*/

  /* function exit code */
  __pyx_L0:;
  return __pyx_r;
}
#endif /*!(#if !CYTHON_COMPILING_IN_LIMITED_API)*/

/* "cpython/contextvars.pxd":115
 * 
 * 
 * @_cython.c_compile_guard("!CYTHON_COMPILING_IN_LIMITED_API")             # <<<<<<<<<<<<<<
 * cdef inline object get_value(var, default_value=None):
 *     """Return a new reference to the value of the context variable,
*/

#if !CYTHON_COMPILING_IN_LIMITED_API
static CYTHON_INLINE PyObject *__pyx_f_7cpython_11contextvars_get_value(PyObject *__pyx_v_var, struct __pyx_opt_args_7cpython_11contextvars_get_value *__pyx_optional_args) {

  /* "cpython/contextvars.pxd":116
 * 
 * @_cython.c_compile_guard("!CYTHON_COMPILING_IN_LIMITED_API")
 * cdef inline object get_value(var, default_value=None):             # <<<<<<<<<<<<<<
 *     """Return a new reference to the value of the context variable,
 *     or the default value of the context variable,
*/
  PyObject *__pyx_v_default_value = ((PyObject *)Py_None);
  PyObject *__pyx_v_value;
  PyObject *__pyx_v_pyvalue = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_value", 0);
  if (__pyx_optional_args) {
    if (__pyx_optional_args->__pyx_n > 0) {
      __pyx_v_default_value = __pyx_optional_args->default_value;
    }
  }

  /* "cpython/contextvars.pxd":121
 *     or None if no such value or default was found.
 *     """
 *     cdef PyObject *value = NULL             # <<<<<<<<<<<<<<
 *     PyContextVar_Get(var, NULL, &value)
 *     if value is NULL:
*/
  __pyx_v_value = NULL;

  /* "cpython/contextvars.pxd":122
 *     """
 *     cdef PyObject *value = NULL
 *     PyContextVar_Get(var, NULL, &value)             # <<<<<<<<<<<<<<
 *     if value is NULL:
 *         # context variable does not have a default
*/
  __pyx_t_1 = PyContextVar_Get(__pyx_v_var, NULL, (&__pyx_v_value)); if (unlikely(__pyx_t_1 == ((int)-1))) __PYX_ERR(2, 122, __pyx_L1_error)

  /* "cpython/contextvars.pxd":123
 *     cdef PyObject *value = NULL
 *     PyContextVar_Get(var, NULL, &value)
 *     if value is NULL:             # <<<<<<<<<<<<<<
 *         # context variable does not have a default
 *         pyvalue = default_value
*/
  __pyx_t_2 = (__pyx_v_value == NULL);
  if (__pyx_t_2) {

    /* "cpython/contextvars.pxd":125
 *     if value is NULL:
 *         # context variable does not have a default
 *         pyvalue = default_value             # <<<<<<<<<<<<<<
 *     else:
 *         # value or default value of context variable
*/
    __Pyx_INCREF(__pyx_v_default_value);
    __pyx_v_pyvalue = __pyx_v_default_value;

    /* "cpython/contextvars.pxd":123
 *     cdef PyObject *value = NULL
 *     PyContextVar_Get(var, NULL, &value)
 *     if value is NULL:             # <<<<<<<<<<<<<<
 *         # context variable does not have a default
 *         pyvalue = default_value
*/
    goto __pyx_L3;
  }

  /* "cpython/contextvars.pxd":128
 *     else:
 *         # value or default value of context variable
 *         pyvalue = <object>value             # <<<<<<<<<<<<<<
 *         Py_XDECREF(value)  # PyContextVar_Get() returned an owned reference as 'PyObject*'
 *     return pyvalue
*/
  /*else*/ {
    __pyx_t_3 = ((PyObject *)__pyx_v_value);
    __Pyx_INCREF(__pyx_t_3);
    __pyx_v_pyvalue = __pyx_t_3;
    __pyx_t_3 = 0;

    /* "cpython/contextvars.pxd":129
 *         # value or default value of context variable
 *         pyvalue = <object>value
 *         Py_XDECREF(value)  # PyContextVar_Get() returned an owned reference as 'PyObject*'             # <<<<<<<<<<<<<<
 *     return pyvalue
 * 
*/
    Py_XDECREF(__pyx_v_value);
  }
  __pyx_L3:;

  /* "cpython/contextvars.pxd":130
 *         pyvalue = <object>value
 *         Py_XDECREF(value)  # PyContextVar_Get() returned an owned reference as 'PyObject*'
 *     return pyvalue             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_pyvalue);
  __pyx_r = __pyx_v_pyvalue;
  goto __pyx_L0;

  /* "cpython/contextvars.pxd":115
 * 
 * 
 * @_cython.c_compile_guard("!CYTHON_COMPILING_IN_LIMITED_API")             # <<<<<<<<<<<<<<
 * cdef inline object get_value(var, default_value=None):
 *     """Return a new reference to the value of the context variable,
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("cpython.contextvars.get_value", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_pyvalue);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}
#endif /*!(#if !CYTHON_COMPILING_IN_LIMITED_API)*/

/* "cpython/contextvars.pxd":133
 * 
 * 
 * @_cython.c_compile_guard("!CYTHON_COMPILING_IN_LIMITED_API")             # <<<<<<<<<<<<<<
 * cdef inline object get_value_no_default(var, default_value=None):
 *     """Return a new reference to the value of the context variable,
*/

#if !CYTHON_COMPILING_IN_LIMITED_API
static CYTHON_INLINE PyObject *__pyx_f_7cpython_11contextvars_get_value_no_default(PyObject *__pyx_v_var, struct __pyx_opt_args_7cpython_11contextvars_get_value_no_default *__pyx_optional_args) {

  /* "cpython/contextvars.pxd":134
 * 
 * @_cython.c_compile_guard("!CYTHON_COMPILING_IN_LIMITED_API")
 * cdef inline object get_value_no_default(var, default_value=None):             # <<<<<<<<<<<<<<
 *     """Return a new reference to the value of the context variable,
 *     or the provided default value if no such value was found.
*/
  PyObject *__pyx_v_default_value = ((PyObject *)Py_None);
  PyObject *__pyx_v_value;
  PyObject *__pyx_v_pyvalue = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_value_no_default", 0);
  if (__pyx_optional_args) {
    if (__pyx_optional_args->__pyx_n > 0) {
      __pyx_v_default_value = __pyx_optional_args->default_value;
    }
  }

  /* "cpython/contextvars.pxd":140
 *     Ignores the default value of the context variable, if any.
 *     """
 *     cdef PyObject *value = NULL             # <<<<<<<<<<<<<<
 *     PyContextVar_Get(var, <PyObject*>default_value, &value)
 *     # value of context variable or 'default_value'
*/
  __pyx_v_value = NULL;

  /* "cpython/contextvars.pxd":141
 *     """
 *     cdef PyObject *value = NULL
 *     PyContextVar_Get(var, <PyObject*>default_value, &value)             # <<<<<<<<<<<<<<
 *     # value of context variable or 'default_value'
 *     pyvalue = <object>value
*/
  __pyx_t_1 = PyContextVar_Get(__pyx_v_var, ((PyObject *)__pyx_v_default_value), (&__pyx_v_value)); if (unlikely(__pyx_t_1 == ((int)-1))) __PYX_ERR(2, 141, __pyx_L1_error)

  /* "cpython/contextvars.pxd":143
 *     PyContextVar_Get(var, <PyObject*>default_value, &value)
 *     # value of context variable or 'default_value'
 *     pyvalue = <object>value             # <<<<<<<<<<<<<<
 *     Py_XDECREF(value)  # PyContextVar_Get() returned an owned reference as 'PyObject*'
 *     return pyvalue
*/
  __pyx_t_2 = ((PyObject *)__pyx_v_value);
  __Pyx_INCREF(__pyx_t_2);
  __pyx_v_pyvalue = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "cpython/contextvars.pxd":144
 *     # value of context variable or 'default_value'
 *     pyvalue = <object>value
 *     Py_XDECREF(value)  # PyContextVar_Get() returned an owned reference as 'PyObject*'             # <<<<<<<<<<<<<<
 *     return pyvalue
*/
  Py_XDECREF(__pyx_v_value);

  /* "cpython/contextvars.pxd":145
 *     pyvalue = <object>value
 *     Py_XDECREF(value)  # PyContextVar_Get() returned an owned reference as 'PyObject*'
 *     return pyvalue             # <<<<<<<<<<<<<<
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_pyvalue);
  __pyx_r = __pyx_v_pyvalue;
  goto __pyx_L0;

  /* "cpython/contextvars.pxd":133
 * 
 * 
 * @_cython.c_compile_guard("!CYTHON_COMPILING_IN_LIMITED_API")             # <<<<<<<<<<<<<<
 * cdef inline object get_value_no_default(var, default_value=None):
 *     """Return a new reference to the value of the context variable,
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("cpython.contextvars.get_value_no_default", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_pyvalue);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}
#endif /*!(#if !CYTHON_COMPILING_IN_LIMITED_API)*/

/* "plyvel/_plyvel.pyx":83
 * 
 * 
 * cdef int raise_for_status(Status st) except -1:             # <<<<<<<<<<<<<<
 *     if st.ok():
 *         return 0
*/

static int __pyx_f_6plyvel_7_plyvel_raise_for_status(leveldb::Status __pyx_v_st) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  size_t __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("raise_for_status", 0);

  /* "plyvel/_plyvel.pyx":84
 * 
 * cdef int raise_for_status(Status st) except -1:
 *     if st.ok():             # <<<<<<<<<<<<<<
 *         return 0
 * 
*/
  __pyx_t_1 = (__pyx_v_st.ok() != 0);
  if (__pyx_t_1) {

    /* "plyvel/_plyvel.pyx":85
 * cdef int raise_for_status(Status st) except -1:
 *     if st.ok():
 *         return 0             # <<<<<<<<<<<<<<
 * 
 *     if st.IsIOError():
*/
    __pyx_r = 0;
    goto __pyx_L0;

    /* "plyvel/_plyvel.pyx":84
 * 
 * cdef int raise_for_status(Status st) except -1:
 *     if st.ok():             # <<<<<<<<<<<<<<
 *         return 0
 * 
*/
  }

  /* "plyvel/_plyvel.pyx":87
 *         return 0
 * 
 *     if st.IsIOError():             # <<<<<<<<<<<<<<
 *         raise IOError(st.ToString())
 * 
*/
  __pyx_t_1 = (__pyx_v_st.IsIOError() != 0);
  if (unlikely(__pyx_t_1)) {

    /* "plyvel/_plyvel.pyx":88
 * 
 *     if st.IsIOError():
 *         raise IOError(st.ToString())             # <<<<<<<<<<<<<<
 * 
 *     if st.IsCorruption():
*/
    __pyx_t_3 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_IOError); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 88, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = __pyx_convert_PyBytes_string_to_py_6libcpp_6string_std__in_string(__pyx_v_st.ToString()); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 88, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
      assert(__pyx_t_3);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
      __pyx_t_6 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_t_5};
      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 88, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 88, __pyx_L1_error)

    /* "plyvel/_plyvel.pyx":87
 *         return 0
 * 
 *     if st.IsIOError():             # <<<<<<<<<<<<<<
 *         raise IOError(st.ToString())
 * 
*/
  }

  /* "plyvel/_plyvel.pyx":90
 *         raise IOError(st.ToString())
 * 
 *     if st.IsCorruption():             # <<<<<<<<<<<<<<
 *         raise CorruptionError(st.ToString())
 * 
*/
  __pyx_t_1 = (__pyx_v_st.IsCorruption() != 0);
  if (unlikely(__pyx_t_1)) {

    /* "plyvel/_plyvel.pyx":91
 * 
 *     if st.IsCorruption():
 *         raise CorruptionError(st.ToString())             # <<<<<<<<<<<<<<
 * 
 *     # Generic fallback
*/
    __pyx_t_4 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_mstate_global->__pyx_n_u_CorruptionError); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 91, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_3 = __pyx_convert_PyBytes_string_to_py_6libcpp_6string_std__in_string(__pyx_v_st.ToString()); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 91, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_6 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_5);
      assert(__pyx_t_4);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_5, __pyx__function);
      __pyx_t_6 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_t_3};
      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_5, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 91, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 91, __pyx_L1_error)

    /* "plyvel/_plyvel.pyx":90
 *         raise IOError(st.ToString())
 * 
 *     if st.IsCorruption():             # <<<<<<<<<<<<<<
 *         raise CorruptionError(st.ToString())
 * 
*/
  }

  /* "plyvel/_plyvel.pyx":94
 * 
 *     # Generic fallback
 *     raise Error(st.ToString())             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __pyx_t_5 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_mstate_global->__pyx_n_u_Error); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 94, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __pyx_convert_PyBytes_string_to_py_6libcpp_6string_std__in_string(__pyx_v_st.ToString()); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 94, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_6 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_3);
    assert(__pyx_t_5);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_3);
    __Pyx_INCREF(__pyx_t_5);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_3, __pyx__function);
    __pyx_t_6 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_5, __pyx_t_4};
    __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_3, __pyx_callargs+__pyx_t_6, (2-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 94, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
  }
  __Pyx_Raise(__pyx_t_2, 0, 0, 0);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __PYX_ERR(0, 94, __pyx_L1_error)

  /* "plyvel/_plyvel.pyx":83
 * 
 * 
 * cdef int raise_for_status(Status st) except -1:             # <<<<<<<<<<<<<<
 *     if st.ok():
 *         return 0
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("plyvel._plyvel.raise_for_status", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":101
 * #
 * 
 * cdef inline db_get(DB db, bytes key, object default, ReadOptions read_options):             # <<<<<<<<<<<<<<
 *     cdef string value
 *     cdef Status st
*/

static CYTHON_INLINE PyObject *__pyx_f_6plyvel_7_plyvel_db_get(struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_db, PyObject *__pyx_v_key, PyObject *__pyx_v_default, leveldb::ReadOptions __pyx_v_read_options) {
  std::string __pyx_v_value;
  leveldb::Status __pyx_v_st;
  leveldb::Slice __pyx_v_key_slice;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  const char *__pyx_t_1;
  Py_ssize_t __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("db_get", 0);

  /* "plyvel/_plyvel.pyx":104
 *     cdef string value
 *     cdef Status st
 *     cdef Slice key_slice = Slice(key, len(key))             # <<<<<<<<<<<<<<
 * 
 *     with nogil:
*/
  if (unlikely(__pyx_v_key == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "expected bytes, NoneType found");
    __PYX_ERR(0, 104, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_PyBytes_AsString(__pyx_v_key); if (unlikely((!__pyx_t_1) && PyErr_Occurred())) __PYX_ERR(0, 104, __pyx_L1_error)
  if (unlikely(__pyx_v_key == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(0, 104, __pyx_L1_error)
  }
  __pyx_t_2 = __Pyx_PyBytes_GET_SIZE(__pyx_v_key); if (unlikely(__pyx_t_2 == ((Py_ssize_t)-1))) __PYX_ERR(0, 104, __pyx_L1_error)
  __pyx_v_key_slice = leveldb::Slice(__pyx_t_1, __pyx_t_2);

  /* "plyvel/_plyvel.pyx":106
 *     cdef Slice key_slice = Slice(key, len(key))
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 *         st = db._db.Get(read_options, key_slice, &value)
 * 
*/
  {
      PyThreadState * _save;
      _save = PyEval_SaveThread();
      __Pyx_FastGIL_Remember();
      /*try:*/ {

        /* "plyvel/_plyvel.pyx":107
 * 
 *     with nogil:
 *         st = db._db.Get(read_options, key_slice, &value)             # <<<<<<<<<<<<<<
 * 
 *     if st.IsNotFound():
*/
        __pyx_v_st = __pyx_v_db->_db->Get(__pyx_v_read_options, __pyx_v_key_slice, (&__pyx_v_value));
      }

      /* "plyvel/_plyvel.pyx":106
 *     cdef Slice key_slice = Slice(key, len(key))
 * 
 *     with nogil:             # <<<<<<<<<<<<<<
 *         st = db._db.Get(read_options, key_slice, &value)
 * 
*/
      /*finally:*/ {
        /*normal exit:*/{
          __Pyx_FastGIL_Forget();
          PyEval_RestoreThread(_save);
          goto __pyx_L5;
        }
        __pyx_L5:;
      }
  }

  /* "plyvel/_plyvel.pyx":109
 *         st = db._db.Get(read_options, key_slice, &value)
 * 
 *     if st.IsNotFound():             # <<<<<<<<<<<<<<
 *         return default
 *     raise_for_status(st)
*/
  __pyx_t_3 = (__pyx_v_st.IsNotFound() != 0);
  if (__pyx_t_3) {

    /* "plyvel/_plyvel.pyx":110
 * 
 *     if st.IsNotFound():
 *         return default             # <<<<<<<<<<<<<<
 *     raise_for_status(st)
 * 
*/
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_v_default);
    __pyx_r = __pyx_v_default;
    goto __pyx_L0;

    /* "plyvel/_plyvel.pyx":109
 *         st = db._db.Get(read_options, key_slice, &value)
 * 
 *     if st.IsNotFound():             # <<<<<<<<<<<<<<
 *         return default
 *     raise_for_status(st)
*/
  }

  /* "plyvel/_plyvel.pyx":111
 *     if st.IsNotFound():
 *         return default
 *     raise_for_status(st)             # <<<<<<<<<<<<<<
 * 
 *     return value
*/
  __pyx_t_4 = __pyx_f_6plyvel_7_plyvel_raise_for_status(__pyx_v_st); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(0, 111, __pyx_L1_error)

  /* "plyvel/_plyvel.pyx":113
 *     raise_for_status(st)
 * 
 *     return value             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_5 = __pyx_convert_PyBytes_string_to_py_6libcpp_6string_std__in_string(__pyx_v_value); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 113, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_r = __pyx_t_5;
  __pyx_t_5 = 0;
  goto __pyx_L0;

  /* "plyvel/_plyvel.pyx":101
 * #
 * 
 * cdef inline db_get(DB db, bytes key, object default, ReadOptions read_options):             # <<<<<<<<<<<<<<
 *     cdef string value
 *     cdef Status st
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("plyvel._plyvel.db_get", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":116
 * 
 * 
 * cdef bytes to_file_system_name(name):             # <<<<<<<<<<<<<<
 *     if isinstance(name, bytes):
 *         return name
*/

static PyObject *__pyx_f_6plyvel_7_plyvel_to_file_system_name(PyObject *__pyx_v_name) {
  PyObject *__pyx_v_encoding = NULL;
  PyObject *__pyx_v_exc = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  int __pyx_t_12;
  PyObject *__pyx_t_13 = NULL;
  int __pyx_t_14;
  char const *__pyx_t_15;
  PyObject *__pyx_t_16 = NULL;
  PyObject *__pyx_t_17 = NULL;
  PyObject *__pyx_t_18 = NULL;
  PyObject *__pyx_t_19 = NULL;
  PyObject *__pyx_t_20 = NULL;
  PyObject *__pyx_t_21 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("to_file_system_name", 0);

  /* "plyvel/_plyvel.pyx":117
 * 
 * cdef bytes to_file_system_name(name):
 *     if isinstance(name, bytes):             # <<<<<<<<<<<<<<
 *         return name
 * 
*/
  __pyx_t_1 = PyBytes_Check(__pyx_v_name); 
  if (__pyx_t_1) {

    /* "plyvel/_plyvel.pyx":118
 * cdef bytes to_file_system_name(name):
 *     if isinstance(name, bytes):
 *         return name             # <<<<<<<<<<<<<<
 * 
 *     if not isinstance(name, unicode):
*/
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __pyx_v_name;
    __Pyx_INCREF(__pyx_t_2);
    if (!(likely(PyBytes_CheckExact(__pyx_t_2))||((__pyx_t_2) == Py_None) || __Pyx_RaiseUnexpectedTypeError("bytes", __pyx_t_2))) __PYX_ERR(0, 118, __pyx_L1_error)
    __pyx_r = ((PyObject*)__pyx_t_2);
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "plyvel/_plyvel.pyx":117
 * 
 * cdef bytes to_file_system_name(name):
 *     if isinstance(name, bytes):             # <<<<<<<<<<<<<<
 *         return name
 * 
*/
  }

  /* "plyvel/_plyvel.pyx":120
 *         return name
 * 
 *     if not isinstance(name, unicode):             # <<<<<<<<<<<<<<
 *         raise TypeError(
 *             "'name' arg must be a byte string or a unicode string")
*/
  __pyx_t_1 = PyUnicode_Check(__pyx_v_name); 
  __pyx_t_3 = (!__pyx_t_1);
  if (unlikely(__pyx_t_3)) {

    /* "plyvel/_plyvel.pyx":121
 * 
 *     if not isinstance(name, unicode):
 *         raise TypeError(             # <<<<<<<<<<<<<<
 *             "'name' arg must be a byte string or a unicode string")
 * 
*/
    __pyx_t_4 = NULL;
    __pyx_t_5 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_mstate_global->__pyx_kp_u_name_arg_must_be_a_byte_string};
      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_TypeError)), __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 121, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 121, __pyx_L1_error)

    /* "plyvel/_plyvel.pyx":120
 *         return name
 * 
 *     if not isinstance(name, unicode):             # <<<<<<<<<<<<<<
 *         raise TypeError(
 *             "'name' arg must be a byte string or a unicode string")
*/
  }

  /* "plyvel/_plyvel.pyx":124
 *             "'name' arg must be a byte string or a unicode string")
 * 
 *     encoding = sys.getfilesystemencoding() or 'ascii'             # <<<<<<<<<<<<<<
 *     try:
 *         return name.encode(encoding)
*/
  __pyx_t_6 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_sys); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 124, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_getfilesystemencoding); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 124, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_5 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_8))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_8);
    assert(__pyx_t_6);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_8);
    __Pyx_INCREF(__pyx_t_6);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_8, __pyx__function);
    __pyx_t_5 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_6, NULL};
    __pyx_t_4 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_8, __pyx_callargs+__pyx_t_5, (1-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 124, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
  }
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely((__pyx_t_3 < 0))) __PYX_ERR(0, 124, __pyx_L1_error)
  if (!__pyx_t_3) {
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  } else {
    __Pyx_INCREF(__pyx_t_4);
    __pyx_t_2 = __pyx_t_4;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    goto __pyx_L5_bool_binop_done;
  }
  __Pyx_INCREF(__pyx_mstate_global->__pyx_n_u_ascii);
  __pyx_t_2 = __pyx_mstate_global->__pyx_n_u_ascii;
  __pyx_L5_bool_binop_done:;
  __pyx_v_encoding = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "plyvel/_plyvel.pyx":125
 * 
 *     encoding = sys.getfilesystemencoding() or 'ascii'
 *     try:             # <<<<<<<<<<<<<<
 *         return name.encode(encoding)
 *     except UnicodeEncodeError as exc:
*/
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_9, &__pyx_t_10, &__pyx_t_11);
    __Pyx_XGOTREF(__pyx_t_9);
    __Pyx_XGOTREF(__pyx_t_10);
    __Pyx_XGOTREF(__pyx_t_11);
    /*try:*/ {

      /* "plyvel/_plyvel.pyx":126
 *     encoding = sys.getfilesystemencoding() or 'ascii'
 *     try:
 *         return name.encode(encoding)             # <<<<<<<<<<<<<<
 *     except UnicodeEncodeError as exc:
 *         raise ValueError(
*/
      __Pyx_XDECREF(__pyx_r);
      __pyx_t_4 = __pyx_v_name;
      __Pyx_INCREF(__pyx_t_4);
      __pyx_t_5 = 0;
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_v_encoding};
        __pyx_t_2 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_encode, __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 126, __pyx_L7_error)
        __Pyx_GOTREF(__pyx_t_2);
      }
      if (!(likely(PyBytes_CheckExact(__pyx_t_2))||((__pyx_t_2) == Py_None) || __Pyx_RaiseUnexpectedTypeError("bytes", __pyx_t_2))) __PYX_ERR(0, 126, __pyx_L7_error)
      __pyx_r = ((PyObject*)__pyx_t_2);
      __pyx_t_2 = 0;
      goto __pyx_L11_try_return;

      /* "plyvel/_plyvel.pyx":125
 * 
 *     encoding = sys.getfilesystemencoding() or 'ascii'
 *     try:             # <<<<<<<<<<<<<<
 *         return name.encode(encoding)
 *     except UnicodeEncodeError as exc:
*/
    }
    __pyx_L7_error:;
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;

    /* "plyvel/_plyvel.pyx":127
 *     try:
 *         return name.encode(encoding)
 *     except UnicodeEncodeError as exc:             # <<<<<<<<<<<<<<
 *         raise ValueError(
 *             "Cannot convert unicode 'name' to a file system name: %s" % exc)
*/
    __pyx_t_12 = __Pyx_PyErr_ExceptionMatches(((PyObject *)(((PyTypeObject*)PyExc_UnicodeEncodeError))));
    if (__pyx_t_12) {
      __Pyx_AddTraceback("plyvel._plyvel.to_file_system_name", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_2, &__pyx_t_4, &__pyx_t_8) < 0) __PYX_ERR(0, 127, __pyx_L9_except_error)
      __Pyx_XGOTREF(__pyx_t_2);
      __Pyx_XGOTREF(__pyx_t_4);
      __Pyx_XGOTREF(__pyx_t_8);
      __Pyx_INCREF(__pyx_t_4);
      __pyx_v_exc = __pyx_t_4;
      /*try:*/ {

        /* "plyvel/_plyvel.pyx":128
 *         return name.encode(encoding)
 *     except UnicodeEncodeError as exc:
 *         raise ValueError(             # <<<<<<<<<<<<<<
 *             "Cannot convert unicode 'name' to a file system name: %s" % exc)
 * 
*/
        __pyx_t_7 = NULL;

        /* "plyvel/_plyvel.pyx":129
 *     except UnicodeEncodeError as exc:
 *         raise ValueError(
 *             "Cannot convert unicode 'name' to a file system name: %s" % exc)             # <<<<<<<<<<<<<<
 * 
 * 
*/
        __pyx_t_13 = PyUnicode_Format(__pyx_mstate_global->__pyx_kp_u_Cannot_convert_unicode_name_to_a, __pyx_v_exc); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 129, __pyx_L18_error)
        __Pyx_GOTREF(__pyx_t_13);
        __pyx_t_5 = 1;
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_7, __pyx_t_13};
          __pyx_t_6 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_ValueError)), __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
          __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
          if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 128, __pyx_L18_error)
          __Pyx_GOTREF(__pyx_t_6);
        }
        __Pyx_Raise(__pyx_t_6, 0, 0, 0);
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __PYX_ERR(0, 128, __pyx_L18_error)
      }

      /* "plyvel/_plyvel.pyx":127
 *     try:
 *         return name.encode(encoding)
 *     except UnicodeEncodeError as exc:             # <<<<<<<<<<<<<<
 *         raise ValueError(
 *             "Cannot convert unicode 'name' to a file system name: %s" % exc)
*/
      /*finally:*/ {
        __pyx_L18_error:;
        /*exception exit:*/{
          __Pyx_PyThreadState_declare
          __Pyx_PyThreadState_assign
          __pyx_t_16 = 0; __pyx_t_17 = 0; __pyx_t_18 = 0; __pyx_t_19 = 0; __pyx_t_20 = 0; __pyx_t_21 = 0;
          __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
          __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
          __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
           __Pyx_ExceptionSwap(&__pyx_t_19, &__pyx_t_20, &__pyx_t_21);
          if ( unlikely(__Pyx_GetException(&__pyx_t_16, &__pyx_t_17, &__pyx_t_18) < 0)) __Pyx_ErrFetch(&__pyx_t_16, &__pyx_t_17, &__pyx_t_18);
          __Pyx_XGOTREF(__pyx_t_16);
          __Pyx_XGOTREF(__pyx_t_17);
          __Pyx_XGOTREF(__pyx_t_18);
          __Pyx_XGOTREF(__pyx_t_19);
          __Pyx_XGOTREF(__pyx_t_20);
          __Pyx_XGOTREF(__pyx_t_21);
          __pyx_t_12 = __pyx_lineno; __pyx_t_14 = __pyx_clineno; __pyx_t_15 = __pyx_filename;
          {
            __Pyx_DECREF(__pyx_v_exc); __pyx_v_exc = 0;
          }
          __Pyx_XGIVEREF(__pyx_t_19);
          __Pyx_XGIVEREF(__pyx_t_20);
          __Pyx_XGIVEREF(__pyx_t_21);
          __Pyx_ExceptionReset(__pyx_t_19, __pyx_t_20, __pyx_t_21);
          __Pyx_XGIVEREF(__pyx_t_16);
          __Pyx_XGIVEREF(__pyx_t_17);
          __Pyx_XGIVEREF(__pyx_t_18);
          __Pyx_ErrRestore(__pyx_t_16, __pyx_t_17, __pyx_t_18);
          __pyx_t_16 = 0; __pyx_t_17 = 0; __pyx_t_18 = 0; __pyx_t_19 = 0; __pyx_t_20 = 0; __pyx_t_21 = 0;
          __pyx_lineno = __pyx_t_12; __pyx_clineno = __pyx_t_14; __pyx_filename = __pyx_t_15;
          goto __pyx_L9_except_error;
        }
      }
    }
    goto __pyx_L9_except_error;

    /* "plyvel/_plyvel.pyx":125
 * 
 *     encoding = sys.getfilesystemencoding() or 'ascii'
 *     try:             # <<<<<<<<<<<<<<
 *         return name.encode(encoding)
 *     except UnicodeEncodeError as exc:
*/
    __pyx_L9_except_error:;
    __Pyx_XGIVEREF(__pyx_t_9);
    __Pyx_XGIVEREF(__pyx_t_10);
    __Pyx_XGIVEREF(__pyx_t_11);
    __Pyx_ExceptionReset(__pyx_t_9, __pyx_t_10, __pyx_t_11);
    goto __pyx_L1_error;
    __pyx_L11_try_return:;
    __Pyx_XGIVEREF(__pyx_t_9);
    __Pyx_XGIVEREF(__pyx_t_10);
    __Pyx_XGIVEREF(__pyx_t_11);
    __Pyx_ExceptionReset(__pyx_t_9, __pyx_t_10, __pyx_t_11);
    goto __pyx_L0;
  }

  /* "plyvel/_plyvel.pyx":116
 * 
 * 
 * cdef bytes to_file_system_name(name):             # <<<<<<<<<<<<<<
 *     if isinstance(name, bytes):
 *         return name
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_13);
  __Pyx_AddTraceback("plyvel._plyvel.to_file_system_name", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_encoding);
  __Pyx_XDECREF(__pyx_v_exc);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":132
 * 
 * 
 * cdef bytes bytes_increment(bytes s):             # <<<<<<<<<<<<<<
 *     # Increment the last byte that is not 0xff, and returned a new byte
 *     # string truncated after the position that was incremented. We use
*/

static PyObject *__pyx_f_6plyvel_7_plyvel_bytes_increment(PyObject *__pyx_v_s) {
  PyObject *__pyx_v_b = NULL;
  int __pyx_v_i;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  size_t __pyx_t_3;
  Py_ssize_t __pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("bytes_increment", 0);

  /* "plyvel/_plyvel.pyx":138
 *     # works the same in Python 2 and Python 3.
 * 
 *     b = bytearray(s)             # <<<<<<<<<<<<<<
 *     cdef int i = len(s) - 1
 *     while i >= 0:
*/
  __pyx_t_2 = NULL;
  __pyx_t_3 = 1;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_v_s};
    __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)(&PyByteArray_Type), __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (__pyx_t_3*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 138, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_v_b = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "plyvel/_plyvel.pyx":139
 * 
 *     b = bytearray(s)
 *     cdef int i = len(s) - 1             # <<<<<<<<<<<<<<
 *     while i >= 0:
 *         if b[i] == 0xff:
*/
  if (unlikely(__pyx_v_s == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(0, 139, __pyx_L1_error)
  }
  __pyx_t_4 = __Pyx_PyBytes_GET_SIZE(__pyx_v_s); if (unlikely(__pyx_t_4 == ((Py_ssize_t)-1))) __PYX_ERR(0, 139, __pyx_L1_error)
  __pyx_v_i = (__pyx_t_4 - 1);

  /* "plyvel/_plyvel.pyx":140
 *     b = bytearray(s)
 *     cdef int i = len(s) - 1
 *     while i >= 0:             # <<<<<<<<<<<<<<
 *         if b[i] == 0xff:
 *             i = i - 1
*/
  while (1) {
    __pyx_t_5 = (__pyx_v_i >= 0);
    if (!__pyx_t_5) break;

    /* "plyvel/_plyvel.pyx":141
 *     cdef int i = len(s) - 1
 *     while i >= 0:
 *         if b[i] == 0xff:             # <<<<<<<<<<<<<<
 *             i = i - 1
 *             continue
*/
    __pyx_t_6 = __Pyx_GetItemInt_ByteArray(__pyx_v_b, __pyx_v_i, int, 1, __Pyx_PyLong_From_int, 0, 1, 1, 1, __Pyx_ReferenceSharing_OwnStrongReference); if (unlikely(__pyx_t_6 == -1)) __PYX_ERR(0, 141, __pyx_L1_error)
    __pyx_t_5 = (__pyx_t_6 == 0xff);
    if (__pyx_t_5) {

      /* "plyvel/_plyvel.pyx":142
 *     while i >= 0:
 *         if b[i] == 0xff:
 *             i = i - 1             # <<<<<<<<<<<<<<
 *             continue
 * 
*/
      __pyx_v_i = (__pyx_v_i - 1);

      /* "plyvel/_plyvel.pyx":143
 *         if b[i] == 0xff:
 *             i = i - 1
 *             continue             # <<<<<<<<<<<<<<
 * 
 *         # Found byte smaller than 0xff: increment and truncate
*/
      goto __pyx_L3_continue;

      /* "plyvel/_plyvel.pyx":141
 *     cdef int i = len(s) - 1
 *     while i >= 0:
 *         if b[i] == 0xff:             # <<<<<<<<<<<<<<
 *             i = i - 1
 *             continue
*/
    }

    /* "plyvel/_plyvel.pyx":146
 * 
 *         # Found byte smaller than 0xff: increment and truncate
 *         b[i] += 1             # <<<<<<<<<<<<<<
 *         return bytes(b[:i + 1])
 * 
*/
    __pyx_t_6 = __pyx_v_i;
    __pyx_t_7 = __Pyx_GetItemInt_ByteArray(__pyx_v_b, __pyx_t_6, int, 1, __Pyx_PyLong_From_int, 0, 1, 1, 1, __Pyx_ReferenceSharing_OwnStrongReference); if (unlikely(__pyx_t_7 == -1)) __PYX_ERR(0, 146, __pyx_L1_error)
    if (unlikely((__pyx_t_7 + 1) < 0 || (__pyx_t_7 + 1) > 255)) {
      PyErr_SetString(PyExc_ValueError, "byte must be in range(0, 256)"); __PYX_ERR(0, 146, __pyx_L1_error)
    }
    if (unlikely((__Pyx_SetItemInt_ByteArray(__pyx_v_b, __pyx_t_6, ((unsigned char)(__pyx_t_7 + 1)), int, 1, __Pyx_PyLong_From_int, 0, 1, 1, 1, __Pyx_ReferenceSharing_OwnStrongReference) < 0))) __PYX_ERR(0, 146, __pyx_L1_error)

    /* "plyvel/_plyvel.pyx":147
 *         # Found byte smaller than 0xff: increment and truncate
 *         b[i] += 1
 *         return bytes(b[:i + 1])             # <<<<<<<<<<<<<<
 * 
 *     # Input contained only 0xff bytes
*/
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = NULL;
    __pyx_t_8 = PySequence_GetSlice(__pyx_v_b, 0, (__pyx_v_i + 1)); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 147, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_3 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_2, __pyx_t_8};
      __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)(&PyBytes_Type), __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (__pyx_t_3*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 147, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __pyx_r = ((PyObject*)__pyx_t_1);
    __pyx_t_1 = 0;
    goto __pyx_L0;
    __pyx_L3_continue:;
  }

  /* "plyvel/_plyvel.pyx":150
 * 
 *     # Input contained only 0xff bytes
 *     return None             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_r = ((PyObject*)Py_None); __Pyx_INCREF(Py_None);
  goto __pyx_L0;

  /* "plyvel/_plyvel.pyx":132
 * 
 * 
 * cdef bytes bytes_increment(bytes s):             # <<<<<<<<<<<<<<
 *     # Increment the last byte that is not 0xff, and returned a new byte
 *     # string truncated after the position that was incremented. We use
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("plyvel._plyvel.bytes_increment", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_b);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":153
 * 
 * 
 * cdef int parse_options(Options *options, c_bool create_if_missing,             # <<<<<<<<<<<<<<
 *                        c_bool error_if_exists, object paranoid_checks,
 *                        object write_buffer_size, object max_open_files,
*/

static int __pyx_f_6plyvel_7_plyvel_parse_options(leveldb::Options *__pyx_v_options, bool __pyx_v_create_if_missing, bool __pyx_v_error_if_exists, PyObject *__pyx_v_paranoid_checks, PyObject *__pyx_v_write_buffer_size, PyObject *__pyx_v_max_open_files, PyObject *__pyx_v_lru_cache_size, PyObject *__pyx_v_block_size, PyObject *__pyx_v_block_restart_interval, PyObject *__pyx_v_max_file_size, PyObject *__pyx_v_compression, int __pyx_v_bloom_filter_bits, PyObject *__pyx_v_comparator, PyObject *__pyx_v_comparator_name) {
  size_t __pyx_v_c_lru_cache_size;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  bool __pyx_t_2;
  size_t __pyx_t_3;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  int __pyx_t_8;
  const char *__pyx_t_9;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("parse_options", 0);
  __Pyx_INCREF(__pyx_v_compression);

  /* "plyvel/_plyvel.pyx":162
 *     cdef size_t c_lru_cache_size
 * 
 *     options.create_if_missing = create_if_missing             # <<<<<<<<<<<<<<
 *     options.error_if_exists = error_if_exists
 * 
*/
  __pyx_v_options->create_if_missing = __pyx_v_create_if_missing;

  /* "plyvel/_plyvel.pyx":163
 * 
 *     options.create_if_missing = create_if_missing
 *     options.error_if_exists = error_if_exists             # <<<<<<<<<<<<<<
 * 
 *     if paranoid_checks is not None:
*/
  __pyx_v_options->error_if_exists = __pyx_v_error_if_exists;

  /* "plyvel/_plyvel.pyx":165
 *     options.error_if_exists = error_if_exists
 * 
 *     if paranoid_checks is not None:             # <<<<<<<<<<<<<<
 *         options.paranoid_checks = paranoid_checks
 * 
*/
  __pyx_t_1 = (__pyx_v_paranoid_checks != Py_None);
  if (__pyx_t_1) {

    /* "plyvel/_plyvel.pyx":166
 * 
 *     if paranoid_checks is not None:
 *         options.paranoid_checks = paranoid_checks             # <<<<<<<<<<<<<<
 * 
 *     if write_buffer_size is not None:
*/
    __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_v_paranoid_checks); if (unlikely((__pyx_t_2 == ((bool)-1)) && PyErr_Occurred())) __PYX_ERR(0, 166, __pyx_L1_error)
    __pyx_v_options->paranoid_checks = __pyx_t_2;

    /* "plyvel/_plyvel.pyx":165
 *     options.error_if_exists = error_if_exists
 * 
 *     if paranoid_checks is not None:             # <<<<<<<<<<<<<<
 *         options.paranoid_checks = paranoid_checks
 * 
*/
  }

  /* "plyvel/_plyvel.pyx":168
 *         options.paranoid_checks = paranoid_checks
 * 
 *     if write_buffer_size is not None:             # <<<<<<<<<<<<<<
 *         options.write_buffer_size = write_buffer_size
 * 
*/
  __pyx_t_1 = (__pyx_v_write_buffer_size != Py_None);
  if (__pyx_t_1) {

    /* "plyvel/_plyvel.pyx":169
 * 
 *     if write_buffer_size is not None:
 *         options.write_buffer_size = write_buffer_size             # <<<<<<<<<<<<<<
 * 
 *     if max_open_files is not None:
*/
    __pyx_t_3 = __Pyx_PyLong_As_size_t(__pyx_v_write_buffer_size); if (unlikely((__pyx_t_3 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 169, __pyx_L1_error)
    __pyx_v_options->write_buffer_size = __pyx_t_3;

    /* "plyvel/_plyvel.pyx":168
 *         options.paranoid_checks = paranoid_checks
 * 
 *     if write_buffer_size is not None:             # <<<<<<<<<<<<<<
 *         options.write_buffer_size = write_buffer_size
 * 
*/
  }

  /* "plyvel/_plyvel.pyx":171
 *         options.write_buffer_size = write_buffer_size
 * 
 *     if max_open_files is not None:             # <<<<<<<<<<<<<<
 *         options.max_open_files = max_open_files
 * 
*/
  __pyx_t_1 = (__pyx_v_max_open_files != Py_None);
  if (__pyx_t_1) {

    /* "plyvel/_plyvel.pyx":172
 * 
 *     if max_open_files is not None:
 *         options.max_open_files = max_open_files             # <<<<<<<<<<<<<<
 * 
 *     if lru_cache_size is not None:
*/
    __pyx_t_4 = __Pyx_PyLong_As_int(__pyx_v_max_open_files); if (unlikely((__pyx_t_4 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 172, __pyx_L1_error)
    __pyx_v_options->max_open_files = __pyx_t_4;

    /* "plyvel/_plyvel.pyx":171
 *         options.write_buffer_size = write_buffer_size
 * 
 *     if max_open_files is not None:             # <<<<<<<<<<<<<<
 *         options.max_open_files = max_open_files
 * 
*/
  }

  /* "plyvel/_plyvel.pyx":174
 *         options.max_open_files = max_open_files
 * 
 *     if lru_cache_size is not None:             # <<<<<<<<<<<<<<
 *         c_lru_cache_size = lru_cache_size
 *         with nogil:
*/
  __pyx_t_1 = (__pyx_v_lru_cache_size != Py_None);
  if (__pyx_t_1) {

    /* "plyvel/_plyvel.pyx":175
 * 
 *     if lru_cache_size is not None:
 *         c_lru_cache_size = lru_cache_size             # <<<<<<<<<<<<<<
 *         with nogil:
 *             options.block_cache = NewLRUCache(c_lru_cache_size)
*/
    __pyx_t_3 = __Pyx_PyLong_As_size_t(__pyx_v_lru_cache_size); if (unlikely((__pyx_t_3 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 175, __pyx_L1_error)
    __pyx_v_c_lru_cache_size = __pyx_t_3;

    /* "plyvel/_plyvel.pyx":176
 *     if lru_cache_size is not None:
 *         c_lru_cache_size = lru_cache_size
 *         with nogil:             # <<<<<<<<<<<<<<
 *             options.block_cache = NewLRUCache(c_lru_cache_size)
 * 
*/
    {
        PyThreadState * _save;
        _save = PyEval_SaveThread();
        __Pyx_FastGIL_Remember();
        /*try:*/ {

          /* "plyvel/_plyvel.pyx":177
 *         c_lru_cache_size = lru_cache_size
 *         with nogil:
 *             options.block_cache = NewLRUCache(c_lru_cache_size)             # <<<<<<<<<<<<<<
 * 
 *     if block_size is not None:
*/
          __pyx_v_options->block_cache = leveldb::NewLRUCache(__pyx_v_c_lru_cache_size);
        }

        /* "plyvel/_plyvel.pyx":176
 *     if lru_cache_size is not None:
 *         c_lru_cache_size = lru_cache_size
 *         with nogil:             # <<<<<<<<<<<<<<
 *             options.block_cache = NewLRUCache(c_lru_cache_size)
 * 
*/
        /*finally:*/ {
          /*normal exit:*/{
            __Pyx_FastGIL_Forget();
            PyEval_RestoreThread(_save);
            goto __pyx_L9;
          }
          __pyx_L9:;
        }
    }

    /* "plyvel/_plyvel.pyx":174
 *         options.max_open_files = max_open_files
 * 
 *     if lru_cache_size is not None:             # <<<<<<<<<<<<<<
 *         c_lru_cache_size = lru_cache_size
 *         with nogil:
*/
  }

  /* "plyvel/_plyvel.pyx":179
 *             options.block_cache = NewLRUCache(c_lru_cache_size)
 * 
 *     if block_size is not None:             # <<<<<<<<<<<<<<
 *         options.block_size = block_size
 * 
*/
  __pyx_t_1 = (__pyx_v_block_size != Py_None);
  if (__pyx_t_1) {

    /* "plyvel/_plyvel.pyx":180
 * 
 *     if block_size is not None:
 *         options.block_size = block_size             # <<<<<<<<<<<<<<
 * 
 *     if block_restart_interval is not None:
*/
    __pyx_t_3 = __Pyx_PyLong_As_size_t(__pyx_v_block_size); if (unlikely((__pyx_t_3 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 180, __pyx_L1_error)
    __pyx_v_options->block_size = __pyx_t_3;

    /* "plyvel/_plyvel.pyx":179
 *             options.block_cache = NewLRUCache(c_lru_cache_size)
 * 
 *     if block_size is not None:             # <<<<<<<<<<<<<<
 *         options.block_size = block_size
 * 
*/
  }

  /* "plyvel/_plyvel.pyx":182
 *         options.block_size = block_size
 * 
 *     if block_restart_interval is not None:             # <<<<<<<<<<<<<<
 *         options.block_restart_interval = block_restart_interval
 * 
*/
  __pyx_t_1 = (__pyx_v_block_restart_interval != Py_None);
  if (__pyx_t_1) {

    /* "plyvel/_plyvel.pyx":183
 * 
 *     if block_restart_interval is not None:
 *         options.block_restart_interval = block_restart_interval             # <<<<<<<<<<<<<<
 * 
 *     if max_file_size is not None:
*/
    __pyx_t_4 = __Pyx_PyLong_As_int(__pyx_v_block_restart_interval); if (unlikely((__pyx_t_4 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 183, __pyx_L1_error)
    __pyx_v_options->block_restart_interval = __pyx_t_4;

    /* "plyvel/_plyvel.pyx":182
 *         options.block_size = block_size
 * 
 *     if block_restart_interval is not None:             # <<<<<<<<<<<<<<
 *         options.block_restart_interval = block_restart_interval
 * 
*/
  }

  /* "plyvel/_plyvel.pyx":185
 *         options.block_restart_interval = block_restart_interval
 * 
 *     if max_file_size is not None:             # <<<<<<<<<<<<<<
 *         options.max_file_size = max_file_size
 * 
*/
  __pyx_t_1 = (__pyx_v_max_file_size != Py_None);
  if (__pyx_t_1) {

    /* "plyvel/_plyvel.pyx":186
 * 
 *     if max_file_size is not None:
 *         options.max_file_size = max_file_size             # <<<<<<<<<<<<<<
 * 
 *     if compression is None:
*/
    __pyx_t_3 = __Pyx_PyLong_As_size_t(__pyx_v_max_file_size); if (unlikely((__pyx_t_3 == (size_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 186, __pyx_L1_error)
    __pyx_v_options->max_file_size = __pyx_t_3;

    /* "plyvel/_plyvel.pyx":185
 *         options.block_restart_interval = block_restart_interval
 * 
 *     if max_file_size is not None:             # <<<<<<<<<<<<<<
 *         options.max_file_size = max_file_size
 * 
*/
  }

  /* "plyvel/_plyvel.pyx":188
 *         options.max_file_size = max_file_size
 * 
 *     if compression is None:             # <<<<<<<<<<<<<<
 *         options.compression = leveldb.kNoCompression
 *     else:
*/
  __pyx_t_1 = (__pyx_v_compression == Py_None);
  if (__pyx_t_1) {

    /* "plyvel/_plyvel.pyx":189
 * 
 *     if compression is None:
 *         options.compression = leveldb.kNoCompression             # <<<<<<<<<<<<<<
 *     else:
 *         if isinstance(compression, bytes):
*/
    __pyx_v_options->compression = leveldb::kNoCompression;

    /* "plyvel/_plyvel.pyx":188
 *         options.max_file_size = max_file_size
 * 
 *     if compression is None:             # <<<<<<<<<<<<<<
 *         options.compression = leveldb.kNoCompression
 *     else:
*/
    goto __pyx_L13;
  }

  /* "plyvel/_plyvel.pyx":191
 *         options.compression = leveldb.kNoCompression
 *     else:
 *         if isinstance(compression, bytes):             # <<<<<<<<<<<<<<
 *             compression = compression.decode('UTF-8')
 *         if not isinstance(compression, unicode):
*/
  /*else*/ {
    __pyx_t_1 = PyBytes_Check(__pyx_v_compression); 
    if (__pyx_t_1) {

      /* "plyvel/_plyvel.pyx":192
 *     else:
 *         if isinstance(compression, bytes):
 *             compression = compression.decode('UTF-8')             # <<<<<<<<<<<<<<
 *         if not isinstance(compression, unicode):
 *             raise TypeError("'compression' must be None or a string")
*/
      __pyx_t_6 = __pyx_v_compression;
      __Pyx_INCREF(__pyx_t_6);
      __pyx_t_3 = 0;
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_6, __pyx_mstate_global->__pyx_kp_u_UTF_8};
        __pyx_t_5 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_decode, __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
        if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 192, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
      }
      __Pyx_DECREF_SET(__pyx_v_compression, __pyx_t_5);
      __pyx_t_5 = 0;

      /* "plyvel/_plyvel.pyx":191
 *         options.compression = leveldb.kNoCompression
 *     else:
 *         if isinstance(compression, bytes):             # <<<<<<<<<<<<<<
 *             compression = compression.decode('UTF-8')
 *         if not isinstance(compression, unicode):
*/
    }

    /* "plyvel/_plyvel.pyx":193
 *         if isinstance(compression, bytes):
 *             compression = compression.decode('UTF-8')
 *         if not isinstance(compression, unicode):             # <<<<<<<<<<<<<<
 *             raise TypeError("'compression' must be None or a string")
 *         if compression == u'snappy':
*/
    __pyx_t_1 = PyUnicode_Check(__pyx_v_compression); 
    __pyx_t_7 = (!__pyx_t_1);
    if (unlikely(__pyx_t_7)) {

      /* "plyvel/_plyvel.pyx":194
 *             compression = compression.decode('UTF-8')
 *         if not isinstance(compression, unicode):
 *             raise TypeError("'compression' must be None or a string")             # <<<<<<<<<<<<<<
 *         if compression == u'snappy':
 *             options.compression = leveldb.kSnappyCompression
*/
      __pyx_t_6 = NULL;
      __pyx_t_3 = 1;
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_6, __pyx_mstate_global->__pyx_kp_u_compression_must_be_None_or_a_s};
        __pyx_t_5 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_TypeError)), __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (__pyx_t_3*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
        if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 194, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
      }
      __Pyx_Raise(__pyx_t_5, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __PYX_ERR(0, 194, __pyx_L1_error)

      /* "plyvel/_plyvel.pyx":193
 *         if isinstance(compression, bytes):
 *             compression = compression.decode('UTF-8')
 *         if not isinstance(compression, unicode):             # <<<<<<<<<<<<<<
 *             raise TypeError("'compression' must be None or a string")
 *         if compression == u'snappy':
*/
    }

    /* "plyvel/_plyvel.pyx":195
 *         if not isinstance(compression, unicode):
 *             raise TypeError("'compression' must be None or a string")
 *         if compression == u'snappy':             # <<<<<<<<<<<<<<
 *             options.compression = leveldb.kSnappyCompression
 *         else:
*/
    __pyx_t_7 = (__Pyx_PyUnicode_Equals(__pyx_v_compression, __pyx_mstate_global->__pyx_n_u_snappy, Py_EQ)); if (unlikely((__pyx_t_7 < 0))) __PYX_ERR(0, 195, __pyx_L1_error)
    if (likely(__pyx_t_7)) {

      /* "plyvel/_plyvel.pyx":196
 *             raise TypeError("'compression' must be None or a string")
 *         if compression == u'snappy':
 *             options.compression = leveldb.kSnappyCompression             # <<<<<<<<<<<<<<
 *         else:
 *             raise ValueError("'compression' must be None or 'snappy'")
*/
      __pyx_v_options->compression = leveldb::kSnappyCompression;

      /* "plyvel/_plyvel.pyx":195
 *         if not isinstance(compression, unicode):
 *             raise TypeError("'compression' must be None or a string")
 *         if compression == u'snappy':             # <<<<<<<<<<<<<<
 *             options.compression = leveldb.kSnappyCompression
 *         else:
*/
      goto __pyx_L16;
    }

    /* "plyvel/_plyvel.pyx":198
 *             options.compression = leveldb.kSnappyCompression
 *         else:
 *             raise ValueError("'compression' must be None or 'snappy'")             # <<<<<<<<<<<<<<
 * 
 *     if bloom_filter_bits > 0:
*/
    /*else*/ {
      __pyx_t_6 = NULL;
      __pyx_t_3 = 1;
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_6, __pyx_mstate_global->__pyx_kp_u_compression_must_be_None_or_sna};
        __pyx_t_5 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_ValueError)), __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (__pyx_t_3*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
        if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 198, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
      }
      __Pyx_Raise(__pyx_t_5, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __PYX_ERR(0, 198, __pyx_L1_error)
    }
    __pyx_L16:;
  }
  __pyx_L13:;

  /* "plyvel/_plyvel.pyx":200
 *             raise ValueError("'compression' must be None or 'snappy'")
 * 
 *     if bloom_filter_bits > 0:             # <<<<<<<<<<<<<<
 *         with nogil:
 *             options.filter_policy = NewBloomFilterPolicy(bloom_filter_bits)
*/
  __pyx_t_7 = (__pyx_v_bloom_filter_bits > 0);
  if (__pyx_t_7) {

    /* "plyvel/_plyvel.pyx":201
 * 
 *     if bloom_filter_bits > 0:
 *         with nogil:             # <<<<<<<<<<<<<<
 *             options.filter_policy = NewBloomFilterPolicy(bloom_filter_bits)
 * 
*/
    {
        PyThreadState * _save;
        _save = PyEval_SaveThread();
        __Pyx_FastGIL_Remember();
        /*try:*/ {

          /* "plyvel/_plyvel.pyx":202
 *     if bloom_filter_bits > 0:
 *         with nogil:
 *             options.filter_policy = NewBloomFilterPolicy(bloom_filter_bits)             # <<<<<<<<<<<<<<
 * 
 *     if (comparator is None) != (comparator_name is None):
*/
          __pyx_v_options->filter_policy = leveldb::NewBloomFilterPolicy(__pyx_v_bloom_filter_bits);
        }

        /* "plyvel/_plyvel.pyx":201
 * 
 *     if bloom_filter_bits > 0:
 *         with nogil:             # <<<<<<<<<<<<<<
 *             options.filter_policy = NewBloomFilterPolicy(bloom_filter_bits)
 * 
*/
        /*finally:*/ {
          /*normal exit:*/{
            __Pyx_FastGIL_Forget();
            PyEval_RestoreThread(_save);
            goto __pyx_L20;
          }
          __pyx_L20:;
        }
    }

    /* "plyvel/_plyvel.pyx":200
 *             raise ValueError("'compression' must be None or 'snappy'")
 * 
 *     if bloom_filter_bits > 0:             # <<<<<<<<<<<<<<
 *         with nogil:
 *             options.filter_policy = NewBloomFilterPolicy(bloom_filter_bits)
*/
  }

  /* "plyvel/_plyvel.pyx":204
 *             options.filter_policy = NewBloomFilterPolicy(bloom_filter_bits)
 * 
 *     if (comparator is None) != (comparator_name is None):             # <<<<<<<<<<<<<<
 *         raise ValueError(
 *             "'comparator' and 'comparator_name' must be specified together")
*/
  __pyx_t_7 = (__pyx_v_comparator == Py_None);
  __pyx_t_1 = (__pyx_v_comparator_name == ((PyObject*)Py_None));
  __pyx_t_8 = (__pyx_t_7 != __pyx_t_1);
  if (unlikely(__pyx_t_8)) {

    /* "plyvel/_plyvel.pyx":205
 * 
 *     if (comparator is None) != (comparator_name is None):
 *         raise ValueError(             # <<<<<<<<<<<<<<
 *             "'comparator' and 'comparator_name' must be specified together")
 * 
*/
    __pyx_t_6 = NULL;
    __pyx_t_3 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_6, __pyx_mstate_global->__pyx_kp_u_comparator_and_comparator_name};
      __pyx_t_5 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_ValueError)), __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (__pyx_t_3*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 205, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
    }
    __Pyx_Raise(__pyx_t_5, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __PYX_ERR(0, 205, __pyx_L1_error)

    /* "plyvel/_plyvel.pyx":204
 *             options.filter_policy = NewBloomFilterPolicy(bloom_filter_bits)
 * 
 *     if (comparator is None) != (comparator_name is None):             # <<<<<<<<<<<<<<
 *         raise ValueError(
 *             "'comparator' and 'comparator_name' must be specified together")
*/
  }

  /* "plyvel/_plyvel.pyx":208
 *             "'comparator' and 'comparator_name' must be specified together")
 * 
 *     if comparator is not None:             # <<<<<<<<<<<<<<
 *         if not callable(comparator):
 *             raise TypeError("custom comparator object must be callable")
*/
  __pyx_t_8 = (__pyx_v_comparator != Py_None);
  if (__pyx_t_8) {

    /* "plyvel/_plyvel.pyx":209
 * 
 *     if comparator is not None:
 *         if not callable(comparator):             # <<<<<<<<<<<<<<
 *             raise TypeError("custom comparator object must be callable")
 * 
*/
    __pyx_t_8 = __Pyx_PyCallable_Check(__pyx_v_comparator); if (unlikely(__pyx_t_8 == ((int)-1))) __PYX_ERR(0, 209, __pyx_L1_error)
    __pyx_t_1 = (!__pyx_t_8);
    if (unlikely(__pyx_t_1)) {

      /* "plyvel/_plyvel.pyx":210
 *     if comparator is not None:
 *         if not callable(comparator):
 *             raise TypeError("custom comparator object must be callable")             # <<<<<<<<<<<<<<
 * 
 *         options.comparator = NewPlyvelCallbackComparator(
*/
      __pyx_t_6 = NULL;
      __pyx_t_3 = 1;
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_6, __pyx_mstate_global->__pyx_kp_u_custom_comparator_object_must_be};
        __pyx_t_5 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_TypeError)), __pyx_callargs+__pyx_t_3, (2-__pyx_t_3) | (__pyx_t_3*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
        if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 210, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
      }
      __Pyx_Raise(__pyx_t_5, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __PYX_ERR(0, 210, __pyx_L1_error)

      /* "plyvel/_plyvel.pyx":209
 * 
 *     if comparator is not None:
 *         if not callable(comparator):             # <<<<<<<<<<<<<<
 *             raise TypeError("custom comparator object must be callable")
 * 
*/
    }

    /* "plyvel/_plyvel.pyx":213
 * 
 *         options.comparator = NewPlyvelCallbackComparator(
 *             comparator_name, comparator)             # <<<<<<<<<<<<<<
 * 
 * 
*/
    if (unlikely(__pyx_v_comparator_name == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "expected bytes, NoneType found");
      __PYX_ERR(0, 213, __pyx_L1_error)
    }
    __pyx_t_9 = __Pyx_PyBytes_AsString(__pyx_v_comparator_name); if (unlikely((!__pyx_t_9) && PyErr_Occurred())) __PYX_ERR(0, 213, __pyx_L1_error)

    /* "plyvel/_plyvel.pyx":212
 *             raise TypeError("custom comparator object must be callable")
 * 
 *         options.comparator = NewPlyvelCallbackComparator(             # <<<<<<<<<<<<<<
 *             comparator_name, comparator)
 * 
*/
    __pyx_v_options->comparator = NewPlyvelCallbackComparator(__pyx_t_9, __pyx_v_comparator);

    /* "plyvel/_plyvel.pyx":208
 *             "'comparator' and 'comparator_name' must be specified together")
 * 
 *     if comparator is not None:             # <<<<<<<<<<<<<<
 *         if not callable(comparator):
 *             raise TypeError("custom comparator object must be callable")
*/
  }

  /* "plyvel/_plyvel.pyx":153
 * 
 * 
 * cdef int parse_options(Options *options, c_bool create_if_missing,             # <<<<<<<<<<<<<<
 *                        c_bool error_if_exists, object paranoid_checks,
 *                        object write_buffer_size, object max_open_files,
*/

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("plyvel._plyvel.parse_options", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_compression);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":228
 *     cdef dict iterators
 * 
 *     def __init__(self, name, *, bool create_if_missing=False,             # <<<<<<<<<<<<<<
 *                  bool error_if_exists=False, paranoid_checks=None,
 *                  write_buffer_size=None, max_open_files=None,
*/

/* Python wrapper */
static int __pyx_pw_6plyvel_7_plyvel_2DB_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_6plyvel_7_plyvel_2DB_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_name = 0;
  PyLongObject *__pyx_v_create_if_missing = 0;
  PyLongObject *__pyx_v_error_if_exists = 0;
  PyObject *__pyx_v_paranoid_checks = 0;
  PyObject *__pyx_v_write_buffer_size = 0;
  PyObject *__pyx_v_max_open_files = 0;
  PyObject *__pyx_v_lru_cache_size = 0;
  PyObject *__pyx_v_block_size = 0;
  PyObject *__pyx_v_block_restart_interval = 0;
  PyObject *__pyx_v_max_file_size = 0;
  PyObject *__pyx_v_compression = 0;
  int __pyx_v_bloom_filter_bits;
  PyObject *__pyx_v_comparator = 0;
  PyObject *__pyx_v_comparator_name = 0;
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[14] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return -1;
  #endif
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_name,&__pyx_mstate_global->__pyx_n_u_create_if_missing,&__pyx_mstate_global->__pyx_n_u_error_if_exists,&__pyx_mstate_global->__pyx_n_u_paranoid_checks,&__pyx_mstate_global->__pyx_n_u_write_buffer_size,&__pyx_mstate_global->__pyx_n_u_max_open_files,&__pyx_mstate_global->__pyx_n_u_lru_cache_size,&__pyx_mstate_global->__pyx_n_u_block_size,&__pyx_mstate_global->__pyx_n_u_block_restart_interval,&__pyx_mstate_global->__pyx_n_u_max_file_size,&__pyx_mstate_global->__pyx_n_u_compression,&__pyx_mstate_global->__pyx_n_u_bloom_filter_bits,&__pyx_mstate_global->__pyx_n_u_comparator,&__pyx_mstate_global->__pyx_n_u_comparator_name,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_VARARGS(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(0, 228, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 228, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__init__", 0) < (0)) __PYX_ERR(0, 228, __pyx_L3_error)
      if (!values[1]) values[1] = __Pyx_NewRef((PyObject *)((PyLongObject *)Py_False));

      /* "plyvel/_plyvel.pyx":229
 * 
 *     def __init__(self, name, *, bool create_if_missing=False,
 *                  bool error_if_exists=False, paranoid_checks=None,             # <<<<<<<<<<<<<<
 *                  write_buffer_size=None, max_open_files=None,
 *                  lru_cache_size=None, block_size=None,
*/
      if (!values[2]) values[2] = __Pyx_NewRef((PyObject *)((PyLongObject *)Py_False));
      if (!values[3]) values[3] = __Pyx_NewRef(((PyObject *)Py_None));

      /* "plyvel/_plyvel.pyx":230
 *     def __init__(self, name, *, bool create_if_missing=False,
 *                  bool error_if_exists=False, paranoid_checks=None,
 *                  write_buffer_size=None, max_open_files=None,             # <<<<<<<<<<<<<<
 *                  lru_cache_size=None, block_size=None,
 *                  block_restart_interval=None, max_file_size=None,
*/
      if (!values[4]) values[4] = __Pyx_NewRef(((PyObject *)Py_None));
      if (!values[5]) values[5] = __Pyx_NewRef(((PyObject *)Py_None));

      /* "plyvel/_plyvel.pyx":231
 *                  bool error_if_exists=False, paranoid_checks=None,
 *                  write_buffer_size=None, max_open_files=None,
 *                  lru_cache_size=None, block_size=None,             # <<<<<<<<<<<<<<
 *                  block_restart_interval=None, max_file_size=None,
 *                  compression='snappy', int bloom_filter_bits=0,
*/
      if (!values[6]) values[6] = __Pyx_NewRef(((PyObject *)Py_None));
      if (!values[7]) values[7] = __Pyx_NewRef(((PyObject *)Py_None));

      /* "plyvel/_plyvel.pyx":232
 *                  write_buffer_size=None, max_open_files=None,
 *                  lru_cache_size=None, block_size=None,
 *                  block_restart_interval=None, max_file_size=None,             # <<<<<<<<<<<<<<
 *                  compression='snappy', int bloom_filter_bits=0,
 *                  object comparator=None, bytes comparator_name=None):
*/
      if (!values[8]) values[8] = __Pyx_NewRef(((PyObject *)Py_None));
      if (!values[9]) values[9] = __Pyx_NewRef(((PyObject *)Py_None));
      if (!values[10]) values[10] = __Pyx_NewRef(((PyObject *)__pyx_mstate_global->__pyx_n_u_snappy));

      /* "plyvel/_plyvel.pyx":234
 *                  block_restart_interval=None, max_file_size=None,
 *                  compression='snappy', int bloom_filter_bits=0,
 *                  object comparator=None, bytes comparator_name=None):             # <<<<<<<<<<<<<<
 *         cdef Status st
 *         cdef string fsname
*/
      if (!values[12]) values[12] = __Pyx_NewRef(((PyObject *)Py_None));
      if (!values[13]) values[13] = __Pyx_NewRef(((PyObject*)Py_None));
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__init__", 1, 1, 1, i); __PYX_ERR(0, 228, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 228, __pyx_L3_error)

      /* "plyvel/_plyvel.pyx":228
 *     cdef dict iterators
 * 
 *     def __init__(self, name, *, bool create_if_missing=False,             # <<<<<<<<<<<<<<
 *                  bool error_if_exists=False, paranoid_checks=None,
 *                  write_buffer_size=None, max_open_files=None,
*/
      if (!values[1]) values[1] = __Pyx_NewRef((PyObject *)((PyLongObject *)Py_False));

      /* "plyvel/_plyvel.pyx":229
 * 
 *     def __init__(self, name, *, bool create_if_missing=False,
 *                  bool error_if_exists=False, paranoid_checks=None,             # <<<<<<<<<<<<<<
 *                  write_buffer_size=None, max_open_files=None,
 *                  lru_cache_size=None, block_size=None,
*/
      if (!values[2]) values[2] = __Pyx_NewRef((PyObject *)((PyLongObject *)Py_False));
      if (!values[3]) values[3] = __Pyx_NewRef(((PyObject *)Py_None));

      /* "plyvel/_plyvel.pyx":230
 *     def __init__(self, name, *, bool create_if_missing=False,
 *                  bool error_if_exists=False, paranoid_checks=None,
 *                  write_buffer_size=None, max_open_files=None,             # <<<<<<<<<<<<<<
 *                  lru_cache_size=None, block_size=None,
 *                  block_restart_interval=None, max_file_size=None,
*/
      if (!values[4]) values[4] = __Pyx_NewRef(((PyObject *)Py_None));
      if (!values[5]) values[5] = __Pyx_NewRef(((PyObject *)Py_None));

      /* "plyvel/_plyvel.pyx":231
 *                  bool error_if_exists=False, paranoid_checks=None,
 *                  write_buffer_size=None, max_open_files=None,
 *                  lru_cache_size=None, block_size=None,             # <<<<<<<<<<<<<<
 *                  block_restart_interval=None, max_file_size=None,
 *                  compression='snappy', int bloom_filter_bits=0,
*/
      if (!values[6]) values[6] = __Pyx_NewRef(((PyObject *)Py_None));
      if (!values[7]) values[7] = __Pyx_NewRef(((PyObject *)Py_None));

      /* "plyvel/_plyvel.pyx":232
 *                  write_buffer_size=None, max_open_files=None,
 *                  lru_cache_size=None, block_size=None,
 *                  block_restart_interval=None, max_file_size=None,             # <<<<<<<<<<<<<<
 *                  compression='snappy', int bloom_filter_bits=0,
 *                  object comparator=None, bytes comparator_name=None):
*/
      if (!values[8]) values[8] = __Pyx_NewRef(((PyObject *)Py_None));
      if (!values[9]) values[9] = __Pyx_NewRef(((PyObject *)Py_None));
      if (!values[10]) values[10] = __Pyx_NewRef(((PyObject *)__pyx_mstate_global->__pyx_n_u_snappy));

      /* "plyvel/_plyvel.pyx":234
 *                  block_restart_interval=None, max_file_size=None,
 *                  compression='snappy', int bloom_filter_bits=0,
 *                  object comparator=None, bytes comparator_name=None):             # <<<<<<<<<<<<<<
 *         cdef Status st
 *         cdef string fsname
*/
      if (!values[12]) values[12] = __Pyx_NewRef(((PyObject *)Py_None));
      if (!values[13]) values[13] = __Pyx_NewRef(((PyObject*)Py_None));
    }
    __pyx_v_name = values[0];
    __pyx_v_create_if_missing = ((PyLongObject *)values[1]);
    __pyx_v_error_if_exists = ((PyLongObject *)values[2]);
    __pyx_v_paranoid_checks = values[3];
    __pyx_v_write_buffer_size = values[4];
    __pyx_v_max_open_files = values[5];
    __pyx_v_lru_cache_size = values[6];
    __pyx_v_block_size = values[7];
    __pyx_v_block_restart_interval = values[8];
    __pyx_v_max_file_size = values[9];
    __pyx_v_compression = values[10];
    if (values[11]) {
      __pyx_v_bloom_filter_bits = __Pyx_PyLong_As_int(values[11]); if (unlikely((__pyx_v_bloom_filter_bits == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 233, __pyx_L3_error)
    } else {
      __pyx_v_bloom_filter_bits = ((int)0);
    }
    __pyx_v_comparator = values[12];
    __pyx_v_comparator_name = ((PyObject*)values[13]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 228, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("plyvel._plyvel.DB.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_create_if_missing), __pyx_mstate_global->__pyx_ptype_7cpython_4bool_bool, 1, "create_if_missing", 0))) __PYX_ERR(0, 228, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_error_if_exists), __pyx_mstate_global->__pyx_ptype_7cpython_4bool_bool, 1, "error_if_exists", 0))) __PYX_ERR(0, 229, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_comparator_name), (&PyBytes_Type), 1, "comparator_name", 1))) __PYX_ERR(0, 234, __pyx_L1_error)
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_2DB___init__(((struct __pyx_obj_6plyvel_7_plyvel_DB *)__pyx_v_self), __pyx_v_name, __pyx_v_create_if_missing, __pyx_v_error_if_exists, __pyx_v_paranoid_checks, __pyx_v_write_buffer_size, __pyx_v_max_open_files, __pyx_v_lru_cache_size, __pyx_v_block_size, __pyx_v_block_restart_interval, __pyx_v_max_file_size, __pyx_v_compression, __pyx_v_bloom_filter_bits, __pyx_v_comparator, __pyx_v_comparator_name);

  /* "plyvel/_plyvel.pyx":228
 *     cdef dict iterators
 * 
 *     def __init__(self, name, *, bool create_if_missing=False,             # <<<<<<<<<<<<<<
 *                  bool error_if_exists=False, paranoid_checks=None,
 *                  write_buffer_size=None, max_open_files=None,
*/

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = -1;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6plyvel_7_plyvel_2DB___init__(struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_self, PyObject *__pyx_v_name, PyLongObject *__pyx_v_create_if_missing, PyLongObject *__pyx_v_error_if_exists, PyObject *__pyx_v_paranoid_checks, PyObject *__pyx_v_write_buffer_size, PyObject *__pyx_v_max_open_files, PyObject *__pyx_v_lru_cache_size, PyObject *__pyx_v_block_size, PyObject *__pyx_v_block_restart_interval, PyObject *__pyx_v_max_file_size, PyObject *__pyx_v_compression, int __pyx_v_bloom_filter_bits, PyObject *__pyx_v_comparator, PyObject *__pyx_v_comparator_name) {
  leveldb::Status __pyx_v_st;
  std::string __pyx_v_fsname;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  std::string __pyx_t_2;
  bool __pyx_t_3;
  bool __pyx_t_4;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  size_t __pyx_t_9;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "plyvel/_plyvel.pyx":237
 *         cdef Status st
 *         cdef string fsname
 *         self.name = name             # <<<<<<<<<<<<<<
 * 
 *         fsname = to_file_system_name(name)
*/
  __Pyx_INCREF(__pyx_v_name);
  __Pyx_GIVEREF(__pyx_v_name);
  __Pyx_GOTREF(__pyx_v_self->name);
  __Pyx_DECREF(__pyx_v_self->name);
  __pyx_v_self->name = __pyx_v_name;

  /* "plyvel/_plyvel.pyx":239
 *         self.name = name
 * 
 *         fsname = to_file_system_name(name)             # <<<<<<<<<<<<<<
 *         parse_options(
 *             &self.options, create_if_missing, error_if_exists, paranoid_checks,
*/
  __pyx_t_1 = __pyx_f_6plyvel_7_plyvel_to_file_system_name(__pyx_v_name); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 239, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_convert_string_from_py_6libcpp_6string_std__in_string(__pyx_t_1); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 239, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_fsname = __PYX_STD_MOVE_IF_SUPPORTED(__pyx_t_2);

  /* "plyvel/_plyvel.pyx":241
 *         fsname = to_file_system_name(name)
 *         parse_options(
 *             &self.options, create_if_missing, error_if_exists, paranoid_checks,             # <<<<<<<<<<<<<<
 *             write_buffer_size, max_open_files, lru_cache_size, block_size,
 *             block_restart_interval, max_file_size, compression, bloom_filter_bits,
*/
  __pyx_t_3 = __Pyx_PyObject_IsTrue(((PyObject *)__pyx_v_create_if_missing)); if (unlikely((__pyx_t_3 == ((bool)-1)) && PyErr_Occurred())) __PYX_ERR(0, 241, __pyx_L1_error)
  __pyx_t_4 = __Pyx_PyObject_IsTrue(((PyObject *)__pyx_v_error_if_exists)); if (unlikely((__pyx_t_4 == ((bool)-1)) && PyErr_Occurred())) __PYX_ERR(0, 241, __pyx_L1_error)

  /* "plyvel/_plyvel.pyx":240
 * 
 *         fsname = to_file_system_name(name)
 *         parse_options(             # <<<<<<<<<<<<<<
 *             &self.options, create_if_missing, error_if_exists, paranoid_checks,
 *             write_buffer_size, max_open_files, lru_cache_size, block_size,
*/
  __pyx_t_5 = __pyx_f_6plyvel_7_plyvel_parse_options((&__pyx_v_self->options), __pyx_t_3, __pyx_t_4, __pyx_v_paranoid_checks, __pyx_v_write_buffer_size, __pyx_v_max_open_files, __pyx_v_lru_cache_size, __pyx_v_block_size, __pyx_v_block_restart_interval, __pyx_v_max_file_size, __pyx_v_compression, __pyx_v_bloom_filter_bits, __pyx_v_comparator, __pyx_v_comparator_name); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(0, 240, __pyx_L1_error)

  /* "plyvel/_plyvel.pyx":245
 *             block_restart_interval, max_file_size, compression, bloom_filter_bits,
 *             comparator, comparator_name)
 *         with nogil:             # <<<<<<<<<<<<<<
 *             st = leveldb.DB_Open(self.options, fsname, &self._db)
 *         raise_for_status(st)
*/
  {
      PyThreadState * _save;
      _save = PyEval_SaveThread();
      __Pyx_FastGIL_Remember();
      /*try:*/ {

        /* "plyvel/_plyvel.pyx":246
 *             comparator, comparator_name)
 *         with nogil:
 *             st = leveldb.DB_Open(self.options, fsname, &self._db)             # <<<<<<<<<<<<<<
 *         raise_for_status(st)
 * 
*/
        __pyx_v_st = leveldb::DB::Open(__pyx_v_self->options, __pyx_v_fsname, (&__pyx_v_self->_db));
      }

      /* "plyvel/_plyvel.pyx":245
 *             block_restart_interval, max_file_size, compression, bloom_filter_bits,
 *             comparator, comparator_name)
 *         with nogil:             # <<<<<<<<<<<<<<
 *             st = leveldb.DB_Open(self.options, fsname, &self._db)
 *         raise_for_status(st)
*/
      /*finally:*/ {
        /*normal exit:*/{
          __Pyx_FastGIL_Forget();
          PyEval_RestoreThread(_save);
          goto __pyx_L5;
        }
        __pyx_L5:;
      }
  }

  /* "plyvel/_plyvel.pyx":247
 *         with nogil:
 *             st = leveldb.DB_Open(self.options, fsname, &self._db)
 *         raise_for_status(st)             # <<<<<<<<<<<<<<
 * 
 *         # Keep weak references to open iterators, since deleting a C++
*/
  __pyx_t_5 = __pyx_f_6plyvel_7_plyvel_raise_for_status(__pyx_v_st); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(0, 247, __pyx_L1_error)

  /* "plyvel/_plyvel.pyx":254
 *         # leveldb/db.h). We don't use weakref.WeakValueDictionary here
 *         # for performance reasons.
 *         self.lock = threading.Lock()             # <<<<<<<<<<<<<<
 *         self.iterators = dict()
 * 
*/
  __pyx_t_6 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_threading); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 254, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_Lock); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 254, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_9 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_8))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_8);
    assert(__pyx_t_6);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_8);
    __Pyx_INCREF(__pyx_t_6);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_8, __pyx__function);
    __pyx_t_9 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_6, NULL};
    __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_8, __pyx_callargs+__pyx_t_9, (1-__pyx_t_9) | (__pyx_t_9*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 254, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->lock);
  __Pyx_DECREF(__pyx_v_self->lock);
  __pyx_v_self->lock = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "plyvel/_plyvel.pyx":255
 *         # for performance reasons.
 *         self.lock = threading.Lock()
 *         self.iterators = dict()             # <<<<<<<<<<<<<<
 * 
 *     cpdef close(self):
*/
  __pyx_t_1 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 255, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->iterators);
  __Pyx_DECREF(__pyx_v_self->iterators);
  __pyx_v_self->iterators = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "plyvel/_plyvel.pyx":228
 *     cdef dict iterators
 * 
 *     def __init__(self, name, *, bool create_if_missing=False,             # <<<<<<<<<<<<<<
 *                  bool error_if_exists=False, paranoid_checks=None,
 *                  write_buffer_size=None, max_open_files=None,
*/

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("plyvel._plyvel.DB.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":257
 *         self.iterators = dict()
 * 
 *     cpdef close(self):             # <<<<<<<<<<<<<<
 *         # If the constructor raised an exception (and hence never
 *         # completed), self.iterators can be None. In that case no
*/

static PyObject *__pyx_pw_6plyvel_7_plyvel_2DB_3close(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyObject *__pyx_f_6plyvel_7_plyvel_2DB_close(struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_self, CYTHON_UNUSED int __pyx_skip_dispatch) {
  struct __pyx_obj_6plyvel_7_plyvel_BaseIterator *__pyx_v_iterator = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  size_t __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  int __pyx_t_12;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("close", 0);

  /* "plyvel/_plyvel.pyx":262
 *         # iterators need to be cleaned anyway.
 *         cdef BaseIterator iterator
 *         if self.iterators is not None:             # <<<<<<<<<<<<<<
 *             with self.lock:
 *                 while self.iterators:
*/
  __pyx_t_1 = (__pyx_v_self->iterators != ((PyObject*)Py_None));
  if (__pyx_t_1) {

    /* "plyvel/_plyvel.pyx":263
 *         cdef BaseIterator iterator
 *         if self.iterators is not None:
 *             with self.lock:             # <<<<<<<<<<<<<<
 *                 while self.iterators:
 *                     iterator = self.iterators.popitem()[1]()
*/
    /*with:*/ {
      __pyx_t_2 = __Pyx_PyObject_LookupSpecial(__pyx_v_self->lock, __pyx_mstate_global->__pyx_n_u_exit); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 263, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_4 = NULL;
      __pyx_t_5 = __Pyx_PyObject_LookupSpecial(__pyx_v_self->lock, __pyx_mstate_global->__pyx_n_u_enter); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 263, __pyx_L4_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_6 = 1;
      #if CYTHON_UNPACK_METHODS
      if (likely(PyMethod_Check(__pyx_t_5))) {
        __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_5);
        assert(__pyx_t_4);
        PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_5);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(__pyx__function);
        __Pyx_DECREF_SET(__pyx_t_5, __pyx__function);
        __pyx_t_6 = 0;
      }
      #endif
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_4, NULL};
        __pyx_t_3 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_5, __pyx_callargs+__pyx_t_6, (1-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 263, __pyx_L4_error)
        __Pyx_GOTREF(__pyx_t_3);
      }
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      /*try:*/ {
        {
          __Pyx_PyThreadState_declare
          __Pyx_PyThreadState_assign
          __Pyx_ExceptionSave(&__pyx_t_7, &__pyx_t_8, &__pyx_t_9);
          __Pyx_XGOTREF(__pyx_t_7);
          __Pyx_XGOTREF(__pyx_t_8);
          __Pyx_XGOTREF(__pyx_t_9);
          /*try:*/ {

            /* "plyvel/_plyvel.pyx":264
 *         if self.iterators is not None:
 *             with self.lock:
 *                 while self.iterators:             # <<<<<<<<<<<<<<
 *                     iterator = self.iterators.popitem()[1]()
 *                     if iterator is not None:
*/
            while (1) {
              __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_self->iterators); if (unlikely((__pyx_t_1 < 0))) __PYX_ERR(0, 264, __pyx_L8_error)
              if (!__pyx_t_1) break;

              /* "plyvel/_plyvel.pyx":265
 *             with self.lock:
 *                 while self.iterators:
 *                     iterator = self.iterators.popitem()[1]()             # <<<<<<<<<<<<<<
 *                     if iterator is not None:
 *                         iterator.close()
*/
              __pyx_t_5 = NULL;
              __pyx_t_10 = __pyx_v_self->iterators;
              __Pyx_INCREF(__pyx_t_10);
              __pyx_t_6 = 0;
              {
                PyObject *__pyx_callargs[2] = {__pyx_t_10, NULL};
                __pyx_t_4 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_popitem, __pyx_callargs+__pyx_t_6, (1-__pyx_t_6) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
                __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
                if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 265, __pyx_L8_error)
                __Pyx_GOTREF(__pyx_t_4);
              }
              __pyx_t_10 = __Pyx_GetItemInt_Tuple(__pyx_t_4, 1, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_OwnStrongReference); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 265, __pyx_L8_error)
              __Pyx_GOTREF(__pyx_t_10);
              __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
              __pyx_t_6 = 1;
              #if CYTHON_UNPACK_METHODS
              if (unlikely(PyMethod_Check(__pyx_t_10))) {
                __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_10);
                assert(__pyx_t_5);
                PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_10);
                __Pyx_INCREF(__pyx_t_5);
                __Pyx_INCREF(__pyx__function);
                __Pyx_DECREF_SET(__pyx_t_10, __pyx__function);
                __pyx_t_6 = 0;
              }
              #endif
              {
                PyObject *__pyx_callargs[2] = {__pyx_t_5, NULL};
                __pyx_t_3 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_10, __pyx_callargs+__pyx_t_6, (1-__pyx_t_6) | (__pyx_t_6*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
                __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
                __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
                if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 265, __pyx_L8_error)
                __Pyx_GOTREF(__pyx_t_3);
              }
              if (!(likely(((__pyx_t_3) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_3, __pyx_mstate_global->__pyx_ptype_6plyvel_7_plyvel_BaseIterator))))) __PYX_ERR(0, 265, __pyx_L8_error)
              __Pyx_XDECREF_SET(__pyx_v_iterator, ((struct __pyx_obj_6plyvel_7_plyvel_BaseIterator *)__pyx_t_3));
              __pyx_t_3 = 0;

              /* "plyvel/_plyvel.pyx":266
 *                 while self.iterators:
 *                     iterator = self.iterators.popitem()[1]()
 *                     if iterator is not None:             # <<<<<<<<<<<<<<
 *                         iterator.close()
 * 
*/
              __pyx_t_1 = (((PyObject *)__pyx_v_iterator) != Py_None);
              if (__pyx_t_1) {

                /* "plyvel/_plyvel.pyx":267
 *                     iterator = self.iterators.popitem()[1]()
 *                     if iterator is not None:
 *                         iterator.close()             # <<<<<<<<<<<<<<
 * 
 *         if self._db is not NULL:
*/
                __pyx_t_3 = ((struct __pyx_vtabstruct_6plyvel_7_plyvel_BaseIterator *)__pyx_v_iterator->__pyx_vtab)->close(__pyx_v_iterator, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 267, __pyx_L8_error)
                __Pyx_GOTREF(__pyx_t_3);
                __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

                /* "plyvel/_plyvel.pyx":266
 *                 while self.iterators:
 *                     iterator = self.iterators.popitem()[1]()
 *                     if iterator is not None:             # <<<<<<<<<<<<<<
 *                         iterator.close()
 * 
*/
              }
            }

            /* "plyvel/_plyvel.pyx":263
 *         cdef BaseIterator iterator
 *         if self.iterators is not None:
 *             with self.lock:             # <<<<<<<<<<<<<<
 *                 while self.iterators:
 *                     iterator = self.iterators.popitem()[1]()
*/
          }
          __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
          __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
          __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
          goto __pyx_L13_try_end;
          __pyx_L8_error:;
          __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
          __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
          /*except:*/ {
            __Pyx_AddTraceback("plyvel._plyvel.DB.close", __pyx_clineno, __pyx_lineno, __pyx_filename);
            if (__Pyx_GetException(&__pyx_t_3, &__pyx_t_10, &__pyx_t_5) < 0) __PYX_ERR(0, 263, __pyx_L10_except_error)
            __Pyx_XGOTREF(__pyx_t_3);
            __Pyx_XGOTREF(__pyx_t_10);
            __Pyx_XGOTREF(__pyx_t_5);
            __pyx_t_4 = PyTuple_Pack(3, __pyx_t_3, __pyx_t_10, __pyx_t_5); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 263, __pyx_L10_except_error)
            __Pyx_GOTREF(__pyx_t_4);
            __pyx_t_11 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL);
            __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
            __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
            if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 263, __pyx_L10_except_error)
            __Pyx_GOTREF(__pyx_t_11);
            __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_t_11);
            __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
            if (__pyx_t_1 < (0)) __PYX_ERR(0, 263, __pyx_L10_except_error)
            __pyx_t_12 = (!__pyx_t_1);
            if (unlikely(__pyx_t_12)) {
              __Pyx_GIVEREF(__pyx_t_3);
              __Pyx_GIVEREF(__pyx_t_10);
              __Pyx_XGIVEREF(__pyx_t_5);
              __Pyx_ErrRestoreWithState(__pyx_t_3, __pyx_t_10, __pyx_t_5);
              __pyx_t_3 = 0;  __pyx_t_10 = 0;  __pyx_t_5 = 0; 
              __PYX_ERR(0, 263, __pyx_L10_except_error)
            }
            __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
            __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
            __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
            goto __pyx_L9_exception_handled;
          }
          __pyx_L10_except_error:;
          __Pyx_XGIVEREF(__pyx_t_7);
          __Pyx_XGIVEREF(__pyx_t_8);
          __Pyx_XGIVEREF(__pyx_t_9);
          __Pyx_ExceptionReset(__pyx_t_7, __pyx_t_8, __pyx_t_9);
          goto __pyx_L1_error;
          __pyx_L9_exception_handled:;
          __Pyx_XGIVEREF(__pyx_t_7);
          __Pyx_XGIVEREF(__pyx_t_8);
          __Pyx_XGIVEREF(__pyx_t_9);
          __Pyx_ExceptionReset(__pyx_t_7, __pyx_t_8, __pyx_t_9);
          __pyx_L13_try_end:;
        }
      }
      /*finally:*/ {
        /*normal exit:*/{
          if (__pyx_t_2) {
            __pyx_t_9 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_mstate_global->__pyx_tuple[0], NULL);
            __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
            if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 263, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_9);
            __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
          }
          goto __pyx_L7;
        }
        __pyx_L7:;
      }
      goto __pyx_L20;
      __pyx_L4_error:;
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      goto __pyx_L1_error;
      __pyx_L20:;
    }

    /* "plyvel/_plyvel.pyx":262
 *         # iterators need to be cleaned anyway.
 *         cdef BaseIterator iterator
 *         if self.iterators is not None:             # <<<<<<<<<<<<<<
 *             with self.lock:
 *                 while self.iterators:
*/
  }

  /* "plyvel/_plyvel.pyx":269
 *                         iterator.close()
 * 
 *         if self._db is not NULL:             # <<<<<<<<<<<<<<
 *             del self._db
 *             self._db = NULL
*/
  __pyx_t_12 = (__pyx_v_self->_db != NULL);
  if (__pyx_t_12) {

    /* "plyvel/_plyvel.pyx":270
 * 
 *         if self._db is not NULL:
 *             del self._db             # <<<<<<<<<<<<<<
 *             self._db = NULL
 * 
*/
    delete __pyx_v_self->_db;

    /* "plyvel/_plyvel.pyx":271
 *         if self._db is not NULL:
 *             del self._db
 *             self._db = NULL             # <<<<<<<<<<<<<<
 * 
 *         if self.options.block_cache is not NULL:
*/
    __pyx_v_self->_db = NULL;

    /* "plyvel/_plyvel.pyx":269
 *                         iterator.close()
 * 
 *         if self._db is not NULL:             # <<<<<<<<<<<<<<
 *             del self._db
 *             self._db = NULL
*/
  }

  /* "plyvel/_plyvel.pyx":273
 *             self._db = NULL
 * 
 *         if self.options.block_cache is not NULL:             # <<<<<<<<<<<<<<
 *             del self.options.block_cache
 *             self.options.block_cache = NULL
*/
  __pyx_t_12 = (__pyx_v_self->options.block_cache != NULL);
  if (__pyx_t_12) {

    /* "plyvel/_plyvel.pyx":274
 * 
 *         if self.options.block_cache is not NULL:
 *             del self.options.block_cache             # <<<<<<<<<<<<<<
 *             self.options.block_cache = NULL
 * 
*/
    delete __pyx_v_self->options.block_cache;

    /* "plyvel/_plyvel.pyx":275
 *         if self.options.block_cache is not NULL:
 *             del self.options.block_cache
 *             self.options.block_cache = NULL             # <<<<<<<<<<<<<<
 * 
 *         if self.options.filter_policy is not NULL:
*/
    __pyx_v_self->options.block_cache = NULL;

    /* "plyvel/_plyvel.pyx":273
 *             self._db = NULL
 * 
 *         if self.options.block_cache is not NULL:             # <<<<<<<<<<<<<<
 *             del self.options.block_cache
 *             self.options.block_cache = NULL
*/
  }

  /* "plyvel/_plyvel.pyx":277
 *             self.options.block_cache = NULL
 * 
 *         if self.options.filter_policy is not NULL:             # <<<<<<<<<<<<<<
 *             del self.options.filter_policy
 *             self.options.filter_policy = NULL
*/
  __pyx_t_12 = (__pyx_v_self->options.filter_policy != NULL);
  if (__pyx_t_12) {

    /* "plyvel/_plyvel.pyx":278
 * 
 *         if self.options.filter_policy is not NULL:
 *             del self.options.filter_policy             # <<<<<<<<<<<<<<
 *             self.options.filter_policy = NULL
 * 
*/
    delete __pyx_v_self->options.filter_policy;

    /* "plyvel/_plyvel.pyx":279
 *         if self.options.filter_policy is not NULL:
 *             del self.options.filter_policy
 *             self.options.filter_policy = NULL             # <<<<<<<<<<<<<<
 * 
 *         if self.options.comparator is not NULL:
*/
    __pyx_v_self->options.filter_policy = NULL;

    /* "plyvel/_plyvel.pyx":277
 *             self.options.block_cache = NULL
 * 
 *         if self.options.filter_policy is not NULL:             # <<<<<<<<<<<<<<
 *             del self.options.filter_policy
 *             self.options.filter_policy = NULL
*/
  }

  /* "plyvel/_plyvel.pyx":281
 *             self.options.filter_policy = NULL
 * 
 *         if self.options.comparator is not NULL:             # <<<<<<<<<<<<<<
 *             # The built-in BytewiseComparator must not be deleted
 *             if self.options.comparator is not BytewiseComparator():
*/
  __pyx_t_12 = (__pyx_v_self->options.comparator != NULL);
  if (__pyx_t_12) {

    /* "plyvel/_plyvel.pyx":283
 *         if self.options.comparator is not NULL:
 *             # The built-in BytewiseComparator must not be deleted
 *             if self.options.comparator is not BytewiseComparator():             # <<<<<<<<<<<<<<
 *                 del self.options.comparator
 *                 self.options.comparator = NULL
*/
    __pyx_t_12 = (__pyx_v_self->options.comparator != leveldb::BytewiseComparator());
    if (__pyx_t_12) {

      /* "plyvel/_plyvel.pyx":284
 *             # The built-in BytewiseComparator must not be deleted
 *             if self.options.comparator is not BytewiseComparator():
 *                 del self.options.comparator             # <<<<<<<<<<<<<<
 *                 self.options.comparator = NULL
 * 
*/
      delete __pyx_v_self->options.comparator;

      /* "plyvel/_plyvel.pyx":285
 *             if self.options.comparator is not BytewiseComparator():
 *                 del self.options.comparator
 *                 self.options.comparator = NULL             # <<<<<<<<<<<<<<
 * 
 *     property closed:
*/
      __pyx_v_self->options.comparator = NULL;

      /* "plyvel/_plyvel.pyx":283
 *         if self.options.comparator is not NULL:
 *             # The built-in BytewiseComparator must not be deleted
 *             if self.options.comparator is not BytewiseComparator():             # <<<<<<<<<<<<<<
 *                 del self.options.comparator
 *                 self.options.comparator = NULL
*/
    }

    /* "plyvel/_plyvel.pyx":281
 *             self.options.filter_policy = NULL
 * 
 *         if self.options.comparator is not NULL:             # <<<<<<<<<<<<<<
 *             # The built-in BytewiseComparator must not be deleted
 *             if self.options.comparator is not BytewiseComparator():
*/
  }

  /* "plyvel/_plyvel.pyx":257
 *         self.iterators = dict()
 * 
 *     cpdef close(self):             # <<<<<<<<<<<<<<
 *         # If the constructor raised an exception (and hence never
 *         # completed), self.iterators can be None. In that case no
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_AddTraceback("plyvel._plyvel.DB.close", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_iterator);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_2DB_3close(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6plyvel_7_plyvel_2DB_2close, "DB.close(self)");
static PyMethodDef __pyx_mdef_6plyvel_7_plyvel_2DB_3close = {"close", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_2DB_3close, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_2DB_2close};
static PyObject *__pyx_pw_6plyvel_7_plyvel_2DB_3close(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("close (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("close", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("close", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_2DB_2close(((struct __pyx_obj_6plyvel_7_plyvel_DB *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_2DB_2close(struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("close", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_6plyvel_7_plyvel_2DB_close(__pyx_v_self, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 257, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("plyvel._plyvel.DB.close", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":288
 * 
 *     property closed:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._db is NULL
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_2DB_6closed_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6plyvel_7_plyvel_2DB_6closed_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_2DB_6closed___get__(((struct __pyx_obj_6plyvel_7_plyvel_DB *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_2DB_6closed___get__(struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "plyvel/_plyvel.pyx":289
 *     property closed:
 *         def __get__(self):
 *             return self._db is NULL             # <<<<<<<<<<<<<<
 * 
 *     def __dealloc__(self):
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyBool_FromLong((__pyx_v_self->_db == NULL)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 289, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "plyvel/_plyvel.pyx":288
 * 
 *     property closed:
 *         def __get__(self):             # <<<<<<<<<<<<<<
 *             return self._db is NULL
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("plyvel._plyvel.DB.closed.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":291
 *             return self._db is NULL
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         self.close()
 * 
*/

/* Python wrapper */
static void __pyx_pw_6plyvel_7_plyvel_2DB_5__dealloc__(PyObject *__pyx_v_self); /*proto*/
static void __pyx_pw_6plyvel_7_plyvel_2DB_5__dealloc__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__dealloc__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_pf_6plyvel_7_plyvel_2DB_4__dealloc__(((struct __pyx_obj_6plyvel_7_plyvel_DB *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

static void __pyx_pf_6plyvel_7_plyvel_2DB_4__dealloc__(struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__dealloc__", 0);

  /* "plyvel/_plyvel.pyx":292
 * 
 *     def __dealloc__(self):
 *         self.close()             # <<<<<<<<<<<<<<
 * 
 *     def __repr__(self):
*/
  __pyx_t_1 = __pyx_f_6plyvel_7_plyvel_2DB_close(__pyx_v_self, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 292, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "plyvel/_plyvel.pyx":291
 *             return self._db is NULL
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         self.close()
 * 
*/

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_WriteUnraisable("plyvel._plyvel.DB.__dealloc__", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "plyvel/_plyvel.pyx":294
 *         self.close()
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return '<plyvel.DB with name %r%s at 0x%s>' % (
 *             self.name,
*/

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_2DB_7__repr__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6plyvel_7_plyvel_2DB_7__repr__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__repr__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_2DB_6__repr__(((struct __pyx_obj_6plyvel_7_plyvel_DB *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_2DB_6__repr__(struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6[6];
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__repr__", 0);

  /* "plyvel/_plyvel.pyx":295
 * 
 *     def __repr__(self):
 *         return '<plyvel.DB with name %r%s at 0x%s>' % (             # <<<<<<<<<<<<<<
 *             self.name,
 *             ' (closed)' if self.closed else '',
*/
  __Pyx_XDECREF(__pyx_r);

  /* "plyvel/_plyvel.pyx":296
 *     def __repr__(self):
 *         return '<plyvel.DB with name %r%s at 0x%s>' % (
 *             self.name,             # <<<<<<<<<<<<<<
 *             ' (closed)' if self.closed else '',
 *             hex(id(self)),
*/
  __pyx_t_1 = __Pyx_PyObject_FormatSimpleAndDecref(PyObject_Repr(__pyx_v_self->name), __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 296, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);

  /* "plyvel/_plyvel.pyx":297
 *         return '<plyvel.DB with name %r%s at 0x%s>' % (
 *             self.name,
 *             ' (closed)' if self.closed else '',             # <<<<<<<<<<<<<<
 *             hex(id(self)),
 *         )
*/
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_closed); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 297, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely((__pyx_t_4 < 0))) __PYX_ERR(0, 297, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (__pyx_t_4) {
    __Pyx_INCREF(__pyx_mstate_global->__pyx_kp_u_closed_2);
    __pyx_t_2 = __pyx_mstate_global->__pyx_kp_u_closed_2;
  } else {
    __Pyx_INCREF(__pyx_mstate_global->__pyx_kp_u_);
    __pyx_t_2 = __pyx_mstate_global->__pyx_kp_u_;
  }
  __pyx_t_3 = __Pyx_PyUnicode_Unicode(__pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 297, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "plyvel/_plyvel.pyx":298
 *             self.name,
 *             ' (closed)' if self.closed else '',
 *             hex(id(self)),             # <<<<<<<<<<<<<<
 *         )
 * 
*/
  __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_builtin_id, ((PyObject *)__pyx_v_self)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 298, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_5 = __Pyx_PyNumber_Hex(__pyx_t_2); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 298, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyUnicode_Unicode(__pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 298, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_t_6[0] = __pyx_mstate_global->__pyx_kp_u_plyvel_DB_with_name;
  __pyx_t_6[1] = __pyx_t_1;
  __pyx_t_6[2] = __pyx_t_3;
  __pyx_t_6[3] = __pyx_mstate_global->__pyx_kp_u_at_0x;
  __pyx_t_6[4] = __pyx_t_2;
  __pyx_t_6[5] = __pyx_mstate_global->__pyx_kp_u__2;

  /* "plyvel/_plyvel.pyx":295
 * 
 *     def __repr__(self):
 *         return '<plyvel.DB with name %r%s at 0x%s>' % (             # <<<<<<<<<<<<<<
 *             self.name,
 *             ' (closed)' if self.closed else '',
*/
  __pyx_t_5 = __Pyx_PyUnicode_Join(__pyx_t_6, 6, 21 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_1) + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_3) + 6 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_2) + 1, 127 | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_1) | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_3) | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_2));
  if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 295, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_5;
  __pyx_t_5 = 0;
  goto __pyx_L0;

  /* "plyvel/_plyvel.pyx":294
 *         self.close()
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return '<plyvel.DB with name %r%s at 0x%s>' % (
 *             self.name,
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("plyvel._plyvel.DB.__repr__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":301
 *         )
 * 
 *     def get(self, bytes key not None, default=None, *,             # <<<<<<<<<<<<<<
 *             bool verify_checksums=False, bool fill_cache=True):
 *         if self._db is NULL:
*/

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_2DB_9get(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6plyvel_7_plyvel_2DB_8get, "DB.get(self, bytes key, default=None, *, bool verify_checksums=False, bool fill_cache=True)");
static PyMethodDef __pyx_mdef_6plyvel_7_plyvel_2DB_9get = {"get", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_2DB_9get, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_2DB_8get};
static PyObject *__pyx_pw_6plyvel_7_plyvel_2DB_9get(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_key = 0;
  PyObject *__pyx_v_default = 0;
  PyLongObject *__pyx_v_verify_checksums = 0;
  PyLongObject *__pyx_v_fill_cache = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[4] = {0,0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_key,&__pyx_mstate_global->__pyx_n_u_default,&__pyx_mstate_global->__pyx_n_u_verify_checksums,&__pyx_mstate_global->__pyx_n_u_fill_cache,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(0, 301, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 301, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 301, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "get", 0) < (0)) __PYX_ERR(0, 301, __pyx_L3_error)
      if (!values[1]) values[1] = __Pyx_NewRef(((PyObject *)Py_None));

      /* "plyvel/_plyvel.pyx":302
 * 
 *     def get(self, bytes key not None, default=None, *,
 *             bool verify_checksums=False, bool fill_cache=True):             # <<<<<<<<<<<<<<
 *         if self._db is NULL:
 *             raise RuntimeError("Database is closed")
*/
      if (!values[2]) values[2] = __Pyx_NewRef((PyObject *)((PyLongObject *)Py_False));
      if (!values[3]) values[3] = __Pyx_NewRef((PyObject *)((PyLongObject *)Py_True));
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("get", 0, 1, 2, i); __PYX_ERR(0, 301, __pyx_L3_error) }
      }
    } else {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 301, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 301, __pyx_L3_error)
        break;
        default: goto __pyx_L5_argtuple_error;
      }

      /* "plyvel/_plyvel.pyx":301
 *         )
 * 
 *     def get(self, bytes key not None, default=None, *,             # <<<<<<<<<<<<<<
 *             bool verify_checksums=False, bool fill_cache=True):
 *         if self._db is NULL:
*/
      if (!values[1]) values[1] = __Pyx_NewRef(((PyObject *)Py_None));

      /* "plyvel/_plyvel.pyx":302
 * 
 *     def get(self, bytes key not None, default=None, *,
 *             bool verify_checksums=False, bool fill_cache=True):             # <<<<<<<<<<<<<<
 *         if self._db is NULL:
 *             raise RuntimeError("Database is closed")
*/
      if (!values[2]) values[2] = __Pyx_NewRef((PyObject *)((PyLongObject *)Py_False));
      if (!values[3]) values[3] = __Pyx_NewRef((PyObject *)((PyLongObject *)Py_True));
    }
    __pyx_v_key = ((PyObject*)values[0]);
    __pyx_v_default = values[1];
    __pyx_v_verify_checksums = ((PyLongObject *)values[2]);
    __pyx_v_fill_cache = ((PyLongObject *)values[3]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("get", 0, 1, 2, __pyx_nargs); __PYX_ERR(0, 301, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("plyvel._plyvel.DB.get", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_key), (&PyBytes_Type), 0, "key", 1))) __PYX_ERR(0, 301, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_verify_checksums), __pyx_mstate_global->__pyx_ptype_7cpython_4bool_bool, 1, "verify_checksums", 0))) __PYX_ERR(0, 302, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_fill_cache), __pyx_mstate_global->__pyx_ptype_7cpython_4bool_bool, 1, "fill_cache", 0))) __PYX_ERR(0, 302, __pyx_L1_error)
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_2DB_8get(((struct __pyx_obj_6plyvel_7_plyvel_DB *)__pyx_v_self), __pyx_v_key, __pyx_v_default, __pyx_v_verify_checksums, __pyx_v_fill_cache);

  /* "plyvel/_plyvel.pyx":301
 *         )
 * 
 *     def get(self, bytes key not None, default=None, *,             # <<<<<<<<<<<<<<
 *             bool verify_checksums=False, bool fill_cache=True):
 *         if self._db is NULL:
*/

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_2DB_8get(struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_self, PyObject *__pyx_v_key, PyObject *__pyx_v_default, PyLongObject *__pyx_v_verify_checksums, PyLongObject *__pyx_v_fill_cache) {
  leveldb::ReadOptions __pyx_v_read_options;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  bool __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get", 0);

  /* "plyvel/_plyvel.pyx":303
 *     def get(self, bytes key not None, default=None, *,
 *             bool verify_checksums=False, bool fill_cache=True):
 *         if self._db is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database is closed")
 * 
*/
  __pyx_t_1 = (__pyx_v_self->_db == NULL);
  if (unlikely(__pyx_t_1)) {

    /* "plyvel/_plyvel.pyx":304
 *             bool verify_checksums=False, bool fill_cache=True):
 *         if self._db is NULL:
 *             raise RuntimeError("Database is closed")             # <<<<<<<<<<<<<<
 * 
 *         cdef ReadOptions read_options
*/
    __pyx_t_3 = NULL;
    __pyx_t_4 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_mstate_global->__pyx_kp_u_Database_is_closed};
      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_RuntimeError)), __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 304, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 304, __pyx_L1_error)

    /* "plyvel/_plyvel.pyx":303
 *     def get(self, bytes key not None, default=None, *,
 *             bool verify_checksums=False, bool fill_cache=True):
 *         if self._db is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database is closed")
 * 
*/
  }

  /* "plyvel/_plyvel.pyx":307
 * 
 *         cdef ReadOptions read_options
 *         read_options.verify_checksums = verify_checksums             # <<<<<<<<<<<<<<
 *         read_options.fill_cache = fill_cache
 * 
*/
  __pyx_t_5 = __Pyx_PyObject_IsTrue(((PyObject *)__pyx_v_verify_checksums)); if (unlikely((__pyx_t_5 == ((bool)-1)) && PyErr_Occurred())) __PYX_ERR(0, 307, __pyx_L1_error)
  __pyx_v_read_options.verify_checksums = __pyx_t_5;

  /* "plyvel/_plyvel.pyx":308
 *         cdef ReadOptions read_options
 *         read_options.verify_checksums = verify_checksums
 *         read_options.fill_cache = fill_cache             # <<<<<<<<<<<<<<
 * 
 *         return db_get(self, key, default, read_options)
*/
  __pyx_t_5 = __Pyx_PyObject_IsTrue(((PyObject *)__pyx_v_fill_cache)); if (unlikely((__pyx_t_5 == ((bool)-1)) && PyErr_Occurred())) __PYX_ERR(0, 308, __pyx_L1_error)
  __pyx_v_read_options.fill_cache = __pyx_t_5;

  /* "plyvel/_plyvel.pyx":310
 *         read_options.fill_cache = fill_cache
 * 
 *         return db_get(self, key, default, read_options)             # <<<<<<<<<<<<<<
 * 
 *     def put(self, bytes key not None, value not None, *, bool sync=False):
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __pyx_f_6plyvel_7_plyvel_db_get(__pyx_v_self, __pyx_v_key, __pyx_v_default, __pyx_v_read_options); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 310, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "plyvel/_plyvel.pyx":301
 *         )
 * 
 *     def get(self, bytes key not None, default=None, *,             # <<<<<<<<<<<<<<
 *             bool verify_checksums=False, bool fill_cache=True):
 *         if self._db is NULL:
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("plyvel._plyvel.DB.get", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":312
 *         return db_get(self, key, default, read_options)
 * 
 *     def put(self, bytes key not None, value not None, *, bool sync=False):             # <<<<<<<<<<<<<<
 *         if self._db is NULL:
 *             raise RuntimeError("Database is closed")
*/

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_2DB_11put(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6plyvel_7_plyvel_2DB_10put, "DB.put(self, bytes key, value, *, bool sync=False)");
static PyMethodDef __pyx_mdef_6plyvel_7_plyvel_2DB_11put = {"put", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_2DB_11put, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_2DB_10put};
static PyObject *__pyx_pw_6plyvel_7_plyvel_2DB_11put(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_key = 0;
  PyObject *__pyx_v_value = 0;
  PyLongObject *__pyx_v_sync = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[3] = {0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("put (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_key,&__pyx_mstate_global->__pyx_n_u_value,&__pyx_mstate_global->__pyx_n_u_sync,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(0, 312, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 312, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 312, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "put", 0) < (0)) __PYX_ERR(0, 312, __pyx_L3_error)
      if (!values[2]) values[2] = __Pyx_NewRef((PyObject *)((PyLongObject *)Py_False));
      for (Py_ssize_t i = __pyx_nargs; i < 2; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("put", 1, 2, 2, i); __PYX_ERR(0, 312, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 2)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 312, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 312, __pyx_L3_error)
      if (!values[2]) values[2] = __Pyx_NewRef((PyObject *)((PyLongObject *)Py_False));
    }
    __pyx_v_key = ((PyObject*)values[0]);
    __pyx_v_value = values[1];
    __pyx_v_sync = ((PyLongObject *)values[2]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("put", 1, 2, 2, __pyx_nargs); __PYX_ERR(0, 312, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("plyvel._plyvel.DB.put", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_key), (&PyBytes_Type), 0, "key", 1))) __PYX_ERR(0, 312, __pyx_L1_error)
  if (unlikely(((PyObject *)__pyx_v_value) == Py_None)) {
    PyErr_Format(PyExc_TypeError, "Argument '%.200s' must not be None", "value"); __PYX_ERR(0, 312, __pyx_L1_error)
  }
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_sync), __pyx_mstate_global->__pyx_ptype_7cpython_4bool_bool, 1, "sync", 0))) __PYX_ERR(0, 312, __pyx_L1_error)
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_2DB_10put(((struct __pyx_obj_6plyvel_7_plyvel_DB *)__pyx_v_self), __pyx_v_key, __pyx_v_value, __pyx_v_sync);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_2DB_10put(struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_self, PyObject *__pyx_v_key, PyObject *__pyx_v_value, PyLongObject *__pyx_v_sync) {
  leveldb::WriteOptions __pyx_v_write_options;
  leveldb::Slice __pyx_v_key_slice;
  Py_buffer __pyx_v_value_buffer;
  leveldb::Status __pyx_v_st;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  bool __pyx_t_5;
  const char *__pyx_t_6;
  Py_ssize_t __pyx_t_7;
  int __pyx_t_8;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("put", 0);

  /* "plyvel/_plyvel.pyx":313
 * 
 *     def put(self, bytes key not None, value not None, *, bool sync=False):
 *         if self._db is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database is closed")
 * 
*/
  __pyx_t_1 = (__pyx_v_self->_db == NULL);
  if (unlikely(__pyx_t_1)) {

    /* "plyvel/_plyvel.pyx":314
 *     def put(self, bytes key not None, value not None, *, bool sync=False):
 *         if self._db is NULL:
 *             raise RuntimeError("Database is closed")             # <<<<<<<<<<<<<<
 * 
 *         cdef WriteOptions write_options = WriteOptions()
*/
    __pyx_t_3 = NULL;
    __pyx_t_4 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_mstate_global->__pyx_kp_u_Database_is_closed};
      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_RuntimeError)), __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 314, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 314, __pyx_L1_error)

    /* "plyvel/_plyvel.pyx":313
 * 
 *     def put(self, bytes key not None, value not None, *, bool sync=False):
 *         if self._db is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database is closed")
 * 
*/
  }

  /* "plyvel/_plyvel.pyx":316
 *             raise RuntimeError("Database is closed")
 * 
 *         cdef WriteOptions write_options = WriteOptions()             # <<<<<<<<<<<<<<
 *         write_options.sync = sync
 * 
*/
  __pyx_v_write_options = leveldb::WriteOptions();

  /* "plyvel/_plyvel.pyx":317
 * 
 *         cdef WriteOptions write_options = WriteOptions()
 *         write_options.sync = sync             # <<<<<<<<<<<<<<
 * 
 *         cdef Slice key_slice = Slice(key, len(key))
*/
  __pyx_t_5 = __Pyx_PyObject_IsTrue(((PyObject *)__pyx_v_sync)); if (unlikely((__pyx_t_5 == ((bool)-1)) && PyErr_Occurred())) __PYX_ERR(0, 317, __pyx_L1_error)
  __pyx_v_write_options.sync = __pyx_t_5;

  /* "plyvel/_plyvel.pyx":319
 *         write_options.sync = sync
 * 
 *         cdef Slice key_slice = Slice(key, len(key))             # <<<<<<<<<<<<<<
 *         cdef Py_buffer value_buffer
 *         cdef Status st
*/
  __pyx_t_6 = __Pyx_PyBytes_AsString(__pyx_v_key); if (unlikely((!__pyx_t_6) && PyErr_Occurred())) __PYX_ERR(0, 319, __pyx_L1_error)
  __pyx_t_7 = __Pyx_PyBytes_GET_SIZE(__pyx_v_key); if (unlikely(__pyx_t_7 == ((Py_ssize_t)-1))) __PYX_ERR(0, 319, __pyx_L1_error)
  __pyx_v_key_slice = leveldb::Slice(__pyx_t_6, __pyx_t_7);

  /* "plyvel/_plyvel.pyx":322
 *         cdef Py_buffer value_buffer
 *         cdef Status st
 *         PyObject_GetBuffer(value, &value_buffer, PyBUF_SIMPLE)             # <<<<<<<<<<<<<<
 *         try:
 *             with nogil:
*/
  __pyx_t_8 = PyObject_GetBuffer(__pyx_v_value, (&__pyx_v_value_buffer), PyBUF_SIMPLE); if (unlikely(__pyx_t_8 == ((int)-1))) __PYX_ERR(0, 322, __pyx_L1_error)

  /* "plyvel/_plyvel.pyx":323
 *         cdef Status st
 *         PyObject_GetBuffer(value, &value_buffer, PyBUF_SIMPLE)
 *         try:             # <<<<<<<<<<<<<<
 *             with nogil:
 *                 st = self._db.Put(
*/
  /*try:*/ {

    /* "plyvel/_plyvel.pyx":324
 *         PyObject_GetBuffer(value, &value_buffer, PyBUF_SIMPLE)
 *         try:
 *             with nogil:             # <<<<<<<<<<<<<<
 *                 st = self._db.Put(
 *                     write_options,
*/
    {
        PyThreadState * _save;
        _save = PyEval_SaveThread();
        __Pyx_FastGIL_Remember();
        /*try:*/ {

          /* "plyvel/_plyvel.pyx":325
 *         try:
 *             with nogil:
 *                 st = self._db.Put(             # <<<<<<<<<<<<<<
 *                     write_options,
 *                     key_slice,
*/
          __pyx_v_st = __pyx_v_self->_db->Put(__pyx_v_write_options, __pyx_v_key_slice, leveldb::Slice(((const char *)__pyx_v_value_buffer.buf), __pyx_v_value_buffer.len));
        }

        /* "plyvel/_plyvel.pyx":324
 *         PyObject_GetBuffer(value, &value_buffer, PyBUF_SIMPLE)
 *         try:
 *             with nogil:             # <<<<<<<<<<<<<<
 *                 st = self._db.Put(
 *                     write_options,
*/
        /*finally:*/ {
          /*normal exit:*/{
            __Pyx_FastGIL_Forget();
            PyEval_RestoreThread(_save);
            goto __pyx_L9;
          }
          __pyx_L9:;
        }
    }
  }

  /* "plyvel/_plyvel.pyx":330
 *                     Slice(<const_char *>value_buffer.buf, value_buffer.len))
 *         finally:
 *             PyBuffer_Release(&value_buffer)             # <<<<<<<<<<<<<<
 *         raise_for_status(st)
 * 
*/
  /*finally:*/ {
    /*normal exit:*/{
      PyBuffer_Release((&__pyx_v_value_buffer));
      goto __pyx_L6;
    }
    __pyx_L6:;
  }

  /* "plyvel/_plyvel.pyx":331
 *         finally:
 *             PyBuffer_Release(&value_buffer)
 *         raise_for_status(st)             # <<<<<<<<<<<<<<
 * 
 *     def delete(self, bytes key not None, *, bool sync=False):
*/
  __pyx_t_8 = __pyx_f_6plyvel_7_plyvel_raise_for_status(__pyx_v_st); if (unlikely(__pyx_t_8 == ((int)-1))) __PYX_ERR(0, 331, __pyx_L1_error)

  /* "plyvel/_plyvel.pyx":312
 *         return db_get(self, key, default, read_options)
 * 
 *     def put(self, bytes key not None, value not None, *, bool sync=False):             # <<<<<<<<<<<<<<
 *         if self._db is NULL:
 *             raise RuntimeError("Database is closed")
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("plyvel._plyvel.DB.put", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":333
 *         raise_for_status(st)
 * 
 *     def delete(self, bytes key not None, *, bool sync=False):             # <<<<<<<<<<<<<<
 *         if self._db is NULL:
 *             raise RuntimeError("Database is closed")
*/

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_2DB_13delete(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6plyvel_7_plyvel_2DB_12delete, "DB.delete(self, bytes key, *, bool sync=False)");
static PyMethodDef __pyx_mdef_6plyvel_7_plyvel_2DB_13delete = {"delete", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_2DB_13delete, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_2DB_12delete};
static PyObject *__pyx_pw_6plyvel_7_plyvel_2DB_13delete(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_key = 0;
  PyLongObject *__pyx_v_sync = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("delete (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_key,&__pyx_mstate_global->__pyx_n_u_sync,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(0, 333, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 333, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "delete", 0) < (0)) __PYX_ERR(0, 333, __pyx_L3_error)
      if (!values[1]) values[1] = __Pyx_NewRef((PyObject *)((PyLongObject *)Py_False));
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("delete", 1, 1, 1, i); __PYX_ERR(0, 333, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 333, __pyx_L3_error)
      if (!values[1]) values[1] = __Pyx_NewRef((PyObject *)((PyLongObject *)Py_False));
    }
    __pyx_v_key = ((PyObject*)values[0]);
    __pyx_v_sync = ((PyLongObject *)values[1]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("delete", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 333, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("plyvel._plyvel.DB.delete", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_key), (&PyBytes_Type), 0, "key", 1))) __PYX_ERR(0, 333, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_sync), __pyx_mstate_global->__pyx_ptype_7cpython_4bool_bool, 1, "sync", 0))) __PYX_ERR(0, 333, __pyx_L1_error)
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_2DB_12delete(((struct __pyx_obj_6plyvel_7_plyvel_DB *)__pyx_v_self), __pyx_v_key, __pyx_v_sync);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_2DB_12delete(struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_self, PyObject *__pyx_v_key, PyLongObject *__pyx_v_sync) {
  leveldb::Status __pyx_v_st;
  leveldb::WriteOptions __pyx_v_write_options;
  leveldb::Slice __pyx_v_key_slice;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  bool __pyx_t_5;
  const char *__pyx_t_6;
  Py_ssize_t __pyx_t_7;
  int __pyx_t_8;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("delete", 0);

  /* "plyvel/_plyvel.pyx":334
 * 
 *     def delete(self, bytes key not None, *, bool sync=False):
 *         if self._db is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database is closed")
 * 
*/
  __pyx_t_1 = (__pyx_v_self->_db == NULL);
  if (unlikely(__pyx_t_1)) {

    /* "plyvel/_plyvel.pyx":335
 *     def delete(self, bytes key not None, *, bool sync=False):
 *         if self._db is NULL:
 *             raise RuntimeError("Database is closed")             # <<<<<<<<<<<<<<
 * 
 *         cdef Status st
*/
    __pyx_t_3 = NULL;
    __pyx_t_4 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_mstate_global->__pyx_kp_u_Database_is_closed};
      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_RuntimeError)), __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 335, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 335, __pyx_L1_error)

    /* "plyvel/_plyvel.pyx":334
 * 
 *     def delete(self, bytes key not None, *, bool sync=False):
 *         if self._db is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database is closed")
 * 
*/
  }

  /* "plyvel/_plyvel.pyx":339
 *         cdef Status st
 *         cdef WriteOptions write_options
 *         write_options.sync = sync             # <<<<<<<<<<<<<<
 * 
 *         cdef Slice key_slice = Slice(key, len(key))
*/
  __pyx_t_5 = __Pyx_PyObject_IsTrue(((PyObject *)__pyx_v_sync)); if (unlikely((__pyx_t_5 == ((bool)-1)) && PyErr_Occurred())) __PYX_ERR(0, 339, __pyx_L1_error)
  __pyx_v_write_options.sync = __pyx_t_5;

  /* "plyvel/_plyvel.pyx":341
 *         write_options.sync = sync
 * 
 *         cdef Slice key_slice = Slice(key, len(key))             # <<<<<<<<<<<<<<
 *         with nogil:
 *             st = self._db.Delete(write_options, key_slice)
*/
  __pyx_t_6 = __Pyx_PyBytes_AsString(__pyx_v_key); if (unlikely((!__pyx_t_6) && PyErr_Occurred())) __PYX_ERR(0, 341, __pyx_L1_error)
  __pyx_t_7 = __Pyx_PyBytes_GET_SIZE(__pyx_v_key); if (unlikely(__pyx_t_7 == ((Py_ssize_t)-1))) __PYX_ERR(0, 341, __pyx_L1_error)
  __pyx_v_key_slice = leveldb::Slice(__pyx_t_6, __pyx_t_7);

  /* "plyvel/_plyvel.pyx":342
 * 
 *         cdef Slice key_slice = Slice(key, len(key))
 *         with nogil:             # <<<<<<<<<<<<<<
 *             st = self._db.Delete(write_options, key_slice)
 *         raise_for_status(st)
*/
  {
      PyThreadState * _save;
      _save = PyEval_SaveThread();
      __Pyx_FastGIL_Remember();
      /*try:*/ {

        /* "plyvel/_plyvel.pyx":343
 *         cdef Slice key_slice = Slice(key, len(key))
 *         with nogil:
 *             st = self._db.Delete(write_options, key_slice)             # <<<<<<<<<<<<<<
 *         raise_for_status(st)
 * 
*/
        __pyx_v_st = __pyx_v_self->_db->Delete(__pyx_v_write_options, __pyx_v_key_slice);
      }

      /* "plyvel/_plyvel.pyx":342
 * 
 *         cdef Slice key_slice = Slice(key, len(key))
 *         with nogil:             # <<<<<<<<<<<<<<
 *             st = self._db.Delete(write_options, key_slice)
 *         raise_for_status(st)
*/
      /*finally:*/ {
        /*normal exit:*/{
          __Pyx_FastGIL_Forget();
          PyEval_RestoreThread(_save);
          goto __pyx_L6;
        }
        __pyx_L6:;
      }
  }

  /* "plyvel/_plyvel.pyx":344
 *         with nogil:
 *             st = self._db.Delete(write_options, key_slice)
 *         raise_for_status(st)             # <<<<<<<<<<<<<<
 * 
 *     def write_batch(self, *, bool transaction=False, bool sync=False):
*/
  __pyx_t_8 = __pyx_f_6plyvel_7_plyvel_raise_for_status(__pyx_v_st); if (unlikely(__pyx_t_8 == ((int)-1))) __PYX_ERR(0, 344, __pyx_L1_error)

  /* "plyvel/_plyvel.pyx":333
 *         raise_for_status(st)
 * 
 *     def delete(self, bytes key not None, *, bool sync=False):             # <<<<<<<<<<<<<<
 *         if self._db is NULL:
 *             raise RuntimeError("Database is closed")
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("plyvel._plyvel.DB.delete", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":346
 *         raise_for_status(st)
 * 
 *     def write_batch(self, *, bool transaction=False, bool sync=False):             # <<<<<<<<<<<<<<
 *         if self._db is NULL:
 *             raise RuntimeError("Database is closed")
*/

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_2DB_15write_batch(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6plyvel_7_plyvel_2DB_14write_batch, "DB.write_batch(self, *, bool transaction=False, bool sync=False)");
static PyMethodDef __pyx_mdef_6plyvel_7_plyvel_2DB_15write_batch = {"write_batch", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_2DB_15write_batch, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_2DB_14write_batch};
static PyObject *__pyx_pw_6plyvel_7_plyvel_2DB_15write_batch(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyLongObject *__pyx_v_transaction = 0;
  PyLongObject *__pyx_v_sync = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("write_batch (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_transaction,&__pyx_mstate_global->__pyx_n_u_sync,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(0, 346, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, 0, __pyx_kwds_len, "write_batch", 0) < (0)) __PYX_ERR(0, 346, __pyx_L3_error)
      if (!values[0]) values[0] = __Pyx_NewRef((PyObject *)((PyLongObject *)Py_False));
      if (!values[1]) values[1] = __Pyx_NewRef((PyObject *)((PyLongObject *)Py_False));
    } else if (unlikely(__pyx_nargs != 0)) {
      goto __pyx_L5_argtuple_error;
    } else {
      if (!values[0]) values[0] = __Pyx_NewRef((PyObject *)((PyLongObject *)Py_False));
      if (!values[1]) values[1] = __Pyx_NewRef((PyObject *)((PyLongObject *)Py_False));
    }
    __pyx_v_transaction = ((PyLongObject *)values[0]);
    __pyx_v_sync = ((PyLongObject *)values[1]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("write_batch", 1, 0, 0, __pyx_nargs); __PYX_ERR(0, 346, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("plyvel._plyvel.DB.write_batch", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_transaction), __pyx_mstate_global->__pyx_ptype_7cpython_4bool_bool, 1, "transaction", 0))) __PYX_ERR(0, 346, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_sync), __pyx_mstate_global->__pyx_ptype_7cpython_4bool_bool, 1, "sync", 0))) __PYX_ERR(0, 346, __pyx_L1_error)
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_2DB_14write_batch(((struct __pyx_obj_6plyvel_7_plyvel_DB *)__pyx_v_self), __pyx_v_transaction, __pyx_v_sync);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_2DB_14write_batch(struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_self, PyLongObject *__pyx_v_transaction, PyLongObject *__pyx_v_sync) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("write_batch", 0);

  /* "plyvel/_plyvel.pyx":347
 * 
 *     def write_batch(self, *, bool transaction=False, bool sync=False):
 *         if self._db is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database is closed")
 * 
*/
  __pyx_t_1 = (__pyx_v_self->_db == NULL);
  if (unlikely(__pyx_t_1)) {

    /* "plyvel/_plyvel.pyx":348
 *     def write_batch(self, *, bool transaction=False, bool sync=False):
 *         if self._db is NULL:
 *             raise RuntimeError("Database is closed")             # <<<<<<<<<<<<<<
 * 
 *         return WriteBatch(self, None, transaction, sync)
*/
    __pyx_t_3 = NULL;
    __pyx_t_4 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_mstate_global->__pyx_kp_u_Database_is_closed};
      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_RuntimeError)), __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 348, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 348, __pyx_L1_error)

    /* "plyvel/_plyvel.pyx":347
 * 
 *     def write_batch(self, *, bool transaction=False, bool sync=False):
 *         if self._db is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database is closed")
 * 
*/
  }

  /* "plyvel/_plyvel.pyx":350
 *             raise RuntimeError("Database is closed")
 * 
 *         return WriteBatch(self, None, transaction, sync)             # <<<<<<<<<<<<<<
 * 
 *     def __iter__(self):
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_3 = NULL;
  __pyx_t_4 = 1;
  {
    PyObject *__pyx_callargs[5] = {__pyx_t_3, ((PyObject *)__pyx_v_self), Py_None, ((PyObject *)__pyx_v_transaction), ((PyObject *)__pyx_v_sync)};
    __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_mstate_global->__pyx_ptype_6plyvel_7_plyvel_WriteBatch, __pyx_callargs+__pyx_t_4, (5-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 350, __pyx_L1_error)
    __Pyx_GOTREF((PyObject *)__pyx_t_2);
  }
  __pyx_r = ((PyObject *)__pyx_t_2);
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "plyvel/_plyvel.pyx":346
 *         raise_for_status(st)
 * 
 *     def write_batch(self, *, bool transaction=False, bool sync=False):             # <<<<<<<<<<<<<<
 *         if self._db is NULL:
 *             raise RuntimeError("Database is closed")
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("plyvel._plyvel.DB.write_batch", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":352
 *         return WriteBatch(self, None, transaction, sync)
 * 
 *     def __iter__(self):             # <<<<<<<<<<<<<<
 *         if self._db is NULL:
 *             raise RuntimeError("Database is closed")
*/

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_2DB_17__iter__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6plyvel_7_plyvel_2DB_17__iter__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__iter__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_2DB_16__iter__(((struct __pyx_obj_6plyvel_7_plyvel_DB *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_2DB_16__iter__(struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__iter__", 0);

  /* "plyvel/_plyvel.pyx":353
 * 
 *     def __iter__(self):
 *         if self._db is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database is closed")
 * 
*/
  __pyx_t_1 = (__pyx_v_self->_db == NULL);
  if (unlikely(__pyx_t_1)) {

    /* "plyvel/_plyvel.pyx":354
 *     def __iter__(self):
 *         if self._db is NULL:
 *             raise RuntimeError("Database is closed")             # <<<<<<<<<<<<<<
 * 
 *         return self.iterator()
*/
    __pyx_t_3 = NULL;
    __pyx_t_4 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_mstate_global->__pyx_kp_u_Database_is_closed};
      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_RuntimeError)), __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 354, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 354, __pyx_L1_error)

    /* "plyvel/_plyvel.pyx":353
 * 
 *     def __iter__(self):
 *         if self._db is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database is closed")
 * 
*/
  }

  /* "plyvel/_plyvel.pyx":356
 *             raise RuntimeError("Database is closed")
 * 
 *         return self.iterator()             # <<<<<<<<<<<<<<
 * 
 *     def iterator(self, *, reverse=False, start=None, stop=None,
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_3 = ((PyObject *)__pyx_v_self);
  __Pyx_INCREF(__pyx_t_3);
  __pyx_t_4 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_3, NULL};
    __pyx_t_2 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_iterator, __pyx_callargs+__pyx_t_4, (1-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 356, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
  }
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "plyvel/_plyvel.pyx":352
 *         return WriteBatch(self, None, transaction, sync)
 * 
 *     def __iter__(self):             # <<<<<<<<<<<<<<
 *         if self._db is NULL:
 *             raise RuntimeError("Database is closed")
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("plyvel._plyvel.DB.__iter__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":358
 *         return self.iterator()
 * 
 *     def iterator(self, *, reverse=False, start=None, stop=None,             # <<<<<<<<<<<<<<
 *                  include_start=True, include_stop=False, prefix=None,
 *                  include_key=True, include_value=True,
*/

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_2DB_19iterator(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6plyvel_7_plyvel_2DB_18iterator, "DB.iterator(self, *, reverse=False, start=None, stop=None, include_start=True, include_stop=False, prefix=None, include_key=True, include_value=True, bool verify_checksums=False, bool fill_cache=True)");
static PyMethodDef __pyx_mdef_6plyvel_7_plyvel_2DB_19iterator = {"iterator", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_2DB_19iterator, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_2DB_18iterator};
static PyObject *__pyx_pw_6plyvel_7_plyvel_2DB_19iterator(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_reverse = 0;
  PyObject *__pyx_v_start = 0;
  PyObject *__pyx_v_stop = 0;
  PyObject *__pyx_v_include_start = 0;
  PyObject *__pyx_v_include_stop = 0;
  PyObject *__pyx_v_prefix = 0;
  PyObject *__pyx_v_include_key = 0;
  PyObject *__pyx_v_include_value = 0;
  PyLongObject *__pyx_v_verify_checksums = 0;
  PyLongObject *__pyx_v_fill_cache = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[10] = {0,0,0,0,0,0,0,0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("iterator (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_reverse,&__pyx_mstate_global->__pyx_n_u_start,&__pyx_mstate_global->__pyx_n_u_stop,&__pyx_mstate_global->__pyx_n_u_include_start,&__pyx_mstate_global->__pyx_n_u_include_stop,&__pyx_mstate_global->__pyx_n_u_prefix,&__pyx_mstate_global->__pyx_n_u_include_key,&__pyx_mstate_global->__pyx_n_u_include_value,&__pyx_mstate_global->__pyx_n_u_verify_checksums,&__pyx_mstate_global->__pyx_n_u_fill_cache,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(0, 358, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, 0, __pyx_kwds_len, "iterator", 0) < (0)) __PYX_ERR(0, 358, __pyx_L3_error)
      if (!values[0]) values[0] = __Pyx_NewRef(((PyObject *)Py_False));
      if (!values[1]) values[1] = __Pyx_NewRef(((PyObject *)Py_None));
      if (!values[2]) values[2] = __Pyx_NewRef(((PyObject *)Py_None));

      /* "plyvel/_plyvel.pyx":359
 * 
 *     def iterator(self, *, reverse=False, start=None, stop=None,
 *                  include_start=True, include_stop=False, prefix=None,             # <<<<<<<<<<<<<<
 *                  include_key=True, include_value=True,
 *                  bool verify_checksums=False, bool fill_cache=True):
*/
      if (!values[3]) values[3] = __Pyx_NewRef(((PyObject *)Py_True));
      if (!values[4]) values[4] = __Pyx_NewRef(((PyObject *)Py_False));
      if (!values[5]) values[5] = __Pyx_NewRef(((PyObject *)Py_None));

      /* "plyvel/_plyvel.pyx":360
 *     def iterator(self, *, reverse=False, start=None, stop=None,
 *                  include_start=True, include_stop=False, prefix=None,
 *                  include_key=True, include_value=True,             # <<<<<<<<<<<<<<
 *                  bool verify_checksums=False, bool fill_cache=True):
 *         return Iterator(
*/
      if (!values[6]) values[6] = __Pyx_NewRef(((PyObject *)Py_True));
      if (!values[7]) values[7] = __Pyx_NewRef(((PyObject *)Py_True));

      /* "plyvel/_plyvel.pyx":361
 *                  include_start=True, include_stop=False, prefix=None,
 *                  include_key=True, include_value=True,
 *                  bool verify_checksums=False, bool fill_cache=True):             # <<<<<<<<<<<<<<
 *         return Iterator(
 *             self,  # db
*/
      if (!values[8]) values[8] = __Pyx_NewRef((PyObject *)((PyLongObject *)Py_False));
      if (!values[9]) values[9] = __Pyx_NewRef((PyObject *)((PyLongObject *)Py_True));
    } else if (unlikely(__pyx_nargs != 0)) {
      goto __pyx_L5_argtuple_error;
    } else {

      /* "plyvel/_plyvel.pyx":358
 *         return self.iterator()
 * 
 *     def iterator(self, *, reverse=False, start=None, stop=None,             # <<<<<<<<<<<<<<
 *                  include_start=True, include_stop=False, prefix=None,
 *                  include_key=True, include_value=True,
*/
      if (!values[0]) values[0] = __Pyx_NewRef(((PyObject *)Py_False));
      if (!values[1]) values[1] = __Pyx_NewRef(((PyObject *)Py_None));
      if (!values[2]) values[2] = __Pyx_NewRef(((PyObject *)Py_None));

      /* "plyvel/_plyvel.pyx":359
 * 
 *     def iterator(self, *, reverse=False, start=None, stop=None,
 *                  include_start=True, include_stop=False, prefix=None,             # <<<<<<<<<<<<<<
 *                  include_key=True, include_value=True,
 *                  bool verify_checksums=False, bool fill_cache=True):
*/
      if (!values[3]) values[3] = __Pyx_NewRef(((PyObject *)Py_True));
      if (!values[4]) values[4] = __Pyx_NewRef(((PyObject *)Py_False));
      if (!values[5]) values[5] = __Pyx_NewRef(((PyObject *)Py_None));

      /* "plyvel/_plyvel.pyx":360
 *     def iterator(self, *, reverse=False, start=None, stop=None,
 *                  include_start=True, include_stop=False, prefix=None,
 *                  include_key=True, include_value=True,             # <<<<<<<<<<<<<<
 *                  bool verify_checksums=False, bool fill_cache=True):
 *         return Iterator(
*/
      if (!values[6]) values[6] = __Pyx_NewRef(((PyObject *)Py_True));
      if (!values[7]) values[7] = __Pyx_NewRef(((PyObject *)Py_True));

      /* "plyvel/_plyvel.pyx":361
 *                  include_start=True, include_stop=False, prefix=None,
 *                  include_key=True, include_value=True,
 *                  bool verify_checksums=False, bool fill_cache=True):             # <<<<<<<<<<<<<<
 *         return Iterator(
 *             self,  # db
*/
      if (!values[8]) values[8] = __Pyx_NewRef((PyObject *)((PyLongObject *)Py_False));
      if (!values[9]) values[9] = __Pyx_NewRef((PyObject *)((PyLongObject *)Py_True));
    }
    __pyx_v_reverse = values[0];
    __pyx_v_start = values[1];
    __pyx_v_stop = values[2];
    __pyx_v_include_start = values[3];
    __pyx_v_include_stop = values[4];
    __pyx_v_prefix = values[5];
    __pyx_v_include_key = values[6];
    __pyx_v_include_value = values[7];
    __pyx_v_verify_checksums = ((PyLongObject *)values[8]);
    __pyx_v_fill_cache = ((PyLongObject *)values[9]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("iterator", 1, 0, 0, __pyx_nargs); __PYX_ERR(0, 358, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("plyvel._plyvel.DB.iterator", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_verify_checksums), __pyx_mstate_global->__pyx_ptype_7cpython_4bool_bool, 1, "verify_checksums", 0))) __PYX_ERR(0, 361, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_fill_cache), __pyx_mstate_global->__pyx_ptype_7cpython_4bool_bool, 1, "fill_cache", 0))) __PYX_ERR(0, 361, __pyx_L1_error)
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_2DB_18iterator(((struct __pyx_obj_6plyvel_7_plyvel_DB *)__pyx_v_self), __pyx_v_reverse, __pyx_v_start, __pyx_v_stop, __pyx_v_include_start, __pyx_v_include_stop, __pyx_v_prefix, __pyx_v_include_key, __pyx_v_include_value, __pyx_v_verify_checksums, __pyx_v_fill_cache);

  /* "plyvel/_plyvel.pyx":358
 *         return self.iterator()
 * 
 *     def iterator(self, *, reverse=False, start=None, stop=None,             # <<<<<<<<<<<<<<
 *                  include_start=True, include_stop=False, prefix=None,
 *                  include_key=True, include_value=True,
*/

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_2DB_18iterator(struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_self, PyObject *__pyx_v_reverse, PyObject *__pyx_v_start, PyObject *__pyx_v_stop, PyObject *__pyx_v_include_start, PyObject *__pyx_v_include_stop, PyObject *__pyx_v_prefix, PyObject *__pyx_v_include_key, PyObject *__pyx_v_include_value, PyLongObject *__pyx_v_verify_checksums, PyLongObject *__pyx_v_fill_cache) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  size_t __pyx_t_3;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("iterator", 0);

  /* "plyvel/_plyvel.pyx":362
 *                  include_key=True, include_value=True,
 *                  bool verify_checksums=False, bool fill_cache=True):
 *         return Iterator(             # <<<<<<<<<<<<<<
 *             self,  # db
 *             None,  # db_prefix
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = NULL;

  /* "plyvel/_plyvel.pyx":375
 *             verify_checksums,
 *             fill_cache,
 *             None,  # snapshot             # <<<<<<<<<<<<<<
 *         )
 * 
*/
  __pyx_t_3 = 1;
  {
    PyObject *__pyx_callargs[14] = {__pyx_t_2, ((PyObject *)__pyx_v_self), Py_None, __pyx_v_reverse, __pyx_v_start, __pyx_v_stop, __pyx_v_include_start, __pyx_v_include_stop, __pyx_v_prefix, __pyx_v_include_key, __pyx_v_include_value, ((PyObject *)__pyx_v_verify_checksums), ((PyObject *)__pyx_v_fill_cache), Py_None};
    __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_mstate_global->__pyx_ptype_6plyvel_7_plyvel_Iterator, __pyx_callargs+__pyx_t_3, (14-__pyx_t_3) | (__pyx_t_3*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 362, __pyx_L1_error)
    __Pyx_GOTREF((PyObject *)__pyx_t_1);
  }
  __pyx_r = ((PyObject *)__pyx_t_1);
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "plyvel/_plyvel.pyx":358
 *         return self.iterator()
 * 
 *     def iterator(self, *, reverse=False, start=None, stop=None,             # <<<<<<<<<<<<<<
 *                  include_start=True, include_stop=False, prefix=None,
 *                  include_key=True, include_value=True,
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("plyvel._plyvel.DB.iterator", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":378
 *         )
 * 
 *     def raw_iterator(self, *, bool verify_checksums=False, bool fill_cache=True):             # <<<<<<<<<<<<<<
 *         return RawIterator(
 *             self,  # db
*/

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_2DB_21raw_iterator(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6plyvel_7_plyvel_2DB_20raw_iterator, "DB.raw_iterator(self, *, bool verify_checksums=False, bool fill_cache=True)");
static PyMethodDef __pyx_mdef_6plyvel_7_plyvel_2DB_21raw_iterator = {"raw_iterator", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_2DB_21raw_iterator, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_2DB_20raw_iterator};
static PyObject *__pyx_pw_6plyvel_7_plyvel_2DB_21raw_iterator(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyLongObject *__pyx_v_verify_checksums = 0;
  PyLongObject *__pyx_v_fill_cache = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("raw_iterator (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_verify_checksums,&__pyx_mstate_global->__pyx_n_u_fill_cache,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(0, 378, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, 0, __pyx_kwds_len, "raw_iterator", 0) < (0)) __PYX_ERR(0, 378, __pyx_L3_error)
      if (!values[0]) values[0] = __Pyx_NewRef((PyObject *)((PyLongObject *)Py_False));
      if (!values[1]) values[1] = __Pyx_NewRef((PyObject *)((PyLongObject *)Py_True));
    } else if (unlikely(__pyx_nargs != 0)) {
      goto __pyx_L5_argtuple_error;
    } else {
      if (!values[0]) values[0] = __Pyx_NewRef((PyObject *)((PyLongObject *)Py_False));
      if (!values[1]) values[1] = __Pyx_NewRef((PyObject *)((PyLongObject *)Py_True));
    }
    __pyx_v_verify_checksums = ((PyLongObject *)values[0]);
    __pyx_v_fill_cache = ((PyLongObject *)values[1]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("raw_iterator", 1, 0, 0, __pyx_nargs); __PYX_ERR(0, 378, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("plyvel._plyvel.DB.raw_iterator", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_verify_checksums), __pyx_mstate_global->__pyx_ptype_7cpython_4bool_bool, 1, "verify_checksums", 0))) __PYX_ERR(0, 378, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_fill_cache), __pyx_mstate_global->__pyx_ptype_7cpython_4bool_bool, 1, "fill_cache", 0))) __PYX_ERR(0, 378, __pyx_L1_error)
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_2DB_20raw_iterator(((struct __pyx_obj_6plyvel_7_plyvel_DB *)__pyx_v_self), __pyx_v_verify_checksums, __pyx_v_fill_cache);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_2DB_20raw_iterator(struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_self, PyLongObject *__pyx_v_verify_checksums, PyLongObject *__pyx_v_fill_cache) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  size_t __pyx_t_3;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("raw_iterator", 0);

  /* "plyvel/_plyvel.pyx":379
 * 
 *     def raw_iterator(self, *, bool verify_checksums=False, bool fill_cache=True):
 *         return RawIterator(             # <<<<<<<<<<<<<<
 *             self,  # db
 *             verify_checksums,
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = NULL;

  /* "plyvel/_plyvel.pyx":383
 *             verify_checksums,
 *             fill_cache,
 *             None,  # snapshot             # <<<<<<<<<<<<<<
 *         )
 * 
*/
  __pyx_t_3 = 1;
  {
    PyObject *__pyx_callargs[5] = {__pyx_t_2, ((PyObject *)__pyx_v_self), ((PyObject *)__pyx_v_verify_checksums), ((PyObject *)__pyx_v_fill_cache), Py_None};
    __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_mstate_global->__pyx_ptype_6plyvel_7_plyvel_RawIterator, __pyx_callargs+__pyx_t_3, (5-__pyx_t_3) | (__pyx_t_3*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 379, __pyx_L1_error)
    __Pyx_GOTREF((PyObject *)__pyx_t_1);
  }
  __pyx_r = ((PyObject *)__pyx_t_1);
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "plyvel/_plyvel.pyx":378
 *         )
 * 
 *     def raw_iterator(self, *, bool verify_checksums=False, bool fill_cache=True):             # <<<<<<<<<<<<<<
 *         return RawIterator(
 *             self,  # db
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("plyvel._plyvel.DB.raw_iterator", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":386
 *         )
 * 
 *     def snapshot(self):             # <<<<<<<<<<<<<<
 *         return Snapshot(db=self)
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_2DB_23snapshot(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6plyvel_7_plyvel_2DB_22snapshot, "DB.snapshot(self)");
static PyMethodDef __pyx_mdef_6plyvel_7_plyvel_2DB_23snapshot = {"snapshot", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_2DB_23snapshot, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_2DB_22snapshot};
static PyObject *__pyx_pw_6plyvel_7_plyvel_2DB_23snapshot(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("snapshot (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("snapshot", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("snapshot", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_2DB_22snapshot(((struct __pyx_obj_6plyvel_7_plyvel_DB *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_2DB_22snapshot(struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  size_t __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("snapshot", 0);

  /* "plyvel/_plyvel.pyx":387
 * 
 *     def snapshot(self):
 *         return Snapshot(db=self)             # <<<<<<<<<<<<<<
 * 
 *     def get_property(self, bytes name not None):
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = NULL;
  __pyx_t_3 = 1;
  {
    PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 1 : 0)] = {__pyx_t_2, NULL};
    __pyx_t_4 = __Pyx_MakeVectorcallBuilderKwds(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 387, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_db, ((PyObject *)__pyx_v_self), __pyx_t_4, __pyx_callargs+1, 0) < (0)) __PYX_ERR(0, 387, __pyx_L1_error)
    __pyx_t_1 = __Pyx_Object_Vectorcall_CallFromBuilder((PyObject*)__pyx_mstate_global->__pyx_ptype_6plyvel_7_plyvel_Snapshot, __pyx_callargs+__pyx_t_3, (1-__pyx_t_3) | (__pyx_t_3*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 387, __pyx_L1_error)
    __Pyx_GOTREF((PyObject *)__pyx_t_1);
  }
  __pyx_r = ((PyObject *)__pyx_t_1);
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "plyvel/_plyvel.pyx":386
 *         )
 * 
 *     def snapshot(self):             # <<<<<<<<<<<<<<
 *         return Snapshot(db=self)
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("plyvel._plyvel.DB.snapshot", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":389
 *         return Snapshot(db=self)
 * 
 *     def get_property(self, bytes name not None):             # <<<<<<<<<<<<<<
 *         if self._db is NULL:
 *             raise RuntimeError("Database is closed")
*/

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_2DB_25get_property(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6plyvel_7_plyvel_2DB_24get_property, "DB.get_property(self, bytes name)");
static PyMethodDef __pyx_mdef_6plyvel_7_plyvel_2DB_25get_property = {"get_property", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_2DB_25get_property, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_2DB_24get_property};
static PyObject *__pyx_pw_6plyvel_7_plyvel_2DB_25get_property(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_name = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_property (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_name,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(0, 389, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 389, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "get_property", 0) < (0)) __PYX_ERR(0, 389, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("get_property", 1, 1, 1, i); __PYX_ERR(0, 389, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 389, __pyx_L3_error)
    }
    __pyx_v_name = ((PyObject*)values[0]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("get_property", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 389, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("plyvel._plyvel.DB.get_property", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_name), (&PyBytes_Type), 0, "name", 1))) __PYX_ERR(0, 389, __pyx_L1_error)
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_2DB_24get_property(((struct __pyx_obj_6plyvel_7_plyvel_DB *)__pyx_v_self), __pyx_v_name);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_2DB_24get_property(struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_self, PyObject *__pyx_v_name) {
  leveldb::Slice __pyx_v_sl;
  std::string __pyx_v_value;
  bool __pyx_v_result;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  const char *__pyx_t_5;
  Py_ssize_t __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_property", 0);

  /* "plyvel/_plyvel.pyx":390
 * 
 *     def get_property(self, bytes name not None):
 *         if self._db is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database is closed")
 * 
*/
  __pyx_t_1 = (__pyx_v_self->_db == NULL);
  if (unlikely(__pyx_t_1)) {

    /* "plyvel/_plyvel.pyx":391
 *     def get_property(self, bytes name not None):
 *         if self._db is NULL:
 *             raise RuntimeError("Database is closed")             # <<<<<<<<<<<<<<
 * 
 *         cdef Slice sl = Slice(name, len(name))
*/
    __pyx_t_3 = NULL;
    __pyx_t_4 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_mstate_global->__pyx_kp_u_Database_is_closed};
      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_RuntimeError)), __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 391, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 391, __pyx_L1_error)

    /* "plyvel/_plyvel.pyx":390
 * 
 *     def get_property(self, bytes name not None):
 *         if self._db is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database is closed")
 * 
*/
  }

  /* "plyvel/_plyvel.pyx":393
 *             raise RuntimeError("Database is closed")
 * 
 *         cdef Slice sl = Slice(name, len(name))             # <<<<<<<<<<<<<<
 *         cdef string value
 *         cdef c_bool result
*/
  __pyx_t_5 = __Pyx_PyBytes_AsString(__pyx_v_name); if (unlikely((!__pyx_t_5) && PyErr_Occurred())) __PYX_ERR(0, 393, __pyx_L1_error)
  __pyx_t_6 = __Pyx_PyBytes_GET_SIZE(__pyx_v_name); if (unlikely(__pyx_t_6 == ((Py_ssize_t)-1))) __PYX_ERR(0, 393, __pyx_L1_error)
  __pyx_v_sl = leveldb::Slice(__pyx_t_5, __pyx_t_6);

  /* "plyvel/_plyvel.pyx":397
 *         cdef c_bool result
 * 
 *         with nogil:             # <<<<<<<<<<<<<<
 *             result = self._db.GetProperty(sl, &value)
 * 
*/
  {
      PyThreadState * _save;
      _save = PyEval_SaveThread();
      __Pyx_FastGIL_Remember();
      /*try:*/ {

        /* "plyvel/_plyvel.pyx":398
 * 
 *         with nogil:
 *             result = self._db.GetProperty(sl, &value)             # <<<<<<<<<<<<<<
 * 
 *         return value if result else None
*/
        __pyx_v_result = __pyx_v_self->_db->GetProperty(__pyx_v_sl, (&__pyx_v_value));
      }

      /* "plyvel/_plyvel.pyx":397
 *         cdef c_bool result
 * 
 *         with nogil:             # <<<<<<<<<<<<<<
 *             result = self._db.GetProperty(sl, &value)
 * 
*/
      /*finally:*/ {
        /*normal exit:*/{
          __Pyx_FastGIL_Forget();
          PyEval_RestoreThread(_save);
          goto __pyx_L6;
        }
        __pyx_L6:;
      }
  }

  /* "plyvel/_plyvel.pyx":400
 *             result = self._db.GetProperty(sl, &value)
 * 
 *         return value if result else None             # <<<<<<<<<<<<<<
 * 
 *     def compact_range(self, *, bytes start=None, bytes stop=None):
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = (__pyx_v_result != 0);
  if (__pyx_t_1) {
    __pyx_t_3 = __pyx_convert_PyBytes_string_to_py_6libcpp_6string_std__in_string(__pyx_v_value); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 400, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_2 = __pyx_t_3;
    __pyx_t_3 = 0;
  } else {
    __Pyx_INCREF(Py_None);
    __pyx_t_2 = Py_None;
  }
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "plyvel/_plyvel.pyx":389
 *         return Snapshot(db=self)
 * 
 *     def get_property(self, bytes name not None):             # <<<<<<<<<<<<<<
 *         if self._db is NULL:
 *             raise RuntimeError("Database is closed")
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("plyvel._plyvel.DB.get_property", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":402
 *         return value if result else None
 * 
 *     def compact_range(self, *, bytes start=None, bytes stop=None):             # <<<<<<<<<<<<<<
 *         if self._db is NULL:
 *             raise RuntimeError("Database is closed")
*/

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_2DB_27compact_range(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6plyvel_7_plyvel_2DB_26compact_range, "DB.compact_range(self, *, bytes start=None, bytes stop=None)");
static PyMethodDef __pyx_mdef_6plyvel_7_plyvel_2DB_27compact_range = {"compact_range", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_2DB_27compact_range, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_2DB_26compact_range};
static PyObject *__pyx_pw_6plyvel_7_plyvel_2DB_27compact_range(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_start = 0;
  PyObject *__pyx_v_stop = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("compact_range (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_start,&__pyx_mstate_global->__pyx_n_u_stop,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(0, 402, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, 0, __pyx_kwds_len, "compact_range", 0) < (0)) __PYX_ERR(0, 402, __pyx_L3_error)
      if (!values[0]) values[0] = __Pyx_NewRef(((PyObject*)Py_None));
      if (!values[1]) values[1] = __Pyx_NewRef(((PyObject*)Py_None));
    } else if (unlikely(__pyx_nargs != 0)) {
      goto __pyx_L5_argtuple_error;
    } else {
      if (!values[0]) values[0] = __Pyx_NewRef(((PyObject*)Py_None));
      if (!values[1]) values[1] = __Pyx_NewRef(((PyObject*)Py_None));
    }
    __pyx_v_start = ((PyObject*)values[0]);
    __pyx_v_stop = ((PyObject*)values[1]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("compact_range", 1, 0, 0, __pyx_nargs); __PYX_ERR(0, 402, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("plyvel._plyvel.DB.compact_range", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_start), (&PyBytes_Type), 1, "start", 1))) __PYX_ERR(0, 402, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_stop), (&PyBytes_Type), 1, "stop", 1))) __PYX_ERR(0, 402, __pyx_L1_error)
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_2DB_26compact_range(((struct __pyx_obj_6plyvel_7_plyvel_DB *)__pyx_v_self), __pyx_v_start, __pyx_v_stop);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_2DB_26compact_range(struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_self, PyObject *__pyx_v_start, PyObject *__pyx_v_stop) {
  leveldb::Slice __pyx_v_start_slice;
  leveldb::Slice __pyx_v_stop_slice;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  const char *__pyx_t_5;
  Py_ssize_t __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("compact_range", 0);

  /* "plyvel/_plyvel.pyx":403
 * 
 *     def compact_range(self, *, bytes start=None, bytes stop=None):
 *         if self._db is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database is closed")
 * 
*/
  __pyx_t_1 = (__pyx_v_self->_db == NULL);
  if (unlikely(__pyx_t_1)) {

    /* "plyvel/_plyvel.pyx":404
 *     def compact_range(self, *, bytes start=None, bytes stop=None):
 *         if self._db is NULL:
 *             raise RuntimeError("Database is closed")             # <<<<<<<<<<<<<<
 * 
 *         cdef Slice start_slice
*/
    __pyx_t_3 = NULL;
    __pyx_t_4 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_mstate_global->__pyx_kp_u_Database_is_closed};
      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_RuntimeError)), __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 404, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 404, __pyx_L1_error)

    /* "plyvel/_plyvel.pyx":403
 * 
 *     def compact_range(self, *, bytes start=None, bytes stop=None):
 *         if self._db is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database is closed")
 * 
*/
  }

  /* "plyvel/_plyvel.pyx":409
 *         cdef Slice stop_slice
 * 
 *         if start is not None:             # <<<<<<<<<<<<<<
 *             start_slice = Slice(start, len(start))
 * 
*/
  __pyx_t_1 = (__pyx_v_start != ((PyObject*)Py_None));
  if (__pyx_t_1) {

    /* "plyvel/_plyvel.pyx":410
 * 
 *         if start is not None:
 *             start_slice = Slice(start, len(start))             # <<<<<<<<<<<<<<
 * 
 *         if stop is not None:
*/
    if (unlikely(__pyx_v_start == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "expected bytes, NoneType found");
      __PYX_ERR(0, 410, __pyx_L1_error)
    }
    __pyx_t_5 = __Pyx_PyBytes_AsString(__pyx_v_start); if (unlikely((!__pyx_t_5) && PyErr_Occurred())) __PYX_ERR(0, 410, __pyx_L1_error)
    if (unlikely(__pyx_v_start == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
      __PYX_ERR(0, 410, __pyx_L1_error)
    }
    __pyx_t_6 = __Pyx_PyBytes_GET_SIZE(__pyx_v_start); if (unlikely(__pyx_t_6 == ((Py_ssize_t)-1))) __PYX_ERR(0, 410, __pyx_L1_error)
    __pyx_v_start_slice = leveldb::Slice(__pyx_t_5, __pyx_t_6);

    /* "plyvel/_plyvel.pyx":409
 *         cdef Slice stop_slice
 * 
 *         if start is not None:             # <<<<<<<<<<<<<<
 *             start_slice = Slice(start, len(start))
 * 
*/
  }

  /* "plyvel/_plyvel.pyx":412
 *             start_slice = Slice(start, len(start))
 * 
 *         if stop is not None:             # <<<<<<<<<<<<<<
 *             stop_slice = Slice(stop, len(stop))
 * 
*/
  __pyx_t_1 = (__pyx_v_stop != ((PyObject*)Py_None));
  if (__pyx_t_1) {

    /* "plyvel/_plyvel.pyx":413
 * 
 *         if stop is not None:
 *             stop_slice = Slice(stop, len(stop))             # <<<<<<<<<<<<<<
 * 
 *         with nogil:
*/
    if (unlikely(__pyx_v_stop == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "expected bytes, NoneType found");
      __PYX_ERR(0, 413, __pyx_L1_error)
    }
    __pyx_t_5 = __Pyx_PyBytes_AsString(__pyx_v_stop); if (unlikely((!__pyx_t_5) && PyErr_Occurred())) __PYX_ERR(0, 413, __pyx_L1_error)
    if (unlikely(__pyx_v_stop == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
      __PYX_ERR(0, 413, __pyx_L1_error)
    }
    __pyx_t_6 = __Pyx_PyBytes_GET_SIZE(__pyx_v_stop); if (unlikely(__pyx_t_6 == ((Py_ssize_t)-1))) __PYX_ERR(0, 413, __pyx_L1_error)
    __pyx_v_stop_slice = leveldb::Slice(__pyx_t_5, __pyx_t_6);

    /* "plyvel/_plyvel.pyx":412
 *             start_slice = Slice(start, len(start))
 * 
 *         if stop is not None:             # <<<<<<<<<<<<<<
 *             stop_slice = Slice(stop, len(stop))
 * 
*/
  }

  /* "plyvel/_plyvel.pyx":415
 *             stop_slice = Slice(stop, len(stop))
 * 
 *         with nogil:             # <<<<<<<<<<<<<<
 *             self._db.CompactRange(&start_slice, &stop_slice)
 * 
*/
  {
      PyThreadState * _save;
      _save = PyEval_SaveThread();
      __Pyx_FastGIL_Remember();
      /*try:*/ {

        /* "plyvel/_plyvel.pyx":416
 * 
 *         with nogil:
 *             self._db.CompactRange(&start_slice, &stop_slice)             # <<<<<<<<<<<<<<
 * 
 *     def approximate_size(self, bytes start not None, bytes stop not None):
*/
        __pyx_v_self->_db->CompactRange((&__pyx_v_start_slice), (&__pyx_v_stop_slice));
      }

      /* "plyvel/_plyvel.pyx":415
 *             stop_slice = Slice(stop, len(stop))
 * 
 *         with nogil:             # <<<<<<<<<<<<<<
 *             self._db.CompactRange(&start_slice, &stop_slice)
 * 
*/
      /*finally:*/ {
        /*normal exit:*/{
          __Pyx_FastGIL_Forget();
          PyEval_RestoreThread(_save);
          goto __pyx_L8;
        }
        __pyx_L8:;
      }
  }

  /* "plyvel/_plyvel.pyx":402
 *         return value if result else None
 * 
 *     def compact_range(self, *, bytes start=None, bytes stop=None):             # <<<<<<<<<<<<<<
 *         if self._db is NULL:
 *             raise RuntimeError("Database is closed")
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("plyvel._plyvel.DB.compact_range", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":418
 *             self._db.CompactRange(&start_slice, &stop_slice)
 * 
 *     def approximate_size(self, bytes start not None, bytes stop not None):             # <<<<<<<<<<<<<<
 *         if self._db is NULL:
 *             raise RuntimeError("Database is closed")
*/

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_2DB_29approximate_size(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6plyvel_7_plyvel_2DB_28approximate_size, "DB.approximate_size(self, bytes start, bytes stop)");
static PyMethodDef __pyx_mdef_6plyvel_7_plyvel_2DB_29approximate_size = {"approximate_size", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_2DB_29approximate_size, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_2DB_28approximate_size};
static PyObject *__pyx_pw_6plyvel_7_plyvel_2DB_29approximate_size(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_start = 0;
  PyObject *__pyx_v_stop = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("approximate_size (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_start,&__pyx_mstate_global->__pyx_n_u_stop,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(0, 418, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 418, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 418, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "approximate_size", 0) < (0)) __PYX_ERR(0, 418, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 2; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("approximate_size", 1, 2, 2, i); __PYX_ERR(0, 418, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 2)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 418, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 418, __pyx_L3_error)
    }
    __pyx_v_start = ((PyObject*)values[0]);
    __pyx_v_stop = ((PyObject*)values[1]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("approximate_size", 1, 2, 2, __pyx_nargs); __PYX_ERR(0, 418, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("plyvel._plyvel.DB.approximate_size", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_start), (&PyBytes_Type), 0, "start", 1))) __PYX_ERR(0, 418, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_stop), (&PyBytes_Type), 0, "stop", 1))) __PYX_ERR(0, 418, __pyx_L1_error)
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_2DB_28approximate_size(((struct __pyx_obj_6plyvel_7_plyvel_DB *)__pyx_v_self), __pyx_v_start, __pyx_v_stop);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_2DB_28approximate_size(struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_self, PyObject *__pyx_v_start, PyObject *__pyx_v_stop) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("approximate_size", 0);

  /* "plyvel/_plyvel.pyx":419
 * 
 *     def approximate_size(self, bytes start not None, bytes stop not None):
 *         if self._db is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database is closed")
 * 
*/
  __pyx_t_1 = (__pyx_v_self->_db == NULL);
  if (unlikely(__pyx_t_1)) {

    /* "plyvel/_plyvel.pyx":420
 *     def approximate_size(self, bytes start not None, bytes stop not None):
 *         if self._db is NULL:
 *             raise RuntimeError("Database is closed")             # <<<<<<<<<<<<<<
 * 
 *         return self.approximate_sizes((start, stop))[0]
*/
    __pyx_t_3 = NULL;
    __pyx_t_4 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_mstate_global->__pyx_kp_u_Database_is_closed};
      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_RuntimeError)), __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 420, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 420, __pyx_L1_error)

    /* "plyvel/_plyvel.pyx":419
 * 
 *     def approximate_size(self, bytes start not None, bytes stop not None):
 *         if self._db is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database is closed")
 * 
*/
  }

  /* "plyvel/_plyvel.pyx":422
 *             raise RuntimeError("Database is closed")
 * 
 *         return self.approximate_sizes((start, stop))[0]             # <<<<<<<<<<<<<<
 * 
 *     def approximate_sizes(self, *ranges):
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_3 = ((PyObject *)__pyx_v_self);
  __Pyx_INCREF(__pyx_t_3);
  __pyx_t_5 = PyTuple_New(2); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 422, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_INCREF(__pyx_v_start);
  __Pyx_GIVEREF(__pyx_v_start);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_v_start) != (0)) __PYX_ERR(0, 422, __pyx_L1_error);
  __Pyx_INCREF(__pyx_v_stop);
  __Pyx_GIVEREF(__pyx_v_stop);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_v_stop) != (0)) __PYX_ERR(0, 422, __pyx_L1_error);
  __pyx_t_4 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_t_5};
    __pyx_t_2 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_approximate_sizes, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 422, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
  }
  __pyx_t_5 = __Pyx_GetItemInt(__pyx_t_2, 0, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_OwnStrongReference); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 422, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_5;
  __pyx_t_5 = 0;
  goto __pyx_L0;

  /* "plyvel/_plyvel.pyx":418
 *             self._db.CompactRange(&start_slice, &stop_slice)
 * 
 *     def approximate_size(self, bytes start not None, bytes stop not None):             # <<<<<<<<<<<<<<
 *         if self._db is NULL:
 *             raise RuntimeError("Database is closed")
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("plyvel._plyvel.DB.approximate_size", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":424
 *         return self.approximate_sizes((start, stop))[0]
 * 
 *     def approximate_sizes(self, *ranges):             # <<<<<<<<<<<<<<
 *         if self._db is NULL:
 *             raise RuntimeError("Database is closed")
*/

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_2DB_31approximate_sizes(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
PyDoc_STRVAR(__pyx_doc_6plyvel_7_plyvel_2DB_30approximate_sizes, "DB.approximate_sizes(self, *ranges)");
static PyMethodDef __pyx_mdef_6plyvel_7_plyvel_2DB_31approximate_sizes = {"approximate_sizes", (PyCFunction)(void(*)(void))(PyCFunctionWithKeywords)__pyx_pw_6plyvel_7_plyvel_2DB_31approximate_sizes, METH_VARARGS|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_2DB_30approximate_sizes};
static PyObject *__pyx_pw_6plyvel_7_plyvel_2DB_31approximate_sizes(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_ranges = 0;
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("approximate_sizes (wrapper)", 0);
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_VARARGS(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("approximate_sizes", __pyx_kwds); return NULL;}
  __Pyx_INCREF(__pyx_args);
  __pyx_v_ranges = __pyx_args;
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_2DB_30approximate_sizes(((struct __pyx_obj_6plyvel_7_plyvel_DB *)__pyx_v_self), __pyx_v_ranges);

  /* function exit code */
  __Pyx_DECREF(__pyx_v_ranges);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_2DB_30approximate_sizes(struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_self, PyObject *__pyx_v_ranges) {
  int __pyx_v_n_ranges;
  leveldb::Range *__pyx_v_c_ranges;
  uint64_t *__pyx_v_sizes;
  int __pyx_v_i;
  PyObject *__pyx_v_start = NULL;
  PyObject *__pyx_v_stop = NULL;
  int __pyx_7genexpr__pyx_v_i;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  Py_ssize_t __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;
  int __pyx_t_8;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  PyObject *(*__pyx_t_11)(PyObject *);
  int __pyx_t_12;
  int __pyx_t_13;
  const char *__pyx_t_14;
  const char *__pyx_t_15;
  Py_ssize_t __pyx_t_16;
  char const *__pyx_t_17;
  PyObject *__pyx_t_18 = NULL;
  PyObject *__pyx_t_19 = NULL;
  PyObject *__pyx_t_20 = NULL;
  PyObject *__pyx_t_21 = NULL;
  PyObject *__pyx_t_22 = NULL;
  PyObject *__pyx_t_23 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("approximate_sizes", 0);

  /* "plyvel/_plyvel.pyx":425
 * 
 *     def approximate_sizes(self, *ranges):
 *         if self._db is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database is closed")
 * 
*/
  __pyx_t_1 = (__pyx_v_self->_db == NULL);
  if (unlikely(__pyx_t_1)) {

    /* "plyvel/_plyvel.pyx":426
 *     def approximate_sizes(self, *ranges):
 *         if self._db is NULL:
 *             raise RuntimeError("Database is closed")             # <<<<<<<<<<<<<<
 * 
 *         cdef int n_ranges = len(ranges)
*/
    __pyx_t_3 = NULL;
    __pyx_t_4 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_mstate_global->__pyx_kp_u_Database_is_closed};
      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_RuntimeError)), __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 426, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 426, __pyx_L1_error)

    /* "plyvel/_plyvel.pyx":425
 * 
 *     def approximate_sizes(self, *ranges):
 *         if self._db is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database is closed")
 * 
*/
  }

  /* "plyvel/_plyvel.pyx":428
 *             raise RuntimeError("Database is closed")
 * 
 *         cdef int n_ranges = len(ranges)             # <<<<<<<<<<<<<<
 *         cdef Range *c_ranges = <Range *>malloc(n_ranges * sizeof(Range))
 *         cdef uint64_t *sizes = <uint64_t *>malloc(n_ranges * sizeof(uint64_t))
*/
  __pyx_t_5 = __Pyx_PyTuple_GET_SIZE(__pyx_v_ranges); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1))) __PYX_ERR(0, 428, __pyx_L1_error)
  __pyx_v_n_ranges = __pyx_t_5;

  /* "plyvel/_plyvel.pyx":429
 * 
 *         cdef int n_ranges = len(ranges)
 *         cdef Range *c_ranges = <Range *>malloc(n_ranges * sizeof(Range))             # <<<<<<<<<<<<<<
 *         cdef uint64_t *sizes = <uint64_t *>malloc(n_ranges * sizeof(uint64_t))
 *         try:
*/
  __pyx_v_c_ranges = ((leveldb::Range *)malloc((__pyx_v_n_ranges * (sizeof(leveldb::Range)))));

  /* "plyvel/_plyvel.pyx":430
 *         cdef int n_ranges = len(ranges)
 *         cdef Range *c_ranges = <Range *>malloc(n_ranges * sizeof(Range))
 *         cdef uint64_t *sizes = <uint64_t *>malloc(n_ranges * sizeof(uint64_t))             # <<<<<<<<<<<<<<
 *         try:
 *             for i in xrange(n_ranges):
*/
  __pyx_v_sizes = ((uint64_t *)malloc((__pyx_v_n_ranges * (sizeof(uint64_t)))));

  /* "plyvel/_plyvel.pyx":431
 *         cdef Range *c_ranges = <Range *>malloc(n_ranges * sizeof(Range))
 *         cdef uint64_t *sizes = <uint64_t *>malloc(n_ranges * sizeof(uint64_t))
 *         try:             # <<<<<<<<<<<<<<
 *             for i in xrange(n_ranges):
 *                 start, stop = ranges[i]
*/
  /*try:*/ {

    /* "plyvel/_plyvel.pyx":432
 *         cdef uint64_t *sizes = <uint64_t *>malloc(n_ranges * sizeof(uint64_t))
 *         try:
 *             for i in xrange(n_ranges):             # <<<<<<<<<<<<<<
 *                 start, stop = ranges[i]
 *                 if not isinstance(start, bytes) or not isinstance(stop, bytes):
*/
    __pyx_t_6 = __pyx_v_n_ranges;
    __pyx_t_7 = __pyx_t_6;
    for (__pyx_t_8 = 0; __pyx_t_8 < __pyx_t_7; __pyx_t_8+=1) {
      __pyx_v_i = __pyx_t_8;

      /* "plyvel/_plyvel.pyx":433
 *         try:
 *             for i in xrange(n_ranges):
 *                 start, stop = ranges[i]             # <<<<<<<<<<<<<<
 *                 if not isinstance(start, bytes) or not isinstance(stop, bytes):
 *                     raise TypeError(
*/
      __pyx_t_2 = __Pyx_GetItemInt_Tuple(__pyx_v_ranges, __pyx_v_i, int, 1, __Pyx_PyLong_From_int, 0, 1, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 433, __pyx_L5_error)
      __Pyx_GOTREF(__pyx_t_2);
      if ((likely(PyTuple_CheckExact(__pyx_t_2))) || (PyList_CheckExact(__pyx_t_2))) {
        PyObject* sequence = __pyx_t_2;
        Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
        if (unlikely(size != 2)) {
          if (size > 2) __Pyx_RaiseTooManyValuesError(2);
          else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
          __PYX_ERR(0, 433, __pyx_L5_error)
        }
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        if (likely(PyTuple_CheckExact(sequence))) {
          __pyx_t_3 = PyTuple_GET_ITEM(sequence, 0);
          __Pyx_INCREF(__pyx_t_3);
          __pyx_t_9 = PyTuple_GET_ITEM(sequence, 1);
          __Pyx_INCREF(__pyx_t_9);
        } else {
          __pyx_t_3 = __Pyx_PyList_GetItemRefFast(sequence, 0, __Pyx_ReferenceSharing_SharedReference);
          if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 433, __pyx_L5_error)
          __Pyx_XGOTREF(__pyx_t_3);
          __pyx_t_9 = __Pyx_PyList_GetItemRefFast(sequence, 1, __Pyx_ReferenceSharing_SharedReference);
          if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 433, __pyx_L5_error)
          __Pyx_XGOTREF(__pyx_t_9);
        }
        #else
        __pyx_t_3 = __Pyx_PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 433, __pyx_L5_error)
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_9 = __Pyx_PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 433, __pyx_L5_error)
        __Pyx_GOTREF(__pyx_t_9);
        #endif
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      } else {
        Py_ssize_t index = -1;
        __pyx_t_10 = PyObject_GetIter(__pyx_t_2); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 433, __pyx_L5_error)
        __Pyx_GOTREF(__pyx_t_10);
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_t_11 = (CYTHON_COMPILING_IN_LIMITED_API) ? PyIter_Next : __Pyx_PyObject_GetIterNextFunc(__pyx_t_10);
        index = 0; __pyx_t_3 = __pyx_t_11(__pyx_t_10); if (unlikely(!__pyx_t_3)) goto __pyx_L9_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_3);
        index = 1; __pyx_t_9 = __pyx_t_11(__pyx_t_10); if (unlikely(!__pyx_t_9)) goto __pyx_L9_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_9);
        if (__Pyx_IternextUnpackEndCheck(__pyx_t_11(__pyx_t_10), 2) < (0)) __PYX_ERR(0, 433, __pyx_L5_error)
        __pyx_t_11 = NULL;
        __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
        goto __pyx_L10_unpacking_done;
        __pyx_L9_unpacking_failed:;
        __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
        __pyx_t_11 = NULL;
        if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
        __PYX_ERR(0, 433, __pyx_L5_error)
        __pyx_L10_unpacking_done:;
      }
      __Pyx_XDECREF_SET(__pyx_v_start, __pyx_t_3);
      __pyx_t_3 = 0;
      __Pyx_XDECREF_SET(__pyx_v_stop, __pyx_t_9);
      __pyx_t_9 = 0;

      /* "plyvel/_plyvel.pyx":434
 *             for i in xrange(n_ranges):
 *                 start, stop = ranges[i]
 *                 if not isinstance(start, bytes) or not isinstance(stop, bytes):             # <<<<<<<<<<<<<<
 *                     raise TypeError(
 *                         "Start and stop of range must be byte strings")
*/
      __pyx_t_12 = PyBytes_Check(__pyx_v_start); 
      __pyx_t_13 = (!__pyx_t_12);
      if (!__pyx_t_13) {
      } else {
        __pyx_t_1 = __pyx_t_13;
        goto __pyx_L12_bool_binop_done;
      }
      __pyx_t_13 = PyBytes_Check(__pyx_v_stop); 
      __pyx_t_12 = (!__pyx_t_13);
      __pyx_t_1 = __pyx_t_12;
      __pyx_L12_bool_binop_done:;
      if (unlikely(__pyx_t_1)) {

        /* "plyvel/_plyvel.pyx":435
 *                 start, stop = ranges[i]
 *                 if not isinstance(start, bytes) or not isinstance(stop, bytes):
 *                     raise TypeError(             # <<<<<<<<<<<<<<
 *                         "Start and stop of range must be byte strings")
 *                 c_ranges[i] = Range(
*/
        __pyx_t_9 = NULL;
        __pyx_t_4 = 1;
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_9, __pyx_mstate_global->__pyx_kp_u_Start_and_stop_of_range_must_be};
          __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_TypeError)), __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
          if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 435, __pyx_L5_error)
          __Pyx_GOTREF(__pyx_t_2);
        }
        __Pyx_Raise(__pyx_t_2, 0, 0, 0);
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __PYX_ERR(0, 435, __pyx_L5_error)

        /* "plyvel/_plyvel.pyx":434
 *             for i in xrange(n_ranges):
 *                 start, stop = ranges[i]
 *                 if not isinstance(start, bytes) or not isinstance(stop, bytes):             # <<<<<<<<<<<<<<
 *                     raise TypeError(
 *                         "Start and stop of range must be byte strings")
*/
      }

      /* "plyvel/_plyvel.pyx":438
 *                         "Start and stop of range must be byte strings")
 *                 c_ranges[i] = Range(
 *                     Slice(start, len(start)),             # <<<<<<<<<<<<<<
 *                     Slice(stop, len(stop)))
 * 
*/
      __pyx_t_14 = __Pyx_PyObject_AsString(__pyx_v_start); if (unlikely((!__pyx_t_14) && PyErr_Occurred())) __PYX_ERR(0, 438, __pyx_L5_error)
      __pyx_t_5 = PyObject_Length(__pyx_v_start); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1))) __PYX_ERR(0, 438, __pyx_L5_error)

      /* "plyvel/_plyvel.pyx":439
 *                 c_ranges[i] = Range(
 *                     Slice(start, len(start)),
 *                     Slice(stop, len(stop)))             # <<<<<<<<<<<<<<
 * 
 *             with nogil:
*/
      __pyx_t_15 = __Pyx_PyObject_AsString(__pyx_v_stop); if (unlikely((!__pyx_t_15) && PyErr_Occurred())) __PYX_ERR(0, 439, __pyx_L5_error)
      __pyx_t_16 = PyObject_Length(__pyx_v_stop); if (unlikely(__pyx_t_16 == ((Py_ssize_t)-1))) __PYX_ERR(0, 439, __pyx_L5_error)

      /* "plyvel/_plyvel.pyx":437
 *                     raise TypeError(
 *                         "Start and stop of range must be byte strings")
 *                 c_ranges[i] = Range(             # <<<<<<<<<<<<<<
 *                     Slice(start, len(start)),
 *                     Slice(stop, len(stop)))
*/
      (__pyx_v_c_ranges[__pyx_v_i]) = leveldb::Range(leveldb::Slice(__pyx_t_14, __pyx_t_5), leveldb::Slice(__pyx_t_15, __pyx_t_16));
    }

    /* "plyvel/_plyvel.pyx":441
 *                     Slice(stop, len(stop)))
 * 
 *             with nogil:             # <<<<<<<<<<<<<<
 *                 self._db.GetApproximateSizes(c_ranges, n_ranges, sizes)
 * 
*/
    {
        PyThreadState * _save;
        _save = PyEval_SaveThread();
        __Pyx_FastGIL_Remember();
        /*try:*/ {

          /* "plyvel/_plyvel.pyx":442
 * 
 *             with nogil:
 *                 self._db.GetApproximateSizes(c_ranges, n_ranges, sizes)             # <<<<<<<<<<<<<<
 * 
 *             return [sizes[i] for i in xrange(n_ranges)]
*/
          __pyx_v_self->_db->GetApproximateSizes(__pyx_v_c_ranges, __pyx_v_n_ranges, __pyx_v_sizes);
        }

        /* "plyvel/_plyvel.pyx":441
 *                     Slice(stop, len(stop)))
 * 
 *             with nogil:             # <<<<<<<<<<<<<<
 *                 self._db.GetApproximateSizes(c_ranges, n_ranges, sizes)
 * 
*/
        /*finally:*/ {
          /*normal exit:*/{
            __Pyx_FastGIL_Forget();
            PyEval_RestoreThread(_save);
            goto __pyx_L16;
          }
          __pyx_L16:;
        }
    }

    /* "plyvel/_plyvel.pyx":444
 *                 self._db.GetApproximateSizes(c_ranges, n_ranges, sizes)
 * 
 *             return [sizes[i] for i in xrange(n_ranges)]             # <<<<<<<<<<<<<<
 *         finally:
 *             free(c_ranges)
*/
    __Pyx_XDECREF(__pyx_r);
    { /* enter inner scope */
      __pyx_t_2 = PyList_New(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 444, __pyx_L5_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_6 = __pyx_v_n_ranges;
      __pyx_t_7 = __pyx_t_6;
      for (__pyx_t_8 = 0; __pyx_t_8 < __pyx_t_7; __pyx_t_8+=1) {
        __pyx_7genexpr__pyx_v_i = __pyx_t_8;
        __pyx_t_9 = __Pyx_PyLong_From_uint64_t((__pyx_v_sizes[__pyx_7genexpr__pyx_v_i])); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 444, __pyx_L5_error)
        __Pyx_GOTREF(__pyx_t_9);
        if (unlikely(__Pyx_ListComp_Append(__pyx_t_2, (PyObject*)__pyx_t_9))) __PYX_ERR(0, 444, __pyx_L5_error)
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      }
    } /* exit inner scope */
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L4_return;
  }

  /* "plyvel/_plyvel.pyx":446
 *             return [sizes[i] for i in xrange(n_ranges)]
 *         finally:
 *             free(c_ranges)             # <<<<<<<<<<<<<<
 *             free(sizes)
 * 
*/
  /*finally:*/ {
    __pyx_L5_error:;
    /*exception exit:*/{
      __Pyx_PyThreadState_declare
      __Pyx_PyThreadState_assign
      __pyx_t_18 = 0; __pyx_t_19 = 0; __pyx_t_20 = 0; __pyx_t_21 = 0; __pyx_t_22 = 0; __pyx_t_23 = 0;
      __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
       __Pyx_ExceptionSwap(&__pyx_t_21, &__pyx_t_22, &__pyx_t_23);
      if ( unlikely(__Pyx_GetException(&__pyx_t_18, &__pyx_t_19, &__pyx_t_20) < 0)) __Pyx_ErrFetch(&__pyx_t_18, &__pyx_t_19, &__pyx_t_20);
      __Pyx_XGOTREF(__pyx_t_18);
      __Pyx_XGOTREF(__pyx_t_19);
      __Pyx_XGOTREF(__pyx_t_20);
      __Pyx_XGOTREF(__pyx_t_21);
      __Pyx_XGOTREF(__pyx_t_22);
      __Pyx_XGOTREF(__pyx_t_23);
      __pyx_t_6 = __pyx_lineno; __pyx_t_7 = __pyx_clineno; __pyx_t_17 = __pyx_filename;
      {
        free(__pyx_v_c_ranges);

        /* "plyvel/_plyvel.pyx":447
 *         finally:
 *             free(c_ranges)
 *             free(sizes)             # <<<<<<<<<<<<<<
 * 
 *     def prefixed_db(self, bytes prefix not None):
*/
        free(__pyx_v_sizes);
      }
      __Pyx_XGIVEREF(__pyx_t_21);
      __Pyx_XGIVEREF(__pyx_t_22);
      __Pyx_XGIVEREF(__pyx_t_23);
      __Pyx_ExceptionReset(__pyx_t_21, __pyx_t_22, __pyx_t_23);
      __Pyx_XGIVEREF(__pyx_t_18);
      __Pyx_XGIVEREF(__pyx_t_19);
      __Pyx_XGIVEREF(__pyx_t_20);
      __Pyx_ErrRestore(__pyx_t_18, __pyx_t_19, __pyx_t_20);
      __pyx_t_18 = 0; __pyx_t_19 = 0; __pyx_t_20 = 0; __pyx_t_21 = 0; __pyx_t_22 = 0; __pyx_t_23 = 0;
      __pyx_lineno = __pyx_t_6; __pyx_clineno = __pyx_t_7; __pyx_filename = __pyx_t_17;
      goto __pyx_L1_error;
    }
    __pyx_L4_return: {
      __pyx_t_23 = __pyx_r;
      __pyx_r = 0;

      /* "plyvel/_plyvel.pyx":446
 *             return [sizes[i] for i in xrange(n_ranges)]
 *         finally:
 *             free(c_ranges)             # <<<<<<<<<<<<<<
 *             free(sizes)
 * 
*/
      free(__pyx_v_c_ranges);

      /* "plyvel/_plyvel.pyx":447
 *         finally:
 *             free(c_ranges)
 *             free(sizes)             # <<<<<<<<<<<<<<
 * 
 *     def prefixed_db(self, bytes prefix not None):
*/
      free(__pyx_v_sizes);
      __pyx_r = __pyx_t_23;
      __pyx_t_23 = 0;
      goto __pyx_L0;
    }
  }

  /* "plyvel/_plyvel.pyx":424
 *         return self.approximate_sizes((start, stop))[0]
 * 
 *     def approximate_sizes(self, *ranges):             # <<<<<<<<<<<<<<
 *         if self._db is NULL:
 *             raise RuntimeError("Database is closed")
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_AddTraceback("plyvel._plyvel.DB.approximate_sizes", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_start);
  __Pyx_XDECREF(__pyx_v_stop);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":449
 *             free(sizes)
 * 
 *     def prefixed_db(self, bytes prefix not None):             # <<<<<<<<<<<<<<
 *         return PrefixedDB(db=self, prefix=prefix)
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_2DB_33prefixed_db(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6plyvel_7_plyvel_2DB_32prefixed_db, "DB.prefixed_db(self, bytes prefix)");
static PyMethodDef __pyx_mdef_6plyvel_7_plyvel_2DB_33prefixed_db = {"prefixed_db", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_2DB_33prefixed_db, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_2DB_32prefixed_db};
static PyObject *__pyx_pw_6plyvel_7_plyvel_2DB_33prefixed_db(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_prefix = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("prefixed_db (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_prefix,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(0, 449, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 449, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "prefixed_db", 0) < (0)) __PYX_ERR(0, 449, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("prefixed_db", 1, 1, 1, i); __PYX_ERR(0, 449, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 449, __pyx_L3_error)
    }
    __pyx_v_prefix = ((PyObject*)values[0]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("prefixed_db", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 449, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("plyvel._plyvel.DB.prefixed_db", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_prefix), (&PyBytes_Type), 0, "prefix", 1))) __PYX_ERR(0, 449, __pyx_L1_error)
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_2DB_32prefixed_db(((struct __pyx_obj_6plyvel_7_plyvel_DB *)__pyx_v_self), __pyx_v_prefix);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_2DB_32prefixed_db(struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_self, PyObject *__pyx_v_prefix) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  size_t __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("prefixed_db", 0);

  /* "plyvel/_plyvel.pyx":450
 * 
 *     def prefixed_db(self, bytes prefix not None):
 *         return PrefixedDB(db=self, prefix=prefix)             # <<<<<<<<<<<<<<
 * 
 *     def __enter__(self):
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = NULL;
  __pyx_t_3 = 1;
  {
    PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 2 : 0)] = {__pyx_t_2, NULL};
    __pyx_t_4 = __Pyx_MakeVectorcallBuilderKwds(2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 450, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_db, ((PyObject *)__pyx_v_self), __pyx_t_4, __pyx_callargs+1, 0) < (0)) __PYX_ERR(0, 450, __pyx_L1_error)
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_prefix, __pyx_v_prefix, __pyx_t_4, __pyx_callargs+1, 1) < (0)) __PYX_ERR(0, 450, __pyx_L1_error)
    __pyx_t_1 = __Pyx_Object_Vectorcall_CallFromBuilder((PyObject*)__pyx_mstate_global->__pyx_ptype_6plyvel_7_plyvel_PrefixedDB, __pyx_callargs+__pyx_t_3, (1-__pyx_t_3) | (__pyx_t_3*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 450, __pyx_L1_error)
    __Pyx_GOTREF((PyObject *)__pyx_t_1);
  }
  __pyx_r = ((PyObject *)__pyx_t_1);
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "plyvel/_plyvel.pyx":449
 *             free(sizes)
 * 
 *     def prefixed_db(self, bytes prefix not None):             # <<<<<<<<<<<<<<
 *         return PrefixedDB(db=self, prefix=prefix)
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("plyvel._plyvel.DB.prefixed_db", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":452
 *         return PrefixedDB(db=self, prefix=prefix)
 * 
 *     def __enter__(self):             # <<<<<<<<<<<<<<
 *         return self
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_2DB_35__enter__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6plyvel_7_plyvel_2DB_34__enter__, "DB.__enter__(self)");
static PyMethodDef __pyx_mdef_6plyvel_7_plyvel_2DB_35__enter__ = {"__enter__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_2DB_35__enter__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_2DB_34__enter__};
static PyObject *__pyx_pw_6plyvel_7_plyvel_2DB_35__enter__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__enter__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("__enter__", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("__enter__", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_2DB_34__enter__(((struct __pyx_obj_6plyvel_7_plyvel_DB *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_2DB_34__enter__(struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__enter__", 0);

  /* "plyvel/_plyvel.pyx":453
 * 
 *     def __enter__(self):
 *         return self             # <<<<<<<<<<<<<<
 * 
 *     def __exit__(self, exc_type, exc_val, exc_tb):
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF((PyObject *)__pyx_v_self);
  __pyx_r = ((PyObject *)__pyx_v_self);
  goto __pyx_L0;

  /* "plyvel/_plyvel.pyx":452
 *         return PrefixedDB(db=self, prefix=prefix)
 * 
 *     def __enter__(self):             # <<<<<<<<<<<<<<
 *         return self
 * 
*/

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":455
 *         return self
 * 
 *     def __exit__(self, exc_type, exc_val, exc_tb):             # <<<<<<<<<<<<<<
 *         self.close()
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_2DB_37__exit__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6plyvel_7_plyvel_2DB_36__exit__, "DB.__exit__(self, exc_type, exc_val, exc_tb)");
static PyMethodDef __pyx_mdef_6plyvel_7_plyvel_2DB_37__exit__ = {"__exit__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_2DB_37__exit__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_2DB_36__exit__};
static PyObject *__pyx_pw_6plyvel_7_plyvel_2DB_37__exit__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  CYTHON_UNUSED PyObject *__pyx_v_exc_type = 0;
  CYTHON_UNUSED PyObject *__pyx_v_exc_val = 0;
  CYTHON_UNUSED PyObject *__pyx_v_exc_tb = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[3] = {0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__exit__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_exc_type,&__pyx_mstate_global->__pyx_n_u_exc_val,&__pyx_mstate_global->__pyx_n_u_exc_tb,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(0, 455, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 455, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 455, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 455, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__exit__", 0) < (0)) __PYX_ERR(0, 455, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 3; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__exit__", 1, 3, 3, i); __PYX_ERR(0, 455, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 3)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 455, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 455, __pyx_L3_error)
      values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 455, __pyx_L3_error)
    }
    __pyx_v_exc_type = values[0];
    __pyx_v_exc_val = values[1];
    __pyx_v_exc_tb = values[2];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__exit__", 1, 3, 3, __pyx_nargs); __PYX_ERR(0, 455, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("plyvel._plyvel.DB.__exit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_2DB_36__exit__(((struct __pyx_obj_6plyvel_7_plyvel_DB *)__pyx_v_self), __pyx_v_exc_type, __pyx_v_exc_val, __pyx_v_exc_tb);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_2DB_36__exit__(struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v_exc_type, CYTHON_UNUSED PyObject *__pyx_v_exc_val, CYTHON_UNUSED PyObject *__pyx_v_exc_tb) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__exit__", 0);

  /* "plyvel/_plyvel.pyx":456
 * 
 *     def __exit__(self, exc_type, exc_val, exc_tb):
 *         self.close()             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __pyx_t_1 = __pyx_f_6plyvel_7_plyvel_2DB_close(__pyx_v_self, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 456, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "plyvel/_plyvel.pyx":455
 *         return self
 * 
 *     def __exit__(self, exc_type, exc_val, exc_tb):             # <<<<<<<<<<<<<<
 *         self.close()
 * 
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("plyvel._plyvel.DB.__exit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":224
 *     cdef leveldb.DB* _db
 *     cdef Options options
 *     cdef readonly object name             # <<<<<<<<<<<<<<
 *     cdef object lock
 *     cdef dict iterators
*/

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_2DB_4name_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6plyvel_7_plyvel_2DB_4name_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_2DB_4name___get__(((struct __pyx_obj_6plyvel_7_plyvel_DB *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_2DB_4name___get__(struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->name);
  __pyx_r = __pyx_v_self->name;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self._db,self.options cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):
*/

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_2DB_39__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6plyvel_7_plyvel_2DB_38__reduce_cython__, "DB.__reduce_cython__(self)");
static PyMethodDef __pyx_mdef_6plyvel_7_plyvel_2DB_39__reduce_cython__ = {"__reduce_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_2DB_39__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_2DB_38__reduce_cython__};
static PyObject *__pyx_pw_6plyvel_7_plyvel_2DB_39__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("__reduce_cython__", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("__reduce_cython__", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_2DB_38__reduce_cython__(((struct __pyx_obj_6plyvel_7_plyvel_DB *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_2DB_38__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError, "self._db,self.options cannot be converted to a Python object for pickling"             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError, "self._db,self.options cannot be converted to a Python object for pickling"
*/
  __Pyx_Raise(((PyObject *)(((PyTypeObject*)PyExc_TypeError))), __pyx_mstate_global->__pyx_kp_u_self__db_self_options_cannot_be, 0, 0);
  __PYX_ERR(1, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self._db,self.options cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("plyvel._plyvel.DB.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError, "self._db,self.options cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self._db,self.options cannot be converted to a Python object for pickling"
*/

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_2DB_41__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6plyvel_7_plyvel_2DB_40__setstate_cython__, "DB.__setstate_cython__(self, __pyx_state)");
static PyMethodDef __pyx_mdef_6plyvel_7_plyvel_2DB_41__setstate_cython__ = {"__setstate_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_2DB_41__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_2DB_40__setstate_cython__};
static PyObject *__pyx_pw_6plyvel_7_plyvel_2DB_41__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  CYTHON_UNUSED PyObject *__pyx_v___pyx_state = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_pyx_state,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(1, 3, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(1, 3, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__setstate_cython__", 0) < (0)) __PYX_ERR(1, 3, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, i); __PYX_ERR(1, 3, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(1, 3, __pyx_L3_error)
    }
    __pyx_v___pyx_state = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, __pyx_nargs); __PYX_ERR(1, 3, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("plyvel._plyvel.DB.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_2DB_40__setstate_cython__(((struct __pyx_obj_6plyvel_7_plyvel_DB *)__pyx_v_self), __pyx_v___pyx_state);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_2DB_40__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError, "self._db,self.options cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError, "self._db,self.options cannot be converted to a Python object for pickling"             # <<<<<<<<<<<<<<
*/
  __Pyx_Raise(((PyObject *)(((PyTypeObject*)PyExc_TypeError))), __pyx_mstate_global->__pyx_kp_u_self__db_self_options_cannot_be, 0, 0);
  __PYX_ERR(1, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError, "self._db,self.options cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self._db,self.options cannot be converted to a Python object for pickling"
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("plyvel._plyvel.DB.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":463
 *     cdef readonly bytes prefix
 * 
 *     def __init__(self, *, DB db not None, bytes prefix not None):             # <<<<<<<<<<<<<<
 *         self.db = db
 *         self.prefix = prefix
*/

/* Python wrapper */
static int __pyx_pw_6plyvel_7_plyvel_10PrefixedDB_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_6plyvel_7_plyvel_10PrefixedDB_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_db = 0;
  PyObject *__pyx_v_prefix = 0;
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return -1;
  #endif
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_db,&__pyx_mstate_global->__pyx_n_u_prefix,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_VARARGS(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(0, 463, __pyx_L3_error)
    if (likely(__pyx_kwds_len > 0)) {
      switch (__pyx_nargs) {
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, 0, __pyx_kwds_len, "__init__", 0) < (0)) __PYX_ERR(0, 463, __pyx_L3_error)
      for (Py_ssize_t i = 0; i < 2; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseKeywordRequired("__init__", *(__pyx_pyargnames[i - 0])); __PYX_ERR(0, 463, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 0)) {
      goto __pyx_L5_argtuple_error;
    } else {
      __Pyx_RaiseKeywordRequired("__init__", __pyx_mstate_global->__pyx_n_u_db); __PYX_ERR(0, 463, __pyx_L3_error)
    }
    __pyx_v_db = ((struct __pyx_obj_6plyvel_7_plyvel_DB *)values[0]);
    __pyx_v_prefix = ((PyObject*)values[1]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 0, 0, __pyx_nargs); __PYX_ERR(0, 463, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("plyvel._plyvel.PrefixedDB.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_db), __pyx_mstate_global->__pyx_ptype_6plyvel_7_plyvel_DB, 0, "db", 0))) __PYX_ERR(0, 463, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_prefix), (&PyBytes_Type), 0, "prefix", 1))) __PYX_ERR(0, 463, __pyx_L1_error)
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_10PrefixedDB___init__(((struct __pyx_obj_6plyvel_7_plyvel_PrefixedDB *)__pyx_v_self), __pyx_v_db, __pyx_v_prefix);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = -1;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6plyvel_7_plyvel_10PrefixedDB___init__(struct __pyx_obj_6plyvel_7_plyvel_PrefixedDB *__pyx_v_self, struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_db, PyObject *__pyx_v_prefix) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "plyvel/_plyvel.pyx":464
 * 
 *     def __init__(self, *, DB db not None, bytes prefix not None):
 *         self.db = db             # <<<<<<<<<<<<<<
 *         self.prefix = prefix
 * 
*/
  __Pyx_INCREF((PyObject *)__pyx_v_db);
  __Pyx_GIVEREF((PyObject *)__pyx_v_db);
  __Pyx_GOTREF((PyObject *)__pyx_v_self->db);
  __Pyx_DECREF((PyObject *)__pyx_v_self->db);
  __pyx_v_self->db = __pyx_v_db;

  /* "plyvel/_plyvel.pyx":465
 *     def __init__(self, *, DB db not None, bytes prefix not None):
 *         self.db = db
 *         self.prefix = prefix             # <<<<<<<<<<<<<<
 * 
 *     def __repr__(self):
*/
  __Pyx_INCREF(__pyx_v_prefix);
  __Pyx_GIVEREF(__pyx_v_prefix);
  __Pyx_GOTREF(__pyx_v_self->prefix);
  __Pyx_DECREF(__pyx_v_self->prefix);
  __pyx_v_self->prefix = __pyx_v_prefix;

  /* "plyvel/_plyvel.pyx":463
 *     cdef readonly bytes prefix
 * 
 *     def __init__(self, *, DB db not None, bytes prefix not None):             # <<<<<<<<<<<<<<
 *         self.db = db
 *         self.prefix = prefix
*/

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":467
 *         self.prefix = prefix
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return '<plyvel.PrefixedDB with prefix %r at 0x%s>' % (
 *             self.prefix,
*/

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_10PrefixedDB_3__repr__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6plyvel_7_plyvel_10PrefixedDB_3__repr__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__repr__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_10PrefixedDB_2__repr__(((struct __pyx_obj_6plyvel_7_plyvel_PrefixedDB *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_10PrefixedDB_2__repr__(struct __pyx_obj_6plyvel_7_plyvel_PrefixedDB *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4[5];
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__repr__", 0);

  /* "plyvel/_plyvel.pyx":468
 * 
 *     def __repr__(self):
 *         return '<plyvel.PrefixedDB with prefix %r at 0x%s>' % (             # <<<<<<<<<<<<<<
 *             self.prefix,
 *             hex(id(self)),
*/
  __Pyx_XDECREF(__pyx_r);

  /* "plyvel/_plyvel.pyx":469
 *     def __repr__(self):
 *         return '<plyvel.PrefixedDB with prefix %r at 0x%s>' % (
 *             self.prefix,             # <<<<<<<<<<<<<<
 *             hex(id(self)),
 *         )
*/
  __pyx_t_1 = __Pyx_PyObject_FormatSimpleAndDecref(PyObject_Repr(__pyx_v_self->prefix), __pyx_mstate_global->__pyx_empty_unicode); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 469, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);

  /* "plyvel/_plyvel.pyx":470
 *         return '<plyvel.PrefixedDB with prefix %r at 0x%s>' % (
 *             self.prefix,
 *             hex(id(self)),             # <<<<<<<<<<<<<<
 *         )
 * 
*/
  __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_builtin_id, ((PyObject *)__pyx_v_self)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 470, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyNumber_Hex(__pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 470, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PyUnicode_Unicode(__pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 470, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_4[0] = __pyx_mstate_global->__pyx_kp_u_plyvel_PrefixedDB_with_prefix;
  __pyx_t_4[1] = __pyx_t_1;
  __pyx_t_4[2] = __pyx_mstate_global->__pyx_kp_u_at_0x;
  __pyx_t_4[3] = __pyx_t_2;
  __pyx_t_4[4] = __pyx_mstate_global->__pyx_kp_u__2;

  /* "plyvel/_plyvel.pyx":468
 * 
 *     def __repr__(self):
 *         return '<plyvel.PrefixedDB with prefix %r at 0x%s>' % (             # <<<<<<<<<<<<<<
 *             self.prefix,
 *             hex(id(self)),
*/
  __pyx_t_3 = __Pyx_PyUnicode_Join(__pyx_t_4, 5, 31 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_1) + 6 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_2) + 1, 127 | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_1) | __Pyx_PyUnicode_MAX_CHAR_VALUE(__pyx_t_2));
  if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 468, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "plyvel/_plyvel.pyx":467
 *         self.prefix = prefix
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return '<plyvel.PrefixedDB with prefix %r at 0x%s>' % (
 *             self.prefix,
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("plyvel._plyvel.PrefixedDB.__repr__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":473
 *         )
 * 
 *     def get(self, bytes key not None, default=None, *,             # <<<<<<<<<<<<<<
 *             bool verify_checksums=False, bool fill_cache=True):
 *         return self.db.get(
*/

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_10PrefixedDB_5get(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6plyvel_7_plyvel_10PrefixedDB_4get, "PrefixedDB.get(self, bytes key, default=None, *, bool verify_checksums=False, bool fill_cache=True)");
static PyMethodDef __pyx_mdef_6plyvel_7_plyvel_10PrefixedDB_5get = {"get", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_10PrefixedDB_5get, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_10PrefixedDB_4get};
static PyObject *__pyx_pw_6plyvel_7_plyvel_10PrefixedDB_5get(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_key = 0;
  PyObject *__pyx_v_default = 0;
  PyLongObject *__pyx_v_verify_checksums = 0;
  PyLongObject *__pyx_v_fill_cache = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[4] = {0,0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_key,&__pyx_mstate_global->__pyx_n_u_default,&__pyx_mstate_global->__pyx_n_u_verify_checksums,&__pyx_mstate_global->__pyx_n_u_fill_cache,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(0, 473, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 473, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 473, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "get", 0) < (0)) __PYX_ERR(0, 473, __pyx_L3_error)
      if (!values[1]) values[1] = __Pyx_NewRef(((PyObject *)Py_None));

      /* "plyvel/_plyvel.pyx":474
 * 
 *     def get(self, bytes key not None, default=None, *,
 *             bool verify_checksums=False, bool fill_cache=True):             # <<<<<<<<<<<<<<
 *         return self.db.get(
 *             self.prefix + key,
*/
      if (!values[2]) values[2] = __Pyx_NewRef((PyObject *)((PyLongObject *)Py_False));
      if (!values[3]) values[3] = __Pyx_NewRef((PyObject *)((PyLongObject *)Py_True));
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("get", 0, 1, 2, i); __PYX_ERR(0, 473, __pyx_L3_error) }
      }
    } else {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 473, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 473, __pyx_L3_error)
        break;
        default: goto __pyx_L5_argtuple_error;
      }

      /* "plyvel/_plyvel.pyx":473
 *         )
 * 
 *     def get(self, bytes key not None, default=None, *,             # <<<<<<<<<<<<<<
 *             bool verify_checksums=False, bool fill_cache=True):
 *         return self.db.get(
*/
      if (!values[1]) values[1] = __Pyx_NewRef(((PyObject *)Py_None));

      /* "plyvel/_plyvel.pyx":474
 * 
 *     def get(self, bytes key not None, default=None, *,
 *             bool verify_checksums=False, bool fill_cache=True):             # <<<<<<<<<<<<<<
 *         return self.db.get(
 *             self.prefix + key,
*/
      if (!values[2]) values[2] = __Pyx_NewRef((PyObject *)((PyLongObject *)Py_False));
      if (!values[3]) values[3] = __Pyx_NewRef((PyObject *)((PyLongObject *)Py_True));
    }
    __pyx_v_key = ((PyObject*)values[0]);
    __pyx_v_default = values[1];
    __pyx_v_verify_checksums = ((PyLongObject *)values[2]);
    __pyx_v_fill_cache = ((PyLongObject *)values[3]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("get", 0, 1, 2, __pyx_nargs); __PYX_ERR(0, 473, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("plyvel._plyvel.PrefixedDB.get", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_key), (&PyBytes_Type), 0, "key", 1))) __PYX_ERR(0, 473, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_verify_checksums), __pyx_mstate_global->__pyx_ptype_7cpython_4bool_bool, 1, "verify_checksums", 0))) __PYX_ERR(0, 474, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_fill_cache), __pyx_mstate_global->__pyx_ptype_7cpython_4bool_bool, 1, "fill_cache", 0))) __PYX_ERR(0, 474, __pyx_L1_error)
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_10PrefixedDB_4get(((struct __pyx_obj_6plyvel_7_plyvel_PrefixedDB *)__pyx_v_self), __pyx_v_key, __pyx_v_default, __pyx_v_verify_checksums, __pyx_v_fill_cache);

  /* "plyvel/_plyvel.pyx":473
 *         )
 * 
 *     def get(self, bytes key not None, default=None, *,             # <<<<<<<<<<<<<<
 *             bool verify_checksums=False, bool fill_cache=True):
 *         return self.db.get(
*/

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_10PrefixedDB_4get(struct __pyx_obj_6plyvel_7_plyvel_PrefixedDB *__pyx_v_self, PyObject *__pyx_v_key, PyObject *__pyx_v_default, PyLongObject *__pyx_v_verify_checksums, PyLongObject *__pyx_v_fill_cache) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get", 0);

  /* "plyvel/_plyvel.pyx":475
 *     def get(self, bytes key not None, default=None, *,
 *             bool verify_checksums=False, bool fill_cache=True):
 *         return self.db.get(             # <<<<<<<<<<<<<<
 *             self.prefix + key,
 *             default=default,
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = ((PyObject *)__pyx_v_self->db);
  __Pyx_INCREF(__pyx_t_2);

  /* "plyvel/_plyvel.pyx":476
 *             bool verify_checksums=False, bool fill_cache=True):
 *         return self.db.get(
 *             self.prefix + key,             # <<<<<<<<<<<<<<
 *             default=default,
 *             verify_checksums=verify_checksums,
*/
  __pyx_t_3 = PyNumber_Add(__pyx_v_self->prefix, __pyx_v_key); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 476, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);

  /* "plyvel/_plyvel.pyx":479
 *             default=default,
 *             verify_checksums=verify_checksums,
 *             fill_cache=fill_cache)             # <<<<<<<<<<<<<<
 * 
 *     def put(self, bytes key not None, value not None, *,
*/
  __pyx_t_4 = 0;
  {
    PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 3 : 0)] = {__pyx_t_2, __pyx_t_3};
    __pyx_t_5 = __Pyx_MakeVectorcallBuilderKwds(3); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 475, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_default, __pyx_v_default, __pyx_t_5, __pyx_callargs+2, 0) < (0)) __PYX_ERR(0, 475, __pyx_L1_error)
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_verify_checksums, ((PyObject *)__pyx_v_verify_checksums), __pyx_t_5, __pyx_callargs+2, 1) < (0)) __PYX_ERR(0, 475, __pyx_L1_error)
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_fill_cache, ((PyObject *)__pyx_v_fill_cache), __pyx_t_5, __pyx_callargs+2, 2) < (0)) __PYX_ERR(0, 475, __pyx_L1_error)
    __pyx_t_1 = __Pyx_Object_VectorcallMethod_CallFromBuilder((PyObject*)__pyx_mstate_global->__pyx_n_u_get, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_5);
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 475, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "plyvel/_plyvel.pyx":473
 *         )
 * 
 *     def get(self, bytes key not None, default=None, *,             # <<<<<<<<<<<<<<
 *             bool verify_checksums=False, bool fill_cache=True):
 *         return self.db.get(
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("plyvel._plyvel.PrefixedDB.get", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":481
 *             fill_cache=fill_cache)
 * 
 *     def put(self, bytes key not None, value not None, *,             # <<<<<<<<<<<<<<
 *             bool sync=False):
 *         return self.db.put(self.prefix + key, value, sync=sync)
*/

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_10PrefixedDB_7put(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6plyvel_7_plyvel_10PrefixedDB_6put, "PrefixedDB.put(self, bytes key, value, *, bool sync=False)");
static PyMethodDef __pyx_mdef_6plyvel_7_plyvel_10PrefixedDB_7put = {"put", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_10PrefixedDB_7put, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_10PrefixedDB_6put};
static PyObject *__pyx_pw_6plyvel_7_plyvel_10PrefixedDB_7put(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_key = 0;
  PyObject *__pyx_v_value = 0;
  PyLongObject *__pyx_v_sync = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[3] = {0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("put (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_key,&__pyx_mstate_global->__pyx_n_u_value,&__pyx_mstate_global->__pyx_n_u_sync,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(0, 481, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 481, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 481, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "put", 0) < (0)) __PYX_ERR(0, 481, __pyx_L3_error)

      /* "plyvel/_plyvel.pyx":482
 * 
 *     def put(self, bytes key not None, value not None, *,
 *             bool sync=False):             # <<<<<<<<<<<<<<
 *         return self.db.put(self.prefix + key, value, sync=sync)
 * 
*/
      if (!values[2]) values[2] = __Pyx_NewRef((PyObject *)((PyLongObject *)Py_False));
      for (Py_ssize_t i = __pyx_nargs; i < 2; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("put", 1, 2, 2, i); __PYX_ERR(0, 481, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 2)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 481, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 481, __pyx_L3_error)
      if (!values[2]) values[2] = __Pyx_NewRef((PyObject *)((PyLongObject *)Py_False));
    }
    __pyx_v_key = ((PyObject*)values[0]);
    __pyx_v_value = values[1];
    __pyx_v_sync = ((PyLongObject *)values[2]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("put", 1, 2, 2, __pyx_nargs); __PYX_ERR(0, 481, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("plyvel._plyvel.PrefixedDB.put", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_key), (&PyBytes_Type), 0, "key", 1))) __PYX_ERR(0, 481, __pyx_L1_error)
  if (unlikely(((PyObject *)__pyx_v_value) == Py_None)) {
    PyErr_Format(PyExc_TypeError, "Argument '%.200s' must not be None", "value"); __PYX_ERR(0, 481, __pyx_L1_error)
  }
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_sync), __pyx_mstate_global->__pyx_ptype_7cpython_4bool_bool, 1, "sync", 0))) __PYX_ERR(0, 482, __pyx_L1_error)
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_10PrefixedDB_6put(((struct __pyx_obj_6plyvel_7_plyvel_PrefixedDB *)__pyx_v_self), __pyx_v_key, __pyx_v_value, __pyx_v_sync);

  /* "plyvel/_plyvel.pyx":481
 *             fill_cache=fill_cache)
 * 
 *     def put(self, bytes key not None, value not None, *,             # <<<<<<<<<<<<<<
 *             bool sync=False):
 *         return self.db.put(self.prefix + key, value, sync=sync)
*/

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_10PrefixedDB_6put(struct __pyx_obj_6plyvel_7_plyvel_PrefixedDB *__pyx_v_self, PyObject *__pyx_v_key, PyObject *__pyx_v_value, PyLongObject *__pyx_v_sync) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("put", 0);

  /* "plyvel/_plyvel.pyx":483
 *     def put(self, bytes key not None, value not None, *,
 *             bool sync=False):
 *         return self.db.put(self.prefix + key, value, sync=sync)             # <<<<<<<<<<<<<<
 * 
 *     def delete(self, bytes key not None, *, bool sync=False):
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = ((PyObject *)__pyx_v_self->db);
  __Pyx_INCREF(__pyx_t_2);
  __pyx_t_3 = PyNumber_Add(__pyx_v_self->prefix, __pyx_v_key); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 483, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = 0;
  {
    PyObject *__pyx_callargs[3 + ((CYTHON_VECTORCALL) ? 1 : 0)] = {__pyx_t_2, __pyx_t_3, __pyx_v_value};
    __pyx_t_5 = __Pyx_MakeVectorcallBuilderKwds(1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 483, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_sync, ((PyObject *)__pyx_v_sync), __pyx_t_5, __pyx_callargs+3, 0) < (0)) __PYX_ERR(0, 483, __pyx_L1_error)
    __pyx_t_1 = __Pyx_Object_VectorcallMethod_CallFromBuilder((PyObject*)__pyx_mstate_global->__pyx_n_u_put, __pyx_callargs+__pyx_t_4, (3-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_5);
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 483, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "plyvel/_plyvel.pyx":481
 *             fill_cache=fill_cache)
 * 
 *     def put(self, bytes key not None, value not None, *,             # <<<<<<<<<<<<<<
 *             bool sync=False):
 *         return self.db.put(self.prefix + key, value, sync=sync)
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("plyvel._plyvel.PrefixedDB.put", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":485
 *         return self.db.put(self.prefix + key, value, sync=sync)
 * 
 *     def delete(self, bytes key not None, *, bool sync=False):             # <<<<<<<<<<<<<<
 *         return self.db.delete(self.prefix + key, sync=sync)
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_10PrefixedDB_9delete(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6plyvel_7_plyvel_10PrefixedDB_8delete, "PrefixedDB.delete(self, bytes key, *, bool sync=False)");
static PyMethodDef __pyx_mdef_6plyvel_7_plyvel_10PrefixedDB_9delete = {"delete", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_10PrefixedDB_9delete, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_10PrefixedDB_8delete};
static PyObject *__pyx_pw_6plyvel_7_plyvel_10PrefixedDB_9delete(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_key = 0;
  PyLongObject *__pyx_v_sync = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("delete (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_key,&__pyx_mstate_global->__pyx_n_u_sync,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(0, 485, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 485, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "delete", 0) < (0)) __PYX_ERR(0, 485, __pyx_L3_error)
      if (!values[1]) values[1] = __Pyx_NewRef((PyObject *)((PyLongObject *)Py_False));
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("delete", 1, 1, 1, i); __PYX_ERR(0, 485, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 485, __pyx_L3_error)
      if (!values[1]) values[1] = __Pyx_NewRef((PyObject *)((PyLongObject *)Py_False));
    }
    __pyx_v_key = ((PyObject*)values[0]);
    __pyx_v_sync = ((PyLongObject *)values[1]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("delete", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 485, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("plyvel._plyvel.PrefixedDB.delete", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_key), (&PyBytes_Type), 0, "key", 1))) __PYX_ERR(0, 485, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_sync), __pyx_mstate_global->__pyx_ptype_7cpython_4bool_bool, 1, "sync", 0))) __PYX_ERR(0, 485, __pyx_L1_error)
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_10PrefixedDB_8delete(((struct __pyx_obj_6plyvel_7_plyvel_PrefixedDB *)__pyx_v_self), __pyx_v_key, __pyx_v_sync);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_10PrefixedDB_8delete(struct __pyx_obj_6plyvel_7_plyvel_PrefixedDB *__pyx_v_self, PyObject *__pyx_v_key, PyLongObject *__pyx_v_sync) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("delete", 0);

  /* "plyvel/_plyvel.pyx":486
 * 
 *     def delete(self, bytes key not None, *, bool sync=False):
 *         return self.db.delete(self.prefix + key, sync=sync)             # <<<<<<<<<<<<<<
 * 
 *     def write_batch(self, *, transaction=False, bool sync=False):
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = ((PyObject *)__pyx_v_self->db);
  __Pyx_INCREF(__pyx_t_2);
  __pyx_t_3 = PyNumber_Add(__pyx_v_self->prefix, __pyx_v_key); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 486, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = 0;
  {
    PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 1 : 0)] = {__pyx_t_2, __pyx_t_3};
    __pyx_t_5 = __Pyx_MakeVectorcallBuilderKwds(1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 486, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_sync, ((PyObject *)__pyx_v_sync), __pyx_t_5, __pyx_callargs+2, 0) < (0)) __PYX_ERR(0, 486, __pyx_L1_error)
    __pyx_t_1 = __Pyx_Object_VectorcallMethod_CallFromBuilder((PyObject*)__pyx_mstate_global->__pyx_n_u_delete, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_5);
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 486, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "plyvel/_plyvel.pyx":485
 *         return self.db.put(self.prefix + key, value, sync=sync)
 * 
 *     def delete(self, bytes key not None, *, bool sync=False):             # <<<<<<<<<<<<<<
 *         return self.db.delete(self.prefix + key, sync=sync)
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("plyvel._plyvel.PrefixedDB.delete", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":488
 *         return self.db.delete(self.prefix + key, sync=sync)
 * 
 *     def write_batch(self, *, transaction=False, bool sync=False):             # <<<<<<<<<<<<<<
 *         return WriteBatch(self.db, self.prefix, transaction, sync)
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_10PrefixedDB_11write_batch(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6plyvel_7_plyvel_10PrefixedDB_10write_batch, "PrefixedDB.write_batch(self, *, transaction=False, bool sync=False)");
static PyMethodDef __pyx_mdef_6plyvel_7_plyvel_10PrefixedDB_11write_batch = {"write_batch", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_10PrefixedDB_11write_batch, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_10PrefixedDB_10write_batch};
static PyObject *__pyx_pw_6plyvel_7_plyvel_10PrefixedDB_11write_batch(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_transaction = 0;
  PyLongObject *__pyx_v_sync = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("write_batch (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_transaction,&__pyx_mstate_global->__pyx_n_u_sync,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(0, 488, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, 0, __pyx_kwds_len, "write_batch", 0) < (0)) __PYX_ERR(0, 488, __pyx_L3_error)
      if (!values[0]) values[0] = __Pyx_NewRef(((PyObject *)Py_False));
      if (!values[1]) values[1] = __Pyx_NewRef((PyObject *)((PyLongObject *)Py_False));
    } else if (unlikely(__pyx_nargs != 0)) {
      goto __pyx_L5_argtuple_error;
    } else {
      if (!values[0]) values[0] = __Pyx_NewRef(((PyObject *)Py_False));
      if (!values[1]) values[1] = __Pyx_NewRef((PyObject *)((PyLongObject *)Py_False));
    }
    __pyx_v_transaction = values[0];
    __pyx_v_sync = ((PyLongObject *)values[1]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("write_batch", 1, 0, 0, __pyx_nargs); __PYX_ERR(0, 488, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("plyvel._plyvel.PrefixedDB.write_batch", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_sync), __pyx_mstate_global->__pyx_ptype_7cpython_4bool_bool, 1, "sync", 0))) __PYX_ERR(0, 488, __pyx_L1_error)
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_10PrefixedDB_10write_batch(((struct __pyx_obj_6plyvel_7_plyvel_PrefixedDB *)__pyx_v_self), __pyx_v_transaction, __pyx_v_sync);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_10PrefixedDB_10write_batch(struct __pyx_obj_6plyvel_7_plyvel_PrefixedDB *__pyx_v_self, PyObject *__pyx_v_transaction, PyLongObject *__pyx_v_sync) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  size_t __pyx_t_3;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("write_batch", 0);

  /* "plyvel/_plyvel.pyx":489
 * 
 *     def write_batch(self, *, transaction=False, bool sync=False):
 *         return WriteBatch(self.db, self.prefix, transaction, sync)             # <<<<<<<<<<<<<<
 * 
 *     def __iter__(self):
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = NULL;
  __pyx_t_3 = 1;
  {
    PyObject *__pyx_callargs[5] = {__pyx_t_2, ((PyObject *)__pyx_v_self->db), __pyx_v_self->prefix, __pyx_v_transaction, ((PyObject *)__pyx_v_sync)};
    __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_mstate_global->__pyx_ptype_6plyvel_7_plyvel_WriteBatch, __pyx_callargs+__pyx_t_3, (5-__pyx_t_3) | (__pyx_t_3*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 489, __pyx_L1_error)
    __Pyx_GOTREF((PyObject *)__pyx_t_1);
  }
  __pyx_r = ((PyObject *)__pyx_t_1);
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "plyvel/_plyvel.pyx":488
 *         return self.db.delete(self.prefix + key, sync=sync)
 * 
 *     def write_batch(self, *, transaction=False, bool sync=False):             # <<<<<<<<<<<<<<
 *         return WriteBatch(self.db, self.prefix, transaction, sync)
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("plyvel._plyvel.PrefixedDB.write_batch", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":491
 *         return WriteBatch(self.db, self.prefix, transaction, sync)
 * 
 *     def __iter__(self):             # <<<<<<<<<<<<<<
 *         return self.iterator()
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_10PrefixedDB_13__iter__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6plyvel_7_plyvel_10PrefixedDB_13__iter__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__iter__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_10PrefixedDB_12__iter__(((struct __pyx_obj_6plyvel_7_plyvel_PrefixedDB *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_10PrefixedDB_12__iter__(struct __pyx_obj_6plyvel_7_plyvel_PrefixedDB *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  size_t __pyx_t_3;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__iter__", 0);

  /* "plyvel/_plyvel.pyx":492
 * 
 *     def __iter__(self):
 *         return self.iterator()             # <<<<<<<<<<<<<<
 * 
 *     def iterator(self, *, reverse=False, start=None, stop=None,
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = ((PyObject *)__pyx_v_self);
  __Pyx_INCREF(__pyx_t_2);
  __pyx_t_3 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, NULL};
    __pyx_t_1 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_iterator, __pyx_callargs+__pyx_t_3, (1-__pyx_t_3) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 492, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "plyvel/_plyvel.pyx":491
 *         return WriteBatch(self.db, self.prefix, transaction, sync)
 * 
 *     def __iter__(self):             # <<<<<<<<<<<<<<
 *         return self.iterator()
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("plyvel._plyvel.PrefixedDB.__iter__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":494
 *         return self.iterator()
 * 
 *     def iterator(self, *, reverse=False, start=None, stop=None,             # <<<<<<<<<<<<<<
 *                  include_start=True, include_stop=False, prefix=None,
 *                  include_key=True, include_value=True,
*/

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_10PrefixedDB_15iterator(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6plyvel_7_plyvel_10PrefixedDB_14iterator, "PrefixedDB.iterator(self, *, reverse=False, start=None, stop=None, include_start=True, include_stop=False, prefix=None, include_key=True, include_value=True, bool verify_checksums=False, bool fill_cache=True)");
static PyMethodDef __pyx_mdef_6plyvel_7_plyvel_10PrefixedDB_15iterator = {"iterator", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_10PrefixedDB_15iterator, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_10PrefixedDB_14iterator};
static PyObject *__pyx_pw_6plyvel_7_plyvel_10PrefixedDB_15iterator(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_reverse = 0;
  PyObject *__pyx_v_start = 0;
  PyObject *__pyx_v_stop = 0;
  PyObject *__pyx_v_include_start = 0;
  PyObject *__pyx_v_include_stop = 0;
  PyObject *__pyx_v_prefix = 0;
  PyObject *__pyx_v_include_key = 0;
  PyObject *__pyx_v_include_value = 0;
  PyLongObject *__pyx_v_verify_checksums = 0;
  PyLongObject *__pyx_v_fill_cache = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[10] = {0,0,0,0,0,0,0,0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("iterator (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_reverse,&__pyx_mstate_global->__pyx_n_u_start,&__pyx_mstate_global->__pyx_n_u_stop,&__pyx_mstate_global->__pyx_n_u_include_start,&__pyx_mstate_global->__pyx_n_u_include_stop,&__pyx_mstate_global->__pyx_n_u_prefix,&__pyx_mstate_global->__pyx_n_u_include_key,&__pyx_mstate_global->__pyx_n_u_include_value,&__pyx_mstate_global->__pyx_n_u_verify_checksums,&__pyx_mstate_global->__pyx_n_u_fill_cache,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(0, 494, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, 0, __pyx_kwds_len, "iterator", 0) < (0)) __PYX_ERR(0, 494, __pyx_L3_error)
      if (!values[0]) values[0] = __Pyx_NewRef(((PyObject *)Py_False));
      if (!values[1]) values[1] = __Pyx_NewRef(((PyObject *)Py_None));
      if (!values[2]) values[2] = __Pyx_NewRef(((PyObject *)Py_None));

      /* "plyvel/_plyvel.pyx":495
 * 
 *     def iterator(self, *, reverse=False, start=None, stop=None,
 *                  include_start=True, include_stop=False, prefix=None,             # <<<<<<<<<<<<<<
 *                  include_key=True, include_value=True,
 *                  bool verify_checksums=False, bool fill_cache=True):
*/
      if (!values[3]) values[3] = __Pyx_NewRef(((PyObject *)Py_True));
      if (!values[4]) values[4] = __Pyx_NewRef(((PyObject *)Py_False));
      if (!values[5]) values[5] = __Pyx_NewRef(((PyObject *)Py_None));

      /* "plyvel/_plyvel.pyx":496
 *     def iterator(self, *, reverse=False, start=None, stop=None,
 *                  include_start=True, include_stop=False, prefix=None,
 *                  include_key=True, include_value=True,             # <<<<<<<<<<<<<<
 *                  bool verify_checksums=False, bool fill_cache=True):
 *         return Iterator(
*/
      if (!values[6]) values[6] = __Pyx_NewRef(((PyObject *)Py_True));
      if (!values[7]) values[7] = __Pyx_NewRef(((PyObject *)Py_True));

      /* "plyvel/_plyvel.pyx":497
 *                  include_start=True, include_stop=False, prefix=None,
 *                  include_key=True, include_value=True,
 *                  bool verify_checksums=False, bool fill_cache=True):             # <<<<<<<<<<<<<<
 *         return Iterator(
 *             self.db,
*/
      if (!values[8]) values[8] = __Pyx_NewRef((PyObject *)((PyLongObject *)Py_False));
      if (!values[9]) values[9] = __Pyx_NewRef((PyObject *)((PyLongObject *)Py_True));
    } else if (unlikely(__pyx_nargs != 0)) {
      goto __pyx_L5_argtuple_error;
    } else {

      /* "plyvel/_plyvel.pyx":494
 *         return self.iterator()
 * 
 *     def iterator(self, *, reverse=False, start=None, stop=None,             # <<<<<<<<<<<<<<
 *                  include_start=True, include_stop=False, prefix=None,
 *                  include_key=True, include_value=True,
*/
      if (!values[0]) values[0] = __Pyx_NewRef(((PyObject *)Py_False));
      if (!values[1]) values[1] = __Pyx_NewRef(((PyObject *)Py_None));
      if (!values[2]) values[2] = __Pyx_NewRef(((PyObject *)Py_None));

      /* "plyvel/_plyvel.pyx":495
 * 
 *     def iterator(self, *, reverse=False, start=None, stop=None,
 *                  include_start=True, include_stop=False, prefix=None,             # <<<<<<<<<<<<<<
 *                  include_key=True, include_value=True,
 *                  bool verify_checksums=False, bool fill_cache=True):
*/
      if (!values[3]) values[3] = __Pyx_NewRef(((PyObject *)Py_True));
      if (!values[4]) values[4] = __Pyx_NewRef(((PyObject *)Py_False));
      if (!values[5]) values[5] = __Pyx_NewRef(((PyObject *)Py_None));

      /* "plyvel/_plyvel.pyx":496
 *     def iterator(self, *, reverse=False, start=None, stop=None,
 *                  include_start=True, include_stop=False, prefix=None,
 *                  include_key=True, include_value=True,             # <<<<<<<<<<<<<<
 *                  bool verify_checksums=False, bool fill_cache=True):
 *         return Iterator(
*/
      if (!values[6]) values[6] = __Pyx_NewRef(((PyObject *)Py_True));
      if (!values[7]) values[7] = __Pyx_NewRef(((PyObject *)Py_True));

      /* "plyvel/_plyvel.pyx":497
 *                  include_start=True, include_stop=False, prefix=None,
 *                  include_key=True, include_value=True,
 *                  bool verify_checksums=False, bool fill_cache=True):             # <<<<<<<<<<<<<<
 *         return Iterator(
 *             self.db,
*/
      if (!values[8]) values[8] = __Pyx_NewRef((PyObject *)((PyLongObject *)Py_False));
      if (!values[9]) values[9] = __Pyx_NewRef((PyObject *)((PyLongObject *)Py_True));
    }
    __pyx_v_reverse = values[0];
    __pyx_v_start = values[1];
    __pyx_v_stop = values[2];
    __pyx_v_include_start = values[3];
    __pyx_v_include_stop = values[4];
    __pyx_v_prefix = values[5];
    __pyx_v_include_key = values[6];
    __pyx_v_include_value = values[7];
    __pyx_v_verify_checksums = ((PyLongObject *)values[8]);
    __pyx_v_fill_cache = ((PyLongObject *)values[9]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("iterator", 1, 0, 0, __pyx_nargs); __PYX_ERR(0, 494, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("plyvel._plyvel.PrefixedDB.iterator", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_verify_checksums), __pyx_mstate_global->__pyx_ptype_7cpython_4bool_bool, 1, "verify_checksums", 0))) __PYX_ERR(0, 497, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_fill_cache), __pyx_mstate_global->__pyx_ptype_7cpython_4bool_bool, 1, "fill_cache", 0))) __PYX_ERR(0, 497, __pyx_L1_error)
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_10PrefixedDB_14iterator(((struct __pyx_obj_6plyvel_7_plyvel_PrefixedDB *)__pyx_v_self), __pyx_v_reverse, __pyx_v_start, __pyx_v_stop, __pyx_v_include_start, __pyx_v_include_stop, __pyx_v_prefix, __pyx_v_include_key, __pyx_v_include_value, __pyx_v_verify_checksums, __pyx_v_fill_cache);

  /* "plyvel/_plyvel.pyx":494
 *         return self.iterator()
 * 
 *     def iterator(self, *, reverse=False, start=None, stop=None,             # <<<<<<<<<<<<<<
 *                  include_start=True, include_stop=False, prefix=None,
 *                  include_key=True, include_value=True,
*/

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_10PrefixedDB_14iterator(struct __pyx_obj_6plyvel_7_plyvel_PrefixedDB *__pyx_v_self, PyObject *__pyx_v_reverse, PyObject *__pyx_v_start, PyObject *__pyx_v_stop, PyObject *__pyx_v_include_start, PyObject *__pyx_v_include_stop, PyObject *__pyx_v_prefix, PyObject *__pyx_v_include_key, PyObject *__pyx_v_include_value, PyLongObject *__pyx_v_verify_checksums, PyLongObject *__pyx_v_fill_cache) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  size_t __pyx_t_3;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("iterator", 0);

  /* "plyvel/_plyvel.pyx":498
 *                  include_key=True, include_value=True,
 *                  bool verify_checksums=False, bool fill_cache=True):
 *         return Iterator(             # <<<<<<<<<<<<<<
 *             self.db,
 *             self.prefix,
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = NULL;

  /* "plyvel/_plyvel.pyx":511
 *             verify_checksums,
 *             fill_cache,
 *             None,  # snapshot             # <<<<<<<<<<<<<<
 *         )
 * 
*/
  __pyx_t_3 = 1;
  {
    PyObject *__pyx_callargs[14] = {__pyx_t_2, ((PyObject *)__pyx_v_self->db), __pyx_v_self->prefix, __pyx_v_reverse, __pyx_v_start, __pyx_v_stop, __pyx_v_include_start, __pyx_v_include_stop, __pyx_v_prefix, __pyx_v_include_key, __pyx_v_include_value, ((PyObject *)__pyx_v_verify_checksums), ((PyObject *)__pyx_v_fill_cache), Py_None};
    __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)__pyx_mstate_global->__pyx_ptype_6plyvel_7_plyvel_Iterator, __pyx_callargs+__pyx_t_3, (14-__pyx_t_3) | (__pyx_t_3*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 498, __pyx_L1_error)
    __Pyx_GOTREF((PyObject *)__pyx_t_1);
  }
  __pyx_r = ((PyObject *)__pyx_t_1);
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "plyvel/_plyvel.pyx":494
 *         return self.iterator()
 * 
 *     def iterator(self, *, reverse=False, start=None, stop=None,             # <<<<<<<<<<<<<<
 *                  include_start=True, include_stop=False, prefix=None,
 *                  include_key=True, include_value=True,
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("plyvel._plyvel.PrefixedDB.iterator", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":514
 *         )
 * 
 *     def snapshot(self):             # <<<<<<<<<<<<<<
 *         return Snapshot(db=self.db, prefix=self.prefix)
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_10PrefixedDB_17snapshot(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6plyvel_7_plyvel_10PrefixedDB_16snapshot, "PrefixedDB.snapshot(self)");
static PyMethodDef __pyx_mdef_6plyvel_7_plyvel_10PrefixedDB_17snapshot = {"snapshot", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_10PrefixedDB_17snapshot, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_10PrefixedDB_16snapshot};
static PyObject *__pyx_pw_6plyvel_7_plyvel_10PrefixedDB_17snapshot(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("snapshot (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("snapshot", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("snapshot", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_10PrefixedDB_16snapshot(((struct __pyx_obj_6plyvel_7_plyvel_PrefixedDB *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_10PrefixedDB_16snapshot(struct __pyx_obj_6plyvel_7_plyvel_PrefixedDB *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  size_t __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("snapshot", 0);

  /* "plyvel/_plyvel.pyx":515
 * 
 *     def snapshot(self):
 *         return Snapshot(db=self.db, prefix=self.prefix)             # <<<<<<<<<<<<<<
 * 
 *     def prefixed_db(self, bytes prefix not None):
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = NULL;
  __pyx_t_3 = 1;
  {
    PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 2 : 0)] = {__pyx_t_2, NULL};
    __pyx_t_4 = __Pyx_MakeVectorcallBuilderKwds(2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 515, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_db, ((PyObject *)__pyx_v_self->db), __pyx_t_4, __pyx_callargs+1, 0) < (0)) __PYX_ERR(0, 515, __pyx_L1_error)
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_prefix, __pyx_v_self->prefix, __pyx_t_4, __pyx_callargs+1, 1) < (0)) __PYX_ERR(0, 515, __pyx_L1_error)
    __pyx_t_1 = __Pyx_Object_Vectorcall_CallFromBuilder((PyObject*)__pyx_mstate_global->__pyx_ptype_6plyvel_7_plyvel_Snapshot, __pyx_callargs+__pyx_t_3, (1-__pyx_t_3) | (__pyx_t_3*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 515, __pyx_L1_error)
    __Pyx_GOTREF((PyObject *)__pyx_t_1);
  }
  __pyx_r = ((PyObject *)__pyx_t_1);
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "plyvel/_plyvel.pyx":514
 *         )
 * 
 *     def snapshot(self):             # <<<<<<<<<<<<<<
 *         return Snapshot(db=self.db, prefix=self.prefix)
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("plyvel._plyvel.PrefixedDB.snapshot", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":517
 *         return Snapshot(db=self.db, prefix=self.prefix)
 * 
 *     def prefixed_db(self, bytes prefix not None):             # <<<<<<<<<<<<<<
 *         return PrefixedDB(db=self.db, prefix=self.prefix + prefix)
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_10PrefixedDB_19prefixed_db(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6plyvel_7_plyvel_10PrefixedDB_18prefixed_db, "PrefixedDB.prefixed_db(self, bytes prefix)");
static PyMethodDef __pyx_mdef_6plyvel_7_plyvel_10PrefixedDB_19prefixed_db = {"prefixed_db", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_10PrefixedDB_19prefixed_db, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_10PrefixedDB_18prefixed_db};
static PyObject *__pyx_pw_6plyvel_7_plyvel_10PrefixedDB_19prefixed_db(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_prefix = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("prefixed_db (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_prefix,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(0, 517, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 517, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "prefixed_db", 0) < (0)) __PYX_ERR(0, 517, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("prefixed_db", 1, 1, 1, i); __PYX_ERR(0, 517, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 517, __pyx_L3_error)
    }
    __pyx_v_prefix = ((PyObject*)values[0]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("prefixed_db", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 517, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("plyvel._plyvel.PrefixedDB.prefixed_db", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_prefix), (&PyBytes_Type), 0, "prefix", 1))) __PYX_ERR(0, 517, __pyx_L1_error)
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_10PrefixedDB_18prefixed_db(((struct __pyx_obj_6plyvel_7_plyvel_PrefixedDB *)__pyx_v_self), __pyx_v_prefix);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_10PrefixedDB_18prefixed_db(struct __pyx_obj_6plyvel_7_plyvel_PrefixedDB *__pyx_v_self, PyObject *__pyx_v_prefix) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("prefixed_db", 0);

  /* "plyvel/_plyvel.pyx":518
 * 
 *     def prefixed_db(self, bytes prefix not None):
 *         return PrefixedDB(db=self.db, prefix=self.prefix + prefix)             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = NULL;
  __pyx_t_3 = PyNumber_Add(__pyx_v_self->prefix, __pyx_v_prefix); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 518, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = 1;
  {
    PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 2 : 0)] = {__pyx_t_2, NULL};
    __pyx_t_5 = __Pyx_MakeVectorcallBuilderKwds(2); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 518, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_db, ((PyObject *)__pyx_v_self->db), __pyx_t_5, __pyx_callargs+1, 0) < (0)) __PYX_ERR(0, 518, __pyx_L1_error)
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_prefix, __pyx_t_3, __pyx_t_5, __pyx_callargs+1, 1) < (0)) __PYX_ERR(0, 518, __pyx_L1_error)
    __pyx_t_1 = __Pyx_Object_Vectorcall_CallFromBuilder((PyObject*)__pyx_mstate_global->__pyx_ptype_6plyvel_7_plyvel_PrefixedDB, __pyx_callargs+__pyx_t_4, (1-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_5);
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 518, __pyx_L1_error)
    __Pyx_GOTREF((PyObject *)__pyx_t_1);
  }
  __pyx_r = ((PyObject *)__pyx_t_1);
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "plyvel/_plyvel.pyx":517
 *         return Snapshot(db=self.db, prefix=self.prefix)
 * 
 *     def prefixed_db(self, bytes prefix not None):             # <<<<<<<<<<<<<<
 *         return PrefixedDB(db=self.db, prefix=self.prefix + prefix)
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("plyvel._plyvel.PrefixedDB.prefixed_db", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":460
 * 
 * cdef class PrefixedDB:
 *     cdef readonly DB db             # <<<<<<<<<<<<<<
 *     cdef readonly bytes prefix
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_10PrefixedDB_2db_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6plyvel_7_plyvel_10PrefixedDB_2db_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_10PrefixedDB_2db___get__(((struct __pyx_obj_6plyvel_7_plyvel_PrefixedDB *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_10PrefixedDB_2db___get__(struct __pyx_obj_6plyvel_7_plyvel_PrefixedDB *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF((PyObject *)__pyx_v_self->db);
  __pyx_r = ((PyObject *)__pyx_v_self->db);
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":461
 * cdef class PrefixedDB:
 *     cdef readonly DB db
 *     cdef readonly bytes prefix             # <<<<<<<<<<<<<<
 * 
 *     def __init__(self, *, DB db not None, bytes prefix not None):
*/

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_10PrefixedDB_6prefix_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6plyvel_7_plyvel_10PrefixedDB_6prefix_1__get__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_10PrefixedDB_6prefix___get__(((struct __pyx_obj_6plyvel_7_plyvel_PrefixedDB *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_10PrefixedDB_6prefix___get__(struct __pyx_obj_6plyvel_7_plyvel_PrefixedDB *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->prefix);
  __pyx_r = __pyx_v_self->prefix;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
*/

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_10PrefixedDB_21__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6plyvel_7_plyvel_10PrefixedDB_20__reduce_cython__, "PrefixedDB.__reduce_cython__(self)");
static PyMethodDef __pyx_mdef_6plyvel_7_plyvel_10PrefixedDB_21__reduce_cython__ = {"__reduce_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_10PrefixedDB_21__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_10PrefixedDB_20__reduce_cython__};
static PyObject *__pyx_pw_6plyvel_7_plyvel_10PrefixedDB_21__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("__reduce_cython__", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("__reduce_cython__", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_10PrefixedDB_20__reduce_cython__(((struct __pyx_obj_6plyvel_7_plyvel_PrefixedDB *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_10PrefixedDB_20__reduce_cython__(struct __pyx_obj_6plyvel_7_plyvel_PrefixedDB *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self.db, self.prefix)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None and _dict:
*/
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF((PyObject *)__pyx_v_self->db);
  __Pyx_GIVEREF((PyObject *)__pyx_v_self->db);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)__pyx_v_self->db)) != (0)) __PYX_ERR(1, 5, __pyx_L1_error);
  __Pyx_INCREF(__pyx_v_self->prefix);
  __Pyx_GIVEREF(__pyx_v_self->prefix);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_v_self->prefix) != (0)) __PYX_ERR(1, 5, __pyx_L1_error);
  __pyx_v_state = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self.db, self.prefix)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None and _dict:
 *         state += (_dict,)
*/
  __pyx_t_1 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_dict, Py_None); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v__dict = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "(tree fragment)":7
 *     state = (self.db, self.prefix)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None and _dict:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
*/
  __pyx_t_3 = (__pyx_v__dict != Py_None);
  if (__pyx_t_3) {
  } else {
    __pyx_t_2 = __pyx_t_3;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_v__dict); if (unlikely((__pyx_t_3 < 0))) __PYX_ERR(1, 7, __pyx_L1_error)
  __pyx_t_2 = __pyx_t_3;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_2) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None and _dict:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
*/
    __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v__dict) != (0)) __PYX_ERR(1, 8, __pyx_L1_error);
    __pyx_t_4 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_1); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_4));
    __pyx_t_4 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None and _dict:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = ('self.db is not None or self.prefix is not None',)
*/
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self.db, self.prefix)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None and _dict:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
*/
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = ('self.db is not None or self.prefix is not None',)             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_PrefixedDB, (type(self), 0xe9c0b3a, None), state
*/
  /*else*/ {
    __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_mstate_global->__pyx_tuple[1]); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 11, __pyx_L1_error)
    __pyx_v_use_setstate = __pyx_t_2;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = ('self.db is not None or self.prefix is not None',)
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_PrefixedDB, (type(self), 0xe9c0b3a, None), state
 *     else:
*/
  if (__pyx_v_use_setstate) {

    /* "(tree fragment)":13
 *         use_setstate = ('self.db is not None or self.prefix is not None',)
 *     if use_setstate:
 *         return __pyx_unpickle_PrefixedDB, (type(self), 0xe9c0b3a, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_PrefixedDB, (type(self), 0xe9c0b3a, state)
*/
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_pyx_unpickle_PrefixedDB); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_1 = PyTuple_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self)))) != (0)) __PYX_ERR(1, 13, __pyx_L1_error);
    __Pyx_INCREF(__pyx_mstate_global->__pyx_int_245107514);
    __Pyx_GIVEREF(__pyx_mstate_global->__pyx_int_245107514);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_mstate_global->__pyx_int_245107514) != (0)) __PYX_ERR(1, 13, __pyx_L1_error);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 2, Py_None) != (0)) __PYX_ERR(1, 13, __pyx_L1_error);
    __pyx_t_5 = PyTuple_New(3); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_4);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4) != (0)) __PYX_ERR(1, 13, __pyx_L1_error);
    __Pyx_GIVEREF(__pyx_t_1);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_1) != (0)) __PYX_ERR(1, 13, __pyx_L1_error);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_v_state) != (0)) __PYX_ERR(1, 13, __pyx_L1_error);
    __pyx_t_4 = 0;
    __pyx_t_1 = 0;
    __pyx_r = __pyx_t_5;
    __pyx_t_5 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = ('self.db is not None or self.prefix is not None',)
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_PrefixedDB, (type(self), 0xe9c0b3a, None), state
 *     else:
*/
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_PrefixedDB, (type(self), 0xe9c0b3a, None), state
 *     else:
 *         return __pyx_unpickle_PrefixedDB, (type(self), 0xe9c0b3a, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_PrefixedDB__set_state(self, __pyx_state)
*/
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_mstate_global->__pyx_n_u_pyx_unpickle_PrefixedDB); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_1 = PyTuple_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self)))) != (0)) __PYX_ERR(1, 15, __pyx_L1_error);
    __Pyx_INCREF(__pyx_mstate_global->__pyx_int_245107514);
    __Pyx_GIVEREF(__pyx_mstate_global->__pyx_int_245107514);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_mstate_global->__pyx_int_245107514) != (0)) __PYX_ERR(1, 15, __pyx_L1_error);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_v_state) != (0)) __PYX_ERR(1, 15, __pyx_L1_error);
    __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_5);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_5) != (0)) __PYX_ERR(1, 15, __pyx_L1_error);
    __Pyx_GIVEREF(__pyx_t_1);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_1) != (0)) __PYX_ERR(1, 15, __pyx_L1_error);
    __pyx_t_5 = 0;
    __pyx_t_1 = 0;
    __pyx_r = __pyx_t_4;
    __pyx_t_4 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("plyvel._plyvel.PrefixedDB.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_PrefixedDB, (type(self), 0xe9c0b3a, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_PrefixedDB__set_state(self, __pyx_state)
*/

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_10PrefixedDB_23__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6plyvel_7_plyvel_10PrefixedDB_22__setstate_cython__, "PrefixedDB.__setstate_cython__(self, __pyx_state)");
static PyMethodDef __pyx_mdef_6plyvel_7_plyvel_10PrefixedDB_23__setstate_cython__ = {"__setstate_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_10PrefixedDB_23__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_10PrefixedDB_22__setstate_cython__};
static PyObject *__pyx_pw_6plyvel_7_plyvel_10PrefixedDB_23__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v___pyx_state = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_pyx_state,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(1, 16, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(1, 16, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__setstate_cython__", 0) < (0)) __PYX_ERR(1, 16, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, i); __PYX_ERR(1, 16, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(1, 16, __pyx_L3_error)
    }
    __pyx_v___pyx_state = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, __pyx_nargs); __PYX_ERR(1, 16, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("plyvel._plyvel.PrefixedDB.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_10PrefixedDB_22__setstate_cython__(((struct __pyx_obj_6plyvel_7_plyvel_PrefixedDB *)__pyx_v_self), __pyx_v___pyx_state);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_10PrefixedDB_22__setstate_cython__(struct __pyx_obj_6plyvel_7_plyvel_PrefixedDB *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_PrefixedDB, (type(self), 0xe9c0b3a, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_PrefixedDB__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
*/
  __pyx_t_1 = __pyx_v___pyx_state;
  __Pyx_INCREF(__pyx_t_1);
  if (!(likely(PyTuple_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("tuple", __pyx_t_1))) __PYX_ERR(1, 17, __pyx_L1_error)
  if (unlikely(__pyx_t_1 == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "cannot pass None into a C function argument that is declared 'not None'");
    __PYX_ERR(1, 17, __pyx_L1_error)
  }
  __pyx_t_2 = __pyx_f_6plyvel_7_plyvel___pyx_unpickle_PrefixedDB__set_state(__pyx_v_self, ((PyObject*)__pyx_t_1)); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_PrefixedDB, (type(self), 0xe9c0b3a, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_PrefixedDB__set_state(self, __pyx_state)
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("plyvel._plyvel.PrefixedDB.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":521
 * 
 * 
 * def repair_db(name, *, paranoid_checks=None, write_buffer_size=None,             # <<<<<<<<<<<<<<
 *               max_open_files=None, lru_cache_size=None, block_size=None,
 *               block_restart_interval=None, max_file_size=None,
*/

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_1repair_db(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6plyvel_7_plyvel_repair_db, "repair_db(name, *, paranoid_checks=None, write_buffer_size=None, max_open_files=None, lru_cache_size=None, block_size=None, block_restart_interval=None, max_file_size=None, compression='snappy', int bloom_filter_bits=0, comparator=None, bytes comparator_name=None)");
static PyMethodDef __pyx_mdef_6plyvel_7_plyvel_1repair_db = {"repair_db", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_1repair_db, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_repair_db};
static PyObject *__pyx_pw_6plyvel_7_plyvel_1repair_db(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_name = 0;
  PyObject *__pyx_v_paranoid_checks = 0;
  PyObject *__pyx_v_write_buffer_size = 0;
  PyObject *__pyx_v_max_open_files = 0;
  PyObject *__pyx_v_lru_cache_size = 0;
  PyObject *__pyx_v_block_size = 0;
  PyObject *__pyx_v_block_restart_interval = 0;
  PyObject *__pyx_v_max_file_size = 0;
  PyObject *__pyx_v_compression = 0;
  int __pyx_v_bloom_filter_bits;
  PyObject *__pyx_v_comparator = 0;
  PyObject *__pyx_v_comparator_name = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[12] = {0,0,0,0,0,0,0,0,0,0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("repair_db (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_name,&__pyx_mstate_global->__pyx_n_u_paranoid_checks,&__pyx_mstate_global->__pyx_n_u_write_buffer_size,&__pyx_mstate_global->__pyx_n_u_max_open_files,&__pyx_mstate_global->__pyx_n_u_lru_cache_size,&__pyx_mstate_global->__pyx_n_u_block_size,&__pyx_mstate_global->__pyx_n_u_block_restart_interval,&__pyx_mstate_global->__pyx_n_u_max_file_size,&__pyx_mstate_global->__pyx_n_u_compression,&__pyx_mstate_global->__pyx_n_u_bloom_filter_bits,&__pyx_mstate_global->__pyx_n_u_comparator,&__pyx_mstate_global->__pyx_n_u_comparator_name,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(0, 521, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 521, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "repair_db", 0) < (0)) __PYX_ERR(0, 521, __pyx_L3_error)
      if (!values[1]) values[1] = __Pyx_NewRef(((PyObject *)Py_None));
      if (!values[2]) values[2] = __Pyx_NewRef(((PyObject *)Py_None));

      /* "plyvel/_plyvel.pyx":522
 * 
 * def repair_db(name, *, paranoid_checks=None, write_buffer_size=None,
 *               max_open_files=None, lru_cache_size=None, block_size=None,             # <<<<<<<<<<<<<<
 *               block_restart_interval=None, max_file_size=None,
 *               compression='snappy', int bloom_filter_bits=0, comparator=None,
*/
      if (!values[3]) values[3] = __Pyx_NewRef(((PyObject *)Py_None));
      if (!values[4]) values[4] = __Pyx_NewRef(((PyObject *)Py_None));
      if (!values[5]) values[5] = __Pyx_NewRef(((PyObject *)Py_None));

      /* "plyvel/_plyvel.pyx":523
 * def repair_db(name, *, paranoid_checks=None, write_buffer_size=None,
 *               max_open_files=None, lru_cache_size=None, block_size=None,
 *               block_restart_interval=None, max_file_size=None,             # <<<<<<<<<<<<<<
 *               compression='snappy', int bloom_filter_bits=0, comparator=None,
 *               bytes comparator_name=None):
*/
      if (!values[6]) values[6] = __Pyx_NewRef(((PyObject *)Py_None));
      if (!values[7]) values[7] = __Pyx_NewRef(((PyObject *)Py_None));
      if (!values[8]) values[8] = __Pyx_NewRef(((PyObject *)((PyObject*)__pyx_mstate_global->__pyx_n_u_snappy)));

      /* "plyvel/_plyvel.pyx":524
 *               max_open_files=None, lru_cache_size=None, block_size=None,
 *               block_restart_interval=None, max_file_size=None,
 *               compression='snappy', int bloom_filter_bits=0, comparator=None,             # <<<<<<<<<<<<<<
 *               bytes comparator_name=None):
 *     cdef Options options = Options()
*/
      if (!values[10]) values[10] = __Pyx_NewRef(((PyObject *)Py_None));

      /* "plyvel/_plyvel.pyx":525
 *               block_restart_interval=None, max_file_size=None,
 *               compression='snappy', int bloom_filter_bits=0, comparator=None,
 *               bytes comparator_name=None):             # <<<<<<<<<<<<<<
 *     cdef Options options = Options()
 *     cdef Status st
*/
      if (!values[11]) values[11] = __Pyx_NewRef(((PyObject*)Py_None));
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("repair_db", 1, 1, 1, i); __PYX_ERR(0, 521, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 521, __pyx_L3_error)

      /* "plyvel/_plyvel.pyx":521
 * 
 * 
 * def repair_db(name, *, paranoid_checks=None, write_buffer_size=None,             # <<<<<<<<<<<<<<
 *               max_open_files=None, lru_cache_size=None, block_size=None,
 *               block_restart_interval=None, max_file_size=None,
*/
      if (!values[1]) values[1] = __Pyx_NewRef(((PyObject *)Py_None));
      if (!values[2]) values[2] = __Pyx_NewRef(((PyObject *)Py_None));

      /* "plyvel/_plyvel.pyx":522
 * 
 * def repair_db(name, *, paranoid_checks=None, write_buffer_size=None,
 *               max_open_files=None, lru_cache_size=None, block_size=None,             # <<<<<<<<<<<<<<
 *               block_restart_interval=None, max_file_size=None,
 *               compression='snappy', int bloom_filter_bits=0, comparator=None,
*/
      if (!values[3]) values[3] = __Pyx_NewRef(((PyObject *)Py_None));
      if (!values[4]) values[4] = __Pyx_NewRef(((PyObject *)Py_None));
      if (!values[5]) values[5] = __Pyx_NewRef(((PyObject *)Py_None));

      /* "plyvel/_plyvel.pyx":523
 * def repair_db(name, *, paranoid_checks=None, write_buffer_size=None,
 *               max_open_files=None, lru_cache_size=None, block_size=None,
 *               block_restart_interval=None, max_file_size=None,             # <<<<<<<<<<<<<<
 *               compression='snappy', int bloom_filter_bits=0, comparator=None,
 *               bytes comparator_name=None):
*/
      if (!values[6]) values[6] = __Pyx_NewRef(((PyObject *)Py_None));
      if (!values[7]) values[7] = __Pyx_NewRef(((PyObject *)Py_None));
      if (!values[8]) values[8] = __Pyx_NewRef(((PyObject *)((PyObject*)__pyx_mstate_global->__pyx_n_u_snappy)));

      /* "plyvel/_plyvel.pyx":524
 *               max_open_files=None, lru_cache_size=None, block_size=None,
 *               block_restart_interval=None, max_file_size=None,
 *               compression='snappy', int bloom_filter_bits=0, comparator=None,             # <<<<<<<<<<<<<<
 *               bytes comparator_name=None):
 *     cdef Options options = Options()
*/
      if (!values[10]) values[10] = __Pyx_NewRef(((PyObject *)Py_None));

      /* "plyvel/_plyvel.pyx":525
 *               block_restart_interval=None, max_file_size=None,
 *               compression='snappy', int bloom_filter_bits=0, comparator=None,
 *               bytes comparator_name=None):             # <<<<<<<<<<<<<<
 *     cdef Options options = Options()
 *     cdef Status st
*/
      if (!values[11]) values[11] = __Pyx_NewRef(((PyObject*)Py_None));
    }
    __pyx_v_name = values[0];
    __pyx_v_paranoid_checks = values[1];
    __pyx_v_write_buffer_size = values[2];
    __pyx_v_max_open_files = values[3];
    __pyx_v_lru_cache_size = values[4];
    __pyx_v_block_size = values[5];
    __pyx_v_block_restart_interval = values[6];
    __pyx_v_max_file_size = values[7];
    __pyx_v_compression = values[8];
    if (values[9]) {
      __pyx_v_bloom_filter_bits = __Pyx_PyLong_As_int(values[9]); if (unlikely((__pyx_v_bloom_filter_bits == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 524, __pyx_L3_error)
    } else {
      __pyx_v_bloom_filter_bits = ((int)((int)0));
    }
    __pyx_v_comparator = values[10];
    __pyx_v_comparator_name = ((PyObject*)values[11]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("repair_db", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 521, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("plyvel._plyvel.repair_db", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_comparator_name), (&PyBytes_Type), 1, "comparator_name", 1))) __PYX_ERR(0, 525, __pyx_L1_error)
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_repair_db(__pyx_self, __pyx_v_name, __pyx_v_paranoid_checks, __pyx_v_write_buffer_size, __pyx_v_max_open_files, __pyx_v_lru_cache_size, __pyx_v_block_size, __pyx_v_block_restart_interval, __pyx_v_max_file_size, __pyx_v_compression, __pyx_v_bloom_filter_bits, __pyx_v_comparator, __pyx_v_comparator_name);

  /* "plyvel/_plyvel.pyx":521
 * 
 * 
 * def repair_db(name, *, paranoid_checks=None, write_buffer_size=None,             # <<<<<<<<<<<<<<
 *               max_open_files=None, lru_cache_size=None, block_size=None,
 *               block_restart_interval=None, max_file_size=None,
*/

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_repair_db(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_name, PyObject *__pyx_v_paranoid_checks, PyObject *__pyx_v_write_buffer_size, PyObject *__pyx_v_max_open_files, PyObject *__pyx_v_lru_cache_size, PyObject *__pyx_v_block_size, PyObject *__pyx_v_block_restart_interval, PyObject *__pyx_v_max_file_size, PyObject *__pyx_v_compression, int __pyx_v_bloom_filter_bits, PyObject *__pyx_v_comparator, PyObject *__pyx_v_comparator_name) {
  leveldb::Options __pyx_v_options;
  leveldb::Status __pyx_v_st;
  std::string __pyx_v_fsname;
  int __pyx_v_create_if_missing;
  int __pyx_v_error_if_exists;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  std::string __pyx_t_2;
  int __pyx_t_3;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("repair_db", 0);

  /* "plyvel/_plyvel.pyx":526
 *               compression='snappy', int bloom_filter_bits=0, comparator=None,
 *               bytes comparator_name=None):
 *     cdef Options options = Options()             # <<<<<<<<<<<<<<
 *     cdef Status st
 *     cdef string fsname
*/
  __pyx_v_options = leveldb::Options();

  /* "plyvel/_plyvel.pyx":530
 *     cdef string fsname
 * 
 *     fsname = to_file_system_name(name)             # <<<<<<<<<<<<<<
 *     create_if_missing = False
 *     error_if_exists = True
*/
  __pyx_t_1 = __pyx_f_6plyvel_7_plyvel_to_file_system_name(__pyx_v_name); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 530, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_convert_string_from_py_6libcpp_6string_std__in_string(__pyx_t_1); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 530, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_fsname = __PYX_STD_MOVE_IF_SUPPORTED(__pyx_t_2);

  /* "plyvel/_plyvel.pyx":531
 * 
 *     fsname = to_file_system_name(name)
 *     create_if_missing = False             # <<<<<<<<<<<<<<
 *     error_if_exists = True
 *     parse_options(
*/
  __pyx_v_create_if_missing = 0;

  /* "plyvel/_plyvel.pyx":532
 *     fsname = to_file_system_name(name)
 *     create_if_missing = False
 *     error_if_exists = True             # <<<<<<<<<<<<<<
 *     parse_options(
 *         &options, create_if_missing, error_if_exists, paranoid_checks,
*/
  __pyx_v_error_if_exists = 1;

  /* "plyvel/_plyvel.pyx":533
 *     create_if_missing = False
 *     error_if_exists = True
 *     parse_options(             # <<<<<<<<<<<<<<
 *         &options, create_if_missing, error_if_exists, paranoid_checks,
 *         write_buffer_size, max_open_files, lru_cache_size, block_size,
*/
  __pyx_t_3 = __pyx_f_6plyvel_7_plyvel_parse_options((&__pyx_v_options), __pyx_v_create_if_missing, __pyx_v_error_if_exists, __pyx_v_paranoid_checks, __pyx_v_write_buffer_size, __pyx_v_max_open_files, __pyx_v_lru_cache_size, __pyx_v_block_size, __pyx_v_block_restart_interval, __pyx_v_max_file_size, __pyx_v_compression, __pyx_v_bloom_filter_bits, __pyx_v_comparator, __pyx_v_comparator_name); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(0, 533, __pyx_L1_error)

  /* "plyvel/_plyvel.pyx":538
 *         block_restart_interval, max_file_size, compression, bloom_filter_bits,
 *         comparator, comparator_name)
 *     with nogil:             # <<<<<<<<<<<<<<
 *         st = RepairDB(fsname, options)
 *     raise_for_status(st)
*/
  {
      PyThreadState * _save;
      _save = PyEval_SaveThread();
      __Pyx_FastGIL_Remember();
      /*try:*/ {

        /* "plyvel/_plyvel.pyx":539
 *         comparator, comparator_name)
 *     with nogil:
 *         st = RepairDB(fsname, options)             # <<<<<<<<<<<<<<
 *     raise_for_status(st)
 * 
*/
        __pyx_v_st = leveldb::RepairDB(__pyx_v_fsname, __pyx_v_options);
      }

      /* "plyvel/_plyvel.pyx":538
 *         block_restart_interval, max_file_size, compression, bloom_filter_bits,
 *         comparator, comparator_name)
 *     with nogil:             # <<<<<<<<<<<<<<
 *         st = RepairDB(fsname, options)
 *     raise_for_status(st)
*/
      /*finally:*/ {
        /*normal exit:*/{
          __Pyx_FastGIL_Forget();
          PyEval_RestoreThread(_save);
          goto __pyx_L5;
        }
        __pyx_L5:;
      }
  }

  /* "plyvel/_plyvel.pyx":540
 *     with nogil:
 *         st = RepairDB(fsname, options)
 *     raise_for_status(st)             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __pyx_t_3 = __pyx_f_6plyvel_7_plyvel_raise_for_status(__pyx_v_st); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(0, 540, __pyx_L1_error)

  /* "plyvel/_plyvel.pyx":521
 * 
 * 
 * def repair_db(name, *, paranoid_checks=None, write_buffer_size=None,             # <<<<<<<<<<<<<<
 *               max_open_files=None, lru_cache_size=None, block_size=None,
 *               block_restart_interval=None, max_file_size=None,
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("plyvel._plyvel.repair_db", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":543
 * 
 * 
 * def destroy_db(name):             # <<<<<<<<<<<<<<
 *     cdef Options options = Options()
 *     cdef Status st
*/

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_3destroy_db(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6plyvel_7_plyvel_2destroy_db, "destroy_db(name)");
static PyMethodDef __pyx_mdef_6plyvel_7_plyvel_3destroy_db = {"destroy_db", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_3destroy_db, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_2destroy_db};
static PyObject *__pyx_pw_6plyvel_7_plyvel_3destroy_db(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_name = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("destroy_db (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_name,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(0, 543, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 543, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "destroy_db", 0) < (0)) __PYX_ERR(0, 543, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("destroy_db", 1, 1, 1, i); __PYX_ERR(0, 543, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 543, __pyx_L3_error)
    }
    __pyx_v_name = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("destroy_db", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 543, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("plyvel._plyvel.destroy_db", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_2destroy_db(__pyx_self, __pyx_v_name);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_2destroy_db(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_name) {
  leveldb::Options __pyx_v_options;
  leveldb::Status __pyx_v_st;
  std::string __pyx_v_fsname;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  std::string __pyx_t_2;
  int __pyx_t_3;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("destroy_db", 0);

  /* "plyvel/_plyvel.pyx":544
 * 
 * def destroy_db(name):
 *     cdef Options options = Options()             # <<<<<<<<<<<<<<
 *     cdef Status st
 *     cdef string fsname
*/
  __pyx_v_options = leveldb::Options();

  /* "plyvel/_plyvel.pyx":548
 *     cdef string fsname
 * 
 *     fsname = to_file_system_name(name)             # <<<<<<<<<<<<<<
 *     with nogil:
 *         st = DestroyDB(fsname, options)
*/
  __pyx_t_1 = __pyx_f_6plyvel_7_plyvel_to_file_system_name(__pyx_v_name); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 548, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_convert_string_from_py_6libcpp_6string_std__in_string(__pyx_t_1); if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 548, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_fsname = __PYX_STD_MOVE_IF_SUPPORTED(__pyx_t_2);

  /* "plyvel/_plyvel.pyx":549
 * 
 *     fsname = to_file_system_name(name)
 *     with nogil:             # <<<<<<<<<<<<<<
 *         st = DestroyDB(fsname, options)
 *     raise_for_status(st)
*/
  {
      PyThreadState * _save;
      _save = PyEval_SaveThread();
      __Pyx_FastGIL_Remember();
      /*try:*/ {

        /* "plyvel/_plyvel.pyx":550
 *     fsname = to_file_system_name(name)
 *     with nogil:
 *         st = DestroyDB(fsname, options)             # <<<<<<<<<<<<<<
 *     raise_for_status(st)
 * 
*/
        __pyx_v_st = leveldb::DestroyDB(__pyx_v_fsname, __pyx_v_options);
      }

      /* "plyvel/_plyvel.pyx":549
 * 
 *     fsname = to_file_system_name(name)
 *     with nogil:             # <<<<<<<<<<<<<<
 *         st = DestroyDB(fsname, options)
 *     raise_for_status(st)
*/
      /*finally:*/ {
        /*normal exit:*/{
          __Pyx_FastGIL_Forget();
          PyEval_RestoreThread(_save);
          goto __pyx_L5;
        }
        __pyx_L5:;
      }
  }

  /* "plyvel/_plyvel.pyx":551
 *     with nogil:
 *         st = DestroyDB(fsname, options)
 *     raise_for_status(st)             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __pyx_t_3 = __pyx_f_6plyvel_7_plyvel_raise_for_status(__pyx_v_st); if (unlikely(__pyx_t_3 == ((int)-1))) __PYX_ERR(0, 551, __pyx_L1_error)

  /* "plyvel/_plyvel.pyx":543
 * 
 * 
 * def destroy_db(name):             # <<<<<<<<<<<<<<
 *     cdef Options options = Options()
 *     cdef Status st
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("plyvel._plyvel.destroy_db", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":566
 *     cdef c_bool transaction
 * 
 *     def __init__(self, DB db not None, bytes prefix, bool transaction, sync):             # <<<<<<<<<<<<<<
 *         self.db = db
 *         self.prefix = prefix
*/

/* Python wrapper */
static int __pyx_pw_6plyvel_7_plyvel_10WriteBatch_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_6plyvel_7_plyvel_10WriteBatch_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_db = 0;
  PyObject *__pyx_v_prefix = 0;
  PyLongObject *__pyx_v_transaction = 0;
  PyObject *__pyx_v_sync = 0;
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[4] = {0,0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return -1;
  #endif
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_db,&__pyx_mstate_global->__pyx_n_u_prefix,&__pyx_mstate_global->__pyx_n_u_transaction,&__pyx_mstate_global->__pyx_n_u_sync,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_VARARGS(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(0, 566, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  4:
        values[3] = __Pyx_ArgRef_VARARGS(__pyx_args, 3);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[3])) __PYX_ERR(0, 566, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  3:
        values[2] = __Pyx_ArgRef_VARARGS(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 566, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_VARARGS(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 566, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 566, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__init__", 0) < (0)) __PYX_ERR(0, 566, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 4; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__init__", 1, 4, 4, i); __PYX_ERR(0, 566, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 4)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 566, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_VARARGS(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 566, __pyx_L3_error)
      values[2] = __Pyx_ArgRef_VARARGS(__pyx_args, 2);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 566, __pyx_L3_error)
      values[3] = __Pyx_ArgRef_VARARGS(__pyx_args, 3);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[3])) __PYX_ERR(0, 566, __pyx_L3_error)
    }
    __pyx_v_db = ((struct __pyx_obj_6plyvel_7_plyvel_DB *)values[0]);
    __pyx_v_prefix = ((PyObject*)values[1]);
    __pyx_v_transaction = ((PyLongObject *)values[2]);
    __pyx_v_sync = values[3];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 4, 4, __pyx_nargs); __PYX_ERR(0, 566, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("plyvel._plyvel.WriteBatch.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_db), __pyx_mstate_global->__pyx_ptype_6plyvel_7_plyvel_DB, 0, "db", 0))) __PYX_ERR(0, 566, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_prefix), (&PyBytes_Type), 1, "prefix", 1))) __PYX_ERR(0, 566, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_transaction), __pyx_mstate_global->__pyx_ptype_7cpython_4bool_bool, 1, "transaction", 0))) __PYX_ERR(0, 566, __pyx_L1_error)
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_10WriteBatch___init__(((struct __pyx_obj_6plyvel_7_plyvel_WriteBatch *)__pyx_v_self), __pyx_v_db, __pyx_v_prefix, __pyx_v_transaction, __pyx_v_sync);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = -1;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6plyvel_7_plyvel_10WriteBatch___init__(struct __pyx_obj_6plyvel_7_plyvel_WriteBatch *__pyx_v_self, struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_db, PyObject *__pyx_v_prefix, PyLongObject *__pyx_v_transaction, PyObject *__pyx_v_sync) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  bool __pyx_t_1;
  int __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "plyvel/_plyvel.pyx":567
 * 
 *     def __init__(self, DB db not None, bytes prefix, bool transaction, sync):
 *         self.db = db             # <<<<<<<<<<<<<<
 *         self.prefix = prefix
 *         self.transaction = transaction
*/
  __Pyx_INCREF((PyObject *)__pyx_v_db);
  __Pyx_GIVEREF((PyObject *)__pyx_v_db);
  __Pyx_GOTREF((PyObject *)__pyx_v_self->db);
  __Pyx_DECREF((PyObject *)__pyx_v_self->db);
  __pyx_v_self->db = __pyx_v_db;

  /* "plyvel/_plyvel.pyx":568
 *     def __init__(self, DB db not None, bytes prefix, bool transaction, sync):
 *         self.db = db
 *         self.prefix = prefix             # <<<<<<<<<<<<<<
 *         self.transaction = transaction
 * 
*/
  __Pyx_INCREF(__pyx_v_prefix);
  __Pyx_GIVEREF(__pyx_v_prefix);
  __Pyx_GOTREF(__pyx_v_self->prefix);
  __Pyx_DECREF(__pyx_v_self->prefix);
  __pyx_v_self->prefix = __pyx_v_prefix;

  /* "plyvel/_plyvel.pyx":569
 *         self.db = db
 *         self.prefix = prefix
 *         self.transaction = transaction             # <<<<<<<<<<<<<<
 * 
 *         self.write_options = WriteOptions()
*/
  __pyx_t_1 = __Pyx_PyObject_IsTrue(((PyObject *)__pyx_v_transaction)); if (unlikely((__pyx_t_1 == ((bool)-1)) && PyErr_Occurred())) __PYX_ERR(0, 569, __pyx_L1_error)
  __pyx_v_self->transaction = __pyx_t_1;

  /* "plyvel/_plyvel.pyx":571
 *         self.transaction = transaction
 * 
 *         self.write_options = WriteOptions()             # <<<<<<<<<<<<<<
 *         if sync is not None:
 *             self.write_options.sync = sync
*/
  __pyx_v_self->write_options = leveldb::WriteOptions();

  /* "plyvel/_plyvel.pyx":572
 * 
 *         self.write_options = WriteOptions()
 *         if sync is not None:             # <<<<<<<<<<<<<<
 *             self.write_options.sync = sync
 * 
*/
  __pyx_t_2 = (__pyx_v_sync != Py_None);
  if (__pyx_t_2) {

    /* "plyvel/_plyvel.pyx":573
 *         self.write_options = WriteOptions()
 *         if sync is not None:
 *             self.write_options.sync = sync             # <<<<<<<<<<<<<<
 * 
 *         self._write_batch = new leveldb.WriteBatch()
*/
    __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_sync); if (unlikely((__pyx_t_1 == ((bool)-1)) && PyErr_Occurred())) __PYX_ERR(0, 573, __pyx_L1_error)
    __pyx_v_self->write_options.sync = __pyx_t_1;

    /* "plyvel/_plyvel.pyx":572
 * 
 *         self.write_options = WriteOptions()
 *         if sync is not None:             # <<<<<<<<<<<<<<
 *             self.write_options.sync = sync
 * 
*/
  }

  /* "plyvel/_plyvel.pyx":575
 *             self.write_options.sync = sync
 * 
 *         self._write_batch = new leveldb.WriteBatch()             # <<<<<<<<<<<<<<
 * 
 *     def __dealloc__(self):
*/
  __pyx_v_self->_write_batch = new leveldb::WriteBatch();

  /* "plyvel/_plyvel.pyx":566
 *     cdef c_bool transaction
 * 
 *     def __init__(self, DB db not None, bytes prefix, bool transaction, sync):             # <<<<<<<<<<<<<<
 *         self.db = db
 *         self.prefix = prefix
*/

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("plyvel._plyvel.WriteBatch.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":577
 *         self._write_batch = new leveldb.WriteBatch()
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         del self._write_batch
 * 
*/

/* Python wrapper */
static void __pyx_pw_6plyvel_7_plyvel_10WriteBatch_3__dealloc__(PyObject *__pyx_v_self); /*proto*/
static void __pyx_pw_6plyvel_7_plyvel_10WriteBatch_3__dealloc__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__dealloc__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_pf_6plyvel_7_plyvel_10WriteBatch_2__dealloc__(((struct __pyx_obj_6plyvel_7_plyvel_WriteBatch *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

static void __pyx_pf_6plyvel_7_plyvel_10WriteBatch_2__dealloc__(struct __pyx_obj_6plyvel_7_plyvel_WriteBatch *__pyx_v_self) {

  /* "plyvel/_plyvel.pyx":578
 * 
 *     def __dealloc__(self):
 *         del self._write_batch             # <<<<<<<<<<<<<<
 * 
 *     def put(self, bytes key not None, value not None):
*/
  delete __pyx_v_self->_write_batch;

  /* "plyvel/_plyvel.pyx":577
 *         self._write_batch = new leveldb.WriteBatch()
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         del self._write_batch
 * 
*/

  /* function exit code */
}

/* "plyvel/_plyvel.pyx":580
 *         del self._write_batch
 * 
 *     def put(self, bytes key not None, value not None):             # <<<<<<<<<<<<<<
 *         if self.db._db is NULL:
 *             raise RuntimeError("Database is closed")
*/

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_10WriteBatch_5put(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6plyvel_7_plyvel_10WriteBatch_4put, "WriteBatch.put(self, bytes key, value)");
static PyMethodDef __pyx_mdef_6plyvel_7_plyvel_10WriteBatch_5put = {"put", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_10WriteBatch_5put, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_10WriteBatch_4put};
static PyObject *__pyx_pw_6plyvel_7_plyvel_10WriteBatch_5put(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_key = 0;
  PyObject *__pyx_v_value = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("put (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_key,&__pyx_mstate_global->__pyx_n_u_value,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(0, 580, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 580, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 580, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "put", 0) < (0)) __PYX_ERR(0, 580, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 2; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("put", 1, 2, 2, i); __PYX_ERR(0, 580, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 2)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 580, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 580, __pyx_L3_error)
    }
    __pyx_v_key = ((PyObject*)values[0]);
    __pyx_v_value = values[1];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("put", 1, 2, 2, __pyx_nargs); __PYX_ERR(0, 580, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("plyvel._plyvel.WriteBatch.put", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_key), (&PyBytes_Type), 0, "key", 1))) __PYX_ERR(0, 580, __pyx_L1_error)
  if (unlikely(((PyObject *)__pyx_v_value) == Py_None)) {
    PyErr_Format(PyExc_TypeError, "Argument '%.200s' must not be None", "value"); __PYX_ERR(0, 580, __pyx_L1_error)
  }
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_10WriteBatch_4put(((struct __pyx_obj_6plyvel_7_plyvel_WriteBatch *)__pyx_v_self), __pyx_v_key, __pyx_v_value);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_10WriteBatch_4put(struct __pyx_obj_6plyvel_7_plyvel_WriteBatch *__pyx_v_self, PyObject *__pyx_v_key, PyObject *__pyx_v_value) {
  leveldb::Slice __pyx_v_key_slice;
  Py_buffer __pyx_v_value_buffer;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  const char *__pyx_t_5;
  Py_ssize_t __pyx_t_6;
  int __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("put", 0);
  __Pyx_INCREF(__pyx_v_key);

  /* "plyvel/_plyvel.pyx":581
 * 
 *     def put(self, bytes key not None, value not None):
 *         if self.db._db is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database is closed")
 * 
*/
  __pyx_t_1 = (__pyx_v_self->db->_db == NULL);
  if (unlikely(__pyx_t_1)) {

    /* "plyvel/_plyvel.pyx":582
 *     def put(self, bytes key not None, value not None):
 *         if self.db._db is NULL:
 *             raise RuntimeError("Database is closed")             # <<<<<<<<<<<<<<
 * 
 *         if self.prefix is not None:
*/
    __pyx_t_3 = NULL;
    __pyx_t_4 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_mstate_global->__pyx_kp_u_Database_is_closed};
      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_RuntimeError)), __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 582, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 582, __pyx_L1_error)

    /* "plyvel/_plyvel.pyx":581
 * 
 *     def put(self, bytes key not None, value not None):
 *         if self.db._db is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database is closed")
 * 
*/
  }

  /* "plyvel/_plyvel.pyx":584
 *             raise RuntimeError("Database is closed")
 * 
 *         if self.prefix is not None:             # <<<<<<<<<<<<<<
 *             key = self.prefix + key
 * 
*/
  __pyx_t_1 = (__pyx_v_self->prefix != ((PyObject*)Py_None));
  if (__pyx_t_1) {

    /* "plyvel/_plyvel.pyx":585
 * 
 *         if self.prefix is not None:
 *             key = self.prefix + key             # <<<<<<<<<<<<<<
 * 
 *         cdef Slice key_slice = Slice(key, len(key))
*/
    __pyx_t_2 = PyNumber_Add(__pyx_v_self->prefix, __pyx_v_key); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 585, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF_SET(__pyx_v_key, ((PyObject*)__pyx_t_2));
    __pyx_t_2 = 0;

    /* "plyvel/_plyvel.pyx":584
 *             raise RuntimeError("Database is closed")
 * 
 *         if self.prefix is not None:             # <<<<<<<<<<<<<<
 *             key = self.prefix + key
 * 
*/
  }

  /* "plyvel/_plyvel.pyx":587
 *             key = self.prefix + key
 * 
 *         cdef Slice key_slice = Slice(key, len(key))             # <<<<<<<<<<<<<<
 *         cdef Py_buffer value_buffer
 *         PyObject_GetBuffer(value, &value_buffer, PyBUF_SIMPLE)
*/
  __pyx_t_5 = __Pyx_PyBytes_AsString(__pyx_v_key); if (unlikely((!__pyx_t_5) && PyErr_Occurred())) __PYX_ERR(0, 587, __pyx_L1_error)
  __pyx_t_6 = __Pyx_PyBytes_GET_SIZE(__pyx_v_key); if (unlikely(__pyx_t_6 == ((Py_ssize_t)-1))) __PYX_ERR(0, 587, __pyx_L1_error)
  __pyx_v_key_slice = leveldb::Slice(__pyx_t_5, __pyx_t_6);

  /* "plyvel/_plyvel.pyx":589
 *         cdef Slice key_slice = Slice(key, len(key))
 *         cdef Py_buffer value_buffer
 *         PyObject_GetBuffer(value, &value_buffer, PyBUF_SIMPLE)             # <<<<<<<<<<<<<<
 *         try:
 *             with nogil:
*/
  __pyx_t_7 = PyObject_GetBuffer(__pyx_v_value, (&__pyx_v_value_buffer), PyBUF_SIMPLE); if (unlikely(__pyx_t_7 == ((int)-1))) __PYX_ERR(0, 589, __pyx_L1_error)

  /* "plyvel/_plyvel.pyx":590
 *         cdef Py_buffer value_buffer
 *         PyObject_GetBuffer(value, &value_buffer, PyBUF_SIMPLE)
 *         try:             # <<<<<<<<<<<<<<
 *             with nogil:
 *                 self._write_batch.Put(
*/
  /*try:*/ {

    /* "plyvel/_plyvel.pyx":591
 *         PyObject_GetBuffer(value, &value_buffer, PyBUF_SIMPLE)
 *         try:
 *             with nogil:             # <<<<<<<<<<<<<<
 *                 self._write_batch.Put(
 *                     key_slice,
*/
    {
        PyThreadState * _save;
        _save = PyEval_SaveThread();
        __Pyx_FastGIL_Remember();
        /*try:*/ {

          /* "plyvel/_plyvel.pyx":592
 *         try:
 *             with nogil:
 *                 self._write_batch.Put(             # <<<<<<<<<<<<<<
 *                     key_slice,
 *                     Slice(<const_char *>value_buffer.buf, value_buffer.len))
*/
          __pyx_v_self->_write_batch->Put(__pyx_v_key_slice, leveldb::Slice(((const char *)__pyx_v_value_buffer.buf), __pyx_v_value_buffer.len));
        }

        /* "plyvel/_plyvel.pyx":591
 *         PyObject_GetBuffer(value, &value_buffer, PyBUF_SIMPLE)
 *         try:
 *             with nogil:             # <<<<<<<<<<<<<<
 *                 self._write_batch.Put(
 *                     key_slice,
*/
        /*finally:*/ {
          /*normal exit:*/{
            __Pyx_FastGIL_Forget();
            PyEval_RestoreThread(_save);
            goto __pyx_L10;
          }
          __pyx_L10:;
        }
    }
  }

  /* "plyvel/_plyvel.pyx":596
 *                     Slice(<const_char *>value_buffer.buf, value_buffer.len))
 *         finally:
 *             PyBuffer_Release(&value_buffer)             # <<<<<<<<<<<<<<
 * 
 *     def delete(self, bytes key not None):
*/
  /*finally:*/ {
    /*normal exit:*/{
      PyBuffer_Release((&__pyx_v_value_buffer));
      goto __pyx_L7;
    }
    __pyx_L7:;
  }

  /* "plyvel/_plyvel.pyx":580
 *         del self._write_batch
 * 
 *     def put(self, bytes key not None, value not None):             # <<<<<<<<<<<<<<
 *         if self.db._db is NULL:
 *             raise RuntimeError("Database is closed")
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("plyvel._plyvel.WriteBatch.put", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_key);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":598
 *             PyBuffer_Release(&value_buffer)
 * 
 *     def delete(self, bytes key not None):             # <<<<<<<<<<<<<<
 *         if self.db._db is NULL:
 *             raise RuntimeError("Database is closed")
*/

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_10WriteBatch_7delete(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6plyvel_7_plyvel_10WriteBatch_6delete, "WriteBatch.delete(self, bytes key)");
static PyMethodDef __pyx_mdef_6plyvel_7_plyvel_10WriteBatch_7delete = {"delete", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_10WriteBatch_7delete, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_10WriteBatch_6delete};
static PyObject *__pyx_pw_6plyvel_7_plyvel_10WriteBatch_7delete(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_key = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("delete (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_key,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(0, 598, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 598, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "delete", 0) < (0)) __PYX_ERR(0, 598, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("delete", 1, 1, 1, i); __PYX_ERR(0, 598, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 598, __pyx_L3_error)
    }
    __pyx_v_key = ((PyObject*)values[0]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("delete", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 598, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("plyvel._plyvel.WriteBatch.delete", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_key), (&PyBytes_Type), 0, "key", 1))) __PYX_ERR(0, 598, __pyx_L1_error)
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_10WriteBatch_6delete(((struct __pyx_obj_6plyvel_7_plyvel_WriteBatch *)__pyx_v_self), __pyx_v_key);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_10WriteBatch_6delete(struct __pyx_obj_6plyvel_7_plyvel_WriteBatch *__pyx_v_self, PyObject *__pyx_v_key) {
  leveldb::Slice __pyx_v_key_slice;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  const char *__pyx_t_5;
  Py_ssize_t __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("delete", 0);
  __Pyx_INCREF(__pyx_v_key);

  /* "plyvel/_plyvel.pyx":599
 * 
 *     def delete(self, bytes key not None):
 *         if self.db._db is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database is closed")
 * 
*/
  __pyx_t_1 = (__pyx_v_self->db->_db == NULL);
  if (unlikely(__pyx_t_1)) {

    /* "plyvel/_plyvel.pyx":600
 *     def delete(self, bytes key not None):
 *         if self.db._db is NULL:
 *             raise RuntimeError("Database is closed")             # <<<<<<<<<<<<<<
 * 
 *         if self.prefix is not None:
*/
    __pyx_t_3 = NULL;
    __pyx_t_4 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_mstate_global->__pyx_kp_u_Database_is_closed};
      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_RuntimeError)), __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 600, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 600, __pyx_L1_error)

    /* "plyvel/_plyvel.pyx":599
 * 
 *     def delete(self, bytes key not None):
 *         if self.db._db is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database is closed")
 * 
*/
  }

  /* "plyvel/_plyvel.pyx":602
 *             raise RuntimeError("Database is closed")
 * 
 *         if self.prefix is not None:             # <<<<<<<<<<<<<<
 *             key = self.prefix + key
 * 
*/
  __pyx_t_1 = (__pyx_v_self->prefix != ((PyObject*)Py_None));
  if (__pyx_t_1) {

    /* "plyvel/_plyvel.pyx":603
 * 
 *         if self.prefix is not None:
 *             key = self.prefix + key             # <<<<<<<<<<<<<<
 * 
 *         cdef Slice key_slice = Slice(key, len(key))
*/
    __pyx_t_2 = PyNumber_Add(__pyx_v_self->prefix, __pyx_v_key); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 603, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF_SET(__pyx_v_key, ((PyObject*)__pyx_t_2));
    __pyx_t_2 = 0;

    /* "plyvel/_plyvel.pyx":602
 *             raise RuntimeError("Database is closed")
 * 
 *         if self.prefix is not None:             # <<<<<<<<<<<<<<
 *             key = self.prefix + key
 * 
*/
  }

  /* "plyvel/_plyvel.pyx":605
 *             key = self.prefix + key
 * 
 *         cdef Slice key_slice = Slice(key, len(key))             # <<<<<<<<<<<<<<
 *         with nogil:
 *             self._write_batch.Delete(key_slice)
*/
  __pyx_t_5 = __Pyx_PyBytes_AsString(__pyx_v_key); if (unlikely((!__pyx_t_5) && PyErr_Occurred())) __PYX_ERR(0, 605, __pyx_L1_error)
  __pyx_t_6 = __Pyx_PyBytes_GET_SIZE(__pyx_v_key); if (unlikely(__pyx_t_6 == ((Py_ssize_t)-1))) __PYX_ERR(0, 605, __pyx_L1_error)
  __pyx_v_key_slice = leveldb::Slice(__pyx_t_5, __pyx_t_6);

  /* "plyvel/_plyvel.pyx":606
 * 
 *         cdef Slice key_slice = Slice(key, len(key))
 *         with nogil:             # <<<<<<<<<<<<<<
 *             self._write_batch.Delete(key_slice)
 * 
*/
  {
      PyThreadState * _save;
      _save = PyEval_SaveThread();
      __Pyx_FastGIL_Remember();
      /*try:*/ {

        /* "plyvel/_plyvel.pyx":607
 *         cdef Slice key_slice = Slice(key, len(key))
 *         with nogil:
 *             self._write_batch.Delete(key_slice)             # <<<<<<<<<<<<<<
 * 
 *     def clear(self):
*/
        __pyx_v_self->_write_batch->Delete(__pyx_v_key_slice);
      }

      /* "plyvel/_plyvel.pyx":606
 * 
 *         cdef Slice key_slice = Slice(key, len(key))
 *         with nogil:             # <<<<<<<<<<<<<<
 *             self._write_batch.Delete(key_slice)
 * 
*/
      /*finally:*/ {
        /*normal exit:*/{
          __Pyx_FastGIL_Forget();
          PyEval_RestoreThread(_save);
          goto __pyx_L7;
        }
        __pyx_L7:;
      }
  }

  /* "plyvel/_plyvel.pyx":598
 *             PyBuffer_Release(&value_buffer)
 * 
 *     def delete(self, bytes key not None):             # <<<<<<<<<<<<<<
 *         if self.db._db is NULL:
 *             raise RuntimeError("Database is closed")
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("plyvel._plyvel.WriteBatch.delete", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_key);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":609
 *             self._write_batch.Delete(key_slice)
 * 
 *     def clear(self):             # <<<<<<<<<<<<<<
 *         if self.db._db is NULL:
 *             raise RuntimeError("Database is closed")
*/

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_10WriteBatch_9clear(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6plyvel_7_plyvel_10WriteBatch_8clear, "WriteBatch.clear(self)");
static PyMethodDef __pyx_mdef_6plyvel_7_plyvel_10WriteBatch_9clear = {"clear", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_10WriteBatch_9clear, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_10WriteBatch_8clear};
static PyObject *__pyx_pw_6plyvel_7_plyvel_10WriteBatch_9clear(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("clear (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("clear", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("clear", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_10WriteBatch_8clear(((struct __pyx_obj_6plyvel_7_plyvel_WriteBatch *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_10WriteBatch_8clear(struct __pyx_obj_6plyvel_7_plyvel_WriteBatch *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("clear", 0);

  /* "plyvel/_plyvel.pyx":610
 * 
 *     def clear(self):
 *         if self.db._db is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database is closed")
 * 
*/
  __pyx_t_1 = (__pyx_v_self->db->_db == NULL);
  if (unlikely(__pyx_t_1)) {

    /* "plyvel/_plyvel.pyx":611
 *     def clear(self):
 *         if self.db._db is NULL:
 *             raise RuntimeError("Database is closed")             # <<<<<<<<<<<<<<
 * 
 *         with nogil:
*/
    __pyx_t_3 = NULL;
    __pyx_t_4 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_mstate_global->__pyx_kp_u_Database_is_closed};
      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_RuntimeError)), __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 611, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 611, __pyx_L1_error)

    /* "plyvel/_plyvel.pyx":610
 * 
 *     def clear(self):
 *         if self.db._db is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database is closed")
 * 
*/
  }

  /* "plyvel/_plyvel.pyx":613
 *             raise RuntimeError("Database is closed")
 * 
 *         with nogil:             # <<<<<<<<<<<<<<
 *             self._write_batch.Clear()
 * 
*/
  {
      PyThreadState * _save;
      _save = PyEval_SaveThread();
      __Pyx_FastGIL_Remember();
      /*try:*/ {

        /* "plyvel/_plyvel.pyx":614
 * 
 *         with nogil:
 *             self._write_batch.Clear()             # <<<<<<<<<<<<<<
 * 
 *     def write(self):
*/
        __pyx_v_self->_write_batch->Clear();
      }

      /* "plyvel/_plyvel.pyx":613
 *             raise RuntimeError("Database is closed")
 * 
 *         with nogil:             # <<<<<<<<<<<<<<
 *             self._write_batch.Clear()
 * 
*/
      /*finally:*/ {
        /*normal exit:*/{
          __Pyx_FastGIL_Forget();
          PyEval_RestoreThread(_save);
          goto __pyx_L6;
        }
        __pyx_L6:;
      }
  }

  /* "plyvel/_plyvel.pyx":609
 *             self._write_batch.Delete(key_slice)
 * 
 *     def clear(self):             # <<<<<<<<<<<<<<
 *         if self.db._db is NULL:
 *             raise RuntimeError("Database is closed")
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("plyvel._plyvel.WriteBatch.clear", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":616
 *             self._write_batch.Clear()
 * 
 *     def write(self):             # <<<<<<<<<<<<<<
 *         if self.db._db is NULL:
 *             raise RuntimeError("Database is closed")
*/

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_10WriteBatch_11write(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6plyvel_7_plyvel_10WriteBatch_10write, "WriteBatch.write(self)");
static PyMethodDef __pyx_mdef_6plyvel_7_plyvel_10WriteBatch_11write = {"write", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_10WriteBatch_11write, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_10WriteBatch_10write};
static PyObject *__pyx_pw_6plyvel_7_plyvel_10WriteBatch_11write(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("write (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("write", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("write", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_10WriteBatch_10write(((struct __pyx_obj_6plyvel_7_plyvel_WriteBatch *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_10WriteBatch_10write(struct __pyx_obj_6plyvel_7_plyvel_WriteBatch *__pyx_v_self) {
  leveldb::Status __pyx_v_st;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  int __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("write", 0);

  /* "plyvel/_plyvel.pyx":617
 * 
 *     def write(self):
 *         if self.db._db is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database is closed")
 * 
*/
  __pyx_t_1 = (__pyx_v_self->db->_db == NULL);
  if (unlikely(__pyx_t_1)) {

    /* "plyvel/_plyvel.pyx":618
 *     def write(self):
 *         if self.db._db is NULL:
 *             raise RuntimeError("Database is closed")             # <<<<<<<<<<<<<<
 * 
 *         cdef Status st
*/
    __pyx_t_3 = NULL;
    __pyx_t_4 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_mstate_global->__pyx_kp_u_Database_is_closed};
      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_RuntimeError)), __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 618, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 618, __pyx_L1_error)

    /* "plyvel/_plyvel.pyx":617
 * 
 *     def write(self):
 *         if self.db._db is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database is closed")
 * 
*/
  }

  /* "plyvel/_plyvel.pyx":621
 * 
 *         cdef Status st
 *         with nogil:             # <<<<<<<<<<<<<<
 *             st = self.db._db.Write(self.write_options, self._write_batch)
 *         raise_for_status(st)
*/
  {
      PyThreadState * _save;
      _save = PyEval_SaveThread();
      __Pyx_FastGIL_Remember();
      /*try:*/ {

        /* "plyvel/_plyvel.pyx":622
 *         cdef Status st
 *         with nogil:
 *             st = self.db._db.Write(self.write_options, self._write_batch)             # <<<<<<<<<<<<<<
 *         raise_for_status(st)
 * 
*/
        __pyx_v_st = __pyx_v_self->db->_db->Write(__pyx_v_self->write_options, __pyx_v_self->_write_batch);
      }

      /* "plyvel/_plyvel.pyx":621
 * 
 *         cdef Status st
 *         with nogil:             # <<<<<<<<<<<<<<
 *             st = self.db._db.Write(self.write_options, self._write_batch)
 *         raise_for_status(st)
*/
      /*finally:*/ {
        /*normal exit:*/{
          __Pyx_FastGIL_Forget();
          PyEval_RestoreThread(_save);
          goto __pyx_L6;
        }
        __pyx_L6:;
      }
  }

  /* "plyvel/_plyvel.pyx":623
 *         with nogil:
 *             st = self.db._db.Write(self.write_options, self._write_batch)
 *         raise_for_status(st)             # <<<<<<<<<<<<<<
 * 
 *     def approximate_size(self):
*/
  __pyx_t_5 = __pyx_f_6plyvel_7_plyvel_raise_for_status(__pyx_v_st); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(0, 623, __pyx_L1_error)

  /* "plyvel/_plyvel.pyx":616
 *             self._write_batch.Clear()
 * 
 *     def write(self):             # <<<<<<<<<<<<<<
 *         if self.db._db is NULL:
 *             raise RuntimeError("Database is closed")
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("plyvel._plyvel.WriteBatch.write", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":625
 *         raise_for_status(st)
 * 
 *     def approximate_size(self):             # <<<<<<<<<<<<<<
 *         if self.db._db is NULL:
 *             raise RuntimeError("Database is closed")
*/

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_10WriteBatch_13approximate_size(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6plyvel_7_plyvel_10WriteBatch_12approximate_size, "WriteBatch.approximate_size(self)");
static PyMethodDef __pyx_mdef_6plyvel_7_plyvel_10WriteBatch_13approximate_size = {"approximate_size", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_10WriteBatch_13approximate_size, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_10WriteBatch_12approximate_size};
static PyObject *__pyx_pw_6plyvel_7_plyvel_10WriteBatch_13approximate_size(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("approximate_size (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("approximate_size", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("approximate_size", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_10WriteBatch_12approximate_size(((struct __pyx_obj_6plyvel_7_plyvel_WriteBatch *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_10WriteBatch_12approximate_size(struct __pyx_obj_6plyvel_7_plyvel_WriteBatch *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("approximate_size", 0);

  /* "plyvel/_plyvel.pyx":626
 * 
 *     def approximate_size(self):
 *         if self.db._db is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database is closed")
 * 
*/
  __pyx_t_1 = (__pyx_v_self->db->_db == NULL);
  if (unlikely(__pyx_t_1)) {

    /* "plyvel/_plyvel.pyx":627
 *     def approximate_size(self):
 *         if self.db._db is NULL:
 *             raise RuntimeError("Database is closed")             # <<<<<<<<<<<<<<
 * 
 *         return self._write_batch.ApproximateSize()
*/
    __pyx_t_3 = NULL;
    __pyx_t_4 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_mstate_global->__pyx_kp_u_Database_is_closed};
      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_RuntimeError)), __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 627, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 627, __pyx_L1_error)

    /* "plyvel/_plyvel.pyx":626
 * 
 *     def approximate_size(self):
 *         if self.db._db is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database is closed")
 * 
*/
  }

  /* "plyvel/_plyvel.pyx":629
 *             raise RuntimeError("Database is closed")
 * 
 *         return self._write_batch.ApproximateSize()             # <<<<<<<<<<<<<<
 * 
 *     def append(self, WriteBatch source not None):
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyLong_FromSize_t(__pyx_v_self->_write_batch->ApproximateSize()); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 629, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "plyvel/_plyvel.pyx":625
 *         raise_for_status(st)
 * 
 *     def approximate_size(self):             # <<<<<<<<<<<<<<
 *         if self.db._db is NULL:
 *             raise RuntimeError("Database is closed")
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("plyvel._plyvel.WriteBatch.approximate_size", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":631
 *         return self._write_batch.ApproximateSize()
 * 
 *     def append(self, WriteBatch source not None):             # <<<<<<<<<<<<<<
 *         if self.db._db is NULL:
 *             raise RuntimeError("Database is closed")
*/

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_10WriteBatch_15append(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6plyvel_7_plyvel_10WriteBatch_14append, "WriteBatch.append(self, WriteBatch source)");
static PyMethodDef __pyx_mdef_6plyvel_7_plyvel_10WriteBatch_15append = {"append", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_10WriteBatch_15append, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_10WriteBatch_14append};
static PyObject *__pyx_pw_6plyvel_7_plyvel_10WriteBatch_15append(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  struct __pyx_obj_6plyvel_7_plyvel_WriteBatch *__pyx_v_source = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("append (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_source,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(0, 631, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 631, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "append", 0) < (0)) __PYX_ERR(0, 631, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("append", 1, 1, 1, i); __PYX_ERR(0, 631, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 631, __pyx_L3_error)
    }
    __pyx_v_source = ((struct __pyx_obj_6plyvel_7_plyvel_WriteBatch *)values[0]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("append", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 631, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("plyvel._plyvel.WriteBatch.append", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_source), __pyx_mstate_global->__pyx_ptype_6plyvel_7_plyvel_WriteBatch, 0, "source", 0))) __PYX_ERR(0, 631, __pyx_L1_error)
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_10WriteBatch_14append(((struct __pyx_obj_6plyvel_7_plyvel_WriteBatch *)__pyx_v_self), __pyx_v_source);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_10WriteBatch_14append(struct __pyx_obj_6plyvel_7_plyvel_WriteBatch *__pyx_v_self, struct __pyx_obj_6plyvel_7_plyvel_WriteBatch *__pyx_v_source) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("append", 0);

  /* "plyvel/_plyvel.pyx":632
 * 
 *     def append(self, WriteBatch source not None):
 *         if self.db._db is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database is closed")
 * 
*/
  __pyx_t_1 = (__pyx_v_self->db->_db == NULL);
  if (unlikely(__pyx_t_1)) {

    /* "plyvel/_plyvel.pyx":633
 *     def append(self, WriteBatch source not None):
 *         if self.db._db is NULL:
 *             raise RuntimeError("Database is closed")             # <<<<<<<<<<<<<<
 * 
 *         self._write_batch.Append(source._write_batch[0])
*/
    __pyx_t_3 = NULL;
    __pyx_t_4 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_mstate_global->__pyx_kp_u_Database_is_closed};
      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_RuntimeError)), __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 633, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 633, __pyx_L1_error)

    /* "plyvel/_plyvel.pyx":632
 * 
 *     def append(self, WriteBatch source not None):
 *         if self.db._db is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database is closed")
 * 
*/
  }

  /* "plyvel/_plyvel.pyx":635
 *             raise RuntimeError("Database is closed")
 * 
 *         self._write_batch.Append(source._write_batch[0])             # <<<<<<<<<<<<<<
 * 
 *     def __enter__(self):
*/
  __pyx_v_self->_write_batch->Append((__pyx_v_source->_write_batch[0]));

  /* "plyvel/_plyvel.pyx":631
 *         return self._write_batch.ApproximateSize()
 * 
 *     def append(self, WriteBatch source not None):             # <<<<<<<<<<<<<<
 *         if self.db._db is NULL:
 *             raise RuntimeError("Database is closed")
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("plyvel._plyvel.WriteBatch.append", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":637
 *         self._write_batch.Append(source._write_batch[0])
 * 
 *     def __enter__(self):             # <<<<<<<<<<<<<<
 *         if self.db._db is NULL:
 *             raise RuntimeError("Database is closed")
*/

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_10WriteBatch_17__enter__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6plyvel_7_plyvel_10WriteBatch_16__enter__, "WriteBatch.__enter__(self)");
static PyMethodDef __pyx_mdef_6plyvel_7_plyvel_10WriteBatch_17__enter__ = {"__enter__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_10WriteBatch_17__enter__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_10WriteBatch_16__enter__};
static PyObject *__pyx_pw_6plyvel_7_plyvel_10WriteBatch_17__enter__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__enter__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("__enter__", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("__enter__", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_10WriteBatch_16__enter__(((struct __pyx_obj_6plyvel_7_plyvel_WriteBatch *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_10WriteBatch_16__enter__(struct __pyx_obj_6plyvel_7_plyvel_WriteBatch *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__enter__", 0);

  /* "plyvel/_plyvel.pyx":638
 * 
 *     def __enter__(self):
 *         if self.db._db is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database is closed")
 * 
*/
  __pyx_t_1 = (__pyx_v_self->db->_db == NULL);
  if (unlikely(__pyx_t_1)) {

    /* "plyvel/_plyvel.pyx":639
 *     def __enter__(self):
 *         if self.db._db is NULL:
 *             raise RuntimeError("Database is closed")             # <<<<<<<<<<<<<<
 * 
 *         return self
*/
    __pyx_t_3 = NULL;
    __pyx_t_4 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_mstate_global->__pyx_kp_u_Database_is_closed};
      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_RuntimeError)), __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 639, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 639, __pyx_L1_error)

    /* "plyvel/_plyvel.pyx":638
 * 
 *     def __enter__(self):
 *         if self.db._db is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database is closed")
 * 
*/
  }

  /* "plyvel/_plyvel.pyx":641
 *             raise RuntimeError("Database is closed")
 * 
 *         return self             # <<<<<<<<<<<<<<
 * 
 *     def __exit__(self, exc_type, exc_val, exc_tb):
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF((PyObject *)__pyx_v_self);
  __pyx_r = ((PyObject *)__pyx_v_self);
  goto __pyx_L0;

  /* "plyvel/_plyvel.pyx":637
 *         self._write_batch.Append(source._write_batch[0])
 * 
 *     def __enter__(self):             # <<<<<<<<<<<<<<
 *         if self.db._db is NULL:
 *             raise RuntimeError("Database is closed")
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("plyvel._plyvel.WriteBatch.__enter__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":643
 *         return self
 * 
 *     def __exit__(self, exc_type, exc_val, exc_tb):             # <<<<<<<<<<<<<<
 *         if self.db._db is NULL:
 *             raise RuntimeError("Database is closed")
*/

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_10WriteBatch_19__exit__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6plyvel_7_plyvel_10WriteBatch_18__exit__, "WriteBatch.__exit__(self, exc_type, exc_val, exc_tb)");
static PyMethodDef __pyx_mdef_6plyvel_7_plyvel_10WriteBatch_19__exit__ = {"__exit__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_10WriteBatch_19__exit__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_10WriteBatch_18__exit__};
static PyObject *__pyx_pw_6plyvel_7_plyvel_10WriteBatch_19__exit__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_exc_type = 0;
  CYTHON_UNUSED PyObject *__pyx_v_exc_val = 0;
  CYTHON_UNUSED PyObject *__pyx_v_exc_tb = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[3] = {0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__exit__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_exc_type,&__pyx_mstate_global->__pyx_n_u_exc_val,&__pyx_mstate_global->__pyx_n_u_exc_tb,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(0, 643, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 643, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 643, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 643, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__exit__", 0) < (0)) __PYX_ERR(0, 643, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 3; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__exit__", 1, 3, 3, i); __PYX_ERR(0, 643, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 3)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 643, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 643, __pyx_L3_error)
      values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 643, __pyx_L3_error)
    }
    __pyx_v_exc_type = values[0];
    __pyx_v_exc_val = values[1];
    __pyx_v_exc_tb = values[2];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__exit__", 1, 3, 3, __pyx_nargs); __PYX_ERR(0, 643, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("plyvel._plyvel.WriteBatch.__exit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_10WriteBatch_18__exit__(((struct __pyx_obj_6plyvel_7_plyvel_WriteBatch *)__pyx_v_self), __pyx_v_exc_type, __pyx_v_exc_val, __pyx_v_exc_tb);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_10WriteBatch_18__exit__(struct __pyx_obj_6plyvel_7_plyvel_WriteBatch *__pyx_v_self, PyObject *__pyx_v_exc_type, CYTHON_UNUSED PyObject *__pyx_v_exc_val, CYTHON_UNUSED PyObject *__pyx_v_exc_tb) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  int __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__exit__", 0);

  /* "plyvel/_plyvel.pyx":644
 * 
 *     def __exit__(self, exc_type, exc_val, exc_tb):
 *         if self.db._db is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database is closed")
 * 
*/
  __pyx_t_1 = (__pyx_v_self->db->_db == NULL);
  if (unlikely(__pyx_t_1)) {

    /* "plyvel/_plyvel.pyx":645
 *     def __exit__(self, exc_type, exc_val, exc_tb):
 *         if self.db._db is NULL:
 *             raise RuntimeError("Database is closed")             # <<<<<<<<<<<<<<
 * 
 *         if self.transaction and exc_type is not None:
*/
    __pyx_t_3 = NULL;
    __pyx_t_4 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_mstate_global->__pyx_kp_u_Database_is_closed};
      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_RuntimeError)), __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 645, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 645, __pyx_L1_error)

    /* "plyvel/_plyvel.pyx":644
 * 
 *     def __exit__(self, exc_type, exc_val, exc_tb):
 *         if self.db._db is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database is closed")
 * 
*/
  }

  /* "plyvel/_plyvel.pyx":647
 *             raise RuntimeError("Database is closed")
 * 
 *         if self.transaction and exc_type is not None:             # <<<<<<<<<<<<<<
 *             # Exception occurred in transaction; do not write the batch
 *             self.clear()
*/
  __pyx_t_5 = (__pyx_v_self->transaction != 0);
  if (__pyx_t_5) {
  } else {
    __pyx_t_1 = __pyx_t_5;
    goto __pyx_L5_bool_binop_done;
  }
  __pyx_t_5 = (__pyx_v_exc_type != Py_None);
  __pyx_t_1 = __pyx_t_5;
  __pyx_L5_bool_binop_done:;
  if (__pyx_t_1) {

    /* "plyvel/_plyvel.pyx":649
 *         if self.transaction and exc_type is not None:
 *             # Exception occurred in transaction; do not write the batch
 *             self.clear()             # <<<<<<<<<<<<<<
 *             return
 * 
*/
    __pyx_t_3 = ((PyObject *)__pyx_v_self);
    __Pyx_INCREF(__pyx_t_3);
    __pyx_t_4 = 0;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, NULL};
      __pyx_t_2 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_clear, __pyx_callargs+__pyx_t_4, (1-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 649, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "plyvel/_plyvel.pyx":650
 *             # Exception occurred in transaction; do not write the batch
 *             self.clear()
 *             return             # <<<<<<<<<<<<<<
 * 
 *         self.write()
*/
    __Pyx_XDECREF(__pyx_r);
    __pyx_r = Py_None; __Pyx_INCREF(Py_None);
    goto __pyx_L0;

    /* "plyvel/_plyvel.pyx":647
 *             raise RuntimeError("Database is closed")
 * 
 *         if self.transaction and exc_type is not None:             # <<<<<<<<<<<<<<
 *             # Exception occurred in transaction; do not write the batch
 *             self.clear()
*/
  }

  /* "plyvel/_plyvel.pyx":652
 *             return
 * 
 *         self.write()             # <<<<<<<<<<<<<<
 *         self.clear()
 * 
*/
  __pyx_t_3 = ((PyObject *)__pyx_v_self);
  __Pyx_INCREF(__pyx_t_3);
  __pyx_t_4 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_3, NULL};
    __pyx_t_2 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_write, __pyx_callargs+__pyx_t_4, (1-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 652, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "plyvel/_plyvel.pyx":653
 * 
 *         self.write()
 *         self.clear()             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __pyx_t_3 = ((PyObject *)__pyx_v_self);
  __Pyx_INCREF(__pyx_t_3);
  __pyx_t_4 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_3, NULL};
    __pyx_t_2 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_clear, __pyx_callargs+__pyx_t_4, (1-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 653, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "plyvel/_plyvel.pyx":643
 *         return self
 * 
 *     def __exit__(self, exc_type, exc_val, exc_tb):             # <<<<<<<<<<<<<<
 *         if self.db._db is NULL:
 *             raise RuntimeError("Database is closed")
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("plyvel._plyvel.WriteBatch.__exit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self._write_batch,self.write_options cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):
*/

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_10WriteBatch_21__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6plyvel_7_plyvel_10WriteBatch_20__reduce_cython__, "WriteBatch.__reduce_cython__(self)");
static PyMethodDef __pyx_mdef_6plyvel_7_plyvel_10WriteBatch_21__reduce_cython__ = {"__reduce_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_10WriteBatch_21__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_10WriteBatch_20__reduce_cython__};
static PyObject *__pyx_pw_6plyvel_7_plyvel_10WriteBatch_21__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("__reduce_cython__", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("__reduce_cython__", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_10WriteBatch_20__reduce_cython__(((struct __pyx_obj_6plyvel_7_plyvel_WriteBatch *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_10WriteBatch_20__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_6plyvel_7_plyvel_WriteBatch *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError, "self._write_batch,self.write_options cannot be converted to a Python object for pickling"             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError, "self._write_batch,self.write_options cannot be converted to a Python object for pickling"
*/
  __Pyx_Raise(((PyObject *)(((PyTypeObject*)PyExc_TypeError))), __pyx_mstate_global->__pyx_kp_u_self__write_batch_self_write_opt, 0, 0);
  __PYX_ERR(1, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self._write_batch,self.write_options cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("plyvel._plyvel.WriteBatch.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError, "self._write_batch,self.write_options cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self._write_batch,self.write_options cannot be converted to a Python object for pickling"
*/

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_10WriteBatch_23__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6plyvel_7_plyvel_10WriteBatch_22__setstate_cython__, "WriteBatch.__setstate_cython__(self, __pyx_state)");
static PyMethodDef __pyx_mdef_6plyvel_7_plyvel_10WriteBatch_23__setstate_cython__ = {"__setstate_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_10WriteBatch_23__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_10WriteBatch_22__setstate_cython__};
static PyObject *__pyx_pw_6plyvel_7_plyvel_10WriteBatch_23__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  CYTHON_UNUSED PyObject *__pyx_v___pyx_state = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_pyx_state,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(1, 3, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(1, 3, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__setstate_cython__", 0) < (0)) __PYX_ERR(1, 3, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, i); __PYX_ERR(1, 3, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(1, 3, __pyx_L3_error)
    }
    __pyx_v___pyx_state = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, __pyx_nargs); __PYX_ERR(1, 3, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("plyvel._plyvel.WriteBatch.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_10WriteBatch_22__setstate_cython__(((struct __pyx_obj_6plyvel_7_plyvel_WriteBatch *)__pyx_v_self), __pyx_v___pyx_state);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_10WriteBatch_22__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_6plyvel_7_plyvel_WriteBatch *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError, "self._write_batch,self.write_options cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError, "self._write_batch,self.write_options cannot be converted to a Python object for pickling"             # <<<<<<<<<<<<<<
*/
  __Pyx_Raise(((PyObject *)(((PyTypeObject*)PyExc_TypeError))), __pyx_mstate_global->__pyx_kp_u_self__write_batch_self_write_opt, 0, 0);
  __PYX_ERR(1, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError, "self._write_batch,self.write_options cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self._write_batch,self.write_options cannot be converted to a Python object for pickling"
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("plyvel._plyvel.WriteBatch.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":680
 *     cdef object __weakref__
 * 
 *     def __init__(self, DB db, bool verify_checksums, bool fill_cache,             # <<<<<<<<<<<<<<
 *                  Snapshot snapshot):
 *         if db._db is NULL:
*/

/* Python wrapper */
static int __pyx_pw_6plyvel_7_plyvel_12BaseIterator_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_6plyvel_7_plyvel_12BaseIterator_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_db = 0;
  PyLongObject *__pyx_v_verify_checksums = 0;
  PyLongObject *__pyx_v_fill_cache = 0;
  struct __pyx_obj_6plyvel_7_plyvel_Snapshot *__pyx_v_snapshot = 0;
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[4] = {0,0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return -1;
  #endif
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_db,&__pyx_mstate_global->__pyx_n_u_verify_checksums,&__pyx_mstate_global->__pyx_n_u_fill_cache,&__pyx_mstate_global->__pyx_n_u_snapshot,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_VARARGS(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(0, 680, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  4:
        values[3] = __Pyx_ArgRef_VARARGS(__pyx_args, 3);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[3])) __PYX_ERR(0, 680, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  3:
        values[2] = __Pyx_ArgRef_VARARGS(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 680, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_VARARGS(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 680, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 680, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__init__", 0) < (0)) __PYX_ERR(0, 680, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 4; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__init__", 1, 4, 4, i); __PYX_ERR(0, 680, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 4)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 680, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_VARARGS(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 680, __pyx_L3_error)
      values[2] = __Pyx_ArgRef_VARARGS(__pyx_args, 2);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 680, __pyx_L3_error)
      values[3] = __Pyx_ArgRef_VARARGS(__pyx_args, 3);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[3])) __PYX_ERR(0, 680, __pyx_L3_error)
    }
    __pyx_v_db = ((struct __pyx_obj_6plyvel_7_plyvel_DB *)values[0]);
    __pyx_v_verify_checksums = ((PyLongObject *)values[1]);
    __pyx_v_fill_cache = ((PyLongObject *)values[2]);
    __pyx_v_snapshot = ((struct __pyx_obj_6plyvel_7_plyvel_Snapshot *)values[3]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 4, 4, __pyx_nargs); __PYX_ERR(0, 680, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("plyvel._plyvel.BaseIterator.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_db), __pyx_mstate_global->__pyx_ptype_6plyvel_7_plyvel_DB, 1, "db", 0))) __PYX_ERR(0, 680, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_verify_checksums), __pyx_mstate_global->__pyx_ptype_7cpython_4bool_bool, 1, "verify_checksums", 0))) __PYX_ERR(0, 680, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_fill_cache), __pyx_mstate_global->__pyx_ptype_7cpython_4bool_bool, 1, "fill_cache", 0))) __PYX_ERR(0, 680, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_snapshot), __pyx_mstate_global->__pyx_ptype_6plyvel_7_plyvel_Snapshot, 1, "snapshot", 0))) __PYX_ERR(0, 681, __pyx_L1_error)
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_12BaseIterator___init__(((struct __pyx_obj_6plyvel_7_plyvel_BaseIterator *)__pyx_v_self), __pyx_v_db, __pyx_v_verify_checksums, __pyx_v_fill_cache, __pyx_v_snapshot);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = -1;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":701
 *         ref_dict[iterator_id] = weakref_ref(
 *             self,
 *             lambda wr: ref_dict.pop(iterator_id))             # <<<<<<<<<<<<<<
 * 
 *     cpdef close(self):
*/

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_12BaseIterator_8__init___lambda(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyMethodDef __pyx_mdef_6plyvel_7_plyvel_12BaseIterator_8__init___lambda = {"lambda", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_12BaseIterator_8__init___lambda, __Pyx_METH_FASTCALL|METH_KEYWORDS, 0};
static PyObject *__pyx_pw_6plyvel_7_plyvel_12BaseIterator_8__init___lambda(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  CYTHON_UNUSED PyObject *__pyx_v_wr = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("lambda (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_wr,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(0, 701, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 701, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "lambda", 0) < (0)) __PYX_ERR(0, 701, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("lambda", 1, 1, 1, i); __PYX_ERR(0, 701, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 701, __pyx_L3_error)
    }
    __pyx_v_wr = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("lambda", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 701, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("plyvel._plyvel.BaseIterator.__init__.lambda", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_lambda_funcdef_lambda(__pyx_self, __pyx_v_wr);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_lambda_funcdef_lambda(PyObject *__pyx_self, CYTHON_UNUSED PyObject *__pyx_v_wr) {
  struct __pyx_obj_6plyvel_7_plyvel___pyx_scope_struct____init__ *__pyx_cur_scope;
  struct __pyx_obj_6plyvel_7_plyvel___pyx_scope_struct____init__ *__pyx_outer_scope;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("lambda", 0);
  __pyx_outer_scope = (struct __pyx_obj_6plyvel_7_plyvel___pyx_scope_struct____init__ *) __Pyx_CyFunction_GetClosure(__pyx_self);
  __pyx_cur_scope = __pyx_outer_scope;
  __Pyx_XDECREF(__pyx_r);
  if (unlikely(!__pyx_cur_scope->__pyx_v_ref_dict)) { __Pyx_RaiseClosureNameError("ref_dict"); __PYX_ERR(0, 701, __pyx_L1_error) }
  if (unlikely(__pyx_cur_scope->__pyx_v_ref_dict == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "pop");
    __PYX_ERR(0, 701, __pyx_L1_error)
  }
  if (unlikely(!__pyx_cur_scope->__pyx_v_iterator_id)) { __Pyx_RaiseClosureNameError("iterator_id"); __PYX_ERR(0, 701, __pyx_L1_error) }
  __pyx_t_1 = __Pyx_PyDict_Pop(__pyx_cur_scope->__pyx_v_ref_dict, __pyx_cur_scope->__pyx_v_iterator_id, ((PyObject *)NULL)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 701, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("plyvel._plyvel.BaseIterator.__init__.lambda", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":680
 *     cdef object __weakref__
 * 
 *     def __init__(self, DB db, bool verify_checksums, bool fill_cache,             # <<<<<<<<<<<<<<
 *                  Snapshot snapshot):
 *         if db._db is NULL:
*/

static int __pyx_pf_6plyvel_7_plyvel_12BaseIterator___init__(struct __pyx_obj_6plyvel_7_plyvel_BaseIterator *__pyx_v_self, struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_db, PyLongObject *__pyx_v_verify_checksums, PyLongObject *__pyx_v_fill_cache, struct __pyx_obj_6plyvel_7_plyvel_Snapshot *__pyx_v_snapshot) {
  struct __pyx_obj_6plyvel_7_plyvel___pyx_scope_struct____init__ *__pyx_cur_scope;
  leveldb::ReadOptions __pyx_v_read_options;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  bool __pyx_t_5;
  leveldb::Snapshot *__pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);
  __pyx_cur_scope = (struct __pyx_obj_6plyvel_7_plyvel___pyx_scope_struct____init__ *)__pyx_tp_new_6plyvel_7_plyvel___pyx_scope_struct____init__(__pyx_mstate_global->__pyx_ptype_6plyvel_7_plyvel___pyx_scope_struct____init__, __pyx_mstate_global->__pyx_empty_tuple, NULL);
  if (unlikely(!__pyx_cur_scope)) {
    __pyx_cur_scope = ((struct __pyx_obj_6plyvel_7_plyvel___pyx_scope_struct____init__ *)Py_None);
    __Pyx_INCREF(Py_None);
    __PYX_ERR(0, 680, __pyx_L1_error)
  } else {
    __Pyx_GOTREF((PyObject *)__pyx_cur_scope);
  }

  /* "plyvel/_plyvel.pyx":682
 *     def __init__(self, DB db, bool verify_checksums, bool fill_cache,
 *                  Snapshot snapshot):
 *         if db._db is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database or iterator is closed")
 * 
*/
  __pyx_t_1 = (__pyx_v_db->_db == NULL);
  if (unlikely(__pyx_t_1)) {

    /* "plyvel/_plyvel.pyx":683
 *                  Snapshot snapshot):
 *         if db._db is NULL:
 *             raise RuntimeError("Database or iterator is closed")             # <<<<<<<<<<<<<<
 * 
 *         self.db = db
*/
    __pyx_t_3 = NULL;
    __pyx_t_4 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_mstate_global->__pyx_kp_u_Database_or_iterator_is_closed};
      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_RuntimeError)), __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 683, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 683, __pyx_L1_error)

    /* "plyvel/_plyvel.pyx":682
 *     def __init__(self, DB db, bool verify_checksums, bool fill_cache,
 *                  Snapshot snapshot):
 *         if db._db is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database or iterator is closed")
 * 
*/
  }

  /* "plyvel/_plyvel.pyx":685
 *             raise RuntimeError("Database or iterator is closed")
 * 
 *         self.db = db             # <<<<<<<<<<<<<<
 * 
 *         cdef ReadOptions read_options
*/
  __Pyx_INCREF((PyObject *)__pyx_v_db);
  __Pyx_GIVEREF((PyObject *)__pyx_v_db);
  __Pyx_GOTREF((PyObject *)__pyx_v_self->db);
  __Pyx_DECREF((PyObject *)__pyx_v_self->db);
  __pyx_v_self->db = __pyx_v_db;

  /* "plyvel/_plyvel.pyx":688
 * 
 *         cdef ReadOptions read_options
 *         read_options.verify_checksums = verify_checksums             # <<<<<<<<<<<<<<
 *         read_options.fill_cache = fill_cache
 *         if snapshot is not None:
*/
  __pyx_t_5 = __Pyx_PyObject_IsTrue(((PyObject *)__pyx_v_verify_checksums)); if (unlikely((__pyx_t_5 == ((bool)-1)) && PyErr_Occurred())) __PYX_ERR(0, 688, __pyx_L1_error)
  __pyx_v_read_options.verify_checksums = __pyx_t_5;

  /* "plyvel/_plyvel.pyx":689
 *         cdef ReadOptions read_options
 *         read_options.verify_checksums = verify_checksums
 *         read_options.fill_cache = fill_cache             # <<<<<<<<<<<<<<
 *         if snapshot is not None:
 *             read_options.snapshot = snapshot._snapshot
*/
  __pyx_t_5 = __Pyx_PyObject_IsTrue(((PyObject *)__pyx_v_fill_cache)); if (unlikely((__pyx_t_5 == ((bool)-1)) && PyErr_Occurred())) __PYX_ERR(0, 689, __pyx_L1_error)
  __pyx_v_read_options.fill_cache = __pyx_t_5;

  /* "plyvel/_plyvel.pyx":690
 *         read_options.verify_checksums = verify_checksums
 *         read_options.fill_cache = fill_cache
 *         if snapshot is not None:             # <<<<<<<<<<<<<<
 *             read_options.snapshot = snapshot._snapshot
 * 
*/
  __pyx_t_1 = (((PyObject *)__pyx_v_snapshot) != Py_None);
  if (__pyx_t_1) {

    /* "plyvel/_plyvel.pyx":691
 *         read_options.fill_cache = fill_cache
 *         if snapshot is not None:
 *             read_options.snapshot = snapshot._snapshot             # <<<<<<<<<<<<<<
 * 
 *         with nogil:
*/
    __pyx_t_6 = __pyx_v_snapshot->_snapshot;
    __pyx_v_read_options.snapshot = __pyx_t_6;

    /* "plyvel/_plyvel.pyx":690
 *         read_options.verify_checksums = verify_checksums
 *         read_options.fill_cache = fill_cache
 *         if snapshot is not None:             # <<<<<<<<<<<<<<
 *             read_options.snapshot = snapshot._snapshot
 * 
*/
  }

  /* "plyvel/_plyvel.pyx":693
 *             read_options.snapshot = snapshot._snapshot
 * 
 *         with nogil:             # <<<<<<<<<<<<<<
 *             self._iter = db._db.NewIterator(read_options)
 * 
*/
  {
      PyThreadState * _save;
      _save = PyEval_SaveThread();
      __Pyx_FastGIL_Remember();
      /*try:*/ {

        /* "plyvel/_plyvel.pyx":694
 * 
 *         with nogil:
 *             self._iter = db._db.NewIterator(read_options)             # <<<<<<<<<<<<<<
 * 
 *         # Store a weak reference on the db (needed when closing db)
*/
        __pyx_v_self->_iter = __pyx_v_db->_db->NewIterator(__pyx_v_read_options);
      }

      /* "plyvel/_plyvel.pyx":693
 *             read_options.snapshot = snapshot._snapshot
 * 
 *         with nogil:             # <<<<<<<<<<<<<<
 *             self._iter = db._db.NewIterator(read_options)
 * 
*/
      /*finally:*/ {
        /*normal exit:*/{
          __Pyx_FastGIL_Forget();
          PyEval_RestoreThread(_save);
          goto __pyx_L7;
        }
        __pyx_L7:;
      }
  }

  /* "plyvel/_plyvel.pyx":697
 * 
 *         # Store a weak reference on the db (needed when closing db)
 *         iterator_id = id(self)             # <<<<<<<<<<<<<<
 *         ref_dict = db.iterators
 *         ref_dict[iterator_id] = weakref_ref(
*/
  __pyx_t_3 = NULL;
  __pyx_t_4 = 1;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_3, ((PyObject *)__pyx_v_self)};
    __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_builtin_id, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 697, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
  }
  __Pyx_GIVEREF(__pyx_t_2);
  __pyx_cur_scope->__pyx_v_iterator_id = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "plyvel/_plyvel.pyx":698
 *         # Store a weak reference on the db (needed when closing db)
 *         iterator_id = id(self)
 *         ref_dict = db.iterators             # <<<<<<<<<<<<<<
 *         ref_dict[iterator_id] = weakref_ref(
 *             self,
*/
  __pyx_t_2 = __pyx_v_db->iterators;
  __Pyx_INCREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_2);
  __pyx_cur_scope->__pyx_v_ref_dict = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "plyvel/_plyvel.pyx":699
 *         iterator_id = id(self)
 *         ref_dict = db.iterators
 *         ref_dict[iterator_id] = weakref_ref(             # <<<<<<<<<<<<<<
 *             self,
 *             lambda wr: ref_dict.pop(iterator_id))
*/
  __pyx_t_3 = NULL;
  __Pyx_GetModuleGlobalName(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_weakref_ref); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 699, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);

  /* "plyvel/_plyvel.pyx":701
 *         ref_dict[iterator_id] = weakref_ref(
 *             self,
 *             lambda wr: ref_dict.pop(iterator_id))             # <<<<<<<<<<<<<<
 * 
 *     cpdef close(self):
*/
  __pyx_t_8 = __Pyx_CyFunction_New(&__pyx_mdef_6plyvel_7_plyvel_12BaseIterator_8__init___lambda, 0, __pyx_mstate_global->__pyx_n_u_init___locals_lambda, ((PyObject*)__pyx_cur_scope), __pyx_mstate_global->__pyx_n_u_plyvel__plyvel, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[0])); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 701, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_8);
  __pyx_t_4 = 1;
  #if CYTHON_UNPACK_METHODS
  if (unlikely(PyMethod_Check(__pyx_t_7))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_7);
    assert(__pyx_t_3);
    PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_7);
    __Pyx_INCREF(__pyx_t_3);
    __Pyx_INCREF(__pyx__function);
    __Pyx_DECREF_SET(__pyx_t_7, __pyx__function);
    __pyx_t_4 = 0;
  }
  #endif
  {
    PyObject *__pyx_callargs[3] = {__pyx_t_3, ((PyObject *)__pyx_v_self), __pyx_t_8};
    __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_7, __pyx_callargs+__pyx_t_4, (3-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 699, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
  }

  /* "plyvel/_plyvel.pyx":699
 *         iterator_id = id(self)
 *         ref_dict = db.iterators
 *         ref_dict[iterator_id] = weakref_ref(             # <<<<<<<<<<<<<<
 *             self,
 *             lambda wr: ref_dict.pop(iterator_id))
*/
  if (unlikely(__pyx_cur_scope->__pyx_v_ref_dict == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(0, 699, __pyx_L1_error)
  }
  if (unlikely((PyDict_SetItem(__pyx_cur_scope->__pyx_v_ref_dict, __pyx_cur_scope->__pyx_v_iterator_id, __pyx_t_2) < 0))) __PYX_ERR(0, 699, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "plyvel/_plyvel.pyx":680
 *     cdef object __weakref__
 * 
 *     def __init__(self, DB db, bool verify_checksums, bool fill_cache,             # <<<<<<<<<<<<<<
 *                  Snapshot snapshot):
 *         if db._db is NULL:
*/

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("plyvel._plyvel.BaseIterator.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_DECREF((PyObject *)__pyx_cur_scope);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":703
 *             lambda wr: ref_dict.pop(iterator_id))
 * 
 *     cpdef close(self):             # <<<<<<<<<<<<<<
 *         if self._iter is not NULL:
 *             del self._iter
*/

static PyObject *__pyx_pw_6plyvel_7_plyvel_12BaseIterator_3close(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyObject *__pyx_f_6plyvel_7_plyvel_12BaseIterator_close(struct __pyx_obj_6plyvel_7_plyvel_BaseIterator *__pyx_v_self, int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("close", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (
  #if !CYTHON_USE_TYPE_SLOTS
  unlikely(Py_TYPE(((PyObject *)__pyx_v_self)) != __pyx_mstate_global->__pyx_ptype_6plyvel_7_plyvel_BaseIterator &&
  __Pyx_PyType_HasFeature(Py_TYPE(((PyObject *)__pyx_v_self)), Py_TPFLAGS_HAVE_GC))
  #else
  unlikely(Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0 || __Pyx_PyType_HasFeature(Py_TYPE(((PyObject *)__pyx_v_self)), (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))
  #endif
  ) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_typedict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_mstate_global->__pyx_n_u_close); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 703, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!__Pyx_IsSameCFunction(__pyx_t_1, (void(*)(void)) __pyx_pw_6plyvel_7_plyvel_12BaseIterator_3close)) {
        __Pyx_XDECREF(__pyx_r);
        __pyx_t_3 = NULL;
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_4 = __pyx_t_1; 
        __pyx_t_5 = 1;
        #if CYTHON_UNPACK_METHODS
        if (unlikely(PyMethod_Check(__pyx_t_4))) {
          __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_4);
          assert(__pyx_t_3);
          PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_4);
          __Pyx_INCREF(__pyx_t_3);
          __Pyx_INCREF(__pyx__function);
          __Pyx_DECREF_SET(__pyx_t_4, __pyx__function);
          __pyx_t_5 = 0;
        }
        #endif
        {
          PyObject *__pyx_callargs[2] = {__pyx_t_3, NULL};
          __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_4, __pyx_callargs+__pyx_t_5, (1-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
          __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 703, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
        }
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_typedict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "plyvel/_plyvel.pyx":704
 * 
 *     cpdef close(self):
 *         if self._iter is not NULL:             # <<<<<<<<<<<<<<
 *             del self._iter
 *             self._iter = NULL
*/
  __pyx_t_6 = (__pyx_v_self->_iter != NULL);
  if (__pyx_t_6) {

    /* "plyvel/_plyvel.pyx":705
 *     cpdef close(self):
 *         if self._iter is not NULL:
 *             del self._iter             # <<<<<<<<<<<<<<
 *             self._iter = NULL
 * 
*/
    delete __pyx_v_self->_iter;

    /* "plyvel/_plyvel.pyx":706
 *         if self._iter is not NULL:
 *             del self._iter
 *             self._iter = NULL             # <<<<<<<<<<<<<<
 * 
 *     def __dealloc__(self):
*/
    __pyx_v_self->_iter = NULL;

    /* "plyvel/_plyvel.pyx":704
 * 
 *     cpdef close(self):
 *         if self._iter is not NULL:             # <<<<<<<<<<<<<<
 *             del self._iter
 *             self._iter = NULL
*/
  }

  /* "plyvel/_plyvel.pyx":703
 *             lambda wr: ref_dict.pop(iterator_id))
 * 
 *     cpdef close(self):             # <<<<<<<<<<<<<<
 *         if self._iter is not NULL:
 *             del self._iter
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("plyvel._plyvel.BaseIterator.close", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_12BaseIterator_3close(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6plyvel_7_plyvel_12BaseIterator_2close, "BaseIterator.close(self)");
static PyMethodDef __pyx_mdef_6plyvel_7_plyvel_12BaseIterator_3close = {"close", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_12BaseIterator_3close, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_12BaseIterator_2close};
static PyObject *__pyx_pw_6plyvel_7_plyvel_12BaseIterator_3close(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("close (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("close", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("close", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_12BaseIterator_2close(((struct __pyx_obj_6plyvel_7_plyvel_BaseIterator *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_12BaseIterator_2close(struct __pyx_obj_6plyvel_7_plyvel_BaseIterator *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("close", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_6plyvel_7_plyvel_12BaseIterator_close(__pyx_v_self, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 703, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("plyvel._plyvel.BaseIterator.close", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":708
 *             self._iter = NULL
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         self.close()
 * 
*/

/* Python wrapper */
static void __pyx_pw_6plyvel_7_plyvel_12BaseIterator_5__dealloc__(PyObject *__pyx_v_self); /*proto*/
static void __pyx_pw_6plyvel_7_plyvel_12BaseIterator_5__dealloc__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__dealloc__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_pf_6plyvel_7_plyvel_12BaseIterator_4__dealloc__(((struct __pyx_obj_6plyvel_7_plyvel_BaseIterator *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

static void __pyx_pf_6plyvel_7_plyvel_12BaseIterator_4__dealloc__(struct __pyx_obj_6plyvel_7_plyvel_BaseIterator *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__dealloc__", 0);

  /* "plyvel/_plyvel.pyx":709
 * 
 *     def __dealloc__(self):
 *         self.close()             # <<<<<<<<<<<<<<
 * 
 *     def __enter__(self):
*/
  __pyx_t_1 = ((struct __pyx_vtabstruct_6plyvel_7_plyvel_BaseIterator *)__pyx_v_self->__pyx_vtab)->close(__pyx_v_self, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 709, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "plyvel/_plyvel.pyx":708
 *             self._iter = NULL
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         self.close()
 * 
*/

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_WriteUnraisable("plyvel._plyvel.BaseIterator.__dealloc__", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "plyvel/_plyvel.pyx":711
 *         self.close()
 * 
 *     def __enter__(self):             # <<<<<<<<<<<<<<
 *         return self
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_12BaseIterator_7__enter__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6plyvel_7_plyvel_12BaseIterator_6__enter__, "BaseIterator.__enter__(self)");
static PyMethodDef __pyx_mdef_6plyvel_7_plyvel_12BaseIterator_7__enter__ = {"__enter__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_12BaseIterator_7__enter__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_12BaseIterator_6__enter__};
static PyObject *__pyx_pw_6plyvel_7_plyvel_12BaseIterator_7__enter__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__enter__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("__enter__", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("__enter__", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_12BaseIterator_6__enter__(((struct __pyx_obj_6plyvel_7_plyvel_BaseIterator *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_12BaseIterator_6__enter__(struct __pyx_obj_6plyvel_7_plyvel_BaseIterator *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__enter__", 0);

  /* "plyvel/_plyvel.pyx":712
 * 
 *     def __enter__(self):
 *         return self             # <<<<<<<<<<<<<<
 * 
 *     def __exit__(self, exc_type, exc_val, exc_tb):
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF((PyObject *)__pyx_v_self);
  __pyx_r = ((PyObject *)__pyx_v_self);
  goto __pyx_L0;

  /* "plyvel/_plyvel.pyx":711
 *         self.close()
 * 
 *     def __enter__(self):             # <<<<<<<<<<<<<<
 *         return self
 * 
*/

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":714
 *         return self
 * 
 *     def __exit__(self, exc_type, exc_val, exc_tb):             # <<<<<<<<<<<<<<
 *         self.close()
 *         return False  # propagate exceptions
*/

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_12BaseIterator_9__exit__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6plyvel_7_plyvel_12BaseIterator_8__exit__, "BaseIterator.__exit__(self, exc_type, exc_val, exc_tb)");
static PyMethodDef __pyx_mdef_6plyvel_7_plyvel_12BaseIterator_9__exit__ = {"__exit__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_12BaseIterator_9__exit__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_12BaseIterator_8__exit__};
static PyObject *__pyx_pw_6plyvel_7_plyvel_12BaseIterator_9__exit__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  CYTHON_UNUSED PyObject *__pyx_v_exc_type = 0;
  CYTHON_UNUSED PyObject *__pyx_v_exc_val = 0;
  CYTHON_UNUSED PyObject *__pyx_v_exc_tb = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[3] = {0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__exit__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_exc_type,&__pyx_mstate_global->__pyx_n_u_exc_val,&__pyx_mstate_global->__pyx_n_u_exc_tb,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(0, 714, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 714, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 714, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 714, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__exit__", 0) < (0)) __PYX_ERR(0, 714, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 3; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__exit__", 1, 3, 3, i); __PYX_ERR(0, 714, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 3)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 714, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 714, __pyx_L3_error)
      values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 714, __pyx_L3_error)
    }
    __pyx_v_exc_type = values[0];
    __pyx_v_exc_val = values[1];
    __pyx_v_exc_tb = values[2];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__exit__", 1, 3, 3, __pyx_nargs); __PYX_ERR(0, 714, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("plyvel._plyvel.BaseIterator.__exit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_12BaseIterator_8__exit__(((struct __pyx_obj_6plyvel_7_plyvel_BaseIterator *)__pyx_v_self), __pyx_v_exc_type, __pyx_v_exc_val, __pyx_v_exc_tb);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_12BaseIterator_8__exit__(struct __pyx_obj_6plyvel_7_plyvel_BaseIterator *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v_exc_type, CYTHON_UNUSED PyObject *__pyx_v_exc_val, CYTHON_UNUSED PyObject *__pyx_v_exc_tb) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__exit__", 0);

  /* "plyvel/_plyvel.pyx":715
 * 
 *     def __exit__(self, exc_type, exc_val, exc_tb):
 *         self.close()             # <<<<<<<<<<<<<<
 *         return False  # propagate exceptions
 * 
*/
  __pyx_t_1 = ((struct __pyx_vtabstruct_6plyvel_7_plyvel_BaseIterator *)__pyx_v_self->__pyx_vtab)->close(__pyx_v_self, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 715, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "plyvel/_plyvel.pyx":716
 *     def __exit__(self, exc_type, exc_val, exc_tb):
 *         self.close()
 *         return False  # propagate exceptions             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(Py_False);
  __pyx_r = Py_False;
  goto __pyx_L0;

  /* "plyvel/_plyvel.pyx":714
 *         return self
 * 
 *     def __exit__(self, exc_type, exc_val, exc_tb):             # <<<<<<<<<<<<<<
 *         self.close()
 *         return False  # propagate exceptions
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("plyvel._plyvel.BaseIterator.__exit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self._iter cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):
*/

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_12BaseIterator_11__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6plyvel_7_plyvel_12BaseIterator_10__reduce_cython__, "BaseIterator.__reduce_cython__(self)");
static PyMethodDef __pyx_mdef_6plyvel_7_plyvel_12BaseIterator_11__reduce_cython__ = {"__reduce_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_12BaseIterator_11__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_12BaseIterator_10__reduce_cython__};
static PyObject *__pyx_pw_6plyvel_7_plyvel_12BaseIterator_11__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("__reduce_cython__", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("__reduce_cython__", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_12BaseIterator_10__reduce_cython__(((struct __pyx_obj_6plyvel_7_plyvel_BaseIterator *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_12BaseIterator_10__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_6plyvel_7_plyvel_BaseIterator *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError, "self._iter cannot be converted to a Python object for pickling"             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError, "self._iter cannot be converted to a Python object for pickling"
*/
  __Pyx_Raise(((PyObject *)(((PyTypeObject*)PyExc_TypeError))), __pyx_mstate_global->__pyx_kp_u_self__iter_cannot_be_converted_t, 0, 0);
  __PYX_ERR(1, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self._iter cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("plyvel._plyvel.BaseIterator.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError, "self._iter cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self._iter cannot be converted to a Python object for pickling"
*/

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_12BaseIterator_13__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6plyvel_7_plyvel_12BaseIterator_12__setstate_cython__, "BaseIterator.__setstate_cython__(self, __pyx_state)");
static PyMethodDef __pyx_mdef_6plyvel_7_plyvel_12BaseIterator_13__setstate_cython__ = {"__setstate_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_12BaseIterator_13__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_12BaseIterator_12__setstate_cython__};
static PyObject *__pyx_pw_6plyvel_7_plyvel_12BaseIterator_13__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  CYTHON_UNUSED PyObject *__pyx_v___pyx_state = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_pyx_state,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(1, 3, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(1, 3, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__setstate_cython__", 0) < (0)) __PYX_ERR(1, 3, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, i); __PYX_ERR(1, 3, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(1, 3, __pyx_L3_error)
    }
    __pyx_v___pyx_state = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, __pyx_nargs); __PYX_ERR(1, 3, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("plyvel._plyvel.BaseIterator.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_12BaseIterator_12__setstate_cython__(((struct __pyx_obj_6plyvel_7_plyvel_BaseIterator *)__pyx_v_self), __pyx_v___pyx_state);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_12BaseIterator_12__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_6plyvel_7_plyvel_BaseIterator *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError, "self._iter cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError, "self._iter cannot be converted to a Python object for pickling"             # <<<<<<<<<<<<<<
*/
  __Pyx_Raise(((PyObject *)(((PyTypeObject*)PyExc_TypeError))), __pyx_mstate_global->__pyx_kp_u_self__iter_cannot_be_converted_t, 0, 0);
  __PYX_ERR(1, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError, "self._iter cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self._iter cannot be converted to a Python object for pickling"
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("plyvel._plyvel.BaseIterator.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":736
 *     cdef size_t db_prefix_len
 * 
 *     def __init__(self, DB db, bytes db_prefix, bool reverse, bytes start,             # <<<<<<<<<<<<<<
 *                  bytes stop, bool include_start, bool include_stop,
 *                  bytes prefix, bool include_key, bool include_value,
*/

/* Python wrapper */
static int __pyx_pw_6plyvel_7_plyvel_8Iterator_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_6plyvel_7_plyvel_8Iterator_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_db = 0;
  PyObject *__pyx_v_db_prefix = 0;
  PyLongObject *__pyx_v_reverse = 0;
  PyObject *__pyx_v_start = 0;
  PyObject *__pyx_v_stop = 0;
  PyLongObject *__pyx_v_include_start = 0;
  PyLongObject *__pyx_v_include_stop = 0;
  PyObject *__pyx_v_prefix = 0;
  PyLongObject *__pyx_v_include_key = 0;
  PyLongObject *__pyx_v_include_value = 0;
  PyLongObject *__pyx_v_verify_checksums = 0;
  PyLongObject *__pyx_v_fill_cache = 0;
  struct __pyx_obj_6plyvel_7_plyvel_Snapshot *__pyx_v_snapshot = 0;
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[13] = {0,0,0,0,0,0,0,0,0,0,0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return -1;
  #endif
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_db,&__pyx_mstate_global->__pyx_n_u_db_prefix,&__pyx_mstate_global->__pyx_n_u_reverse,&__pyx_mstate_global->__pyx_n_u_start,&__pyx_mstate_global->__pyx_n_u_stop,&__pyx_mstate_global->__pyx_n_u_include_start,&__pyx_mstate_global->__pyx_n_u_include_stop,&__pyx_mstate_global->__pyx_n_u_prefix,&__pyx_mstate_global->__pyx_n_u_include_key,&__pyx_mstate_global->__pyx_n_u_include_value,&__pyx_mstate_global->__pyx_n_u_verify_checksums,&__pyx_mstate_global->__pyx_n_u_fill_cache,&__pyx_mstate_global->__pyx_n_u_snapshot,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_VARARGS(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(0, 736, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case 13:
        values[12] = __Pyx_ArgRef_VARARGS(__pyx_args, 12);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[12])) __PYX_ERR(0, 736, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case 12:
        values[11] = __Pyx_ArgRef_VARARGS(__pyx_args, 11);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[11])) __PYX_ERR(0, 736, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case 11:
        values[10] = __Pyx_ArgRef_VARARGS(__pyx_args, 10);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[10])) __PYX_ERR(0, 736, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case 10:
        values[9] = __Pyx_ArgRef_VARARGS(__pyx_args, 9);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[9])) __PYX_ERR(0, 736, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  9:
        values[8] = __Pyx_ArgRef_VARARGS(__pyx_args, 8);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[8])) __PYX_ERR(0, 736, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  8:
        values[7] = __Pyx_ArgRef_VARARGS(__pyx_args, 7);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[7])) __PYX_ERR(0, 736, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  7:
        values[6] = __Pyx_ArgRef_VARARGS(__pyx_args, 6);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[6])) __PYX_ERR(0, 736, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  6:
        values[5] = __Pyx_ArgRef_VARARGS(__pyx_args, 5);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[5])) __PYX_ERR(0, 736, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  5:
        values[4] = __Pyx_ArgRef_VARARGS(__pyx_args, 4);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[4])) __PYX_ERR(0, 736, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  4:
        values[3] = __Pyx_ArgRef_VARARGS(__pyx_args, 3);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[3])) __PYX_ERR(0, 736, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  3:
        values[2] = __Pyx_ArgRef_VARARGS(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 736, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_VARARGS(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 736, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 736, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__init__", 0) < (0)) __PYX_ERR(0, 736, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 13; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__init__", 1, 13, 13, i); __PYX_ERR(0, 736, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 13)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_VARARGS(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 736, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_VARARGS(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 736, __pyx_L3_error)
      values[2] = __Pyx_ArgRef_VARARGS(__pyx_args, 2);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 736, __pyx_L3_error)
      values[3] = __Pyx_ArgRef_VARARGS(__pyx_args, 3);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[3])) __PYX_ERR(0, 736, __pyx_L3_error)
      values[4] = __Pyx_ArgRef_VARARGS(__pyx_args, 4);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[4])) __PYX_ERR(0, 736, __pyx_L3_error)
      values[5] = __Pyx_ArgRef_VARARGS(__pyx_args, 5);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[5])) __PYX_ERR(0, 736, __pyx_L3_error)
      values[6] = __Pyx_ArgRef_VARARGS(__pyx_args, 6);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[6])) __PYX_ERR(0, 736, __pyx_L3_error)
      values[7] = __Pyx_ArgRef_VARARGS(__pyx_args, 7);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[7])) __PYX_ERR(0, 736, __pyx_L3_error)
      values[8] = __Pyx_ArgRef_VARARGS(__pyx_args, 8);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[8])) __PYX_ERR(0, 736, __pyx_L3_error)
      values[9] = __Pyx_ArgRef_VARARGS(__pyx_args, 9);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[9])) __PYX_ERR(0, 736, __pyx_L3_error)
      values[10] = __Pyx_ArgRef_VARARGS(__pyx_args, 10);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[10])) __PYX_ERR(0, 736, __pyx_L3_error)
      values[11] = __Pyx_ArgRef_VARARGS(__pyx_args, 11);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[11])) __PYX_ERR(0, 736, __pyx_L3_error)
      values[12] = __Pyx_ArgRef_VARARGS(__pyx_args, 12);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[12])) __PYX_ERR(0, 736, __pyx_L3_error)
    }
    __pyx_v_db = ((struct __pyx_obj_6plyvel_7_plyvel_DB *)values[0]);
    __pyx_v_db_prefix = ((PyObject*)values[1]);
    __pyx_v_reverse = ((PyLongObject *)values[2]);
    __pyx_v_start = ((PyObject*)values[3]);
    __pyx_v_stop = ((PyObject*)values[4]);
    __pyx_v_include_start = ((PyLongObject *)values[5]);
    __pyx_v_include_stop = ((PyLongObject *)values[6]);
    __pyx_v_prefix = ((PyObject*)values[7]);
    __pyx_v_include_key = ((PyLongObject *)values[8]);
    __pyx_v_include_value = ((PyLongObject *)values[9]);
    __pyx_v_verify_checksums = ((PyLongObject *)values[10]);
    __pyx_v_fill_cache = ((PyLongObject *)values[11]);
    __pyx_v_snapshot = ((struct __pyx_obj_6plyvel_7_plyvel_Snapshot *)values[12]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 13, 13, __pyx_nargs); __PYX_ERR(0, 736, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("plyvel._plyvel.Iterator.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_db), __pyx_mstate_global->__pyx_ptype_6plyvel_7_plyvel_DB, 1, "db", 0))) __PYX_ERR(0, 736, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_db_prefix), (&PyBytes_Type), 1, "db_prefix", 1))) __PYX_ERR(0, 736, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_reverse), __pyx_mstate_global->__pyx_ptype_7cpython_4bool_bool, 1, "reverse", 0))) __PYX_ERR(0, 736, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_start), (&PyBytes_Type), 1, "start", 1))) __PYX_ERR(0, 736, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_stop), (&PyBytes_Type), 1, "stop", 1))) __PYX_ERR(0, 737, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_include_start), __pyx_mstate_global->__pyx_ptype_7cpython_4bool_bool, 1, "include_start", 0))) __PYX_ERR(0, 737, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_include_stop), __pyx_mstate_global->__pyx_ptype_7cpython_4bool_bool, 1, "include_stop", 0))) __PYX_ERR(0, 737, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_prefix), (&PyBytes_Type), 1, "prefix", 1))) __PYX_ERR(0, 738, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_include_key), __pyx_mstate_global->__pyx_ptype_7cpython_4bool_bool, 1, "include_key", 0))) __PYX_ERR(0, 738, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_include_value), __pyx_mstate_global->__pyx_ptype_7cpython_4bool_bool, 1, "include_value", 0))) __PYX_ERR(0, 738, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_verify_checksums), __pyx_mstate_global->__pyx_ptype_7cpython_4bool_bool, 1, "verify_checksums", 0))) __PYX_ERR(0, 739, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_fill_cache), __pyx_mstate_global->__pyx_ptype_7cpython_4bool_bool, 1, "fill_cache", 0))) __PYX_ERR(0, 739, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_snapshot), __pyx_mstate_global->__pyx_ptype_6plyvel_7_plyvel_Snapshot, 1, "snapshot", 0))) __PYX_ERR(0, 739, __pyx_L1_error)
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_8Iterator___init__(((struct __pyx_obj_6plyvel_7_plyvel_Iterator *)__pyx_v_self), __pyx_v_db, __pyx_v_db_prefix, __pyx_v_reverse, __pyx_v_start, __pyx_v_stop, __pyx_v_include_start, __pyx_v_include_stop, __pyx_v_prefix, __pyx_v_include_key, __pyx_v_include_value, __pyx_v_verify_checksums, __pyx_v_fill_cache, __pyx_v_snapshot);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = -1;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6plyvel_7_plyvel_8Iterator___init__(struct __pyx_obj_6plyvel_7_plyvel_Iterator *__pyx_v_self, struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_db, PyObject *__pyx_v_db_prefix, PyLongObject *__pyx_v_reverse, PyObject *__pyx_v_start, PyObject *__pyx_v_stop, PyLongObject *__pyx_v_include_start, PyLongObject *__pyx_v_include_stop, PyObject *__pyx_v_prefix, PyLongObject *__pyx_v_include_key, PyLongObject *__pyx_v_include_value, PyLongObject *__pyx_v_verify_checksums, PyLongObject *__pyx_v_fill_cache, struct __pyx_obj_6plyvel_7_plyvel_Snapshot *__pyx_v_snapshot) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  enum __pyx_t_6plyvel_7_plyvel_IteratorDirection __pyx_t_6;
  int __pyx_t_7;
  int __pyx_t_8;
  Py_ssize_t __pyx_t_9;
  const char *__pyx_t_10;
  bool __pyx_t_11;
  int __pyx_t_12;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);
  __Pyx_INCREF(__pyx_v_start);
  __Pyx_INCREF(__pyx_v_stop);
  __Pyx_INCREF((PyObject *)__pyx_v_include_start);
  __Pyx_INCREF((PyObject *)__pyx_v_include_stop);
  __Pyx_INCREF(__pyx_v_prefix);

  /* "plyvel/_plyvel.pyx":741
 *                  bool verify_checksums, bool fill_cache, Snapshot snapshot):
 * 
 *         super(Iterator, self).__init__(             # <<<<<<<<<<<<<<
 *             db=db,
 *             verify_checksums=verify_checksums,
*/
  __pyx_t_4 = NULL;
  __pyx_t_5 = 1;
  {
    PyObject *__pyx_callargs[3] = {__pyx_t_4, ((PyObject *)__pyx_mstate_global->__pyx_ptype_6plyvel_7_plyvel_Iterator), ((PyObject *)__pyx_v_self)};
    __pyx_t_3 = __Pyx_PyObject_FastCall((PyObject*)__pyx_builtin_super, __pyx_callargs+__pyx_t_5, (3-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 741, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
  }
  __pyx_t_2 = __pyx_t_3;
  __Pyx_INCREF(__pyx_t_2);

  /* "plyvel/_plyvel.pyx":745
 *             verify_checksums=verify_checksums,
 *             fill_cache=fill_cache,
 *             snapshot=snapshot)             # <<<<<<<<<<<<<<
 * 
 *         self.comparator = <leveldb.Comparator*>db.options.comparator
*/
  __pyx_t_5 = 0;
  {
    PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 4 : 0)] = {__pyx_t_2, NULL};
    __pyx_t_4 = __Pyx_MakeVectorcallBuilderKwds(4); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 741, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_db, ((PyObject *)__pyx_v_db), __pyx_t_4, __pyx_callargs+1, 0) < (0)) __PYX_ERR(0, 741, __pyx_L1_error)
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_verify_checksums, ((PyObject *)__pyx_v_verify_checksums), __pyx_t_4, __pyx_callargs+1, 1) < (0)) __PYX_ERR(0, 741, __pyx_L1_error)
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_fill_cache, ((PyObject *)__pyx_v_fill_cache), __pyx_t_4, __pyx_callargs+1, 2) < (0)) __PYX_ERR(0, 741, __pyx_L1_error)
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_snapshot, ((PyObject *)__pyx_v_snapshot), __pyx_t_4, __pyx_callargs+1, 3) < (0)) __PYX_ERR(0, 741, __pyx_L1_error)
    __pyx_t_1 = __Pyx_Object_VectorcallMethod_CallFromBuilder((PyObject*)__pyx_mstate_global->__pyx_n_u_init, __pyx_callargs+__pyx_t_5, (1-__pyx_t_5) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 741, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "plyvel/_plyvel.pyx":747
 *             snapshot=snapshot)
 * 
 *         self.comparator = <leveldb.Comparator*>db.options.comparator             # <<<<<<<<<<<<<<
 *         self.direction = FORWARD if not reverse else REVERSE
 * 
*/
  __pyx_v_self->comparator = ((leveldb::Comparator *)__pyx_v_db->options.comparator);

  /* "plyvel/_plyvel.pyx":748
 * 
 *         self.comparator = <leveldb.Comparator*>db.options.comparator
 *         self.direction = FORWARD if not reverse else REVERSE             # <<<<<<<<<<<<<<
 * 
 *         if db_prefix is None:
*/
  __pyx_t_7 = __Pyx_PyObject_IsTrue(((PyObject *)__pyx_v_reverse)); if (unlikely((__pyx_t_7 < 0))) __PYX_ERR(0, 748, __pyx_L1_error)
  __pyx_t_8 = (!__pyx_t_7);
  if (__pyx_t_8) {
    __pyx_t_6 = __pyx_e_6plyvel_7_plyvel_FORWARD;
  } else {
    __pyx_t_6 = __pyx_e_6plyvel_7_plyvel_REVERSE;
  }
  __pyx_v_self->direction = __pyx_t_6;

  /* "plyvel/_plyvel.pyx":750
 *         self.direction = FORWARD if not reverse else REVERSE
 * 
 *         if db_prefix is None:             # <<<<<<<<<<<<<<
 *             self.db_prefix_len = 0
 *         else:
*/
  __pyx_t_8 = (__pyx_v_db_prefix == ((PyObject*)Py_None));
  if (__pyx_t_8) {

    /* "plyvel/_plyvel.pyx":751
 * 
 *         if db_prefix is None:
 *             self.db_prefix_len = 0             # <<<<<<<<<<<<<<
 *         else:
 *             # This is an iterator on a PrefixedDB.
*/
    __pyx_v_self->db_prefix_len = 0;

    /* "plyvel/_plyvel.pyx":750
 *         self.direction = FORWARD if not reverse else REVERSE
 * 
 *         if db_prefix is None:             # <<<<<<<<<<<<<<
 *             self.db_prefix_len = 0
 *         else:
*/
    goto __pyx_L3;
  }

  /* "plyvel/_plyvel.pyx":754
 *         else:
 *             # This is an iterator on a PrefixedDB.
 *             self.db_prefix = db_prefix             # <<<<<<<<<<<<<<
 *             self.db_prefix_len = len(db_prefix)
 * 
*/
  /*else*/ {
    __Pyx_INCREF(__pyx_v_db_prefix);
    __Pyx_GIVEREF(__pyx_v_db_prefix);
    __Pyx_GOTREF(__pyx_v_self->db_prefix);
    __Pyx_DECREF(__pyx_v_self->db_prefix);
    __pyx_v_self->db_prefix = __pyx_v_db_prefix;

    /* "plyvel/_plyvel.pyx":755
 *             # This is an iterator on a PrefixedDB.
 *             self.db_prefix = db_prefix
 *             self.db_prefix_len = len(db_prefix)             # <<<<<<<<<<<<<<
 * 
 *             # Transform args so that the database key prefix is taken
*/
    if (unlikely(__pyx_v_db_prefix == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
      __PYX_ERR(0, 755, __pyx_L1_error)
    }
    __pyx_t_9 = __Pyx_PyBytes_GET_SIZE(__pyx_v_db_prefix); if (unlikely(__pyx_t_9 == ((Py_ssize_t)-1))) __PYX_ERR(0, 755, __pyx_L1_error)
    __pyx_v_self->db_prefix_len = __pyx_t_9;

    /* "plyvel/_plyvel.pyx":759
 *             # Transform args so that the database key prefix is taken
 *             # into account.
 *             if prefix is not None:             # <<<<<<<<<<<<<<
 *                 # Both database key prefix and prefix on the iterator
 *                 prefix = db_prefix + prefix
*/
    __pyx_t_8 = (__pyx_v_prefix != ((PyObject*)Py_None));
    if (__pyx_t_8) {

      /* "plyvel/_plyvel.pyx":761
 *             if prefix is not None:
 *                 # Both database key prefix and prefix on the iterator
 *                 prefix = db_prefix + prefix             # <<<<<<<<<<<<<<
 *             else:
 *                 # Adapt start and stop keys to use the database key
*/
      __pyx_t_1 = PyNumber_Add(__pyx_v_db_prefix, __pyx_v_prefix); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 761, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF_SET(__pyx_v_prefix, ((PyObject*)__pyx_t_1));
      __pyx_t_1 = 0;

      /* "plyvel/_plyvel.pyx":759
 *             # Transform args so that the database key prefix is taken
 *             # into account.
 *             if prefix is not None:             # <<<<<<<<<<<<<<
 *                 # Both database key prefix and prefix on the iterator
 *                 prefix = db_prefix + prefix
*/
      goto __pyx_L4;
    }

    /* "plyvel/_plyvel.pyx":765
 *                 # Adapt start and stop keys to use the database key
 *                 # prefix.
 *                 if start is None:             # <<<<<<<<<<<<<<
 *                     start = db_prefix
 *                     include_start = True
*/
    /*else*/ {
      __pyx_t_8 = (__pyx_v_start == ((PyObject*)Py_None));
      if (__pyx_t_8) {

        /* "plyvel/_plyvel.pyx":766
 *                 # prefix.
 *                 if start is None:
 *                     start = db_prefix             # <<<<<<<<<<<<<<
 *                     include_start = True
 *                 else:
*/
        __Pyx_INCREF(__pyx_v_db_prefix);
        __Pyx_DECREF_SET(__pyx_v_start, __pyx_v_db_prefix);

        /* "plyvel/_plyvel.pyx":767
 *                 if start is None:
 *                     start = db_prefix
 *                     include_start = True             # <<<<<<<<<<<<<<
 *                 else:
 *                     start = db_prefix + start
*/
        __Pyx_INCREF(Py_True);
        __Pyx_DECREF_SET(__pyx_v_include_start, ((PyLongObject *)Py_True));

        /* "plyvel/_plyvel.pyx":765
 *                 # Adapt start and stop keys to use the database key
 *                 # prefix.
 *                 if start is None:             # <<<<<<<<<<<<<<
 *                     start = db_prefix
 *                     include_start = True
*/
        goto __pyx_L5;
      }

      /* "plyvel/_plyvel.pyx":769
 *                     include_start = True
 *                 else:
 *                     start = db_prefix + start             # <<<<<<<<<<<<<<
 * 
 *                 if stop is None:
*/
      /*else*/ {
        __pyx_t_1 = PyNumber_Add(__pyx_v_db_prefix, __pyx_v_start); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 769, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF_SET(__pyx_v_start, ((PyObject*)__pyx_t_1));
        __pyx_t_1 = 0;
      }
      __pyx_L5:;

      /* "plyvel/_plyvel.pyx":771
 *                     start = db_prefix + start
 * 
 *                 if stop is None:             # <<<<<<<<<<<<<<
 *                     stop = bytes_increment(db_prefix)
 *                     include_stop = False
*/
      __pyx_t_8 = (__pyx_v_stop == ((PyObject*)Py_None));
      if (__pyx_t_8) {

        /* "plyvel/_plyvel.pyx":772
 * 
 *                 if stop is None:
 *                     stop = bytes_increment(db_prefix)             # <<<<<<<<<<<<<<
 *                     include_stop = False
 *                 else:
*/
        __pyx_t_1 = __pyx_f_6plyvel_7_plyvel_bytes_increment(__pyx_v_db_prefix); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 772, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF_SET(__pyx_v_stop, ((PyObject*)__pyx_t_1));
        __pyx_t_1 = 0;

        /* "plyvel/_plyvel.pyx":773
 *                 if stop is None:
 *                     stop = bytes_increment(db_prefix)
 *                     include_stop = False             # <<<<<<<<<<<<<<
 *                 else:
 *                     stop = db_prefix + stop
*/
        __Pyx_INCREF(Py_False);
        __Pyx_DECREF_SET(__pyx_v_include_stop, ((PyLongObject *)Py_False));

        /* "plyvel/_plyvel.pyx":771
 *                     start = db_prefix + start
 * 
 *                 if stop is None:             # <<<<<<<<<<<<<<
 *                     stop = bytes_increment(db_prefix)
 *                     include_stop = False
*/
        goto __pyx_L6;
      }

      /* "plyvel/_plyvel.pyx":775
 *                     include_stop = False
 *                 else:
 *                     stop = db_prefix + stop             # <<<<<<<<<<<<<<
 * 
 *         if prefix is not None:
*/
      /*else*/ {
        __pyx_t_1 = PyNumber_Add(__pyx_v_db_prefix, __pyx_v_stop); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 775, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF_SET(__pyx_v_stop, ((PyObject*)__pyx_t_1));
        __pyx_t_1 = 0;
      }
      __pyx_L6:;
    }
    __pyx_L4:;
  }
  __pyx_L3:;

  /* "plyvel/_plyvel.pyx":777
 *                     stop = db_prefix + stop
 * 
 *         if prefix is not None:             # <<<<<<<<<<<<<<
 *             if start is not None or stop is not None:
 *                 raise TypeError(
*/
  __pyx_t_8 = (__pyx_v_prefix != ((PyObject*)Py_None));
  if (__pyx_t_8) {

    /* "plyvel/_plyvel.pyx":778
 * 
 *         if prefix is not None:
 *             if start is not None or stop is not None:             # <<<<<<<<<<<<<<
 *                 raise TypeError(
 *                     "'prefix' cannot be used together with 'start' or 'stop'")
*/
    __pyx_t_7 = (__pyx_v_start != ((PyObject*)Py_None));
    if (!__pyx_t_7) {
    } else {
      __pyx_t_8 = __pyx_t_7;
      goto __pyx_L9_bool_binop_done;
    }
    __pyx_t_7 = (__pyx_v_stop != ((PyObject*)Py_None));
    __pyx_t_8 = __pyx_t_7;
    __pyx_L9_bool_binop_done:;
    if (unlikely(__pyx_t_8)) {

      /* "plyvel/_plyvel.pyx":779
 *         if prefix is not None:
 *             if start is not None or stop is not None:
 *                 raise TypeError(             # <<<<<<<<<<<<<<
 *                     "'prefix' cannot be used together with 'start' or 'stop'")
 *             # Use prefix to construct start and stop keys, and ignore
*/
      __pyx_t_3 = NULL;
      __pyx_t_5 = 1;
      {
        PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_mstate_global->__pyx_kp_u_prefix_cannot_be_used_together};
        __pyx_t_1 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_TypeError)), __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 779, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
      }
      __Pyx_Raise(__pyx_t_1, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __PYX_ERR(0, 779, __pyx_L1_error)

      /* "plyvel/_plyvel.pyx":778
 * 
 *         if prefix is not None:
 *             if start is not None or stop is not None:             # <<<<<<<<<<<<<<
 *                 raise TypeError(
 *                     "'prefix' cannot be used together with 'start' or 'stop'")
*/
    }

    /* "plyvel/_plyvel.pyx":783
 *             # Use prefix to construct start and stop keys, and ignore
 *             # include_start and include_stop args
 *             start = prefix             # <<<<<<<<<<<<<<
 *             stop = bytes_increment(prefix)
 *             include_start = True
*/
    __Pyx_INCREF(__pyx_v_prefix);
    __Pyx_DECREF_SET(__pyx_v_start, __pyx_v_prefix);

    /* "plyvel/_plyvel.pyx":784
 *             # include_start and include_stop args
 *             start = prefix
 *             stop = bytes_increment(prefix)             # <<<<<<<<<<<<<<
 *             include_start = True
 *             include_stop = False
*/
    __pyx_t_1 = __pyx_f_6plyvel_7_plyvel_bytes_increment(__pyx_v_prefix); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 784, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF_SET(__pyx_v_stop, ((PyObject*)__pyx_t_1));
    __pyx_t_1 = 0;

    /* "plyvel/_plyvel.pyx":785
 *             start = prefix
 *             stop = bytes_increment(prefix)
 *             include_start = True             # <<<<<<<<<<<<<<
 *             include_stop = False
 * 
*/
    __Pyx_INCREF(Py_True);
    __Pyx_DECREF_SET(__pyx_v_include_start, ((PyLongObject *)Py_True));

    /* "plyvel/_plyvel.pyx":786
 *             stop = bytes_increment(prefix)
 *             include_start = True
 *             include_stop = False             # <<<<<<<<<<<<<<
 * 
 *         if start is not None:
*/
    __Pyx_INCREF(Py_False);
    __Pyx_DECREF_SET(__pyx_v_include_stop, ((PyLongObject *)Py_False));

    /* "plyvel/_plyvel.pyx":777
 *                     stop = db_prefix + stop
 * 
 *         if prefix is not None:             # <<<<<<<<<<<<<<
 *             if start is not None or stop is not None:
 *                 raise TypeError(
*/
  }

  /* "plyvel/_plyvel.pyx":788
 *             include_stop = False
 * 
 *         if start is not None:             # <<<<<<<<<<<<<<
 *             self.start = start
 *             self.start_slice = Slice(start, len(start))
*/
  __pyx_t_8 = (__pyx_v_start != ((PyObject*)Py_None));
  if (__pyx_t_8) {

    /* "plyvel/_plyvel.pyx":789
 * 
 *         if start is not None:
 *             self.start = start             # <<<<<<<<<<<<<<
 *             self.start_slice = Slice(start, len(start))
 * 
*/
    __Pyx_INCREF(__pyx_v_start);
    __Pyx_GIVEREF(__pyx_v_start);
    __Pyx_GOTREF(__pyx_v_self->start);
    __Pyx_DECREF(__pyx_v_self->start);
    __pyx_v_self->start = __pyx_v_start;

    /* "plyvel/_plyvel.pyx":790
 *         if start is not None:
 *             self.start = start
 *             self.start_slice = Slice(start, len(start))             # <<<<<<<<<<<<<<
 * 
 *         if stop is not None:
*/
    if (unlikely(__pyx_v_start == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "expected bytes, NoneType found");
      __PYX_ERR(0, 790, __pyx_L1_error)
    }
    __pyx_t_10 = __Pyx_PyBytes_AsString(__pyx_v_start); if (unlikely((!__pyx_t_10) && PyErr_Occurred())) __PYX_ERR(0, 790, __pyx_L1_error)
    if (unlikely(__pyx_v_start == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
      __PYX_ERR(0, 790, __pyx_L1_error)
    }
    __pyx_t_9 = __Pyx_PyBytes_GET_SIZE(__pyx_v_start); if (unlikely(__pyx_t_9 == ((Py_ssize_t)-1))) __PYX_ERR(0, 790, __pyx_L1_error)
    __pyx_v_self->start_slice = leveldb::Slice(__pyx_t_10, __pyx_t_9);

    /* "plyvel/_plyvel.pyx":788
 *             include_stop = False
 * 
 *         if start is not None:             # <<<<<<<<<<<<<<
 *             self.start = start
 *             self.start_slice = Slice(start, len(start))
*/
  }

  /* "plyvel/_plyvel.pyx":792
 *             self.start_slice = Slice(start, len(start))
 * 
 *         if stop is not None:             # <<<<<<<<<<<<<<
 *             self.stop = stop
 *             self.stop_slice = Slice(stop, len(stop))
*/
  __pyx_t_8 = (__pyx_v_stop != ((PyObject*)Py_None));
  if (__pyx_t_8) {

    /* "plyvel/_plyvel.pyx":793
 * 
 *         if stop is not None:
 *             self.stop = stop             # <<<<<<<<<<<<<<
 *             self.stop_slice = Slice(stop, len(stop))
 * 
*/
    __Pyx_INCREF(__pyx_v_stop);
    __Pyx_GIVEREF(__pyx_v_stop);
    __Pyx_GOTREF(__pyx_v_self->stop);
    __Pyx_DECREF(__pyx_v_self->stop);
    __pyx_v_self->stop = __pyx_v_stop;

    /* "plyvel/_plyvel.pyx":794
 *         if stop is not None:
 *             self.stop = stop
 *             self.stop_slice = Slice(stop, len(stop))             # <<<<<<<<<<<<<<
 * 
 *         self.include_start = include_start
*/
    if (unlikely(__pyx_v_stop == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "expected bytes, NoneType found");
      __PYX_ERR(0, 794, __pyx_L1_error)
    }
    __pyx_t_10 = __Pyx_PyBytes_AsString(__pyx_v_stop); if (unlikely((!__pyx_t_10) && PyErr_Occurred())) __PYX_ERR(0, 794, __pyx_L1_error)
    if (unlikely(__pyx_v_stop == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
      __PYX_ERR(0, 794, __pyx_L1_error)
    }
    __pyx_t_9 = __Pyx_PyBytes_GET_SIZE(__pyx_v_stop); if (unlikely(__pyx_t_9 == ((Py_ssize_t)-1))) __PYX_ERR(0, 794, __pyx_L1_error)
    __pyx_v_self->stop_slice = leveldb::Slice(__pyx_t_10, __pyx_t_9);

    /* "plyvel/_plyvel.pyx":792
 *             self.start_slice = Slice(start, len(start))
 * 
 *         if stop is not None:             # <<<<<<<<<<<<<<
 *             self.stop = stop
 *             self.stop_slice = Slice(stop, len(stop))
*/
  }

  /* "plyvel/_plyvel.pyx":796
 *             self.stop_slice = Slice(stop, len(stop))
 * 
 *         self.include_start = include_start             # <<<<<<<<<<<<<<
 *         self.include_stop = include_stop
 *         self.include_key = include_key
*/
  __pyx_t_11 = __Pyx_PyObject_IsTrue(((PyObject *)__pyx_v_include_start)); if (unlikely((__pyx_t_11 == ((bool)-1)) && PyErr_Occurred())) __PYX_ERR(0, 796, __pyx_L1_error)
  __pyx_v_self->include_start = __pyx_t_11;

  /* "plyvel/_plyvel.pyx":797
 * 
 *         self.include_start = include_start
 *         self.include_stop = include_stop             # <<<<<<<<<<<<<<
 *         self.include_key = include_key
 *         self.include_value = include_value
*/
  __pyx_t_11 = __Pyx_PyObject_IsTrue(((PyObject *)__pyx_v_include_stop)); if (unlikely((__pyx_t_11 == ((bool)-1)) && PyErr_Occurred())) __PYX_ERR(0, 797, __pyx_L1_error)
  __pyx_v_self->include_stop = __pyx_t_11;

  /* "plyvel/_plyvel.pyx":798
 *         self.include_start = include_start
 *         self.include_stop = include_stop
 *         self.include_key = include_key             # <<<<<<<<<<<<<<
 *         self.include_value = include_value
 * 
*/
  __pyx_t_11 = __Pyx_PyObject_IsTrue(((PyObject *)__pyx_v_include_key)); if (unlikely((__pyx_t_11 == ((bool)-1)) && PyErr_Occurred())) __PYX_ERR(0, 798, __pyx_L1_error)
  __pyx_v_self->include_key = __pyx_t_11;

  /* "plyvel/_plyvel.pyx":799
 *         self.include_stop = include_stop
 *         self.include_key = include_key
 *         self.include_value = include_value             # <<<<<<<<<<<<<<
 * 
 *         if self.direction == FORWARD:
*/
  __pyx_t_11 = __Pyx_PyObject_IsTrue(((PyObject *)__pyx_v_include_value)); if (unlikely((__pyx_t_11 == ((bool)-1)) && PyErr_Occurred())) __PYX_ERR(0, 799, __pyx_L1_error)
  __pyx_v_self->include_value = __pyx_t_11;

  /* "plyvel/_plyvel.pyx":801
 *         self.include_value = include_value
 * 
 *         if self.direction == FORWARD:             # <<<<<<<<<<<<<<
 *             self.seek_to_start()
 *         else:
*/
  __pyx_t_8 = (__pyx_v_self->direction == __pyx_e_6plyvel_7_plyvel_FORWARD);
  if (__pyx_t_8) {

    /* "plyvel/_plyvel.pyx":802
 * 
 *         if self.direction == FORWARD:
 *             self.seek_to_start()             # <<<<<<<<<<<<<<
 *         else:
 *             self.seek_to_stop()
*/
    __pyx_t_3 = ((PyObject *)__pyx_v_self);
    __Pyx_INCREF(__pyx_t_3);
    __pyx_t_5 = 0;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, NULL};
      __pyx_t_1 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_seek_to_start, __pyx_callargs+__pyx_t_5, (1-__pyx_t_5) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 802, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "plyvel/_plyvel.pyx":801
 *         self.include_value = include_value
 * 
 *         if self.direction == FORWARD:             # <<<<<<<<<<<<<<
 *             self.seek_to_start()
 *         else:
*/
    goto __pyx_L13;
  }

  /* "plyvel/_plyvel.pyx":804
 *             self.seek_to_start()
 *         else:
 *             self.seek_to_stop()             # <<<<<<<<<<<<<<
 * 
 *         raise_for_status(self._iter.status())
*/
  /*else*/ {
    __pyx_t_3 = ((PyObject *)__pyx_v_self);
    __Pyx_INCREF(__pyx_t_3);
    __pyx_t_5 = 0;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, NULL};
      __pyx_t_1 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_seek_to_stop, __pyx_callargs+__pyx_t_5, (1-__pyx_t_5) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 804, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  }
  __pyx_L13:;

  /* "plyvel/_plyvel.pyx":806
 *             self.seek_to_stop()
 * 
 *         raise_for_status(self._iter.status())             # <<<<<<<<<<<<<<
 * 
 *     def __iter__(self):
*/
  __pyx_t_12 = __pyx_f_6plyvel_7_plyvel_raise_for_status(__pyx_v_self->__pyx_base._iter->status()); if (unlikely(__pyx_t_12 == ((int)-1))) __PYX_ERR(0, 806, __pyx_L1_error)

  /* "plyvel/_plyvel.pyx":736
 *     cdef size_t db_prefix_len
 * 
 *     def __init__(self, DB db, bytes db_prefix, bool reverse, bytes start,             # <<<<<<<<<<<<<<
 *                  bytes stop, bool include_start, bool include_stop,
 *                  bytes prefix, bool include_key, bool include_value,
*/

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("plyvel._plyvel.Iterator.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_start);
  __Pyx_XDECREF(__pyx_v_stop);
  __Pyx_XDECREF((PyObject *)__pyx_v_include_start);
  __Pyx_XDECREF((PyObject *)__pyx_v_include_stop);
  __Pyx_XDECREF(__pyx_v_prefix);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":808
 *         raise_for_status(self._iter.status())
 * 
 *     def __iter__(self):             # <<<<<<<<<<<<<<
 *         return self
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_8Iterator_3__iter__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6plyvel_7_plyvel_8Iterator_3__iter__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__iter__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_8Iterator_2__iter__(((struct __pyx_obj_6plyvel_7_plyvel_Iterator *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_8Iterator_2__iter__(struct __pyx_obj_6plyvel_7_plyvel_Iterator *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__iter__", 0);

  /* "plyvel/_plyvel.pyx":809
 * 
 *     def __iter__(self):
 *         return self             # <<<<<<<<<<<<<<
 * 
 *     cdef object current(self):
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF((PyObject *)__pyx_v_self);
  __pyx_r = ((PyObject *)__pyx_v_self);
  goto __pyx_L0;

  /* "plyvel/_plyvel.pyx":808
 *         raise_for_status(self._iter.status())
 * 
 *     def __iter__(self):             # <<<<<<<<<<<<<<
 *         return self
 * 
*/

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":811
 *         return self
 * 
 *     cdef object current(self):             # <<<<<<<<<<<<<<
 *         """Return the current iterator key/value.
 * 
*/

static PyObject *__pyx_f_6plyvel_7_plyvel_8Iterator_current(struct __pyx_obj_6plyvel_7_plyvel_Iterator *__pyx_v_self) {
  leveldb::Slice __pyx_v_key_slice;
  PyObject *__pyx_v_key = 0;
  leveldb::Slice __pyx_v_value_slice;
  PyObject *__pyx_v_value = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("current", 0);

  /* "plyvel/_plyvel.pyx":818
 *         """
 *         cdef Slice key_slice
 *         cdef bytes key = None             # <<<<<<<<<<<<<<
 *         cdef Slice value_slice
 *         cdef bytes value = None
*/
  __Pyx_INCREF(Py_None);
  __pyx_v_key = ((PyObject*)Py_None);

  /* "plyvel/_plyvel.pyx":820
 *         cdef bytes key = None
 *         cdef Slice value_slice
 *         cdef bytes value = None             # <<<<<<<<<<<<<<
 * 
 *         # Only build Python strings that will be returned. Also chop off
*/
  __Pyx_INCREF(Py_None);
  __pyx_v_value = ((PyObject*)Py_None);

  /* "plyvel/_plyvel.pyx":824
 *         # Only build Python strings that will be returned. Also chop off
 *         # the db prefix (for PrefixedDB iterators).
 *         if self.include_key:             # <<<<<<<<<<<<<<
 *             key_slice = self._iter.key()
 *             key = key_slice.data()[self.db_prefix_len:key_slice.size()]
*/
  __pyx_t_1 = (__pyx_v_self->include_key != 0);
  if (__pyx_t_1) {

    /* "plyvel/_plyvel.pyx":825
 *         # the db prefix (for PrefixedDB iterators).
 *         if self.include_key:
 *             key_slice = self._iter.key()             # <<<<<<<<<<<<<<
 *             key = key_slice.data()[self.db_prefix_len:key_slice.size()]
 * 
*/
    __pyx_v_key_slice = __pyx_v_self->__pyx_base._iter->key();

    /* "plyvel/_plyvel.pyx":826
 *         if self.include_key:
 *             key_slice = self._iter.key()
 *             key = key_slice.data()[self.db_prefix_len:key_slice.size()]             # <<<<<<<<<<<<<<
 * 
 *         if self.include_value:
*/
    __pyx_t_2 = __Pyx_PyBytes_FromStringAndSize(__pyx_v_key_slice.data() + __pyx_v_self->db_prefix_len, __pyx_v_key_slice.size() - __pyx_v_self->db_prefix_len); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 826, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF_SET(__pyx_v_key, ((PyObject*)__pyx_t_2));
    __pyx_t_2 = 0;

    /* "plyvel/_plyvel.pyx":824
 *         # Only build Python strings that will be returned. Also chop off
 *         # the db prefix (for PrefixedDB iterators).
 *         if self.include_key:             # <<<<<<<<<<<<<<
 *             key_slice = self._iter.key()
 *             key = key_slice.data()[self.db_prefix_len:key_slice.size()]
*/
  }

  /* "plyvel/_plyvel.pyx":828
 *             key = key_slice.data()[self.db_prefix_len:key_slice.size()]
 * 
 *         if self.include_value:             # <<<<<<<<<<<<<<
 *             value_slice = self._iter.value()
 *             value = value_slice.data()[:value_slice.size()]
*/
  __pyx_t_1 = (__pyx_v_self->include_value != 0);
  if (__pyx_t_1) {

    /* "plyvel/_plyvel.pyx":829
 * 
 *         if self.include_value:
 *             value_slice = self._iter.value()             # <<<<<<<<<<<<<<
 *             value = value_slice.data()[:value_slice.size()]
 * 
*/
    __pyx_v_value_slice = __pyx_v_self->__pyx_base._iter->value();

    /* "plyvel/_plyvel.pyx":830
 *         if self.include_value:
 *             value_slice = self._iter.value()
 *             value = value_slice.data()[:value_slice.size()]             # <<<<<<<<<<<<<<
 * 
 *         if self.include_key and self.include_value:
*/
    __pyx_t_2 = __Pyx_PyBytes_FromStringAndSize(__pyx_v_value_slice.data() + 0, __pyx_v_value_slice.size() - 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 830, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF_SET(__pyx_v_value, ((PyObject*)__pyx_t_2));
    __pyx_t_2 = 0;

    /* "plyvel/_plyvel.pyx":828
 *             key = key_slice.data()[self.db_prefix_len:key_slice.size()]
 * 
 *         if self.include_value:             # <<<<<<<<<<<<<<
 *             value_slice = self._iter.value()
 *             value = value_slice.data()[:value_slice.size()]
*/
  }

  /* "plyvel/_plyvel.pyx":832
 *             value = value_slice.data()[:value_slice.size()]
 * 
 *         if self.include_key and self.include_value:             # <<<<<<<<<<<<<<
 *             return (key, value)
 *         if self.include_key:
*/
  __pyx_t_3 = (__pyx_v_self->include_key != 0);
  if (__pyx_t_3) {
  } else {
    __pyx_t_1 = __pyx_t_3;
    goto __pyx_L6_bool_binop_done;
  }
  __pyx_t_3 = (__pyx_v_self->include_value != 0);
  __pyx_t_1 = __pyx_t_3;
  __pyx_L6_bool_binop_done:;
  if (__pyx_t_1) {

    /* "plyvel/_plyvel.pyx":833
 * 
 *         if self.include_key and self.include_value:
 *             return (key, value)             # <<<<<<<<<<<<<<
 *         if self.include_key:
 *             return key
*/
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 833, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_v_key);
    __Pyx_GIVEREF(__pyx_v_key);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_v_key) != (0)) __PYX_ERR(0, 833, __pyx_L1_error);
    __Pyx_INCREF(__pyx_v_value);
    __Pyx_GIVEREF(__pyx_v_value);
    if (__Pyx_PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_v_value) != (0)) __PYX_ERR(0, 833, __pyx_L1_error);
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "plyvel/_plyvel.pyx":832
 *             value = value_slice.data()[:value_slice.size()]
 * 
 *         if self.include_key and self.include_value:             # <<<<<<<<<<<<<<
 *             return (key, value)
 *         if self.include_key:
*/
  }

  /* "plyvel/_plyvel.pyx":834
 *         if self.include_key and self.include_value:
 *             return (key, value)
 *         if self.include_key:             # <<<<<<<<<<<<<<
 *             return key
 *         if self.include_value:
*/
  __pyx_t_1 = (__pyx_v_self->include_key != 0);
  if (__pyx_t_1) {

    /* "plyvel/_plyvel.pyx":835
 *             return (key, value)
 *         if self.include_key:
 *             return key             # <<<<<<<<<<<<<<
 *         if self.include_value:
 *             return value
*/
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_v_key);
    __pyx_r = __pyx_v_key;
    goto __pyx_L0;

    /* "plyvel/_plyvel.pyx":834
 *         if self.include_key and self.include_value:
 *             return (key, value)
 *         if self.include_key:             # <<<<<<<<<<<<<<
 *             return key
 *         if self.include_value:
*/
  }

  /* "plyvel/_plyvel.pyx":836
 *         if self.include_key:
 *             return key
 *         if self.include_value:             # <<<<<<<<<<<<<<
 *             return value
 *         return None
*/
  __pyx_t_1 = (__pyx_v_self->include_value != 0);
  if (__pyx_t_1) {

    /* "plyvel/_plyvel.pyx":837
 *             return key
 *         if self.include_value:
 *             return value             # <<<<<<<<<<<<<<
 *         return None
 * 
*/
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_v_value);
    __pyx_r = __pyx_v_value;
    goto __pyx_L0;

    /* "plyvel/_plyvel.pyx":836
 *         if self.include_key:
 *             return key
 *         if self.include_value:             # <<<<<<<<<<<<<<
 *             return value
 *         return None
*/
  }

  /* "plyvel/_plyvel.pyx":838
 *         if self.include_value:
 *             return value
 *         return None             # <<<<<<<<<<<<<<
 * 
 *     def __next__(self):
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;

  /* "plyvel/_plyvel.pyx":811
 *         return self
 * 
 *     cdef object current(self):             # <<<<<<<<<<<<<<
 *         """Return the current iterator key/value.
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("plyvel._plyvel.Iterator.current", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_key);
  __Pyx_XDECREF(__pyx_v_value);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":840
 *         return None
 * 
 *     def __next__(self):             # <<<<<<<<<<<<<<
 *         """Return the next iterator entry.
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_8Iterator_5__next__(PyObject *__pyx_v_self); /*proto*/
PyDoc_STRVAR(__pyx_doc_6plyvel_7_plyvel_8Iterator_4__next__, "Iterator.__next__(self)\n\nReturn the next iterator entry.\n\nNote: Cython will also create a .next() method that does the\nsame as this method.");
#if CYTHON_UPDATE_DESCRIPTOR_DOC
struct wrapperbase __pyx_wrapperbase_6plyvel_7_plyvel_8Iterator_4__next__;
#endif
static PyObject *__pyx_pw_6plyvel_7_plyvel_8Iterator_5__next__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__next__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_8Iterator_4__next__(((struct __pyx_obj_6plyvel_7_plyvel_Iterator *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_8Iterator_4__next__(struct __pyx_obj_6plyvel_7_plyvel_Iterator *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__next__", 0);

  /* "plyvel/_plyvel.pyx":846
 *         same as this method.
 *         """
 *         if self.direction == FORWARD:             # <<<<<<<<<<<<<<
 *             return self.real_next()
 *         else:
*/
  __pyx_t_1 = (__pyx_v_self->direction == __pyx_e_6plyvel_7_plyvel_FORWARD);
  if (__pyx_t_1) {

    /* "plyvel/_plyvel.pyx":847
 *         """
 *         if self.direction == FORWARD:
 *             return self.real_next()             # <<<<<<<<<<<<<<
 *         else:
 *             return self.real_prev()
*/
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __pyx_f_6plyvel_7_plyvel_8Iterator_real_next(__pyx_v_self); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 847, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "plyvel/_plyvel.pyx":846
 *         same as this method.
 *         """
 *         if self.direction == FORWARD:             # <<<<<<<<<<<<<<
 *             return self.real_next()
 *         else:
*/
  }

  /* "plyvel/_plyvel.pyx":849
 *             return self.real_next()
 *         else:
 *             return self.real_prev()             # <<<<<<<<<<<<<<
 * 
 *     def prev(self):
*/
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __pyx_f_6plyvel_7_plyvel_8Iterator_real_prev(__pyx_v_self); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 849, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;
  }

  /* "plyvel/_plyvel.pyx":840
 *         return None
 * 
 *     def __next__(self):             # <<<<<<<<<<<<<<
 *         """Return the next iterator entry.
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("plyvel._plyvel.Iterator.__next__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":851
 *             return self.real_prev()
 * 
 *     def prev(self):             # <<<<<<<<<<<<<<
 *         if self.direction == FORWARD:
 *             return self.real_prev()
*/

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_8Iterator_7prev(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6plyvel_7_plyvel_8Iterator_6prev, "Iterator.prev(self)");
static PyMethodDef __pyx_mdef_6plyvel_7_plyvel_8Iterator_7prev = {"prev", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_8Iterator_7prev, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_8Iterator_6prev};
static PyObject *__pyx_pw_6plyvel_7_plyvel_8Iterator_7prev(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("prev (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("prev", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("prev", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_8Iterator_6prev(((struct __pyx_obj_6plyvel_7_plyvel_Iterator *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_8Iterator_6prev(struct __pyx_obj_6plyvel_7_plyvel_Iterator *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("prev", 0);

  /* "plyvel/_plyvel.pyx":852
 * 
 *     def prev(self):
 *         if self.direction == FORWARD:             # <<<<<<<<<<<<<<
 *             return self.real_prev()
 *         else:
*/
  __pyx_t_1 = (__pyx_v_self->direction == __pyx_e_6plyvel_7_plyvel_FORWARD);
  if (__pyx_t_1) {

    /* "plyvel/_plyvel.pyx":853
 *     def prev(self):
 *         if self.direction == FORWARD:
 *             return self.real_prev()             # <<<<<<<<<<<<<<
 *         else:
 *             return self.real_next()
*/
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __pyx_f_6plyvel_7_plyvel_8Iterator_real_prev(__pyx_v_self); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 853, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "plyvel/_plyvel.pyx":852
 * 
 *     def prev(self):
 *         if self.direction == FORWARD:             # <<<<<<<<<<<<<<
 *             return self.real_prev()
 *         else:
*/
  }

  /* "plyvel/_plyvel.pyx":855
 *             return self.real_prev()
 *         else:
 *             return self.real_next()             # <<<<<<<<<<<<<<
 * 
 *     cdef real_next(self):
*/
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __pyx_f_6plyvel_7_plyvel_8Iterator_real_next(__pyx_v_self); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 855, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;
  }

  /* "plyvel/_plyvel.pyx":851
 *             return self.real_prev()
 * 
 *     def prev(self):             # <<<<<<<<<<<<<<
 *         if self.direction == FORWARD:
 *             return self.real_prev()
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("plyvel._plyvel.Iterator.prev", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":857
 *             return self.real_next()
 * 
 *     cdef real_next(self):             # <<<<<<<<<<<<<<
 *         if self._iter is NULL:
 *             raise RuntimeError("Database or iterator is closed")
*/

static PyObject *__pyx_f_6plyvel_7_plyvel_8Iterator_real_next(struct __pyx_obj_6plyvel_7_plyvel_Iterator *__pyx_v_self) {
  long __pyx_v_n;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;
  long __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("real_next", 0);

  /* "plyvel/_plyvel.pyx":858
 * 
 *     cdef real_next(self):
 *         if self._iter is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database or iterator is closed")
 * 
*/
  __pyx_t_1 = (__pyx_v_self->__pyx_base._iter == NULL);
  if (unlikely(__pyx_t_1)) {

    /* "plyvel/_plyvel.pyx":859
 *     cdef real_next(self):
 *         if self._iter is NULL:
 *             raise RuntimeError("Database or iterator is closed")             # <<<<<<<<<<<<<<
 * 
 *         if self.state == IN_BETWEEN:
*/
    __pyx_t_3 = NULL;
    __pyx_t_4 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_mstate_global->__pyx_kp_u_Database_or_iterator_is_closed};
      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_RuntimeError)), __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 859, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 859, __pyx_L1_error)

    /* "plyvel/_plyvel.pyx":858
 * 
 *     cdef real_next(self):
 *         if self._iter is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database or iterator is closed")
 * 
*/
  }

  /* "plyvel/_plyvel.pyx":861
 *             raise RuntimeError("Database or iterator is closed")
 * 
 *         if self.state == IN_BETWEEN:             # <<<<<<<<<<<<<<
 *             with nogil:
 *                 self._iter.Next()
*/
  switch (__pyx_v_self->state) {
    case __pyx_e_6plyvel_7_plyvel_IN_BETWEEN:

    /* "plyvel/_plyvel.pyx":862
 * 
 *         if self.state == IN_BETWEEN:
 *             with nogil:             # <<<<<<<<<<<<<<
 *                 self._iter.Next()
 *             if not self._iter.Valid():
*/
    {
        PyThreadState * _save;
        _save = PyEval_SaveThread();
        __Pyx_FastGIL_Remember();
        /*try:*/ {

          /* "plyvel/_plyvel.pyx":863
 *         if self.state == IN_BETWEEN:
 *             with nogil:
 *                 self._iter.Next()             # <<<<<<<<<<<<<<
 *             if not self._iter.Valid():
 *                 self.state = AFTER_STOP
*/
          __pyx_v_self->__pyx_base._iter->Next();
        }

        /* "plyvel/_plyvel.pyx":862
 * 
 *         if self.state == IN_BETWEEN:
 *             with nogil:             # <<<<<<<<<<<<<<
 *                 self._iter.Next()
 *             if not self._iter.Valid():
*/
        /*finally:*/ {
          /*normal exit:*/{
            __Pyx_FastGIL_Forget();
            PyEval_RestoreThread(_save);
            goto __pyx_L6;
          }
          __pyx_L6:;
        }
    }

    /* "plyvel/_plyvel.pyx":864
 *             with nogil:
 *                 self._iter.Next()
 *             if not self._iter.Valid():             # <<<<<<<<<<<<<<
 *                 self.state = AFTER_STOP
 *                 raise StopIteration
*/
    __pyx_t_1 = (!(__pyx_v_self->__pyx_base._iter->Valid() != 0));
    if (unlikely(__pyx_t_1)) {

      /* "plyvel/_plyvel.pyx":865
 *                 self._iter.Next()
 *             if not self._iter.Valid():
 *                 self.state = AFTER_STOP             # <<<<<<<<<<<<<<
 *                 raise StopIteration
 *         elif self.state == IN_BETWEEN_ALREADY_POSITIONED:
*/
      __pyx_v_self->state = __pyx_e_6plyvel_7_plyvel_AFTER_STOP;

      /* "plyvel/_plyvel.pyx":866
 *             if not self._iter.Valid():
 *                 self.state = AFTER_STOP
 *                 raise StopIteration             # <<<<<<<<<<<<<<
 *         elif self.state == IN_BETWEEN_ALREADY_POSITIONED:
 *             self.state = IN_BETWEEN
*/
      __Pyx_Raise(((PyObject *)(((PyTypeObject*)PyExc_StopIteration))), 0, 0, 0);
      __PYX_ERR(0, 866, __pyx_L1_error)

      /* "plyvel/_plyvel.pyx":864
 *             with nogil:
 *                 self._iter.Next()
 *             if not self._iter.Valid():             # <<<<<<<<<<<<<<
 *                 self.state = AFTER_STOP
 *                 raise StopIteration
*/
    }

    /* "plyvel/_plyvel.pyx":861
 *             raise RuntimeError("Database or iterator is closed")
 * 
 *         if self.state == IN_BETWEEN:             # <<<<<<<<<<<<<<
 *             with nogil:
 *                 self._iter.Next()
*/
    break;
    case __pyx_e_6plyvel_7_plyvel_IN_BETWEEN_ALREADY_POSITIONED:

    /* "plyvel/_plyvel.pyx":868
 *                 raise StopIteration
 *         elif self.state == IN_BETWEEN_ALREADY_POSITIONED:
 *             self.state = IN_BETWEEN             # <<<<<<<<<<<<<<
 *         elif self.state == BEFORE_START:
 *             if self.start is None:
*/
    __pyx_v_self->state = __pyx_e_6plyvel_7_plyvel_IN_BETWEEN;

    /* "plyvel/_plyvel.pyx":867
 *                 self.state = AFTER_STOP
 *                 raise StopIteration
 *         elif self.state == IN_BETWEEN_ALREADY_POSITIONED:             # <<<<<<<<<<<<<<
 *             self.state = IN_BETWEEN
 *         elif self.state == BEFORE_START:
*/
    break;
    case __pyx_e_6plyvel_7_plyvel_BEFORE_START:

    /* "plyvel/_plyvel.pyx":870
 *             self.state = IN_BETWEEN
 *         elif self.state == BEFORE_START:
 *             if self.start is None:             # <<<<<<<<<<<<<<
 *                 with nogil:
 *                     self._iter.SeekToFirst()
*/
    __pyx_t_1 = (__pyx_v_self->start == ((PyObject*)Py_None));
    if (__pyx_t_1) {

      /* "plyvel/_plyvel.pyx":871
 *         elif self.state == BEFORE_START:
 *             if self.start is None:
 *                 with nogil:             # <<<<<<<<<<<<<<
 *                     self._iter.SeekToFirst()
 *             else:
*/
      {
          PyThreadState * _save;
          _save = PyEval_SaveThread();
          __Pyx_FastGIL_Remember();
          /*try:*/ {

            /* "plyvel/_plyvel.pyx":872
 *             if self.start is None:
 *                 with nogil:
 *                     self._iter.SeekToFirst()             # <<<<<<<<<<<<<<
 *             else:
 *                 with nogil:
*/
            __pyx_v_self->__pyx_base._iter->SeekToFirst();
          }

          /* "plyvel/_plyvel.pyx":871
 *         elif self.state == BEFORE_START:
 *             if self.start is None:
 *                 with nogil:             # <<<<<<<<<<<<<<
 *                     self._iter.SeekToFirst()
 *             else:
*/
          /*finally:*/ {
            /*normal exit:*/{
              __Pyx_FastGIL_Forget();
              PyEval_RestoreThread(_save);
              goto __pyx_L11;
            }
            __pyx_L11:;
          }
      }

      /* "plyvel/_plyvel.pyx":870
 *             self.state = IN_BETWEEN
 *         elif self.state == BEFORE_START:
 *             if self.start is None:             # <<<<<<<<<<<<<<
 *                 with nogil:
 *                     self._iter.SeekToFirst()
*/
      goto __pyx_L8;
    }

    /* "plyvel/_plyvel.pyx":874
 *                     self._iter.SeekToFirst()
 *             else:
 *                 with nogil:             # <<<<<<<<<<<<<<
 *                     self._iter.Seek(self.start_slice)
 *             if not self._iter.Valid():
*/
    /*else*/ {
      {
          PyThreadState * _save;
          _save = PyEval_SaveThread();
          __Pyx_FastGIL_Remember();
          /*try:*/ {

            /* "plyvel/_plyvel.pyx":875
 *             else:
 *                 with nogil:
 *                     self._iter.Seek(self.start_slice)             # <<<<<<<<<<<<<<
 *             if not self._iter.Valid():
 *                 # Iterator is empty
*/
            __pyx_v_self->__pyx_base._iter->Seek(__pyx_v_self->start_slice);
          }

          /* "plyvel/_plyvel.pyx":874
 *                     self._iter.SeekToFirst()
 *             else:
 *                 with nogil:             # <<<<<<<<<<<<<<
 *                     self._iter.Seek(self.start_slice)
 *             if not self._iter.Valid():
*/
          /*finally:*/ {
            /*normal exit:*/{
              __Pyx_FastGIL_Forget();
              PyEval_RestoreThread(_save);
              goto __pyx_L14;
            }
            __pyx_L14:;
          }
      }
    }
    __pyx_L8:;

    /* "plyvel/_plyvel.pyx":876
 *                 with nogil:
 *                     self._iter.Seek(self.start_slice)
 *             if not self._iter.Valid():             # <<<<<<<<<<<<<<
 *                 # Iterator is empty
 *                 raise StopIteration
*/
    __pyx_t_1 = (!(__pyx_v_self->__pyx_base._iter->Valid() != 0));
    if (unlikely(__pyx_t_1)) {

      /* "plyvel/_plyvel.pyx":878
 *             if not self._iter.Valid():
 *                 # Iterator is empty
 *                 raise StopIteration             # <<<<<<<<<<<<<<
 *             if self.start is not None and not self.include_start:
 *                 # Start key is excluded, so skip past it if the db
*/
      __Pyx_Raise(((PyObject *)(((PyTypeObject*)PyExc_StopIteration))), 0, 0, 0);
      __PYX_ERR(0, 878, __pyx_L1_error)

      /* "plyvel/_plyvel.pyx":876
 *                 with nogil:
 *                     self._iter.Seek(self.start_slice)
 *             if not self._iter.Valid():             # <<<<<<<<<<<<<<
 *                 # Iterator is empty
 *                 raise StopIteration
*/
    }

    /* "plyvel/_plyvel.pyx":879
 *                 # Iterator is empty
 *                 raise StopIteration
 *             if self.start is not None and not self.include_start:             # <<<<<<<<<<<<<<
 *                 # Start key is excluded, so skip past it if the db
 *                 # contains it.
*/
    __pyx_t_5 = (__pyx_v_self->start != ((PyObject*)Py_None));
    if (__pyx_t_5) {
    } else {
      __pyx_t_1 = __pyx_t_5;
      goto __pyx_L17_bool_binop_done;
    }
    __pyx_t_5 = (!(__pyx_v_self->include_start != 0));
    __pyx_t_1 = __pyx_t_5;
    __pyx_L17_bool_binop_done:;
    if (__pyx_t_1) {

      /* "plyvel/_plyvel.pyx":883
 *                 # contains it.
 *                 if self.comparator.Compare(self._iter.key(),
 *                                            self.start_slice) == 0:             # <<<<<<<<<<<<<<
 *                     with nogil:
 *                         self._iter.Next()
*/
      __pyx_t_1 = (__pyx_v_self->comparator->Compare(__pyx_v_self->__pyx_base._iter->key(), __pyx_v_self->start_slice) == 0);

      /* "plyvel/_plyvel.pyx":882
 *                 # Start key is excluded, so skip past it if the db
 *                 # contains it.
 *                 if self.comparator.Compare(self._iter.key(),             # <<<<<<<<<<<<<<
 *                                            self.start_slice) == 0:
 *                     with nogil:
*/
      if (__pyx_t_1) {

        /* "plyvel/_plyvel.pyx":884
 *                 if self.comparator.Compare(self._iter.key(),
 *                                            self.start_slice) == 0:
 *                     with nogil:             # <<<<<<<<<<<<<<
 *                         self._iter.Next()
 *                     if not self._iter.Valid():
*/
        {
            PyThreadState * _save;
            _save = PyEval_SaveThread();
            __Pyx_FastGIL_Remember();
            /*try:*/ {

              /* "plyvel/_plyvel.pyx":885
 *                                            self.start_slice) == 0:
 *                     with nogil:
 *                         self._iter.Next()             # <<<<<<<<<<<<<<
 *                     if not self._iter.Valid():
 *                         raise StopIteration
*/
              __pyx_v_self->__pyx_base._iter->Next();
            }

            /* "plyvel/_plyvel.pyx":884
 *                 if self.comparator.Compare(self._iter.key(),
 *                                            self.start_slice) == 0:
 *                     with nogil:             # <<<<<<<<<<<<<<
 *                         self._iter.Next()
 *                     if not self._iter.Valid():
*/
            /*finally:*/ {
              /*normal exit:*/{
                __Pyx_FastGIL_Forget();
                PyEval_RestoreThread(_save);
                goto __pyx_L22;
              }
              __pyx_L22:;
            }
        }

        /* "plyvel/_plyvel.pyx":886
 *                     with nogil:
 *                         self._iter.Next()
 *                     if not self._iter.Valid():             # <<<<<<<<<<<<<<
 *                         raise StopIteration
 *             self.state = IN_BETWEEN
*/
        __pyx_t_1 = (!(__pyx_v_self->__pyx_base._iter->Valid() != 0));
        if (unlikely(__pyx_t_1)) {

          /* "plyvel/_plyvel.pyx":887
 *                         self._iter.Next()
 *                     if not self._iter.Valid():
 *                         raise StopIteration             # <<<<<<<<<<<<<<
 *             self.state = IN_BETWEEN
 *         elif self.state == AFTER_STOP:
*/
          __Pyx_Raise(((PyObject *)(((PyTypeObject*)PyExc_StopIteration))), 0, 0, 0);
          __PYX_ERR(0, 887, __pyx_L1_error)

          /* "plyvel/_plyvel.pyx":886
 *                     with nogil:
 *                         self._iter.Next()
 *                     if not self._iter.Valid():             # <<<<<<<<<<<<<<
 *                         raise StopIteration
 *             self.state = IN_BETWEEN
*/
        }

        /* "plyvel/_plyvel.pyx":882
 *                 # Start key is excluded, so skip past it if the db
 *                 # contains it.
 *                 if self.comparator.Compare(self._iter.key(),             # <<<<<<<<<<<<<<
 *                                            self.start_slice) == 0:
 *                     with nogil:
*/
      }

      /* "plyvel/_plyvel.pyx":879
 *                 # Iterator is empty
 *                 raise StopIteration
 *             if self.start is not None and not self.include_start:             # <<<<<<<<<<<<<<
 *                 # Start key is excluded, so skip past it if the db
 *                 # contains it.
*/
    }

    /* "plyvel/_plyvel.pyx":888
 *                     if not self._iter.Valid():
 *                         raise StopIteration
 *             self.state = IN_BETWEEN             # <<<<<<<<<<<<<<
 *         elif self.state == AFTER_STOP:
 *             raise StopIteration
*/
    __pyx_v_self->state = __pyx_e_6plyvel_7_plyvel_IN_BETWEEN;

    /* "plyvel/_plyvel.pyx":869
 *         elif self.state == IN_BETWEEN_ALREADY_POSITIONED:
 *             self.state = IN_BETWEEN
 *         elif self.state == BEFORE_START:             # <<<<<<<<<<<<<<
 *             if self.start is None:
 *                 with nogil:
*/
    break;
    case __pyx_e_6plyvel_7_plyvel_AFTER_STOP:

    /* "plyvel/_plyvel.pyx":890
 *             self.state = IN_BETWEEN
 *         elif self.state == AFTER_STOP:
 *             raise StopIteration             # <<<<<<<<<<<<<<
 * 
 *         raise_for_status(self._iter.status())
*/
    __Pyx_Raise(((PyObject *)(((PyTypeObject*)PyExc_StopIteration))), 0, 0, 0);
    __PYX_ERR(0, 890, __pyx_L1_error)

    /* "plyvel/_plyvel.pyx":889
 *                         raise StopIteration
 *             self.state = IN_BETWEEN
 *         elif self.state == AFTER_STOP:             # <<<<<<<<<<<<<<
 *             raise StopIteration
 * 
*/
    break;
    default: break;
  }

  /* "plyvel/_plyvel.pyx":892
 *             raise StopIteration
 * 
 *         raise_for_status(self._iter.status())             # <<<<<<<<<<<<<<
 * 
 *         # Check range boundaries
*/
  __pyx_t_6 = __pyx_f_6plyvel_7_plyvel_raise_for_status(__pyx_v_self->__pyx_base._iter->status()); if (unlikely(__pyx_t_6 == ((int)-1))) __PYX_ERR(0, 892, __pyx_L1_error)

  /* "plyvel/_plyvel.pyx":895
 * 
 *         # Check range boundaries
 *         if self.stop is not None:             # <<<<<<<<<<<<<<
 *             n = 1 if self.include_stop else 0
 *             if self.comparator.Compare(self._iter.key(), self.stop_slice) >= n:
*/
  __pyx_t_1 = (__pyx_v_self->stop != ((PyObject*)Py_None));
  if (__pyx_t_1) {

    /* "plyvel/_plyvel.pyx":896
 *         # Check range boundaries
 *         if self.stop is not None:
 *             n = 1 if self.include_stop else 0             # <<<<<<<<<<<<<<
 *             if self.comparator.Compare(self._iter.key(), self.stop_slice) >= n:
 *                 self.state = AFTER_STOP
*/
    __pyx_t_1 = (__pyx_v_self->include_stop != 0);
    if (__pyx_t_1) {
      __pyx_t_7 = 1;
    } else {
      __pyx_t_7 = 0;
    }
    __pyx_v_n = __pyx_t_7;

    /* "plyvel/_plyvel.pyx":897
 *         if self.stop is not None:
 *             n = 1 if self.include_stop else 0
 *             if self.comparator.Compare(self._iter.key(), self.stop_slice) >= n:             # <<<<<<<<<<<<<<
 *                 self.state = AFTER_STOP
 *                 raise StopIteration
*/
    __pyx_t_1 = (__pyx_v_self->comparator->Compare(__pyx_v_self->__pyx_base._iter->key(), __pyx_v_self->stop_slice) >= __pyx_v_n);
    if (unlikely(__pyx_t_1)) {

      /* "plyvel/_plyvel.pyx":898
 *             n = 1 if self.include_stop else 0
 *             if self.comparator.Compare(self._iter.key(), self.stop_slice) >= n:
 *                 self.state = AFTER_STOP             # <<<<<<<<<<<<<<
 *                 raise StopIteration
 * 
*/
      __pyx_v_self->state = __pyx_e_6plyvel_7_plyvel_AFTER_STOP;

      /* "plyvel/_plyvel.pyx":899
 *             if self.comparator.Compare(self._iter.key(), self.stop_slice) >= n:
 *                 self.state = AFTER_STOP
 *                 raise StopIteration             # <<<<<<<<<<<<<<
 * 
 *         return self.current()
*/
      __Pyx_Raise(((PyObject *)(((PyTypeObject*)PyExc_StopIteration))), 0, 0, 0);
      __PYX_ERR(0, 899, __pyx_L1_error)

      /* "plyvel/_plyvel.pyx":897
 *         if self.stop is not None:
 *             n = 1 if self.include_stop else 0
 *             if self.comparator.Compare(self._iter.key(), self.stop_slice) >= n:             # <<<<<<<<<<<<<<
 *                 self.state = AFTER_STOP
 *                 raise StopIteration
*/
    }

    /* "plyvel/_plyvel.pyx":895
 * 
 *         # Check range boundaries
 *         if self.stop is not None:             # <<<<<<<<<<<<<<
 *             n = 1 if self.include_stop else 0
 *             if self.comparator.Compare(self._iter.key(), self.stop_slice) >= n:
*/
  }

  /* "plyvel/_plyvel.pyx":901
 *                 raise StopIteration
 * 
 *         return self.current()             # <<<<<<<<<<<<<<
 * 
 *     cdef real_prev(self):
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __pyx_f_6plyvel_7_plyvel_8Iterator_current(__pyx_v_self); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 901, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "plyvel/_plyvel.pyx":857
 *             return self.real_next()
 * 
 *     cdef real_next(self):             # <<<<<<<<<<<<<<
 *         if self._iter is NULL:
 *             raise RuntimeError("Database or iterator is closed")
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("plyvel._plyvel.Iterator.real_next", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":903
 *         return self.current()
 * 
 *     cdef real_prev(self):             # <<<<<<<<<<<<<<
 *         if self._iter is NULL:
 *             raise RuntimeError("Database or iterator is closed")
*/

static PyObject *__pyx_f_6plyvel_7_plyvel_8Iterator_real_prev(struct __pyx_obj_6plyvel_7_plyvel_Iterator *__pyx_v_self) {
  PyObject *__pyx_v_out = NULL;
  long __pyx_v_n;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;
  long __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("real_prev", 0);

  /* "plyvel/_plyvel.pyx":904
 * 
 *     cdef real_prev(self):
 *         if self._iter is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database or iterator is closed")
 * 
*/
  __pyx_t_1 = (__pyx_v_self->__pyx_base._iter == NULL);
  if (unlikely(__pyx_t_1)) {

    /* "plyvel/_plyvel.pyx":905
 *     cdef real_prev(self):
 *         if self._iter is NULL:
 *             raise RuntimeError("Database or iterator is closed")             # <<<<<<<<<<<<<<
 * 
 *         if self.state == IN_BETWEEN:
*/
    __pyx_t_3 = NULL;
    __pyx_t_4 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_mstate_global->__pyx_kp_u_Database_or_iterator_is_closed};
      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_RuntimeError)), __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 905, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 905, __pyx_L1_error)

    /* "plyvel/_plyvel.pyx":904
 * 
 *     cdef real_prev(self):
 *         if self._iter is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database or iterator is closed")
 * 
*/
  }

  /* "plyvel/_plyvel.pyx":907
 *             raise RuntimeError("Database or iterator is closed")
 * 
 *         if self.state == IN_BETWEEN:             # <<<<<<<<<<<<<<
 *             pass
 *         elif self.state == IN_BETWEEN_ALREADY_POSITIONED:
*/
  switch (__pyx_v_self->state) {
    case __pyx_e_6plyvel_7_plyvel_IN_BETWEEN:
    break;
    case __pyx_e_6plyvel_7_plyvel_IN_BETWEEN_ALREADY_POSITIONED:

    /* "plyvel/_plyvel.pyx":910
 *             pass
 *         elif self.state == IN_BETWEEN_ALREADY_POSITIONED:
 *             assert self._iter.Valid()             # <<<<<<<<<<<<<<
 *             with nogil:
 *                 self._iter.Prev()
*/
    #ifndef CYTHON_WITHOUT_ASSERTIONS
    if (unlikely(__pyx_assertions_enabled())) {
      __pyx_t_1 = (__pyx_v_self->__pyx_base._iter->Valid() != 0);
      if (unlikely(!__pyx_t_1)) {
        __Pyx_Raise(((PyObject *)(((PyTypeObject*)PyExc_AssertionError))), 0, 0, 0);
        __PYX_ERR(0, 910, __pyx_L1_error)
      }
    }
    #else
    if ((1)); else __PYX_ERR(0, 910, __pyx_L1_error)
    #endif

    /* "plyvel/_plyvel.pyx":911
 *         elif self.state == IN_BETWEEN_ALREADY_POSITIONED:
 *             assert self._iter.Valid()
 *             with nogil:             # <<<<<<<<<<<<<<
 *                 self._iter.Prev()
 *             if not self._iter.Valid():
*/
    {
        PyThreadState * _save;
        _save = PyEval_SaveThread();
        __Pyx_FastGIL_Remember();
        /*try:*/ {

          /* "plyvel/_plyvel.pyx":912
 *             assert self._iter.Valid()
 *             with nogil:
 *                 self._iter.Prev()             # <<<<<<<<<<<<<<
 *             if not self._iter.Valid():
 *                 # The .seek() resulted in the first key in the database
*/
          __pyx_v_self->__pyx_base._iter->Prev();
        }

        /* "plyvel/_plyvel.pyx":911
 *         elif self.state == IN_BETWEEN_ALREADY_POSITIONED:
 *             assert self._iter.Valid()
 *             with nogil:             # <<<<<<<<<<<<<<
 *                 self._iter.Prev()
 *             if not self._iter.Valid():
*/
        /*finally:*/ {
          /*normal exit:*/{
            __Pyx_FastGIL_Forget();
            PyEval_RestoreThread(_save);
            goto __pyx_L6;
          }
          __pyx_L6:;
        }
    }

    /* "plyvel/_plyvel.pyx":913
 *             with nogil:
 *                 self._iter.Prev()
 *             if not self._iter.Valid():             # <<<<<<<<<<<<<<
 *                 # The .seek() resulted in the first key in the database
 *                 self.state = BEFORE_START
*/
    __pyx_t_1 = (!(__pyx_v_self->__pyx_base._iter->Valid() != 0));
    if (unlikely(__pyx_t_1)) {

      /* "plyvel/_plyvel.pyx":915
 *             if not self._iter.Valid():
 *                 # The .seek() resulted in the first key in the database
 *                 self.state = BEFORE_START             # <<<<<<<<<<<<<<
 *                 raise StopIteration
 *             raise_for_status(self._iter.status())
*/
      __pyx_v_self->state = __pyx_e_6plyvel_7_plyvel_BEFORE_START;

      /* "plyvel/_plyvel.pyx":916
 *                 # The .seek() resulted in the first key in the database
 *                 self.state = BEFORE_START
 *                 raise StopIteration             # <<<<<<<<<<<<<<
 *             raise_for_status(self._iter.status())
 *         elif self.state == BEFORE_START:
*/
      __Pyx_Raise(((PyObject *)(((PyTypeObject*)PyExc_StopIteration))), 0, 0, 0);
      __PYX_ERR(0, 916, __pyx_L1_error)

      /* "plyvel/_plyvel.pyx":913
 *             with nogil:
 *                 self._iter.Prev()
 *             if not self._iter.Valid():             # <<<<<<<<<<<<<<
 *                 # The .seek() resulted in the first key in the database
 *                 self.state = BEFORE_START
*/
    }

    /* "plyvel/_plyvel.pyx":917
 *                 self.state = BEFORE_START
 *                 raise StopIteration
 *             raise_for_status(self._iter.status())             # <<<<<<<<<<<<<<
 *         elif self.state == BEFORE_START:
 *             raise StopIteration
*/
    __pyx_t_5 = __pyx_f_6plyvel_7_plyvel_raise_for_status(__pyx_v_self->__pyx_base._iter->status()); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(0, 917, __pyx_L1_error)

    /* "plyvel/_plyvel.pyx":909
 *         if self.state == IN_BETWEEN:
 *             pass
 *         elif self.state == IN_BETWEEN_ALREADY_POSITIONED:             # <<<<<<<<<<<<<<
 *             assert self._iter.Valid()
 *             with nogil:
*/
    break;
    case __pyx_e_6plyvel_7_plyvel_BEFORE_START:

    /* "plyvel/_plyvel.pyx":919
 *             raise_for_status(self._iter.status())
 *         elif self.state == BEFORE_START:
 *             raise StopIteration             # <<<<<<<<<<<<<<
 *         elif self.state == AFTER_STOP:
 *             if self.stop is None:
*/
    __Pyx_Raise(((PyObject *)(((PyTypeObject*)PyExc_StopIteration))), 0, 0, 0);
    __PYX_ERR(0, 919, __pyx_L1_error)

    /* "plyvel/_plyvel.pyx":918
 *                 raise StopIteration
 *             raise_for_status(self._iter.status())
 *         elif self.state == BEFORE_START:             # <<<<<<<<<<<<<<
 *             raise StopIteration
 *         elif self.state == AFTER_STOP:
*/
    break;
    case __pyx_e_6plyvel_7_plyvel_AFTER_STOP:

    /* "plyvel/_plyvel.pyx":921
 *             raise StopIteration
 *         elif self.state == AFTER_STOP:
 *             if self.stop is None:             # <<<<<<<<<<<<<<
 *                 # No stop key, seek to last entry
 *                 with nogil:
*/
    __pyx_t_1 = (__pyx_v_self->stop == ((PyObject*)Py_None));
    if (__pyx_t_1) {

      /* "plyvel/_plyvel.pyx":923
 *             if self.stop is None:
 *                 # No stop key, seek to last entry
 *                 with nogil:             # <<<<<<<<<<<<<<
 *                     self._iter.SeekToLast()
 *             else:
*/
      {
          PyThreadState * _save;
          _save = PyEval_SaveThread();
          __Pyx_FastGIL_Remember();
          /*try:*/ {

            /* "plyvel/_plyvel.pyx":924
 *                 # No stop key, seek to last entry
 *                 with nogil:
 *                     self._iter.SeekToLast()             # <<<<<<<<<<<<<<
 *             else:
 *                 # Seek to stop key
*/
            __pyx_v_self->__pyx_base._iter->SeekToLast();
          }

          /* "plyvel/_plyvel.pyx":923
 *             if self.stop is None:
 *                 # No stop key, seek to last entry
 *                 with nogil:             # <<<<<<<<<<<<<<
 *                     self._iter.SeekToLast()
 *             else:
*/
          /*finally:*/ {
            /*normal exit:*/{
              __Pyx_FastGIL_Forget();
              PyEval_RestoreThread(_save);
              goto __pyx_L11;
            }
            __pyx_L11:;
          }
      }

      /* "plyvel/_plyvel.pyx":921
 *             raise StopIteration
 *         elif self.state == AFTER_STOP:
 *             if self.stop is None:             # <<<<<<<<<<<<<<
 *                 # No stop key, seek to last entry
 *                 with nogil:
*/
      goto __pyx_L8;
    }

    /* "plyvel/_plyvel.pyx":927
 *             else:
 *                 # Seek to stop key
 *                 with nogil:             # <<<<<<<<<<<<<<
 *                     self._iter.Seek(self.stop_slice)
 * 
*/
    /*else*/ {
      {
          PyThreadState * _save;
          _save = PyEval_SaveThread();
          __Pyx_FastGIL_Remember();
          /*try:*/ {

            /* "plyvel/_plyvel.pyx":928
 *                 # Seek to stop key
 *                 with nogil:
 *                     self._iter.Seek(self.stop_slice)             # <<<<<<<<<<<<<<
 * 
 *                 if self._iter.Valid():
*/
            __pyx_v_self->__pyx_base._iter->Seek(__pyx_v_self->stop_slice);
          }

          /* "plyvel/_plyvel.pyx":927
 *             else:
 *                 # Seek to stop key
 *                 with nogil:             # <<<<<<<<<<<<<<
 *                     self._iter.Seek(self.stop_slice)
 * 
*/
          /*finally:*/ {
            /*normal exit:*/{
              __Pyx_FastGIL_Forget();
              PyEval_RestoreThread(_save);
              goto __pyx_L14;
            }
            __pyx_L14:;
          }
      }

      /* "plyvel/_plyvel.pyx":930
 *                     self._iter.Seek(self.stop_slice)
 * 
 *                 if self._iter.Valid():             # <<<<<<<<<<<<<<
 *                     # Move one step back if stop is exclusive.
 *                     if not self.include_stop:
*/
      __pyx_t_1 = (__pyx_v_self->__pyx_base._iter->Valid() != 0);
      if (__pyx_t_1) {

        /* "plyvel/_plyvel.pyx":932
 *                 if self._iter.Valid():
 *                     # Move one step back if stop is exclusive.
 *                     if not self.include_stop:             # <<<<<<<<<<<<<<
 *                         with nogil:
 *                             self._iter.Prev()
*/
        __pyx_t_1 = (!(__pyx_v_self->include_stop != 0));
        if (__pyx_t_1) {

          /* "plyvel/_plyvel.pyx":933
 *                     # Move one step back if stop is exclusive.
 *                     if not self.include_stop:
 *                         with nogil:             # <<<<<<<<<<<<<<
 *                             self._iter.Prev()
 *                 else:
*/
          {
              PyThreadState * _save;
              _save = PyEval_SaveThread();
              __Pyx_FastGIL_Remember();
              /*try:*/ {

                /* "plyvel/_plyvel.pyx":934
 *                     if not self.include_stop:
 *                         with nogil:
 *                             self._iter.Prev()             # <<<<<<<<<<<<<<
 *                 else:
 *                     # Stop key did not exist; position at the last
*/
                __pyx_v_self->__pyx_base._iter->Prev();
              }

              /* "plyvel/_plyvel.pyx":933
 *                     # Move one step back if stop is exclusive.
 *                     if not self.include_stop:
 *                         with nogil:             # <<<<<<<<<<<<<<
 *                             self._iter.Prev()
 *                 else:
*/
              /*finally:*/ {
                /*normal exit:*/{
                  __Pyx_FastGIL_Forget();
                  PyEval_RestoreThread(_save);
                  goto __pyx_L19;
                }
                __pyx_L19:;
              }
          }

          /* "plyvel/_plyvel.pyx":932
 *                 if self._iter.Valid():
 *                     # Move one step back if stop is exclusive.
 *                     if not self.include_stop:             # <<<<<<<<<<<<<<
 *                         with nogil:
 *                             self._iter.Prev()
*/
        }

        /* "plyvel/_plyvel.pyx":930
 *                     self._iter.Seek(self.stop_slice)
 * 
 *                 if self._iter.Valid():             # <<<<<<<<<<<<<<
 *                     # Move one step back if stop is exclusive.
 *                     if not self.include_stop:
*/
        goto __pyx_L15;
      }

      /* "plyvel/_plyvel.pyx":938
 *                     # Stop key did not exist; position at the last
 *                     # database entry instead.
 *                     with nogil:             # <<<<<<<<<<<<<<
 *                         self._iter.SeekToLast()
 * 
*/
      /*else*/ {
        {
            PyThreadState * _save;
            _save = PyEval_SaveThread();
            __Pyx_FastGIL_Remember();
            /*try:*/ {

              /* "plyvel/_plyvel.pyx":939
 *                     # database entry instead.
 *                     with nogil:
 *                         self._iter.SeekToLast()             # <<<<<<<<<<<<<<
 * 
 *                 # Make sure the iterator is not past the stop key
*/
              __pyx_v_self->__pyx_base._iter->SeekToLast();
            }

            /* "plyvel/_plyvel.pyx":938
 *                     # Stop key did not exist; position at the last
 *                     # database entry instead.
 *                     with nogil:             # <<<<<<<<<<<<<<
 *                         self._iter.SeekToLast()
 * 
*/
            /*finally:*/ {
              /*normal exit:*/{
                __Pyx_FastGIL_Forget();
                PyEval_RestoreThread(_save);
                goto __pyx_L22;
              }
              __pyx_L22:;
            }
        }
      }
      __pyx_L15:;

      /* "plyvel/_plyvel.pyx":942
 * 
 *                 # Make sure the iterator is not past the stop key
 *                 if self._iter.Valid() and self.comparator.Compare(self._iter.key(), self.stop_slice) > 0:             # <<<<<<<<<<<<<<
 *                     with nogil:
 *                         self._iter.Prev()
*/
      __pyx_t_6 = (__pyx_v_self->__pyx_base._iter->Valid() != 0);
      if (__pyx_t_6) {
      } else {
        __pyx_t_1 = __pyx_t_6;
        goto __pyx_L24_bool_binop_done;
      }
      __pyx_t_6 = (__pyx_v_self->comparator->Compare(__pyx_v_self->__pyx_base._iter->key(), __pyx_v_self->stop_slice) > 0);
      __pyx_t_1 = __pyx_t_6;
      __pyx_L24_bool_binop_done:;
      if (__pyx_t_1) {

        /* "plyvel/_plyvel.pyx":943
 *                 # Make sure the iterator is not past the stop key
 *                 if self._iter.Valid() and self.comparator.Compare(self._iter.key(), self.stop_slice) > 0:
 *                     with nogil:             # <<<<<<<<<<<<<<
 *                         self._iter.Prev()
 * 
*/
        {
            PyThreadState * _save;
            _save = PyEval_SaveThread();
            __Pyx_FastGIL_Remember();
            /*try:*/ {

              /* "plyvel/_plyvel.pyx":944
 *                 if self._iter.Valid() and self.comparator.Compare(self._iter.key(), self.stop_slice) > 0:
 *                     with nogil:
 *                         self._iter.Prev()             # <<<<<<<<<<<<<<
 * 
 *             if not self._iter.Valid():
*/
              __pyx_v_self->__pyx_base._iter->Prev();
            }

            /* "plyvel/_plyvel.pyx":943
 *                 # Make sure the iterator is not past the stop key
 *                 if self._iter.Valid() and self.comparator.Compare(self._iter.key(), self.stop_slice) > 0:
 *                     with nogil:             # <<<<<<<<<<<<<<
 *                         self._iter.Prev()
 * 
*/
            /*finally:*/ {
              /*normal exit:*/{
                __Pyx_FastGIL_Forget();
                PyEval_RestoreThread(_save);
                goto __pyx_L28;
              }
              __pyx_L28:;
            }
        }

        /* "plyvel/_plyvel.pyx":942
 * 
 *                 # Make sure the iterator is not past the stop key
 *                 if self._iter.Valid() and self.comparator.Compare(self._iter.key(), self.stop_slice) > 0:             # <<<<<<<<<<<<<<
 *                     with nogil:
 *                         self._iter.Prev()
*/
      }
    }
    __pyx_L8:;

    /* "plyvel/_plyvel.pyx":946
 *                         self._iter.Prev()
 * 
 *             if not self._iter.Valid():             # <<<<<<<<<<<<<<
 *                 # No entries left
 *                 raise StopIteration
*/
    __pyx_t_1 = (!(__pyx_v_self->__pyx_base._iter->Valid() != 0));
    if (unlikely(__pyx_t_1)) {

      /* "plyvel/_plyvel.pyx":948
 *             if not self._iter.Valid():
 *                 # No entries left
 *                 raise StopIteration             # <<<<<<<<<<<<<<
 * 
 *             # After all the stepping back, we might even have ended up
*/
      __Pyx_Raise(((PyObject *)(((PyTypeObject*)PyExc_StopIteration))), 0, 0, 0);
      __PYX_ERR(0, 948, __pyx_L1_error)

      /* "plyvel/_plyvel.pyx":946
 *                         self._iter.Prev()
 * 
 *             if not self._iter.Valid():             # <<<<<<<<<<<<<<
 *                 # No entries left
 *                 raise StopIteration
*/
    }

    /* "plyvel/_plyvel.pyx":953
 *             # *before* the start key. In this case the iterator does not
 *             # yield any items.
 *             if self.start is not None and self.comparator.Compare(self.start_slice, self._iter.key()) >= 0:             # <<<<<<<<<<<<<<
 *                 raise StopIteration
 * 
*/
    __pyx_t_6 = (__pyx_v_self->start != ((PyObject*)Py_None));
    if (__pyx_t_6) {
    } else {
      __pyx_t_1 = __pyx_t_6;
      goto __pyx_L31_bool_binop_done;
    }
    __pyx_t_6 = (__pyx_v_self->comparator->Compare(__pyx_v_self->start_slice, __pyx_v_self->__pyx_base._iter->key()) >= 0);
    __pyx_t_1 = __pyx_t_6;
    __pyx_L31_bool_binop_done:;
    if (unlikely(__pyx_t_1)) {

      /* "plyvel/_plyvel.pyx":954
 *             # yield any items.
 *             if self.start is not None and self.comparator.Compare(self.start_slice, self._iter.key()) >= 0:
 *                 raise StopIteration             # <<<<<<<<<<<<<<
 * 
 *             raise_for_status(self._iter.status())
*/
      __Pyx_Raise(((PyObject *)(((PyTypeObject*)PyExc_StopIteration))), 0, 0, 0);
      __PYX_ERR(0, 954, __pyx_L1_error)

      /* "plyvel/_plyvel.pyx":953
 *             # *before* the start key. In this case the iterator does not
 *             # yield any items.
 *             if self.start is not None and self.comparator.Compare(self.start_slice, self._iter.key()) >= 0:             # <<<<<<<<<<<<<<
 *                 raise StopIteration
 * 
*/
    }

    /* "plyvel/_plyvel.pyx":956
 *                 raise StopIteration
 * 
 *             raise_for_status(self._iter.status())             # <<<<<<<<<<<<<<
 * 
 *         # Unlike .real_next(), first obtain the value, then move the
*/
    __pyx_t_5 = __pyx_f_6plyvel_7_plyvel_raise_for_status(__pyx_v_self->__pyx_base._iter->status()); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(0, 956, __pyx_L1_error)

    /* "plyvel/_plyvel.pyx":920
 *         elif self.state == BEFORE_START:
 *             raise StopIteration
 *         elif self.state == AFTER_STOP:             # <<<<<<<<<<<<<<
 *             if self.stop is None:
 *                 # No stop key, seek to last entry
*/
    break;
    default: break;
  }

  /* "plyvel/_plyvel.pyx":962
 *         # repeatedly calling it.prev() and next(it) will work as
 *         # designed.
 *         out = self.current()             # <<<<<<<<<<<<<<
 *         with nogil:
 *             self._iter.Prev()
*/
  __pyx_t_2 = __pyx_f_6plyvel_7_plyvel_8Iterator_current(__pyx_v_self); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 962, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_v_out = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "plyvel/_plyvel.pyx":963
 *         # designed.
 *         out = self.current()
 *         with nogil:             # <<<<<<<<<<<<<<
 *             self._iter.Prev()
 *         if not self._iter.Valid():
*/
  {
      PyThreadState * _save;
      _save = PyEval_SaveThread();
      __Pyx_FastGIL_Remember();
      /*try:*/ {

        /* "plyvel/_plyvel.pyx":964
 *         out = self.current()
 *         with nogil:
 *             self._iter.Prev()             # <<<<<<<<<<<<<<
 *         if not self._iter.Valid():
 *             # Moved before the first key in the database
*/
        __pyx_v_self->__pyx_base._iter->Prev();
      }

      /* "plyvel/_plyvel.pyx":963
 *         # designed.
 *         out = self.current()
 *         with nogil:             # <<<<<<<<<<<<<<
 *             self._iter.Prev()
 *         if not self._iter.Valid():
*/
      /*finally:*/ {
        /*normal exit:*/{
          __Pyx_FastGIL_Forget();
          PyEval_RestoreThread(_save);
          goto __pyx_L35;
        }
        __pyx_L35:;
      }
  }

  /* "plyvel/_plyvel.pyx":965
 *         with nogil:
 *             self._iter.Prev()
 *         if not self._iter.Valid():             # <<<<<<<<<<<<<<
 *             # Moved before the first key in the database
 *             self.state = BEFORE_START
*/
  __pyx_t_1 = (!(__pyx_v_self->__pyx_base._iter->Valid() != 0));
  if (__pyx_t_1) {

    /* "plyvel/_plyvel.pyx":967
 *         if not self._iter.Valid():
 *             # Moved before the first key in the database
 *             self.state = BEFORE_START             # <<<<<<<<<<<<<<
 *         else:
 *             if self.start is None:
*/
    __pyx_v_self->state = __pyx_e_6plyvel_7_plyvel_BEFORE_START;

    /* "plyvel/_plyvel.pyx":965
 *         with nogil:
 *             self._iter.Prev()
 *         if not self._iter.Valid():             # <<<<<<<<<<<<<<
 *             # Moved before the first key in the database
 *             self.state = BEFORE_START
*/
    goto __pyx_L36;
  }

  /* "plyvel/_plyvel.pyx":969
 *             self.state = BEFORE_START
 *         else:
 *             if self.start is None:             # <<<<<<<<<<<<<<
 *                 # Iterator is valid
 *                 self.state = IN_BETWEEN
*/
  /*else*/ {
    __pyx_t_1 = (__pyx_v_self->start == ((PyObject*)Py_None));
    if (__pyx_t_1) {

      /* "plyvel/_plyvel.pyx":971
 *             if self.start is None:
 *                 # Iterator is valid
 *                 self.state = IN_BETWEEN             # <<<<<<<<<<<<<<
 *             else:
 *                 # Check range boundaries
*/
      __pyx_v_self->state = __pyx_e_6plyvel_7_plyvel_IN_BETWEEN;

      /* "plyvel/_plyvel.pyx":969
 *             self.state = BEFORE_START
 *         else:
 *             if self.start is None:             # <<<<<<<<<<<<<<
 *                 # Iterator is valid
 *                 self.state = IN_BETWEEN
*/
      goto __pyx_L37;
    }

    /* "plyvel/_plyvel.pyx":974
 *             else:
 *                 # Check range boundaries
 *                 n = 0 if self.include_start else 1             # <<<<<<<<<<<<<<
 *                 if self.comparator.Compare(
 *                         self._iter.key(), self.start_slice) >= n:
*/
    /*else*/ {
      __pyx_t_1 = (__pyx_v_self->include_start != 0);
      if (__pyx_t_1) {
        __pyx_t_7 = 0;
      } else {
        __pyx_t_7 = 1;
      }
      __pyx_v_n = __pyx_t_7;

      /* "plyvel/_plyvel.pyx":976
 *                 n = 0 if self.include_start else 1
 *                 if self.comparator.Compare(
 *                         self._iter.key(), self.start_slice) >= n:             # <<<<<<<<<<<<<<
 *                     # Iterator is valid and within range boundaries
 *                     self.state = IN_BETWEEN
*/
      __pyx_t_1 = (__pyx_v_self->comparator->Compare(__pyx_v_self->__pyx_base._iter->key(), __pyx_v_self->start_slice) >= __pyx_v_n);

      /* "plyvel/_plyvel.pyx":975
 *                 # Check range boundaries
 *                 n = 0 if self.include_start else 1
 *                 if self.comparator.Compare(             # <<<<<<<<<<<<<<
 *                         self._iter.key(), self.start_slice) >= n:
 *                     # Iterator is valid and within range boundaries
*/
      if (__pyx_t_1) {

        /* "plyvel/_plyvel.pyx":978
 *                         self._iter.key(), self.start_slice) >= n:
 *                     # Iterator is valid and within range boundaries
 *                     self.state = IN_BETWEEN             # <<<<<<<<<<<<<<
 *                 else:
 *                     # Iterator is valid, but has moved before the
*/
        __pyx_v_self->state = __pyx_e_6plyvel_7_plyvel_IN_BETWEEN;

        /* "plyvel/_plyvel.pyx":975
 *                 # Check range boundaries
 *                 n = 0 if self.include_start else 1
 *                 if self.comparator.Compare(             # <<<<<<<<<<<<<<
 *                         self._iter.key(), self.start_slice) >= n:
 *                     # Iterator is valid and within range boundaries
*/
        goto __pyx_L38;
      }

      /* "plyvel/_plyvel.pyx":982
 *                     # Iterator is valid, but has moved before the
 *                     # 'start' key
 *                     self.state = BEFORE_START             # <<<<<<<<<<<<<<
 * 
 *         raise_for_status(self._iter.status())
*/
      /*else*/ {
        __pyx_v_self->state = __pyx_e_6plyvel_7_plyvel_BEFORE_START;
      }
      __pyx_L38:;
    }
    __pyx_L37:;
  }
  __pyx_L36:;

  /* "plyvel/_plyvel.pyx":984
 *                     self.state = BEFORE_START
 * 
 *         raise_for_status(self._iter.status())             # <<<<<<<<<<<<<<
 *         return out
 * 
*/
  __pyx_t_5 = __pyx_f_6plyvel_7_plyvel_raise_for_status(__pyx_v_self->__pyx_base._iter->status()); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(0, 984, __pyx_L1_error)

  /* "plyvel/_plyvel.pyx":985
 * 
 *         raise_for_status(self._iter.status())
 *         return out             # <<<<<<<<<<<<<<
 * 
 *     def seek_to_start(self):
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_out);
  __pyx_r = __pyx_v_out;
  goto __pyx_L0;

  /* "plyvel/_plyvel.pyx":903
 *         return self.current()
 * 
 *     cdef real_prev(self):             # <<<<<<<<<<<<<<
 *         if self._iter is NULL:
 *             raise RuntimeError("Database or iterator is closed")
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("plyvel._plyvel.Iterator.real_prev", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_out);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":987
 *         return out
 * 
 *     def seek_to_start(self):             # <<<<<<<<<<<<<<
 *         if self._iter is NULL:
 *             raise RuntimeError("Database or iterator is closed")
*/

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_8Iterator_9seek_to_start(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6plyvel_7_plyvel_8Iterator_8seek_to_start, "Iterator.seek_to_start(self)");
static PyMethodDef __pyx_mdef_6plyvel_7_plyvel_8Iterator_9seek_to_start = {"seek_to_start", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_8Iterator_9seek_to_start, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_8Iterator_8seek_to_start};
static PyObject *__pyx_pw_6plyvel_7_plyvel_8Iterator_9seek_to_start(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("seek_to_start (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("seek_to_start", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("seek_to_start", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_8Iterator_8seek_to_start(((struct __pyx_obj_6plyvel_7_plyvel_Iterator *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_8Iterator_8seek_to_start(struct __pyx_obj_6plyvel_7_plyvel_Iterator *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("seek_to_start", 0);

  /* "plyvel/_plyvel.pyx":988
 * 
 *     def seek_to_start(self):
 *         if self._iter is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database or iterator is closed")
 * 
*/
  __pyx_t_1 = (__pyx_v_self->__pyx_base._iter == NULL);
  if (unlikely(__pyx_t_1)) {

    /* "plyvel/_plyvel.pyx":989
 *     def seek_to_start(self):
 *         if self._iter is NULL:
 *             raise RuntimeError("Database or iterator is closed")             # <<<<<<<<<<<<<<
 * 
 *         self.state = BEFORE_START
*/
    __pyx_t_3 = NULL;
    __pyx_t_4 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_mstate_global->__pyx_kp_u_Database_or_iterator_is_closed};
      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_RuntimeError)), __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 989, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 989, __pyx_L1_error)

    /* "plyvel/_plyvel.pyx":988
 * 
 *     def seek_to_start(self):
 *         if self._iter is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database or iterator is closed")
 * 
*/
  }

  /* "plyvel/_plyvel.pyx":991
 *             raise RuntimeError("Database or iterator is closed")
 * 
 *         self.state = BEFORE_START             # <<<<<<<<<<<<<<
 * 
 *     def seek_to_stop(self):
*/
  __pyx_v_self->state = __pyx_e_6plyvel_7_plyvel_BEFORE_START;

  /* "plyvel/_plyvel.pyx":987
 *         return out
 * 
 *     def seek_to_start(self):             # <<<<<<<<<<<<<<
 *         if self._iter is NULL:
 *             raise RuntimeError("Database or iterator is closed")
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("plyvel._plyvel.Iterator.seek_to_start", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":993
 *         self.state = BEFORE_START
 * 
 *     def seek_to_stop(self):             # <<<<<<<<<<<<<<
 *         if self._iter is NULL:
 *             raise RuntimeError("Database or iterator is closed")
*/

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_8Iterator_11seek_to_stop(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6plyvel_7_plyvel_8Iterator_10seek_to_stop, "Iterator.seek_to_stop(self)");
static PyMethodDef __pyx_mdef_6plyvel_7_plyvel_8Iterator_11seek_to_stop = {"seek_to_stop", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_8Iterator_11seek_to_stop, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_8Iterator_10seek_to_stop};
static PyObject *__pyx_pw_6plyvel_7_plyvel_8Iterator_11seek_to_stop(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("seek_to_stop (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("seek_to_stop", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("seek_to_stop", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_8Iterator_10seek_to_stop(((struct __pyx_obj_6plyvel_7_plyvel_Iterator *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_8Iterator_10seek_to_stop(struct __pyx_obj_6plyvel_7_plyvel_Iterator *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("seek_to_stop", 0);

  /* "plyvel/_plyvel.pyx":994
 * 
 *     def seek_to_stop(self):
 *         if self._iter is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database or iterator is closed")
 * 
*/
  __pyx_t_1 = (__pyx_v_self->__pyx_base._iter == NULL);
  if (unlikely(__pyx_t_1)) {

    /* "plyvel/_plyvel.pyx":995
 *     def seek_to_stop(self):
 *         if self._iter is NULL:
 *             raise RuntimeError("Database or iterator is closed")             # <<<<<<<<<<<<<<
 * 
 *         self.state = AFTER_STOP
*/
    __pyx_t_3 = NULL;
    __pyx_t_4 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_mstate_global->__pyx_kp_u_Database_or_iterator_is_closed};
      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_RuntimeError)), __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 995, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 995, __pyx_L1_error)

    /* "plyvel/_plyvel.pyx":994
 * 
 *     def seek_to_stop(self):
 *         if self._iter is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database or iterator is closed")
 * 
*/
  }

  /* "plyvel/_plyvel.pyx":997
 *             raise RuntimeError("Database or iterator is closed")
 * 
 *         self.state = AFTER_STOP             # <<<<<<<<<<<<<<
 * 
 *     def seek(self, bytes target not None):
*/
  __pyx_v_self->state = __pyx_e_6plyvel_7_plyvel_AFTER_STOP;

  /* "plyvel/_plyvel.pyx":993
 *         self.state = BEFORE_START
 * 
 *     def seek_to_stop(self):             # <<<<<<<<<<<<<<
 *         if self._iter is NULL:
 *             raise RuntimeError("Database or iterator is closed")
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("plyvel._plyvel.Iterator.seek_to_stop", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":999
 *         self.state = AFTER_STOP
 * 
 *     def seek(self, bytes target not None):             # <<<<<<<<<<<<<<
 *         if self._iter is NULL:
 *             raise RuntimeError("Database or iterator is closed")
*/

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_8Iterator_13seek(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6plyvel_7_plyvel_8Iterator_12seek, "Iterator.seek(self, bytes target)");
static PyMethodDef __pyx_mdef_6plyvel_7_plyvel_8Iterator_13seek = {"seek", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_8Iterator_13seek, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_8Iterator_12seek};
static PyObject *__pyx_pw_6plyvel_7_plyvel_8Iterator_13seek(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_target = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("seek (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_target,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(0, 999, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 999, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "seek", 0) < (0)) __PYX_ERR(0, 999, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("seek", 1, 1, 1, i); __PYX_ERR(0, 999, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 999, __pyx_L3_error)
    }
    __pyx_v_target = ((PyObject*)values[0]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("seek", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 999, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("plyvel._plyvel.Iterator.seek", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_target), (&PyBytes_Type), 0, "target", 1))) __PYX_ERR(0, 999, __pyx_L1_error)
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_8Iterator_12seek(((struct __pyx_obj_6plyvel_7_plyvel_Iterator *)__pyx_v_self), __pyx_v_target);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_8Iterator_12seek(struct __pyx_obj_6plyvel_7_plyvel_Iterator *__pyx_v_self, PyObject *__pyx_v_target) {
  leveldb::Slice __pyx_v_target_slice;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  const char *__pyx_t_5;
  Py_ssize_t __pyx_t_6;
  int __pyx_t_7;
  leveldb::Slice __pyx_t_8;
  int __pyx_t_9;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("seek", 0);
  __Pyx_INCREF(__pyx_v_target);

  /* "plyvel/_plyvel.pyx":1000
 * 
 *     def seek(self, bytes target not None):
 *         if self._iter is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database or iterator is closed")
 * 
*/
  __pyx_t_1 = (__pyx_v_self->__pyx_base._iter == NULL);
  if (unlikely(__pyx_t_1)) {

    /* "plyvel/_plyvel.pyx":1001
 *     def seek(self, bytes target not None):
 *         if self._iter is NULL:
 *             raise RuntimeError("Database or iterator is closed")             # <<<<<<<<<<<<<<
 * 
 *         if self.db_prefix is not None:
*/
    __pyx_t_3 = NULL;
    __pyx_t_4 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_mstate_global->__pyx_kp_u_Database_or_iterator_is_closed};
      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_RuntimeError)), __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1001, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 1001, __pyx_L1_error)

    /* "plyvel/_plyvel.pyx":1000
 * 
 *     def seek(self, bytes target not None):
 *         if self._iter is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database or iterator is closed")
 * 
*/
  }

  /* "plyvel/_plyvel.pyx":1003
 *             raise RuntimeError("Database or iterator is closed")
 * 
 *         if self.db_prefix is not None:             # <<<<<<<<<<<<<<
 *             target = self.db_prefix + target
 * 
*/
  __pyx_t_1 = (__pyx_v_self->db_prefix != ((PyObject*)Py_None));
  if (__pyx_t_1) {

    /* "plyvel/_plyvel.pyx":1004
 * 
 *         if self.db_prefix is not None:
 *             target = self.db_prefix + target             # <<<<<<<<<<<<<<
 * 
 *         cdef Slice target_slice = Slice(target, len(target))
*/
    __pyx_t_2 = PyNumber_Add(__pyx_v_self->db_prefix, __pyx_v_target); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1004, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF_SET(__pyx_v_target, ((PyObject*)__pyx_t_2));
    __pyx_t_2 = 0;

    /* "plyvel/_plyvel.pyx":1003
 *             raise RuntimeError("Database or iterator is closed")
 * 
 *         if self.db_prefix is not None:             # <<<<<<<<<<<<<<
 *             target = self.db_prefix + target
 * 
*/
  }

  /* "plyvel/_plyvel.pyx":1006
 *             target = self.db_prefix + target
 * 
 *         cdef Slice target_slice = Slice(target, len(target))             # <<<<<<<<<<<<<<
 * 
 *         # Seek only within the start/stop boundaries
*/
  __pyx_t_5 = __Pyx_PyBytes_AsString(__pyx_v_target); if (unlikely((!__pyx_t_5) && PyErr_Occurred())) __PYX_ERR(0, 1006, __pyx_L1_error)
  __pyx_t_6 = __Pyx_PyBytes_GET_SIZE(__pyx_v_target); if (unlikely(__pyx_t_6 == ((Py_ssize_t)-1))) __PYX_ERR(0, 1006, __pyx_L1_error)
  __pyx_v_target_slice = leveldb::Slice(__pyx_t_5, __pyx_t_6);

  /* "plyvel/_plyvel.pyx":1009
 * 
 *         # Seek only within the start/stop boundaries
 *         if self.start is not None and self.comparator.Compare(             # <<<<<<<<<<<<<<
 *                 target_slice, self.start_slice) < 0:
 *             target_slice = self.start_slice
*/
  __pyx_t_7 = (__pyx_v_self->start != ((PyObject*)Py_None));
  if (__pyx_t_7) {
  } else {
    __pyx_t_1 = __pyx_t_7;
    goto __pyx_L6_bool_binop_done;
  }

  /* "plyvel/_plyvel.pyx":1010
 *         # Seek only within the start/stop boundaries
 *         if self.start is not None and self.comparator.Compare(
 *                 target_slice, self.start_slice) < 0:             # <<<<<<<<<<<<<<
 *             target_slice = self.start_slice
 *         if self.stop is not None and self.comparator.Compare(
*/
  __pyx_t_7 = (__pyx_v_self->comparator->Compare(__pyx_v_target_slice, __pyx_v_self->start_slice) < 0);
  __pyx_t_1 = __pyx_t_7;
  __pyx_L6_bool_binop_done:;

  /* "plyvel/_plyvel.pyx":1009
 * 
 *         # Seek only within the start/stop boundaries
 *         if self.start is not None and self.comparator.Compare(             # <<<<<<<<<<<<<<
 *                 target_slice, self.start_slice) < 0:
 *             target_slice = self.start_slice
*/
  if (__pyx_t_1) {

    /* "plyvel/_plyvel.pyx":1011
 *         if self.start is not None and self.comparator.Compare(
 *                 target_slice, self.start_slice) < 0:
 *             target_slice = self.start_slice             # <<<<<<<<<<<<<<
 *         if self.stop is not None and self.comparator.Compare(
 *                 target_slice, self.stop_slice) > 0:
*/
    __pyx_t_8 = __pyx_v_self->start_slice;
    __pyx_v_target_slice = __PYX_STD_MOVE_IF_SUPPORTED(__pyx_t_8);

    /* "plyvel/_plyvel.pyx":1009
 * 
 *         # Seek only within the start/stop boundaries
 *         if self.start is not None and self.comparator.Compare(             # <<<<<<<<<<<<<<
 *                 target_slice, self.start_slice) < 0:
 *             target_slice = self.start_slice
*/
  }

  /* "plyvel/_plyvel.pyx":1012
 *                 target_slice, self.start_slice) < 0:
 *             target_slice = self.start_slice
 *         if self.stop is not None and self.comparator.Compare(             # <<<<<<<<<<<<<<
 *                 target_slice, self.stop_slice) > 0:
 *             target_slice = self.stop_slice
*/
  __pyx_t_7 = (__pyx_v_self->stop != ((PyObject*)Py_None));
  if (__pyx_t_7) {
  } else {
    __pyx_t_1 = __pyx_t_7;
    goto __pyx_L9_bool_binop_done;
  }

  /* "plyvel/_plyvel.pyx":1013
 *             target_slice = self.start_slice
 *         if self.stop is not None and self.comparator.Compare(
 *                 target_slice, self.stop_slice) > 0:             # <<<<<<<<<<<<<<
 *             target_slice = self.stop_slice
 * 
*/
  __pyx_t_7 = (__pyx_v_self->comparator->Compare(__pyx_v_target_slice, __pyx_v_self->stop_slice) > 0);
  __pyx_t_1 = __pyx_t_7;
  __pyx_L9_bool_binop_done:;

  /* "plyvel/_plyvel.pyx":1012
 *                 target_slice, self.start_slice) < 0:
 *             target_slice = self.start_slice
 *         if self.stop is not None and self.comparator.Compare(             # <<<<<<<<<<<<<<
 *                 target_slice, self.stop_slice) > 0:
 *             target_slice = self.stop_slice
*/
  if (__pyx_t_1) {

    /* "plyvel/_plyvel.pyx":1014
 *         if self.stop is not None and self.comparator.Compare(
 *                 target_slice, self.stop_slice) > 0:
 *             target_slice = self.stop_slice             # <<<<<<<<<<<<<<
 * 
 *         with nogil:
*/
    __pyx_t_8 = __pyx_v_self->stop_slice;
    __pyx_v_target_slice = __PYX_STD_MOVE_IF_SUPPORTED(__pyx_t_8);

    /* "plyvel/_plyvel.pyx":1012
 *                 target_slice, self.start_slice) < 0:
 *             target_slice = self.start_slice
 *         if self.stop is not None and self.comparator.Compare(             # <<<<<<<<<<<<<<
 *                 target_slice, self.stop_slice) > 0:
 *             target_slice = self.stop_slice
*/
  }

  /* "plyvel/_plyvel.pyx":1016
 *             target_slice = self.stop_slice
 * 
 *         with nogil:             # <<<<<<<<<<<<<<
 *             self._iter.Seek(target_slice)
 *         if not self._iter.Valid():
*/
  {
      PyThreadState * _save;
      _save = PyEval_SaveThread();
      __Pyx_FastGIL_Remember();
      /*try:*/ {

        /* "plyvel/_plyvel.pyx":1017
 * 
 *         with nogil:
 *             self._iter.Seek(target_slice)             # <<<<<<<<<<<<<<
 *         if not self._iter.Valid():
 *             # Moved past the end (or empty database)
*/
        __pyx_v_self->__pyx_base._iter->Seek(__pyx_v_target_slice);
      }

      /* "plyvel/_plyvel.pyx":1016
 *             target_slice = self.stop_slice
 * 
 *         with nogil:             # <<<<<<<<<<<<<<
 *             self._iter.Seek(target_slice)
 *         if not self._iter.Valid():
*/
      /*finally:*/ {
        /*normal exit:*/{
          __Pyx_FastGIL_Forget();
          PyEval_RestoreThread(_save);
          goto __pyx_L13;
        }
        __pyx_L13:;
      }
  }

  /* "plyvel/_plyvel.pyx":1018
 *         with nogil:
 *             self._iter.Seek(target_slice)
 *         if not self._iter.Valid():             # <<<<<<<<<<<<<<
 *             # Moved past the end (or empty database)
 *             self.state = AFTER_STOP
*/
  __pyx_t_1 = (!(__pyx_v_self->__pyx_base._iter->Valid() != 0));
  if (__pyx_t_1) {

    /* "plyvel/_plyvel.pyx":1020
 *         if not self._iter.Valid():
 *             # Moved past the end (or empty database)
 *             self.state = AFTER_STOP             # <<<<<<<<<<<<<<
 *             return
 * 
*/
    __pyx_v_self->state = __pyx_e_6plyvel_7_plyvel_AFTER_STOP;

    /* "plyvel/_plyvel.pyx":1021
 *             # Moved past the end (or empty database)
 *             self.state = AFTER_STOP
 *             return             # <<<<<<<<<<<<<<
 * 
 *         self.state = IN_BETWEEN_ALREADY_POSITIONED
*/
    __Pyx_XDECREF(__pyx_r);
    __pyx_r = Py_None; __Pyx_INCREF(Py_None);
    goto __pyx_L0;

    /* "plyvel/_plyvel.pyx":1018
 *         with nogil:
 *             self._iter.Seek(target_slice)
 *         if not self._iter.Valid():             # <<<<<<<<<<<<<<
 *             # Moved past the end (or empty database)
 *             self.state = AFTER_STOP
*/
  }

  /* "plyvel/_plyvel.pyx":1023
 *             return
 * 
 *         self.state = IN_BETWEEN_ALREADY_POSITIONED             # <<<<<<<<<<<<<<
 *         raise_for_status(self._iter.status())
 * 
*/
  __pyx_v_self->state = __pyx_e_6plyvel_7_plyvel_IN_BETWEEN_ALREADY_POSITIONED;

  /* "plyvel/_plyvel.pyx":1024
 * 
 *         self.state = IN_BETWEEN_ALREADY_POSITIONED
 *         raise_for_status(self._iter.status())             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __pyx_t_9 = __pyx_f_6plyvel_7_plyvel_raise_for_status(__pyx_v_self->__pyx_base._iter->status()); if (unlikely(__pyx_t_9 == ((int)-1))) __PYX_ERR(0, 1024, __pyx_L1_error)

  /* "plyvel/_plyvel.pyx":999
 *         self.state = AFTER_STOP
 * 
 *     def seek(self, bytes target not None):             # <<<<<<<<<<<<<<
 *         if self._iter is NULL:
 *             raise RuntimeError("Database or iterator is closed")
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("plyvel._plyvel.Iterator.seek", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_target);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self._iter,self.comparator,self.start_slice,self.stop_slice cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):
*/

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_8Iterator_15__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6plyvel_7_plyvel_8Iterator_14__reduce_cython__, "Iterator.__reduce_cython__(self)");
static PyMethodDef __pyx_mdef_6plyvel_7_plyvel_8Iterator_15__reduce_cython__ = {"__reduce_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_8Iterator_15__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_8Iterator_14__reduce_cython__};
static PyObject *__pyx_pw_6plyvel_7_plyvel_8Iterator_15__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("__reduce_cython__", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("__reduce_cython__", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_8Iterator_14__reduce_cython__(((struct __pyx_obj_6plyvel_7_plyvel_Iterator *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_8Iterator_14__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_6plyvel_7_plyvel_Iterator *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError, "self._iter,self.comparator,self.start_slice,self.stop_slice cannot be converted to a Python object for pickling"             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError, "self._iter,self.comparator,self.start_slice,self.stop_slice cannot be converted to a Python object for pickling"
*/
  __Pyx_Raise(((PyObject *)(((PyTypeObject*)PyExc_TypeError))), __pyx_mstate_global->__pyx_kp_u_self__iter_self_comparator_self, 0, 0);
  __PYX_ERR(1, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self._iter,self.comparator,self.start_slice,self.stop_slice cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("plyvel._plyvel.Iterator.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError, "self._iter,self.comparator,self.start_slice,self.stop_slice cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self._iter,self.comparator,self.start_slice,self.stop_slice cannot be converted to a Python object for pickling"
*/

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_8Iterator_17__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6plyvel_7_plyvel_8Iterator_16__setstate_cython__, "Iterator.__setstate_cython__(self, __pyx_state)");
static PyMethodDef __pyx_mdef_6plyvel_7_plyvel_8Iterator_17__setstate_cython__ = {"__setstate_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_8Iterator_17__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_8Iterator_16__setstate_cython__};
static PyObject *__pyx_pw_6plyvel_7_plyvel_8Iterator_17__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  CYTHON_UNUSED PyObject *__pyx_v___pyx_state = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_pyx_state,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(1, 3, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(1, 3, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__setstate_cython__", 0) < (0)) __PYX_ERR(1, 3, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, i); __PYX_ERR(1, 3, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(1, 3, __pyx_L3_error)
    }
    __pyx_v___pyx_state = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, __pyx_nargs); __PYX_ERR(1, 3, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("plyvel._plyvel.Iterator.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_8Iterator_16__setstate_cython__(((struct __pyx_obj_6plyvel_7_plyvel_Iterator *)__pyx_v_self), __pyx_v___pyx_state);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_8Iterator_16__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_6plyvel_7_plyvel_Iterator *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError, "self._iter,self.comparator,self.start_slice,self.stop_slice cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError, "self._iter,self.comparator,self.start_slice,self.stop_slice cannot be converted to a Python object for pickling"             # <<<<<<<<<<<<<<
*/
  __Pyx_Raise(((PyObject *)(((PyTypeObject*)PyExc_TypeError))), __pyx_mstate_global->__pyx_kp_u_self__iter_self_comparator_self, 0, 0);
  __PYX_ERR(1, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError, "self._iter,self.comparator,self.start_slice,self.stop_slice cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self._iter,self.comparator,self.start_slice,self.stop_slice cannot be converted to a Python object for pickling"
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("plyvel._plyvel.Iterator.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":1029
 * @cython.final
 * cdef class RawIterator(BaseIterator):
 *     def valid(self):             # <<<<<<<<<<<<<<
 *         if self._iter is NULL:
 *             raise RuntimeError("Database or iterator is closed")
*/

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_11RawIterator_1valid(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6plyvel_7_plyvel_11RawIterator_valid, "RawIterator.valid(self)");
static PyMethodDef __pyx_mdef_6plyvel_7_plyvel_11RawIterator_1valid = {"valid", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_11RawIterator_1valid, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_11RawIterator_valid};
static PyObject *__pyx_pw_6plyvel_7_plyvel_11RawIterator_1valid(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("valid (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("valid", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("valid", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_11RawIterator_valid(((struct __pyx_obj_6plyvel_7_plyvel_RawIterator *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_11RawIterator_valid(struct __pyx_obj_6plyvel_7_plyvel_RawIterator *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("valid", 0);

  /* "plyvel/_plyvel.pyx":1030
 * cdef class RawIterator(BaseIterator):
 *     def valid(self):
 *         if self._iter is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database or iterator is closed")
 * 
*/
  __pyx_t_1 = (__pyx_v_self->__pyx_base._iter == NULL);
  if (unlikely(__pyx_t_1)) {

    /* "plyvel/_plyvel.pyx":1031
 *     def valid(self):
 *         if self._iter is NULL:
 *             raise RuntimeError("Database or iterator is closed")             # <<<<<<<<<<<<<<
 * 
 *         return self._iter.Valid()
*/
    __pyx_t_3 = NULL;
    __pyx_t_4 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_mstate_global->__pyx_kp_u_Database_or_iterator_is_closed};
      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_RuntimeError)), __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1031, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 1031, __pyx_L1_error)

    /* "plyvel/_plyvel.pyx":1030
 * cdef class RawIterator(BaseIterator):
 *     def valid(self):
 *         if self._iter is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database or iterator is closed")
 * 
*/
  }

  /* "plyvel/_plyvel.pyx":1033
 *             raise RuntimeError("Database or iterator is closed")
 * 
 *         return self._iter.Valid()             # <<<<<<<<<<<<<<
 * 
 *     def seek_to_first(self):
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base._iter->Valid()); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1033, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "plyvel/_plyvel.pyx":1029
 * @cython.final
 * cdef class RawIterator(BaseIterator):
 *     def valid(self):             # <<<<<<<<<<<<<<
 *         if self._iter is NULL:
 *             raise RuntimeError("Database or iterator is closed")
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("plyvel._plyvel.RawIterator.valid", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":1035
 *         return self._iter.Valid()
 * 
 *     def seek_to_first(self):             # <<<<<<<<<<<<<<
 *         if self._iter is NULL:
 *             raise RuntimeError("Database or iterator is closed")
*/

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_11RawIterator_3seek_to_first(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6plyvel_7_plyvel_11RawIterator_2seek_to_first, "RawIterator.seek_to_first(self)");
static PyMethodDef __pyx_mdef_6plyvel_7_plyvel_11RawIterator_3seek_to_first = {"seek_to_first", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_11RawIterator_3seek_to_first, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_11RawIterator_2seek_to_first};
static PyObject *__pyx_pw_6plyvel_7_plyvel_11RawIterator_3seek_to_first(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("seek_to_first (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("seek_to_first", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("seek_to_first", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_11RawIterator_2seek_to_first(((struct __pyx_obj_6plyvel_7_plyvel_RawIterator *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_11RawIterator_2seek_to_first(struct __pyx_obj_6plyvel_7_plyvel_RawIterator *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  int __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("seek_to_first", 0);

  /* "plyvel/_plyvel.pyx":1036
 * 
 *     def seek_to_first(self):
 *         if self._iter is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database or iterator is closed")
 * 
*/
  __pyx_t_1 = (__pyx_v_self->__pyx_base._iter == NULL);
  if (unlikely(__pyx_t_1)) {

    /* "plyvel/_plyvel.pyx":1037
 *     def seek_to_first(self):
 *         if self._iter is NULL:
 *             raise RuntimeError("Database or iterator is closed")             # <<<<<<<<<<<<<<
 * 
 *         with nogil:
*/
    __pyx_t_3 = NULL;
    __pyx_t_4 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_mstate_global->__pyx_kp_u_Database_or_iterator_is_closed};
      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_RuntimeError)), __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1037, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 1037, __pyx_L1_error)

    /* "plyvel/_plyvel.pyx":1036
 * 
 *     def seek_to_first(self):
 *         if self._iter is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database or iterator is closed")
 * 
*/
  }

  /* "plyvel/_plyvel.pyx":1039
 *             raise RuntimeError("Database or iterator is closed")
 * 
 *         with nogil:             # <<<<<<<<<<<<<<
 *             self._iter.SeekToFirst()
 * 
*/
  {
      PyThreadState * _save;
      _save = PyEval_SaveThread();
      __Pyx_FastGIL_Remember();
      /*try:*/ {

        /* "plyvel/_plyvel.pyx":1040
 * 
 *         with nogil:
 *             self._iter.SeekToFirst()             # <<<<<<<<<<<<<<
 * 
 *         raise_for_status(self._iter.status())
*/
        __pyx_v_self->__pyx_base._iter->SeekToFirst();
      }

      /* "plyvel/_plyvel.pyx":1039
 *             raise RuntimeError("Database or iterator is closed")
 * 
 *         with nogil:             # <<<<<<<<<<<<<<
 *             self._iter.SeekToFirst()
 * 
*/
      /*finally:*/ {
        /*normal exit:*/{
          __Pyx_FastGIL_Forget();
          PyEval_RestoreThread(_save);
          goto __pyx_L6;
        }
        __pyx_L6:;
      }
  }

  /* "plyvel/_plyvel.pyx":1042
 *             self._iter.SeekToFirst()
 * 
 *         raise_for_status(self._iter.status())             # <<<<<<<<<<<<<<
 * 
 *     def seek_to_last(self):
*/
  __pyx_t_5 = __pyx_f_6plyvel_7_plyvel_raise_for_status(__pyx_v_self->__pyx_base._iter->status()); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(0, 1042, __pyx_L1_error)

  /* "plyvel/_plyvel.pyx":1035
 *         return self._iter.Valid()
 * 
 *     def seek_to_first(self):             # <<<<<<<<<<<<<<
 *         if self._iter is NULL:
 *             raise RuntimeError("Database or iterator is closed")
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("plyvel._plyvel.RawIterator.seek_to_first", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":1044
 *         raise_for_status(self._iter.status())
 * 
 *     def seek_to_last(self):             # <<<<<<<<<<<<<<
 *         if self._iter is NULL:
 *             raise RuntimeError("Database or iterator is closed")
*/

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_11RawIterator_5seek_to_last(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6plyvel_7_plyvel_11RawIterator_4seek_to_last, "RawIterator.seek_to_last(self)");
static PyMethodDef __pyx_mdef_6plyvel_7_plyvel_11RawIterator_5seek_to_last = {"seek_to_last", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_11RawIterator_5seek_to_last, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_11RawIterator_4seek_to_last};
static PyObject *__pyx_pw_6plyvel_7_plyvel_11RawIterator_5seek_to_last(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("seek_to_last (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("seek_to_last", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("seek_to_last", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_11RawIterator_4seek_to_last(((struct __pyx_obj_6plyvel_7_plyvel_RawIterator *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_11RawIterator_4seek_to_last(struct __pyx_obj_6plyvel_7_plyvel_RawIterator *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  int __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("seek_to_last", 0);

  /* "plyvel/_plyvel.pyx":1045
 * 
 *     def seek_to_last(self):
 *         if self._iter is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database or iterator is closed")
 * 
*/
  __pyx_t_1 = (__pyx_v_self->__pyx_base._iter == NULL);
  if (unlikely(__pyx_t_1)) {

    /* "plyvel/_plyvel.pyx":1046
 *     def seek_to_last(self):
 *         if self._iter is NULL:
 *             raise RuntimeError("Database or iterator is closed")             # <<<<<<<<<<<<<<
 * 
 *         with nogil:
*/
    __pyx_t_3 = NULL;
    __pyx_t_4 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_mstate_global->__pyx_kp_u_Database_or_iterator_is_closed};
      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_RuntimeError)), __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1046, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 1046, __pyx_L1_error)

    /* "plyvel/_plyvel.pyx":1045
 * 
 *     def seek_to_last(self):
 *         if self._iter is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database or iterator is closed")
 * 
*/
  }

  /* "plyvel/_plyvel.pyx":1048
 *             raise RuntimeError("Database or iterator is closed")
 * 
 *         with nogil:             # <<<<<<<<<<<<<<
 *             self._iter.SeekToLast()
 * 
*/
  {
      PyThreadState * _save;
      _save = PyEval_SaveThread();
      __Pyx_FastGIL_Remember();
      /*try:*/ {

        /* "plyvel/_plyvel.pyx":1049
 * 
 *         with nogil:
 *             self._iter.SeekToLast()             # <<<<<<<<<<<<<<
 * 
 *         raise_for_status(self._iter.status())
*/
        __pyx_v_self->__pyx_base._iter->SeekToLast();
      }

      /* "plyvel/_plyvel.pyx":1048
 *             raise RuntimeError("Database or iterator is closed")
 * 
 *         with nogil:             # <<<<<<<<<<<<<<
 *             self._iter.SeekToLast()
 * 
*/
      /*finally:*/ {
        /*normal exit:*/{
          __Pyx_FastGIL_Forget();
          PyEval_RestoreThread(_save);
          goto __pyx_L6;
        }
        __pyx_L6:;
      }
  }

  /* "plyvel/_plyvel.pyx":1051
 *             self._iter.SeekToLast()
 * 
 *         raise_for_status(self._iter.status())             # <<<<<<<<<<<<<<
 * 
 *     def seek(self, bytes target not None):
*/
  __pyx_t_5 = __pyx_f_6plyvel_7_plyvel_raise_for_status(__pyx_v_self->__pyx_base._iter->status()); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(0, 1051, __pyx_L1_error)

  /* "plyvel/_plyvel.pyx":1044
 *         raise_for_status(self._iter.status())
 * 
 *     def seek_to_last(self):             # <<<<<<<<<<<<<<
 *         if self._iter is NULL:
 *             raise RuntimeError("Database or iterator is closed")
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("plyvel._plyvel.RawIterator.seek_to_last", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":1053
 *         raise_for_status(self._iter.status())
 * 
 *     def seek(self, bytes target not None):             # <<<<<<<<<<<<<<
 *         if self._iter is NULL:
 *             raise RuntimeError("Database or iterator is closed")
*/

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_11RawIterator_7seek(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6plyvel_7_plyvel_11RawIterator_6seek, "RawIterator.seek(self, bytes target)");
static PyMethodDef __pyx_mdef_6plyvel_7_plyvel_11RawIterator_7seek = {"seek", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_11RawIterator_7seek, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_11RawIterator_6seek};
static PyObject *__pyx_pw_6plyvel_7_plyvel_11RawIterator_7seek(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_target = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("seek (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_target,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(0, 1053, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1053, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "seek", 0) < (0)) __PYX_ERR(0, 1053, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("seek", 1, 1, 1, i); __PYX_ERR(0, 1053, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1053, __pyx_L3_error)
    }
    __pyx_v_target = ((PyObject*)values[0]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("seek", 1, 1, 1, __pyx_nargs); __PYX_ERR(0, 1053, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("plyvel._plyvel.RawIterator.seek", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_target), (&PyBytes_Type), 0, "target", 1))) __PYX_ERR(0, 1053, __pyx_L1_error)
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_11RawIterator_6seek(((struct __pyx_obj_6plyvel_7_plyvel_RawIterator *)__pyx_v_self), __pyx_v_target);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_11RawIterator_6seek(struct __pyx_obj_6plyvel_7_plyvel_RawIterator *__pyx_v_self, PyObject *__pyx_v_target) {
  leveldb::Slice __pyx_v_target_slice;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  const char *__pyx_t_5;
  Py_ssize_t __pyx_t_6;
  int __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("seek", 0);

  /* "plyvel/_plyvel.pyx":1054
 * 
 *     def seek(self, bytes target not None):
 *         if self._iter is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database or iterator is closed")
 * 
*/
  __pyx_t_1 = (__pyx_v_self->__pyx_base._iter == NULL);
  if (unlikely(__pyx_t_1)) {

    /* "plyvel/_plyvel.pyx":1055
 *     def seek(self, bytes target not None):
 *         if self._iter is NULL:
 *             raise RuntimeError("Database or iterator is closed")             # <<<<<<<<<<<<<<
 * 
 *         cdef Slice target_slice = Slice(target, len(target))
*/
    __pyx_t_3 = NULL;
    __pyx_t_4 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_mstate_global->__pyx_kp_u_Database_or_iterator_is_closed};
      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_RuntimeError)), __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1055, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 1055, __pyx_L1_error)

    /* "plyvel/_plyvel.pyx":1054
 * 
 *     def seek(self, bytes target not None):
 *         if self._iter is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database or iterator is closed")
 * 
*/
  }

  /* "plyvel/_plyvel.pyx":1057
 *             raise RuntimeError("Database or iterator is closed")
 * 
 *         cdef Slice target_slice = Slice(target, len(target))             # <<<<<<<<<<<<<<
 *         with nogil:
 *             self._iter.Seek(target_slice)
*/
  __pyx_t_5 = __Pyx_PyBytes_AsString(__pyx_v_target); if (unlikely((!__pyx_t_5) && PyErr_Occurred())) __PYX_ERR(0, 1057, __pyx_L1_error)
  __pyx_t_6 = __Pyx_PyBytes_GET_SIZE(__pyx_v_target); if (unlikely(__pyx_t_6 == ((Py_ssize_t)-1))) __PYX_ERR(0, 1057, __pyx_L1_error)
  __pyx_v_target_slice = leveldb::Slice(__pyx_t_5, __pyx_t_6);

  /* "plyvel/_plyvel.pyx":1058
 * 
 *         cdef Slice target_slice = Slice(target, len(target))
 *         with nogil:             # <<<<<<<<<<<<<<
 *             self._iter.Seek(target_slice)
 * 
*/
  {
      PyThreadState * _save;
      _save = PyEval_SaveThread();
      __Pyx_FastGIL_Remember();
      /*try:*/ {

        /* "plyvel/_plyvel.pyx":1059
 *         cdef Slice target_slice = Slice(target, len(target))
 *         with nogil:
 *             self._iter.Seek(target_slice)             # <<<<<<<<<<<<<<
 * 
 *         raise_for_status(self._iter.status())
*/
        __pyx_v_self->__pyx_base._iter->Seek(__pyx_v_target_slice);
      }

      /* "plyvel/_plyvel.pyx":1058
 * 
 *         cdef Slice target_slice = Slice(target, len(target))
 *         with nogil:             # <<<<<<<<<<<<<<
 *             self._iter.Seek(target_slice)
 * 
*/
      /*finally:*/ {
        /*normal exit:*/{
          __Pyx_FastGIL_Forget();
          PyEval_RestoreThread(_save);
          goto __pyx_L6;
        }
        __pyx_L6:;
      }
  }

  /* "plyvel/_plyvel.pyx":1061
 *             self._iter.Seek(target_slice)
 * 
 *         raise_for_status(self._iter.status())             # <<<<<<<<<<<<<<
 * 
 *     def next(self):
*/
  __pyx_t_7 = __pyx_f_6plyvel_7_plyvel_raise_for_status(__pyx_v_self->__pyx_base._iter->status()); if (unlikely(__pyx_t_7 == ((int)-1))) __PYX_ERR(0, 1061, __pyx_L1_error)

  /* "plyvel/_plyvel.pyx":1053
 *         raise_for_status(self._iter.status())
 * 
 *     def seek(self, bytes target not None):             # <<<<<<<<<<<<<<
 *         if self._iter is NULL:
 *             raise RuntimeError("Database or iterator is closed")
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("plyvel._plyvel.RawIterator.seek", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":1063
 *         raise_for_status(self._iter.status())
 * 
 *     def next(self):             # <<<<<<<<<<<<<<
 *         if self._iter is NULL:
 *             raise RuntimeError("Database or iterator is closed")
*/

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_11RawIterator_9next(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6plyvel_7_plyvel_11RawIterator_8next, "RawIterator.next(self)");
static PyMethodDef __pyx_mdef_6plyvel_7_plyvel_11RawIterator_9next = {"next", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_11RawIterator_9next, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_11RawIterator_8next};
static PyObject *__pyx_pw_6plyvel_7_plyvel_11RawIterator_9next(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("next (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("next", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("next", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_11RawIterator_8next(((struct __pyx_obj_6plyvel_7_plyvel_RawIterator *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_11RawIterator_8next(struct __pyx_obj_6plyvel_7_plyvel_RawIterator *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("next", 0);

  /* "plyvel/_plyvel.pyx":1064
 * 
 *     def next(self):
 *         if self._iter is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database or iterator is closed")
 * 
*/
  __pyx_t_1 = (__pyx_v_self->__pyx_base._iter == NULL);
  if (unlikely(__pyx_t_1)) {

    /* "plyvel/_plyvel.pyx":1065
 *     def next(self):
 *         if self._iter is NULL:
 *             raise RuntimeError("Database or iterator is closed")             # <<<<<<<<<<<<<<
 * 
 *         if not self._iter.Valid():
*/
    __pyx_t_3 = NULL;
    __pyx_t_4 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_mstate_global->__pyx_kp_u_Database_or_iterator_is_closed};
      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_RuntimeError)), __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1065, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 1065, __pyx_L1_error)

    /* "plyvel/_plyvel.pyx":1064
 * 
 *     def next(self):
 *         if self._iter is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database or iterator is closed")
 * 
*/
  }

  /* "plyvel/_plyvel.pyx":1067
 *             raise RuntimeError("Database or iterator is closed")
 * 
 *         if not self._iter.Valid():             # <<<<<<<<<<<<<<
 *             raise IteratorInvalidError()
 * 
*/
  __pyx_t_1 = (!(__pyx_v_self->__pyx_base._iter->Valid() != 0));
  if (unlikely(__pyx_t_1)) {

    /* "plyvel/_plyvel.pyx":1068
 * 
 *         if not self._iter.Valid():
 *             raise IteratorInvalidError()             # <<<<<<<<<<<<<<
 * 
 *         with nogil:
*/
    __pyx_t_3 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_mstate_global->__pyx_n_u_IteratorInvalidError); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1068, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_4 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_5);
      assert(__pyx_t_3);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_5, __pyx__function);
      __pyx_t_4 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, NULL};
      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_5, __pyx_callargs+__pyx_t_4, (1-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1068, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 1068, __pyx_L1_error)

    /* "plyvel/_plyvel.pyx":1067
 *             raise RuntimeError("Database or iterator is closed")
 * 
 *         if not self._iter.Valid():             # <<<<<<<<<<<<<<
 *             raise IteratorInvalidError()
 * 
*/
  }

  /* "plyvel/_plyvel.pyx":1070
 *             raise IteratorInvalidError()
 * 
 *         with nogil:             # <<<<<<<<<<<<<<
 *             self._iter.Next()
 * 
*/
  {
      PyThreadState * _save;
      _save = PyEval_SaveThread();
      __Pyx_FastGIL_Remember();
      /*try:*/ {

        /* "plyvel/_plyvel.pyx":1071
 * 
 *         with nogil:
 *             self._iter.Next()             # <<<<<<<<<<<<<<
 * 
 *         raise_for_status(self._iter.status())
*/
        __pyx_v_self->__pyx_base._iter->Next();
      }

      /* "plyvel/_plyvel.pyx":1070
 *             raise IteratorInvalidError()
 * 
 *         with nogil:             # <<<<<<<<<<<<<<
 *             self._iter.Next()
 * 
*/
      /*finally:*/ {
        /*normal exit:*/{
          __Pyx_FastGIL_Forget();
          PyEval_RestoreThread(_save);
          goto __pyx_L7;
        }
        __pyx_L7:;
      }
  }

  /* "plyvel/_plyvel.pyx":1073
 *             self._iter.Next()
 * 
 *         raise_for_status(self._iter.status())             # <<<<<<<<<<<<<<
 * 
 *     def prev(self):
*/
  __pyx_t_6 = __pyx_f_6plyvel_7_plyvel_raise_for_status(__pyx_v_self->__pyx_base._iter->status()); if (unlikely(__pyx_t_6 == ((int)-1))) __PYX_ERR(0, 1073, __pyx_L1_error)

  /* "plyvel/_plyvel.pyx":1063
 *         raise_for_status(self._iter.status())
 * 
 *     def next(self):             # <<<<<<<<<<<<<<
 *         if self._iter is NULL:
 *             raise RuntimeError("Database or iterator is closed")
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("plyvel._plyvel.RawIterator.next", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":1075
 *         raise_for_status(self._iter.status())
 * 
 *     def prev(self):             # <<<<<<<<<<<<<<
 *         if self._iter is NULL:
 *             raise RuntimeError("Database or iterator is closed")
*/

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_11RawIterator_11prev(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6plyvel_7_plyvel_11RawIterator_10prev, "RawIterator.prev(self)");
static PyMethodDef __pyx_mdef_6plyvel_7_plyvel_11RawIterator_11prev = {"prev", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_11RawIterator_11prev, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_11RawIterator_10prev};
static PyObject *__pyx_pw_6plyvel_7_plyvel_11RawIterator_11prev(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("prev (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("prev", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("prev", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_11RawIterator_10prev(((struct __pyx_obj_6plyvel_7_plyvel_RawIterator *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_11RawIterator_10prev(struct __pyx_obj_6plyvel_7_plyvel_RawIterator *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("prev", 0);

  /* "plyvel/_plyvel.pyx":1076
 * 
 *     def prev(self):
 *         if self._iter is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database or iterator is closed")
 * 
*/
  __pyx_t_1 = (__pyx_v_self->__pyx_base._iter == NULL);
  if (unlikely(__pyx_t_1)) {

    /* "plyvel/_plyvel.pyx":1077
 *     def prev(self):
 *         if self._iter is NULL:
 *             raise RuntimeError("Database or iterator is closed")             # <<<<<<<<<<<<<<
 * 
 *         if not self._iter.Valid():
*/
    __pyx_t_3 = NULL;
    __pyx_t_4 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_mstate_global->__pyx_kp_u_Database_or_iterator_is_closed};
      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_RuntimeError)), __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1077, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 1077, __pyx_L1_error)

    /* "plyvel/_plyvel.pyx":1076
 * 
 *     def prev(self):
 *         if self._iter is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database or iterator is closed")
 * 
*/
  }

  /* "plyvel/_plyvel.pyx":1079
 *             raise RuntimeError("Database or iterator is closed")
 * 
 *         if not self._iter.Valid():             # <<<<<<<<<<<<<<
 *             raise IteratorInvalidError()
 * 
*/
  __pyx_t_1 = (!(__pyx_v_self->__pyx_base._iter->Valid() != 0));
  if (unlikely(__pyx_t_1)) {

    /* "plyvel/_plyvel.pyx":1080
 * 
 *         if not self._iter.Valid():
 *             raise IteratorInvalidError()             # <<<<<<<<<<<<<<
 * 
 *         with nogil:
*/
    __pyx_t_3 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_mstate_global->__pyx_n_u_IteratorInvalidError); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1080, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_4 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_5);
      assert(__pyx_t_3);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_5, __pyx__function);
      __pyx_t_4 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, NULL};
      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_5, __pyx_callargs+__pyx_t_4, (1-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1080, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 1080, __pyx_L1_error)

    /* "plyvel/_plyvel.pyx":1079
 *             raise RuntimeError("Database or iterator is closed")
 * 
 *         if not self._iter.Valid():             # <<<<<<<<<<<<<<
 *             raise IteratorInvalidError()
 * 
*/
  }

  /* "plyvel/_plyvel.pyx":1082
 *             raise IteratorInvalidError()
 * 
 *         with nogil:             # <<<<<<<<<<<<<<
 *             self._iter.Prev()
 * 
*/
  {
      PyThreadState * _save;
      _save = PyEval_SaveThread();
      __Pyx_FastGIL_Remember();
      /*try:*/ {

        /* "plyvel/_plyvel.pyx":1083
 * 
 *         with nogil:
 *             self._iter.Prev()             # <<<<<<<<<<<<<<
 * 
 *         raise_for_status(self._iter.status())
*/
        __pyx_v_self->__pyx_base._iter->Prev();
      }

      /* "plyvel/_plyvel.pyx":1082
 *             raise IteratorInvalidError()
 * 
 *         with nogil:             # <<<<<<<<<<<<<<
 *             self._iter.Prev()
 * 
*/
      /*finally:*/ {
        /*normal exit:*/{
          __Pyx_FastGIL_Forget();
          PyEval_RestoreThread(_save);
          goto __pyx_L7;
        }
        __pyx_L7:;
      }
  }

  /* "plyvel/_plyvel.pyx":1085
 *             self._iter.Prev()
 * 
 *         raise_for_status(self._iter.status())             # <<<<<<<<<<<<<<
 * 
 *     cpdef key(self):
*/
  __pyx_t_6 = __pyx_f_6plyvel_7_plyvel_raise_for_status(__pyx_v_self->__pyx_base._iter->status()); if (unlikely(__pyx_t_6 == ((int)-1))) __PYX_ERR(0, 1085, __pyx_L1_error)

  /* "plyvel/_plyvel.pyx":1075
 *         raise_for_status(self._iter.status())
 * 
 *     def prev(self):             # <<<<<<<<<<<<<<
 *         if self._iter is NULL:
 *             raise RuntimeError("Database or iterator is closed")
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("plyvel._plyvel.RawIterator.prev", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":1087
 *         raise_for_status(self._iter.status())
 * 
 *     cpdef key(self):             # <<<<<<<<<<<<<<
 *         if self._iter is NULL:
 *             raise RuntimeError("Database or iterator is closed")
*/

static PyObject *__pyx_pw_6plyvel_7_plyvel_11RawIterator_13key(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyObject *__pyx_f_6plyvel_7_plyvel_11RawIterator_key(struct __pyx_obj_6plyvel_7_plyvel_RawIterator *__pyx_v_self, CYTHON_UNUSED int __pyx_skip_dispatch) {
  leveldb::Slice __pyx_v_key_slice;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("key", 0);

  /* "plyvel/_plyvel.pyx":1088
 * 
 *     cpdef key(self):
 *         if self._iter is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database or iterator is closed")
 * 
*/
  __pyx_t_1 = (__pyx_v_self->__pyx_base._iter == NULL);
  if (unlikely(__pyx_t_1)) {

    /* "plyvel/_plyvel.pyx":1089
 *     cpdef key(self):
 *         if self._iter is NULL:
 *             raise RuntimeError("Database or iterator is closed")             # <<<<<<<<<<<<<<
 * 
 *         if not self._iter.Valid():
*/
    __pyx_t_3 = NULL;
    __pyx_t_4 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_mstate_global->__pyx_kp_u_Database_or_iterator_is_closed};
      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_RuntimeError)), __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1089, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 1089, __pyx_L1_error)

    /* "plyvel/_plyvel.pyx":1088
 * 
 *     cpdef key(self):
 *         if self._iter is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database or iterator is closed")
 * 
*/
  }

  /* "plyvel/_plyvel.pyx":1091
 *             raise RuntimeError("Database or iterator is closed")
 * 
 *         if not self._iter.Valid():             # <<<<<<<<<<<<<<
 *             raise IteratorInvalidError()
 * 
*/
  __pyx_t_1 = (!(__pyx_v_self->__pyx_base._iter->Valid() != 0));
  if (unlikely(__pyx_t_1)) {

    /* "plyvel/_plyvel.pyx":1092
 * 
 *         if not self._iter.Valid():
 *             raise IteratorInvalidError()             # <<<<<<<<<<<<<<
 * 
 *         cdef Slice key_slice
*/
    __pyx_t_3 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_mstate_global->__pyx_n_u_IteratorInvalidError); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1092, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_4 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_5);
      assert(__pyx_t_3);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_5, __pyx__function);
      __pyx_t_4 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, NULL};
      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_5, __pyx_callargs+__pyx_t_4, (1-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1092, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 1092, __pyx_L1_error)

    /* "plyvel/_plyvel.pyx":1091
 *             raise RuntimeError("Database or iterator is closed")
 * 
 *         if not self._iter.Valid():             # <<<<<<<<<<<<<<
 *             raise IteratorInvalidError()
 * 
*/
  }

  /* "plyvel/_plyvel.pyx":1095
 * 
 *         cdef Slice key_slice
 *         key_slice = self._iter.key()             # <<<<<<<<<<<<<<
 *         return key_slice.data()[:key_slice.size()]
 * 
*/
  __pyx_v_key_slice = __pyx_v_self->__pyx_base._iter->key();

  /* "plyvel/_plyvel.pyx":1096
 *         cdef Slice key_slice
 *         key_slice = self._iter.key()
 *         return key_slice.data()[:key_slice.size()]             # <<<<<<<<<<<<<<
 * 
 *     cpdef value(self):
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyBytes_FromStringAndSize(__pyx_v_key_slice.data() + 0, __pyx_v_key_slice.size() - 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1096, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "plyvel/_plyvel.pyx":1087
 *         raise_for_status(self._iter.status())
 * 
 *     cpdef key(self):             # <<<<<<<<<<<<<<
 *         if self._iter is NULL:
 *             raise RuntimeError("Database or iterator is closed")
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("plyvel._plyvel.RawIterator.key", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_11RawIterator_13key(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6plyvel_7_plyvel_11RawIterator_12key, "RawIterator.key(self)");
static PyMethodDef __pyx_mdef_6plyvel_7_plyvel_11RawIterator_13key = {"key", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_11RawIterator_13key, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_11RawIterator_12key};
static PyObject *__pyx_pw_6plyvel_7_plyvel_11RawIterator_13key(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("key (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("key", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("key", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_11RawIterator_12key(((struct __pyx_obj_6plyvel_7_plyvel_RawIterator *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_11RawIterator_12key(struct __pyx_obj_6plyvel_7_plyvel_RawIterator *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("key", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_6plyvel_7_plyvel_11RawIterator_key(__pyx_v_self, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1087, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("plyvel._plyvel.RawIterator.key", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":1098
 *         return key_slice.data()[:key_slice.size()]
 * 
 *     cpdef value(self):             # <<<<<<<<<<<<<<
 *         if self._iter is NULL:
 *             raise RuntimeError("Database or iterator is closed")
*/

static PyObject *__pyx_pw_6plyvel_7_plyvel_11RawIterator_15value(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyObject *__pyx_f_6plyvel_7_plyvel_11RawIterator_value(struct __pyx_obj_6plyvel_7_plyvel_RawIterator *__pyx_v_self, CYTHON_UNUSED int __pyx_skip_dispatch) {
  leveldb::Slice __pyx_v_value_slice;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("value", 0);

  /* "plyvel/_plyvel.pyx":1099
 * 
 *     cpdef value(self):
 *         if self._iter is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database or iterator is closed")
 * 
*/
  __pyx_t_1 = (__pyx_v_self->__pyx_base._iter == NULL);
  if (unlikely(__pyx_t_1)) {

    /* "plyvel/_plyvel.pyx":1100
 *     cpdef value(self):
 *         if self._iter is NULL:
 *             raise RuntimeError("Database or iterator is closed")             # <<<<<<<<<<<<<<
 * 
 *         if not self._iter.Valid():
*/
    __pyx_t_3 = NULL;
    __pyx_t_4 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_mstate_global->__pyx_kp_u_Database_or_iterator_is_closed};
      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_RuntimeError)), __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1100, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 1100, __pyx_L1_error)

    /* "plyvel/_plyvel.pyx":1099
 * 
 *     cpdef value(self):
 *         if self._iter is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database or iterator is closed")
 * 
*/
  }

  /* "plyvel/_plyvel.pyx":1102
 *             raise RuntimeError("Database or iterator is closed")
 * 
 *         if not self._iter.Valid():             # <<<<<<<<<<<<<<
 *             raise IteratorInvalidError()
 * 
*/
  __pyx_t_1 = (!(__pyx_v_self->__pyx_base._iter->Valid() != 0));
  if (unlikely(__pyx_t_1)) {

    /* "plyvel/_plyvel.pyx":1103
 * 
 *         if not self._iter.Valid():
 *             raise IteratorInvalidError()             # <<<<<<<<<<<<<<
 * 
 *         cdef Slice value_slice
*/
    __pyx_t_3 = NULL;
    __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_mstate_global->__pyx_n_u_IteratorInvalidError); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 1103, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_4 = 1;
    #if CYTHON_UNPACK_METHODS
    if (unlikely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_5);
      assert(__pyx_t_3);
      PyObject* __pyx__function = PyMethod_GET_FUNCTION(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(__pyx__function);
      __Pyx_DECREF_SET(__pyx_t_5, __pyx__function);
      __pyx_t_4 = 0;
    }
    #endif
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, NULL};
      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)__pyx_t_5, __pyx_callargs+__pyx_t_4, (1-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1103, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 1103, __pyx_L1_error)

    /* "plyvel/_plyvel.pyx":1102
 *             raise RuntimeError("Database or iterator is closed")
 * 
 *         if not self._iter.Valid():             # <<<<<<<<<<<<<<
 *             raise IteratorInvalidError()
 * 
*/
  }

  /* "plyvel/_plyvel.pyx":1106
 * 
 *         cdef Slice value_slice
 *         value_slice = self._iter.value()             # <<<<<<<<<<<<<<
 *         return value_slice.data()[:value_slice.size()]
 * 
*/
  __pyx_v_value_slice = __pyx_v_self->__pyx_base._iter->value();

  /* "plyvel/_plyvel.pyx":1107
 *         cdef Slice value_slice
 *         value_slice = self._iter.value()
 *         return value_slice.data()[:value_slice.size()]             # <<<<<<<<<<<<<<
 * 
 *     def item(self):
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyBytes_FromStringAndSize(__pyx_v_value_slice.data() + 0, __pyx_v_value_slice.size() - 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1107, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "plyvel/_plyvel.pyx":1098
 *         return key_slice.data()[:key_slice.size()]
 * 
 *     cpdef value(self):             # <<<<<<<<<<<<<<
 *         if self._iter is NULL:
 *             raise RuntimeError("Database or iterator is closed")
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("plyvel._plyvel.RawIterator.value", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_11RawIterator_15value(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6plyvel_7_plyvel_11RawIterator_14value, "RawIterator.value(self)");
static PyMethodDef __pyx_mdef_6plyvel_7_plyvel_11RawIterator_15value = {"value", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_11RawIterator_15value, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_11RawIterator_14value};
static PyObject *__pyx_pw_6plyvel_7_plyvel_11RawIterator_15value(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("value (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("value", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("value", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_11RawIterator_14value(((struct __pyx_obj_6plyvel_7_plyvel_RawIterator *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_11RawIterator_14value(struct __pyx_obj_6plyvel_7_plyvel_RawIterator *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("value", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_6plyvel_7_plyvel_11RawIterator_value(__pyx_v_self, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1098, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("plyvel._plyvel.RawIterator.value", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":1109
 *         return value_slice.data()[:value_slice.size()]
 * 
 *     def item(self):             # <<<<<<<<<<<<<<
 *         return self.key(), self.value()
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_11RawIterator_17item(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6plyvel_7_plyvel_11RawIterator_16item, "RawIterator.item(self)");
static PyMethodDef __pyx_mdef_6plyvel_7_plyvel_11RawIterator_17item = {"item", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_11RawIterator_17item, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_11RawIterator_16item};
static PyObject *__pyx_pw_6plyvel_7_plyvel_11RawIterator_17item(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("item (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("item", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("item", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_11RawIterator_16item(((struct __pyx_obj_6plyvel_7_plyvel_RawIterator *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_11RawIterator_16item(struct __pyx_obj_6plyvel_7_plyvel_RawIterator *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("item", 0);

  /* "plyvel/_plyvel.pyx":1110
 * 
 *     def item(self):
 *         return self.key(), self.value()             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_6plyvel_7_plyvel_11RawIterator_key(__pyx_v_self, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1110, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_f_6plyvel_7_plyvel_11RawIterator_value(__pyx_v_self, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1110, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1110, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_1);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_1) != (0)) __PYX_ERR(0, 1110, __pyx_L1_error);
  __Pyx_GIVEREF(__pyx_t_2);
  if (__Pyx_PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_2) != (0)) __PYX_ERR(0, 1110, __pyx_L1_error);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "plyvel/_plyvel.pyx":1109
 *         return value_slice.data()[:value_slice.size()]
 * 
 *     def item(self):             # <<<<<<<<<<<<<<
 *         return self.key(), self.value()
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("plyvel._plyvel.RawIterator.item", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self._iter cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):
*/

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_11RawIterator_19__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6plyvel_7_plyvel_11RawIterator_18__reduce_cython__, "RawIterator.__reduce_cython__(self)");
static PyMethodDef __pyx_mdef_6plyvel_7_plyvel_11RawIterator_19__reduce_cython__ = {"__reduce_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_11RawIterator_19__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_11RawIterator_18__reduce_cython__};
static PyObject *__pyx_pw_6plyvel_7_plyvel_11RawIterator_19__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("__reduce_cython__", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("__reduce_cython__", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_11RawIterator_18__reduce_cython__(((struct __pyx_obj_6plyvel_7_plyvel_RawIterator *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_11RawIterator_18__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_6plyvel_7_plyvel_RawIterator *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError, "self._iter cannot be converted to a Python object for pickling"             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError, "self._iter cannot be converted to a Python object for pickling"
*/
  __Pyx_Raise(((PyObject *)(((PyTypeObject*)PyExc_TypeError))), __pyx_mstate_global->__pyx_kp_u_self__iter_cannot_be_converted_t, 0, 0);
  __PYX_ERR(1, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self._iter cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("plyvel._plyvel.RawIterator.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError, "self._iter cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self._iter cannot be converted to a Python object for pickling"
*/

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_11RawIterator_21__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6plyvel_7_plyvel_11RawIterator_20__setstate_cython__, "RawIterator.__setstate_cython__(self, __pyx_state)");
static PyMethodDef __pyx_mdef_6plyvel_7_plyvel_11RawIterator_21__setstate_cython__ = {"__setstate_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_11RawIterator_21__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_11RawIterator_20__setstate_cython__};
static PyObject *__pyx_pw_6plyvel_7_plyvel_11RawIterator_21__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  CYTHON_UNUSED PyObject *__pyx_v___pyx_state = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_pyx_state,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(1, 3, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(1, 3, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__setstate_cython__", 0) < (0)) __PYX_ERR(1, 3, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, i); __PYX_ERR(1, 3, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(1, 3, __pyx_L3_error)
    }
    __pyx_v___pyx_state = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, __pyx_nargs); __PYX_ERR(1, 3, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("plyvel._plyvel.RawIterator.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_11RawIterator_20__setstate_cython__(((struct __pyx_obj_6plyvel_7_plyvel_RawIterator *)__pyx_v_self), __pyx_v___pyx_state);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_11RawIterator_20__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_6plyvel_7_plyvel_RawIterator *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError, "self._iter cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError, "self._iter cannot be converted to a Python object for pickling"             # <<<<<<<<<<<<<<
*/
  __Pyx_Raise(((PyObject *)(((PyTypeObject*)PyExc_TypeError))), __pyx_mstate_global->__pyx_kp_u_self__iter_cannot_be_converted_t, 0, 0);
  __PYX_ERR(1, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError, "self._iter cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self._iter cannot be converted to a Python object for pickling"
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("plyvel._plyvel.RawIterator.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":1123
 *     cdef bytes prefix
 * 
 *     def __init__(self, *, DB db not None, bytes prefix=None):             # <<<<<<<<<<<<<<
 *         if db._db is NULL:
 *             raise RuntimeError("Cannot operate on closed LevelDB database")
*/

/* Python wrapper */
static int __pyx_pw_6plyvel_7_plyvel_8Snapshot_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_6plyvel_7_plyvel_8Snapshot_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_db = 0;
  PyObject *__pyx_v_prefix = 0;
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return -1;
  #endif
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_db,&__pyx_mstate_global->__pyx_n_u_prefix,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_VARARGS(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(0, 1123, __pyx_L3_error)
    if (likely(__pyx_kwds_len > 0)) {
      switch (__pyx_nargs) {
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, 0, __pyx_kwds_len, "__init__", 0) < (0)) __PYX_ERR(0, 1123, __pyx_L3_error)
      if (!values[1]) values[1] = __Pyx_NewRef(((PyObject*)Py_None));
      for (Py_ssize_t i = 0; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseKeywordRequired("__init__", *(__pyx_pyargnames[i - 0])); __PYX_ERR(0, 1123, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 0)) {
      goto __pyx_L5_argtuple_error;
    } else {
      __Pyx_RaiseKeywordRequired("__init__", __pyx_mstate_global->__pyx_n_u_db); __PYX_ERR(0, 1123, __pyx_L3_error)
    }
    __pyx_v_db = ((struct __pyx_obj_6plyvel_7_plyvel_DB *)values[0]);
    __pyx_v_prefix = ((PyObject*)values[1]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 0, 0, __pyx_nargs); __PYX_ERR(0, 1123, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("plyvel._plyvel.Snapshot.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_db), __pyx_mstate_global->__pyx_ptype_6plyvel_7_plyvel_DB, 0, "db", 0))) __PYX_ERR(0, 1123, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_prefix), (&PyBytes_Type), 1, "prefix", 1))) __PYX_ERR(0, 1123, __pyx_L1_error)
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_8Snapshot___init__(((struct __pyx_obj_6plyvel_7_plyvel_Snapshot *)__pyx_v_self), __pyx_v_db, __pyx_v_prefix);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = -1;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_6plyvel_7_plyvel_8Snapshot___init__(struct __pyx_obj_6plyvel_7_plyvel_Snapshot *__pyx_v_self, struct __pyx_obj_6plyvel_7_plyvel_DB *__pyx_v_db, PyObject *__pyx_v_prefix) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "plyvel/_plyvel.pyx":1124
 * 
 *     def __init__(self, *, DB db not None, bytes prefix=None):
 *         if db._db is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Cannot operate on closed LevelDB database")
 * 
*/
  __pyx_t_1 = (__pyx_v_db->_db == NULL);
  if (unlikely(__pyx_t_1)) {

    /* "plyvel/_plyvel.pyx":1125
 *     def __init__(self, *, DB db not None, bytes prefix=None):
 *         if db._db is NULL:
 *             raise RuntimeError("Cannot operate on closed LevelDB database")             # <<<<<<<<<<<<<<
 * 
 *         self.db = db
*/
    __pyx_t_3 = NULL;
    __pyx_t_4 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_mstate_global->__pyx_kp_u_Cannot_operate_on_closed_LevelDB};
      __pyx_t_2 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_RuntimeError)), __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (__pyx_t_4*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1125, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
    }
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 1125, __pyx_L1_error)

    /* "plyvel/_plyvel.pyx":1124
 * 
 *     def __init__(self, *, DB db not None, bytes prefix=None):
 *         if db._db is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Cannot operate on closed LevelDB database")
 * 
*/
  }

  /* "plyvel/_plyvel.pyx":1127
 *             raise RuntimeError("Cannot operate on closed LevelDB database")
 * 
 *         self.db = db             # <<<<<<<<<<<<<<
 *         self.prefix = prefix
 *         with nogil:
*/
  __Pyx_INCREF((PyObject *)__pyx_v_db);
  __Pyx_GIVEREF((PyObject *)__pyx_v_db);
  __Pyx_GOTREF((PyObject *)__pyx_v_self->db);
  __Pyx_DECREF((PyObject *)__pyx_v_self->db);
  __pyx_v_self->db = __pyx_v_db;

  /* "plyvel/_plyvel.pyx":1128
 * 
 *         self.db = db
 *         self.prefix = prefix             # <<<<<<<<<<<<<<
 *         with nogil:
 *             self._snapshot = <leveldb.Snapshot*>db._db.GetSnapshot()
*/
  __Pyx_INCREF(__pyx_v_prefix);
  __Pyx_GIVEREF(__pyx_v_prefix);
  __Pyx_GOTREF(__pyx_v_self->prefix);
  __Pyx_DECREF(__pyx_v_self->prefix);
  __pyx_v_self->prefix = __pyx_v_prefix;

  /* "plyvel/_plyvel.pyx":1129
 *         self.db = db
 *         self.prefix = prefix
 *         with nogil:             # <<<<<<<<<<<<<<
 *             self._snapshot = <leveldb.Snapshot*>db._db.GetSnapshot()
 * 
*/
  {
      PyThreadState * _save;
      _save = PyEval_SaveThread();
      __Pyx_FastGIL_Remember();
      /*try:*/ {

        /* "plyvel/_plyvel.pyx":1130
 *         self.prefix = prefix
 *         with nogil:
 *             self._snapshot = <leveldb.Snapshot*>db._db.GetSnapshot()             # <<<<<<<<<<<<<<
 * 
 *     def __dealloc__(self):
*/
        __pyx_v_self->_snapshot = ((leveldb::Snapshot *)__pyx_v_db->_db->GetSnapshot());
      }

      /* "plyvel/_plyvel.pyx":1129
 *         self.db = db
 *         self.prefix = prefix
 *         with nogil:             # <<<<<<<<<<<<<<
 *             self._snapshot = <leveldb.Snapshot*>db._db.GetSnapshot()
 * 
*/
      /*finally:*/ {
        /*normal exit:*/{
          __Pyx_FastGIL_Forget();
          PyEval_RestoreThread(_save);
          goto __pyx_L6;
        }
        __pyx_L6:;
      }
  }

  /* "plyvel/_plyvel.pyx":1123
 *     cdef bytes prefix
 * 
 *     def __init__(self, *, DB db not None, bytes prefix=None):             # <<<<<<<<<<<<<<
 *         if db._db is NULL:
 *             raise RuntimeError("Cannot operate on closed LevelDB database")
*/

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("plyvel._plyvel.Snapshot.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":1132
 *             self._snapshot = <leveldb.Snapshot*>db._db.GetSnapshot()
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         self.close()
 * 
*/

/* Python wrapper */
static void __pyx_pw_6plyvel_7_plyvel_8Snapshot_3__dealloc__(PyObject *__pyx_v_self); /*proto*/
static void __pyx_pw_6plyvel_7_plyvel_8Snapshot_3__dealloc__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__dealloc__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_pf_6plyvel_7_plyvel_8Snapshot_2__dealloc__(((struct __pyx_obj_6plyvel_7_plyvel_Snapshot *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

static void __pyx_pf_6plyvel_7_plyvel_8Snapshot_2__dealloc__(struct __pyx_obj_6plyvel_7_plyvel_Snapshot *__pyx_v_self) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__dealloc__", 0);

  /* "plyvel/_plyvel.pyx":1133
 * 
 *     def __dealloc__(self):
 *         self.close()             # <<<<<<<<<<<<<<
 * 
 *     cpdef close(self):
*/
  __pyx_t_1 = __pyx_f_6plyvel_7_plyvel_8Snapshot_close(__pyx_v_self, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1133, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "plyvel/_plyvel.pyx":1132
 *             self._snapshot = <leveldb.Snapshot*>db._db.GetSnapshot()
 * 
 *     def __dealloc__(self):             # <<<<<<<<<<<<<<
 *         self.close()
 * 
*/

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_WriteUnraisable("plyvel._plyvel.Snapshot.__dealloc__", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
}

/* "plyvel/_plyvel.pyx":1135
 *         self.close()
 * 
 *     cpdef close(self):             # <<<<<<<<<<<<<<
 *         if self.db._db is NULL or self._snapshot is NULL:
 *             return  # nothing to do
*/

static PyObject *__pyx_pw_6plyvel_7_plyvel_8Snapshot_5close(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
static PyObject *__pyx_f_6plyvel_7_plyvel_8Snapshot_close(struct __pyx_obj_6plyvel_7_plyvel_Snapshot *__pyx_v_self, CYTHON_UNUSED int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  __Pyx_RefNannySetupContext("close", 0);

  /* "plyvel/_plyvel.pyx":1136
 * 
 *     cpdef close(self):
 *         if self.db._db is NULL or self._snapshot is NULL:             # <<<<<<<<<<<<<<
 *             return  # nothing to do
 * 
*/
  __pyx_t_2 = (__pyx_v_self->db->_db == NULL);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = (__pyx_v_self->_snapshot == NULL);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_1) {

    /* "plyvel/_plyvel.pyx":1137
 *     cpdef close(self):
 *         if self.db._db is NULL or self._snapshot is NULL:
 *             return  # nothing to do             # <<<<<<<<<<<<<<
 * 
 *         with nogil:
*/
    __Pyx_XDECREF(__pyx_r);
    __pyx_r = Py_None; __Pyx_INCREF(Py_None);
    goto __pyx_L0;

    /* "plyvel/_plyvel.pyx":1136
 * 
 *     cpdef close(self):
 *         if self.db._db is NULL or self._snapshot is NULL:             # <<<<<<<<<<<<<<
 *             return  # nothing to do
 * 
*/
  }

  /* "plyvel/_plyvel.pyx":1139
 *             return  # nothing to do
 * 
 *         with nogil:             # <<<<<<<<<<<<<<
 *             self.db._db.ReleaseSnapshot(self._snapshot)
 *             self._snapshot = NULL
*/
  {
      PyThreadState * _save;
      _save = PyEval_SaveThread();
      __Pyx_FastGIL_Remember();
      /*try:*/ {

        /* "plyvel/_plyvel.pyx":1140
 * 
 *         with nogil:
 *             self.db._db.ReleaseSnapshot(self._snapshot)             # <<<<<<<<<<<<<<
 *             self._snapshot = NULL
 * 
*/
        __pyx_v_self->db->_db->ReleaseSnapshot(__pyx_v_self->_snapshot);

        /* "plyvel/_plyvel.pyx":1141
 *         with nogil:
 *             self.db._db.ReleaseSnapshot(self._snapshot)
 *             self._snapshot = NULL             # <<<<<<<<<<<<<<
 * 
 *     def release(self):
*/
        __pyx_v_self->_snapshot = NULL;
      }

      /* "plyvel/_plyvel.pyx":1139
 *             return  # nothing to do
 * 
 *         with nogil:             # <<<<<<<<<<<<<<
 *             self.db._db.ReleaseSnapshot(self._snapshot)
 *             self._snapshot = NULL
*/
      /*finally:*/ {
        /*normal exit:*/{
          __Pyx_FastGIL_Forget();
          PyEval_RestoreThread(_save);
          goto __pyx_L8;
        }
        __pyx_L8:;
      }
  }

  /* "plyvel/_plyvel.pyx":1135
 *         self.close()
 * 
 *     cpdef close(self):             # <<<<<<<<<<<<<<
 *         if self.db._db is NULL or self._snapshot is NULL:
 *             return  # nothing to do
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_8Snapshot_5close(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6plyvel_7_plyvel_8Snapshot_4close, "Snapshot.close(self)");
static PyMethodDef __pyx_mdef_6plyvel_7_plyvel_8Snapshot_5close = {"close", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_8Snapshot_5close, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_8Snapshot_4close};
static PyObject *__pyx_pw_6plyvel_7_plyvel_8Snapshot_5close(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("close (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("close", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("close", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_8Snapshot_4close(((struct __pyx_obj_6plyvel_7_plyvel_Snapshot *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_8Snapshot_4close(struct __pyx_obj_6plyvel_7_plyvel_Snapshot *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("close", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_6plyvel_7_plyvel_8Snapshot_close(__pyx_v_self, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1135, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("plyvel._plyvel.Snapshot.close", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":1143
 *             self._snapshot = NULL
 * 
 *     def release(self):             # <<<<<<<<<<<<<<
 *         self.close()
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_8Snapshot_7release(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6plyvel_7_plyvel_8Snapshot_6release, "Snapshot.release(self)");
static PyMethodDef __pyx_mdef_6plyvel_7_plyvel_8Snapshot_7release = {"release", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_8Snapshot_7release, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_8Snapshot_6release};
static PyObject *__pyx_pw_6plyvel_7_plyvel_8Snapshot_7release(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("release (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("release", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("release", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_8Snapshot_6release(((struct __pyx_obj_6plyvel_7_plyvel_Snapshot *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_8Snapshot_6release(struct __pyx_obj_6plyvel_7_plyvel_Snapshot *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("release", 0);

  /* "plyvel/_plyvel.pyx":1144
 * 
 *     def release(self):
 *         self.close()             # <<<<<<<<<<<<<<
 * 
 *     def __enter__(self):
*/
  __pyx_t_1 = __pyx_f_6plyvel_7_plyvel_8Snapshot_close(__pyx_v_self, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1144, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "plyvel/_plyvel.pyx":1143
 *             self._snapshot = NULL
 * 
 *     def release(self):             # <<<<<<<<<<<<<<
 *         self.close()
 * 
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("plyvel._plyvel.Snapshot.release", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":1146
 *         self.close()
 * 
 *     def __enter__(self):             # <<<<<<<<<<<<<<
 *         return self
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_8Snapshot_9__enter__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6plyvel_7_plyvel_8Snapshot_8__enter__, "Snapshot.__enter__(self)");
static PyMethodDef __pyx_mdef_6plyvel_7_plyvel_8Snapshot_9__enter__ = {"__enter__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_8Snapshot_9__enter__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_8Snapshot_8__enter__};
static PyObject *__pyx_pw_6plyvel_7_plyvel_8Snapshot_9__enter__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__enter__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("__enter__", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("__enter__", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_8Snapshot_8__enter__(((struct __pyx_obj_6plyvel_7_plyvel_Snapshot *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_8Snapshot_8__enter__(struct __pyx_obj_6plyvel_7_plyvel_Snapshot *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__enter__", 0);

  /* "plyvel/_plyvel.pyx":1147
 * 
 *     def __enter__(self):
 *         return self             # <<<<<<<<<<<<<<
 * 
 *     def __exit__(self, exc_type, exc_val, exc_tb):
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF((PyObject *)__pyx_v_self);
  __pyx_r = ((PyObject *)__pyx_v_self);
  goto __pyx_L0;

  /* "plyvel/_plyvel.pyx":1146
 *         self.close()
 * 
 *     def __enter__(self):             # <<<<<<<<<<<<<<
 *         return self
 * 
*/

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":1149
 *         return self
 * 
 *     def __exit__(self, exc_type, exc_val, exc_tb):             # <<<<<<<<<<<<<<
 *         self.close()
 *         return False  # propagate exceptions
*/

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_8Snapshot_11__exit__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6plyvel_7_plyvel_8Snapshot_10__exit__, "Snapshot.__exit__(self, exc_type, exc_val, exc_tb)");
static PyMethodDef __pyx_mdef_6plyvel_7_plyvel_8Snapshot_11__exit__ = {"__exit__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_8Snapshot_11__exit__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_8Snapshot_10__exit__};
static PyObject *__pyx_pw_6plyvel_7_plyvel_8Snapshot_11__exit__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  CYTHON_UNUSED PyObject *__pyx_v_exc_type = 0;
  CYTHON_UNUSED PyObject *__pyx_v_exc_val = 0;
  CYTHON_UNUSED PyObject *__pyx_v_exc_tb = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[3] = {0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__exit__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_exc_type,&__pyx_mstate_global->__pyx_n_u_exc_val,&__pyx_mstate_global->__pyx_n_u_exc_tb,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(0, 1149, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 1149, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 1149, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1149, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__exit__", 0) < (0)) __PYX_ERR(0, 1149, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 3; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__exit__", 1, 3, 3, i); __PYX_ERR(0, 1149, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 3)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1149, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 1149, __pyx_L3_error)
      values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(0, 1149, __pyx_L3_error)
    }
    __pyx_v_exc_type = values[0];
    __pyx_v_exc_val = values[1];
    __pyx_v_exc_tb = values[2];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__exit__", 1, 3, 3, __pyx_nargs); __PYX_ERR(0, 1149, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("plyvel._plyvel.Snapshot.__exit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_8Snapshot_10__exit__(((struct __pyx_obj_6plyvel_7_plyvel_Snapshot *)__pyx_v_self), __pyx_v_exc_type, __pyx_v_exc_val, __pyx_v_exc_tb);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_8Snapshot_10__exit__(struct __pyx_obj_6plyvel_7_plyvel_Snapshot *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v_exc_type, CYTHON_UNUSED PyObject *__pyx_v_exc_val, CYTHON_UNUSED PyObject *__pyx_v_exc_tb) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__exit__", 0);

  /* "plyvel/_plyvel.pyx":1150
 * 
 *     def __exit__(self, exc_type, exc_val, exc_tb):
 *         self.close()             # <<<<<<<<<<<<<<
 *         return False  # propagate exceptions
 * 
*/
  __pyx_t_1 = __pyx_f_6plyvel_7_plyvel_8Snapshot_close(__pyx_v_self, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1150, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "plyvel/_plyvel.pyx":1151
 *     def __exit__(self, exc_type, exc_val, exc_tb):
 *         self.close()
 *         return False  # propagate exceptions             # <<<<<<<<<<<<<<
 * 
 *     def get(self, bytes key not None, default=None, *,
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(Py_False);
  __pyx_r = Py_False;
  goto __pyx_L0;

  /* "plyvel/_plyvel.pyx":1149
 *         return self
 * 
 *     def __exit__(self, exc_type, exc_val, exc_tb):             # <<<<<<<<<<<<<<
 *         self.close()
 *         return False  # propagate exceptions
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("plyvel._plyvel.Snapshot.__exit__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":1153
 *         return False  # propagate exceptions
 * 
 *     def get(self, bytes key not None, default=None, *,             # <<<<<<<<<<<<<<
 *             bool verify_checksums=False, bool fill_cache=True):
 *         if self.db._db is NULL or self._snapshot is NULL:
*/

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_8Snapshot_13get(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6plyvel_7_plyvel_8Snapshot_12get, "Snapshot.get(self, bytes key, default=None, *, bool verify_checksums=False, bool fill_cache=True)");
static PyMethodDef __pyx_mdef_6plyvel_7_plyvel_8Snapshot_13get = {"get", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_8Snapshot_13get, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_8Snapshot_12get};
static PyObject *__pyx_pw_6plyvel_7_plyvel_8Snapshot_13get(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_key = 0;
  PyObject *__pyx_v_default = 0;
  PyLongObject *__pyx_v_verify_checksums = 0;
  PyLongObject *__pyx_v_fill_cache = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[4] = {0,0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_key,&__pyx_mstate_global->__pyx_n_u_default,&__pyx_mstate_global->__pyx_n_u_verify_checksums,&__pyx_mstate_global->__pyx_n_u_fill_cache,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(0, 1153, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 1153, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1153, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "get", 0) < (0)) __PYX_ERR(0, 1153, __pyx_L3_error)
      if (!values[1]) values[1] = __Pyx_NewRef(((PyObject *)Py_None));

      /* "plyvel/_plyvel.pyx":1154
 * 
 *     def get(self, bytes key not None, default=None, *,
 *             bool verify_checksums=False, bool fill_cache=True):             # <<<<<<<<<<<<<<
 *         if self.db._db is NULL or self._snapshot is NULL:
 *             raise RuntimeError("Database or snapshot is closed")
*/
      if (!values[2]) values[2] = __Pyx_NewRef((PyObject *)((PyLongObject *)Py_False));
      if (!values[3]) values[3] = __Pyx_NewRef((PyObject *)((PyLongObject *)Py_True));
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("get", 0, 1, 2, i); __PYX_ERR(0, 1153, __pyx_L3_error) }
      }
    } else {
      switch (__pyx_nargs) {
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(0, 1153, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(0, 1153, __pyx_L3_error)
        break;
        default: goto __pyx_L5_argtuple_error;
      }

      /* "plyvel/_plyvel.pyx":1153
 *         return False  # propagate exceptions
 * 
 *     def get(self, bytes key not None, default=None, *,             # <<<<<<<<<<<<<<
 *             bool verify_checksums=False, bool fill_cache=True):
 *         if self.db._db is NULL or self._snapshot is NULL:
*/
      if (!values[1]) values[1] = __Pyx_NewRef(((PyObject *)Py_None));

      /* "plyvel/_plyvel.pyx":1154
 * 
 *     def get(self, bytes key not None, default=None, *,
 *             bool verify_checksums=False, bool fill_cache=True):             # <<<<<<<<<<<<<<
 *         if self.db._db is NULL or self._snapshot is NULL:
 *             raise RuntimeError("Database or snapshot is closed")
*/
      if (!values[2]) values[2] = __Pyx_NewRef((PyObject *)((PyLongObject *)Py_False));
      if (!values[3]) values[3] = __Pyx_NewRef((PyObject *)((PyLongObject *)Py_True));
    }
    __pyx_v_key = ((PyObject*)values[0]);
    __pyx_v_default = values[1];
    __pyx_v_verify_checksums = ((PyLongObject *)values[2]);
    __pyx_v_fill_cache = ((PyLongObject *)values[3]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("get", 0, 1, 2, __pyx_nargs); __PYX_ERR(0, 1153, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("plyvel._plyvel.Snapshot.get", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_key), (&PyBytes_Type), 0, "key", 1))) __PYX_ERR(0, 1153, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_verify_checksums), __pyx_mstate_global->__pyx_ptype_7cpython_4bool_bool, 1, "verify_checksums", 0))) __PYX_ERR(0, 1154, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_fill_cache), __pyx_mstate_global->__pyx_ptype_7cpython_4bool_bool, 1, "fill_cache", 0))) __PYX_ERR(0, 1154, __pyx_L1_error)
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_8Snapshot_12get(((struct __pyx_obj_6plyvel_7_plyvel_Snapshot *)__pyx_v_self), __pyx_v_key, __pyx_v_default, __pyx_v_verify_checksums, __pyx_v_fill_cache);

  /* "plyvel/_plyvel.pyx":1153
 *         return False  # propagate exceptions
 * 
 *     def get(self, bytes key not None, default=None, *,             # <<<<<<<<<<<<<<
 *             bool verify_checksums=False, bool fill_cache=True):
 *         if self.db._db is NULL or self._snapshot is NULL:
*/

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_8Snapshot_12get(struct __pyx_obj_6plyvel_7_plyvel_Snapshot *__pyx_v_self, PyObject *__pyx_v_key, PyObject *__pyx_v_default, PyLongObject *__pyx_v_verify_checksums, PyLongObject *__pyx_v_fill_cache) {
  leveldb::ReadOptions __pyx_v_read_options;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  bool __pyx_t_6;
  leveldb::Snapshot *__pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get", 0);
  __Pyx_INCREF(__pyx_v_key);

  /* "plyvel/_plyvel.pyx":1155
 *     def get(self, bytes key not None, default=None, *,
 *             bool verify_checksums=False, bool fill_cache=True):
 *         if self.db._db is NULL or self._snapshot is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database or snapshot is closed")
 * 
*/
  __pyx_t_2 = (__pyx_v_self->db->_db == NULL);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = (__pyx_v_self->_snapshot == NULL);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  if (unlikely(__pyx_t_1)) {

    /* "plyvel/_plyvel.pyx":1156
 *             bool verify_checksums=False, bool fill_cache=True):
 *         if self.db._db is NULL or self._snapshot is NULL:
 *             raise RuntimeError("Database or snapshot is closed")             # <<<<<<<<<<<<<<
 * 
 *         cdef ReadOptions read_options
*/
    __pyx_t_4 = NULL;
    __pyx_t_5 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_mstate_global->__pyx_kp_u_Database_or_snapshot_is_closed};
      __pyx_t_3 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_RuntimeError)), __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1156, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
    }
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(0, 1156, __pyx_L1_error)

    /* "plyvel/_plyvel.pyx":1155
 *     def get(self, bytes key not None, default=None, *,
 *             bool verify_checksums=False, bool fill_cache=True):
 *         if self.db._db is NULL or self._snapshot is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database or snapshot is closed")
 * 
*/
  }

  /* "plyvel/_plyvel.pyx":1159
 * 
 *         cdef ReadOptions read_options
 *         read_options.verify_checksums = verify_checksums             # <<<<<<<<<<<<<<
 *         read_options.fill_cache = fill_cache
 *         read_options.snapshot = self._snapshot
*/
  __pyx_t_6 = __Pyx_PyObject_IsTrue(((PyObject *)__pyx_v_verify_checksums)); if (unlikely((__pyx_t_6 == ((bool)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1159, __pyx_L1_error)
  __pyx_v_read_options.verify_checksums = __pyx_t_6;

  /* "plyvel/_plyvel.pyx":1160
 *         cdef ReadOptions read_options
 *         read_options.verify_checksums = verify_checksums
 *         read_options.fill_cache = fill_cache             # <<<<<<<<<<<<<<
 *         read_options.snapshot = self._snapshot
 * 
*/
  __pyx_t_6 = __Pyx_PyObject_IsTrue(((PyObject *)__pyx_v_fill_cache)); if (unlikely((__pyx_t_6 == ((bool)-1)) && PyErr_Occurred())) __PYX_ERR(0, 1160, __pyx_L1_error)
  __pyx_v_read_options.fill_cache = __pyx_t_6;

  /* "plyvel/_plyvel.pyx":1161
 *         read_options.verify_checksums = verify_checksums
 *         read_options.fill_cache = fill_cache
 *         read_options.snapshot = self._snapshot             # <<<<<<<<<<<<<<
 * 
 *         if self.prefix is not None:
*/
  __pyx_t_7 = __pyx_v_self->_snapshot;
  __pyx_v_read_options.snapshot = __pyx_t_7;

  /* "plyvel/_plyvel.pyx":1163
 *         read_options.snapshot = self._snapshot
 * 
 *         if self.prefix is not None:             # <<<<<<<<<<<<<<
 *             key = self.prefix + key
 * 
*/
  __pyx_t_1 = (__pyx_v_self->prefix != ((PyObject*)Py_None));
  if (__pyx_t_1) {

    /* "plyvel/_plyvel.pyx":1164
 * 
 *         if self.prefix is not None:
 *             key = self.prefix + key             # <<<<<<<<<<<<<<
 * 
 *         return db_get(self.db, key, default, read_options)
*/
    __pyx_t_3 = PyNumber_Add(__pyx_v_self->prefix, __pyx_v_key); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1164, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF_SET(__pyx_v_key, ((PyObject*)__pyx_t_3));
    __pyx_t_3 = 0;

    /* "plyvel/_plyvel.pyx":1163
 *         read_options.snapshot = self._snapshot
 * 
 *         if self.prefix is not None:             # <<<<<<<<<<<<<<
 *             key = self.prefix + key
 * 
*/
  }

  /* "plyvel/_plyvel.pyx":1166
 *             key = self.prefix + key
 * 
 *         return db_get(self.db, key, default, read_options)             # <<<<<<<<<<<<<<
 * 
 *     def __iter__(self):
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_3 = ((PyObject *)__pyx_v_self->db);
  __Pyx_INCREF(__pyx_t_3);
  __pyx_t_4 = __pyx_f_6plyvel_7_plyvel_db_get(((struct __pyx_obj_6plyvel_7_plyvel_DB *)__pyx_t_3), __pyx_v_key, __pyx_v_default, __pyx_v_read_options); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 1166, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_r = __pyx_t_4;
  __pyx_t_4 = 0;
  goto __pyx_L0;

  /* "plyvel/_plyvel.pyx":1153
 *         return False  # propagate exceptions
 * 
 *     def get(self, bytes key not None, default=None, *,             # <<<<<<<<<<<<<<
 *             bool verify_checksums=False, bool fill_cache=True):
 *         if self.db._db is NULL or self._snapshot is NULL:
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("plyvel._plyvel.Snapshot.get", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_key);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":1168
 *         return db_get(self.db, key, default, read_options)
 * 
 *     def __iter__(self):             # <<<<<<<<<<<<<<
 *         return self.iterator()
 * 
*/

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_8Snapshot_15__iter__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_6plyvel_7_plyvel_8Snapshot_15__iter__(PyObject *__pyx_v_self) {
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__iter__ (wrapper)", 0);
  __pyx_kwvalues = __Pyx_KwValues_VARARGS(__pyx_args, __pyx_nargs);
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_8Snapshot_14__iter__(((struct __pyx_obj_6plyvel_7_plyvel_Snapshot *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_8Snapshot_14__iter__(struct __pyx_obj_6plyvel_7_plyvel_Snapshot *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  size_t __pyx_t_3;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__iter__", 0);

  /* "plyvel/_plyvel.pyx":1169
 * 
 *     def __iter__(self):
 *         return self.iterator()             # <<<<<<<<<<<<<<
 * 
 *     def iterator(self, *, reverse=False, start=None, stop=None,
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = ((PyObject *)__pyx_v_self);
  __Pyx_INCREF(__pyx_t_2);
  __pyx_t_3 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_2, NULL};
    __pyx_t_1 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_iterator, __pyx_callargs+__pyx_t_3, (1-__pyx_t_3) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1169, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
  }
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "plyvel/_plyvel.pyx":1168
 *         return db_get(self.db, key, default, read_options)
 * 
 *     def __iter__(self):             # <<<<<<<<<<<<<<
 *         return self.iterator()
 * 
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("plyvel._plyvel.Snapshot.__iter__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":1171
 *         return self.iterator()
 * 
 *     def iterator(self, *, reverse=False, start=None, stop=None,             # <<<<<<<<<<<<<<
 *                  include_start=True, include_stop=False, prefix=None,
 *                  include_key=True, include_value=True,
*/

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_8Snapshot_17iterator(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6plyvel_7_plyvel_8Snapshot_16iterator, "Snapshot.iterator(self, *, reverse=False, start=None, stop=None, include_start=True, include_stop=False, prefix=None, include_key=True, include_value=True, bool verify_checksums=False, bool fill_cache=True)");
static PyMethodDef __pyx_mdef_6plyvel_7_plyvel_8Snapshot_17iterator = {"iterator", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_8Snapshot_17iterator, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_8Snapshot_16iterator};
static PyObject *__pyx_pw_6plyvel_7_plyvel_8Snapshot_17iterator(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v_reverse = 0;
  PyObject *__pyx_v_start = 0;
  PyObject *__pyx_v_stop = 0;
  PyObject *__pyx_v_include_start = 0;
  PyObject *__pyx_v_include_stop = 0;
  PyObject *__pyx_v_prefix = 0;
  PyObject *__pyx_v_include_key = 0;
  PyObject *__pyx_v_include_value = 0;
  PyLongObject *__pyx_v_verify_checksums = 0;
  PyLongObject *__pyx_v_fill_cache = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[10] = {0,0,0,0,0,0,0,0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("iterator (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_reverse,&__pyx_mstate_global->__pyx_n_u_start,&__pyx_mstate_global->__pyx_n_u_stop,&__pyx_mstate_global->__pyx_n_u_include_start,&__pyx_mstate_global->__pyx_n_u_include_stop,&__pyx_mstate_global->__pyx_n_u_prefix,&__pyx_mstate_global->__pyx_n_u_include_key,&__pyx_mstate_global->__pyx_n_u_include_value,&__pyx_mstate_global->__pyx_n_u_verify_checksums,&__pyx_mstate_global->__pyx_n_u_fill_cache,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(0, 1171, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, 0, __pyx_kwds_len, "iterator", 0) < (0)) __PYX_ERR(0, 1171, __pyx_L3_error)
      if (!values[0]) values[0] = __Pyx_NewRef(((PyObject *)Py_False));
      if (!values[1]) values[1] = __Pyx_NewRef(((PyObject *)Py_None));
      if (!values[2]) values[2] = __Pyx_NewRef(((PyObject *)Py_None));

      /* "plyvel/_plyvel.pyx":1172
 * 
 *     def iterator(self, *, reverse=False, start=None, stop=None,
 *                  include_start=True, include_stop=False, prefix=None,             # <<<<<<<<<<<<<<
 *                  include_key=True, include_value=True,
 *                  bool verify_checksums=False, bool fill_cache=True):
*/
      if (!values[3]) values[3] = __Pyx_NewRef(((PyObject *)Py_True));
      if (!values[4]) values[4] = __Pyx_NewRef(((PyObject *)Py_False));
      if (!values[5]) values[5] = __Pyx_NewRef(((PyObject *)Py_None));

      /* "plyvel/_plyvel.pyx":1173
 *     def iterator(self, *, reverse=False, start=None, stop=None,
 *                  include_start=True, include_stop=False, prefix=None,
 *                  include_key=True, include_value=True,             # <<<<<<<<<<<<<<
 *                  bool verify_checksums=False, bool fill_cache=True):
 *         if self.db._db is NULL or self._snapshot is NULL:
*/
      if (!values[6]) values[6] = __Pyx_NewRef(((PyObject *)Py_True));
      if (!values[7]) values[7] = __Pyx_NewRef(((PyObject *)Py_True));

      /* "plyvel/_plyvel.pyx":1174
 *                  include_start=True, include_stop=False, prefix=None,
 *                  include_key=True, include_value=True,
 *                  bool verify_checksums=False, bool fill_cache=True):             # <<<<<<<<<<<<<<
 *         if self.db._db is NULL or self._snapshot is NULL:
 *             raise RuntimeError("Database or snapshot is closed")
*/
      if (!values[8]) values[8] = __Pyx_NewRef((PyObject *)((PyLongObject *)Py_False));
      if (!values[9]) values[9] = __Pyx_NewRef((PyObject *)((PyLongObject *)Py_True));
    } else if (unlikely(__pyx_nargs != 0)) {
      goto __pyx_L5_argtuple_error;
    } else {

      /* "plyvel/_plyvel.pyx":1171
 *         return self.iterator()
 * 
 *     def iterator(self, *, reverse=False, start=None, stop=None,             # <<<<<<<<<<<<<<
 *                  include_start=True, include_stop=False, prefix=None,
 *                  include_key=True, include_value=True,
*/
      if (!values[0]) values[0] = __Pyx_NewRef(((PyObject *)Py_False));
      if (!values[1]) values[1] = __Pyx_NewRef(((PyObject *)Py_None));
      if (!values[2]) values[2] = __Pyx_NewRef(((PyObject *)Py_None));

      /* "plyvel/_plyvel.pyx":1172
 * 
 *     def iterator(self, *, reverse=False, start=None, stop=None,
 *                  include_start=True, include_stop=False, prefix=None,             # <<<<<<<<<<<<<<
 *                  include_key=True, include_value=True,
 *                  bool verify_checksums=False, bool fill_cache=True):
*/
      if (!values[3]) values[3] = __Pyx_NewRef(((PyObject *)Py_True));
      if (!values[4]) values[4] = __Pyx_NewRef(((PyObject *)Py_False));
      if (!values[5]) values[5] = __Pyx_NewRef(((PyObject *)Py_None));

      /* "plyvel/_plyvel.pyx":1173
 *     def iterator(self, *, reverse=False, start=None, stop=None,
 *                  include_start=True, include_stop=False, prefix=None,
 *                  include_key=True, include_value=True,             # <<<<<<<<<<<<<<
 *                  bool verify_checksums=False, bool fill_cache=True):
 *         if self.db._db is NULL or self._snapshot is NULL:
*/
      if (!values[6]) values[6] = __Pyx_NewRef(((PyObject *)Py_True));
      if (!values[7]) values[7] = __Pyx_NewRef(((PyObject *)Py_True));

      /* "plyvel/_plyvel.pyx":1174
 *                  include_start=True, include_stop=False, prefix=None,
 *                  include_key=True, include_value=True,
 *                  bool verify_checksums=False, bool fill_cache=True):             # <<<<<<<<<<<<<<
 *         if self.db._db is NULL or self._snapshot is NULL:
 *             raise RuntimeError("Database or snapshot is closed")
*/
      if (!values[8]) values[8] = __Pyx_NewRef((PyObject *)((PyLongObject *)Py_False));
      if (!values[9]) values[9] = __Pyx_NewRef((PyObject *)((PyLongObject *)Py_True));
    }
    __pyx_v_reverse = values[0];
    __pyx_v_start = values[1];
    __pyx_v_stop = values[2];
    __pyx_v_include_start = values[3];
    __pyx_v_include_stop = values[4];
    __pyx_v_prefix = values[5];
    __pyx_v_include_key = values[6];
    __pyx_v_include_value = values[7];
    __pyx_v_verify_checksums = ((PyLongObject *)values[8]);
    __pyx_v_fill_cache = ((PyLongObject *)values[9]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("iterator", 1, 0, 0, __pyx_nargs); __PYX_ERR(0, 1171, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("plyvel._plyvel.Snapshot.iterator", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_verify_checksums), __pyx_mstate_global->__pyx_ptype_7cpython_4bool_bool, 1, "verify_checksums", 0))) __PYX_ERR(0, 1174, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_fill_cache), __pyx_mstate_global->__pyx_ptype_7cpython_4bool_bool, 1, "fill_cache", 0))) __PYX_ERR(0, 1174, __pyx_L1_error)
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_8Snapshot_16iterator(((struct __pyx_obj_6plyvel_7_plyvel_Snapshot *)__pyx_v_self), __pyx_v_reverse, __pyx_v_start, __pyx_v_stop, __pyx_v_include_start, __pyx_v_include_stop, __pyx_v_prefix, __pyx_v_include_key, __pyx_v_include_value, __pyx_v_verify_checksums, __pyx_v_fill_cache);

  /* "plyvel/_plyvel.pyx":1171
 *         return self.iterator()
 * 
 *     def iterator(self, *, reverse=False, start=None, stop=None,             # <<<<<<<<<<<<<<
 *                  include_start=True, include_stop=False, prefix=None,
 *                  include_key=True, include_value=True,
*/

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_8Snapshot_16iterator(struct __pyx_obj_6plyvel_7_plyvel_Snapshot *__pyx_v_self, PyObject *__pyx_v_reverse, PyObject *__pyx_v_start, PyObject *__pyx_v_stop, PyObject *__pyx_v_include_start, PyObject *__pyx_v_include_stop, PyObject *__pyx_v_prefix, PyObject *__pyx_v_include_key, PyObject *__pyx_v_include_value, PyLongObject *__pyx_v_verify_checksums, PyLongObject *__pyx_v_fill_cache) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("iterator", 0);

  /* "plyvel/_plyvel.pyx":1175
 *                  include_key=True, include_value=True,
 *                  bool verify_checksums=False, bool fill_cache=True):
 *         if self.db._db is NULL or self._snapshot is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database or snapshot is closed")
 * 
*/
  __pyx_t_2 = (__pyx_v_self->db->_db == NULL);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = (__pyx_v_self->_snapshot == NULL);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  if (unlikely(__pyx_t_1)) {

    /* "plyvel/_plyvel.pyx":1176
 *                  bool verify_checksums=False, bool fill_cache=True):
 *         if self.db._db is NULL or self._snapshot is NULL:
 *             raise RuntimeError("Database or snapshot is closed")             # <<<<<<<<<<<<<<
 * 
 *         return Iterator(
*/
    __pyx_t_4 = NULL;
    __pyx_t_5 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_mstate_global->__pyx_kp_u_Database_or_snapshot_is_closed};
      __pyx_t_3 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_RuntimeError)), __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1176, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
    }
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(0, 1176, __pyx_L1_error)

    /* "plyvel/_plyvel.pyx":1175
 *                  include_key=True, include_value=True,
 *                  bool verify_checksums=False, bool fill_cache=True):
 *         if self.db._db is NULL or self._snapshot is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database or snapshot is closed")
 * 
*/
  }

  /* "plyvel/_plyvel.pyx":1178
 *             raise RuntimeError("Database or snapshot is closed")
 * 
 *         return Iterator(             # <<<<<<<<<<<<<<
 *             db=self.db, db_prefix=self.prefix, reverse=reverse, start=start,
 *             stop=stop, include_start=include_start, include_stop=include_stop,
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_4 = NULL;

  /* "plyvel/_plyvel.pyx":1183
 *             prefix=prefix, include_key=include_key,
 *             include_value=include_value, verify_checksums=verify_checksums,
 *             fill_cache=fill_cache, snapshot=self)             # <<<<<<<<<<<<<<
 * 
 *     def raw_iterator(self, *, bool verify_checksums=False,
*/
  __pyx_t_5 = 1;
  {
    PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 13 : 0)] = {__pyx_t_4, NULL};
    __pyx_t_6 = __Pyx_MakeVectorcallBuilderKwds(13); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1178, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_db, ((PyObject *)__pyx_v_self->db), __pyx_t_6, __pyx_callargs+1, 0) < (0)) __PYX_ERR(0, 1178, __pyx_L1_error)
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_db_prefix, __pyx_v_self->prefix, __pyx_t_6, __pyx_callargs+1, 1) < (0)) __PYX_ERR(0, 1178, __pyx_L1_error)
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_reverse, __pyx_v_reverse, __pyx_t_6, __pyx_callargs+1, 2) < (0)) __PYX_ERR(0, 1178, __pyx_L1_error)
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_start, __pyx_v_start, __pyx_t_6, __pyx_callargs+1, 3) < (0)) __PYX_ERR(0, 1178, __pyx_L1_error)
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_stop, __pyx_v_stop, __pyx_t_6, __pyx_callargs+1, 4) < (0)) __PYX_ERR(0, 1178, __pyx_L1_error)
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_include_start, __pyx_v_include_start, __pyx_t_6, __pyx_callargs+1, 5) < (0)) __PYX_ERR(0, 1178, __pyx_L1_error)
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_include_stop, __pyx_v_include_stop, __pyx_t_6, __pyx_callargs+1, 6) < (0)) __PYX_ERR(0, 1178, __pyx_L1_error)
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_prefix, __pyx_v_prefix, __pyx_t_6, __pyx_callargs+1, 7) < (0)) __PYX_ERR(0, 1178, __pyx_L1_error)
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_include_key, __pyx_v_include_key, __pyx_t_6, __pyx_callargs+1, 8) < (0)) __PYX_ERR(0, 1178, __pyx_L1_error)
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_include_value, __pyx_v_include_value, __pyx_t_6, __pyx_callargs+1, 9) < (0)) __PYX_ERR(0, 1178, __pyx_L1_error)
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_verify_checksums, ((PyObject *)__pyx_v_verify_checksums), __pyx_t_6, __pyx_callargs+1, 10) < (0)) __PYX_ERR(0, 1178, __pyx_L1_error)
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_fill_cache, ((PyObject *)__pyx_v_fill_cache), __pyx_t_6, __pyx_callargs+1, 11) < (0)) __PYX_ERR(0, 1178, __pyx_L1_error)
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_snapshot, ((PyObject *)__pyx_v_self), __pyx_t_6, __pyx_callargs+1, 12) < (0)) __PYX_ERR(0, 1178, __pyx_L1_error)
    __pyx_t_3 = __Pyx_Object_Vectorcall_CallFromBuilder((PyObject*)__pyx_mstate_global->__pyx_ptype_6plyvel_7_plyvel_Iterator, __pyx_callargs+__pyx_t_5, (1-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_6);
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1178, __pyx_L1_error)
    __Pyx_GOTREF((PyObject *)__pyx_t_3);
  }
  __pyx_r = ((PyObject *)__pyx_t_3);
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "plyvel/_plyvel.pyx":1171
 *         return self.iterator()
 * 
 *     def iterator(self, *, reverse=False, start=None, stop=None,             # <<<<<<<<<<<<<<
 *                  include_start=True, include_stop=False, prefix=None,
 *                  include_key=True, include_value=True,
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("plyvel._plyvel.Snapshot.iterator", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "plyvel/_plyvel.pyx":1185
 *             fill_cache=fill_cache, snapshot=self)
 * 
 *     def raw_iterator(self, *, bool verify_checksums=False,             # <<<<<<<<<<<<<<
 *                      bool fill_cache=True):
 *         if self.db._db is NULL or self._snapshot is NULL:
*/

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_8Snapshot_19raw_iterator(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6plyvel_7_plyvel_8Snapshot_18raw_iterator, "Snapshot.raw_iterator(self, *, bool verify_checksums=False, bool fill_cache=True)");
static PyMethodDef __pyx_mdef_6plyvel_7_plyvel_8Snapshot_19raw_iterator = {"raw_iterator", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_8Snapshot_19raw_iterator, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_8Snapshot_18raw_iterator};
static PyObject *__pyx_pw_6plyvel_7_plyvel_8Snapshot_19raw_iterator(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyLongObject *__pyx_v_verify_checksums = 0;
  PyLongObject *__pyx_v_fill_cache = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[2] = {0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("raw_iterator (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_verify_checksums,&__pyx_mstate_global->__pyx_n_u_fill_cache,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(0, 1185, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, 0, __pyx_kwds_len, "raw_iterator", 0) < (0)) __PYX_ERR(0, 1185, __pyx_L3_error)
      if (!values[0]) values[0] = __Pyx_NewRef((PyObject *)((PyLongObject *)Py_False));

      /* "plyvel/_plyvel.pyx":1186
 * 
 *     def raw_iterator(self, *, bool verify_checksums=False,
 *                      bool fill_cache=True):             # <<<<<<<<<<<<<<
 *         if self.db._db is NULL or self._snapshot is NULL:
 *             raise RuntimeError("Database or snapshot is closed")
*/
      if (!values[1]) values[1] = __Pyx_NewRef((PyObject *)((PyLongObject *)Py_True));
    } else if (unlikely(__pyx_nargs != 0)) {
      goto __pyx_L5_argtuple_error;
    } else {

      /* "plyvel/_plyvel.pyx":1185
 *             fill_cache=fill_cache, snapshot=self)
 * 
 *     def raw_iterator(self, *, bool verify_checksums=False,             # <<<<<<<<<<<<<<
 *                      bool fill_cache=True):
 *         if self.db._db is NULL or self._snapshot is NULL:
*/
      if (!values[0]) values[0] = __Pyx_NewRef((PyObject *)((PyLongObject *)Py_False));

      /* "plyvel/_plyvel.pyx":1186
 * 
 *     def raw_iterator(self, *, bool verify_checksums=False,
 *                      bool fill_cache=True):             # <<<<<<<<<<<<<<
 *         if self.db._db is NULL or self._snapshot is NULL:
 *             raise RuntimeError("Database or snapshot is closed")
*/
      if (!values[1]) values[1] = __Pyx_NewRef((PyObject *)((PyLongObject *)Py_True));
    }
    __pyx_v_verify_checksums = ((PyLongObject *)values[0]);
    __pyx_v_fill_cache = ((PyLongObject *)values[1]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("raw_iterator", 1, 0, 0, __pyx_nargs); __PYX_ERR(0, 1185, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("plyvel._plyvel.Snapshot.raw_iterator", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_verify_checksums), __pyx_mstate_global->__pyx_ptype_7cpython_4bool_bool, 1, "verify_checksums", 0))) __PYX_ERR(0, 1185, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_fill_cache), __pyx_mstate_global->__pyx_ptype_7cpython_4bool_bool, 1, "fill_cache", 0))) __PYX_ERR(0, 1186, __pyx_L1_error)
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_8Snapshot_18raw_iterator(((struct __pyx_obj_6plyvel_7_plyvel_Snapshot *)__pyx_v_self), __pyx_v_verify_checksums, __pyx_v_fill_cache);

  /* "plyvel/_plyvel.pyx":1185
 *             fill_cache=fill_cache, snapshot=self)
 * 
 *     def raw_iterator(self, *, bool verify_checksums=False,             # <<<<<<<<<<<<<<
 *                      bool fill_cache=True):
 *         if self.db._db is NULL or self._snapshot is NULL:
*/

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_8Snapshot_18raw_iterator(struct __pyx_obj_6plyvel_7_plyvel_Snapshot *__pyx_v_self, PyLongObject *__pyx_v_verify_checksums, PyLongObject *__pyx_v_fill_cache) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  size_t __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("raw_iterator", 0);

  /* "plyvel/_plyvel.pyx":1187
 *     def raw_iterator(self, *, bool verify_checksums=False,
 *                      bool fill_cache=True):
 *         if self.db._db is NULL or self._snapshot is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database or snapshot is closed")
 * 
*/
  __pyx_t_2 = (__pyx_v_self->db->_db == NULL);
  if (!__pyx_t_2) {
  } else {
    __pyx_t_1 = __pyx_t_2;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_2 = (__pyx_v_self->_snapshot == NULL);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L4_bool_binop_done:;
  if (unlikely(__pyx_t_1)) {

    /* "plyvel/_plyvel.pyx":1188
 *                      bool fill_cache=True):
 *         if self.db._db is NULL or self._snapshot is NULL:
 *             raise RuntimeError("Database or snapshot is closed")             # <<<<<<<<<<<<<<
 * 
 *         return RawIterator(
*/
    __pyx_t_4 = NULL;
    __pyx_t_5 = 1;
    {
      PyObject *__pyx_callargs[2] = {__pyx_t_4, __pyx_mstate_global->__pyx_kp_u_Database_or_snapshot_is_closed};
      __pyx_t_3 = __Pyx_PyObject_FastCall((PyObject*)(((PyTypeObject*)PyExc_RuntimeError)), __pyx_callargs+__pyx_t_5, (2-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
      __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1188, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
    }
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(0, 1188, __pyx_L1_error)

    /* "plyvel/_plyvel.pyx":1187
 *     def raw_iterator(self, *, bool verify_checksums=False,
 *                      bool fill_cache=True):
 *         if self.db._db is NULL or self._snapshot is NULL:             # <<<<<<<<<<<<<<
 *             raise RuntimeError("Database or snapshot is closed")
 * 
*/
  }

  /* "plyvel/_plyvel.pyx":1190
 *             raise RuntimeError("Database or snapshot is closed")
 * 
 *         return RawIterator(             # <<<<<<<<<<<<<<
 *             db=self.db,
 *             verify_checksums=verify_checksums,
*/
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_4 = NULL;

  /* "plyvel/_plyvel.pyx":1194
 *             verify_checksums=verify_checksums,
 *             fill_cache=fill_cache,
 *             snapshot=self)             # <<<<<<<<<<<<<<
*/
  __pyx_t_5 = 1;
  {
    PyObject *__pyx_callargs[2 + ((CYTHON_VECTORCALL) ? 4 : 0)] = {__pyx_t_4, NULL};
    __pyx_t_6 = __Pyx_MakeVectorcallBuilderKwds(4); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 1190, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_db, ((PyObject *)__pyx_v_self->db), __pyx_t_6, __pyx_callargs+1, 0) < (0)) __PYX_ERR(0, 1190, __pyx_L1_error)
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_verify_checksums, ((PyObject *)__pyx_v_verify_checksums), __pyx_t_6, __pyx_callargs+1, 1) < (0)) __PYX_ERR(0, 1190, __pyx_L1_error)
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_fill_cache, ((PyObject *)__pyx_v_fill_cache), __pyx_t_6, __pyx_callargs+1, 2) < (0)) __PYX_ERR(0, 1190, __pyx_L1_error)
    if (__Pyx_VectorcallBuilder_AddArg(__pyx_mstate_global->__pyx_n_u_snapshot, ((PyObject *)__pyx_v_self), __pyx_t_6, __pyx_callargs+1, 3) < (0)) __PYX_ERR(0, 1190, __pyx_L1_error)
    __pyx_t_3 = __Pyx_Object_Vectorcall_CallFromBuilder((PyObject*)__pyx_mstate_global->__pyx_ptype_6plyvel_7_plyvel_RawIterator, __pyx_callargs+__pyx_t_5, (1-__pyx_t_5) | (__pyx_t_5*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET), __pyx_t_6);
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 1190, __pyx_L1_error)
    __Pyx_GOTREF((PyObject *)__pyx_t_3);
  }
  __pyx_r = ((PyObject *)__pyx_t_3);
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "plyvel/_plyvel.pyx":1185
 *             fill_cache=fill_cache, snapshot=self)
 * 
 *     def raw_iterator(self, *, bool verify_checksums=False,             # <<<<<<<<<<<<<<
 *                      bool fill_cache=True):
 *         if self.db._db is NULL or self._snapshot is NULL:
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("plyvel._plyvel.Snapshot.raw_iterator", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self._snapshot cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):
*/

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_8Snapshot_21__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6plyvel_7_plyvel_8Snapshot_20__reduce_cython__, "Snapshot.__reduce_cython__(self)");
static PyMethodDef __pyx_mdef_6plyvel_7_plyvel_8Snapshot_21__reduce_cython__ = {"__reduce_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_8Snapshot_21__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_8Snapshot_20__reduce_cython__};
static PyObject *__pyx_pw_6plyvel_7_plyvel_8Snapshot_21__reduce_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  if (unlikely(__pyx_nargs > 0)) { __Pyx_RaiseArgtupleInvalid("__reduce_cython__", 1, 0, 0, __pyx_nargs); return NULL; }
  const Py_ssize_t __pyx_kwds_len = unlikely(__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
  if (unlikely(__pyx_kwds_len < 0)) return NULL;
  if (unlikely(__pyx_kwds_len > 0)) {__Pyx_RejectKeywords("__reduce_cython__", __pyx_kwds); return NULL;}
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_8Snapshot_20__reduce_cython__(((struct __pyx_obj_6plyvel_7_plyvel_Snapshot *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_8Snapshot_20__reduce_cython__(CYTHON_UNUSED struct __pyx_obj_6plyvel_7_plyvel_Snapshot *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":2
 * def __reduce_cython__(self):
 *     raise TypeError, "self._snapshot cannot be converted to a Python object for pickling"             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError, "self._snapshot cannot be converted to a Python object for pickling"
*/
  __Pyx_Raise(((PyObject *)(((PyTypeObject*)PyExc_TypeError))), __pyx_mstate_global->__pyx_kp_u_self__snapshot_cannot_be_convert, 0, 0);
  __PYX_ERR(1, 2, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self._snapshot cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("plyvel._plyvel.Snapshot.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError, "self._snapshot cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self._snapshot cannot be converted to a Python object for pickling"
*/

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_8Snapshot_23__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6plyvel_7_plyvel_8Snapshot_22__setstate_cython__, "Snapshot.__setstate_cython__(self, __pyx_state)");
static PyMethodDef __pyx_mdef_6plyvel_7_plyvel_8Snapshot_23__setstate_cython__ = {"__setstate_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_8Snapshot_23__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_8Snapshot_22__setstate_cython__};
static PyObject *__pyx_pw_6plyvel_7_plyvel_8Snapshot_23__setstate_cython__(PyObject *__pyx_v_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  CYTHON_UNUSED PyObject *__pyx_v___pyx_state = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[1] = {0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_pyx_state,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(1, 3, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(1, 3, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__setstate_cython__", 0) < (0)) __PYX_ERR(1, 3, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 1; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, i); __PYX_ERR(1, 3, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 1)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(1, 3, __pyx_L3_error)
    }
    __pyx_v___pyx_state = values[0];
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__setstate_cython__", 1, 1, 1, __pyx_nargs); __PYX_ERR(1, 3, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("plyvel._plyvel.Snapshot.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_8Snapshot_22__setstate_cython__(((struct __pyx_obj_6plyvel_7_plyvel_Snapshot *)__pyx_v_self), __pyx_v___pyx_state);

  /* function exit code */
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_8Snapshot_22__setstate_cython__(CYTHON_UNUSED struct __pyx_obj_6plyvel_7_plyvel_Snapshot *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":4
 *     raise TypeError, "self._snapshot cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):
 *     raise TypeError, "self._snapshot cannot be converted to a Python object for pickling"             # <<<<<<<<<<<<<<
*/
  __Pyx_Raise(((PyObject *)(((PyTypeObject*)PyExc_TypeError))), __pyx_mstate_global->__pyx_kp_u_self__snapshot_cannot_be_convert, 0, 0);
  __PYX_ERR(1, 4, __pyx_L1_error)

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError, "self._snapshot cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self._snapshot cannot be converted to a Python object for pickling"
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("plyvel._plyvel.Snapshot.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":4
 *     int __Pyx_CheckUnpickleChecksum(long, long, long, long, const char*) except -1
 *     int __Pyx_UpdateUnpickledDict(object, object, Py_ssize_t) except -1
 * def __pyx_unpickle_PrefixedDB(__pyx_type, long __pyx_checksum, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_result
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0xe9c0b3a, 0x8ba9a5d, 0x11ea7f1, b'db, prefix')
*/

/* Python wrapper */
static PyObject *__pyx_pw_6plyvel_7_plyvel_5__pyx_unpickle_PrefixedDB(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
); /*proto*/
PyDoc_STRVAR(__pyx_doc_6plyvel_7_plyvel_4__pyx_unpickle_PrefixedDB, "__pyx_unpickle_PrefixedDB(__pyx_type, long __pyx_checksum, tuple __pyx_state)");
static PyMethodDef __pyx_mdef_6plyvel_7_plyvel_5__pyx_unpickle_PrefixedDB = {"__pyx_unpickle_PrefixedDB", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_5__pyx_unpickle_PrefixedDB, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_4__pyx_unpickle_PrefixedDB};
static PyObject *__pyx_pw_6plyvel_7_plyvel_5__pyx_unpickle_PrefixedDB(PyObject *__pyx_self, 
#if CYTHON_METH_FASTCALL
PyObject *const *__pyx_args, Py_ssize_t __pyx_nargs, PyObject *__pyx_kwds
#else
PyObject *__pyx_args, PyObject *__pyx_kwds
#endif
) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  #if !CYTHON_METH_FASTCALL
  CYTHON_UNUSED Py_ssize_t __pyx_nargs;
  #endif
  CYTHON_UNUSED PyObject *const *__pyx_kwvalues;
  PyObject* values[3] = {0,0,0};
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_PrefixedDB (wrapper)", 0);
  #if !CYTHON_METH_FASTCALL
  #if CYTHON_ASSUME_SAFE_SIZE
  __pyx_nargs = PyTuple_GET_SIZE(__pyx_args);
  #else
  __pyx_nargs = PyTuple_Size(__pyx_args); if (unlikely(__pyx_nargs < 0)) return NULL;
  #endif
  #endif
  __pyx_kwvalues = __Pyx_KwValues_FASTCALL(__pyx_args, __pyx_nargs);
  {
    PyObject ** const __pyx_pyargnames[] = {&__pyx_mstate_global->__pyx_n_u_pyx_type,&__pyx_mstate_global->__pyx_n_u_pyx_checksum,&__pyx_mstate_global->__pyx_n_u_pyx_state,0};
    const Py_ssize_t __pyx_kwds_len = (__pyx_kwds) ? __Pyx_NumKwargs_FASTCALL(__pyx_kwds) : 0;
    if (unlikely(__pyx_kwds_len) < 0) __PYX_ERR(1, 4, __pyx_L3_error)
    if (__pyx_kwds_len > 0) {
      switch (__pyx_nargs) {
        case  3:
        values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(1, 4, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  2:
        values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(1, 4, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  1:
        values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
        if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(1, 4, __pyx_L3_error)
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      const Py_ssize_t kwd_pos_args = __pyx_nargs;
      if (__Pyx_ParseKeywords(__pyx_kwds, __pyx_kwvalues, __pyx_pyargnames, 0, values, kwd_pos_args, __pyx_kwds_len, "__pyx_unpickle_PrefixedDB", 0) < (0)) __PYX_ERR(1, 4, __pyx_L3_error)
      for (Py_ssize_t i = __pyx_nargs; i < 3; i++) {
        if (unlikely(!values[i])) { __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_PrefixedDB", 1, 3, 3, i); __PYX_ERR(1, 4, __pyx_L3_error) }
      }
    } else if (unlikely(__pyx_nargs != 3)) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = __Pyx_ArgRef_FASTCALL(__pyx_args, 0);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[0])) __PYX_ERR(1, 4, __pyx_L3_error)
      values[1] = __Pyx_ArgRef_FASTCALL(__pyx_args, 1);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[1])) __PYX_ERR(1, 4, __pyx_L3_error)
      values[2] = __Pyx_ArgRef_FASTCALL(__pyx_args, 2);
      if (!CYTHON_ASSUME_SAFE_MACROS && unlikely(!values[2])) __PYX_ERR(1, 4, __pyx_L3_error)
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyLong_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(1, 4, __pyx_L3_error)
    __pyx_v___pyx_state = ((PyObject*)values[2]);
  }
  goto __pyx_L6_skip;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_PrefixedDB", 1, 3, 3, __pyx_nargs); __PYX_ERR(1, 4, __pyx_L3_error)
  __pyx_L6_skip:;
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L3_error:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __Pyx_AddTraceback("plyvel._plyvel.__pyx_unpickle_PrefixedDB", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v___pyx_state), (&PyTuple_Type), 1, "__pyx_state", 1))) __PYX_ERR(1, 4, __pyx_L1_error)
  __pyx_r = __pyx_pf_6plyvel_7_plyvel_4__pyx_unpickle_PrefixedDB(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  goto __pyx_L7_cleaned_up;
  __pyx_L0:;
  for (Py_ssize_t __pyx_temp=0; __pyx_temp < (Py_ssize_t)(sizeof(values)/sizeof(values[0])); ++__pyx_temp) {
    Py_XDECREF(values[__pyx_temp]);
  }
  __pyx_L7_cleaned_up:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_6plyvel_7_plyvel_4__pyx_unpickle_PrefixedDB(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  size_t __pyx_t_4;
  int __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_PrefixedDB", 0);

  /* "(tree fragment)":6
 * def __pyx_unpickle_PrefixedDB(__pyx_type, long __pyx_checksum, tuple __pyx_state):
 *     cdef object __pyx_result
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0xe9c0b3a, 0x8ba9a5d, 0x11ea7f1, b'db, prefix')             # <<<<<<<<<<<<<<
 *     __pyx_result = PrefixedDB.__new__(__pyx_type)
 *     if __pyx_state is not None:
*/
  __pyx_t_1 = __Pyx_CheckUnpickleChecksum(__pyx_v___pyx_checksum, 0xe9c0b3a, 0x8ba9a5d, 0x11ea7f1, __pyx_k_db_prefix_2); if (unlikely(__pyx_t_1 == ((int)-1))) __PYX_ERR(1, 6, __pyx_L1_error)

  /* "(tree fragment)":7
 *     cdef object __pyx_result
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0xe9c0b3a, 0x8ba9a5d, 0x11ea7f1, b'db, prefix')
 *     __pyx_result = PrefixedDB.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_PrefixedDB__set_state(<PrefixedDB> __pyx_result, __pyx_state)
*/
  __pyx_t_3 = ((PyObject *)__pyx_mstate_global->__pyx_ptype_6plyvel_7_plyvel_PrefixedDB);
  __Pyx_INCREF(__pyx_t_3);
  __pyx_t_4 = 0;
  {
    PyObject *__pyx_callargs[2] = {__pyx_t_3, __pyx_v___pyx_type};
    __pyx_t_2 = __Pyx_PyObject_FastCallMethod((PyObject*)__pyx_mstate_global->__pyx_n_u_new, __pyx_callargs+__pyx_t_4, (2-__pyx_t_4) | (1*__Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET));
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 7, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
  }
  __pyx_v___pyx_result = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "(tree fragment)":8
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0xe9c0b3a, 0x8ba9a5d, 0x11ea7f1, b'db, prefix')
 *     __pyx_result = PrefixedDB.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_PrefixedDB__set_state(<PrefixedDB> __pyx_result, __pyx_state)
 *     return __pyx_result
*/
  __pyx_t_5 = (__pyx_v___pyx_state != ((PyObject*)Py_None));
  if (__pyx_t_5) {

    /* "(tree fragment)":9
 *     __pyx_result = PrefixedDB.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_PrefixedDB__set_state(<PrefixedDB> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_PrefixedDB__set_state(PrefixedDB __pyx_result, __pyx_state: tuple):
*/
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "cannot pass None into a C function argument that is declared 'not None'");
      __PYX_ERR(1, 9, __pyx_L1_error)
    }
    __pyx_t_2 = __pyx_f_6plyvel_7_plyvel___pyx_unpickle_PrefixedDB__set_state(((struct __pyx_obj_6plyvel_7_plyvel_PrefixedDB *)__pyx_v___pyx_result), __pyx_v___pyx_state); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "(tree fragment)":8
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0xe9c0b3a, 0x8ba9a5d, 0x11ea7f1, b'db, prefix')
 *     __pyx_result = PrefixedDB.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_PrefixedDB__set_state(<PrefixedDB> __pyx_result, __pyx_state)
 *     return __pyx_result
*/
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_PrefixedDB__set_state(<PrefixedDB> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_PrefixedDB__set_state(PrefixedDB __pyx_result, __pyx_state: tuple):
 *     __pyx_result.db = __pyx_state[0]; __pyx_result.prefix = __pyx_state[1]
*/
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":4
 *     int __Pyx_CheckUnpickleChecksum(long, long, long, long, const char*) except -1
 *     int __Pyx_UpdateUnpickledDict(object, object, Py_ssize_t) except -1
 * def __pyx_unpickle_PrefixedDB(__pyx_type, long __pyx_checksum, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_result
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0xe9c0b3a, 0x8ba9a5d, 0x11ea7f1, b'db, prefix')
*/

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("plyvel._plyvel.__pyx_unpickle_PrefixedDB", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_PrefixedDB__set_state(<PrefixedDB> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_PrefixedDB__set_state(PrefixedDB __pyx_result, __pyx_state: tuple):             # <<<<<<<<<<<<<<
 *     __pyx_result.db = __pyx_state[0]; __pyx_result.prefix = __pyx_state[1]
 *     __Pyx_UpdateUnpickledDict(__pyx_result, __pyx_state, 2)
*/

static PyObject *__pyx_f_6plyvel_7_plyvel___pyx_unpickle_PrefixedDB__set_state(struct __pyx_obj_6plyvel_7_plyvel_PrefixedDB *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_PrefixedDB__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_PrefixedDB__set_state(PrefixedDB __pyx_result, __pyx_state: tuple):
 *     __pyx_result.db = __pyx_state[0]; __pyx_result.prefix = __pyx_state[1]             # <<<<<<<<<<<<<<
 *     __Pyx_UpdateUnpickledDict(__pyx_result, __pyx_state, 2)
*/
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_mstate_global->__pyx_ptype_6plyvel_7_plyvel_DB))))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF((PyObject *)__pyx_v___pyx_result->db);
  __Pyx_DECREF((PyObject *)__pyx_v___pyx_result->db);
  __pyx_v___pyx_result->db = ((struct __pyx_obj_6plyvel_7_plyvel_DB *)__pyx_t_1);
  __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyLong_From_long, 0, 0, 1, 1, __Pyx_ReferenceSharing_FunctionArgument); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyBytes_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None) || __Pyx_RaiseUnexpectedTypeError("bytes", __pyx_t_1))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->prefix);
  __Pyx_DECREF(__pyx_v___pyx_result->prefix);
  __pyx_v___pyx_result->prefix = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_PrefixedDB__set_state(PrefixedDB __pyx_result, __pyx_state: tuple):
 *     __pyx_result.db = __pyx_state[0]; __pyx_result.prefix = __pyx_state[1]
 *     __Pyx_UpdateUnpickledDict(__pyx_result, __pyx_state, 2)             # <<<<<<<<<<<<<<
*/
  __pyx_t_2 = __Pyx_UpdateUnpickledDict(((PyObject *)__pyx_v___pyx_result), __pyx_v___pyx_state, 2); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(1, 13, __pyx_L1_error)

  /* "(tree fragment)":11
 *         __pyx_unpickle_PrefixedDB__set_state(<PrefixedDB> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_PrefixedDB__set_state(PrefixedDB __pyx_result, __pyx_state: tuple):             # <<<<<<<<<<<<<<
 *     __pyx_result.db = __pyx_state[0]; __pyx_result.prefix = __pyx_state[1]
 *     __Pyx_UpdateUnpickledDict(__pyx_result, __pyx_state, 2)
*/

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("plyvel._plyvel.__pyx_unpickle_PrefixedDB__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}
/* #### Code section: module_exttypes ### */
static struct __pyx_vtabstruct_6plyvel_7_plyvel_DB __pyx_vtable_6plyvel_7_plyvel_DB;

static PyObject *__pyx_tp_new_6plyvel_7_plyvel_DB(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_6plyvel_7_plyvel_DB *p;
  PyObject *o;
  o = __Pyx_AllocateExtensionType(t, 1);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_6plyvel_7_plyvel_DB *)o);
  p->__pyx_vtab = __pyx_vtabptr_6plyvel_7_plyvel_DB;
  __Pyx_default_placement_construct(&(p->options));
  p->name = Py_None; Py_INCREF(Py_None);
  p->lock = Py_None; Py_INCREF(Py_None);
  p->iterators = ((PyObject*)Py_None); Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_6plyvel_7_plyvel_DB(PyObject *o) {
  struct __pyx_obj_6plyvel_7_plyvel_DB *p = (struct __pyx_obj_6plyvel_7_plyvel_DB *)o;
  PyObject_GC_UnTrack(o);
  {
    PyObject *etype, *eval, *etb;
    PyErr_Fetch(&etype, &eval, &etb);
    __Pyx_SET_REFCNT(o, Py_REFCNT(o) + 1);
    __pyx_pw_6plyvel_7_plyvel_2DB_5__dealloc__(o);
    __Pyx_SET_REFCNT(o, Py_REFCNT(o) - 1);
    PyErr_Restore(etype, eval, etb);
  }
  __Pyx_call_destructor(p->options);
  Py_CLEAR(p->name);
  Py_CLEAR(p->lock);
  Py_CLEAR(p->iterators);
  PyTypeObject *tp = Py_TYPE(o);
  #if CYTHON_USE_TYPE_SLOTS
  (*tp->tp_free)(o);
  #else
  {
    freefunc tp_free = (freefunc)PyType_GetSlot(tp, Py_tp_free);
    if (tp_free) tp_free(o);
  }
  #endif
  #if CYTHON_USE_TYPE_SPECS
  Py_DECREF(tp);
  #endif
}

static int __pyx_tp_traverse_6plyvel_7_plyvel_DB(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_6plyvel_7_plyvel_DB *p = (struct __pyx_obj_6plyvel_7_plyvel_DB *)o;
  {
    e = __Pyx_call_type_traverse(o, 1, v, a);
    if (e) return e;
  }
  if (p->name) {
    e = (*v)(p->name, a); if (e) return e;
  }
  if (p->lock) {
    e = (*v)(p->lock, a); if (e) return e;
  }
  if (p->iterators) {
    e = (*v)(p->iterators, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_6plyvel_7_plyvel_DB(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_6plyvel_7_plyvel_DB *p = (struct __pyx_obj_6plyvel_7_plyvel_DB *)o;
  tmp = ((PyObject*)p->name);
  p->name = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->lock);
  p->lock = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->iterators);
  p->iterators = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_6plyvel_7_plyvel_2DB_closed(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6plyvel_7_plyvel_2DB_6closed_1__get__(o);
}

static PyObject *__pyx_getprop_6plyvel_7_plyvel_2DB_name(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6plyvel_7_plyvel_2DB_4name_1__get__(o);
}

static PyMethodDef __pyx_methods_6plyvel_7_plyvel_DB[] = {
  {"get", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_2DB_9get, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_2DB_8get},
  {"put", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_2DB_11put, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_2DB_10put},
  {"delete", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_2DB_13delete, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_2DB_12delete},
  {"write_batch", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_2DB_15write_batch, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_2DB_14write_batch},
  {"iterator", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_2DB_19iterator, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_2DB_18iterator},
  {"raw_iterator", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_2DB_21raw_iterator, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_2DB_20raw_iterator},
  {"snapshot", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_2DB_23snapshot, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_2DB_22snapshot},
  {"get_property", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_2DB_25get_property, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_2DB_24get_property},
  {"compact_range", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_2DB_27compact_range, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_2DB_26compact_range},
  {"approximate_size", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_2DB_29approximate_size, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_2DB_28approximate_size},
  {"approximate_sizes", (PyCFunction)(void(*)(void))(PyCFunctionWithKeywords)__pyx_pw_6plyvel_7_plyvel_2DB_31approximate_sizes, METH_VARARGS|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_2DB_30approximate_sizes},
  {"prefixed_db", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_2DB_33prefixed_db, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_2DB_32prefixed_db},
  {"__enter__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_2DB_35__enter__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_2DB_34__enter__},
  {"__exit__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_2DB_37__exit__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_2DB_36__exit__},
  {"__reduce_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_2DB_39__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_2DB_38__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_2DB_41__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_2DB_40__setstate_cython__},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_6plyvel_7_plyvel_DB[] = {
  {"closed", __pyx_getprop_6plyvel_7_plyvel_2DB_closed, 0, 0, 0},
  {"name", __pyx_getprop_6plyvel_7_plyvel_2DB_name, 0, 0, 0},
  {0, 0, 0, 0, 0}
};
#if CYTHON_USE_TYPE_SPECS
static PyType_Slot __pyx_type_6plyvel_7_plyvel_DB_slots[] = {
  {Py_tp_dealloc, (void *)__pyx_tp_dealloc_6plyvel_7_plyvel_DB},
  {Py_tp_repr, (void *)__pyx_pw_6plyvel_7_plyvel_2DB_7__repr__},
  {Py_tp_doc, (void *)PyDoc_STR("DB(name, bool create_if_missing=False, *, bool error_if_exists=False, paranoid_checks=None, write_buffer_size=None, max_open_files=None, lru_cache_size=None, block_size=None, block_restart_interval=None, max_file_size=None, compression='snappy', int bloom_filter_bits=0, comparator=None, bytes comparator_name=None)")},
  {Py_tp_traverse, (void *)__pyx_tp_traverse_6plyvel_7_plyvel_DB},
  {Py_tp_clear, (void *)__pyx_tp_clear_6plyvel_7_plyvel_DB},
  {Py_tp_iter, (void *)__pyx_pw_6plyvel_7_plyvel_2DB_17__iter__},
  {Py_tp_methods, (void *)__pyx_methods_6plyvel_7_plyvel_DB},
  {Py_tp_getset, (void *)__pyx_getsets_6plyvel_7_plyvel_DB},
  {Py_tp_init, (void *)__pyx_pw_6plyvel_7_plyvel_2DB_1__init__},
  {Py_tp_new, (void *)__pyx_tp_new_6plyvel_7_plyvel_DB},
  {0, 0},
};
static PyType_Spec __pyx_type_6plyvel_7_plyvel_DB_spec = {
  "plyvel._plyvel.DB",
  sizeof(struct __pyx_obj_6plyvel_7_plyvel_DB),
  0,
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC,
  __pyx_type_6plyvel_7_plyvel_DB_slots,
};
#else

static PyTypeObject __pyx_type_6plyvel_7_plyvel_DB = {
  PyVarObject_HEAD_INIT(0, 0)
  "plyvel._plyvel.""DB", /*tp_name*/
  sizeof(struct __pyx_obj_6plyvel_7_plyvel_DB), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_6plyvel_7_plyvel_DB, /*tp_dealloc*/
  0, /*tp_vectorcall_offset*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  0, /*tp_as_async*/
  __pyx_pw_6plyvel_7_plyvel_2DB_7__repr__, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  PyDoc_STR("DB(name, bool create_if_missing=False, *, bool error_if_exists=False, paranoid_checks=None, write_buffer_size=None, max_open_files=None, lru_cache_size=None, block_size=None, block_restart_interval=None, max_file_size=None, compression='snappy', int bloom_filter_bits=0, comparator=None, bytes comparator_name=None)"), /*tp_doc*/
  __pyx_tp_traverse_6plyvel_7_plyvel_DB, /*tp_traverse*/
  __pyx_tp_clear_6plyvel_7_plyvel_DB, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  __pyx_pw_6plyvel_7_plyvel_2DB_17__iter__, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_6plyvel_7_plyvel_DB, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_6plyvel_7_plyvel_DB, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  #if !CYTHON_USE_TYPE_SPECS
  0, /*tp_dictoffset*/
  #endif
  __pyx_pw_6plyvel_7_plyvel_2DB_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_6plyvel_7_plyvel_DB, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if CYTHON_USE_TP_FINALIZE
  0, /*tp_finalize*/
  #else
  NULL, /*tp_finalize*/
  #endif
  #if !CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800
  0, /*tp_vectorcall*/
  #endif
  #if __PYX_NEED_TP_PRINT_SLOT == 1
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030C0000
  0, /*tp_watched*/
  #endif
  #if PY_VERSION_HEX >= 0x030d00A4
  0, /*tp_versions_used*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
  0, /*tp_pypy_flags*/
  #endif
};
#endif

static PyObject *__pyx_tp_new_6plyvel_7_plyvel_PrefixedDB(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_6plyvel_7_plyvel_PrefixedDB *p;
  PyObject *o;
  o = __Pyx_AllocateExtensionType(t, 0);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_6plyvel_7_plyvel_PrefixedDB *)o);
  p->db = ((struct __pyx_obj_6plyvel_7_plyvel_DB *)Py_None); Py_INCREF(Py_None);
  p->prefix = ((PyObject*)Py_None); Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_6plyvel_7_plyvel_PrefixedDB(PyObject *o) {
  struct __pyx_obj_6plyvel_7_plyvel_PrefixedDB *p = (struct __pyx_obj_6plyvel_7_plyvel_PrefixedDB *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(__Pyx_PyObject_GetSlot(o, tp_finalize, destructor)) && !__Pyx_PyObject_GC_IsFinalized(o)) {
    if (__Pyx_PyObject_GetSlot(o, tp_dealloc, destructor) == __pyx_tp_dealloc_6plyvel_7_plyvel_PrefixedDB) {
      if (PyObject_CallFinalizerFromDealloc(o)) return;
    }
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->db);
  Py_CLEAR(p->prefix);
  PyTypeObject *tp = Py_TYPE(o);
  #if CYTHON_USE_TYPE_SLOTS
  (*tp->tp_free)(o);
  #else
  {
    freefunc tp_free = (freefunc)PyType_GetSlot(tp, Py_tp_free);
    if (tp_free) tp_free(o);
  }
  #endif
  #if CYTHON_USE_TYPE_SPECS
  Py_DECREF(tp);
  #endif
}

static int __pyx_tp_traverse_6plyvel_7_plyvel_PrefixedDB(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_6plyvel_7_plyvel_PrefixedDB *p = (struct __pyx_obj_6plyvel_7_plyvel_PrefixedDB *)o;
  {
    e = __Pyx_call_type_traverse(o, 1, v, a);
    if (e) return e;
  }
  if (p->db) {
    e = (*v)(((PyObject *)p->db), a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_6plyvel_7_plyvel_PrefixedDB(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_6plyvel_7_plyvel_PrefixedDB *p = (struct __pyx_obj_6plyvel_7_plyvel_PrefixedDB *)o;
  tmp = ((PyObject*)p->db);
  p->db = ((struct __pyx_obj_6plyvel_7_plyvel_DB *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_6plyvel_7_plyvel_10PrefixedDB_db(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6plyvel_7_plyvel_10PrefixedDB_2db_1__get__(o);
}

static PyObject *__pyx_getprop_6plyvel_7_plyvel_10PrefixedDB_prefix(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_6plyvel_7_plyvel_10PrefixedDB_6prefix_1__get__(o);
}

static PyMethodDef __pyx_methods_6plyvel_7_plyvel_PrefixedDB[] = {
  {"get", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_10PrefixedDB_5get, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_10PrefixedDB_4get},
  {"put", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_10PrefixedDB_7put, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_10PrefixedDB_6put},
  {"delete", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_10PrefixedDB_9delete, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_10PrefixedDB_8delete},
  {"write_batch", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_10PrefixedDB_11write_batch, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_10PrefixedDB_10write_batch},
  {"iterator", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_10PrefixedDB_15iterator, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_10PrefixedDB_14iterator},
  {"snapshot", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_10PrefixedDB_17snapshot, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_10PrefixedDB_16snapshot},
  {"prefixed_db", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_10PrefixedDB_19prefixed_db, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_10PrefixedDB_18prefixed_db},
  {"__reduce_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_10PrefixedDB_21__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_10PrefixedDB_20__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_10PrefixedDB_23__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_10PrefixedDB_22__setstate_cython__},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_6plyvel_7_plyvel_PrefixedDB[] = {
  {"db", __pyx_getprop_6plyvel_7_plyvel_10PrefixedDB_db, 0, 0, 0},
  {"prefix", __pyx_getprop_6plyvel_7_plyvel_10PrefixedDB_prefix, 0, 0, 0},
  {0, 0, 0, 0, 0}
};
#if CYTHON_USE_TYPE_SPECS
static PyType_Slot __pyx_type_6plyvel_7_plyvel_PrefixedDB_slots[] = {
  {Py_tp_dealloc, (void *)__pyx_tp_dealloc_6plyvel_7_plyvel_PrefixedDB},
  {Py_tp_repr, (void *)__pyx_pw_6plyvel_7_plyvel_10PrefixedDB_3__repr__},
  {Py_tp_doc, (void *)PyDoc_STR("PrefixedDB(DB db, *, bytes prefix)")},
  {Py_tp_traverse, (void *)__pyx_tp_traverse_6plyvel_7_plyvel_PrefixedDB},
  {Py_tp_clear, (void *)__pyx_tp_clear_6plyvel_7_plyvel_PrefixedDB},
  {Py_tp_iter, (void *)__pyx_pw_6plyvel_7_plyvel_10PrefixedDB_13__iter__},
  {Py_tp_methods, (void *)__pyx_methods_6plyvel_7_plyvel_PrefixedDB},
  {Py_tp_getset, (void *)__pyx_getsets_6plyvel_7_plyvel_PrefixedDB},
  {Py_tp_init, (void *)__pyx_pw_6plyvel_7_plyvel_10PrefixedDB_1__init__},
  {Py_tp_new, (void *)__pyx_tp_new_6plyvel_7_plyvel_PrefixedDB},
  {0, 0},
};
static PyType_Spec __pyx_type_6plyvel_7_plyvel_PrefixedDB_spec = {
  "plyvel._plyvel.PrefixedDB",
  sizeof(struct __pyx_obj_6plyvel_7_plyvel_PrefixedDB),
  0,
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC,
  __pyx_type_6plyvel_7_plyvel_PrefixedDB_slots,
};
#else

static PyTypeObject __pyx_type_6plyvel_7_plyvel_PrefixedDB = {
  PyVarObject_HEAD_INIT(0, 0)
  "plyvel._plyvel.""PrefixedDB", /*tp_name*/
  sizeof(struct __pyx_obj_6plyvel_7_plyvel_PrefixedDB), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_6plyvel_7_plyvel_PrefixedDB, /*tp_dealloc*/
  0, /*tp_vectorcall_offset*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  0, /*tp_as_async*/
  __pyx_pw_6plyvel_7_plyvel_10PrefixedDB_3__repr__, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  PyDoc_STR("PrefixedDB(DB db, *, bytes prefix)"), /*tp_doc*/
  __pyx_tp_traverse_6plyvel_7_plyvel_PrefixedDB, /*tp_traverse*/
  __pyx_tp_clear_6plyvel_7_plyvel_PrefixedDB, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  __pyx_pw_6plyvel_7_plyvel_10PrefixedDB_13__iter__, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_6plyvel_7_plyvel_PrefixedDB, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_6plyvel_7_plyvel_PrefixedDB, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  #if !CYTHON_USE_TYPE_SPECS
  0, /*tp_dictoffset*/
  #endif
  __pyx_pw_6plyvel_7_plyvel_10PrefixedDB_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_6plyvel_7_plyvel_PrefixedDB, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if CYTHON_USE_TP_FINALIZE
  0, /*tp_finalize*/
  #else
  NULL, /*tp_finalize*/
  #endif
  #if !CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800
  0, /*tp_vectorcall*/
  #endif
  #if __PYX_NEED_TP_PRINT_SLOT == 1
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030C0000
  0, /*tp_watched*/
  #endif
  #if PY_VERSION_HEX >= 0x030d00A4
  0, /*tp_versions_used*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
  0, /*tp_pypy_flags*/
  #endif
};
#endif

static PyObject *__pyx_tp_new_6plyvel_7_plyvel_WriteBatch(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_6plyvel_7_plyvel_WriteBatch *p;
  PyObject *o;
  o = __Pyx_AllocateExtensionType(t, 1);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_6plyvel_7_plyvel_WriteBatch *)o);
  __Pyx_default_placement_construct(&(p->write_options));
  p->db = ((struct __pyx_obj_6plyvel_7_plyvel_DB *)Py_None); Py_INCREF(Py_None);
  p->prefix = ((PyObject*)Py_None); Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_6plyvel_7_plyvel_WriteBatch(PyObject *o) {
  struct __pyx_obj_6plyvel_7_plyvel_WriteBatch *p = (struct __pyx_obj_6plyvel_7_plyvel_WriteBatch *)o;
  PyObject_GC_UnTrack(o);
  {
    PyObject *etype, *eval, *etb;
    PyErr_Fetch(&etype, &eval, &etb);
    __Pyx_SET_REFCNT(o, Py_REFCNT(o) + 1);
    __pyx_pw_6plyvel_7_plyvel_10WriteBatch_3__dealloc__(o);
    __Pyx_SET_REFCNT(o, Py_REFCNT(o) - 1);
    PyErr_Restore(etype, eval, etb);
  }
  __Pyx_call_destructor(p->write_options);
  Py_CLEAR(p->db);
  Py_CLEAR(p->prefix);
  PyTypeObject *tp = Py_TYPE(o);
  #if CYTHON_USE_TYPE_SLOTS
  (*tp->tp_free)(o);
  #else
  {
    freefunc tp_free = (freefunc)PyType_GetSlot(tp, Py_tp_free);
    if (tp_free) tp_free(o);
  }
  #endif
  #if CYTHON_USE_TYPE_SPECS
  Py_DECREF(tp);
  #endif
}

static int __pyx_tp_traverse_6plyvel_7_plyvel_WriteBatch(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_6plyvel_7_plyvel_WriteBatch *p = (struct __pyx_obj_6plyvel_7_plyvel_WriteBatch *)o;
  {
    e = __Pyx_call_type_traverse(o, 1, v, a);
    if (e) return e;
  }
  if (p->db) {
    e = (*v)(((PyObject *)p->db), a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_6plyvel_7_plyvel_WriteBatch(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_6plyvel_7_plyvel_WriteBatch *p = (struct __pyx_obj_6plyvel_7_plyvel_WriteBatch *)o;
  tmp = ((PyObject*)p->db);
  p->db = ((struct __pyx_obj_6plyvel_7_plyvel_DB *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyMethodDef __pyx_methods_6plyvel_7_plyvel_WriteBatch[] = {
  {"put", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_10WriteBatch_5put, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_10WriteBatch_4put},
  {"delete", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_10WriteBatch_7delete, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_10WriteBatch_6delete},
  {"clear", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_10WriteBatch_9clear, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_10WriteBatch_8clear},
  {"write", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_10WriteBatch_11write, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_10WriteBatch_10write},
  {"approximate_size", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_10WriteBatch_13approximate_size, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_10WriteBatch_12approximate_size},
  {"append", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_10WriteBatch_15append, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_10WriteBatch_14append},
  {"__enter__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_10WriteBatch_17__enter__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_10WriteBatch_16__enter__},
  {"__exit__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_10WriteBatch_19__exit__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_10WriteBatch_18__exit__},
  {"__reduce_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_10WriteBatch_21__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_10WriteBatch_20__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_10WriteBatch_23__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_10WriteBatch_22__setstate_cython__},
  {0, 0, 0, 0}
};
#if CYTHON_USE_TYPE_SPECS
static PyType_Slot __pyx_type_6plyvel_7_plyvel_WriteBatch_slots[] = {
  {Py_tp_dealloc, (void *)__pyx_tp_dealloc_6plyvel_7_plyvel_WriteBatch},
  {Py_tp_doc, (void *)PyDoc_STR("WriteBatch(DB db, bytes prefix, bool transaction, sync)")},
  {Py_tp_traverse, (void *)__pyx_tp_traverse_6plyvel_7_plyvel_WriteBatch},
  {Py_tp_clear, (void *)__pyx_tp_clear_6plyvel_7_plyvel_WriteBatch},
  {Py_tp_methods, (void *)__pyx_methods_6plyvel_7_plyvel_WriteBatch},
  {Py_tp_init, (void *)__pyx_pw_6plyvel_7_plyvel_10WriteBatch_1__init__},
  {Py_tp_new, (void *)__pyx_tp_new_6plyvel_7_plyvel_WriteBatch},
  {0, 0},
};
static PyType_Spec __pyx_type_6plyvel_7_plyvel_WriteBatch_spec = {
  "plyvel._plyvel.WriteBatch",
  sizeof(struct __pyx_obj_6plyvel_7_plyvel_WriteBatch),
  0,
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC,
  __pyx_type_6plyvel_7_plyvel_WriteBatch_slots,
};
#else

static PyTypeObject __pyx_type_6plyvel_7_plyvel_WriteBatch = {
  PyVarObject_HEAD_INIT(0, 0)
  "plyvel._plyvel.""WriteBatch", /*tp_name*/
  sizeof(struct __pyx_obj_6plyvel_7_plyvel_WriteBatch), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_6plyvel_7_plyvel_WriteBatch, /*tp_dealloc*/
  0, /*tp_vectorcall_offset*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  0, /*tp_as_async*/
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  PyDoc_STR("WriteBatch(DB db, bytes prefix, bool transaction, sync)"), /*tp_doc*/
  __pyx_tp_traverse_6plyvel_7_plyvel_WriteBatch, /*tp_traverse*/
  __pyx_tp_clear_6plyvel_7_plyvel_WriteBatch, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_6plyvel_7_plyvel_WriteBatch, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  #if !CYTHON_USE_TYPE_SPECS
  0, /*tp_dictoffset*/
  #endif
  __pyx_pw_6plyvel_7_plyvel_10WriteBatch_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_6plyvel_7_plyvel_WriteBatch, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if CYTHON_USE_TP_FINALIZE
  0, /*tp_finalize*/
  #else
  NULL, /*tp_finalize*/
  #endif
  #if !CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800
  0, /*tp_vectorcall*/
  #endif
  #if __PYX_NEED_TP_PRINT_SLOT == 1
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030C0000
  0, /*tp_watched*/
  #endif
  #if PY_VERSION_HEX >= 0x030d00A4
  0, /*tp_versions_used*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
  0, /*tp_pypy_flags*/
  #endif
};
#endif
static struct __pyx_vtabstruct_6plyvel_7_plyvel_BaseIterator __pyx_vtable_6plyvel_7_plyvel_BaseIterator;

static PyObject *__pyx_tp_new_6plyvel_7_plyvel_BaseIterator(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_6plyvel_7_plyvel_BaseIterator *p;
  PyObject *o;
  o = __Pyx_AllocateExtensionType(t, 0);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_6plyvel_7_plyvel_BaseIterator *)o);
  p->__pyx_vtab = __pyx_vtabptr_6plyvel_7_plyvel_BaseIterator;
  p->db = ((struct __pyx_obj_6plyvel_7_plyvel_DB *)Py_None); Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_6plyvel_7_plyvel_BaseIterator(PyObject *o) {
  struct __pyx_obj_6plyvel_7_plyvel_BaseIterator *p = (struct __pyx_obj_6plyvel_7_plyvel_BaseIterator *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(__Pyx_PyObject_GetSlot(o, tp_finalize, destructor)) && !__Pyx_PyObject_GC_IsFinalized(o)) {
    if (__Pyx_PyObject_GetSlot(o, tp_dealloc, destructor) == __pyx_tp_dealloc_6plyvel_7_plyvel_BaseIterator) {
      if (PyObject_CallFinalizerFromDealloc(o)) return;
    }
  }
  #endif
  PyObject_GC_UnTrack(o);
  if (p->__weakref__) PyObject_ClearWeakRefs(o);
  {
    PyObject *etype, *eval, *etb;
    PyErr_Fetch(&etype, &eval, &etb);
    __Pyx_SET_REFCNT(o, Py_REFCNT(o) + 1);
    __pyx_pw_6plyvel_7_plyvel_12BaseIterator_5__dealloc__(o);
    __Pyx_SET_REFCNT(o, Py_REFCNT(o) - 1);
    PyErr_Restore(etype, eval, etb);
  }
  Py_CLEAR(p->db);
  PyTypeObject *tp = Py_TYPE(o);
  #if CYTHON_USE_TYPE_SLOTS
  (*tp->tp_free)(o);
  #else
  {
    freefunc tp_free = (freefunc)PyType_GetSlot(tp, Py_tp_free);
    if (tp_free) tp_free(o);
  }
  #endif
  #if CYTHON_USE_TYPE_SPECS
  Py_DECREF(tp);
  #endif
}

static int __pyx_tp_traverse_6plyvel_7_plyvel_BaseIterator(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_6plyvel_7_plyvel_BaseIterator *p = (struct __pyx_obj_6plyvel_7_plyvel_BaseIterator *)o;
  {
    e = __Pyx_call_type_traverse(o, 1, v, a);
    if (e) return e;
  }
  if (p->db) {
    e = (*v)(((PyObject *)p->db), a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_6plyvel_7_plyvel_BaseIterator(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_6plyvel_7_plyvel_BaseIterator *p = (struct __pyx_obj_6plyvel_7_plyvel_BaseIterator *)o;
  tmp = ((PyObject*)p->db);
  p->db = ((struct __pyx_obj_6plyvel_7_plyvel_DB *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyMethodDef __pyx_methods_6plyvel_7_plyvel_BaseIterator[] = {
  {"__enter__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_12BaseIterator_7__enter__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_12BaseIterator_6__enter__},
  {"__exit__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_12BaseIterator_9__exit__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_12BaseIterator_8__exit__},
  {"__reduce_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_12BaseIterator_11__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_12BaseIterator_10__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_12BaseIterator_13__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_12BaseIterator_12__setstate_cython__},
  {0, 0, 0, 0}
};
#if CYTHON_USE_TYPE_SPECS
static PyMemberDef __pyx_type_6plyvel_7_plyvel_BaseIterator_members[] = {
  #if !CYTHON_USE_TYPE_SLOTS
  {"__weaklistoffset__", T_PYSSIZET, offsetof(struct __pyx_obj_6plyvel_7_plyvel_BaseIterator, __weakref__), READONLY, 0},
  #endif
  {0, 0, 0, 0, 0}
};
#if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x03090000
#if defined(__GNUC__) || defined(__clang__)
#warning "plyvel/_plyvel.pyx:678:16: : __weakref__ is unsupported in the Limited API when running on Python <3.9."
#elif defined(_MSC_VER)
#pragma message("plyvel/_plyvel.pyx:678:16: : __weakref__ is unsupported in the Limited API when running on Python <3.9.")
#endif
#endif
static PyType_Slot __pyx_type_6plyvel_7_plyvel_BaseIterator_slots[] = {
  {Py_tp_dealloc, (void *)__pyx_tp_dealloc_6plyvel_7_plyvel_BaseIterator},
  {Py_tp_doc, (void *)PyDoc_STR("BaseIterator(DB db, bool verify_checksums, bool fill_cache, Snapshot snapshot)")},
  {Py_tp_traverse, (void *)__pyx_tp_traverse_6plyvel_7_plyvel_BaseIterator},
  {Py_tp_clear, (void *)__pyx_tp_clear_6plyvel_7_plyvel_BaseIterator},
  {Py_tp_methods, (void *)__pyx_methods_6plyvel_7_plyvel_BaseIterator},
  {Py_tp_init, (void *)__pyx_pw_6plyvel_7_plyvel_12BaseIterator_1__init__},
  {Py_tp_new, (void *)__pyx_tp_new_6plyvel_7_plyvel_BaseIterator},
  {Py_tp_members, (void*)__pyx_type_6plyvel_7_plyvel_BaseIterator_members},
  {0, 0},
};
static PyType_Spec __pyx_type_6plyvel_7_plyvel_BaseIterator_spec = {
  "plyvel._plyvel.BaseIterator",
  sizeof(struct __pyx_obj_6plyvel_7_plyvel_BaseIterator),
  0,
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC,
  __pyx_type_6plyvel_7_plyvel_BaseIterator_slots,
};
#else

static PyTypeObject __pyx_type_6plyvel_7_plyvel_BaseIterator = {
  PyVarObject_HEAD_INIT(0, 0)
  "plyvel._plyvel.""BaseIterator", /*tp_name*/
  sizeof(struct __pyx_obj_6plyvel_7_plyvel_BaseIterator), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_6plyvel_7_plyvel_BaseIterator, /*tp_dealloc*/
  0, /*tp_vectorcall_offset*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  0, /*tp_as_async*/
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  PyDoc_STR("BaseIterator(DB db, bool verify_checksums, bool fill_cache, Snapshot snapshot)"), /*tp_doc*/
  __pyx_tp_traverse_6plyvel_7_plyvel_BaseIterator, /*tp_traverse*/
  __pyx_tp_clear_6plyvel_7_plyvel_BaseIterator, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_6plyvel_7_plyvel_BaseIterator, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  #if !CYTHON_USE_TYPE_SPECS
  0, /*tp_dictoffset*/
  #endif
  __pyx_pw_6plyvel_7_plyvel_12BaseIterator_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_6plyvel_7_plyvel_BaseIterator, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if CYTHON_USE_TP_FINALIZE
  0, /*tp_finalize*/
  #else
  NULL, /*tp_finalize*/
  #endif
  #if !CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800
  0, /*tp_vectorcall*/
  #endif
  #if __PYX_NEED_TP_PRINT_SLOT == 1
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030C0000
  0, /*tp_watched*/
  #endif
  #if PY_VERSION_HEX >= 0x030d00A4
  0, /*tp_versions_used*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
  0, /*tp_pypy_flags*/
  #endif
};
#endif
static struct __pyx_vtabstruct_6plyvel_7_plyvel_Iterator __pyx_vtable_6plyvel_7_plyvel_Iterator;

static PyObject *__pyx_tp_new_6plyvel_7_plyvel_Iterator(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_6plyvel_7_plyvel_Iterator *p;
  PyObject *o = __pyx_tp_new_6plyvel_7_plyvel_BaseIterator(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_6plyvel_7_plyvel_Iterator *)o);
  p->__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_6plyvel_7_plyvel_BaseIterator*)__pyx_vtabptr_6plyvel_7_plyvel_Iterator;
  __Pyx_default_placement_construct(&(p->start_slice));
  __Pyx_default_placement_construct(&(p->stop_slice));
  p->start = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->stop = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->db_prefix = ((PyObject*)Py_None); Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_6plyvel_7_plyvel_Iterator(PyObject *o) {
  struct __pyx_obj_6plyvel_7_plyvel_Iterator *p = (struct __pyx_obj_6plyvel_7_plyvel_Iterator *)o;
  PyObject_GC_UnTrack(o);
  __Pyx_call_destructor(p->start_slice);
  __Pyx_call_destructor(p->stop_slice);
  Py_CLEAR(p->start);
  Py_CLEAR(p->stop);
  Py_CLEAR(p->db_prefix);
  PyObject_GC_Track(o);
  __pyx_tp_dealloc_6plyvel_7_plyvel_BaseIterator(o);
}

static PyObject *__pyx_specialmethod___pyx_pw_6plyvel_7_plyvel_8Iterator_5__next__(PyObject *self, CYTHON_UNUSED PyObject *arg) {
  PyObject *res = __pyx_pw_6plyvel_7_plyvel_8Iterator_5__next__(self);
  if (!res && !PyErr_Occurred()) { PyErr_SetNone(PyExc_StopIteration); }
  return res;
}

static PyMethodDef __pyx_methods_6plyvel_7_plyvel_Iterator[] = {
  {"__next__", (PyCFunction)__pyx_specialmethod___pyx_pw_6plyvel_7_plyvel_8Iterator_5__next__, METH_NOARGS|METH_COEXIST, __pyx_doc_6plyvel_7_plyvel_8Iterator_4__next__},
  {"prev", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_8Iterator_7prev, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_8Iterator_6prev},
  {"seek_to_start", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_8Iterator_9seek_to_start, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_8Iterator_8seek_to_start},
  {"seek_to_stop", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_8Iterator_11seek_to_stop, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_8Iterator_10seek_to_stop},
  {"seek", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_8Iterator_13seek, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_8Iterator_12seek},
  {"__reduce_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_8Iterator_15__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_8Iterator_14__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_8Iterator_17__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_8Iterator_16__setstate_cython__},
  {0, 0, 0, 0}
};
#if CYTHON_USE_TYPE_SPECS
static PyType_Slot __pyx_type_6plyvel_7_plyvel_Iterator_slots[] = {
  {Py_tp_dealloc, (void *)__pyx_tp_dealloc_6plyvel_7_plyvel_Iterator},
  {Py_tp_doc, (void *)PyDoc_STR("Iterator(DB db, bytes db_prefix, bool reverse, bytes start, bytes stop, bool include_start, bool include_stop, bytes prefix, bool include_key, bool include_value, bool verify_checksums, bool fill_cache, Snapshot snapshot)")},
  {Py_tp_traverse, (void *)__pyx_tp_traverse_6plyvel_7_plyvel_BaseIterator},
  {Py_tp_clear, (void *)__pyx_tp_clear_6plyvel_7_plyvel_BaseIterator},
  {Py_tp_iter, (void *)__pyx_pw_6plyvel_7_plyvel_8Iterator_3__iter__},
  {Py_tp_iternext, (void *)__pyx_pw_6plyvel_7_plyvel_8Iterator_5__next__},
  {Py_tp_methods, (void *)__pyx_methods_6plyvel_7_plyvel_Iterator},
  {Py_tp_init, (void *)__pyx_pw_6plyvel_7_plyvel_8Iterator_1__init__},
  {Py_tp_new, (void *)__pyx_tp_new_6plyvel_7_plyvel_Iterator},
  {0, 0},
};
static PyType_Spec __pyx_type_6plyvel_7_plyvel_Iterator_spec = {
  "plyvel._plyvel.Iterator",
  sizeof(struct __pyx_obj_6plyvel_7_plyvel_Iterator),
  0,
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC,
  __pyx_type_6plyvel_7_plyvel_Iterator_slots,
};
#else

static PyTypeObject __pyx_type_6plyvel_7_plyvel_Iterator = {
  PyVarObject_HEAD_INIT(0, 0)
  "plyvel._plyvel.""Iterator", /*tp_name*/
  sizeof(struct __pyx_obj_6plyvel_7_plyvel_Iterator), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_6plyvel_7_plyvel_Iterator, /*tp_dealloc*/
  0, /*tp_vectorcall_offset*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  0, /*tp_as_async*/
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  PyDoc_STR("Iterator(DB db, bytes db_prefix, bool reverse, bytes start, bytes stop, bool include_start, bool include_stop, bytes prefix, bool include_key, bool include_value, bool verify_checksums, bool fill_cache, Snapshot snapshot)"), /*tp_doc*/
  __pyx_tp_traverse_6plyvel_7_plyvel_BaseIterator, /*tp_traverse*/
  __pyx_tp_clear_6plyvel_7_plyvel_BaseIterator, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  __pyx_pw_6plyvel_7_plyvel_8Iterator_3__iter__, /*tp_iter*/
  __pyx_pw_6plyvel_7_plyvel_8Iterator_5__next__, /*tp_iternext*/
  __pyx_methods_6plyvel_7_plyvel_Iterator, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  #if !CYTHON_USE_TYPE_SPECS
  0, /*tp_dictoffset*/
  #endif
  __pyx_pw_6plyvel_7_plyvel_8Iterator_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_6plyvel_7_plyvel_Iterator, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if CYTHON_USE_TP_FINALIZE
  0, /*tp_finalize*/
  #else
  NULL, /*tp_finalize*/
  #endif
  #if !CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800
  0, /*tp_vectorcall*/
  #endif
  #if __PYX_NEED_TP_PRINT_SLOT == 1
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030C0000
  0, /*tp_watched*/
  #endif
  #if PY_VERSION_HEX >= 0x030d00A4
  0, /*tp_versions_used*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
  0, /*tp_pypy_flags*/
  #endif
};
#endif
static struct __pyx_vtabstruct_6plyvel_7_plyvel_RawIterator __pyx_vtable_6plyvel_7_plyvel_RawIterator;

static PyObject *__pyx_tp_new_6plyvel_7_plyvel_RawIterator(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_6plyvel_7_plyvel_RawIterator *p;
  PyObject *o = __pyx_tp_new_6plyvel_7_plyvel_BaseIterator(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_6plyvel_7_plyvel_RawIterator *)o);
  p->__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_6plyvel_7_plyvel_BaseIterator*)__pyx_vtabptr_6plyvel_7_plyvel_RawIterator;
  return o;
}

static PyMethodDef __pyx_methods_6plyvel_7_plyvel_RawIterator[] = {
  {"valid", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_11RawIterator_1valid, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_11RawIterator_valid},
  {"seek_to_first", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_11RawIterator_3seek_to_first, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_11RawIterator_2seek_to_first},
  {"seek_to_last", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_11RawIterator_5seek_to_last, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_11RawIterator_4seek_to_last},
  {"seek", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_11RawIterator_7seek, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_11RawIterator_6seek},
  {"next", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_11RawIterator_9next, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_11RawIterator_8next},
  {"prev", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_11RawIterator_11prev, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_11RawIterator_10prev},
  {"item", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_11RawIterator_17item, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_11RawIterator_16item},
  {"__reduce_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_11RawIterator_19__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_11RawIterator_18__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_11RawIterator_21__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_11RawIterator_20__setstate_cython__},
  {0, 0, 0, 0}
};
#if CYTHON_USE_TYPE_SPECS
static PyType_Slot __pyx_type_6plyvel_7_plyvel_RawIterator_slots[] = {
  {Py_tp_dealloc, (void *)__pyx_tp_dealloc_6plyvel_7_plyvel_BaseIterator},
  {Py_tp_traverse, (void *)__pyx_tp_traverse_6plyvel_7_plyvel_BaseIterator},
  {Py_tp_clear, (void *)__pyx_tp_clear_6plyvel_7_plyvel_BaseIterator},
  {Py_tp_methods, (void *)__pyx_methods_6plyvel_7_plyvel_RawIterator},
  {Py_tp_new, (void *)__pyx_tp_new_6plyvel_7_plyvel_RawIterator},
  {0, 0},
};
static PyType_Spec __pyx_type_6plyvel_7_plyvel_RawIterator_spec = {
  "plyvel._plyvel.RawIterator",
  sizeof(struct __pyx_obj_6plyvel_7_plyvel_RawIterator),
  0,
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC,
  __pyx_type_6plyvel_7_plyvel_RawIterator_slots,
};
#else

static PyTypeObject __pyx_type_6plyvel_7_plyvel_RawIterator = {
  PyVarObject_HEAD_INIT(0, 0)
  "plyvel._plyvel.""RawIterator", /*tp_name*/
  sizeof(struct __pyx_obj_6plyvel_7_plyvel_RawIterator), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_6plyvel_7_plyvel_BaseIterator, /*tp_dealloc*/
  0, /*tp_vectorcall_offset*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  0, /*tp_as_async*/
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_6plyvel_7_plyvel_BaseIterator, /*tp_traverse*/
  __pyx_tp_clear_6plyvel_7_plyvel_BaseIterator, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_6plyvel_7_plyvel_RawIterator, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  #if !CYTHON_USE_TYPE_SPECS
  0, /*tp_dictoffset*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY || 0
  __pyx_pw_6plyvel_7_plyvel_12BaseIterator_1__init__, /*tp_init*/
  #else
  0, /*tp_init*/
  #endif
  0, /*tp_alloc*/
  __pyx_tp_new_6plyvel_7_plyvel_RawIterator, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if CYTHON_USE_TP_FINALIZE
  0, /*tp_finalize*/
  #else
  NULL, /*tp_finalize*/
  #endif
  #if !CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800
  0, /*tp_vectorcall*/
  #endif
  #if __PYX_NEED_TP_PRINT_SLOT == 1
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030C0000
  0, /*tp_watched*/
  #endif
  #if PY_VERSION_HEX >= 0x030d00A4
  0, /*tp_versions_used*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
  0, /*tp_pypy_flags*/
  #endif
};
#endif
static struct __pyx_vtabstruct_6plyvel_7_plyvel_Snapshot __pyx_vtable_6plyvel_7_plyvel_Snapshot;

static PyObject *__pyx_tp_new_6plyvel_7_plyvel_Snapshot(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_6plyvel_7_plyvel_Snapshot *p;
  PyObject *o;
  o = __Pyx_AllocateExtensionType(t, 1);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_6plyvel_7_plyvel_Snapshot *)o);
  p->__pyx_vtab = __pyx_vtabptr_6plyvel_7_plyvel_Snapshot;
  p->db = ((struct __pyx_obj_6plyvel_7_plyvel_DB *)Py_None); Py_INCREF(Py_None);
  p->prefix = ((PyObject*)Py_None); Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_6plyvel_7_plyvel_Snapshot(PyObject *o) {
  struct __pyx_obj_6plyvel_7_plyvel_Snapshot *p = (struct __pyx_obj_6plyvel_7_plyvel_Snapshot *)o;
  PyObject_GC_UnTrack(o);
  {
    PyObject *etype, *eval, *etb;
    PyErr_Fetch(&etype, &eval, &etb);
    __Pyx_SET_REFCNT(o, Py_REFCNT(o) + 1);
    __pyx_pw_6plyvel_7_plyvel_8Snapshot_3__dealloc__(o);
    __Pyx_SET_REFCNT(o, Py_REFCNT(o) - 1);
    PyErr_Restore(etype, eval, etb);
  }
  Py_CLEAR(p->db);
  Py_CLEAR(p->prefix);
  PyTypeObject *tp = Py_TYPE(o);
  #if CYTHON_USE_TYPE_SLOTS
  (*tp->tp_free)(o);
  #else
  {
    freefunc tp_free = (freefunc)PyType_GetSlot(tp, Py_tp_free);
    if (tp_free) tp_free(o);
  }
  #endif
  #if CYTHON_USE_TYPE_SPECS
  Py_DECREF(tp);
  #endif
}

static int __pyx_tp_traverse_6plyvel_7_plyvel_Snapshot(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_6plyvel_7_plyvel_Snapshot *p = (struct __pyx_obj_6plyvel_7_plyvel_Snapshot *)o;
  {
    e = __Pyx_call_type_traverse(o, 1, v, a);
    if (e) return e;
  }
  if (p->db) {
    e = (*v)(((PyObject *)p->db), a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_6plyvel_7_plyvel_Snapshot(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_6plyvel_7_plyvel_Snapshot *p = (struct __pyx_obj_6plyvel_7_plyvel_Snapshot *)o;
  tmp = ((PyObject*)p->db);
  p->db = ((struct __pyx_obj_6plyvel_7_plyvel_DB *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyMethodDef __pyx_methods_6plyvel_7_plyvel_Snapshot[] = {
  {"release", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_8Snapshot_7release, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_8Snapshot_6release},
  {"__enter__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_8Snapshot_9__enter__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_8Snapshot_8__enter__},
  {"__exit__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_8Snapshot_11__exit__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_8Snapshot_10__exit__},
  {"get", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_8Snapshot_13get, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_8Snapshot_12get},
  {"iterator", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_8Snapshot_17iterator, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_8Snapshot_16iterator},
  {"raw_iterator", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_8Snapshot_19raw_iterator, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_8Snapshot_18raw_iterator},
  {"__reduce_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_8Snapshot_21__reduce_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_8Snapshot_20__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)(void(*)(void))(__Pyx_PyCFunction_FastCallWithKeywords)__pyx_pw_6plyvel_7_plyvel_8Snapshot_23__setstate_cython__, __Pyx_METH_FASTCALL|METH_KEYWORDS, __pyx_doc_6plyvel_7_plyvel_8Snapshot_22__setstate_cython__},
  {0, 0, 0, 0}
};
#if CYTHON_USE_TYPE_SPECS
static PyType_Slot __pyx_type_6plyvel_7_plyvel_Snapshot_slots[] = {
  {Py_tp_dealloc, (void *)__pyx_tp_dealloc_6plyvel_7_plyvel_Snapshot},
  {Py_tp_doc, (void *)PyDoc_STR("Snapshot(DB db, *, bytes prefix=None)")},
  {Py_tp_traverse, (void *)__pyx_tp_traverse_6plyvel_7_plyvel_Snapshot},
  {Py_tp_clear, (void *)__pyx_tp_clear_6plyvel_7_plyvel_Snapshot},
  {Py_tp_iter, (void *)__pyx_pw_6plyvel_7_plyvel_8Snapshot_15__iter__},
  {Py_tp_methods, (void *)__pyx_methods_6plyvel_7_plyvel_Snapshot},
  {Py_tp_init, (void *)__pyx_pw_6plyvel_7_plyvel_8Snapshot_1__init__},
  {Py_tp_new, (void *)__pyx_tp_new_6plyvel_7_plyvel_Snapshot},
  {0, 0},
};
static PyType_Spec __pyx_type_6plyvel_7_plyvel_Snapshot_spec = {
  "plyvel._plyvel.Snapshot",
  sizeof(struct __pyx_obj_6plyvel_7_plyvel_Snapshot),
  0,
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC,
  __pyx_type_6plyvel_7_plyvel_Snapshot_slots,
};
#else

static PyTypeObject __pyx_type_6plyvel_7_plyvel_Snapshot = {
  PyVarObject_HEAD_INIT(0, 0)
  "plyvel._plyvel.""Snapshot", /*tp_name*/
  sizeof(struct __pyx_obj_6plyvel_7_plyvel_Snapshot), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_6plyvel_7_plyvel_Snapshot, /*tp_dealloc*/
  0, /*tp_vectorcall_offset*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  0, /*tp_as_async*/
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  PyDoc_STR("Snapshot(DB db, *, bytes prefix=None)"), /*tp_doc*/
  __pyx_tp_traverse_6plyvel_7_plyvel_Snapshot, /*tp_traverse*/
  __pyx_tp_clear_6plyvel_7_plyvel_Snapshot, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  __pyx_pw_6plyvel_7_plyvel_8Snapshot_15__iter__, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_6plyvel_7_plyvel_Snapshot, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  #if !CYTHON_USE_TYPE_SPECS
  0, /*tp_dictoffset*/
  #endif
  __pyx_pw_6plyvel_7_plyvel_8Snapshot_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_6plyvel_7_plyvel_Snapshot, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if CYTHON_USE_TP_FINALIZE
  0, /*tp_finalize*/
  #else
  NULL, /*tp_finalize*/
  #endif
  #if !CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800
  0, /*tp_vectorcall*/
  #endif
  #if __PYX_NEED_TP_PRINT_SLOT == 1
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030C0000
  0, /*tp_watched*/
  #endif
  #if PY_VERSION_HEX >= 0x030d00A4
  0, /*tp_versions_used*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
  0, /*tp_pypy_flags*/
  #endif
};
#endif

static PyObject *__pyx_tp_new_6plyvel_7_plyvel___pyx_scope_struct____init__(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  PyObject *o;
  #if CYTHON_USE_FREELISTS
  if (likely((int)(__pyx_mstate_global->__pyx_freecount_6plyvel_7_plyvel___pyx_scope_struct____init__ > 0) & __PYX_CHECK_FINAL_TYPE_FOR_FREELISTS(t, __pyx_mstate_global->__pyx_ptype_6plyvel_7_plyvel___pyx_scope_struct____init__, sizeof(struct __pyx_obj_6plyvel_7_plyvel___pyx_scope_struct____init__))))
  {
    o = (PyObject*)__pyx_mstate_global->__pyx_freelist_6plyvel_7_plyvel___pyx_scope_struct____init__[--__pyx_mstate_global->__pyx_freecount_6plyvel_7_plyvel___pyx_scope_struct____init__];
    #if CYTHON_USE_TYPE_SPECS
    Py_DECREF(Py_TYPE(o));
    #endif
    memset(o, 0, sizeof(struct __pyx_obj_6plyvel_7_plyvel___pyx_scope_struct____init__));
    #if CYTHON_COMPILING_IN_LIMITED_API
    (void) PyObject_Init(o, t);
    #else
    (void) PyObject_INIT(o, t);
    #endif
    PyObject_GC_Track(o);
  } else
  #endif
  {
    o = __Pyx_AllocateExtensionType(t, 1);
    if (unlikely(!o)) return 0;
  }
  return o;
}

static void __pyx_tp_dealloc_6plyvel_7_plyvel___pyx_scope_struct____init__(PyObject *o) {
  struct __pyx_obj_6plyvel_7_plyvel___pyx_scope_struct____init__ *p = (struct __pyx_obj_6plyvel_7_plyvel___pyx_scope_struct____init__ *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(__Pyx_PyObject_GetSlot(o, tp_finalize, destructor)) && !__Pyx_PyObject_GC_IsFinalized(o)) {
    if (__Pyx_PyObject_GetSlot(o, tp_dealloc, destructor) == __pyx_tp_dealloc_6plyvel_7_plyvel___pyx_scope_struct____init__) {
      if (PyObject_CallFinalizerFromDealloc(o)) return;
    }
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->__pyx_v_iterator_id);
  Py_CLEAR(p->__pyx_v_ref_dict);
  #if CYTHON_USE_FREELISTS
  if (likely((int)(__pyx_mstate_global->__pyx_freecount_6plyvel_7_plyvel___pyx_scope_struct____init__ < 8) & __PYX_CHECK_FINAL_TYPE_FOR_FREELISTS(Py_TYPE(o), __pyx_mstate_global->__pyx_ptype_6plyvel_7_plyvel___pyx_scope_struct____init__, sizeof(struct __pyx_obj_6plyvel_7_plyvel___pyx_scope_struct____init__))))
  {
    __pyx_mstate_global->__pyx_freelist_6plyvel_7_plyvel___pyx_scope_struct____init__[__pyx_mstate_global->__pyx_freecount_6plyvel_7_plyvel___pyx_scope_struct____init__++] = ((struct __pyx_obj_6plyvel_7_plyvel___pyx_scope_struct____init__ *)o);
  } else
  #endif
  {
    PyTypeObject *tp = Py_TYPE(o);
    #if CYTHON_USE_TYPE_SLOTS
    (*tp->tp_free)(o);
    #else
    {
      freefunc tp_free = (freefunc)PyType_GetSlot(tp, Py_tp_free);
      if (tp_free) tp_free(o);
    }
    #endif
    #if CYTHON_USE_TYPE_SPECS
    Py_DECREF(tp);
    #endif
  }
}

static int __pyx_tp_traverse_6plyvel_7_plyvel___pyx_scope_struct____init__(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_6plyvel_7_plyvel___pyx_scope_struct____init__ *p = (struct __pyx_obj_6plyvel_7_plyvel___pyx_scope_struct____init__ *)o;
  {
    e = __Pyx_call_type_traverse(o, 1, v, a);
    if (e) return e;
  }
  if (p->__pyx_v_iterator_id) {
    e = (*v)(p->__pyx_v_iterator_id, a); if (e) return e;
  }
  if (p->__pyx_v_ref_dict) {
    e = (*v)(p->__pyx_v_ref_dict, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_6plyvel_7_plyvel___pyx_scope_struct____init__(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_6plyvel_7_plyvel___pyx_scope_struct____init__ *p = (struct __pyx_obj_6plyvel_7_plyvel___pyx_scope_struct____init__ *)o;
  tmp = ((PyObject*)p->__pyx_v_iterator_id);
  p->__pyx_v_iterator_id = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->__pyx_v_ref_dict);
  p->__pyx_v_ref_dict = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}
#if CYTHON_USE_TYPE_SPECS
static PyType_Slot __pyx_type_6plyvel_7_plyvel___pyx_scope_struct____init___slots[] = {
  {Py_tp_dealloc, (void *)__pyx_tp_dealloc_6plyvel_7_plyvel___pyx_scope_struct____init__},
  {Py_tp_traverse, (void *)__pyx_tp_traverse_6plyvel_7_plyvel___pyx_scope_struct____init__},
  {Py_tp_clear, (void *)__pyx_tp_clear_6plyvel_7_plyvel___pyx_scope_struct____init__},
  {Py_tp_new, (void *)__pyx_tp_new_6plyvel_7_plyvel___pyx_scope_struct____init__},
  {0, 0},
};
static PyType_Spec __pyx_type_6plyvel_7_plyvel___pyx_scope_struct____init___spec = {
  "plyvel._plyvel.__pyx_scope_struct____init__",
  sizeof(struct __pyx_obj_6plyvel_7_plyvel___pyx_scope_struct____init__),
  0,
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC,
  __pyx_type_6plyvel_7_plyvel___pyx_scope_struct____init___slots,
};
#else

static PyTypeObject __pyx_type_6plyvel_7_plyvel___pyx_scope_struct____init__ = {
  PyVarObject_HEAD_INIT(0, 0)
  "plyvel._plyvel.""__pyx_scope_struct____init__", /*tp_name*/
  sizeof(struct __pyx_obj_6plyvel_7_plyvel___pyx_scope_struct____init__), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_6plyvel_7_plyvel___pyx_scope_struct____init__, /*tp_dealloc*/
  0, /*tp_vectorcall_offset*/
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  0, /*tp_as_async*/
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_6plyvel_7_plyvel___pyx_scope_struct____init__, /*tp_traverse*/
  __pyx_tp_clear_6plyvel_7_plyvel___pyx_scope_struct____init__, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  0, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  #if !CYTHON_USE_TYPE_SPECS
  0, /*tp_dictoffset*/
  #endif
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_6plyvel_7_plyvel___pyx_scope_struct____init__, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if CYTHON_USE_TP_FINALIZE
  0, /*tp_finalize*/
  #else
  NULL, /*tp_finalize*/
  #endif
  #if !CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM >= 0x07030800
  0, /*tp_vectorcall*/
  #endif
  #if __PYX_NEED_TP_PRINT_SLOT == 1
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030C0000
  0, /*tp_watched*/
  #endif
  #if PY_VERSION_HEX >= 0x030d00A4
  0, /*tp_versions_used*/
  #endif
  #if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX >= 0x03090000 && PY_VERSION_HEX < 0x030a0000
  0, /*tp_pypy_flags*/
  #endif
};
#endif

static PyMethodDef __pyx_methods[] = {
  {0, 0, 0, 0}
};
/* #### Code section: initfunc_declarations ### */
static CYTHON_SMALL_CODE int __Pyx_InitCachedBuiltins(__pyx_mstatetype *__pyx_mstate); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_InitCachedConstants(__pyx_mstatetype *__pyx_mstate); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_InitGlobals(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_InitConstants(__pyx_mstatetype *__pyx_mstate); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_global_init_code(__pyx_mstatetype *__pyx_mstate); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_variable_export_code(__pyx_mstatetype *__pyx_mstate); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_function_export_code(__pyx_mstatetype *__pyx_mstate); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_type_init_code(__pyx_mstatetype *__pyx_mstate); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_type_import_code(__pyx_mstatetype *__pyx_mstate); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_variable_import_code(__pyx_mstatetype *__pyx_mstate); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_function_import_code(__pyx_mstatetype *__pyx_mstate); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_CreateCodeObjects(__pyx_mstatetype *__pyx_mstate); /*proto*/
/* #### Code section: init_module ### */

static int __Pyx_modinit_global_init_code(__pyx_mstatetype *__pyx_mstate) {
  __Pyx_RefNannyDeclarations
  CYTHON_UNUSED_VAR(__pyx_mstate);
  __Pyx_RefNannySetupContext("__Pyx_modinit_global_init_code", 0);
  /*--- Global init code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_variable_export_code(__pyx_mstatetype *__pyx_mstate) {
  __Pyx_RefNannyDeclarations
  CYTHON_UNUSED_VAR(__pyx_mstate);
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_export_code", 0);
  /*--- Variable export code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_function_export_code(__pyx_mstatetype *__pyx_mstate) {
  __Pyx_RefNannyDeclarations
  CYTHON_UNUSED_VAR(__pyx_mstate);
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_export_code", 0);
  /*--- Function export code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_type_init_code(__pyx_mstatetype *__pyx_mstate) {
  __Pyx_RefNannyDeclarations
  CYTHON_UNUSED_VAR(__pyx_mstate);
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_init_code", 0);
  /*--- Type init code ---*/
  __pyx_vtabptr_6plyvel_7_plyvel_DB = &__pyx_vtable_6plyvel_7_plyvel_DB;
  __pyx_vtable_6plyvel_7_plyvel_DB.close = (PyObject *(*)(struct __pyx_obj_6plyvel_7_plyvel_DB *, int __pyx_skip_dispatch))__pyx_f_6plyvel_7_plyvel_2DB_close;
  #if CYTHON_USE_TYPE_SPECS
  __pyx_mstate->__pyx_ptype_6plyvel_7_plyvel_DB = (PyTypeObject *) __Pyx_PyType_FromModuleAndSpec(__pyx_m, &__pyx_type_6plyvel_7_plyvel_DB_spec, NULL); if (unlikely(!__pyx_mstate->__pyx_ptype_6plyvel_7_plyvel_DB)) __PYX_ERR(0, 220, __pyx_L1_error)
  if (__Pyx_fix_up_extension_type_from_spec(&__pyx_type_6plyvel_7_plyvel_DB_spec, __pyx_mstate->__pyx_ptype_6plyvel_7_plyvel_DB) < (0)) __PYX_ERR(0, 220, __pyx_L1_error)
  #else
  __pyx_mstate->__pyx_ptype_6plyvel_7_plyvel_DB = &__pyx_type_6plyvel_7_plyvel_DB;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  #endif
  #if !CYTHON_USE_TYPE_SPECS
  if (__Pyx_PyType_Ready(__pyx_mstate->__pyx_ptype_6plyvel_7_plyvel_DB) < (0)) __PYX_ERR(0, 220, __pyx_L1_error)
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount((PyObject*)__pyx_mstate->__pyx_ptype_6plyvel_7_plyvel_DB);
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_mstate->__pyx_ptype_6plyvel_7_plyvel_DB->tp_dictoffset && __pyx_mstate->__pyx_ptype_6plyvel_7_plyvel_DB->tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_mstate->__pyx_ptype_6plyvel_7_plyvel_DB->tp_getattro = PyObject_GenericGetAttr;
  }
  #endif
  if (__Pyx_SetVtable(__pyx_mstate->__pyx_ptype_6plyvel_7_plyvel_DB, __pyx_vtabptr_6plyvel_7_plyvel_DB) < (0)) __PYX_ERR(0, 220, __pyx_L1_error)
  if (__Pyx_MergeVtables(__pyx_mstate->__pyx_ptype_6plyvel_7_plyvel_DB) < (0)) __PYX_ERR(0, 220, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_mstate_global->__pyx_n_u_DB, (PyObject *) __pyx_mstate->__pyx_ptype_6plyvel_7_plyvel_DB) < (0)) __PYX_ERR(0, 220, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject *) __pyx_mstate->__pyx_ptype_6plyvel_7_plyvel_DB) < (0)) __PYX_ERR(0, 220, __pyx_L1_error)
  #if CYTHON_USE_TYPE_SPECS
  __pyx_mstate->__pyx_ptype_6plyvel_7_plyvel_PrefixedDB = (PyTypeObject *) __Pyx_PyType_FromModuleAndSpec(__pyx_m, &__pyx_type_6plyvel_7_plyvel_PrefixedDB_spec, NULL); if (unlikely(!__pyx_mstate->__pyx_ptype_6plyvel_7_plyvel_PrefixedDB)) __PYX_ERR(0, 459, __pyx_L1_error)
  if (__Pyx_fix_up_extension_type_from_spec(&__pyx_type_6plyvel_7_plyvel_PrefixedDB_spec, __pyx_mstate->__pyx_ptype_6plyvel_7_plyvel_PrefixedDB) < (0)) __PYX_ERR(0, 459, __pyx_L1_error)
  #else
  __pyx_mstate->__pyx_ptype_6plyvel_7_plyvel_PrefixedDB = &__pyx_type_6plyvel_7_plyvel_PrefixedDB;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  #endif
  #if !CYTHON_USE_TYPE_SPECS
  if (__Pyx_PyType_Ready(__pyx_mstate->__pyx_ptype_6plyvel_7_plyvel_PrefixedDB) < (0)) __PYX_ERR(0, 459, __pyx_L1_error)
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount((PyObject*)__pyx_mstate->__pyx_ptype_6plyvel_7_plyvel_PrefixedDB);
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_mstate->__pyx_ptype_6plyvel_7_plyvel_PrefixedDB->tp_dictoffset && __pyx_mstate->__pyx_ptype_6plyvel_7_plyvel_PrefixedDB->tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_mstate->__pyx_ptype_6plyvel_7_plyvel_PrefixedDB->tp_getattro = PyObject_GenericGetAttr;
  }
  #endif
  if (PyObject_SetAttr(__pyx_m, __pyx_mstate_global->__pyx_n_u_PrefixedDB, (PyObject *) __pyx_mstate->__pyx_ptype_6plyvel_7_plyvel_PrefixedDB) < (0)) __PYX_ERR(0, 459, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject *) __pyx_mstate->__pyx_ptype_6plyvel_7_plyvel_PrefixedDB) < (0)) __PYX_ERR(0, 459, __pyx_L1_error)
  #if CYTHON_USE_TYPE_SPECS
  __pyx_mstate->__pyx_ptype_6plyvel_7_plyvel_WriteBatch = (PyTypeObject *) __Pyx_PyType_FromModuleAndSpec(__pyx_m, &__pyx_type_6plyvel_7_plyvel_WriteBatch_spec, NULL); if (unlikely(!__pyx_mstate->__pyx_ptype_6plyvel_7_plyvel_WriteBatch)) __PYX_ERR(0, 558, __pyx_L1_error)
  if (__Pyx_fix_up_extension_type_from_spec(&__pyx_type_6plyvel_7_plyvel_WriteBatch_spec, __pyx_mstate->__pyx_ptype_6plyvel_7_plyvel_WriteBatch) < (0)) __PYX_ERR(0, 558, __pyx_L1_error)
  #else
  __pyx_mstate->__pyx_ptype_6plyvel_7_plyvel_WriteBatch = &__pyx_type_6plyvel_7_plyvel_WriteBatch;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  #endif
  #if !CYTHON_USE_TYPE_SPECS
  if (__Pyx_PyType_Ready(__pyx_mstate->__pyx_ptype_6plyvel_7_plyvel_WriteBatch) < (0)) __PYX_ERR(0, 558, __pyx_L1_error)
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount((PyObject*)__pyx_mstate->__pyx_ptype_6plyvel_7_plyvel_WriteBatch);
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_mstate->__pyx_ptype_6plyvel_7_plyvel_WriteBatch->tp_dictoffset && __pyx_mstate->__pyx_ptype_6plyvel_7_plyvel_WriteBatch->tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_mstate->__pyx_ptype_6plyvel_7_plyvel_WriteBatch->tp_getattro = PyObject_GenericGetAttr;
  }
  #endif
  if (PyObject_SetAttr(__pyx_m, __pyx_mstate_global->__pyx_n_u_WriteBatch, (PyObject *) __pyx_mstate->__pyx_ptype_6plyvel_7_plyvel_WriteBatch) < (0)) __PYX_ERR(0, 558, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject *) __pyx_mstate->__pyx_ptype_6plyvel_7_plyvel_WriteBatch) < (0)) __PYX_ERR(0, 558, __pyx_L1_error)
  __pyx_vtabptr_6plyvel_7_plyvel_BaseIterator = &__pyx_vtable_6plyvel_7_plyvel_BaseIterator;
  __pyx_vtable_6plyvel_7_plyvel_BaseIterator.close = (PyObject *(*)(struct __pyx_obj_6plyvel_7_plyvel_BaseIterator *, int __pyx_skip_dispatch))__pyx_f_6plyvel_7_plyvel_12BaseIterator_close;
  #if CYTHON_USE_TYPE_SPECS
  __pyx_mstate->__pyx_ptype_6plyvel_7_plyvel_BaseIterator = (PyTypeObject *) __Pyx_PyType_FromModuleAndSpec(__pyx_m, &__pyx_type_6plyvel_7_plyvel_BaseIterator_spec, NULL); if (unlikely(!__pyx_mstate->__pyx_ptype_6plyvel_7_plyvel_BaseIterator)) __PYX_ERR(0, 672, __pyx_L1_error)
  if (__Pyx_fix_up_extension_type_from_spec(&__pyx_type_6plyvel_7_plyvel_BaseIterator_spec, __pyx_mstate->__pyx_ptype_6plyvel_7_plyvel_BaseIterator) < (0)) __PYX_ERR(0, 672, __pyx_L1_error)
  #else
  __pyx_mstate->__pyx_ptype_6plyvel_7_plyvel_BaseIterator = &__pyx_type_6plyvel_7_plyvel_BaseIterator;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  #endif
  #if !CYTHON_USE_TYPE_SPECS
  if (__Pyx_PyType_Ready(__pyx_mstate->__pyx_ptype_6plyvel_7_plyvel_BaseIterator) < (0)) __PYX_ERR(0, 672, __pyx_L1_error)
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount((PyObject*)__pyx_mstate->__pyx_ptype_6plyvel_7_plyvel_BaseIterator);
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_mstate->__pyx_ptype_6plyvel_7_plyvel_BaseIterator->tp_dictoffset && __pyx_mstate->__pyx_ptype_6plyvel_7_plyvel_BaseIterator->tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_mstate->__pyx_ptype_6plyvel_7_plyvel_BaseIterator->tp_getattro = PyObject_GenericGetAttr;
  }
  #endif
  if (__Pyx_SetVtable(__pyx_mstate->__pyx_ptype_6plyvel_7_plyvel_BaseIterator, __pyx_vtabptr_6plyvel_7_plyvel_BaseIterator) < (0)) __PYX_ERR(0, 672, __pyx_L1_error)
  if (__Pyx_MergeVtables(__pyx_mstate->__pyx_ptype_6plyvel_7_plyvel_BaseIterator) < (0)) __PYX_ERR(0, 672, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_mstate_global->__pyx_n_u_BaseIterator, (PyObject *) __pyx_mstate->__pyx_ptype_6plyvel_7_plyvel_BaseIterator) < (0)) __PYX_ERR(0, 672, __pyx_L1_error)
  #if CYTHON_USE_TYPE_SLOTS
  if (__pyx_mstate->__pyx_ptype_6plyvel_7_plyvel_BaseIterator->tp_weaklistoffset == 0) __pyx_mstate->__pyx_ptype_6plyvel_7_plyvel_BaseIterator->tp_weaklistoffset = offsetof(struct __pyx_obj_6plyvel_7_plyvel_BaseIterator, __weakref__);
  #endif
  if (__Pyx_setup_reduce((PyObject *) __pyx_mstate->__pyx_ptype_6plyvel_7_plyvel_BaseIterator) < (0)) __PYX_ERR(0, 672, __pyx_L1_error)
  __pyx_vtabptr_6plyvel_7_plyvel_Iterator = &__pyx_vtable_6plyvel_7_plyvel_Iterator;
  __pyx_vtable_6plyvel_7_plyvel_Iterator.__pyx_base = *__pyx_vtabptr_6plyvel_7_plyvel_BaseIterator;
  __pyx_vtable_6plyvel_7_plyvel_Iterator.current = (PyObject *(*)(struct __pyx_obj_6plyvel_7_plyvel_Iterator *))__pyx_f_6plyvel_7_plyvel_8Iterator_current;
  __pyx_vtable_6plyvel_7_plyvel_Iterator.real_next = (PyObject *(*)(struct __pyx_obj_6plyvel_7_plyvel_Iterator *))__pyx_f_6plyvel_7_plyvel_8Iterator_real_next;
  __pyx_vtable_6plyvel_7_plyvel_Iterator.real_prev = (PyObject *(*)(struct __pyx_obj_6plyvel_7_plyvel_Iterator *))__pyx_f_6plyvel_7_plyvel_8Iterator_real_prev;
  #if CYTHON_USE_TYPE_SPECS
  __pyx_t_1 = PyTuple_Pack(1, (PyObject *)__pyx_mstate_global->__pyx_ptype_6plyvel_7_plyvel_BaseIterator); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 720, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_mstate->__pyx_ptype_6plyvel_7_plyvel_Iterator = (PyTypeObject *) __Pyx_PyType_FromModuleAndSpec(__pyx_m, &__pyx_type_6plyvel_7_plyvel_Iterator_spec, __pyx_t_1);
  __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (unlikely(!__pyx_mstate->__pyx_ptype_6plyvel_7_plyvel_Iterator)) __PYX_ERR(0, 720, __pyx_L1_error)
  if (__Pyx_fix_up_extension_type_from_spec(&__pyx_type_6plyvel_7_plyvel_Iterator_spec, __pyx_mstate->__pyx_ptype_6plyvel_7_plyvel_Iterator) < (0)) __PYX_ERR(0, 720, __pyx_L1_error)
  #else
  __pyx_mstate->__pyx_ptype_6plyvel_7_plyvel_Iterator = &__pyx_type_6plyvel_7_plyvel_Iterator;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  __pyx_mstate_global->__pyx_ptype_6plyvel_7_plyvel_Iterator->tp_base = __pyx_mstate_global->__pyx_ptype_6plyvel_7_plyvel_BaseIterator;
  #endif
  #if !CYTHON_USE_TYPE_SPECS
  if (__Pyx_PyType_Ready(__pyx_mstate->__pyx_ptype_6plyvel_7_plyvel_Iterator) < (0)) __PYX_ERR(0, 720, __pyx_L1_error)
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount((PyObject*)__pyx_mstate->__pyx_ptype_6plyvel_7_plyvel_Iterator);
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_mstate->__pyx_ptype_6plyvel_7_plyvel_Iterator->tp_dictoffset && __pyx_mstate->__pyx_ptype_6plyvel_7_plyvel_Iterator->tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_mstate->__pyx_ptype_6plyvel_7_plyvel_Iterator->tp_getattro = PyObject_GenericGetAttr;
  }
  #endif
  #if CYTHON_UPDATE_DESCRIPTOR_DOC
  {
    PyObject *wrapper = PyObject_GetAttrString((PyObject *)__pyx_mstate->__pyx_ptype_6plyvel_7_plyvel_Iterator, "__next__"); if (unlikely(!wrapper)) __PYX_ERR(0, 720, __pyx_L1_error)
    if (__Pyx_IS_TYPE(wrapper, &PyWrapperDescr_Type)) {
      __pyx_wrapperbase_6plyvel_7_plyvel_8Iterator_4__next__ = *((PyWrapperDescrObject *)wrapper)->d_base;
      __pyx_wrapperbase_6plyvel_7_plyvel_8Iterator_4__next__.doc = __pyx_doc_6plyvel_7_plyvel_8Iterator_4__next__;
      ((PyWrapperDescrObject *)wrapper)->d_base = &__pyx_wrapperbase_6plyvel_7_plyvel_8Iterator_4__next__;
    }
  }
  #endif
  if (__Pyx_SetVtable(__pyx_mstate->__pyx_ptype_6plyvel_7_plyvel_Iterator, __pyx_vtabptr_6plyvel_7_plyvel_Iterator) < (0)) __PYX_ERR(0, 720, __pyx_L1_error)
  if (__Pyx_MergeVtables(__pyx_mstate->__pyx_ptype_6plyvel_7_plyvel_Iterator) < (0)) __PYX_ERR(0, 720, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_mstate_global->__pyx_n_u_Iterator, (PyObject *) __pyx_mstate->__pyx_ptype_6plyvel_7_plyvel_Iterator) < (0)) __PYX_ERR(0, 720, __pyx_L1_error)
  #if CYTHON_USE_TYPE_SLOTS
  if (__pyx_mstate->__pyx_ptype_6plyvel_7_plyvel_Iterator->tp_weaklistoffset == 0) __pyx_mstate->__pyx_ptype_6plyvel_7_plyvel_Iterator->tp_weaklistoffset = offsetof(struct __pyx_obj_6plyvel_7_plyvel_Iterator, __pyx_base.__weakref__);
  #endif
  if (__Pyx_setup_reduce((PyObject *) __pyx_mstate->__pyx_ptype_6plyvel_7_plyvel_Iterator) < (0)) __PYX_ERR(0, 720, __pyx_L1_error)
  __pyx_vtabptr_6plyvel_7_plyvel_RawIterator = &__pyx_vtable_6plyvel_7_plyvel_RawIterator;
  __pyx_vtable_6plyvel_7_plyvel_RawIterator.__pyx_base = *__pyx_vtabptr_6plyvel_7_plyvel_BaseIterator;
  __pyx_vtable_6plyvel_7_plyvel_RawIterator.key = (PyObject *(*)(struct __pyx_obj_6plyvel_7_plyvel_RawIterator *, int __pyx_skip_dispatch))__pyx_f_6plyvel_7_plyvel_11RawIterator_key;
  __pyx_vtable_6plyvel_7_plyvel_RawIterator.value = (PyObject *(*)(struct __pyx_obj_6plyvel_7_plyvel_RawIterator *, int __pyx_skip_dispatch))__pyx_f_6plyvel_7_plyvel_11RawIterator_value;
  #if CYTHON_USE_TYPE_SPECS
  __pyx_t_1 = PyTuple_Pack(1, (PyObject *)__pyx_mstate_global->__pyx_ptype_6plyvel_7_plyvel_BaseIterator); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1027, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_mstate->__pyx_ptype_6plyvel_7_plyvel_RawIterator = (PyTypeObject *) __Pyx_PyType_FromModuleAndSpec(__pyx_m, &__pyx_type_6plyvel_7_plyvel_RawIterator_spec, __pyx_t_1);
  __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (unlikely(!__pyx_mstate->__pyx_ptype_6plyvel_7_plyvel_RawIterator)) __PYX_ERR(0, 1027, __pyx_L1_error)
  if (__Pyx_fix_up_extension_type_from_spec(&__pyx_type_6plyvel_7_plyvel_RawIterator_spec, __pyx_mstate->__pyx_ptype_6plyvel_7_plyvel_RawIterator) < (0)) __PYX_ERR(0, 1027, __pyx_L1_error)
  #else
  __pyx_mstate->__pyx_ptype_6plyvel_7_plyvel_RawIterator = &__pyx_type_6plyvel_7_plyvel_RawIterator;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  __pyx_mstate_global->__pyx_ptype_6plyvel_7_plyvel_RawIterator->tp_base = __pyx_mstate_global->__pyx_ptype_6plyvel_7_plyvel_BaseIterator;
  #endif
  #if !CYTHON_USE_TYPE_SPECS
  if (__Pyx_PyType_Ready(__pyx_mstate->__pyx_ptype_6plyvel_7_plyvel_RawIterator) < (0)) __PYX_ERR(0, 1027, __pyx_L1_error)
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount((PyObject*)__pyx_mstate->__pyx_ptype_6plyvel_7_plyvel_RawIterator);
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_mstate->__pyx_ptype_6plyvel_7_plyvel_RawIterator->tp_dictoffset && __pyx_mstate->__pyx_ptype_6plyvel_7_plyvel_RawIterator->tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_mstate->__pyx_ptype_6plyvel_7_plyvel_RawIterator->tp_getattro = PyObject_GenericGetAttr;
  }
  #endif
  if (__Pyx_SetVtable(__pyx_mstate->__pyx_ptype_6plyvel_7_plyvel_RawIterator, __pyx_vtabptr_6plyvel_7_plyvel_RawIterator) < (0)) __PYX_ERR(0, 1027, __pyx_L1_error)
  if (__Pyx_MergeVtables(__pyx_mstate->__pyx_ptype_6plyvel_7_plyvel_RawIterator) < (0)) __PYX_ERR(0, 1027, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_mstate_global->__pyx_n_u_RawIterator, (PyObject *) __pyx_mstate->__pyx_ptype_6plyvel_7_plyvel_RawIterator) < (0)) __PYX_ERR(0, 1027, __pyx_L1_error)
  #if CYTHON_USE_TYPE_SLOTS
  if (__pyx_mstate->__pyx_ptype_6plyvel_7_plyvel_RawIterator->tp_weaklistoffset == 0) __pyx_mstate->__pyx_ptype_6plyvel_7_plyvel_RawIterator->tp_weaklistoffset = offsetof(struct __pyx_obj_6plyvel_7_plyvel_RawIterator, __pyx_base.__weakref__);
  #endif
  if (__Pyx_setup_reduce((PyObject *) __pyx_mstate->__pyx_ptype_6plyvel_7_plyvel_RawIterator) < (0)) __PYX_ERR(0, 1027, __pyx_L1_error)
  __pyx_vtabptr_6plyvel_7_plyvel_Snapshot = &__pyx_vtable_6plyvel_7_plyvel_Snapshot;
  __pyx_vtable_6plyvel_7_plyvel_Snapshot.close = (PyObject *(*)(struct __pyx_obj_6plyvel_7_plyvel_Snapshot *, int __pyx_skip_dispatch))__pyx_f_6plyvel_7_plyvel_8Snapshot_close;
  #if CYTHON_USE_TYPE_SPECS
  __pyx_mstate->__pyx_ptype_6plyvel_7_plyvel_Snapshot = (PyTypeObject *) __Pyx_PyType_FromModuleAndSpec(__pyx_m, &__pyx_type_6plyvel_7_plyvel_Snapshot_spec, NULL); if (unlikely(!__pyx_mstate->__pyx_ptype_6plyvel_7_plyvel_Snapshot)) __PYX_ERR(0, 1117, __pyx_L1_error)
  if (__Pyx_fix_up_extension_type_from_spec(&__pyx_type_6plyvel_7_plyvel_Snapshot_spec, __pyx_mstate->__pyx_ptype_6plyvel_7_plyvel_Snapshot) < (0)) __PYX_ERR(0, 1117, __pyx_L1_error)
  #else
  __pyx_mstate->__pyx_ptype_6plyvel_7_plyvel_Snapshot = &__pyx_type_6plyvel_7_plyvel_Snapshot;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  #endif
  #if !CYTHON_USE_TYPE_SPECS
  if (__Pyx_PyType_Ready(__pyx_mstate->__pyx_ptype_6plyvel_7_plyvel_Snapshot) < (0)) __PYX_ERR(0, 1117, __pyx_L1_error)
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount((PyObject*)__pyx_mstate->__pyx_ptype_6plyvel_7_plyvel_Snapshot);
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_mstate->__pyx_ptype_6plyvel_7_plyvel_Snapshot->tp_dictoffset && __pyx_mstate->__pyx_ptype_6plyvel_7_plyvel_Snapshot->tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_mstate->__pyx_ptype_6plyvel_7_plyvel_Snapshot->tp_getattro = PyObject_GenericGetAttr;
  }
  #endif
  if (__Pyx_SetVtable(__pyx_mstate->__pyx_ptype_6plyvel_7_plyvel_Snapshot, __pyx_vtabptr_6plyvel_7_plyvel_Snapshot) < (0)) __PYX_ERR(0, 1117, __pyx_L1_error)
  if (__Pyx_MergeVtables(__pyx_mstate->__pyx_ptype_6plyvel_7_plyvel_Snapshot) < (0)) __PYX_ERR(0, 1117, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_mstate_global->__pyx_n_u_Snapshot, (PyObject *) __pyx_mstate->__pyx_ptype_6plyvel_7_plyvel_Snapshot) < (0)) __PYX_ERR(0, 1117, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject *) __pyx_mstate->__pyx_ptype_6plyvel_7_plyvel_Snapshot) < (0)) __PYX_ERR(0, 1117, __pyx_L1_error)
  #if CYTHON_USE_TYPE_SPECS
  __pyx_mstate->__pyx_ptype_6plyvel_7_plyvel___pyx_scope_struct____init__ = (PyTypeObject *) __Pyx_PyType_FromModuleAndSpec(__pyx_m, &__pyx_type_6plyvel_7_plyvel___pyx_scope_struct____init___spec, NULL); if (unlikely(!__pyx_mstate->__pyx_ptype_6plyvel_7_plyvel___pyx_scope_struct____init__)) __PYX_ERR(0, 680, __pyx_L1_error)
  if (__Pyx_fix_up_extension_type_from_spec(&__pyx_type_6plyvel_7_plyvel___pyx_scope_struct____init___spec, __pyx_mstate->__pyx_ptype_6plyvel_7_plyvel___pyx_scope_struct____init__) < (0)) __PYX_ERR(0, 680, __pyx_L1_error)
  #else
  __pyx_mstate->__pyx_ptype_6plyvel_7_plyvel___pyx_scope_struct____init__ = &__pyx_type_6plyvel_7_plyvel___pyx_scope_struct____init__;
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  #endif
  #if !CYTHON_USE_TYPE_SPECS
  if (__Pyx_PyType_Ready(__pyx_mstate->__pyx_ptype_6plyvel_7_plyvel___pyx_scope_struct____init__) < (0)) __PYX_ERR(0, 680, __pyx_L1_error)
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount((PyObject*)__pyx_mstate->__pyx_ptype_6plyvel_7_plyvel___pyx_scope_struct____init__);
  #endif
  #if !CYTHON_COMPILING_IN_LIMITED_API
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_mstate->__pyx_ptype_6plyvel_7_plyvel___pyx_scope_struct____init__->tp_dictoffset && __pyx_mstate->__pyx_ptype_6plyvel_7_plyvel___pyx_scope_struct____init__->tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_mstate->__pyx_ptype_6plyvel_7_plyvel___pyx_scope_struct____init__->tp_getattro = PyObject_GenericGetAttr;
  }
  #endif
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_type_import_code(__pyx_mstatetype *__pyx_mstate) {
  __Pyx_RefNannyDeclarations
  CYTHON_UNUSED_VAR(__pyx_mstate);
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_import_code", 0);
  /*--- Type import code ---*/
  __pyx_t_1 = PyImport_ImportModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 9, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_mstate->__pyx_ptype_7cpython_4type_type = __Pyx_ImportType_3_2_3(__pyx_t_1, __Pyx_BUILTIN_MODULE_NAME, "type",
  #if defined(PYPY_VERSION_NUM) && PYPY_VERSION_NUM < 0x050B0000
  sizeof(PyTypeObject), __PYX_GET_STRUCT_ALIGNMENT_3_2_3(PyTypeObject),
  #elif CYTHON_COMPILING_IN_LIMITED_API
  0, 0,
  #else
  sizeof(PyHeapTypeObject), __PYX_GET_STRUCT_ALIGNMENT_3_2_3(PyHeapTypeObject),
  #endif
  __Pyx_ImportType_CheckSize_Warn_3_2_3); if (!__pyx_mstate->__pyx_ptype_7cpython_4type_type) __PYX_ERR(3, 9, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_t_1)) __PYX_ERR(4, 8, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_mstate->__pyx_ptype_7cpython_4bool_bool = __Pyx_ImportType_3_2_3(__pyx_t_1, __Pyx_BUILTIN_MODULE_NAME, "bool",
  #if defined(PYPY_VERSION_NUM) && PYPY_VERSION_NUM < 0x050B0000
  sizeof(PyLongObject), __PYX_GET_STRUCT_ALIGNMENT_3_2_3(PyLongObject),
  #elif CYTHON_COMPILING_IN_LIMITED_API
  0, 0,
  #else
  sizeof(PyLongObject), __PYX_GET_STRUCT_ALIGNMENT_3_2_3(PyLongObject),
  #endif
  __Pyx_ImportType_CheckSize_Warn_3_2_3); if (!__pyx_mstate->__pyx_ptype_7cpython_4bool_bool) __PYX_ERR(4, 8, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_t_1)) __PYX_ERR(5, 16, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_mstate->__pyx_ptype_7cpython_7complex_complex = __Pyx_ImportType_3_2_3(__pyx_t_1, __Pyx_BUILTIN_MODULE_NAME, "complex",
  #if defined(PYPY_VERSION_NUM) && PYPY_VERSION_NUM < 0x050B0000
  sizeof(PyComplexObject), __PYX_GET_STRUCT_ALIGNMENT_3_2_3(PyComplexObject),
  #elif CYTHON_COMPILING_IN_LIMITED_API
  0, 0,
  #else
  sizeof(PyComplexObject), __PYX_GET_STRUCT_ALIGNMENT_3_2_3(PyComplexObject),
  #endif
  __Pyx_ImportType_CheckSize_Warn_3_2_3); if (!__pyx_mstate->__pyx_ptype_7cpython_7complex_complex) __PYX_ERR(5, 16, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_variable_import_code(__pyx_mstatetype *__pyx_mstate) {
  __Pyx_RefNannyDeclarations
  CYTHON_UNUSED_VAR(__pyx_mstate);
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_import_code", 0);
  /*--- Variable import code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_function_import_code(__pyx_mstatetype *__pyx_mstate) {
  __Pyx_RefNannyDeclarations
  CYTHON_UNUSED_VAR(__pyx_mstate);
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_import_code", 0);
  /*--- Function import code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

#if CYTHON_PEP489_MULTI_PHASE_INIT
static PyObject* __pyx_pymod_create(PyObject *spec, PyModuleDef *def); /*proto*/
static int __pyx_pymod_exec__plyvel(PyObject* module); /*proto*/
static PyModuleDef_Slot __pyx_moduledef_slots[] = {
  {Py_mod_create, (void*)__pyx_pymod_create},
  {Py_mod_exec, (void*)__pyx_pymod_exec__plyvel},
  #if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
  {Py_mod_gil, __Pyx_FREETHREADING_COMPATIBLE},
  #endif
  #if PY_VERSION_HEX >= 0x030C0000 && CYTHON_USE_MODULE_STATE
  {Py_mod_multiple_interpreters, Py_MOD_MULTIPLE_INTERPRETERS_NOT_SUPPORTED},
  #endif
  {0, NULL}
};
#endif

#ifdef __cplusplus
namespace {
  struct PyModuleDef __pyx_moduledef =
  #else
  static struct PyModuleDef __pyx_moduledef =
  #endif
  {
      PyModuleDef_HEAD_INIT,
      "_plyvel",
      __pyx_k_Plyvel_a_Python_LevelDB_interfa, /* m_doc */
    #if CYTHON_USE_MODULE_STATE
      sizeof(__pyx_mstatetype), /* m_size */
    #else
      (CYTHON_PEP489_MULTI_PHASE_INIT) ? 0 : -1, /* m_size */
    #endif
      __pyx_methods /* m_methods */,
    #if CYTHON_PEP489_MULTI_PHASE_INIT
      __pyx_moduledef_slots, /* m_slots */
    #else
      NULL, /* m_reload */
    #endif
    #if CYTHON_USE_MODULE_STATE
      __pyx_m_traverse, /* m_traverse */
      __pyx_m_clear, /* m_clear */
      NULL /* m_free */
    #else
      NULL, /* m_traverse */
      NULL, /* m_clear */
      NULL /* m_free */
    #endif
  };
  #ifdef __cplusplus
} /* anonymous namespace */
#endif

/* PyModInitFuncType */
#ifndef CYTHON_NO_PYINIT_EXPORT
  #define __Pyx_PyMODINIT_FUNC PyMODINIT_FUNC
#else
  #ifdef __cplusplus
  #define __Pyx_PyMODINIT_FUNC extern "C" PyObject *
  #else
  #define __Pyx_PyMODINIT_FUNC PyObject *
  #endif
#endif

__Pyx_PyMODINIT_FUNC PyInit__plyvel(void) CYTHON_SMALL_CODE; /*proto*/
__Pyx_PyMODINIT_FUNC PyInit__plyvel(void)
#if CYTHON_PEP489_MULTI_PHASE_INIT
{
  return PyModuleDef_Init(&__pyx_moduledef);
}
/* ModuleCreationPEP489 */
#if CYTHON_COMPILING_IN_LIMITED_API && (__PYX_LIMITED_VERSION_HEX < 0x03090000\
      || ((defined(_WIN32) || defined(WIN32) || defined(MS_WINDOWS)) && __PYX_LIMITED_VERSION_HEX < 0x030A0000))
static PY_INT64_T __Pyx_GetCurrentInterpreterId(void) {
    {
        PyObject *module = PyImport_ImportModule("_interpreters"); // 3.13+ I think
        if (!module) {
            PyErr_Clear(); // just try the 3.8-3.12 version
            module = PyImport_ImportModule("_xxsubinterpreters");
            if (!module) goto bad;
        }
        PyObject *current = PyObject_CallMethod(module, "get_current", NULL);
        Py_DECREF(module);
        if (!current) goto bad;
        if (PyTuple_Check(current)) {
            PyObject *new_current = PySequence_GetItem(current, 0);
            Py_DECREF(current);
            current = new_current;
            if (!new_current) goto bad;
        }
        long long as_c_int = PyLong_AsLongLong(current);
        Py_DECREF(current);
        return as_c_int;
    }
  bad:
    PySys_WriteStderr("__Pyx_GetCurrentInterpreterId failed. Try setting the C define CYTHON_PEP489_MULTI_PHASE_INIT=0\n");
    return -1;
}
#endif
#if !CYTHON_USE_MODULE_STATE
static CYTHON_SMALL_CODE int __Pyx_check_single_interpreter(void) {
    static PY_INT64_T main_interpreter_id = -1;
#if CYTHON_COMPILING_IN_GRAAL && defined(GRAALPY_VERSION_NUM) && GRAALPY_VERSION_NUM > 0x19000000
    PY_INT64_T current_id = GraalPyInterpreterState_GetIDFromThreadState(PyThreadState_Get());
#elif CYTHON_COMPILING_IN_GRAAL
    PY_INT64_T current_id = PyInterpreterState_GetIDFromThreadState(PyThreadState_Get());
#elif CYTHON_COMPILING_IN_LIMITED_API && (__PYX_LIMITED_VERSION_HEX < 0x03090000\
      || ((defined(_WIN32) || defined(WIN32) || defined(MS_WINDOWS)) && __PYX_LIMITED_VERSION_HEX < 0x030A0000))
    PY_INT64_T current_id = __Pyx_GetCurrentInterpreterId();
#elif CYTHON_COMPILING_IN_LIMITED_API
    PY_INT64_T current_id = PyInterpreterState_GetID(PyInterpreterState_Get());
#else
    PY_INT64_T current_id = PyInterpreterState_GetID(PyThreadState_Get()->interp);
#endif
    if (unlikely(current_id == -1)) {
        return -1;
    }
    if (main_interpreter_id == -1) {
        main_interpreter_id = current_id;
        return 0;
    } else if (unlikely(main_interpreter_id != current_id)) {
        PyErr_SetString(
            PyExc_ImportError,
            "Interpreter change detected - this module can only be loaded into one interpreter per process.");
        return -1;
    }
    return 0;
}
#endif
static CYTHON_SMALL_CODE int __Pyx_copy_spec_to_module(PyObject *spec, PyObject *moddict, const char* from_name, const char* to_name, int allow_none)
{
    PyObject *value = PyObject_GetAttrString(spec, from_name);
    int result = 0;
    if (likely(value)) {
        if (allow_none || value != Py_None) {
            result = PyDict_SetItemString(moddict, to_name, value);
        }
        Py_DECREF(value);
    } else if (PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Clear();
    } else {
        result = -1;
    }
    return result;
}
static CYTHON_SMALL_CODE PyObject* __pyx_pymod_create(PyObject *spec, PyModuleDef *def) {
    PyObject *module = NULL, *moddict, *modname;
    CYTHON_UNUSED_VAR(def);
    #if !CYTHON_USE_MODULE_STATE
    if (__Pyx_check_single_interpreter())
        return NULL;
    #endif
    if (__pyx_m)
        return __Pyx_NewRef(__pyx_m);
    modname = PyObject_GetAttrString(spec, "name");
    if (unlikely(!modname)) goto bad;
    module = PyModule_NewObject(modname);
    Py_DECREF(modname);
    if (unlikely(!module)) goto bad;
    moddict = PyModule_GetDict(module);
    if (unlikely(!moddict)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "loader", "__loader__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "origin", "__file__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "parent", "__package__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "submodule_search_locations", "__path__", 0) < 0)) goto bad;
    return module;
bad:
    Py_XDECREF(module);
    return NULL;
}


static CYTHON_SMALL_CODE int __pyx_pymod_exec__plyvel(PyObject *__pyx_pyinit_module)
#endif
{
  int stringtab_initialized = 0;
  #if CYTHON_USE_MODULE_STATE
  int pystate_addmodule_run = 0;
  #endif
  __pyx_mstatetype *__pyx_mstate = NULL;
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  Py_ssize_t __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5[3];
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannyDeclarations
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  if (__pyx_m) {
    if (__pyx_m == __pyx_pyinit_module) return 0;
    PyErr_SetString(PyExc_RuntimeError, "Module '_plyvel' has already been imported. Re-initialisation is not supported.");
    return -1;
  }
  #else
  if (__pyx_m) return __Pyx_NewRef(__pyx_m);
  #endif
  /*--- Module creation code ---*/
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  __pyx_t_1 = __pyx_pyinit_module;
  Py_INCREF(__pyx_t_1);
  #else
  __pyx_t_1 = PyModule_Create(&__pyx_moduledef); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #if CYTHON_USE_MODULE_STATE
  {
    int add_module_result = __Pyx_State_AddModule(__pyx_t_1, &__pyx_moduledef);
    __pyx_t_1 = 0; /* transfer ownership from __pyx_t_1 to "_plyvel" pseudovariable */
    if (unlikely((add_module_result < 0))) __PYX_ERR(0, 1, __pyx_L1_error)
    pystate_addmodule_run = 1;
  }
  #else
  __pyx_m = __pyx_t_1;
  #endif
  #if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
  PyUnstable_Module_SetGIL(__pyx_m, Py_MOD_GIL_USED);
  #endif
  __pyx_mstate = __pyx_mstate_global;
  CYTHON_UNUSED_VAR(__pyx_t_1);
  __pyx_mstate->__pyx_d = PyModule_GetDict(__pyx_m); if (unlikely(!__pyx_mstate->__pyx_d)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_mstate->__pyx_d);
  __pyx_mstate->__pyx_b = __Pyx_PyImport_AddModuleRef(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_mstate->__pyx_b)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_mstate->__pyx_cython_runtime = __Pyx_PyImport_AddModuleRef("cython_runtime"); if (unlikely(!__pyx_mstate->__pyx_cython_runtime)) __PYX_ERR(0, 1, __pyx_L1_error)
  if (PyObject_SetAttrString(__pyx_m, "__builtins__", __pyx_mstate->__pyx_b) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /* ImportRefnannyAPI */
  #if CYTHON_REFNANNY
  __Pyx_RefNanny = __Pyx_RefNannyImportAPI("refnanny");
  if (!__Pyx_RefNanny) {
    PyErr_Clear();
    __Pyx_RefNanny = __Pyx_RefNannyImportAPI("Cython.Runtime.refnanny");
    if (!__Pyx_RefNanny)
        Py_FatalError("failed to import 'refnanny' module");
  }
  #endif
  
__Pyx_RefNannySetupContext("PyInit__plyvel", 0);
  __Pyx_init_runtime_version();
  if (__Pyx_check_binary_version(__PYX_LIMITED_VERSION_HEX, __Pyx_get_runtime_version(), CYTHON_COMPILING_IN_LIMITED_API) < (0)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_mstate->__pyx_empty_tuple = PyTuple_New(0); if (unlikely(!__pyx_mstate->__pyx_empty_tuple)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_mstate->__pyx_empty_bytes = PyBytes_FromStringAndSize("", 0); if (unlikely(!__pyx_mstate->__pyx_empty_bytes)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_mstate->__pyx_empty_unicode = PyUnicode_FromStringAndSize("", 0); if (unlikely(!__pyx_mstate->__pyx_empty_unicode)) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Library function declarations ---*/
  /*--- Initialize various global constants etc. ---*/
  if (__Pyx_InitConstants(__pyx_mstate) < (0)) __PYX_ERR(0, 1, __pyx_L1_error)
  stringtab_initialized = 1;
  if (__Pyx_InitGlobals() < (0)) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__pyx_module_is_main_plyvel___plyvel) {
    if (PyObject_SetAttr(__pyx_m, __pyx_mstate_global->__pyx_n_u_name_2, __pyx_mstate_global->__pyx_n_u_main) < (0)) __PYX_ERR(0, 1, __pyx_L1_error)
  }
  {
    PyObject *modules = PyImport_GetModuleDict(); if (unlikely(!modules)) __PYX_ERR(0, 1, __pyx_L1_error)
    if (!PyDict_GetItemString(modules, "plyvel._plyvel")) {
      if (unlikely((PyDict_SetItemString(modules, "plyvel._plyvel", __pyx_m) < 0))) __PYX_ERR(0, 1, __pyx_L1_error)
    }
  }
  /*--- Builtin init code ---*/
  if (__Pyx_InitCachedBuiltins(__pyx_mstate) < (0)) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Constants init code ---*/
  if (__Pyx_InitCachedConstants(__pyx_mstate) < (0)) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_CreateCodeObjects(__pyx_mstate) < (0)) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Global type/function init code ---*/
  (void)__Pyx_modinit_global_init_code(__pyx_mstate);
  (void)__Pyx_modinit_variable_export_code(__pyx_mstate);
  (void)__Pyx_modinit_function_export_code(__pyx_mstate);
  if (unlikely((__Pyx_modinit_type_init_code(__pyx_mstate) < 0))) __PYX_ERR(0, 1, __pyx_L1_error)
  if (unlikely((__Pyx_modinit_type_import_code(__pyx_mstate) < 0))) __PYX_ERR(0, 1, __pyx_L1_error)
  (void)__Pyx_modinit_variable_import_code(__pyx_mstate);
  (void)__Pyx_modinit_function_import_code(__pyx_mstate);
  /*--- Execution code ---*/

  /* "plyvel/_plyvel.pyx":19
 * """
 * 
 * import sys             # <<<<<<<<<<<<<<
 * import threading
 * from weakref import ref as weakref_ref
*/
  __pyx_t_1 = __Pyx_Import(__pyx_mstate_global->__pyx_n_u_sys, 0, 0, NULL, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 19, __pyx_L1_error)
  __pyx_t_2 = __pyx_t_1;
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_sys, __pyx_t_2) < (0)) __PYX_ERR(0, 19, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "plyvel/_plyvel.pyx":20
 * 
 * import sys
 * import threading             # <<<<<<<<<<<<<<
 * from weakref import ref as weakref_ref
 * 
*/
  __pyx_t_1 = __Pyx_Import(__pyx_mstate_global->__pyx_n_u_threading, 0, 0, NULL, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 20, __pyx_L1_error)
  __pyx_t_2 = __pyx_t_1;
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_threading, __pyx_t_2) < (0)) __PYX_ERR(0, 20, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "plyvel/_plyvel.pyx":21
 * import sys
 * import threading
 * from weakref import ref as weakref_ref             # <<<<<<<<<<<<<<
 * 
 * cimport cython
*/
  {
    PyObject* const __pyx_imported_names[] = {__pyx_mstate_global->__pyx_n_u_ref};
    __pyx_t_1 = __Pyx_Import(__pyx_mstate_global->__pyx_n_u_weakref, __pyx_imported_names, 1, NULL, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 21, __pyx_L1_error)
  }
  __pyx_t_2 = __pyx_t_1;
  __Pyx_GOTREF(__pyx_t_2);
  {
    PyObject* const __pyx_imported_names[] = {__pyx_mstate_global->__pyx_n_u_ref};
    __pyx_t_3 = 0; {
      __pyx_t_4 = __Pyx_ImportFrom(__pyx_t_2, __pyx_imported_names[__pyx_t_3]); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 21, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      switch (__pyx_t_3) {
        case 0:
        if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_weakref_ref, __pyx_t_4) < (0)) __PYX_ERR(0, 21, __pyx_L1_error)
        break;
        default:;
      }
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    }
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "plyvel/_plyvel.pyx":59
 * 
 * 
 * __leveldb_version__ = '%d.%d' % (leveldb.kMajorVersion,             # <<<<<<<<<<<<<<
 *                                  leveldb.kMinorVersion)
 * 
*/
  __pyx_t_2 = __Pyx_PyUnicode_From_int(leveldb::kMajorVersion, 0, ' ', 'd'); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 59, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);

  /* "plyvel/_plyvel.pyx":60
 * 
 * __leveldb_version__ = '%d.%d' % (leveldb.kMajorVersion,
 *                                  leveldb.kMinorVersion)             # <<<<<<<<<<<<<<
 * 
 * 
*/
  __pyx_t_4 = __Pyx_PyUnicode_From_int(leveldb::kMinorVersion, 0, ' ', 'd'); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 60, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5[0] = __pyx_t_2;
  __pyx_t_5[1] = __pyx_mstate_global->__pyx_kp_u__3;
  __pyx_t_5[2] = __pyx_t_4;

  /* "plyvel/_plyvel.pyx":59
 * 
 * 
 * __leveldb_version__ = '%d.%d' % (leveldb.kMajorVersion,             # <<<<<<<<<<<<<<
 *                                  leveldb.kMinorVersion)
 * 
*/
  __pyx_t_6 = __Pyx_PyUnicode_Join(__pyx_t_5, 3, __Pyx_PyUnicode_GET_LENGTH(__pyx_t_2) + 1 + __Pyx_PyUnicode_GET_LENGTH(__pyx_t_4), 127);
  if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 59, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_leveldb_version, __pyx_t_6) < (0)) __PYX_ERR(0, 59, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;

  /* "plyvel/_plyvel.pyx":67
 * #
 * 
 * class Error(Exception):             # <<<<<<<<<<<<<<
 *     pass
 * 
*/
  __pyx_t_6 = PyTuple_Pack(1, ((PyObject *)(((PyTypeObject*)PyExc_Exception)))); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 67, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_4 = __Pyx_PEP560_update_bases(__pyx_t_6); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 67, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_2 = __Pyx_CalculateMetaclass(NULL, __pyx_t_4); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 67, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_7 = __Pyx_Py3MetaclassPrepare(__pyx_t_2, __pyx_t_4, __pyx_mstate_global->__pyx_n_u_Error, __pyx_mstate_global->__pyx_n_u_Error, (PyObject *) NULL, __pyx_mstate_global->__pyx_n_u_plyvel__plyvel, (PyObject *) NULL); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 67, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  if (__pyx_t_4 != __pyx_t_6) {
    if (unlikely((PyDict_SetItemString(__pyx_t_7, "__orig_bases__", __pyx_t_6) < 0))) __PYX_ERR(0, 67, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_6 = __Pyx_Py3ClassCreate(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_Error, __pyx_t_4, __pyx_t_7, NULL, 0, 0); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 67, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_6);
  #endif
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_Error, __pyx_t_6) < (0)) __PYX_ERR(0, 67, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "plyvel/_plyvel.pyx":71
 * 
 * 
 * class IOError(Error, IOError):             # <<<<<<<<<<<<<<
 *     pass
 * 
*/
  __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_Error); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 71, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_IOError); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 71, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_7 = PyTuple_Pack(2, __pyx_t_4, __pyx_t_2); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 71, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PEP560_update_bases(__pyx_t_7); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 71, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __Pyx_CalculateMetaclass(NULL, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 71, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_6 = __Pyx_Py3MetaclassPrepare(__pyx_t_4, __pyx_t_2, __pyx_mstate_global->__pyx_n_u_IOError, __pyx_mstate_global->__pyx_n_u_IOError, (PyObject *) NULL, __pyx_mstate_global->__pyx_n_u_plyvel__plyvel, (PyObject *) NULL); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 71, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  if (__pyx_t_2 != __pyx_t_7) {
    if (unlikely((PyDict_SetItemString(__pyx_t_6, "__orig_bases__", __pyx_t_7) < 0))) __PYX_ERR(0, 71, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __pyx_t_7 = __Pyx_Py3ClassCreate(__pyx_t_4, __pyx_mstate_global->__pyx_n_u_IOError, __pyx_t_2, __pyx_t_6, NULL, 0, 0); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 71, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_7);
  #endif
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_IOError, __pyx_t_7) < (0)) __PYX_ERR(0, 71, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "plyvel/_plyvel.pyx":75
 * 
 * 
 * class CorruptionError(Error):             # <<<<<<<<<<<<<<
 *     pass
 * 
*/
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_Error); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 75, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = PyTuple_Pack(1, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 75, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PEP560_update_bases(__pyx_t_4); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 75, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_6 = __Pyx_CalculateMetaclass(NULL, __pyx_t_2); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 75, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __pyx_t_7 = __Pyx_Py3MetaclassPrepare(__pyx_t_6, __pyx_t_2, __pyx_mstate_global->__pyx_n_u_CorruptionError, __pyx_mstate_global->__pyx_n_u_CorruptionError, (PyObject *) NULL, __pyx_mstate_global->__pyx_n_u_plyvel__plyvel, (PyObject *) NULL); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 75, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  if (__pyx_t_2 != __pyx_t_4) {
    if (unlikely((PyDict_SetItemString(__pyx_t_7, "__orig_bases__", __pyx_t_4) < 0))) __PYX_ERR(0, 75, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = __Pyx_Py3ClassCreate(__pyx_t_6, __pyx_mstate_global->__pyx_n_u_CorruptionError, __pyx_t_2, __pyx_t_7, NULL, 0, 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 75, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_4);
  #endif
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_CorruptionError, __pyx_t_4) < (0)) __PYX_ERR(0, 75, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "plyvel/_plyvel.pyx":79
 * 
 * 
 * class IteratorInvalidError(Error):             # <<<<<<<<<<<<<<
 *     pass
 * 
*/
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_Error); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 79, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_6 = PyTuple_Pack(1, __pyx_t_2); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 79, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_PEP560_update_bases(__pyx_t_6); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 79, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_7 = __Pyx_CalculateMetaclass(NULL, __pyx_t_2); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 79, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  __pyx_t_4 = __Pyx_Py3MetaclassPrepare(__pyx_t_7, __pyx_t_2, __pyx_mstate_global->__pyx_n_u_IteratorInvalidError, __pyx_mstate_global->__pyx_n_u_IteratorInvalidError, (PyObject *) NULL, __pyx_mstate_global->__pyx_n_u_plyvel__plyvel, (PyObject *) NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 79, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (__pyx_t_2 != __pyx_t_6) {
    if (unlikely((PyDict_SetItemString(__pyx_t_4, "__orig_bases__", __pyx_t_6) < 0))) __PYX_ERR(0, 79, __pyx_L1_error)
  }
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __pyx_t_6 = __Pyx_Py3ClassCreate(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_IteratorInvalidError, __pyx_t_2, __pyx_t_4, NULL, 0, 0); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 79, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_6);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_6);
  #endif
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_IteratorInvalidError, __pyx_t_6) < (0)) __PYX_ERR(0, 79, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "plyvel/_plyvel.pyx":257
 *         self.iterators = dict()
 * 
 *     cpdef close(self):             # <<<<<<<<<<<<<<
 *         # If the constructor raised an exception (and hence never
 *         # completed), self.iterators can be None. In that case no
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6plyvel_7_plyvel_2DB_3close, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_DB_close, NULL, __pyx_mstate_global->__pyx_n_u_plyvel__plyvel, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[1])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 257, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6plyvel_7_plyvel_DB, __pyx_mstate_global->__pyx_n_u_close, __pyx_t_2) < (0)) __PYX_ERR(0, 257, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "plyvel/_plyvel.pyx":301
 *         )
 * 
 *     def get(self, bytes key not None, default=None, *,             # <<<<<<<<<<<<<<
 *             bool verify_checksums=False, bool fill_cache=True):
 *         if self._db is NULL:
*/
  __pyx_t_2 = __Pyx_PyDict_NewPresized(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 301, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);

  /* "plyvel/_plyvel.pyx":302
 * 
 *     def get(self, bytes key not None, default=None, *,
 *             bool verify_checksums=False, bool fill_cache=True):             # <<<<<<<<<<<<<<
 *         if self._db is NULL:
 *             raise RuntimeError("Database is closed")
*/
  if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_verify_checksums, Py_False) < (0)) __PYX_ERR(0, 301, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_fill_cache, Py_True) < (0)) __PYX_ERR(0, 301, __pyx_L1_error)

  /* "plyvel/_plyvel.pyx":301
 *         )
 * 
 *     def get(self, bytes key not None, default=None, *,             # <<<<<<<<<<<<<<
 *             bool verify_checksums=False, bool fill_cache=True):
 *         if self._db is NULL:
*/
  __pyx_t_7 = __Pyx_CyFunction_New(&__pyx_mdef_6plyvel_7_plyvel_2DB_9get, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_DB_get, NULL, __pyx_mstate_global->__pyx_n_u_plyvel__plyvel, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[2])); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 301, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_7);
  #endif
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_7, __pyx_mstate_global->__pyx_tuple[2]);
  __Pyx_CyFunction_SetDefaultsKwDict(__pyx_t_7, __pyx_t_2);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6plyvel_7_plyvel_DB, __pyx_mstate_global->__pyx_n_u_get, __pyx_t_7) < (0)) __PYX_ERR(0, 301, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "plyvel/_plyvel.pyx":312
 *         return db_get(self, key, default, read_options)
 * 
 *     def put(self, bytes key not None, value not None, *, bool sync=False):             # <<<<<<<<<<<<<<
 *         if self._db is NULL:
 *             raise RuntimeError("Database is closed")
*/
  __pyx_t_7 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 312, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  if (PyDict_SetItem(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_sync, Py_False) < (0)) __PYX_ERR(0, 312, __pyx_L1_error)
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6plyvel_7_plyvel_2DB_11put, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_DB_put, NULL, __pyx_mstate_global->__pyx_n_u_plyvel__plyvel, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[3])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 312, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  __Pyx_CyFunction_SetDefaultsKwDict(__pyx_t_2, __pyx_t_7);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6plyvel_7_plyvel_DB, __pyx_mstate_global->__pyx_n_u_put, __pyx_t_2) < (0)) __PYX_ERR(0, 312, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "plyvel/_plyvel.pyx":333
 *         raise_for_status(st)
 * 
 *     def delete(self, bytes key not None, *, bool sync=False):             # <<<<<<<<<<<<<<
 *         if self._db is NULL:
 *             raise RuntimeError("Database is closed")
*/
  __pyx_t_2 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 333, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_sync, Py_False) < (0)) __PYX_ERR(0, 333, __pyx_L1_error)
  __pyx_t_7 = __Pyx_CyFunction_New(&__pyx_mdef_6plyvel_7_plyvel_2DB_13delete, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_DB_delete, NULL, __pyx_mstate_global->__pyx_n_u_plyvel__plyvel, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[4])); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 333, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_7);
  #endif
  __Pyx_CyFunction_SetDefaultsKwDict(__pyx_t_7, __pyx_t_2);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6plyvel_7_plyvel_DB, __pyx_mstate_global->__pyx_n_u_delete, __pyx_t_7) < (0)) __PYX_ERR(0, 333, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "plyvel/_plyvel.pyx":346
 *         raise_for_status(st)
 * 
 *     def write_batch(self, *, bool transaction=False, bool sync=False):             # <<<<<<<<<<<<<<
 *         if self._db is NULL:
 *             raise RuntimeError("Database is closed")
*/
  __pyx_t_7 = __Pyx_PyDict_NewPresized(2); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 346, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  if (PyDict_SetItem(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_transaction, Py_False) < (0)) __PYX_ERR(0, 346, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_sync, Py_False) < (0)) __PYX_ERR(0, 346, __pyx_L1_error)
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6plyvel_7_plyvel_2DB_15write_batch, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_DB_write_batch, NULL, __pyx_mstate_global->__pyx_n_u_plyvel__plyvel, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[5])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 346, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  __Pyx_CyFunction_SetDefaultsKwDict(__pyx_t_2, __pyx_t_7);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6plyvel_7_plyvel_DB, __pyx_mstate_global->__pyx_n_u_write_batch, __pyx_t_2) < (0)) __PYX_ERR(0, 346, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "plyvel/_plyvel.pyx":358
 *         return self.iterator()
 * 
 *     def iterator(self, *, reverse=False, start=None, stop=None,             # <<<<<<<<<<<<<<
 *                  include_start=True, include_stop=False, prefix=None,
 *                  include_key=True, include_value=True,
*/
  __pyx_t_2 = __Pyx_PyDict_NewPresized(10); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 358, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_reverse, Py_False) < (0)) __PYX_ERR(0, 358, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_start, Py_None) < (0)) __PYX_ERR(0, 358, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_stop, Py_None) < (0)) __PYX_ERR(0, 358, __pyx_L1_error)

  /* "plyvel/_plyvel.pyx":359
 * 
 *     def iterator(self, *, reverse=False, start=None, stop=None,
 *                  include_start=True, include_stop=False, prefix=None,             # <<<<<<<<<<<<<<
 *                  include_key=True, include_value=True,
 *                  bool verify_checksums=False, bool fill_cache=True):
*/
  if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_include_start, Py_True) < (0)) __PYX_ERR(0, 358, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_include_stop, Py_False) < (0)) __PYX_ERR(0, 358, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_prefix, Py_None) < (0)) __PYX_ERR(0, 358, __pyx_L1_error)

  /* "plyvel/_plyvel.pyx":360
 *     def iterator(self, *, reverse=False, start=None, stop=None,
 *                  include_start=True, include_stop=False, prefix=None,
 *                  include_key=True, include_value=True,             # <<<<<<<<<<<<<<
 *                  bool verify_checksums=False, bool fill_cache=True):
 *         return Iterator(
*/
  if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_include_key, Py_True) < (0)) __PYX_ERR(0, 358, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_include_value, Py_True) < (0)) __PYX_ERR(0, 358, __pyx_L1_error)

  /* "plyvel/_plyvel.pyx":361
 *                  include_start=True, include_stop=False, prefix=None,
 *                  include_key=True, include_value=True,
 *                  bool verify_checksums=False, bool fill_cache=True):             # <<<<<<<<<<<<<<
 *         return Iterator(
 *             self,  # db
*/
  if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_verify_checksums, Py_False) < (0)) __PYX_ERR(0, 358, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_fill_cache, Py_True) < (0)) __PYX_ERR(0, 358, __pyx_L1_error)

  /* "plyvel/_plyvel.pyx":358
 *         return self.iterator()
 * 
 *     def iterator(self, *, reverse=False, start=None, stop=None,             # <<<<<<<<<<<<<<
 *                  include_start=True, include_stop=False, prefix=None,
 *                  include_key=True, include_value=True,
*/
  __pyx_t_7 = __Pyx_CyFunction_New(&__pyx_mdef_6plyvel_7_plyvel_2DB_19iterator, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_DB_iterator, NULL, __pyx_mstate_global->__pyx_n_u_plyvel__plyvel, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[6])); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 358, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_7);
  #endif
  __Pyx_CyFunction_SetDefaultsKwDict(__pyx_t_7, __pyx_t_2);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6plyvel_7_plyvel_DB, __pyx_mstate_global->__pyx_n_u_iterator, __pyx_t_7) < (0)) __PYX_ERR(0, 358, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "plyvel/_plyvel.pyx":378
 *         )
 * 
 *     def raw_iterator(self, *, bool verify_checksums=False, bool fill_cache=True):             # <<<<<<<<<<<<<<
 *         return RawIterator(
 *             self,  # db
*/
  __pyx_t_7 = __Pyx_PyDict_NewPresized(2); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 378, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  if (PyDict_SetItem(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_verify_checksums, Py_False) < (0)) __PYX_ERR(0, 378, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_fill_cache, Py_True) < (0)) __PYX_ERR(0, 378, __pyx_L1_error)
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6plyvel_7_plyvel_2DB_21raw_iterator, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_DB_raw_iterator, NULL, __pyx_mstate_global->__pyx_n_u_plyvel__plyvel, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[7])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 378, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  __Pyx_CyFunction_SetDefaultsKwDict(__pyx_t_2, __pyx_t_7);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6plyvel_7_plyvel_DB, __pyx_mstate_global->__pyx_n_u_raw_iterator, __pyx_t_2) < (0)) __PYX_ERR(0, 378, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "plyvel/_plyvel.pyx":386
 *         )
 * 
 *     def snapshot(self):             # <<<<<<<<<<<<<<
 *         return Snapshot(db=self)
 * 
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6plyvel_7_plyvel_2DB_23snapshot, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_DB_snapshot, NULL, __pyx_mstate_global->__pyx_n_u_plyvel__plyvel, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[8])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 386, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6plyvel_7_plyvel_DB, __pyx_mstate_global->__pyx_n_u_snapshot, __pyx_t_2) < (0)) __PYX_ERR(0, 386, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "plyvel/_plyvel.pyx":389
 *         return Snapshot(db=self)
 * 
 *     def get_property(self, bytes name not None):             # <<<<<<<<<<<<<<
 *         if self._db is NULL:
 *             raise RuntimeError("Database is closed")
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6plyvel_7_plyvel_2DB_25get_property, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_DB_get_property, NULL, __pyx_mstate_global->__pyx_n_u_plyvel__plyvel, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[9])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 389, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6plyvel_7_plyvel_DB, __pyx_mstate_global->__pyx_n_u_get_property, __pyx_t_2) < (0)) __PYX_ERR(0, 389, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "plyvel/_plyvel.pyx":402
 *         return value if result else None
 * 
 *     def compact_range(self, *, bytes start=None, bytes stop=None):             # <<<<<<<<<<<<<<
 *         if self._db is NULL:
 *             raise RuntimeError("Database is closed")
*/
  __pyx_t_2 = __Pyx_PyDict_NewPresized(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 402, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_start, Py_None) < (0)) __PYX_ERR(0, 402, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_stop, Py_None) < (0)) __PYX_ERR(0, 402, __pyx_L1_error)
  __pyx_t_7 = __Pyx_CyFunction_New(&__pyx_mdef_6plyvel_7_plyvel_2DB_27compact_range, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_DB_compact_range, NULL, __pyx_mstate_global->__pyx_n_u_plyvel__plyvel, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[10])); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 402, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_7);
  #endif
  __Pyx_CyFunction_SetDefaultsKwDict(__pyx_t_7, __pyx_t_2);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6plyvel_7_plyvel_DB, __pyx_mstate_global->__pyx_n_u_compact_range, __pyx_t_7) < (0)) __PYX_ERR(0, 402, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "plyvel/_plyvel.pyx":418
 *             self._db.CompactRange(&start_slice, &stop_slice)
 * 
 *     def approximate_size(self, bytes start not None, bytes stop not None):             # <<<<<<<<<<<<<<
 *         if self._db is NULL:
 *             raise RuntimeError("Database is closed")
*/
  __pyx_t_7 = __Pyx_CyFunction_New(&__pyx_mdef_6plyvel_7_plyvel_2DB_29approximate_size, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_DB_approximate_size, NULL, __pyx_mstate_global->__pyx_n_u_plyvel__plyvel, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[11])); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 418, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_7);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6plyvel_7_plyvel_DB, __pyx_mstate_global->__pyx_n_u_approximate_size, __pyx_t_7) < (0)) __PYX_ERR(0, 418, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "plyvel/_plyvel.pyx":424
 *         return self.approximate_sizes((start, stop))[0]
 * 
 *     def approximate_sizes(self, *ranges):             # <<<<<<<<<<<<<<
 *         if self._db is NULL:
 *             raise RuntimeError("Database is closed")
*/
  __pyx_t_7 = __Pyx_CyFunction_New(&__pyx_mdef_6plyvel_7_plyvel_2DB_31approximate_sizes, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_DB_approximate_sizes, NULL, __pyx_mstate_global->__pyx_n_u_plyvel__plyvel, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[12])); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 424, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_7);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6plyvel_7_plyvel_DB, __pyx_mstate_global->__pyx_n_u_approximate_sizes, __pyx_t_7) < (0)) __PYX_ERR(0, 424, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "plyvel/_plyvel.pyx":449
 *             free(sizes)
 * 
 *     def prefixed_db(self, bytes prefix not None):             # <<<<<<<<<<<<<<
 *         return PrefixedDB(db=self, prefix=prefix)
 * 
*/
  __pyx_t_7 = __Pyx_CyFunction_New(&__pyx_mdef_6plyvel_7_plyvel_2DB_33prefixed_db, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_DB_prefixed_db, NULL, __pyx_mstate_global->__pyx_n_u_plyvel__plyvel, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[13])); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 449, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_7);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6plyvel_7_plyvel_DB, __pyx_mstate_global->__pyx_n_u_prefixed_db, __pyx_t_7) < (0)) __PYX_ERR(0, 449, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "plyvel/_plyvel.pyx":452
 *         return PrefixedDB(db=self, prefix=prefix)
 * 
 *     def __enter__(self):             # <<<<<<<<<<<<<<
 *         return self
 * 
*/
  __pyx_t_7 = __Pyx_CyFunction_New(&__pyx_mdef_6plyvel_7_plyvel_2DB_35__enter__, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_DB___enter, NULL, __pyx_mstate_global->__pyx_n_u_plyvel__plyvel, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[14])); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 452, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_7);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6plyvel_7_plyvel_DB, __pyx_mstate_global->__pyx_n_u_enter, __pyx_t_7) < (0)) __PYX_ERR(0, 452, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "plyvel/_plyvel.pyx":455
 *         return self
 * 
 *     def __exit__(self, exc_type, exc_val, exc_tb):             # <<<<<<<<<<<<<<
 *         self.close()
 * 
*/
  __pyx_t_7 = __Pyx_CyFunction_New(&__pyx_mdef_6plyvel_7_plyvel_2DB_37__exit__, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_DB___exit, NULL, __pyx_mstate_global->__pyx_n_u_plyvel__plyvel, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[15])); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 455, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_7);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6plyvel_7_plyvel_DB, __pyx_mstate_global->__pyx_n_u_exit, __pyx_t_7) < (0)) __PYX_ERR(0, 455, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self._db,self.options cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):
*/
  __pyx_t_7 = __Pyx_CyFunction_New(&__pyx_mdef_6plyvel_7_plyvel_2DB_39__reduce_cython__, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_DB___reduce_cython, NULL, __pyx_mstate_global->__pyx_n_u_plyvel__plyvel, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[16])); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_7);
  #endif
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_reduce_cython, __pyx_t_7) < (0)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError, "self._db,self.options cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self._db,self.options cannot be converted to a Python object for pickling"
*/
  __pyx_t_7 = __Pyx_CyFunction_New(&__pyx_mdef_6plyvel_7_plyvel_2DB_41__setstate_cython__, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_DB___setstate_cython, NULL, __pyx_mstate_global->__pyx_n_u_plyvel__plyvel, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[17])); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_7);
  #endif
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_setstate_cython, __pyx_t_7) < (0)) __PYX_ERR(1, 3, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "plyvel/_plyvel.pyx":473
 *         )
 * 
 *     def get(self, bytes key not None, default=None, *,             # <<<<<<<<<<<<<<
 *             bool verify_checksums=False, bool fill_cache=True):
 *         return self.db.get(
*/
  __pyx_t_7 = __Pyx_PyDict_NewPresized(2); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 473, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);

  /* "plyvel/_plyvel.pyx":474
 * 
 *     def get(self, bytes key not None, default=None, *,
 *             bool verify_checksums=False, bool fill_cache=True):             # <<<<<<<<<<<<<<
 *         return self.db.get(
 *             self.prefix + key,
*/
  if (PyDict_SetItem(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_verify_checksums, Py_False) < (0)) __PYX_ERR(0, 473, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_fill_cache, Py_True) < (0)) __PYX_ERR(0, 473, __pyx_L1_error)

  /* "plyvel/_plyvel.pyx":473
 *         )
 * 
 *     def get(self, bytes key not None, default=None, *,             # <<<<<<<<<<<<<<
 *             bool verify_checksums=False, bool fill_cache=True):
 *         return self.db.get(
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6plyvel_7_plyvel_10PrefixedDB_5get, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_PrefixedDB_get, NULL, __pyx_mstate_global->__pyx_n_u_plyvel__plyvel, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[18])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 473, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_2, __pyx_mstate_global->__pyx_tuple[2]);
  __Pyx_CyFunction_SetDefaultsKwDict(__pyx_t_2, __pyx_t_7);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6plyvel_7_plyvel_PrefixedDB, __pyx_mstate_global->__pyx_n_u_get, __pyx_t_2) < (0)) __PYX_ERR(0, 473, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "plyvel/_plyvel.pyx":481
 *             fill_cache=fill_cache)
 * 
 *     def put(self, bytes key not None, value not None, *,             # <<<<<<<<<<<<<<
 *             bool sync=False):
 *         return self.db.put(self.prefix + key, value, sync=sync)
*/
  __pyx_t_2 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 481, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);

  /* "plyvel/_plyvel.pyx":482
 * 
 *     def put(self, bytes key not None, value not None, *,
 *             bool sync=False):             # <<<<<<<<<<<<<<
 *         return self.db.put(self.prefix + key, value, sync=sync)
 * 
*/
  if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_sync, Py_False) < (0)) __PYX_ERR(0, 481, __pyx_L1_error)

  /* "plyvel/_plyvel.pyx":481
 *             fill_cache=fill_cache)
 * 
 *     def put(self, bytes key not None, value not None, *,             # <<<<<<<<<<<<<<
 *             bool sync=False):
 *         return self.db.put(self.prefix + key, value, sync=sync)
*/
  __pyx_t_7 = __Pyx_CyFunction_New(&__pyx_mdef_6plyvel_7_plyvel_10PrefixedDB_7put, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_PrefixedDB_put, NULL, __pyx_mstate_global->__pyx_n_u_plyvel__plyvel, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[19])); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 481, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_7);
  #endif
  __Pyx_CyFunction_SetDefaultsKwDict(__pyx_t_7, __pyx_t_2);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6plyvel_7_plyvel_PrefixedDB, __pyx_mstate_global->__pyx_n_u_put, __pyx_t_7) < (0)) __PYX_ERR(0, 481, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "plyvel/_plyvel.pyx":485
 *         return self.db.put(self.prefix + key, value, sync=sync)
 * 
 *     def delete(self, bytes key not None, *, bool sync=False):             # <<<<<<<<<<<<<<
 *         return self.db.delete(self.prefix + key, sync=sync)
 * 
*/
  __pyx_t_7 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 485, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  if (PyDict_SetItem(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_sync, Py_False) < (0)) __PYX_ERR(0, 485, __pyx_L1_error)
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6plyvel_7_plyvel_10PrefixedDB_9delete, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_PrefixedDB_delete, NULL, __pyx_mstate_global->__pyx_n_u_plyvel__plyvel, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[20])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 485, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  __Pyx_CyFunction_SetDefaultsKwDict(__pyx_t_2, __pyx_t_7);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6plyvel_7_plyvel_PrefixedDB, __pyx_mstate_global->__pyx_n_u_delete, __pyx_t_2) < (0)) __PYX_ERR(0, 485, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "plyvel/_plyvel.pyx":488
 *         return self.db.delete(self.prefix + key, sync=sync)
 * 
 *     def write_batch(self, *, transaction=False, bool sync=False):             # <<<<<<<<<<<<<<
 *         return WriteBatch(self.db, self.prefix, transaction, sync)
 * 
*/
  __pyx_t_2 = __Pyx_PyDict_NewPresized(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 488, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_transaction, Py_False) < (0)) __PYX_ERR(0, 488, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_sync, Py_False) < (0)) __PYX_ERR(0, 488, __pyx_L1_error)
  __pyx_t_7 = __Pyx_CyFunction_New(&__pyx_mdef_6plyvel_7_plyvel_10PrefixedDB_11write_batch, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_PrefixedDB_write_batch, NULL, __pyx_mstate_global->__pyx_n_u_plyvel__plyvel, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[21])); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 488, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_7);
  #endif
  __Pyx_CyFunction_SetDefaultsKwDict(__pyx_t_7, __pyx_t_2);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6plyvel_7_plyvel_PrefixedDB, __pyx_mstate_global->__pyx_n_u_write_batch, __pyx_t_7) < (0)) __PYX_ERR(0, 488, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "plyvel/_plyvel.pyx":494
 *         return self.iterator()
 * 
 *     def iterator(self, *, reverse=False, start=None, stop=None,             # <<<<<<<<<<<<<<
 *                  include_start=True, include_stop=False, prefix=None,
 *                  include_key=True, include_value=True,
*/
  __pyx_t_7 = __Pyx_PyDict_NewPresized(10); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 494, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  if (PyDict_SetItem(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_reverse, Py_False) < (0)) __PYX_ERR(0, 494, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_start, Py_None) < (0)) __PYX_ERR(0, 494, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_stop, Py_None) < (0)) __PYX_ERR(0, 494, __pyx_L1_error)

  /* "plyvel/_plyvel.pyx":495
 * 
 *     def iterator(self, *, reverse=False, start=None, stop=None,
 *                  include_start=True, include_stop=False, prefix=None,             # <<<<<<<<<<<<<<
 *                  include_key=True, include_value=True,
 *                  bool verify_checksums=False, bool fill_cache=True):
*/
  if (PyDict_SetItem(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_include_start, Py_True) < (0)) __PYX_ERR(0, 494, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_include_stop, Py_False) < (0)) __PYX_ERR(0, 494, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_prefix, Py_None) < (0)) __PYX_ERR(0, 494, __pyx_L1_error)

  /* "plyvel/_plyvel.pyx":496
 *     def iterator(self, *, reverse=False, start=None, stop=None,
 *                  include_start=True, include_stop=False, prefix=None,
 *                  include_key=True, include_value=True,             # <<<<<<<<<<<<<<
 *                  bool verify_checksums=False, bool fill_cache=True):
 *         return Iterator(
*/
  if (PyDict_SetItem(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_include_key, Py_True) < (0)) __PYX_ERR(0, 494, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_include_value, Py_True) < (0)) __PYX_ERR(0, 494, __pyx_L1_error)

  /* "plyvel/_plyvel.pyx":497
 *                  include_start=True, include_stop=False, prefix=None,
 *                  include_key=True, include_value=True,
 *                  bool verify_checksums=False, bool fill_cache=True):             # <<<<<<<<<<<<<<
 *         return Iterator(
 *             self.db,
*/
  if (PyDict_SetItem(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_verify_checksums, Py_False) < (0)) __PYX_ERR(0, 494, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_fill_cache, Py_True) < (0)) __PYX_ERR(0, 494, __pyx_L1_error)

  /* "plyvel/_plyvel.pyx":494
 *         return self.iterator()
 * 
 *     def iterator(self, *, reverse=False, start=None, stop=None,             # <<<<<<<<<<<<<<
 *                  include_start=True, include_stop=False, prefix=None,
 *                  include_key=True, include_value=True,
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6plyvel_7_plyvel_10PrefixedDB_15iterator, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_PrefixedDB_iterator, NULL, __pyx_mstate_global->__pyx_n_u_plyvel__plyvel, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[22])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 494, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  __Pyx_CyFunction_SetDefaultsKwDict(__pyx_t_2, __pyx_t_7);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6plyvel_7_plyvel_PrefixedDB, __pyx_mstate_global->__pyx_n_u_iterator, __pyx_t_2) < (0)) __PYX_ERR(0, 494, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "plyvel/_plyvel.pyx":514
 *         )
 * 
 *     def snapshot(self):             # <<<<<<<<<<<<<<
 *         return Snapshot(db=self.db, prefix=self.prefix)
 * 
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6plyvel_7_plyvel_10PrefixedDB_17snapshot, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_PrefixedDB_snapshot, NULL, __pyx_mstate_global->__pyx_n_u_plyvel__plyvel, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[23])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 514, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6plyvel_7_plyvel_PrefixedDB, __pyx_mstate_global->__pyx_n_u_snapshot, __pyx_t_2) < (0)) __PYX_ERR(0, 514, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "plyvel/_plyvel.pyx":517
 *         return Snapshot(db=self.db, prefix=self.prefix)
 * 
 *     def prefixed_db(self, bytes prefix not None):             # <<<<<<<<<<<<<<
 *         return PrefixedDB(db=self.db, prefix=self.prefix + prefix)
 * 
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6plyvel_7_plyvel_10PrefixedDB_19prefixed_db, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_PrefixedDB_prefixed_db, NULL, __pyx_mstate_global->__pyx_n_u_plyvel__plyvel, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[24])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 517, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6plyvel_7_plyvel_PrefixedDB, __pyx_mstate_global->__pyx_n_u_prefixed_db, __pyx_t_2) < (0)) __PYX_ERR(0, 517, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6plyvel_7_plyvel_10PrefixedDB_21__reduce_cython__, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_PrefixedDB___reduce_cython, NULL, __pyx_mstate_global->__pyx_n_u_plyvel__plyvel, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[25])); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6plyvel_7_plyvel_PrefixedDB, __pyx_mstate_global->__pyx_n_u_reduce_cython, __pyx_t_2) < (0)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_PrefixedDB, (type(self), 0xe9c0b3a, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_PrefixedDB__set_state(self, __pyx_state)
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6plyvel_7_plyvel_10PrefixedDB_23__setstate_cython__, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_PrefixedDB___setstate_cython, NULL, __pyx_mstate_global->__pyx_n_u_plyvel__plyvel, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[26])); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 16, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6plyvel_7_plyvel_PrefixedDB, __pyx_mstate_global->__pyx_n_u_setstate_cython, __pyx_t_2) < (0)) __PYX_ERR(1, 16, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "plyvel/_plyvel.pyx":521
 * 
 * 
 * def repair_db(name, *, paranoid_checks=None, write_buffer_size=None,             # <<<<<<<<<<<<<<
 *               max_open_files=None, lru_cache_size=None, block_size=None,
 *               block_restart_interval=None, max_file_size=None,
*/
  __pyx_t_2 = __Pyx_PyDict_NewPresized(11); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 521, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_paranoid_checks, Py_None) < (0)) __PYX_ERR(0, 521, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_write_buffer_size, Py_None) < (0)) __PYX_ERR(0, 521, __pyx_L1_error)

  /* "plyvel/_plyvel.pyx":522
 * 
 * def repair_db(name, *, paranoid_checks=None, write_buffer_size=None,
 *               max_open_files=None, lru_cache_size=None, block_size=None,             # <<<<<<<<<<<<<<
 *               block_restart_interval=None, max_file_size=None,
 *               compression='snappy', int bloom_filter_bits=0, comparator=None,
*/
  if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_max_open_files, Py_None) < (0)) __PYX_ERR(0, 521, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_lru_cache_size, Py_None) < (0)) __PYX_ERR(0, 521, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_block_size, Py_None) < (0)) __PYX_ERR(0, 521, __pyx_L1_error)

  /* "plyvel/_plyvel.pyx":523
 * def repair_db(name, *, paranoid_checks=None, write_buffer_size=None,
 *               max_open_files=None, lru_cache_size=None, block_size=None,
 *               block_restart_interval=None, max_file_size=None,             # <<<<<<<<<<<<<<
 *               compression='snappy', int bloom_filter_bits=0, comparator=None,
 *               bytes comparator_name=None):
*/
  if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_block_restart_interval, Py_None) < (0)) __PYX_ERR(0, 521, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_max_file_size, Py_None) < (0)) __PYX_ERR(0, 521, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_compression, ((PyObject*)__pyx_mstate_global->__pyx_n_u_snappy)) < (0)) __PYX_ERR(0, 521, __pyx_L1_error)

  /* "plyvel/_plyvel.pyx":524
 *               max_open_files=None, lru_cache_size=None, block_size=None,
 *               block_restart_interval=None, max_file_size=None,
 *               compression='snappy', int bloom_filter_bits=0, comparator=None,             # <<<<<<<<<<<<<<
 *               bytes comparator_name=None):
 *     cdef Options options = Options()
*/
  __pyx_t_7 = __Pyx_PyLong_From_int(((int)0)); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 524, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_bloom_filter_bits, __pyx_t_7) < (0)) __PYX_ERR(0, 521, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_comparator, Py_None) < (0)) __PYX_ERR(0, 521, __pyx_L1_error)

  /* "plyvel/_plyvel.pyx":525
 *               block_restart_interval=None, max_file_size=None,
 *               compression='snappy', int bloom_filter_bits=0, comparator=None,
 *               bytes comparator_name=None):             # <<<<<<<<<<<<<<
 *     cdef Options options = Options()
 *     cdef Status st
*/
  if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_comparator_name, Py_None) < (0)) __PYX_ERR(0, 521, __pyx_L1_error)

  /* "plyvel/_plyvel.pyx":521
 * 
 * 
 * def repair_db(name, *, paranoid_checks=None, write_buffer_size=None,             # <<<<<<<<<<<<<<
 *               max_open_files=None, lru_cache_size=None, block_size=None,
 *               block_restart_interval=None, max_file_size=None,
*/
  __pyx_t_7 = __Pyx_CyFunction_New(&__pyx_mdef_6plyvel_7_plyvel_1repair_db, 0, __pyx_mstate_global->__pyx_n_u_repair_db, NULL, __pyx_mstate_global->__pyx_n_u_plyvel__plyvel, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[27])); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 521, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_7);
  #endif
  __Pyx_CyFunction_SetDefaultsKwDict(__pyx_t_7, __pyx_t_2);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_repair_db, __pyx_t_7) < (0)) __PYX_ERR(0, 521, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "plyvel/_plyvel.pyx":543
 * 
 * 
 * def destroy_db(name):             # <<<<<<<<<<<<<<
 *     cdef Options options = Options()
 *     cdef Status st
*/
  __pyx_t_7 = __Pyx_CyFunction_New(&__pyx_mdef_6plyvel_7_plyvel_3destroy_db, 0, __pyx_mstate_global->__pyx_n_u_destroy_db, NULL, __pyx_mstate_global->__pyx_n_u_plyvel__plyvel, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[28])); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 543, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_7);
  #endif
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_destroy_db, __pyx_t_7) < (0)) __PYX_ERR(0, 543, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "plyvel/_plyvel.pyx":580
 *         del self._write_batch
 * 
 *     def put(self, bytes key not None, value not None):             # <<<<<<<<<<<<<<
 *         if self.db._db is NULL:
 *             raise RuntimeError("Database is closed")
*/
  __pyx_t_7 = __Pyx_CyFunction_New(&__pyx_mdef_6plyvel_7_plyvel_10WriteBatch_5put, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_WriteBatch_put, NULL, __pyx_mstate_global->__pyx_n_u_plyvel__plyvel, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[29])); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 580, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_7);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6plyvel_7_plyvel_WriteBatch, __pyx_mstate_global->__pyx_n_u_put, __pyx_t_7) < (0)) __PYX_ERR(0, 580, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "plyvel/_plyvel.pyx":598
 *             PyBuffer_Release(&value_buffer)
 * 
 *     def delete(self, bytes key not None):             # <<<<<<<<<<<<<<
 *         if self.db._db is NULL:
 *             raise RuntimeError("Database is closed")
*/
  __pyx_t_7 = __Pyx_CyFunction_New(&__pyx_mdef_6plyvel_7_plyvel_10WriteBatch_7delete, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_WriteBatch_delete, NULL, __pyx_mstate_global->__pyx_n_u_plyvel__plyvel, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[30])); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 598, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_7);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6plyvel_7_plyvel_WriteBatch, __pyx_mstate_global->__pyx_n_u_delete, __pyx_t_7) < (0)) __PYX_ERR(0, 598, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "plyvel/_plyvel.pyx":609
 *             self._write_batch.Delete(key_slice)
 * 
 *     def clear(self):             # <<<<<<<<<<<<<<
 *         if self.db._db is NULL:
 *             raise RuntimeError("Database is closed")
*/
  __pyx_t_7 = __Pyx_CyFunction_New(&__pyx_mdef_6plyvel_7_plyvel_10WriteBatch_9clear, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_WriteBatch_clear, NULL, __pyx_mstate_global->__pyx_n_u_plyvel__plyvel, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[31])); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 609, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_7);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6plyvel_7_plyvel_WriteBatch, __pyx_mstate_global->__pyx_n_u_clear, __pyx_t_7) < (0)) __PYX_ERR(0, 609, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "plyvel/_plyvel.pyx":616
 *             self._write_batch.Clear()
 * 
 *     def write(self):             # <<<<<<<<<<<<<<
 *         if self.db._db is NULL:
 *             raise RuntimeError("Database is closed")
*/
  __pyx_t_7 = __Pyx_CyFunction_New(&__pyx_mdef_6plyvel_7_plyvel_10WriteBatch_11write, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_WriteBatch_write, NULL, __pyx_mstate_global->__pyx_n_u_plyvel__plyvel, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[32])); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 616, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_7);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6plyvel_7_plyvel_WriteBatch, __pyx_mstate_global->__pyx_n_u_write, __pyx_t_7) < (0)) __PYX_ERR(0, 616, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "plyvel/_plyvel.pyx":625
 *         raise_for_status(st)
 * 
 *     def approximate_size(self):             # <<<<<<<<<<<<<<
 *         if self.db._db is NULL:
 *             raise RuntimeError("Database is closed")
*/
  __pyx_t_7 = __Pyx_CyFunction_New(&__pyx_mdef_6plyvel_7_plyvel_10WriteBatch_13approximate_size, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_WriteBatch_approximate_size, NULL, __pyx_mstate_global->__pyx_n_u_plyvel__plyvel, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[33])); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 625, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_7);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6plyvel_7_plyvel_WriteBatch, __pyx_mstate_global->__pyx_n_u_approximate_size, __pyx_t_7) < (0)) __PYX_ERR(0, 625, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "plyvel/_plyvel.pyx":631
 *         return self._write_batch.ApproximateSize()
 * 
 *     def append(self, WriteBatch source not None):             # <<<<<<<<<<<<<<
 *         if self.db._db is NULL:
 *             raise RuntimeError("Database is closed")
*/
  __pyx_t_7 = __Pyx_CyFunction_New(&__pyx_mdef_6plyvel_7_plyvel_10WriteBatch_15append, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_WriteBatch_append, NULL, __pyx_mstate_global->__pyx_n_u_plyvel__plyvel, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[34])); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 631, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_7);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6plyvel_7_plyvel_WriteBatch, __pyx_mstate_global->__pyx_n_u_append, __pyx_t_7) < (0)) __PYX_ERR(0, 631, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "plyvel/_plyvel.pyx":637
 *         self._write_batch.Append(source._write_batch[0])
 * 
 *     def __enter__(self):             # <<<<<<<<<<<<<<
 *         if self.db._db is NULL:
 *             raise RuntimeError("Database is closed")
*/
  __pyx_t_7 = __Pyx_CyFunction_New(&__pyx_mdef_6plyvel_7_plyvel_10WriteBatch_17__enter__, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_WriteBatch___enter, NULL, __pyx_mstate_global->__pyx_n_u_plyvel__plyvel, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[35])); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 637, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_7);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6plyvel_7_plyvel_WriteBatch, __pyx_mstate_global->__pyx_n_u_enter, __pyx_t_7) < (0)) __PYX_ERR(0, 637, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "plyvel/_plyvel.pyx":643
 *         return self
 * 
 *     def __exit__(self, exc_type, exc_val, exc_tb):             # <<<<<<<<<<<<<<
 *         if self.db._db is NULL:
 *             raise RuntimeError("Database is closed")
*/
  __pyx_t_7 = __Pyx_CyFunction_New(&__pyx_mdef_6plyvel_7_plyvel_10WriteBatch_19__exit__, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_WriteBatch___exit, NULL, __pyx_mstate_global->__pyx_n_u_plyvel__plyvel, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[36])); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 643, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_7);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6plyvel_7_plyvel_WriteBatch, __pyx_mstate_global->__pyx_n_u_exit, __pyx_t_7) < (0)) __PYX_ERR(0, 643, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self._write_batch,self.write_options cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):
*/
  __pyx_t_7 = __Pyx_CyFunction_New(&__pyx_mdef_6plyvel_7_plyvel_10WriteBatch_21__reduce_cython__, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_WriteBatch___reduce_cython, NULL, __pyx_mstate_global->__pyx_n_u_plyvel__plyvel, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[37])); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_7);
  #endif
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_reduce_cython, __pyx_t_7) < (0)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError, "self._write_batch,self.write_options cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self._write_batch,self.write_options cannot be converted to a Python object for pickling"
*/
  __pyx_t_7 = __Pyx_CyFunction_New(&__pyx_mdef_6plyvel_7_plyvel_10WriteBatch_23__setstate_cython__, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_WriteBatch___setstate_cython, NULL, __pyx_mstate_global->__pyx_n_u_plyvel__plyvel, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[38])); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_7);
  #endif
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_setstate_cython, __pyx_t_7) < (0)) __PYX_ERR(1, 3, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "plyvel/_plyvel.pyx":703
 *             lambda wr: ref_dict.pop(iterator_id))
 * 
 *     cpdef close(self):             # <<<<<<<<<<<<<<
 *         if self._iter is not NULL:
 *             del self._iter
*/
  __pyx_t_7 = __Pyx_CyFunction_New(&__pyx_mdef_6plyvel_7_plyvel_12BaseIterator_3close, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_BaseIterator_close, NULL, __pyx_mstate_global->__pyx_n_u_plyvel__plyvel, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[39])); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 703, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_7);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6plyvel_7_plyvel_BaseIterator, __pyx_mstate_global->__pyx_n_u_close, __pyx_t_7) < (0)) __PYX_ERR(0, 703, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "plyvel/_plyvel.pyx":711
 *         self.close()
 * 
 *     def __enter__(self):             # <<<<<<<<<<<<<<
 *         return self
 * 
*/
  __pyx_t_7 = __Pyx_CyFunction_New(&__pyx_mdef_6plyvel_7_plyvel_12BaseIterator_7__enter__, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_BaseIterator___enter, NULL, __pyx_mstate_global->__pyx_n_u_plyvel__plyvel, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[40])); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 711, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_7);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6plyvel_7_plyvel_BaseIterator, __pyx_mstate_global->__pyx_n_u_enter, __pyx_t_7) < (0)) __PYX_ERR(0, 711, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "plyvel/_plyvel.pyx":714
 *         return self
 * 
 *     def __exit__(self, exc_type, exc_val, exc_tb):             # <<<<<<<<<<<<<<
 *         self.close()
 *         return False  # propagate exceptions
*/
  __pyx_t_7 = __Pyx_CyFunction_New(&__pyx_mdef_6plyvel_7_plyvel_12BaseIterator_9__exit__, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_BaseIterator___exit, NULL, __pyx_mstate_global->__pyx_n_u_plyvel__plyvel, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[41])); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 714, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_7);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6plyvel_7_plyvel_BaseIterator, __pyx_mstate_global->__pyx_n_u_exit, __pyx_t_7) < (0)) __PYX_ERR(0, 714, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self._iter cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):
*/
  __pyx_t_7 = __Pyx_CyFunction_New(&__pyx_mdef_6plyvel_7_plyvel_12BaseIterator_11__reduce_cython__, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_BaseIterator___reduce_cython, NULL, __pyx_mstate_global->__pyx_n_u_plyvel__plyvel, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[42])); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_7);
  #endif
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_reduce_cython, __pyx_t_7) < (0)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError, "self._iter cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self._iter cannot be converted to a Python object for pickling"
*/
  __pyx_t_7 = __Pyx_CyFunction_New(&__pyx_mdef_6plyvel_7_plyvel_12BaseIterator_13__setstate_cython__, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_BaseIterator___setstate_cython, NULL, __pyx_mstate_global->__pyx_n_u_plyvel__plyvel, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[43])); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_7);
  #endif
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_setstate_cython, __pyx_t_7) < (0)) __PYX_ERR(1, 3, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "plyvel/_plyvel.pyx":851
 *             return self.real_prev()
 * 
 *     def prev(self):             # <<<<<<<<<<<<<<
 *         if self.direction == FORWARD:
 *             return self.real_prev()
*/
  __pyx_t_7 = __Pyx_CyFunction_New(&__pyx_mdef_6plyvel_7_plyvel_8Iterator_7prev, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Iterator_prev, NULL, __pyx_mstate_global->__pyx_n_u_plyvel__plyvel, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[44])); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 851, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_7);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6plyvel_7_plyvel_Iterator, __pyx_mstate_global->__pyx_n_u_prev, __pyx_t_7) < (0)) __PYX_ERR(0, 851, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "plyvel/_plyvel.pyx":987
 *         return out
 * 
 *     def seek_to_start(self):             # <<<<<<<<<<<<<<
 *         if self._iter is NULL:
 *             raise RuntimeError("Database or iterator is closed")
*/
  __pyx_t_7 = __Pyx_CyFunction_New(&__pyx_mdef_6plyvel_7_plyvel_8Iterator_9seek_to_start, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Iterator_seek_to_start, NULL, __pyx_mstate_global->__pyx_n_u_plyvel__plyvel, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[45])); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 987, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_7);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6plyvel_7_plyvel_Iterator, __pyx_mstate_global->__pyx_n_u_seek_to_start, __pyx_t_7) < (0)) __PYX_ERR(0, 987, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "plyvel/_plyvel.pyx":993
 *         self.state = BEFORE_START
 * 
 *     def seek_to_stop(self):             # <<<<<<<<<<<<<<
 *         if self._iter is NULL:
 *             raise RuntimeError("Database or iterator is closed")
*/
  __pyx_t_7 = __Pyx_CyFunction_New(&__pyx_mdef_6plyvel_7_plyvel_8Iterator_11seek_to_stop, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Iterator_seek_to_stop, NULL, __pyx_mstate_global->__pyx_n_u_plyvel__plyvel, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[46])); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 993, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_7);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6plyvel_7_plyvel_Iterator, __pyx_mstate_global->__pyx_n_u_seek_to_stop, __pyx_t_7) < (0)) __PYX_ERR(0, 993, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "plyvel/_plyvel.pyx":999
 *         self.state = AFTER_STOP
 * 
 *     def seek(self, bytes target not None):             # <<<<<<<<<<<<<<
 *         if self._iter is NULL:
 *             raise RuntimeError("Database or iterator is closed")
*/
  __pyx_t_7 = __Pyx_CyFunction_New(&__pyx_mdef_6plyvel_7_plyvel_8Iterator_13seek, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Iterator_seek, NULL, __pyx_mstate_global->__pyx_n_u_plyvel__plyvel, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[47])); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 999, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_7);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6plyvel_7_plyvel_Iterator, __pyx_mstate_global->__pyx_n_u_seek, __pyx_t_7) < (0)) __PYX_ERR(0, 999, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self._iter,self.comparator,self.start_slice,self.stop_slice cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):
*/
  __pyx_t_7 = __Pyx_CyFunction_New(&__pyx_mdef_6plyvel_7_plyvel_8Iterator_15__reduce_cython__, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Iterator___reduce_cython, NULL, __pyx_mstate_global->__pyx_n_u_plyvel__plyvel, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[48])); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_7);
  #endif
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_reduce_cython, __pyx_t_7) < (0)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError, "self._iter,self.comparator,self.start_slice,self.stop_slice cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self._iter,self.comparator,self.start_slice,self.stop_slice cannot be converted to a Python object for pickling"
*/
  __pyx_t_7 = __Pyx_CyFunction_New(&__pyx_mdef_6plyvel_7_plyvel_8Iterator_17__setstate_cython__, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Iterator___setstate_cython, NULL, __pyx_mstate_global->__pyx_n_u_plyvel__plyvel, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[49])); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_7);
  #endif
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_setstate_cython, __pyx_t_7) < (0)) __PYX_ERR(1, 3, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "plyvel/_plyvel.pyx":1029
 * @cython.final
 * cdef class RawIterator(BaseIterator):
 *     def valid(self):             # <<<<<<<<<<<<<<
 *         if self._iter is NULL:
 *             raise RuntimeError("Database or iterator is closed")
*/
  __pyx_t_7 = __Pyx_CyFunction_New(&__pyx_mdef_6plyvel_7_plyvel_11RawIterator_1valid, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_RawIterator_valid, NULL, __pyx_mstate_global->__pyx_n_u_plyvel__plyvel, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[50])); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1029, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_7);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6plyvel_7_plyvel_RawIterator, __pyx_mstate_global->__pyx_n_u_valid, __pyx_t_7) < (0)) __PYX_ERR(0, 1029, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "plyvel/_plyvel.pyx":1035
 *         return self._iter.Valid()
 * 
 *     def seek_to_first(self):             # <<<<<<<<<<<<<<
 *         if self._iter is NULL:
 *             raise RuntimeError("Database or iterator is closed")
*/
  __pyx_t_7 = __Pyx_CyFunction_New(&__pyx_mdef_6plyvel_7_plyvel_11RawIterator_3seek_to_first, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_RawIterator_seek_to_first, NULL, __pyx_mstate_global->__pyx_n_u_plyvel__plyvel, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[51])); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1035, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_7);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6plyvel_7_plyvel_RawIterator, __pyx_mstate_global->__pyx_n_u_seek_to_first, __pyx_t_7) < (0)) __PYX_ERR(0, 1035, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "plyvel/_plyvel.pyx":1044
 *         raise_for_status(self._iter.status())
 * 
 *     def seek_to_last(self):             # <<<<<<<<<<<<<<
 *         if self._iter is NULL:
 *             raise RuntimeError("Database or iterator is closed")
*/
  __pyx_t_7 = __Pyx_CyFunction_New(&__pyx_mdef_6plyvel_7_plyvel_11RawIterator_5seek_to_last, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_RawIterator_seek_to_last, NULL, __pyx_mstate_global->__pyx_n_u_plyvel__plyvel, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[52])); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1044, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_7);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6plyvel_7_plyvel_RawIterator, __pyx_mstate_global->__pyx_n_u_seek_to_last, __pyx_t_7) < (0)) __PYX_ERR(0, 1044, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "plyvel/_plyvel.pyx":1053
 *         raise_for_status(self._iter.status())
 * 
 *     def seek(self, bytes target not None):             # <<<<<<<<<<<<<<
 *         if self._iter is NULL:
 *             raise RuntimeError("Database or iterator is closed")
*/
  __pyx_t_7 = __Pyx_CyFunction_New(&__pyx_mdef_6plyvel_7_plyvel_11RawIterator_7seek, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_RawIterator_seek, NULL, __pyx_mstate_global->__pyx_n_u_plyvel__plyvel, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[53])); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1053, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_7);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6plyvel_7_plyvel_RawIterator, __pyx_mstate_global->__pyx_n_u_seek, __pyx_t_7) < (0)) __PYX_ERR(0, 1053, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "plyvel/_plyvel.pyx":1063
 *         raise_for_status(self._iter.status())
 * 
 *     def next(self):             # <<<<<<<<<<<<<<
 *         if self._iter is NULL:
 *             raise RuntimeError("Database or iterator is closed")
*/
  __pyx_t_7 = __Pyx_CyFunction_New(&__pyx_mdef_6plyvel_7_plyvel_11RawIterator_9next, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_RawIterator_next, NULL, __pyx_mstate_global->__pyx_n_u_plyvel__plyvel, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[54])); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1063, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_7);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6plyvel_7_plyvel_RawIterator, __pyx_mstate_global->__pyx_n_u_next, __pyx_t_7) < (0)) __PYX_ERR(0, 1063, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "plyvel/_plyvel.pyx":1075
 *         raise_for_status(self._iter.status())
 * 
 *     def prev(self):             # <<<<<<<<<<<<<<
 *         if self._iter is NULL:
 *             raise RuntimeError("Database or iterator is closed")
*/
  __pyx_t_7 = __Pyx_CyFunction_New(&__pyx_mdef_6plyvel_7_plyvel_11RawIterator_11prev, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_RawIterator_prev, NULL, __pyx_mstate_global->__pyx_n_u_plyvel__plyvel, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[55])); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1075, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_7);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6plyvel_7_plyvel_RawIterator, __pyx_mstate_global->__pyx_n_u_prev, __pyx_t_7) < (0)) __PYX_ERR(0, 1075, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "plyvel/_plyvel.pyx":1087
 *         raise_for_status(self._iter.status())
 * 
 *     cpdef key(self):             # <<<<<<<<<<<<<<
 *         if self._iter is NULL:
 *             raise RuntimeError("Database or iterator is closed")
*/
  __pyx_t_7 = __Pyx_CyFunction_New(&__pyx_mdef_6plyvel_7_plyvel_11RawIterator_13key, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_RawIterator_key, NULL, __pyx_mstate_global->__pyx_n_u_plyvel__plyvel, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[56])); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1087, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_7);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6plyvel_7_plyvel_RawIterator, __pyx_mstate_global->__pyx_n_u_key, __pyx_t_7) < (0)) __PYX_ERR(0, 1087, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "plyvel/_plyvel.pyx":1098
 *         return key_slice.data()[:key_slice.size()]
 * 
 *     cpdef value(self):             # <<<<<<<<<<<<<<
 *         if self._iter is NULL:
 *             raise RuntimeError("Database or iterator is closed")
*/
  __pyx_t_7 = __Pyx_CyFunction_New(&__pyx_mdef_6plyvel_7_plyvel_11RawIterator_15value, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_RawIterator_value, NULL, __pyx_mstate_global->__pyx_n_u_plyvel__plyvel, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[57])); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1098, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_7);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6plyvel_7_plyvel_RawIterator, __pyx_mstate_global->__pyx_n_u_value, __pyx_t_7) < (0)) __PYX_ERR(0, 1098, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "plyvel/_plyvel.pyx":1109
 *         return value_slice.data()[:value_slice.size()]
 * 
 *     def item(self):             # <<<<<<<<<<<<<<
 *         return self.key(), self.value()
 * 
*/
  __pyx_t_7 = __Pyx_CyFunction_New(&__pyx_mdef_6plyvel_7_plyvel_11RawIterator_17item, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_RawIterator_item, NULL, __pyx_mstate_global->__pyx_n_u_plyvel__plyvel, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[58])); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1109, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_7);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6plyvel_7_plyvel_RawIterator, __pyx_mstate_global->__pyx_n_u_item, __pyx_t_7) < (0)) __PYX_ERR(0, 1109, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self._iter cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):
*/
  __pyx_t_7 = __Pyx_CyFunction_New(&__pyx_mdef_6plyvel_7_plyvel_11RawIterator_19__reduce_cython__, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_RawIterator___reduce_cython, NULL, __pyx_mstate_global->__pyx_n_u_plyvel__plyvel, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[59])); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_7);
  #endif
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_reduce_cython, __pyx_t_7) < (0)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError, "self._iter cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self._iter cannot be converted to a Python object for pickling"
*/
  __pyx_t_7 = __Pyx_CyFunction_New(&__pyx_mdef_6plyvel_7_plyvel_11RawIterator_21__setstate_cython__, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_RawIterator___setstate_cython, NULL, __pyx_mstate_global->__pyx_n_u_plyvel__plyvel, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[60])); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_7);
  #endif
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_setstate_cython, __pyx_t_7) < (0)) __PYX_ERR(1, 3, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "plyvel/_plyvel.pyx":1135
 *         self.close()
 * 
 *     cpdef close(self):             # <<<<<<<<<<<<<<
 *         if self.db._db is NULL or self._snapshot is NULL:
 *             return  # nothing to do
*/
  __pyx_t_7 = __Pyx_CyFunction_New(&__pyx_mdef_6plyvel_7_plyvel_8Snapshot_5close, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Snapshot_close, NULL, __pyx_mstate_global->__pyx_n_u_plyvel__plyvel, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[61])); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1135, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_7);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6plyvel_7_plyvel_Snapshot, __pyx_mstate_global->__pyx_n_u_close, __pyx_t_7) < (0)) __PYX_ERR(0, 1135, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "plyvel/_plyvel.pyx":1143
 *             self._snapshot = NULL
 * 
 *     def release(self):             # <<<<<<<<<<<<<<
 *         self.close()
 * 
*/
  __pyx_t_7 = __Pyx_CyFunction_New(&__pyx_mdef_6plyvel_7_plyvel_8Snapshot_7release, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Snapshot_release, NULL, __pyx_mstate_global->__pyx_n_u_plyvel__plyvel, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[62])); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1143, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_7);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6plyvel_7_plyvel_Snapshot, __pyx_mstate_global->__pyx_n_u_release, __pyx_t_7) < (0)) __PYX_ERR(0, 1143, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "plyvel/_plyvel.pyx":1146
 *         self.close()
 * 
 *     def __enter__(self):             # <<<<<<<<<<<<<<
 *         return self
 * 
*/
  __pyx_t_7 = __Pyx_CyFunction_New(&__pyx_mdef_6plyvel_7_plyvel_8Snapshot_9__enter__, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Snapshot___enter, NULL, __pyx_mstate_global->__pyx_n_u_plyvel__plyvel, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[63])); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1146, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_7);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6plyvel_7_plyvel_Snapshot, __pyx_mstate_global->__pyx_n_u_enter, __pyx_t_7) < (0)) __PYX_ERR(0, 1146, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "plyvel/_plyvel.pyx":1149
 *         return self
 * 
 *     def __exit__(self, exc_type, exc_val, exc_tb):             # <<<<<<<<<<<<<<
 *         self.close()
 *         return False  # propagate exceptions
*/
  __pyx_t_7 = __Pyx_CyFunction_New(&__pyx_mdef_6plyvel_7_plyvel_8Snapshot_11__exit__, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Snapshot___exit, NULL, __pyx_mstate_global->__pyx_n_u_plyvel__plyvel, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[64])); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1149, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_7);
  #endif
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6plyvel_7_plyvel_Snapshot, __pyx_mstate_global->__pyx_n_u_exit, __pyx_t_7) < (0)) __PYX_ERR(0, 1149, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "plyvel/_plyvel.pyx":1153
 *         return False  # propagate exceptions
 * 
 *     def get(self, bytes key not None, default=None, *,             # <<<<<<<<<<<<<<
 *             bool verify_checksums=False, bool fill_cache=True):
 *         if self.db._db is NULL or self._snapshot is NULL:
*/
  __pyx_t_7 = __Pyx_PyDict_NewPresized(2); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1153, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);

  /* "plyvel/_plyvel.pyx":1154
 * 
 *     def get(self, bytes key not None, default=None, *,
 *             bool verify_checksums=False, bool fill_cache=True):             # <<<<<<<<<<<<<<
 *         if self.db._db is NULL or self._snapshot is NULL:
 *             raise RuntimeError("Database or snapshot is closed")
*/
  if (PyDict_SetItem(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_verify_checksums, Py_False) < (0)) __PYX_ERR(0, 1153, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_fill_cache, Py_True) < (0)) __PYX_ERR(0, 1153, __pyx_L1_error)

  /* "plyvel/_plyvel.pyx":1153
 *         return False  # propagate exceptions
 * 
 *     def get(self, bytes key not None, default=None, *,             # <<<<<<<<<<<<<<
 *             bool verify_checksums=False, bool fill_cache=True):
 *         if self.db._db is NULL or self._snapshot is NULL:
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6plyvel_7_plyvel_8Snapshot_13get, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Snapshot_get, NULL, __pyx_mstate_global->__pyx_n_u_plyvel__plyvel, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[65])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1153, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  __Pyx_CyFunction_SetDefaultsTuple(__pyx_t_2, __pyx_mstate_global->__pyx_tuple[2]);
  __Pyx_CyFunction_SetDefaultsKwDict(__pyx_t_2, __pyx_t_7);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6plyvel_7_plyvel_Snapshot, __pyx_mstate_global->__pyx_n_u_get, __pyx_t_2) < (0)) __PYX_ERR(0, 1153, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "plyvel/_plyvel.pyx":1171
 *         return self.iterator()
 * 
 *     def iterator(self, *, reverse=False, start=None, stop=None,             # <<<<<<<<<<<<<<
 *                  include_start=True, include_stop=False, prefix=None,
 *                  include_key=True, include_value=True,
*/
  __pyx_t_2 = __Pyx_PyDict_NewPresized(10); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1171, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_reverse, Py_False) < (0)) __PYX_ERR(0, 1171, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_start, Py_None) < (0)) __PYX_ERR(0, 1171, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_stop, Py_None) < (0)) __PYX_ERR(0, 1171, __pyx_L1_error)

  /* "plyvel/_plyvel.pyx":1172
 * 
 *     def iterator(self, *, reverse=False, start=None, stop=None,
 *                  include_start=True, include_stop=False, prefix=None,             # <<<<<<<<<<<<<<
 *                  include_key=True, include_value=True,
 *                  bool verify_checksums=False, bool fill_cache=True):
*/
  if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_include_start, Py_True) < (0)) __PYX_ERR(0, 1171, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_include_stop, Py_False) < (0)) __PYX_ERR(0, 1171, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_prefix, Py_None) < (0)) __PYX_ERR(0, 1171, __pyx_L1_error)

  /* "plyvel/_plyvel.pyx":1173
 *     def iterator(self, *, reverse=False, start=None, stop=None,
 *                  include_start=True, include_stop=False, prefix=None,
 *                  include_key=True, include_value=True,             # <<<<<<<<<<<<<<
 *                  bool verify_checksums=False, bool fill_cache=True):
 *         if self.db._db is NULL or self._snapshot is NULL:
*/
  if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_include_key, Py_True) < (0)) __PYX_ERR(0, 1171, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_include_value, Py_True) < (0)) __PYX_ERR(0, 1171, __pyx_L1_error)

  /* "plyvel/_plyvel.pyx":1174
 *                  include_start=True, include_stop=False, prefix=None,
 *                  include_key=True, include_value=True,
 *                  bool verify_checksums=False, bool fill_cache=True):             # <<<<<<<<<<<<<<
 *         if self.db._db is NULL or self._snapshot is NULL:
 *             raise RuntimeError("Database or snapshot is closed")
*/
  if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_verify_checksums, Py_False) < (0)) __PYX_ERR(0, 1171, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_2, __pyx_mstate_global->__pyx_n_u_fill_cache, Py_True) < (0)) __PYX_ERR(0, 1171, __pyx_L1_error)

  /* "plyvel/_plyvel.pyx":1171
 *         return self.iterator()
 * 
 *     def iterator(self, *, reverse=False, start=None, stop=None,             # <<<<<<<<<<<<<<
 *                  include_start=True, include_stop=False, prefix=None,
 *                  include_key=True, include_value=True,
*/
  __pyx_t_7 = __Pyx_CyFunction_New(&__pyx_mdef_6plyvel_7_plyvel_8Snapshot_17iterator, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Snapshot_iterator, NULL, __pyx_mstate_global->__pyx_n_u_plyvel__plyvel, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[66])); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1171, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_7);
  #endif
  __Pyx_CyFunction_SetDefaultsKwDict(__pyx_t_7, __pyx_t_2);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6plyvel_7_plyvel_Snapshot, __pyx_mstate_global->__pyx_n_u_iterator, __pyx_t_7) < (0)) __PYX_ERR(0, 1171, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;

  /* "plyvel/_plyvel.pyx":1185
 *             fill_cache=fill_cache, snapshot=self)
 * 
 *     def raw_iterator(self, *, bool verify_checksums=False,             # <<<<<<<<<<<<<<
 *                      bool fill_cache=True):
 *         if self.db._db is NULL or self._snapshot is NULL:
*/
  __pyx_t_7 = __Pyx_PyDict_NewPresized(2); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 1185, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_7);
  if (PyDict_SetItem(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_verify_checksums, Py_False) < (0)) __PYX_ERR(0, 1185, __pyx_L1_error)

  /* "plyvel/_plyvel.pyx":1186
 * 
 *     def raw_iterator(self, *, bool verify_checksums=False,
 *                      bool fill_cache=True):             # <<<<<<<<<<<<<<
 *         if self.db._db is NULL or self._snapshot is NULL:
 *             raise RuntimeError("Database or snapshot is closed")
*/
  if (PyDict_SetItem(__pyx_t_7, __pyx_mstate_global->__pyx_n_u_fill_cache, Py_True) < (0)) __PYX_ERR(0, 1185, __pyx_L1_error)

  /* "plyvel/_plyvel.pyx":1185
 *             fill_cache=fill_cache, snapshot=self)
 * 
 *     def raw_iterator(self, *, bool verify_checksums=False,             # <<<<<<<<<<<<<<
 *                      bool fill_cache=True):
 *         if self.db._db is NULL or self._snapshot is NULL:
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6plyvel_7_plyvel_8Snapshot_19raw_iterator, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Snapshot_raw_iterator, NULL, __pyx_mstate_global->__pyx_n_u_plyvel__plyvel, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[67])); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1185, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  __Pyx_CyFunction_SetDefaultsKwDict(__pyx_t_2, __pyx_t_7);
  __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
  if (__Pyx_SetItemOnTypeDict(__pyx_mstate_global->__pyx_ptype_6plyvel_7_plyvel_Snapshot, __pyx_mstate_global->__pyx_n_u_raw_iterator, __pyx_t_2) < (0)) __PYX_ERR(0, 1185, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self._snapshot cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6plyvel_7_plyvel_8Snapshot_21__reduce_cython__, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Snapshot___reduce_cython, NULL, __pyx_mstate_global->__pyx_n_u_plyvel__plyvel, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[68])); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_reduce_cython, __pyx_t_2) < (0)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":3
 * def __reduce_cython__(self):
 *     raise TypeError, "self._snapshot cannot be converted to a Python object for pickling"
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     raise TypeError, "self._snapshot cannot be converted to a Python object for pickling"
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6plyvel_7_plyvel_8Snapshot_23__setstate_cython__, __Pyx_CYFUNCTION_CCLASS, __pyx_mstate_global->__pyx_n_u_Snapshot___setstate_cython, NULL, __pyx_mstate_global->__pyx_n_u_plyvel__plyvel, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[69])); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_setstate_cython, __pyx_t_2) < (0)) __PYX_ERR(1, 3, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":4
 *     int __Pyx_CheckUnpickleChecksum(long, long, long, long, const char*) except -1
 *     int __Pyx_UpdateUnpickledDict(object, object, Py_ssize_t) except -1
 * def __pyx_unpickle_PrefixedDB(__pyx_type, long __pyx_checksum, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_result
 *     __Pyx_CheckUnpickleChecksum(__pyx_checksum, 0xe9c0b3a, 0x8ba9a5d, 0x11ea7f1, b'db, prefix')
*/
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_6plyvel_7_plyvel_5__pyx_unpickle_PrefixedDB, 0, __pyx_mstate_global->__pyx_n_u_pyx_unpickle_PrefixedDB, NULL, __pyx_mstate_global->__pyx_n_u_plyvel__plyvel, __pyx_mstate_global->__pyx_d, ((PyObject *)__pyx_mstate_global->__pyx_codeobj_tab[70])); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030E0000
  PyUnstable_Object_EnableDeferredRefcount(__pyx_t_2);
  #endif
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_pyx_unpickle_PrefixedDB, __pyx_t_2) < (0)) __PYX_ERR(1, 4, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "plyvel/_plyvel.pyx":1
 * # cython: embedsignature=True, language_level=3             # <<<<<<<<<<<<<<
 * 
 * #
*/
  __pyx_t_2 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_mstate_global->__pyx_d, __pyx_mstate_global->__pyx_n_u_test, __pyx_t_2) < (0)) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /*--- Wrapped vars code ---*/

  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  if (__pyx_m) {
    if (__pyx_mstate->__pyx_d && stringtab_initialized) {
      __Pyx_AddTraceback("init plyvel._plyvel", __pyx_clineno, __pyx_lineno, __pyx_filename);
    }
    #if !CYTHON_USE_MODULE_STATE
    Py_CLEAR(__pyx_m);
    #else
    Py_DECREF(__pyx_m);
    if (pystate_addmodule_run) {
      PyObject *tp, *value, *tb;
      PyErr_Fetch(&tp, &value, &tb);
      PyState_RemoveModule(&__pyx_moduledef);
      PyErr_Restore(tp, value, tb);
    }
    #endif
  } else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_ImportError, "init plyvel._plyvel");
  }
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  return (__pyx_m != NULL) ? 0 : -1;
  #else
  return __pyx_m;
  #endif
}
/* #### Code section: pystring_table ### */
/* #### Code section: cached_builtins ### */

static int __Pyx_InitCachedBuiltins(__pyx_mstatetype *__pyx_mstate) {
  CYTHON_UNUSED_VAR(__pyx_mstate);
  __pyx_builtin_id = __Pyx_GetBuiltinName(__pyx_mstate->__pyx_n_u_id); if (!__pyx_builtin_id) __PYX_ERR(0, 298, __pyx_L1_error)
  __pyx_builtin_super = __Pyx_GetBuiltinName(__pyx_mstate->__pyx_n_u_super); if (!__pyx_builtin_super) __PYX_ERR(0, 741, __pyx_L1_error)

  /* Cached unbound methods */
  __pyx_mstate->__pyx_umethod_PyDict_Type_items.type = (PyObject*)&PyDict_Type;
  __pyx_mstate->__pyx_umethod_PyDict_Type_items.method_name = &__pyx_mstate->__pyx_n_u_items;
  __pyx_mstate->__pyx_umethod_PyDict_Type_pop.type = (PyObject*)&PyDict_Type;
  __pyx_mstate->__pyx_umethod_PyDict_Type_pop.method_name = &__pyx_mstate->__pyx_n_u_pop;
  __pyx_mstate->__pyx_umethod_PyDict_Type_values.type = (PyObject*)&PyDict_Type;
  __pyx_mstate->__pyx_umethod_PyDict_Type_values.method_name = &__pyx_mstate->__pyx_n_u_values;
  return 0;
  __pyx_L1_error:;
  return -1;
}
/* #### Code section: cached_constants ### */

static int __Pyx_InitCachedConstants(__pyx_mstatetype *__pyx_mstate) {
  __Pyx_RefNannyDeclarations
  CYTHON_UNUSED_VAR(__pyx_mstate);
  __Pyx_RefNannySetupContext("__Pyx_InitCachedConstants", 0);

  /* "plyvel/_plyvel.pyx":263
 *         cdef BaseIterator iterator
 *         if self.iterators is not None:
 *             with self.lock:             # <<<<<<<<<<<<<<
 *                 while self.iterators:
 *                     iterator = self.iterators.popitem()[1]()
*/
  __pyx_mstate_global->__pyx_tuple[0] = PyTuple_Pack(3, Py_None, Py_None, Py_None); if (unlikely(!__pyx_mstate_global->__pyx_tuple[0])) __PYX_ERR(0, 263, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_mstate_global->__pyx_tuple[0]);
  __Pyx_GIVEREF(__pyx_mstate_global->__pyx_tuple[0]);

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = ('self.db is not None or self.prefix is not None',)             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_PrefixedDB, (type(self), 0xe9c0b3a, None), state
*/
  __pyx_mstate_global->__pyx_tuple[1] = PyTuple_Pack(1, __pyx_mstate_global->__pyx_kp_u_self_db_is_not_None_or_self_pref); if (unlikely(!__pyx_mstate_global->__pyx_tuple[1])) __PYX_ERR(1, 11, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_mstate_global->__pyx_tuple[1]);
  __Pyx_GIVEREF(__pyx_mstate_global->__pyx_tuple[1]);

  /* "plyvel/_plyvel.pyx":301
 *         )
 * 
 *     def get(self, bytes key not None, default=None, *,             # <<<<<<<<<<<<<<
 *             bool verify_checksums=False, bool fill_cache=True):
 *         if self._db is NULL:
*/
  __pyx_mstate_global->__pyx_tuple[2] = PyTuple_Pack(1, Py_None); if (unlikely(!__pyx_mstate_global->__pyx_tuple[2])) __PYX_ERR(0, 301, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_mstate_global->__pyx_tuple[2]);
  __Pyx_GIVEREF(__pyx_mstate_global->__pyx_tuple[2]);
  #if CYTHON_IMMORTAL_CONSTANTS
  {
    PyObject **table = __pyx_mstate->__pyx_tuple;
    for (Py_ssize_t i=0; i<3; ++i) {
      #if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
      Py_SET_REFCNT(table[i], _Py_IMMORTAL_REFCNT_LOCAL);
      #else
      Py_SET_REFCNT(table[i], _Py_IMMORTAL_INITIAL_REFCNT);
      #endif
    }
  }
  #endif
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}
/* #### Code section: init_constants ### */

static int __Pyx_InitConstants(__pyx_mstatetype *__pyx_mstate) {
  CYTHON_UNUSED_VAR(__pyx_mstate);
  {
    const struct { const unsigned int length: 8; } index[] = {{0},{55},{41},{18},{30},{30},{4},{179},{44},{5},{1},{1},{1},{8},{6},{9},{61},{38},{38},{41},{7},{6},{2},{9},{52},{21},{31},{18},{55},{73},{62},{111},{66},{88},{46},{14},{12},{22},{21},{30},{32},{18},{15},{2},{12},{11},{20},{22},{19},{20},{8},{16},{9},{6},{15},{11},{14},{6},{15},{11},{14},{5},{7},{8},{20},{26},{28},{13},{13},{22},{21},{4},{10},{28},{30},{17},{14},{19},{22},{14},{19},{22},{20},{11},{29},{31},{16},{15},{16},{16},{16},{25},{24},{17},{17},{8},{18},{17},{26},{28},{14},{12},{17},{21},{16},{10},{20},{19},{28},{30},{17},{27},{16},{17},{14},{16},{6},{16},{17},{5},{18},{22},{10},{17},{8},{5},{18},{5},{6},{13},{10},{15},{11},{17},{1},{2},{9},{6},{7},{6},{10},{8},{5},{7},{6},{9},{15},{6},{8},{7},{8},{10},{6},{8},{3},{12},{21},{12},{1},{2},{11},{13},{12},{13},{8},{26},{13},{4},{5},{8},{3},{9},{8},{19},{14},{8},{13},{14},{13},{10},{15},{8},{4},{8},{7},{4},{7},{15},{14},{3},{7},{6},{11},{11},{4},{3},{14},{12},{11},{10},{25},{14},{12},{6},{12},{12},{10},{17},{13},{3},{7},{9},{6},{7},{4},{13},{12},{13},{12},{4},{12},{10},{12},{19},{5},{2},{6},{8},{6},{2},{5},{11},{5},{4},{10},{5},{4},{3},{6},{12},{8},{9},{11},{6},{12},{5},{5},{12},{6},{16},{7},{11},{2},{5},{11},{17},{13},{127},{72},{76},{27},{69},{74},{28},{42},{31},{195},{63},{116},{75},{41},{58},{36},{29},{36},{54},{98},{212},{68},{40},{196},{73},{73},{68},{29},{47},{47},{64},{33},{27},{9},{14},{7},{30},{17},{13},{21},{25},{17},{123},{10},{9},{125},{95},{107},{62},{45},{11},{55},{43},{27}};
    #if (CYTHON_COMPRESS_STRINGS) == 3 && __PYX_LIMITED_VERSION_HEX >= 0x030e0000 /* compression: zstd (3007 bytes) */
const char* const cstring = "(\265/\375`\005\033\255]\000*l\310\026G\000\261\270\001\374?\367\257\343kW\327\267\216\257\341\276\356\353\276\356\253\244\207\312\227\324dFlw\271\266\300Z\256\234\311\311;\274\276\315x\311\377?\003\345\374\247\n\207E|\270(\310\000\321\036\2142\260\\\317\010\247\341!5B\001I\001k\001\345\214Y\226c\314\336X\177\035\376L\020\227;\317\272N\304]\207\252\301\372\302\274\340\2574G\027\017\324\371\366\365\272R\206\277\034\352.t}\231d\215\320\344\330\342\201\035\212\025b}:i\363\263\247\323\354$\273\007\226\234e\026.GX\235\235e\2503$]n\253\274\240\316_K\304:\2038\364n\030\336\317\033\255}\215\337\351$OWk\\\226\270\225Y\371\303\244\356N\227|\215\337i\205\201\235A\020;\020\274\033P/\026\034\320\340\224\320x\237\236\305\236\335\332|Y\334\022oI\216[\357\362n\022(\254\303K\203c\271\314\240=\265\304\334\211c\336wcmEL\202\030\213\326\034O\313\252\261(\215\311s[1\3570\233\236e\215Vc\255o\370y\246\266z\177\230\005.-(^\253\262\240r0bf\204\025\277\\nm\317\320\313!\006\305Qg\2371v_\235=\332`q\264\331\243\216&\257\250w\266\345\250\263\225\301\372zw_\320\302\310\2048\206AX\020\3531\374\2548Z\274o\211\311Q\233,\006\373\302[\237\332\3420\263\354\265\343\266'\374\272\342\311bp\374\262h\332]\342\2308~v\217\242xc\0021x\t\337B\353\335\341\3309\321\331\317\003\372\004\276i!\241\025\306g\264\020P\r\326oo\341#\241u\205\222\240.\257\342u7u\315M\\\323T4\002\240\365\246/\247uCS\007V\317\016\n\215\220\262\226R\027\337\201O\360Q\313\316u\261L\237\254\254K\314b\231\036\326&H\232\337\325[\247\353Nmn\035\352\016\343\262\314\254\031\356:0\274&\314\273;\206E34_\346\354\214\341\216\034\311N\273\276\016\314\243\270g2,\346r,qh\202\343\335X\21331\337W\327\231e\330\351N\347\356\345\202\336\353\276\350\360\272\240Dr<\261\313\014\277\234;,\232\232\205u\276\373u!&Oow\337\014\353\255\3617\202`\327\235\343\016\315\313\302 \006\301\316\003=\020l\211\371\325\2612.?\320\365]\017\177\267\334]\370\351O\357\354\205\2571\207\236\306b\336\346X\216\037j\314\237x\352\233\363\247o\371\201\"\2069\201\365%\201\315\033n]f\226xb""\327KvoX\370f\317\260M\226\231\235\246\353\033A\220c7\356\316\205\261\216\2019\2741QDM'\273\245I\236\273$\273\215\273\375i\334\"\353W\236\215%\214\014\214L9\226\001/\316\177?\307?WOL7\200zP\r\333g\373\236O\302\307F\375\244\203OH\375\322\3270O'q\n\253\324\205EeY\375\317\207\371\023IU\361\215\017\017\n\301\252\032\370\206\245\274\211T\272\321\n'UE\323?\235D\010\010\000\216\032\350\n\205\341KBp'\372\307]u\003\241\332\335iIi\320\320G\365\255\237\210\326\221h\345\274\310\207\030\331V\017\320\213/\365\002\036\357u\035\256r\273l-q\032Z\010T\273\234\203\213\204j\036\224\364=\274L\207Q*}\033\017\3522\\$\302\001\245\312j\245_a\373+&*\367[^\206\017\351\240\356\341w\3600`\274c\305?\251\336\241v=\207\267&\035\376\344\2444@\nCo\320\234\217\301\357\236\"\002y\211\310\272\235B\036\000\006\324X\315\250m\276\005Wq]\301j\006\307W\\\364Ya}\306/GA\373\000\255\003<\326\315~\250 \002J\276BG\371\254\236\031\027\035\2413?\301g\340\301\316\365\026\227\260uO7\001\255\0364\305{<\251\343p\021\310\307z?\367\025\374\022~\345\315\236\002\252q=\207\227x\000\000\325\210\341&J\341\243r|\304\365\003\2415\265\300T\352.R4>\250T\225\207\213|\254\376\005\377\202\223B\030V\324\007\275\200w\351\0268Ih\335vB\240\235\017\\\372\201\236^\345qWYA_\260t\003JM\321\010\320@\245+z\005\376\202\3639\035\003'\371\2547\234\352\332\311\220\022\200\365\224O\355\227\374\000\377\204\305\375\032\023\027\262\362\037\017\242\344m<\237\201H\335\322\013\tP\265\215\306i\307\005\332\201\262\204\250\260\221\031~\202\026\233\321\341[:M\007Z\3503\34284\317\345xN\n\343\207x\233\001EE1\251\246\021\365\204\306\267OI7\371\204\244\032TT\025\020\026\220\325\223ZP\240*\000q@RI\000T\216\263\320\006\354>\343\037h\033\312\250\3457H\355\304|\324T:\222\232\2408\277\325\342\253\314T\337\350\261\235\226@[\177\364K?#\225\204\254%\037n\336UK\241\370g\360#&\220\223`\337\364O|\331o8F\327/\030QMp\274f\353\036%\235\002\337\230slh\271J=\240(\037g\246\237\3401XWu\0244\000\212B\220\346\240,\037,\321[+4\205n\240 \223O\000\343Q\266\376q\031\353\313\374\2157""\221\250+\037\352\247\312\337\276\212.\032\201jxb\270.`\335\270\207c\371\221\307\331\372\014O\245\370\253\337b\242\312\324V>\3457\374\002\022\177\302\"\353\211\311\232\362+\275\200h\365\320X\260\263\355\264@U\350\306\311\263H\"\251n\202\013\340\322\010\351\n\272\241\024~\202\335\250\242\223\022fN\321\210\210\004I\n\222\244\0032\030\010\024\021\306\201\264\350\016\322\251&d!f\254\021\021\021\021\tDF\n\n\nJRI\007:\306\373Tr\245\223\344\260\212xw\235\311\037\372\374\034\255A\227\363E\255\270\306\314\215F\3615UP\351\2413\312x)lKhd\0357c\276\341\351\242\277[^.\t\265\000\007\274\210\271\300u\216\224R\230$=\333\305=1\224nm\224\263*\357\357K\253\364\253#\022\200\211?|\251\336~<\224I\0006n\320\0105\266\344\2113x\323\340\242e)\016\r\270,\207\276\226\250\313\350\316\021\217\366_\001\344\035DXx\212=_\355\243\357s\341\007Tl\225\027R\214\266\224(M\367t_\343C/+X\203B\240\215\003\276\024X9\261\314\024+'y\242\333'\300\220Pf L!\316\0056\307\272o\240K\244\005\356PF\230\205x\213\351\367\024T\014\226\300\337\340\355\374\016\250\014\317\002\266\207\000\3653\024\226,\026\n\230\2758^v\273\036\254\310\331\3008p[g\353\310/\016\2019(\376a\305\265\025\307\316\310\026f\277z0B\004\300\177\014R\272\273I9I\324x|x\210\205\257\230?\342A\230VMWa\240\221\2148u/\245#G(H|\313}\211X\023]\207\006\374\004\224\300\356\t\361\024\235\274\367\307S!0w\332Qw\261\023\236\321#Vs\262\203`\353\017\264\320&[6\240\316b$\371\361\307^\207P\t\000\005eb\350\013\274\2532\361Cs\240 T\337(\031\027r\350\273\001\373~\302\033|\232\275\347\242\356.o\271\224\252R\025~\t@\223F\237~\251\337\236\322\227\022\000M!=\254\300\326\030N\352e\250\361\377u\261\"\372D\201[\276\205.\030\367\262\310O\311\"\252W\252\3545\000\372R\343>K2\360h\352aF\034>\327RH\315\206\355\250\002C\277\204#\340|/\006\r\245\242\333B0\013a\303\262\322\302\305d\240\333\017\034\363\266\tj\n\337Q\330\267>&\320\252\031\332\3032L\277\005I\324@\031}\2461G 3p\013\033\237\242V\200y\347oRb\241\005\351o~i[\335S\231\246\0303\031\027\rc!\346\024w\300\372i\002\002$\363\216\374""\314\016\324\370\321P\350\251P\262<:\266\260J\202Q\373\224@\357\204}\304\005\026\2453\257'\245'\016\262o\2718ZZ1\340\200@A\330\341tPX!\214\234a?\233$y\025\277\313\243#\324\322$\325\344\344b\343\210\017B\271\377\300\247\277\007\230\207\300\254\020\r\327`H\020\027G\\:fh\242\240\263\371\352\305\304\300\237h!\250l\024\315\221\303\374|\351\272Vu\3317\301\232[\214n\014\250\240\300\213\262n\311\rI\346\006\333\334\200\360:&tI\027G\352-\034{\034\232u\372\032d\031U\2654TI\344\036\021e\030Nk\206\004E\032\030\256)\234\212J-\206\246h\n&\001\366\214\301\005\331\271\346 \337y]\325\027\243\374\372\004\255^\\\253\002\035\t\"y.\000\360\017\264R\033\344O\\\367_N\356\346\365\350\341\200?`Y\221\320\003\034\367d\252\234\002\361\342=h\316\366\261\026\004}q\251;\001\205\225e\333\340\006\217\000r\235i\272c\313\n\275mC\377\206\225\036\3500\372\312\360$\016\220b\343\201o\236f{\320\213\020\223,\241\037\007\337\251\003<7e\335\324\213\017\255[\217\344\014\205\344\232\207\220$\334\215B>\241\357\002\215u\370\311&\364\361gI\312\336N\213r\325\376\243\307\323\220\226d\357\352c\261\036\tq1j\241\275'a*\234\307\001\336Y\002\200m\260k\006M5\201\264H\306\013\"\032\344T\032O\032\002\341\307\203\350qmfh\036\260.\321a\306|\362\262\305D)7=[p`\007\310l\345\337p\253\3611*\371k\014d4\033aE\004\224\t\341_+r~V\rn 0\230A\252\017T\022\276\373d\177\240<\336\246\243\233\037g8\2454r7\002^\325C\005^HvN\257\262\217^\330\256\0253\247\231\3607\203\251\006HB\016\355\306\322\324\032]I\331.\017\303\021\263\riz\351m\0060\362'\200\3574z\316\010\206\360\211\231\2157\002?\274\370\234\032\025\000}Fl\244\265=\214<\252\317J\250\266\025\214W\366\254\2068Y\311\370\233\301\025]x]\025\264\330\272\214\200\r\"\363\203(^W\324l\n\005$\372\\6\334\263\005x#3\211q\032BmT|Cs\005-\252\361\275=@\3264\035\352\330\214\033\255\330Yd2\227\327\253\324V@h\262\267,o\210\271f\370\3248\031Yb\0332\033\221\300{EJkA{\242\267\262\336\371\034\361b\233_\366\256\337\206\024j\n\262\246\337\221\221\351\221\024BO\230`k\314\3253e`\363\305\022 m\306\3129\346\237L""\003H\231\372\202\003(]%tjJMq\333\346qq\315\261\\/[\r\034\010LB\350j\307}a\005\006\013\026\274\376%\014\307\316\261\224W\306\306\241\374^3\003~T\313C!_\024eV\327\233\273\314\275\2601a\243*\271\336\214\260\314\372\251\251\320Au\254\203\362\334\235Q;I\330\311 \035\003\363\224Q\002-\320l\300\022\333\222\031h\\\324%\030*1M!\014\224\003\213.<\223\361\222\221\006VNb}a\0250C\373>\220\251\221\222U\266\211\273\241\236\253:q4E0\260\233\231{\177\370\006t*\272\373f\341f\240\"(\004\005\245 V\271\2072\354\027\315\264B\301\007\010\250\2519\332\035\0163\222D_$\016\252\240\220\247\014\355\025[\246\373\330\304T\357\240+\021v\010";
    PyObject *data = __Pyx_DecompressString(cstring, 3007, 3);
    if (unlikely(!data)) __PYX_ERR(0, 1, __pyx_L1_error)
    const char* const bytes = __Pyx_PyBytes_AsString(data);
    #if !CYTHON_ASSUME_SAFE_MACROS
    if (likely(bytes)); else { Py_DECREF(data); __PYX_ERR(0, 1, __pyx_L1_error) }
    #endif
    #elif (CYTHON_COMPRESS_STRINGS) == 2 /* compression: bz2 (3057 bytes) */
const char* const cstring = "BZh91AY&SY\335[\267\313\000\003F\377\377\377\377\377\377\377\377\377\377\377\357\377\366\277\377\377\377\300@@@@@@@@@@@@\000@\000`\014_hT\323N\322V%\002\253@\000\0004\0054\234\001\365\234\203B\nMF\236\2204\000\036\2404mO\032\232M\232OLH\365@3@z\223\321\031\031\032\003\324\3654\302m'\250\321\221\246\201\221\000\004\020\250\336\215S\324\364\322\036\243&\2002d\320\320\000\000\000\000\0006\220\000\032\000h4\320E2Sjz\203\014\22354\006 h\000h\0002\032h\000\003M\031\03244\000\006\200\004\030\000&\000\010b`\230\000\0020\002d\3012`\000FF&#\001\014\000\000a\000\230\001\014\0010\023\020\300\000&L\232a\030F\230\21526\210i\200\000\t\221\2014\022$\023@\204\321\221\240L\231\031MF\310\200\000\003@\000\000h\032\000hh\320i\240\306\243\324\026W`5\r?'\236\007\217\326=t\036\300x\311\313\330\362D\023'J\224\377~I\332\025\244`\261\203 \"\300\366\237\364\023\377T\252\022\036C\337\034KZ:$\250*\342\352\3524&b\\\214\0001M\003i\014M\2466\323\033\023CM\215\246\333l\023I\2644\323\241}\344H\301\006\251\027\302\261\205\270\210\230\2221\275\000b\333@\330\206\300C\030\223\031E0\243U\304\252\253\006fP\305_\306\222T\314k\211r\010\261eV\r\266 \246\n6\030\222\034\0105R\372\325b\227\201R\003\005\335\223[\006\214\313\032\324 RTcU\340\224\334\264\006\214\022JxB\356*A\334fu\3044\010\2512\372\334U\020\\\\\001!\245\364\202\210B\002\002\325\220\3366Z\322R\"R\260\205EcFJB#WQaA0\004\322\311z\201[\216\216\373\332\344\226*$0\302\r\tb\242\301\t\262X\n\347\303f\311\333\325=\315\276 b[\212\341ab\336\251\006\220Y\311\020\342HF \005\261!\203\241\005s\037%?\243\\G\256\327\305\304:+\037\273\026\300~\020\2033\257\355QB$\275\tI$\223C\224\010\255q\315\265H3\371\\\220\\#\222\314W\221d1t\035\014\304\370\343{k\177\244O\312\352\\\341?\003%Btq\022B}\216\032\231Fu+\035J}7\004\003\216\311\226\304\027:/\357\340\001\201<\310\210\363\032P\337\304\323\253swh3i0\304\343\343\244%\323\364\315\246\362\234O7FM\351x\241\262\300Gf\310m\270u\333;\241\267\336\303O\0255\357\356\200N\030^|1\301\227un\006\006F\031\270\031\342\276\213\007U\244fff$""\263+\022Y\260RD,\022a`\317\313pQ\223tdA2D\014\235b\204\305\005\010 \212p\034+G*\337jb\322JM+\240\250\343\021!/o\267\225\004\354\320h\005\271I\020\235\276\235\315TD@\340M\005\024%{hv\223\022\202\030l\032`I\023i\267\253\003\0361\323L\355\262\007\245\243\215\330\331\330m\005n\372xu\360\335\356\212\232\326J\013\214\010Kb\341\037\307A6\2654\201\326\216c\244\2249N\222\254\334K\363\000\035]\201\331\241\"\357Kz\\\031\201\225yz\267\275\247fd2i\252C\023\177\270Z\347\010\273*\225\"\010\212\302\261\330\356\265\322\316\201!g\325o~\004E\331)p\215KM^\224\374q\301\020\324?\033^\307\004\024\251\030\230\277\006\036\3679\263<\350\031\321\303\017\243\201\3736HG,\266\324hp\211\t\304\353\357\247 \241P\342\002\360\200c\025^\305F!b\265\254P\320\332\304w\333\226\327u\032\273U\235/\304m\000\320\266\013\253(\375\372\330\005\013s\336c\000\263\230N\355\200\306\266K\245c\025P\265\233U\211\"@\202\010sT\220%{\356\221\020,\231)A\021\004\300\215\027\004\334\270\351\036\266\260\245\240;\324\235<Pv\341\017\022:\362\213r\270:\035rKW\034\3534\260\224\230ig\235\253\212\376|j\0017\006\244t\0209h\373)\022\262\340\360\246\020TK\347Q\231t\003\310\242E\"\253\\\256\207\370\260\337\251a\211\2334\310F=\314d#\004I\250\313\277~\266FZnw\270y^`\006U\234DW9[\237\255\006\361\232/\333\2667\2621\256\242\271\376\307\3038:\221m\317\340I\031\231\244sjZEp\275#\315\221\301\2237\334^\200z\022n\375i\032\367\303\234U+\245\036\275\265y\253;\357\020\3334d\221\216\304JUd\000\216c\324\005\n\207\177\275\330\320\227|\346%/\005\006N$\347\360d\252\2127!\250\310\014\375\270u\007A\221M\200\266\353\257X\254\216=\330\337R\365\245[\222\335J\245h\305\222\331\202\241\002\312\304\270%\220(\266\235:\312\027\304\027\225\3267#E=M\375\3441\250\023\016\035u\323\246N\372\300\346-T\270\230\254\263\033\202\352\200nAY\261\200^\210\250\246p\270t\370t\231\016\351\"\311K<\316\2205.\247\320\255\231!\203\346\3479\002'MB\332\033\272t\356\326\263gSG\036\374\366\222nr\271\354}s\275_tE\235R\017\3714\262\242B\255\3143R*\032\217U\326sk/a\204\331\004\225I$\222Im""\266\360\264\0303C\305\223X\036\002YP'|\262\262\276\374\363\361\212\367LQ\2666g\034\200\232i\341Y\330F\252$K>\252\\4#C)tb\221\300\224\025\013\320\234\260Z\230_\215m\273T\310\026\005\305\303K\256\233d /\306\314\243\023'Z\022\307F\364i\337\2533S\2768#z$=\270Z\311\265\275\022\t\0046\201\332\341\327\324\254\314i\216\253\367\267\344*\372\257\272\034\253\224\352\347=\266\220X=\244.#|\227F\203[\n\202$\206C'\226GF|\032\221sK\272\030\317\313\307xr\250\030\254\014,\270\304\232hh\026\324\223\212\006\215ir\212\363\301\314eCE\321\262zLa\304\205\263\245\032:\351\216\374\364\343\0062e7\242\210r\372\3663\251f\333\026\025\314\342\025\322\273\243F\373\006\362H8x\265V\363\nC\036\207wys\352\322\010\250\2169\213#Kc\001\341\301\2163\026\262\260\267^\035\263\0256\236\231r\274\2357w8\364\312\342\204\202D\005\314KP\254\253dVE\366T\021u\250Z\365a\324&\212\355\026\216I\025\213\354\207;\220\234\017\222\200CQ\025\341\026\365\325J\226\204\201\270\264\245\"\275\264q\031\233\320@\177d\014o\035\230\005\346\022\305H\222'\2033\333\214\205p\230\027\214\201\0259PR\226Te\350\271\307K\027\030\023\335\000`.}\356\266p\243\236\205M/\n$O\260X\246\222m\235hb\230\257\271\033\327fK\214f\375\010t\010o\270\210!!Tm\266\r\205\207\341\274F\2003m\260L\3168x\213\300\324\215\343d\340Q\237\r\324u\3269\262X\325\022\223\347.\327Gm\034E\315.\n\362]#\273\246MH\205z-d\223\320\321E\333B*.^\347{\332\276\310\207\"Gt&yr</\337\325\"\023*B\"!\005Y&%\230nb\030\330\301\006\250\027L\224w6\351Ee5\t\244\223L\324qp\023\263\345\346\350\313\0206\035\234!\357\2010\234\236\255\035Z\252\024;`\326\364\250(\321\376\022=\222\r|\367\033\350\221\010\254{\300\2643X\360m\r\r3\273h0\34299;\305\342\256\\\220a\347G\2318.\270p]\2345m\004$\3374\r\243\205nw6\335\335\354\203\321o\221\255\2722\322l\257\030\216g\225\222U/Mn\000\005\331{\350u1\035^V\324\355\023\310\014\324\r@\344\326N\366e\252\300\205\020\t:\026\306\352\335\256\013Q\035\261\2169J\2033\031\277J\364V\235\360]@\271\203\233\202\363\220\303r!\r\254P\333\302d \216\010\224\245.\332\216':\352\023""\034n.\265UH\207xv\313u|-\010\307m\311*\3419q\221\232E\024\324!\240cC\305\022\225\246\330d\364\242\370\360\244\2061\266\022\306\345+\rl\0102\022)lu\033\257h\310\274\013\200fh5\324\004O\227\nhb\266\2102z\2424\202\202\363'\302\205\303\302x%}\337\353\003\304#J\016\325\022\3265\323\240\321\256xbj\2570\267\366\205\rH5\212\343\033\361\350\233\234ON\315\310\230\364\305l\n\257\032\206\2030\013\220\347\034\314n\037\306UL!2G\010\r\321\301\221\310I.\020\220\244\001\276\210\0245d\213\032\273wm\351\366v\323\214M\233\r\222\013\244\271\017\025\350\344\000\304U\223\026\223N\236\r\\\372\304T*\244@\246\240\343\231u\"\352\301\375n-\r\372\371B\347?u\262\246\302%)*\022\252\354\305\235\211\177\022de\346 @2H\003c\002\217\356\371\237\204\363\327\315\004\322`\323@\306\t\241\211\200\333^\202\003\323h5\336\331\364\037\361\371\201\350\007|\023\004\277\037\253\345\"\027\243\002\362J\031\312N\250;\340\007\231E\346!x\016\tK\356\t\260>\335\252`\310\202se\344o\272\371\361\220\017|\027\300\314\222+\310.J8\214T\033\320\002\353\301\347\000\232\310F\302\317WGSlsQ\002\2548+/\275\232;\210\331\227\347\036qk\031\222\227\021\000\307\026@\330\223\310~\226265\022\336\360N\350\003e\025H\t\010\300\026\2635\003]$\t\201[t\335;\220\2728D\241\362\033A\335\320a?\257F\363^\031\356\305\202\255m\221\010k\026\030\325\020Rm*\236\230\004O\350)\362B\250\257\270d\r[\260?i\352\214(\000 h\200\006X0\030\322\014Q\260\3243\304\"\034\034\362\374X6\234\r\252\270i\233\303\336\036\367/\326\266\325h\220\214Jt\340\025,\017R\206\276v\020\026\205\305\300B:\257\301\r\243\252\300Y\201\276U\225_40\327\256\006\023{\354\211\203\0173\r\037;\345\251@o\300\353\343^yT\177\374hA\217\315\2245+\350@\220\256\n\265\366\t\341P\305\221\277{F\221\346\361p]\244C\002\272\tN\201Itw#\321\277D\024g^\017\335\007\262\000\322\312\333q2\220L\230\340\226F?\2313\357'v\313&W\315\020\234\326\336\216;\250\302O\rrn\216\330r\"\312\003\"\373Jn\361cc\215\243\361\314\304\002\347\236u\206-\253\301\213{AA\240\030\005\326\353\260\237\364\020=\321$l\rG\2559\002\n\230>`o\2271""\267m\270\351i\374*h\211p?\273\332\202l0\302\273p\363\"\354\013\2715\362\354\3702\347r(/\034\315\024\025t\275Q\020\313\226R\357\204\3348\264\335\227;\032\266$\233\335\3341y\217\034\234\\`E\322\360\rg\263aB\335\221z\200KP\314&E\246\206r\210)#\300\025l\312L\204:\2036\235\021\024\210Y+B\253\271\211\351\302\006!\022\rI\304\2539V:-{q\367\032-f\020hx\006\n\203m\215\337V\331&\311JP\321i\312WJd\347\203\211\327\007b\342WC\350l\212t\270\310\261\035\030E\362(y\275WGJ\251\025\021\341\035\3249\002U\224\224\022\232\201\021'g\0232\240\225c\014\355\002\005Ef)0\252\01010\252\353\377\027rE8P\220\335[\267\313";
    PyObject *data = __Pyx_DecompressString(cstring, 3057, 2);
    if (unlikely(!data)) __PYX_ERR(0, 1, __pyx_L1_error)
    const char* const bytes = __Pyx_PyBytes_AsString(data);
    #if !CYTHON_ASSUME_SAFE_MACROS
    if (likely(bytes)); else { Py_DECREF(data); __PYX_ERR(0, 1, __pyx_L1_error) }
    #endif
    #elif (CYTHON_COMPRESS_STRINGS) != 0 /* compression: zlib (3013 bytes) */
const char* const cstring = "x\332\245W\315w\023G\022GA\030\231\310\330\002\371\2038Y\306\200QX@\211\262\016\020BH\204\r\204\227\204E`\002\273o\367\315\033\315\264\354Y\3133\362\364\310\226\370\330\345\250\343\034\373\330\3079\316QG\037s\324q\216\376\023\362'lU\367\2144\222m\342$\357\331\323\335U\335\325\365\371\353\322\262fY\266\253\350\266\265M\034WiZ\246n\033D)X\332&)(\256\255hJ\315\254\023\205\266\251K6\025$\337V\026\351\262<f7\210\243\271D\261-E\257\333\224\030\312\217d\233\324W\356)\206\346jU\215\222\225hTL\032m\351SlG1]<\217\223\203\270\324\322\032t\035\256\351s\037\333\026yl\303\205\356\272\346*\313mw\035n\006\256A\352fU\250Ro+\324uL\035\004\343&Kyr\377\311\365\245[K\212f\031\212C\376Ct\227*\264Y\325\353\032\245\204*vM\2516\315\272kZ\212\333n\020ZT\036\325\224\266\335T,\002\326\200\375\r\330\227<\340\256\023K\241\304\305\211R\020n\320\\\323\266T8nZk\005\3050\035\270\304\334&x\372\201V\247\244\370\314\325\300\267\250\001u\355\006\336\351h\326\032Q6\233\324U\252D\251\266\301$T\333Z\243\317W\037\\\277u\267\370\255f\030\252\205\246\202\241\237\267\224O\245\003\256\024t{\263\241\t\247\025\204\304\004A\225A\213\245\322\006\321\315\232)\314X#\240\256#\366:\204R\320w\260\017\235\212\356\326\"\025\336\277\253\200Qi\264\013:\320\355Mep\273bW\321\273\375\003\272V\257k\325:1L\212\003\261\360\273\246\233T\316\214(\3054g\255\177DKzB\252\024'\244\244\335i\324\333\220_EH\260\035\323]\027\351\250\304\304'\016\251\231-H\241\210\331\020kEr?S\243]\215v\253 9\005PQd1\334\334\244\t7\311\343\005\212Q+H\233!l\005J\352\265\242jT\257\211\211\335\300\250\323\204\214\250\206d\332h\312\023\231\235\221Wj \246a\352\033u\214\261\020\204\271\377\347NKE\006\021\220k\241\266J\353\246Nb\202\335\220\353?~]\277\024\377\260\204\035\007TV\253\232\253\257K\265$\341\317\272\321\250b\375\343\3218A\0059\212}\202u'\252/\273\351\350\344\356=\200\230G\021\370$\347EU%\026\314Uu\224\3322\335}D\207\030M\235\250\272Pq\037\027@\002b\341\036\302\027\365\274l;NSx\340\276\343\330\316\312\275\225{\003\r\344\\\334+\246#\267\t\332\276;\200""\n\345\351\330-s\023\311\324|E\016 Q\240\t\005p\304\374\321]U@\022\254\001K\211\213\023(\006\371U\3410\000\275\333\206e\214\3300mD\365\006%\201\253&\356v\264\0355\261%N\033\230&\342/l}\364w9D\233\373\243\265\255\325Mc\210\267\337\366\367y\271\317\003\375\266\373\013J\310\306\320BumU\224\312\001T\273\361\243\255o\014\360d0\333\257\312\020o\2372\t\256\364k\202\000\236M\254b\257%H\t\007'\251\315\344\261\330\303\tR\302\323\240@\273\005\377+\360 \252\217I\313}JjO\265\235\330\344\304t\277a\303\314}\226%\331p\337fr\275A\332\311\245\205\027'\326\030\230\344\032\035?\272\306@\324L\207\272\0071\340)\036\242\213\224\031!4\311\263\3105\3618\250\254$E\324W\2020\342\205\004g\237\013\372<QJ\375\025\304\265?\217\243\332'$\013d@\204\324\000lx\201\201\273\207q\033\314\006J\017\323\204\332C\244\021\305\207x\373TOp\001\033\210e\014\023\206\300\"\301\322AO'\261\2269\235 @r&V\"\023\245\374Q\241\373\020I\243\272ij\264m\351\246\r\230\344\330M\350\311\010\255\326\241\022\3016\371\252\231\350\n\010\256\244\3429\230\331\233\220)u\364Q\325t\251.\201\214\nUux'\010\234R]G\323IU\3237D\250d/5\004|\203Wt\244\237J4C\272CP]\263\246n\232@\260\326\014\243jTUY\246\006\301>\305 5\255Yw\245_\014\320\332\261\333P\276\252j`\005\306\203a\353\030V<\320\017/A\304C\331\020Z\352R\322\322U\267*\276\320\226\342\010f\307q\007s\353\252\256\351\353\244FQG\2404-\020\t\231\227\204k\230c\357.[wq\035\350,v\311dPM\023(z\275\tj@\311\306S\341\352\301\302n\304sQUp\312B\035\342\261x\007b\001]\356]\230h\233UC\273\253\232T\355\007\020\241\001\377i\234\364p\021\374\311~\244\177B\255\343\317\006\360\023\274\374\350i 8Mi\241\210\262\252nj\020E\370\266\320xI\304\005\030j\t\n\005&q5\321\245\243r\233\266\321\204}8sl\364\261c\342\036+\312\016\3515\371\205\221\354\340\247\345F\335\010\306\337\262MC\205\373\365\r\032u\216Q\007\331\260\033\360\2076\311\300'PZ\305\\\200\303DL\266\241\030`\002\000,\30547\345\n\262\tRD\316\243@\340\024\343,gMK\2648$\361zH\306\266\213\2353j\274\325\324\352R{iO\022V K\215\270\257\352\303\202\272\017 \372\004""\322\302i-\002!4\300t\300\030\251&\230\001!!\210\274C\260\234\204\342\241\3474\371\212b+&\300'\3624\314\242\002I@\222z\000<\tD\240u\371S#~\342d\353F]y\315\240\313\025G\305u\375F\2276\241\000\020K0\367\341\027\006q\345WrU\025\367\303w\035]\0055\001\350`Q\200\002\360X\263\001\277\\\t\374\034\350+%\036\027\221\3722\377\253\315ZM\200P\223P\360\215Yk\367\343Kw\210\266\001\256\214\006\364\352\216#@0\361&GS!E\030:\324\010\367\322\027\370u\377uw!<\377EP\334M\205\237\224\202\\X\270\333\325\302\314\207\235\245\016\365\026=\312\026\031\345\227\370Fp6(\207\3313\3365\246\261\255\275\314d\247\345m\205\331\311\216\353}\311N\301\006\323o\005\355\335\261]-\314\346\274\031\357\277\\\r~\332\315\205\331\263\336-v\207\303\326\217\330g\275+\267\341\262\354\254\367\206\177\345\227\337w}$~\342(\177G\2203\345\245\242\317o\312Z\352-\375\360K\tO\276f\013#,\360\300\342\345\243\370F\034E\337\224\302\354'\254\202&\203\026\323^\245\227\276\021D\336mz\313H\214N\375\23296>\343\275d\225\275\314y\326\344e\376\334\277\350W|-\2348\215\016\315{\253l\226\247\370\371 \025f\346X\212\345\"Ix\311\337\2745Va\006\377\324\277\340\337\017\246\003\355]*L\317\263\233|\001\244\2368\335\233P\340d.\034?\025f&:\377\3604\257\305\266\302t\336+{\225we\251\314kv\021\325<\353-y\257xn/\236\274+\377z\352\330xVl(\200\210\034\230\020N\315x\006\330\227?\317\\~\333\177\031<\r\266\272\037\200\367\363\263^\233\237\204P\317)|\214k{\221\225\017Y\t}\237\366\3128\234\364\266$\243\345\275\345\017\375\210\365=\023\314\014\233\344}\366\377\370\232_\031f\367\346\256\370\271\230\377\206\337\364\027\366\204\2271\303vz\305\357 \366Syo\205e\270\230|\317\036\373\013\261\2052`\013\2110!\365-[\341\343\376l\220\332\003\371c\036$\357\304^&\333y\340-\204rx\377\361\226\267\3034\244]\362\326Y\225\247F\202\267\227\371\210-`\034\370]\310\310\314x\230;\203z\375\304\363\274\022\346\247\303\3749\226cE?\033<\350^\333\325\366\262\037\2632\004R\373Sw\212\204\231\362&\330\013\240U\016\027\225\355\374$2j\206\277\r\312\301\241\033\177\035;vzRf\3403\226f\017""\300\226\245^\341f\340vKQ\"\036zE\254\310\317\274t\270\032\223\235\255\3671]\357\233\236\362y\220\032\3363Tv\023\321E\307\275\245\336l\201o\371i\277,/\206j\273\332\273z\247[>\260\340Db\337\360fY\n\261i\221mA\356R\177\301/\241\233\277\304D\r\263\347\3304\354\335\346\317\374\024$\236\274'\r\361\313\201\033\276\tRA\337\370a\331{\231\217\3312\010\204Z=\317\332\376\230\257\371mQ\"\260|\343\337\010r\301\225\356\027\335\nf\004\010<\341\375\314J\220\341S\263\020\331\032F-\234\212\352\217\027 7\217\007K\301\253\335\334\356\345_Ra\376#v\205\227\302\271s\341\024:\177\232\021X\2114*\360\263\342\250X\\\346\307y\211\227\367d\276\255\364\346\256\371e\377\237(\005K\273\344\335\207=\027\331*\237\3465`T0\371S^.\224\303a6\315\261\023\220\2355\276\014>N\001\262`^L\003.\345\331\277\374\234\177\t\214\024!k\263,/\037&\003\"\332;\273\310\237\362\035\277\226L\273\035Og\271\221D\227\220#\256\230\347\367\020P.\360E\270;\003\036)\005e\310L\201L;\200~\317!\035\014\377j\360\022\234:\365\027\200\302b\360A\000\2314\317\226\000\352\204\264m\357\005\273\017\205\267\352\317\007\337CJ\210m\327}\007\nSn\213\302;\346\021\021\rq\310\365n\260\031`\001}\034\365\023\340\360\0100P\244~\036R\373\346\000a\016\262!\022\000\324\336\231\313\\C\235\247\001\240\306\340\356\205P8\353\004{\312\034~\316_\364\267~\247\240Y\200\341\313\210\264\362\261Z\0041g\370\327>\tJ\277CP\322\352\275\243X%\354?\204\031\313\332\200'\344\010\262\342\355u\276p\244\253\207\302\037\016\205\353H\347'c\343\007\033\326X%\3618\215K4\353\003\177\364\020D\030%\207\231\331\370\321\205G\020\362\246\305\035\177\332_\013\236w\027\344&\027J\327\005\306\266D_\321\201\260\343\254\224\\,\001b?\364\227\374H\356\253~9\256\371\253\301\\\367L\267\224d\010\243z\351\345\335\322\301E\365W\377\032\266\002\323\007\264\017\230'\363\373\337 Y\266\253P^\370\n\035\372\020E\010\337\371\001:\006\027\001\005\033\213S\235\253\320\307\034\233\355\315^\207\367\371\364\305\336\305[\335Tx\372*\274\345\247/\365.}\003%\010\310p@\377\221\006aaz\006\036\331t\316CU:\347zg\026{""\213\242\217\231\357\315\003V\177\215\257\212\302'\375\267\010\332\320\241DM\313K\257\002\217\235\0267-)!\267\263\352M\213N$\235\355<\004P{\300\256A-\244O\276\333\356\274\000\204\313Ch3g0~\263\236@\257\024\362\000\r&{\223W\321\367\301\311\340\365n\341\027H\325!\nx\272\010\317\350\205\257\272\271\337j\365D9\367\362\237\372)T\3661\226t\336\373\221\247\001y\017y\252'\001\207*\240\364\"\004\351\221_\031\275j\344\211\032\025\037\237\257\001\222\235\363\207\024\355\247\244\354\243>\300\t\372\257\337~b\350@\222?\003I\227\332K\237g[\370\260~\330\375zw\243Wy\032\246\317z\267\331K^\221\036|\343\335\004\001\031|5\376\335-\302\273\223\376\260S\202\353z\305o\273[\207!\373+\361\360\214\361\315\000\266^\221\275~D\276\004\357\324\022o\007\023\335\312\377\001\316\351\3577";
    PyObject *data = __Pyx_DecompressString(cstring, 3013, 1);
    if (unlikely(!data)) __PYX_ERR(0, 1, __pyx_L1_error)
    const char* const bytes = __Pyx_PyBytes_AsString(data);
    #if !CYTHON_ASSUME_SAFE_MACROS
    if (likely(bytes)); else { Py_DECREF(data); __PYX_ERR(0, 1, __pyx_L1_error) }
    #endif
    #else /* compression: none (7173 bytes) */
const char* const bytes = "Cannot convert unicode 'name' to a file system name: %sCannot operate on closed LevelDB databaseDatabase is closedDatabase or iterator is closedDatabase or snapshot is closedNoneNote that Cython is deliberately stricter than PEP-484 and rejects subclasses of builtin types. If you need to pass subclasses then set the 'annotation_typing' directive to False.Start and stop of range must be byte stringsUTF-8>.?add_note at 0x (closed)'comparator' and 'comparator_name' must be specified together'compression' must be None or a string'compression' must be None or 'snappy'custom comparator object must be callabledisableenablegcisenabled'name' arg must be a byte string or a unicode string<plyvel.DB with name <plyvel.PrefixedDB with prefix plyvel/_plyvel.pyx'prefix' cannot be used together with 'start' or 'stop'self._db,self.options cannot be converted to a Python object for picklingself._iter cannot be converted to a Python object for picklingself._iter,self.comparator,self.start_slice,self.stop_slice cannot be converted to a Python object for picklingself._snapshot cannot be converted to a Python object for picklingself._write_batch,self.write_options cannot be converted to a Python object for picklingself.db is not None or self.prefix is not None<stringsource>BaseIteratorBaseIterator.__enter__BaseIterator.__exit__BaseIterator.__reduce_cython__BaseIterator.__setstate_cython__BaseIterator.closeCorruptionErrorDBDB.__enter__DB.__exit__DB.__reduce_cython__DB.__setstate_cython__DB.approximate_sizeDB.approximate_sizesDB.closeDB.compact_rangeDB.deleteDB.getDB.get_propertyDB.iteratorDB.prefixed_dbDB.putDB.raw_iteratorDB.snapshotDB.write_batchErrorIOErrorIteratorIteratorInvalidErrorIterator.__reduce_cython__Iterator.__setstate_cython__Iterator.prevIterator.seekIterator.seek_to_startIterator.seek_to_stopLockPrefixedDBPrefixedDB.__reduce_cython__PrefixedDB.__setstate_cython__PrefixedDB.deletePrefixedDB.getPrefixedDB.iteratorPrefixedDB.prefixed_dbPrefixedDB.putPrefixedDB.snapshotPrefixed""DB.write_batch__Pyx_PyDict_NextRefRawIteratorRawIterator.__reduce_cython__RawIterator.__setstate_cython__RawIterator.itemRawIterator.keyRawIterator.nextRawIterator.prevRawIterator.seekRawIterator.seek_to_firstRawIterator.seek_to_lastRawIterator.validRawIterator.valueSnapshotSnapshot.__enter__Snapshot.__exit__Snapshot.__reduce_cython__Snapshot.__setstate_cython__Snapshot.closeSnapshot.getSnapshot.iteratorSnapshot.raw_iteratorSnapshot.releaseWriteBatchWriteBatch.__enter__WriteBatch.__exit__WriteBatch.__reduce_cython__WriteBatch.__setstate_cython__WriteBatch.appendWriteBatch.approximate_sizeWriteBatch.clearWriteBatch.deleteWriteBatch.putWriteBatch.writeappendapproximate_sizeapproximate_sizesasciiasyncio.coroutinesblock_restart_intervalblock_sizebloom_filter_bitsc_rangesclearcline_in_tracebackcloseclosedcompact_rangecomparatorcomparator_namecompressioncreate_if_missingddbdb_prefixdecodedefaultdeletedestroy_db__dict___dict__doc__encode__enter__error_if_existsexc_tbexc_typeexc_val__exit__fill_cachefsname__func__getget_propertygetfilesystemencoding__getstate__iidinclude_keyinclude_startinclude_stopinclude_value__init____init__.<locals>.<lambda>_is_coroutineitemitemsiteratorkeykey_slice<lambda>__leveldb_version__lru_cache_size__main__max_file_sizemax_open_files__metaclass____module____mro_entries__n_rangesname__name____new__nextoptionsparanoid_checksplyvel._plyvelpoppopitemprefixprefixed_db__prepare__prevput__pyx_checksum__pyx_result__pyx_state__pyx_type__pyx_unpickle_PrefixedDB__pyx_vtable____qualname__rangesraw_iteratorread_options__reduce____reduce_cython____reduce_ex__refreleaserepair_dbresultreverseseekseek_to_firstseek_to_lastseek_to_startseek_to_stopself__set_name__setdefault__setstate____setstate_cython__sizesslsnappysnapshotsourceststartstart_slicestatestopstop_slicesupersyncsystargettarget_slice__test__threadingtransactionupdateuse_setstatevalidvaluevalue_buffervaluesverify_checksumsweakrefweakref_refwrwritewrite_batchwrite_buffer_sizewrite_options\320\004\"\240-""\250{\270!\330\0372\260.\300\001\330\0351\260\021\330'>\270a\330\010\013\2104\210s\220%\220s\230%\230s\240$\240k\260\023\260A\330\014\022\220,\230a\230q\340\010\017\210x\220q\330\014\017\210t\2205\230\n\240$\240i\250x\260y\300\006\300a\330\014\021\220\026\220~\240_\260M\300\021\330\014\023\2208\230<\240q\330\014\032\230/\320):\270!\330\014\027\220|\2409\250A\320\004\"\240-\250{\270!\330\0372\260.\300\001\330\0351\260\021\330'>\270a\330\010\017\210x\220q\330\014\r\330\014\r\330\014\r\330\014\r\330\014\r\330\014\r\330\014\r\330\014\r\330\014\r\330\014\r\330\014\r\330\014\r\330\014\r\320\004\"\240-\250{\270!\330\0372\260.\300\001\330\0351\260\021\330'>\270a\330\010\017\210x\220q\330\014\020\220\001\330\014\020\220\001\330\014\r\330\014\r\330\014\r\330\014\r\330\014\r\330\014\r\330\014\r\330\014\r\330\014\r\330\014\r\330\014\r\320\0044\3204K\3101\330\010\017\210{\230!\330\014\r\330\014\r\330\014\r\330\014\r\320\0044\260A\330%&\330\010\013\2104\210s\220%\220s\230%\230s\240$\240k\260\023\260A\330\014\022\220,\230a\230q\340\010\017\210{\230!\330\014\017\210t\2201\330\014\035\230Q\330\014\027\220q\330\014\025\220Q\320\0046\260a\330\010\013\2104\210u\220C\220q\330\014\022\220,\230a\230q\360\010\000\t\026\220X\230Q\340\010\037\230u\240A\240U\250#\250Q\250a\330\r\016\330\014\021\220\024\220T\230\027\240\001\240\037\260\001\330\010\030\230\001\230\021\320\0046\260a\330\010\017\210t\2203\220g\230Q\230d\240(\250\"\250E\260\025\260a\200\001\330\004\033\2307\240!\360\010\000\005\016\320\r \240\001\240\021\330\t\n\330\010\r\210Y\220a\220x\230q\330\004\024\220A\220Q\200A\330\010\013\2104\210{\230#\230Q\330\014\023\2204\220z\240\021\340\014\023\2204\220z\240\021\200A\360\n\000\t\014\2104\210{\230'\240\021\330\021\025\220Q\330\020\026\220d\230!\330\024\037\230t\240:\250X\260R\260q\270\002\270!\330\024\027\220y\240\007\240q\330\030 \240\006\240a\340\010\013\2104\210u\220G\2301\330\014\020\220\004\220A\330\014\020\220\007\220q\340\010\013\2104\210x\220}\240G\2501\330\014\020\220\004\220H""\230A\330\014\020\220\010\230\017\240q\340\010\013\2104\210x\220\177\240g\250Q\330\014\020\220\004\220H\230A\330\014\020\220\010\320\030)\250\021\340\010\013\2104\210x\220|\2407\250!\340\014\017\210t\2208\230<\240w\320.@\300\001\330\020\024\220D\230\010\240\001\330\020\024\220H\230N\250!\200A\330\010\013\2104\210s\220%\220s\230!\330\014\022\220,\230a\230q\340\010\013\2104\210}\230D\240\t\250\027\260\001\340\014\020\220\006\220a\330\014\r\340\010\014\210F\220!\330\010\014\210F\220!\200A\330\010\013\2104\210s\220%\220s\230!\330\014\022\220,\230a\230q\340\010\013\2104\210x\220w\230a\330\014\022\220$\220h\230b\240\001\340\010\037\230u\240A\240U\250#\250Q\250a\340\010\032\230!\2307\240!\240>\260\021\330\010\t\330\021\022\330\020\024\220M\240\024\240Q\330\024\025\330\024\031\230\021\230.\250\014\260F\270,\300a\340\014\034\230A\230Q\230a\200A\330\010\013\2104\210s\220%\220s\230!\330\014\022\220,\230a\230q\340\010\013\2104\210x\220w\230a\330\014\022\220$\220h\230b\240\001\340\010\037\230u\240A\240U\250#\250Q\250a\330\r\016\330\014\020\220\r\230W\240A\240Q\200A\330\010\013\2104\210s\220%\220s\230!\330\014\022\220,\230a\230q\340\010\014\210M\230\027\240\001\240\026\240}\260A\260Q\200A\330\010\013\2104\210s\220%\220s\230!\330\014\022\220,\230a\230q\360\006\000\016\017\330\014\021\220\024\220S\230\004\230F\240!\2404\320'7\260t\2701\330\010\030\230\001\230\021\200A\330\010\013\2104\210s\220%\220s\230!\330\014\022\220,\230a\230q\340\r\016\330\014\020\220\r\230V\2401\200A\330\010\013\2104\210s\220%\220s\230!\330\014\022\220,\230a\230q\340\010\017\210q\200A\330\010\013\2104\210s\220%\220s\230!\330\014\022\220,\230a\230q\340\010\017\210t\220=\320 0\260\001\200A\330\010\013\2104\210s\220%\220s\230%\230s\240$\240k\260\023\260A\330\014\r\340\r\016\330\014\020\220\003\2204\320\027'\240q\250\004\250A\330\014\020\220\r\230Q\320\004+\320+<\270A\330\010\013\2104\210u\220C\220q\330\014\022\220,\230a\230q\360\n\000\t\014\2106\220\027\230\001\330\014\032\230%\230q\240\007\240s\250!\2501\340""\010\013\2105\220\007\220q\330\014\031\230\025\230a\230v\240S\250\001\250\021\340\r\016\330\014\020\220\004\220M\240\021\240!\240=\260\001\260\021\200A\330\010\013\2104\210u\220C\220q\330\014\022\220,\230a\230q\340\010\034\230C\230q\240\001\330\010\037\230y\250\006\250a\250y\270\002\270!\330\010\037\230|\2506\260\021\260)\2702\270Q\330\010\t\330\014\020\220\005\220V\2301\230A\330\020\027\220w\230f\240A\240Q\330\020\023\2204\220z\240\021\240'\250\027\260\003\2604\260z\300\021\300&\310\001\330\024\032\230)\2401\330\030\031\330\020\030\230\001\230\025\230e\2401\330\024\031\230\021\230'\240\023\240A\240Q\330\024\031\230\021\230&\240\003\2401\240A\340\021\022\330\020\024\220D\320\030,\250A\250Z\260z\300\021\340\014\023\2201\220E\230\021\230#\230T\240\025\240f\250A\250Q\340\014\020\220\001\220\021\330\014\020\220\001\220\021\200A\330\010\013\2104\210u\220C\220q\330\014\022\220,\230a\230q\340\010\030\230\005\230Q\230f\240C\240q\250\001\360\010\000\016\017\330\014\025\220T\230\024\230\\\250\021\250$\250a\250q\340\010\017\210y\230\014\240A\200A\330\010\013\2104\210u\220C\220q\330\014\022\220,\230a\230q\340\010\017\210t\320\023%\240R\240w\250f\260A\260Q\200A\330\010\013\2104\210w\220c\230\021\330\014\022\220,\230a\230q\340\010\013\2104\210{\230'\240\021\330\014\025\220T\230\033\240B\240a\340\010\"\240%\240q\250\010\260\003\2601\260A\360\006\000\t\014\2104\210w\220g\230U\240$\240d\250+\260X\270Q\330\020\036\230d\240.\260\002\260!\330\014\033\2304\230q\330\010\013\2104\210v\220W\230E\240\024\240T\250\033\260H\270A\330\020\036\230d\240-\250r\260\021\330\014\033\2304\230q\340\r\016\330\014\020\220\006\220e\2301\230A\330\010\013\2104\210t\2206\230\026\230q\340\014\020\220\t\230\021\330\014\r\340\010\014\210I\220Q\330\010\030\230\001\230\024\230V\2407\250!\200A\330\010\013\2104\210w\220c\230\021\330\014\022\220,\230a\230q\340\010\013\2104\210t\2206\230\026\230q\330\014\022\320\022&\240a\360\006\000\t\025\220D\230\006\230d\240!\330\010\017\210y\230\005\230R\230r\240\031\250%\250q""\200A\330\010\013\2104\210w\220c\230\021\330\014\022\220,\230a\230q\340\010\013\2104\210t\2206\230\026\230q\330\014\022\320\022&\240a\360\006\000\t\027\220d\230&\240\006\240a\330\010\017\210{\230%\230r\240\022\240;\250e\2601\200A\330\010\013\2104\210w\220c\230\021\330\014\022\220,\230a\230q\340\010\013\2104\210t\2206\230\026\230q\330\014\022\320\022&\240a\340\r\016\330\014\020\220\006\220e\2301\340\010\030\230\001\230\024\230V\2407\250!\200A\330\010\013\2104\210w\220c\230\021\330\014\022\220,\230a\230q\340\010\014\210I\220Q\200A\330\010\013\2104\210w\220c\230\021\330\014\022\220,\230a\230q\340\r\016\330\014\020\220\006\220k\240\021\340\010\030\230\001\230\024\230V\2407\250!\200A\330\010\013\2104\210w\220c\230\021\330\014\022\220,\230a\230q\340\r\016\330\014\020\220\006\220l\240!\340\010\030\230\001\230\024\230V\2407\250!\200A\330\010\013\2104\210w\220c\230\021\330\014\022\220,\230a\230q\340\010\"\240%\240q\250\010\260\003\2601\260A\330\r\016\330\014\020\220\006\220e\2301\230A\340\010\030\230\001\230\024\230V\2407\250!\200A\330\010\013\2104\210w\220c\230\021\330\014\022\220,\230a\230q\340\010\017\210t\2206\230\026\230q\200A\330\010\013\2104\210w\220g\230Q\330\014\020\220\004\220A\330\014\020\220\t\230\021\200A\330\010\014\210F\220!\200A\330\010\014\210F\220!\330\010\017\210q\200A\330\010\017\210q\200A\330\026\027\330\010\017\210t\2203\220d\230!\2304\230x\240r\250\025\250g\260U\270!\200A\330\010\017\210t\2204\220t\2304\230v\240Q\200A\330\010\017\210x\220q\230\003\2301\200A\330\010\017\210x\220q\230\003\2304\230u\240G\2504\250q\200A\330\010\017\210z\230\021\230#\230T\240\025\240g\250T\260\030\270\022\2701\200A\330\010\017\210z\230\021\230#\230V\2407\250!\320\004C\3001\330\010\013\2104\210u\220C\220q\330\014\022\220,\230a\230q\340\010*\250,\260a\330\010\025\220X\230Q\340\010\037\230u\240A\240U\250#\250Q\250a\360\006\000\t\033\230!\2307\240!\240>\260\021\330\010\t\330\021\022\330\020\025\220T\230\024\230T\240\021\330\024\025\330\024\025\330\024\031\230\021\230.\250\014""\260F\270,\300a\340\014\034\230A\230Q\230a\330\010\030\230\001\230\021\210K\220x\230t\2401\240A\200\001\330\004\n\210+\220Q\320\000\027\320\027-\250Q\330\016#\320#8\270\001\330\016+\2501\330\016$\320$=\270Q\330\016\017\330\004\033\2307\240!\360\010\000\005\016\320\r \240\001\240\021\330\004\030\230\001\330\004\026\220a\330\004\021\220\021\330\010\t\210\031\320\022%\320%6\260a\330\010\033\320\033+\320+;\2701\330\010 \240\017\250}\270A\330\010\024\220A\330\t\n\330\010\r\210X\220Q\220h\230a\330\004\024\220A\220Q\200\001\360\010\000\005\016\210T\220\025\220d\230!\330\004\014\210G\2201\220F\230,\240a\330\004\007\200v\210W\220E\230\024\230Q\330\010\022\220!\330\010\027\220q\340\010\030\230\001\330\004\007\200q\330\010\017\320\017+\2504\250q\260\007\260{\300'\310\021\340\010\017\320\017+\2504\250q\260\007\260{\300!\320\004.\250a\330\"9\270\021\330\010\013\2104\210s\220%\220s\230%\230s\240$\240k\260\023\260A\330\014\022\220,\230a\230q\360\006\000\t\025\320\024(\250\001\330\010\024\220N\240!\330\010\024\220L\240\004\240A\340\010\013\2104\210x\220w\230a\330\014\022\220$\220h\230b\240\001\340\010\017\210v\220Q\220d\230%\230u\240I\250Q\320\004.\250a\330\"9\270\021\330\010\013\2104\210u\220C\220q\330\014\022\220,\230a\230q\360\006\000\t\025\320\024(\250\001\330\010\024\220N\240!\340\010\017\210v\220Q\220f\230E\240\031\250!\320\004.\250a\330\"9\270\021\330\010\017\210t\2203\220d\230!\330\014\020\220\010\230\002\230!\330\014\024\220A\330\014\035\230Q\330\014\027\220q\200\001\330\004(\250\001\250\026\250q\200\001\340\004\037\230q\320 0\260\013\270;\300k\320QR\330\004\023\220:\230X\240Q\240a\330\004\007\200|\2207\230!\330\010,\250A\250]\270.\310\001\330\004\013\2101\320\004.\320.?\270q\330\010\013\2104\210u\220C\220q\330\014\022\220,\230a\230q\340\010\017\210z\230\021\230&\240\006\240m\2601\320\004)\320):\270!\330\010\017\210z\230\021\230$\230e\2404\240y\260\r\270Q";
    PyObject *data = NULL;
    CYTHON_UNUSED_VAR(__Pyx_DecompressString);
    #endif
    PyObject **stringtab = __pyx_mstate->__pyx_string_tab;
    Py_ssize_t pos = 0;
    for (int i = 0; i < 257; i++) {
      Py_ssize_t bytes_length = index[i].length;
      PyObject *string = PyUnicode_DecodeUTF8(bytes + pos, bytes_length, NULL);
      if (likely(string) && i >= 36) PyUnicode_InternInPlace(&string);
      if (unlikely(!string)) {
        Py_XDECREF(data);
        __PYX_ERR(0, 1, __pyx_L1_error)
      }
      stringtab[i] = string;
      pos += bytes_length;
    }
    for (int i = 257; i < 311; i++) {
      Py_ssize_t bytes_length = index[i].length;
      PyObject *string = PyBytes_FromStringAndSize(bytes + pos, bytes_length);
      stringtab[i] = string;
      pos += bytes_length;
      if (unlikely(!string)) {
        Py_XDECREF(data);
        __PYX_ERR(0, 1, __pyx_L1_error)
      }
    }
    Py_XDECREF(data);
    for (Py_ssize_t i = 0; i < 311; i++) {
      if (unlikely(PyObject_Hash(stringtab[i]) == -1)) {
        __PYX_ERR(0, 1, __pyx_L1_error)
      }
    }
    #if CYTHON_IMMORTAL_CONSTANTS
    {
      PyObject **table = stringtab + 257;
      for (Py_ssize_t i=0; i<54; ++i) {
        #if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
        Py_SET_REFCNT(table[i], _Py_IMMORTAL_REFCNT_LOCAL);
        #else
        Py_SET_REFCNT(table[i], _Py_IMMORTAL_INITIAL_REFCNT);
        #endif
      }
    }
    #endif
  }
  {
    PyObject **numbertab = __pyx_mstate->__pyx_number_tab + 0;
    int32_t const cint_constants_4[] = {245107514L};
    for (int i = 0; i < 1; i++) {
      numbertab[i] = PyLong_FromLong(cint_constants_4[i - 0]);
      if (unlikely(!numbertab[i])) __PYX_ERR(0, 1, __pyx_L1_error)
    }
  }
  #if CYTHON_IMMORTAL_CONSTANTS
  {
    PyObject **table = __pyx_mstate->__pyx_number_tab;
    for (Py_ssize_t i=0; i<1; ++i) {
      #if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
      Py_SET_REFCNT(table[i], _Py_IMMORTAL_REFCNT_LOCAL);
      #else
      Py_SET_REFCNT(table[i], _Py_IMMORTAL_INITIAL_REFCNT);
      #endif
    }
  }
  #endif
  return 0;
  __pyx_L1_error:;
  return -1;
}
/* #### Code section: init_codeobjects ### */
typedef struct {
    unsigned int argcount : 3;
    unsigned int num_posonly_args : 1;
    unsigned int num_kwonly_args : 4;
    unsigned int nlocals : 5;
    unsigned int flags : 10;
    unsigned int first_line : 11;
} __Pyx_PyCode_New_function_description;
/* NewCodeObj.proto */
static PyObject* __Pyx_PyCode_New(
        const __Pyx_PyCode_New_function_description descr,
        PyObject * const *varnames,
        PyObject *filename,
        PyObject *funcname,
        PyObject *line_table,
        PyObject *tuple_dedup_map
);


static int __Pyx_CreateCodeObjects(__pyx_mstatetype *__pyx_mstate) {
  PyObject* tuple_dedup_map = PyDict_New();
  if (unlikely(!tuple_dedup_map)) return -1;
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 1, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 701};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_wr};
    __pyx_mstate_global->__pyx_codeobj_tab[0] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_plyvel__plyvel_pyx, __pyx_mstate->__pyx_n_u_lambda, __pyx_mstate->__pyx_kp_b_iso88591_Kxt1A, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[0])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 1, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 257};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self};
    __pyx_mstate_global->__pyx_codeobj_tab[1] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_plyvel__plyvel_pyx, __pyx_mstate->__pyx_n_u_close, __pyx_mstate->__pyx_kp_b_iso88591_A_4_Q_d_t_XRq_y_q_a_4uG1_A_q_4x, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[1])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {3, 0, 2, 6, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 301};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_key, __pyx_mstate->__pyx_n_u_default, __pyx_mstate->__pyx_n_u_verify_checksums, __pyx_mstate->__pyx_n_u_fill_cache, __pyx_mstate->__pyx_n_u_read_options};
    __pyx_mstate_global->__pyx_codeobj_tab[2] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_plyvel__plyvel_pyx, __pyx_mstate->__pyx_n_u_get, __pyx_mstate->__pyx_kp_b_iso88591_a_9_4uCq_aq_N_vQfE, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[2])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {3, 0, 1, 8, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 312};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_key, __pyx_mstate->__pyx_n_u_value, __pyx_mstate->__pyx_n_u_sync, __pyx_mstate->__pyx_n_u_write_options, __pyx_mstate->__pyx_n_u_key_slice, __pyx_mstate->__pyx_n_u_value_buffer, __pyx_mstate->__pyx_n_u_st};
    __pyx_mstate_global->__pyx_codeobj_tab[3] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_plyvel__plyvel_pyx, __pyx_mstate->__pyx_n_u_put, __pyx_mstate->__pyx_kp_b_iso88591_C1_4uCq_aq_a_XQ_uAU_Qa_7_T_T_F, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[3])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 1, 6, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 333};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_key, __pyx_mstate->__pyx_n_u_sync, __pyx_mstate->__pyx_n_u_st, __pyx_mstate->__pyx_n_u_write_options, __pyx_mstate->__pyx_n_u_key_slice};
    __pyx_mstate_global->__pyx_codeobj_tab[4] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_plyvel__plyvel_pyx, __pyx_mstate->__pyx_n_u_delete, __pyx_mstate->__pyx_kp_b_iso88591_6a_4uCq_aq_XQ_uAU_Qa_T, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[4])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 2, 3, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 346};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_transaction, __pyx_mstate->__pyx_n_u_sync};
    __pyx_mstate_global->__pyx_codeobj_tab[5] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_plyvel__plyvel_pyx, __pyx_mstate->__pyx_n_u_write_batch, __pyx_mstate->__pyx_kp_b_iso88591_q_4uCq_aq_z_m1, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[5])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 10, 11, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 358};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_reverse, __pyx_mstate->__pyx_n_u_start, __pyx_mstate->__pyx_n_u_stop, __pyx_mstate->__pyx_n_u_include_start, __pyx_mstate->__pyx_n_u_include_stop, __pyx_mstate->__pyx_n_u_prefix, __pyx_mstate->__pyx_n_u_include_key, __pyx_mstate->__pyx_n_u_include_value, __pyx_mstate->__pyx_n_u_verify_checksums, __pyx_mstate->__pyx_n_u_fill_cache};
    __pyx_mstate_global->__pyx_codeobj_tab[6] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_plyvel__plyvel_pyx, __pyx_mstate->__pyx_n_u_iterator, __pyx_mstate->__pyx_kp_b_iso88591_2_1_a_xq, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[6])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 2, 3, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 378};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_verify_checksums, __pyx_mstate->__pyx_n_u_fill_cache};
    __pyx_mstate_global->__pyx_codeobj_tab[7] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_plyvel__plyvel_pyx, __pyx_mstate->__pyx_n_u_raw_iterator, __pyx_mstate->__pyx_kp_b_iso88591_44K1, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[7])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 1, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 386};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self};
    __pyx_mstate_global->__pyx_codeobj_tab[8] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_plyvel__plyvel_pyx, __pyx_mstate->__pyx_n_u_snapshot, __pyx_mstate->__pyx_kp_b_iso88591_A_xq_1, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[8])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 5, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 389};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_name, __pyx_mstate->__pyx_n_u_sl, __pyx_mstate->__pyx_n_u_value, __pyx_mstate->__pyx_n_u_result};
    __pyx_mstate_global->__pyx_codeobj_tab[9] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_plyvel__plyvel_pyx, __pyx_mstate->__pyx_n_u_get_property, __pyx_mstate->__pyx_kp_b_iso88591_A_4uCq_aq_QfCq_T_aq_y_A, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[9])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 2, 5, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 402};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_start, __pyx_mstate->__pyx_n_u_stop, __pyx_mstate->__pyx_n_u_start_slice, __pyx_mstate->__pyx_n_u_stop_slice};
    __pyx_mstate_global->__pyx_codeobj_tab[10] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_plyvel__plyvel_pyx, __pyx_mstate->__pyx_n_u_compact_range, __pyx_mstate->__pyx_kp_b_iso88591_A_4uCq_aq_6_q_s_1_5_q_avS_M, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[10])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {3, 0, 0, 3, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 418};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_start, __pyx_mstate->__pyx_n_u_stop};
    __pyx_mstate_global->__pyx_codeobj_tab[11] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_plyvel__plyvel_pyx, __pyx_mstate->__pyx_n_u_approximate_size, __pyx_mstate->__pyx_kp_b_iso88591_A_4uCq_aq_t_RwfAQ, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[11])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 9, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS|CO_VARARGS), 424};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_ranges, __pyx_mstate->__pyx_n_u_n_ranges, __pyx_mstate->__pyx_n_u_c_ranges, __pyx_mstate->__pyx_n_u_sizes, __pyx_mstate->__pyx_n_u_i, __pyx_mstate->__pyx_n_u_start, __pyx_mstate->__pyx_n_u_stop, __pyx_mstate->__pyx_n_u_i};
    __pyx_mstate_global->__pyx_codeobj_tab[12] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_plyvel__plyvel_pyx, __pyx_mstate->__pyx_n_u_approximate_sizes, __pyx_mstate->__pyx_kp_b_iso88591_A_4uCq_aq_Cq_y_ay_6_2Q_V1A_wfAQ, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[12])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 449};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_prefix};
    __pyx_mstate_global->__pyx_codeobj_tab[13] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_plyvel__plyvel_pyx, __pyx_mstate->__pyx_n_u_prefixed_db, __pyx_mstate->__pyx_kp_b_iso88591_A_z_V7, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[13])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 1, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 452};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self};
    __pyx_mstate_global->__pyx_codeobj_tab[14] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_plyvel__plyvel_pyx, __pyx_mstate->__pyx_n_u_enter, __pyx_mstate->__pyx_kp_b_iso88591_A_q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[14])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {4, 0, 0, 4, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 455};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_exc_type, __pyx_mstate->__pyx_n_u_exc_val, __pyx_mstate->__pyx_n_u_exc_tb};
    __pyx_mstate_global->__pyx_codeobj_tab[15] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_plyvel__plyvel_pyx, __pyx_mstate->__pyx_n_u_exit, __pyx_mstate->__pyx_kp_b_iso88591_A_F, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[15])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 1, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self};
    __pyx_mstate_global->__pyx_codeobj_tab[16] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_stringsource, __pyx_mstate->__pyx_n_u_reduce_cython, __pyx_mstate->__pyx_kp_b_iso88591_Q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[16])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 3};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_pyx_state};
    __pyx_mstate_global->__pyx_codeobj_tab[17] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_stringsource, __pyx_mstate->__pyx_n_u_setstate_cython, __pyx_mstate->__pyx_kp_b_iso88591_Q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[17])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {3, 0, 2, 5, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 473};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_key, __pyx_mstate->__pyx_n_u_default, __pyx_mstate->__pyx_n_u_verify_checksums, __pyx_mstate->__pyx_n_u_fill_cache};
    __pyx_mstate_global->__pyx_codeobj_tab[18] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_plyvel__plyvel_pyx, __pyx_mstate->__pyx_n_u_get, __pyx_mstate->__pyx_kp_b_iso88591_a_9_t3d_A_Q_q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[18])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {3, 0, 1, 4, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 481};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_key, __pyx_mstate->__pyx_n_u_value, __pyx_mstate->__pyx_n_u_sync};
    __pyx_mstate_global->__pyx_codeobj_tab[19] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_plyvel__plyvel_pyx, __pyx_mstate->__pyx_n_u_put, __pyx_mstate->__pyx_kp_b_iso88591_A_t3d_4xr_gU, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[19])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 1, 3, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 485};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_key, __pyx_mstate->__pyx_n_u_sync};
    __pyx_mstate_global->__pyx_codeobj_tab[20] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_plyvel__plyvel_pyx, __pyx_mstate->__pyx_n_u_delete, __pyx_mstate->__pyx_kp_b_iso88591_6a_t3gQd_E_a, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[20])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 2, 3, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 488};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_transaction, __pyx_mstate->__pyx_n_u_sync};
    __pyx_mstate_global->__pyx_codeobj_tab[21] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_plyvel__plyvel_pyx, __pyx_mstate->__pyx_n_u_write_batch, __pyx_mstate->__pyx_kp_b_iso88591_z_e4y_Q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[21])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 10, 11, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 494};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_reverse, __pyx_mstate->__pyx_n_u_start, __pyx_mstate->__pyx_n_u_stop, __pyx_mstate->__pyx_n_u_include_start, __pyx_mstate->__pyx_n_u_include_stop, __pyx_mstate->__pyx_n_u_prefix, __pyx_mstate->__pyx_n_u_include_key, __pyx_mstate->__pyx_n_u_include_value, __pyx_mstate->__pyx_n_u_verify_checksums, __pyx_mstate->__pyx_n_u_fill_cache};
    __pyx_mstate_global->__pyx_codeobj_tab[22] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_plyvel__plyvel_pyx, __pyx_mstate->__pyx_n_u_iterator, __pyx_mstate->__pyx_kp_b_iso88591_2_1_a_xq_2, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[22])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 1, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 514};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self};
    __pyx_mstate_global->__pyx_codeobj_tab[23] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_plyvel__plyvel_pyx, __pyx_mstate->__pyx_n_u_snapshot, __pyx_mstate->__pyx_kp_b_iso88591_A_xq_4uG4q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[23])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 517};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_prefix};
    __pyx_mstate_global->__pyx_codeobj_tab[24] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_plyvel__plyvel_pyx, __pyx_mstate->__pyx_n_u_prefixed_db, __pyx_mstate->__pyx_kp_b_iso88591_A_z_T_gT_1, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[24])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 4, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_state, __pyx_mstate->__pyx_n_u_dict_2, __pyx_mstate->__pyx_n_u_use_setstate};
    __pyx_mstate_global->__pyx_codeobj_tab[25] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_stringsource, __pyx_mstate->__pyx_n_u_reduce_cython, __pyx_mstate->__pyx_kp_b_iso88591_T_d_G1F_a_vWE_Q_q_q_4q_4q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[25])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 16};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_pyx_state};
    __pyx_mstate_global->__pyx_codeobj_tab[26] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_stringsource, __pyx_mstate->__pyx_n_u_setstate_cython, __pyx_mstate->__pyx_kp_b_iso88591_q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[26])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 11, 17, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 521};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_name, __pyx_mstate->__pyx_n_u_paranoid_checks, __pyx_mstate->__pyx_n_u_write_buffer_size, __pyx_mstate->__pyx_n_u_max_open_files, __pyx_mstate->__pyx_n_u_lru_cache_size, __pyx_mstate->__pyx_n_u_block_size, __pyx_mstate->__pyx_n_u_block_restart_interval, __pyx_mstate->__pyx_n_u_max_file_size, __pyx_mstate->__pyx_n_u_compression, __pyx_mstate->__pyx_n_u_bloom_filter_bits, __pyx_mstate->__pyx_n_u_comparator, __pyx_mstate->__pyx_n_u_comparator_name, __pyx_mstate->__pyx_n_u_options, __pyx_mstate->__pyx_n_u_st, __pyx_mstate->__pyx_n_u_fsname, __pyx_mstate->__pyx_n_u_create_if_missing, __pyx_mstate->__pyx_n_u_error_if_exists};
    __pyx_mstate_global->__pyx_codeobj_tab[27] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_plyvel__plyvel_pyx, __pyx_mstate->__pyx_n_u_repair_db, __pyx_mstate->__pyx_kp_b_iso88591_Q_8_1_Q_7_a_6a_1_A_A_XQha_AQ, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[27])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 4, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 543};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_name, __pyx_mstate->__pyx_n_u_options, __pyx_mstate->__pyx_n_u_st, __pyx_mstate->__pyx_n_u_fsname};
    __pyx_mstate_global->__pyx_codeobj_tab[28] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_plyvel__plyvel_pyx, __pyx_mstate->__pyx_n_u_destroy_db, __pyx_mstate->__pyx_kp_b_iso88591_7_Yaxq_AQ, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[28])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {3, 0, 0, 5, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 580};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_key, __pyx_mstate->__pyx_n_u_value, __pyx_mstate->__pyx_n_u_key_slice, __pyx_mstate->__pyx_n_u_value_buffer};
    __pyx_mstate_global->__pyx_codeobj_tab[29] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_plyvel__plyvel_pyx, __pyx_mstate->__pyx_n_u_put, __pyx_mstate->__pyx_kp_b_iso88591_A_4s_s_aq_4xwa_hb_uAU_Qa_7_M_Q_F, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[29])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 3, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 598};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_key, __pyx_mstate->__pyx_n_u_key_slice};
    __pyx_mstate_global->__pyx_codeobj_tab[30] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_plyvel__plyvel_pyx, __pyx_mstate->__pyx_n_u_delete, __pyx_mstate->__pyx_kp_b_iso88591_A_4s_s_aq_4xwa_hb_uAU_Qa_WAQ, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[30])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 1, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 609};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self};
    __pyx_mstate_global->__pyx_codeobj_tab[31] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_plyvel__plyvel_pyx, __pyx_mstate->__pyx_n_u_clear, __pyx_mstate->__pyx_kp_b_iso88591_A_4s_s_aq_V1, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[31])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 616};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_st};
    __pyx_mstate_global->__pyx_codeobj_tab[32] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_plyvel__plyvel_pyx, __pyx_mstate->__pyx_n_u_write, __pyx_mstate->__pyx_kp_b_iso88591_A_4s_s_aq_S_F_4_7t1, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[32])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 1, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 625};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self};
    __pyx_mstate_global->__pyx_codeobj_tab[33] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_plyvel__plyvel_pyx, __pyx_mstate->__pyx_n_u_approximate_size, __pyx_mstate->__pyx_kp_b_iso88591_A_4s_s_aq_t_0, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[33])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 631};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_source};
    __pyx_mstate_global->__pyx_codeobj_tab[34] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_plyvel__plyvel_pyx, __pyx_mstate->__pyx_n_u_append, __pyx_mstate->__pyx_kp_b_iso88591_A_4s_s_aq_M_AQ, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[34])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 1, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 637};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self};
    __pyx_mstate_global->__pyx_codeobj_tab[35] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_plyvel__plyvel_pyx, __pyx_mstate->__pyx_n_u_enter, __pyx_mstate->__pyx_kp_b_iso88591_A_4s_s_aq_q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[35])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {4, 0, 0, 4, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 643};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_exc_type, __pyx_mstate->__pyx_n_u_exc_val, __pyx_mstate->__pyx_n_u_exc_tb};
    __pyx_mstate_global->__pyx_codeobj_tab[36] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_plyvel__plyvel_pyx, __pyx_mstate->__pyx_n_u_exit, __pyx_mstate->__pyx_kp_b_iso88591_A_4s_s_aq_4_D_a_F_F, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[36])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 1, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self};
    __pyx_mstate_global->__pyx_codeobj_tab[37] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_stringsource, __pyx_mstate->__pyx_n_u_reduce_cython, __pyx_mstate->__pyx_kp_b_iso88591_Q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[37])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 3};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_pyx_state};
    __pyx_mstate_global->__pyx_codeobj_tab[38] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_stringsource, __pyx_mstate->__pyx_n_u_setstate_cython, __pyx_mstate->__pyx_kp_b_iso88591_Q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[38])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 1, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 703};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self};
    __pyx_mstate_global->__pyx_codeobj_tab[39] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_plyvel__plyvel_pyx, __pyx_mstate->__pyx_n_u_close, __pyx_mstate->__pyx_kp_b_iso88591_A_4wgQ_A, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[39])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 1, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 711};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self};
    __pyx_mstate_global->__pyx_codeobj_tab[40] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_plyvel__plyvel_pyx, __pyx_mstate->__pyx_n_u_enter, __pyx_mstate->__pyx_kp_b_iso88591_A_q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[40])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {4, 0, 0, 4, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 714};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_exc_type, __pyx_mstate->__pyx_n_u_exc_val, __pyx_mstate->__pyx_n_u_exc_tb};
    __pyx_mstate_global->__pyx_codeobj_tab[41] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_plyvel__plyvel_pyx, __pyx_mstate->__pyx_n_u_exit, __pyx_mstate->__pyx_kp_b_iso88591_A_F_q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[41])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 1, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self};
    __pyx_mstate_global->__pyx_codeobj_tab[42] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_stringsource, __pyx_mstate->__pyx_n_u_reduce_cython, __pyx_mstate->__pyx_kp_b_iso88591_Q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[42])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 3};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_pyx_state};
    __pyx_mstate_global->__pyx_codeobj_tab[43] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_stringsource, __pyx_mstate->__pyx_n_u_setstate_cython, __pyx_mstate->__pyx_kp_b_iso88591_Q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[43])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 1, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 851};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self};
    __pyx_mstate_global->__pyx_codeobj_tab[44] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_plyvel__plyvel_pyx, __pyx_mstate->__pyx_n_u_prev, __pyx_mstate->__pyx_kp_b_iso88591_A_4_Q_4z_4z, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[44])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 1, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 987};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self};
    __pyx_mstate_global->__pyx_codeobj_tab[45] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_plyvel__plyvel_pyx, __pyx_mstate->__pyx_n_u_seek_to_start, __pyx_mstate->__pyx_kp_b_iso88591_A_4wc_aq_IQ, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[45])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 1, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 993};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self};
    __pyx_mstate_global->__pyx_codeobj_tab[46] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_plyvel__plyvel_pyx, __pyx_mstate->__pyx_n_u_seek_to_stop, __pyx_mstate->__pyx_kp_b_iso88591_A_4wc_aq_IQ, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[46])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 3, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 999};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_target, __pyx_mstate->__pyx_n_u_target_slice};
    __pyx_mstate_global->__pyx_codeobj_tab[47] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_plyvel__plyvel_pyx, __pyx_mstate->__pyx_n_u_seek, __pyx_mstate->__pyx_kp_b_iso88591_A_4wc_aq_4_T_Ba_q_1A_4wgU_d_XQ_d, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[47])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 1, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self};
    __pyx_mstate_global->__pyx_codeobj_tab[48] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_stringsource, __pyx_mstate->__pyx_n_u_reduce_cython, __pyx_mstate->__pyx_kp_b_iso88591_Q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[48])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 3};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_pyx_state};
    __pyx_mstate_global->__pyx_codeobj_tab[49] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_stringsource, __pyx_mstate->__pyx_n_u_setstate_cython, __pyx_mstate->__pyx_kp_b_iso88591_Q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[49])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 1, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1029};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self};
    __pyx_mstate_global->__pyx_codeobj_tab[50] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_plyvel__plyvel_pyx, __pyx_mstate->__pyx_n_u_valid, __pyx_mstate->__pyx_kp_b_iso88591_A_4wc_aq_t6_q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[50])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 1, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1035};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self};
    __pyx_mstate_global->__pyx_codeobj_tab[51] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_plyvel__plyvel_pyx, __pyx_mstate->__pyx_n_u_seek_to_first, __pyx_mstate->__pyx_kp_b_iso88591_A_4wc_aq_l_V7, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[51])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 1, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1044};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self};
    __pyx_mstate_global->__pyx_codeobj_tab[52] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_plyvel__plyvel_pyx, __pyx_mstate->__pyx_n_u_seek_to_last, __pyx_mstate->__pyx_kp_b_iso88591_A_4wc_aq_k_V7, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[52])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 3, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1053};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_target, __pyx_mstate->__pyx_n_u_target_slice};
    __pyx_mstate_global->__pyx_codeobj_tab[53] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_plyvel__plyvel_pyx, __pyx_mstate->__pyx_n_u_seek, __pyx_mstate->__pyx_kp_b_iso88591_A_4wc_aq_q_1A_e1A_V7, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[53])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 1, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1063};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self};
    __pyx_mstate_global->__pyx_codeobj_tab[54] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_plyvel__plyvel_pyx, __pyx_mstate->__pyx_n_u_next, __pyx_mstate->__pyx_kp_b_iso88591_A_4wc_aq_4t6_q_a_e1_V7, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[54])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 1, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1075};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self};
    __pyx_mstate_global->__pyx_codeobj_tab[55] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_plyvel__plyvel_pyx, __pyx_mstate->__pyx_n_u_prev, __pyx_mstate->__pyx_kp_b_iso88591_A_4wc_aq_4t6_q_a_e1_V7, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[55])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 1, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1087};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self};
    __pyx_mstate_global->__pyx_codeobj_tab[56] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_plyvel__plyvel_pyx, __pyx_mstate->__pyx_n_u_key, __pyx_mstate->__pyx_kp_b_iso88591_A_4wc_aq_4t6_q_a_D_d_y_Rr_q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[56])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 1, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1098};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self};
    __pyx_mstate_global->__pyx_codeobj_tab[57] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_plyvel__plyvel_pyx, __pyx_mstate->__pyx_n_u_value, __pyx_mstate->__pyx_kp_b_iso88591_A_4wc_aq_4t6_q_a_d_a_r_e1, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[57])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 1, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1109};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self};
    __pyx_mstate_global->__pyx_codeobj_tab[58] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_plyvel__plyvel_pyx, __pyx_mstate->__pyx_n_u_item, __pyx_mstate->__pyx_kp_b_iso88591_A_t4t4vQ, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[58])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 1, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self};
    __pyx_mstate_global->__pyx_codeobj_tab[59] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_stringsource, __pyx_mstate->__pyx_n_u_reduce_cython, __pyx_mstate->__pyx_kp_b_iso88591_Q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[59])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 3};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_pyx_state};
    __pyx_mstate_global->__pyx_codeobj_tab[60] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_stringsource, __pyx_mstate->__pyx_n_u_setstate_cython, __pyx_mstate->__pyx_kp_b_iso88591_Q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[60])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 1, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1135};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self};
    __pyx_mstate_global->__pyx_codeobj_tab[61] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_plyvel__plyvel_pyx, __pyx_mstate->__pyx_n_u_close, __pyx_mstate->__pyx_kp_b_iso88591_A_4s_s_s_k_A_4_q_A_Q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[61])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 1, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1143};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self};
    __pyx_mstate_global->__pyx_codeobj_tab[62] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_plyvel__plyvel_pyx, __pyx_mstate->__pyx_n_u_release, __pyx_mstate->__pyx_kp_b_iso88591_A_F, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[62])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 1, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1146};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self};
    __pyx_mstate_global->__pyx_codeobj_tab[63] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_plyvel__plyvel_pyx, __pyx_mstate->__pyx_n_u_enter, __pyx_mstate->__pyx_kp_b_iso88591_A_q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[63])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {4, 0, 0, 4, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1149};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_exc_type, __pyx_mstate->__pyx_n_u_exc_val, __pyx_mstate->__pyx_n_u_exc_tb};
    __pyx_mstate_global->__pyx_codeobj_tab[64] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_plyvel__plyvel_pyx, __pyx_mstate->__pyx_n_u_exit, __pyx_mstate->__pyx_kp_b_iso88591_A_F_q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[64])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {3, 0, 2, 6, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1153};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_key, __pyx_mstate->__pyx_n_u_default, __pyx_mstate->__pyx_n_u_verify_checksums, __pyx_mstate->__pyx_n_u_fill_cache, __pyx_mstate->__pyx_n_u_read_options};
    __pyx_mstate_global->__pyx_codeobj_tab[65] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_plyvel__plyvel_pyx, __pyx_mstate->__pyx_n_u_get, __pyx_mstate->__pyx_kp_b_iso88591_a_9_4s_s_s_k_A_aq_N_L_A_4xwa_hb, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[65])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 10, 11, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1171};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_reverse, __pyx_mstate->__pyx_n_u_start, __pyx_mstate->__pyx_n_u_stop, __pyx_mstate->__pyx_n_u_include_start, __pyx_mstate->__pyx_n_u_include_stop, __pyx_mstate->__pyx_n_u_prefix, __pyx_mstate->__pyx_n_u_include_key, __pyx_mstate->__pyx_n_u_include_value, __pyx_mstate->__pyx_n_u_verify_checksums, __pyx_mstate->__pyx_n_u_fill_cache};
    __pyx_mstate_global->__pyx_codeobj_tab[66] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_plyvel__plyvel_pyx, __pyx_mstate->__pyx_n_u_iterator, __pyx_mstate->__pyx_kp_b_iso88591_2_1_a_4s_s_s_k_A_aq_xq_t5_ixy_a, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[66])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 2, 3, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1185};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_verify_checksums, __pyx_mstate->__pyx_n_u_fill_cache};
    __pyx_mstate_global->__pyx_codeobj_tab[67] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_plyvel__plyvel_pyx, __pyx_mstate->__pyx_n_u_raw_iterator, __pyx_mstate->__pyx_kp_b_iso88591_4A_4s_s_s_k_A_aq_t1_Q_q_Q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[67])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {1, 0, 0, 1, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 1};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self};
    __pyx_mstate_global->__pyx_codeobj_tab[68] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_stringsource, __pyx_mstate->__pyx_n_u_reduce_cython, __pyx_mstate->__pyx_kp_b_iso88591_Q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[68])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {2, 0, 0, 2, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 3};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_self, __pyx_mstate->__pyx_n_u_pyx_state};
    __pyx_mstate_global->__pyx_codeobj_tab[69] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_stringsource, __pyx_mstate->__pyx_n_u_setstate_cython, __pyx_mstate->__pyx_kp_b_iso88591_Q, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[69])) goto bad;
  }
  {
    const __Pyx_PyCode_New_function_description descr = {3, 0, 0, 4, (unsigned int)(CO_OPTIMIZED|CO_NEWLOCALS), 4};
    PyObject* const varnames[] = {__pyx_mstate->__pyx_n_u_pyx_type, __pyx_mstate->__pyx_n_u_pyx_checksum, __pyx_mstate->__pyx_n_u_pyx_state, __pyx_mstate->__pyx_n_u_pyx_result};
    __pyx_mstate_global->__pyx_codeobj_tab[70] = __Pyx_PyCode_New(descr, varnames, __pyx_mstate->__pyx_kp_u_stringsource, __pyx_mstate->__pyx_n_u_pyx_unpickle_PrefixedDB, __pyx_mstate->__pyx_kp_b_iso88591_q_0_kQR_XQa_7_A_1, tuple_dedup_map); if (unlikely(!__pyx_mstate_global->__pyx_codeobj_tab[70])) goto bad;
  }
  Py_DECREF(tuple_dedup_map);
  return 0;
  bad:
  Py_DECREF(tuple_dedup_map);
  return -1;
}
/* #### Code section: init_globals ### */

static int __Pyx_InitGlobals(void) {
  /* PythonCompatibility.init */
  if (likely(__Pyx_init_co_variables() == 0)); else
  
  if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1, __pyx_L1_error)

  /* CommonTypesMetaclass.init */
  if (likely(__pyx_CommonTypesMetaclass_init(__pyx_m) == 0)); else
  
  if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1, __pyx_L1_error)

  /* CachedMethodType.init */
  #if CYTHON_COMPILING_IN_LIMITED_API
  {
      PyObject *typesModule=NULL;
      typesModule = PyImport_ImportModule("types");
      if (typesModule) {
          __pyx_mstate_global->__Pyx_CachedMethodType = PyObject_GetAttrString(typesModule, "MethodType");
          Py_DECREF(typesModule);
      }
  } // error handling follows
  #endif
  
  if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1, __pyx_L1_error)

  /* CythonFunctionShared.init */
  if (likely(__pyx_CyFunction_init(__pyx_m) == 0)); else
  
  if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1, __pyx_L1_error)

  /* AssertionsEnabled.init */
  if (likely(__Pyx_init_assertions_enabled() == 0)); else
  
  if (unlikely(PyErr_Occurred())) __PYX_ERR(0, 1, __pyx_L1_error)

  return 0;
  __pyx_L1_error:;
  return -1;
}
/* #### Code section: cleanup_globals ### */
/* #### Code section: cleanup_module ### */
/* #### Code section: main_method ### */
/* #### Code section: utility_code_pragmas ### */
#ifdef _MSC_VER
#pragma warning( push )
/* Warning 4127: conditional expression is constant
 * Cython uses constant conditional expressions to allow in inline functions to be optimized at
 * compile-time, so this warning is not useful
 */
#pragma warning( disable : 4127 )
#endif



/* #### Code section: utility_code_def ### */

/* --- Runtime support code --- */
/* Refnanny */
#if CYTHON_REFNANNY
static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname) {
    PyObject *m = NULL, *p = NULL;
    void *r = NULL;
    m = PyImport_ImportModule(modname);
    if (!m) goto end;
    p = PyObject_GetAttrString(m, "RefNannyAPI");
    if (!p) goto end;
    r = PyLong_AsVoidPtr(p);
end:
    Py_XDECREF(p);
    Py_XDECREF(m);
    return (__Pyx_RefNannyAPIStruct *)r;
}
#endif

/* PyErrExceptionMatches (used by PyObjectGetAttrStrNoError) */
#if CYTHON_FAST_THREAD_STATE
static int __Pyx_PyErr_ExceptionMatchesTuple(PyObject *exc_type, PyObject *tuple) {
    Py_ssize_t i, n;
    n = PyTuple_GET_SIZE(tuple);
    for (i=0; i<n; i++) {
        if (exc_type == PyTuple_GET_ITEM(tuple, i)) return 1;
    }
    for (i=0; i<n; i++) {
        if (__Pyx_PyErr_GivenExceptionMatches(exc_type, PyTuple_GET_ITEM(tuple, i))) return 1;
    }
    return 0;
}
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err) {
    int result;
    PyObject *exc_type;
#if PY_VERSION_HEX >= 0x030C00A6
    PyObject *current_exception = tstate->current_exception;
    if (unlikely(!current_exception)) return 0;
    exc_type = (PyObject*) Py_TYPE(current_exception);
    if (exc_type == err) return 1;
#else
    exc_type = tstate->curexc_type;
    if (exc_type == err) return 1;
    if (unlikely(!exc_type)) return 0;
#endif
    #if CYTHON_AVOID_BORROWED_REFS
    Py_INCREF(exc_type);
    #endif
    if (unlikely(PyTuple_Check(err))) {
        result = __Pyx_PyErr_ExceptionMatchesTuple(exc_type, err);
    } else {
        result = __Pyx_PyErr_GivenExceptionMatches(exc_type, err);
    }
    #if CYTHON_AVOID_BORROWED_REFS
    Py_DECREF(exc_type);
    #endif
    return result;
}
#endif

/* PyErrFetchRestore (used by PyObjectGetAttrStrNoError) */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
#if PY_VERSION_HEX >= 0x030C00A6
    PyObject *tmp_value;
    assert(type == NULL || (value != NULL && type == (PyObject*) Py_TYPE(value)));
    if (value) {
        #if CYTHON_COMPILING_IN_CPYTHON
        if (unlikely(((PyBaseExceptionObject*) value)->traceback != tb))
        #endif
            PyException_SetTraceback(value, tb);
    }
    tmp_value = tstate->current_exception;
    tstate->current_exception = value;
    Py_XDECREF(tmp_value);
    Py_XDECREF(type);
    Py_XDECREF(tb);
#else
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    tmp_type = tstate->curexc_type;
    tmp_value = tstate->curexc_value;
    tmp_tb = tstate->curexc_traceback;
    tstate->curexc_type = type;
    tstate->curexc_value = value;
    tstate->curexc_traceback = tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
#endif
}
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
#if PY_VERSION_HEX >= 0x030C00A6
    PyObject* exc_value;
    exc_value = tstate->current_exception;
    tstate->current_exception = 0;
    *value = exc_value;
    *type = NULL;
    *tb = NULL;
    if (exc_value) {
        *type = (PyObject*) Py_TYPE(exc_value);
        Py_INCREF(*type);
        #if CYTHON_COMPILING_IN_CPYTHON
        *tb = ((PyBaseExceptionObject*) exc_value)->traceback;
        Py_XINCREF(*tb);
        #else
        *tb = PyException_GetTraceback(exc_value);
        #endif
    }
#else
    *type = tstate->curexc_type;
    *value = tstate->curexc_value;
    *tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
#endif
}
#endif

/* PyObjectGetAttrStr (used by PyObjectGetAttrStrNoError) */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro))
        return tp->tp_getattro(obj, attr_name);
    return PyObject_GetAttr(obj, attr_name);
}
#endif

/* PyObjectGetAttrStrNoError (used by GetBuiltinName) */
#if __PYX_LIMITED_VERSION_HEX < 0x030d0000
static void __Pyx_PyObject_GetAttrStr_ClearAttributeError(void) {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    if (likely(__Pyx_PyErr_ExceptionMatches(PyExc_AttributeError)))
        __Pyx_PyErr_Clear();
}
#endif
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStrNoError(PyObject* obj, PyObject* attr_name) {
    PyObject *result;
#if __PYX_LIMITED_VERSION_HEX >= 0x030d0000
    (void) PyObject_GetOptionalAttr(obj, attr_name, &result);
    return result;
#else
#if CYTHON_COMPILING_IN_CPYTHON && CYTHON_USE_TYPE_SLOTS
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro == PyObject_GenericGetAttr)) {
        return _PyObject_GenericGetAttrWithDict(obj, attr_name, NULL, 1);
    }
#endif
    result = __Pyx_PyObject_GetAttrStr(obj, attr_name);
    if (unlikely(!result)) {
        __Pyx_PyObject_GetAttrStr_ClearAttributeError();
    }
    return result;
#endif
}

/* GetBuiltinName */
static PyObject *__Pyx_GetBuiltinName(PyObject *name) {
    PyObject* result = __Pyx_PyObject_GetAttrStrNoError(__pyx_mstate_global->__pyx_b, name);
    if (unlikely(!result) && !PyErr_Occurred()) {
        PyErr_Format(PyExc_NameError,
            "name '%U' is not defined", name);
    }
    return result;
}

/* PyDictVersioning (used by GetModuleGlobalName) */
#if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PY_UINT64_T __Pyx_get_tp_dict_version(PyObject *obj) {
    PyObject *dict = Py_TYPE(obj)->tp_dict;
    return likely(dict) ? __PYX_GET_DICT_VERSION(dict) : 0;
}
static CYTHON_INLINE PY_UINT64_T __Pyx_get_object_dict_version(PyObject *obj) {
    PyObject **dictptr = NULL;
    Py_ssize_t offset = Py_TYPE(obj)->tp_dictoffset;
    if (offset) {
#if CYTHON_COMPILING_IN_CPYTHON
        dictptr = (likely(offset > 0)) ? (PyObject **) ((char *)obj + offset) : _PyObject_GetDictPtr(obj);
#else
        dictptr = _PyObject_GetDictPtr(obj);
#endif
    }
    return (dictptr && *dictptr) ? __PYX_GET_DICT_VERSION(*dictptr) : 0;
}
static CYTHON_INLINE int __Pyx_object_dict_version_matches(PyObject* obj, PY_UINT64_T tp_dict_version, PY_UINT64_T obj_dict_version) {
    PyObject *dict = Py_TYPE(obj)->tp_dict;
    if (unlikely(!dict) || unlikely(tp_dict_version != __PYX_GET_DICT_VERSION(dict)))
        return 0;
    return obj_dict_version == __Pyx_get_object_dict_version(obj);
}
#endif

/* GetModuleGlobalName */
#if CYTHON_USE_DICT_VERSIONS
static PyObject *__Pyx__GetModuleGlobalName(PyObject *name, PY_UINT64_T *dict_version, PyObject **dict_cached_value)
#else
static CYTHON_INLINE PyObject *__Pyx__GetModuleGlobalName(PyObject *name)
#endif
{
    PyObject *result;
#if CYTHON_COMPILING_IN_LIMITED_API
    if (unlikely(!__pyx_m)) {
        if (!PyErr_Occurred())
            PyErr_SetNone(PyExc_NameError);
        return NULL;
    }
    result = PyObject_GetAttr(__pyx_m, name);
    if (likely(result)) {
        return result;
    }
    PyErr_Clear();
#elif CYTHON_AVOID_BORROWED_REFS || CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS
    if (unlikely(__Pyx_PyDict_GetItemRef(__pyx_mstate_global->__pyx_d, name, &result) == -1)) PyErr_Clear();
    __PYX_UPDATE_DICT_CACHE(__pyx_mstate_global->__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return result;
    }
#else
    result = _PyDict_GetItem_KnownHash(__pyx_mstate_global->__pyx_d, name, ((PyASCIIObject *) name)->hash);
    __PYX_UPDATE_DICT_CACHE(__pyx_mstate_global->__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    }
    PyErr_Clear();
#endif
    return __Pyx_GetBuiltinName(name);
}

/* PyObjectCall (used by PyObjectFastCall) */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    PyObject *result;
    ternaryfunc call = Py_TYPE(func)->tp_call;
    if (unlikely(!call))
        return PyObject_Call(func, arg, kw);
    if (unlikely(Py_EnterRecursiveCall(" while calling a Python object")))
        return NULL;
    result = (*call)(func, arg, kw);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectCallMethO (used by PyObjectFastCall) */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg) {
    PyObject *self, *result;
    PyCFunction cfunc;
    cfunc = __Pyx_CyOrPyCFunction_GET_FUNCTION(func);
    self = __Pyx_CyOrPyCFunction_GET_SELF(func);
    if (unlikely(Py_EnterRecursiveCall(" while calling a Python object")))
        return NULL;
    result = cfunc(self, arg);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectFastCall */
#if PY_VERSION_HEX < 0x03090000 || CYTHON_COMPILING_IN_LIMITED_API
static PyObject* __Pyx_PyObject_FastCall_fallback(PyObject *func, PyObject * const*args, size_t nargs, PyObject *kwargs) {
    PyObject *argstuple;
    PyObject *result = 0;
    size_t i;
    argstuple = PyTuple_New((Py_ssize_t)nargs);
    if (unlikely(!argstuple)) return NULL;
    for (i = 0; i < nargs; i++) {
        Py_INCREF(args[i]);
        if (__Pyx_PyTuple_SET_ITEM(argstuple, (Py_ssize_t)i, args[i]) != (0)) goto bad;
    }
    result = __Pyx_PyObject_Call(func, argstuple, kwargs);
  bad:
    Py_DECREF(argstuple);
    return result;
}
#endif
#if CYTHON_VECTORCALL && !CYTHON_COMPILING_IN_LIMITED_API
  #if PY_VERSION_HEX < 0x03090000
    #define __Pyx_PyVectorcall_Function(callable) _PyVectorcall_Function(callable)
  #elif CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE vectorcallfunc __Pyx_PyVectorcall_Function(PyObject *callable) {
    PyTypeObject *tp = Py_TYPE(callable);
    #if defined(__Pyx_CyFunction_USED)
    if (__Pyx_CyFunction_CheckExact(callable)) {
        return __Pyx_CyFunction_func_vectorcall(callable);
    }
    #endif
    if (!PyType_HasFeature(tp, Py_TPFLAGS_HAVE_VECTORCALL)) {
        return NULL;
    }
    assert(PyCallable_Check(callable));
    Py_ssize_t offset = tp->tp_vectorcall_offset;
    assert(offset > 0);
    vectorcallfunc ptr;
    memcpy(&ptr, (char *) callable + offset, sizeof(ptr));
    return ptr;
}
  #else
    #define __Pyx_PyVectorcall_Function(callable) PyVectorcall_Function(callable)
  #endif
#endif
static CYTHON_INLINE PyObject* __Pyx_PyObject_FastCallDict(PyObject *func, PyObject *const *args, size_t _nargs, PyObject *kwargs) {
    Py_ssize_t nargs = __Pyx_PyVectorcall_NARGS(_nargs);
#if CYTHON_COMPILING_IN_CPYTHON
    if (nargs == 0 && kwargs == NULL) {
        if (__Pyx_CyOrPyCFunction_Check(func) && likely( __Pyx_CyOrPyCFunction_GET_FLAGS(func) & METH_NOARGS))
            return __Pyx_PyObject_CallMethO(func, NULL);
    }
    else if (nargs == 1 && kwargs == NULL) {
        if (__Pyx_CyOrPyCFunction_Check(func) && likely( __Pyx_CyOrPyCFunction_GET_FLAGS(func) & METH_O))
            return __Pyx_PyObject_CallMethO(func, args[0]);
    }
#endif
    if (kwargs == NULL) {
        #if CYTHON_VECTORCALL
          #if CYTHON_COMPILING_IN_LIMITED_API
            return PyObject_Vectorcall(func, args, _nargs, NULL);
          #else
            vectorcallfunc f = __Pyx_PyVectorcall_Function(func);
            if (f) {
                return f(func, args, _nargs, NULL);
            }
          #endif
        #endif
    }
    if (nargs == 0) {
        return __Pyx_PyObject_Call(func, __pyx_mstate_global->__pyx_empty_tuple, kwargs);
    }
    #if PY_VERSION_HEX >= 0x03090000 && !CYTHON_COMPILING_IN_LIMITED_API
    return PyObject_VectorcallDict(func, args, (size_t)nargs, kwargs);
    #else
    return __Pyx_PyObject_FastCall_fallback(func, args, (size_t)nargs, kwargs);
    #endif
}

/* RaiseException */
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause) {
    PyObject* owned_instance = NULL;
    if (tb == Py_None) {
        tb = 0;
    } else if (tb && !PyTraceBack_Check(tb)) {
        PyErr_SetString(PyExc_TypeError,
            "raise: arg 3 must be a traceback or None");
        goto bad;
    }
    if (value == Py_None)
        value = 0;
    if (PyExceptionInstance_Check(type)) {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto bad;
        }
        value = type;
        type = (PyObject*) Py_TYPE(value);
    } else if (PyExceptionClass_Check(type)) {
        PyObject *instance_class = NULL;
        if (value && PyExceptionInstance_Check(value)) {
            instance_class = (PyObject*) Py_TYPE(value);
            if (instance_class != type) {
                int is_subclass = PyObject_IsSubclass(instance_class, type);
                if (!is_subclass) {
                    instance_class = NULL;
                } else if (unlikely(is_subclass == -1)) {
                    goto bad;
                } else {
                    type = instance_class;
                }
            }
        }
        if (!instance_class) {
            PyObject *args;
            if (!value)
                args = PyTuple_New(0);
            else if (PyTuple_Check(value)) {
                Py_INCREF(value);
                args = value;
            } else
                args = PyTuple_Pack(1, value);
            if (!args)
                goto bad;
            owned_instance = PyObject_Call(type, args, NULL);
            Py_DECREF(args);
            if (!owned_instance)
                goto bad;
            value = owned_instance;
            if (!PyExceptionInstance_Check(value)) {
                PyErr_Format(PyExc_TypeError,
                             "calling %R should have returned an instance of "
                             "BaseException, not %R",
                             type, Py_TYPE(value));
                goto bad;
            }
        }
    } else {
        PyErr_SetString(PyExc_TypeError,
            "raise: exception class must be a subclass of BaseException");
        goto bad;
    }
    if (cause) {
        PyObject *fixed_cause;
        if (cause == Py_None) {
            fixed_cause = NULL;
        } else if (PyExceptionClass_Check(cause)) {
            fixed_cause = PyObject_CallObject(cause, NULL);
            if (fixed_cause == NULL)
                goto bad;
        } else if (PyExceptionInstance_Check(cause)) {
            fixed_cause = cause;
            Py_INCREF(fixed_cause);
        } else {
            PyErr_SetString(PyExc_TypeError,
                            "exception causes must derive from "
                            "BaseException");
            goto bad;
        }
        PyException_SetCause(value, fixed_cause);
    }
    PyErr_SetObject(type, value);
    if (tb) {
#if PY_VERSION_HEX >= 0x030C00A6
        PyException_SetTraceback(value, tb);
#elif CYTHON_FAST_THREAD_STATE
        PyThreadState *tstate = __Pyx_PyThreadState_Current;
        PyObject* tmp_tb = tstate->curexc_traceback;
        if (tb != tmp_tb) {
            Py_INCREF(tb);
            tstate->curexc_traceback = tb;
            Py_XDECREF(tmp_tb);
        }
#else
        PyObject *tmp_type, *tmp_value, *tmp_tb;
        PyErr_Fetch(&tmp_type, &tmp_value, &tmp_tb);
        Py_INCREF(tb);
        PyErr_Restore(tmp_type, tmp_value, tb);
        Py_XDECREF(tmp_tb);
#endif
    }
bad:
    Py_XDECREF(owned_instance);
    return;
}

/* RaiseUnexpectedTypeError */
static int
__Pyx_RaiseUnexpectedTypeError(const char *expected, PyObject *obj)
{
    __Pyx_TypeName obj_type_name = __Pyx_PyType_GetFullyQualifiedName(Py_TYPE(obj));
    PyErr_Format(PyExc_TypeError, "Expected %s, got " __Pyx_FMT_TYPENAME,
                 expected, obj_type_name);
    __Pyx_DECREF_TypeName(obj_type_name);
    return 0;
}

/* PyObjectFastCallMethod */
#if !CYTHON_VECTORCALL || PY_VERSION_HEX < 0x03090000
static PyObject *__Pyx_PyObject_FastCallMethod(PyObject *name, PyObject *const *args, size_t nargsf) {
    PyObject *result;
    PyObject *attr = PyObject_GetAttr(args[0], name);
    if (unlikely(!attr))
        return NULL;
    result = __Pyx_PyObject_FastCall(attr, args+1, nargsf - 1);
    Py_DECREF(attr);
    return result;
}
#endif

/* GetTopmostException (used by SaveResetException) */
#if CYTHON_USE_EXC_INFO_STACK && CYTHON_FAST_THREAD_STATE
static _PyErr_StackItem *
__Pyx_PyErr_GetTopmostException(PyThreadState *tstate)
{
    _PyErr_StackItem *exc_info = tstate->exc_info;
    while ((exc_info->exc_value == NULL || exc_info->exc_value == Py_None) &&
           exc_info->previous_item != NULL)
    {
        exc_info = exc_info->previous_item;
    }
    return exc_info;
}
#endif

/* SaveResetException */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx__ExceptionSave(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
  #if CYTHON_USE_EXC_INFO_STACK && PY_VERSION_HEX >= 0x030B00a4
    _PyErr_StackItem *exc_info = __Pyx_PyErr_GetTopmostException(tstate);
    PyObject *exc_value = exc_info->exc_value;
    if (exc_value == NULL || exc_value == Py_None) {
        *value = NULL;
        *type = NULL;
        *tb = NULL;
    } else {
        *value = exc_value;
        Py_INCREF(*value);
        *type = (PyObject*) Py_TYPE(exc_value);
        Py_INCREF(*type);
        *tb = PyException_GetTraceback(exc_value);
    }
  #elif CYTHON_USE_EXC_INFO_STACK
    _PyErr_StackItem *exc_info = __Pyx_PyErr_GetTopmostException(tstate);
    *type = exc_info->exc_type;
    *value = exc_info->exc_value;
    *tb = exc_info->exc_traceback;
    Py_XINCREF(*type);
    Py_XINCREF(*value);
    Py_XINCREF(*tb);
  #else
    *type = tstate->exc_type;
    *value = tstate->exc_value;
    *tb = tstate->exc_traceback;
    Py_XINCREF(*type);
    Py_XINCREF(*value);
    Py_XINCREF(*tb);
  #endif
}
static CYTHON_INLINE void __Pyx__ExceptionReset(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
  #if CYTHON_USE_EXC_INFO_STACK && PY_VERSION_HEX >= 0x030B00a4
    _PyErr_StackItem *exc_info = tstate->exc_info;
    PyObject *tmp_value = exc_info->exc_value;
    exc_info->exc_value = value;
    Py_XDECREF(tmp_value);
    Py_XDECREF(type);
    Py_XDECREF(tb);
  #else
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    #if CYTHON_USE_EXC_INFO_STACK
    _PyErr_StackItem *exc_info = tstate->exc_info;
    tmp_type = exc_info->exc_type;
    tmp_value = exc_info->exc_value;
    tmp_tb = exc_info->exc_traceback;
    exc_info->exc_type = type;
    exc_info->exc_value = value;
    exc_info->exc_traceback = tb;
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = type;
    tstate->exc_value = value;
    tstate->exc_traceback = tb;
    #endif
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
  #endif
}
#endif

/* GetException */
#if CYTHON_FAST_THREAD_STATE
static int __Pyx__GetException(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb)
#else
static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb)
#endif
{
    PyObject *local_type = NULL, *local_value, *local_tb = NULL;
#if CYTHON_FAST_THREAD_STATE
    PyObject *tmp_type, *tmp_value, *tmp_tb;
  #if PY_VERSION_HEX >= 0x030C0000
    local_value = tstate->current_exception;
    tstate->current_exception = 0;
  #else
    local_type = tstate->curexc_type;
    local_value = tstate->curexc_value;
    local_tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
  #endif
#elif __PYX_LIMITED_VERSION_HEX > 0x030C0000
    local_value = PyErr_GetRaisedException();
#else
    PyErr_Fetch(&local_type, &local_value, &local_tb);
#endif
#if __PYX_LIMITED_VERSION_HEX > 0x030C0000
    if (likely(local_value)) {
        local_type = (PyObject*) Py_TYPE(local_value);
        Py_INCREF(local_type);
        local_tb = PyException_GetTraceback(local_value);
    }
#else
    PyErr_NormalizeException(&local_type, &local_value, &local_tb);
#if CYTHON_FAST_THREAD_STATE
    if (unlikely(tstate->curexc_type))
#else
    if (unlikely(PyErr_Occurred()))
#endif
        goto bad;
    if (local_tb) {
        if (unlikely(PyException_SetTraceback(local_value, local_tb) < 0))
            goto bad;
    }
#endif // __PYX_LIMITED_VERSION_HEX > 0x030C0000
    Py_XINCREF(local_tb);
    Py_XINCREF(local_type);
    Py_XINCREF(local_value);
    *type = local_type;
    *value = local_value;
    *tb = local_tb;
#if CYTHON_FAST_THREAD_STATE
    #if CYTHON_USE_EXC_INFO_STACK
    {
        _PyErr_StackItem *exc_info = tstate->exc_info;
      #if PY_VERSION_HEX >= 0x030B00a4
        tmp_value = exc_info->exc_value;
        exc_info->exc_value = local_value;
        tmp_type = NULL;
        tmp_tb = NULL;
        Py_XDECREF(local_type);
        Py_XDECREF(local_tb);
      #else
        tmp_type = exc_info->exc_type;
        tmp_value = exc_info->exc_value;
        tmp_tb = exc_info->exc_traceback;
        exc_info->exc_type = local_type;
        exc_info->exc_value = local_value;
        exc_info->exc_traceback = local_tb;
      #endif
    }
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = local_type;
    tstate->exc_value = local_value;
    tstate->exc_traceback = local_tb;
    #endif
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
#elif __PYX_LIMITED_VERSION_HEX >= 0x030b0000
    PyErr_SetHandledException(local_value);
    Py_XDECREF(local_value);
    Py_XDECREF(local_type);
    Py_XDECREF(local_tb);
#else
    PyErr_SetExcInfo(local_type, local_value, local_tb);
#endif
    return 0;
#if __PYX_LIMITED_VERSION_HEX <= 0x030C0000
bad:
    *type = 0;
    *value = 0;
    *tb = 0;
    Py_XDECREF(local_type);
    Py_XDECREF(local_value);
    Py_XDECREF(local_tb);
    return -1;
#endif
}

/* SwapException */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx__ExceptionSwap(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
  #if CYTHON_USE_EXC_INFO_STACK && PY_VERSION_HEX >= 0x030B00a4
    _PyErr_StackItem *exc_info = tstate->exc_info;
    tmp_value = exc_info->exc_value;
    exc_info->exc_value = *value;
    if (tmp_value == NULL || tmp_value == Py_None) {
        Py_XDECREF(tmp_value);
        tmp_value = NULL;
        tmp_type = NULL;
        tmp_tb = NULL;
    } else {
        tmp_type = (PyObject*) Py_TYPE(tmp_value);
        Py_INCREF(tmp_type);
        #if CYTHON_COMPILING_IN_CPYTHON
        tmp_tb = ((PyBaseExceptionObject*) tmp_value)->traceback;
        Py_XINCREF(tmp_tb);
        #else
        tmp_tb = PyException_GetTraceback(tmp_value);
        #endif
    }
  #elif CYTHON_USE_EXC_INFO_STACK
    _PyErr_StackItem *exc_info = tstate->exc_info;
    tmp_type = exc_info->exc_type;
    tmp_value = exc_info->exc_value;
    tmp_tb = exc_info->exc_traceback;
    exc_info->exc_type = *type;
    exc_info->exc_value = *value;
    exc_info->exc_traceback = *tb;
  #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = *type;
    tstate->exc_value = *value;
    tstate->exc_traceback = *tb;
  #endif
    *type = tmp_type;
    *value = tmp_value;
    *tb = tmp_tb;
}
#else
static CYTHON_INLINE void __Pyx_ExceptionSwap(PyObject **type, PyObject **value, PyObject **tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    PyErr_GetExcInfo(&tmp_type, &tmp_value, &tmp_tb);
    PyErr_SetExcInfo(*type, *value, *tb);
    *type = tmp_type;
    *value = tmp_value;
    *tb = tmp_tb;
}
#endif

/* SetStringIndexingError (used by GetItemIntByteArray) */
static void __Pyx_SetStringIndexingError(const char* message, int has_gil) {
    if (!has_gil) {
        PyGILState_STATE gil_state = PyGILState_Ensure();
        PyErr_SetString(PyExc_IndexError, message);
        PyGILState_Release(gil_state);
    } else
        PyErr_SetString(PyExc_IndexError, message);
}

/* GetItemIntByteArray */
static CYTHON_INLINE int __Pyx_GetItemInt_ByteArray_Fast_Locked(PyObject* string, Py_ssize_t i,
                                                                int wraparound, int boundscheck, int has_gil) {
    Py_ssize_t length = __Pyx_PyByteArray_GET_SIZE(string);
    #if !CYTHON_ASSUME_SAFE_SIZE
    if (unlikely(length < 0)) return -1;
    #endif
    if (wraparound & unlikely(i < 0)) i += length;
    if ((!boundscheck) || likely(__Pyx_is_valid_index(i, length))) {
        #if !CYTHON_ASSUME_SAFE_MACROS
        char *asString = PyByteArray_AsString(string);
        return likely(asString) ? (unsigned char) asString[i] : -1;
        #else
        return (unsigned char) (PyByteArray_AS_STRING(string)[i]);
        #endif
    } else {
        __Pyx_SetStringIndexingError("bytearray index out of range", has_gil);
        return -1;
    }
}
static CYTHON_INLINE int __Pyx_GetItemInt_ByteArray_Fast(PyObject* string, Py_ssize_t i,
                                                         int wraparound, int boundscheck, int has_gil, int unsafe_shared) {
    CYTHON_MAYBE_UNUSED_VAR(unsafe_shared);
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    wraparound = wraparound && i<0;
#endif
    if (wraparound | boundscheck) {
        int result;
        __Pyx_PyCriticalSection cs;
        int lock = CYTHON_COMPILING_IN_CPYTHON_FREETHREADING && has_gil && !__Pyx_IS_UNIQUELY_REFERENCED(string, unsafe_shared);
        if (lock) { __Pyx_PyCriticalSection_Begin(&cs, string); }
        result = __Pyx_GetItemInt_ByteArray_Fast_Locked(string, i, wraparound, boundscheck, has_gil);
        if (lock) { __Pyx_PyCriticalSection_End(&cs); }
        return result;
    } else {
        #if !CYTHON_ASSUME_SAFE_MACROS
        char *asString = PyByteArray_AsString(string);
        return likely(asString) ? (unsigned char) asString[i] : -1;
        #else
        return (unsigned char) (PyByteArray_AS_STRING(string)[i]);
        #endif
    }
}

/* SetItemIntByteArray */
static CYTHON_INLINE int __Pyx_SetItemInt_ByteArray_Fast_Locked(PyObject* string, Py_ssize_t i, unsigned char v,
                                                                int wraparound, int boundscheck, int has_gil) {
    Py_ssize_t length = __Pyx_PyByteArray_GET_SIZE(string);
    #if !CYTHON_ASSUME_SAFE_SIZE
    if (unlikely(length < 0)) return -1;
    #endif
    if (wraparound & unlikely(i < 0)) i += length;
    if ((!boundscheck) || likely(__Pyx_is_valid_index(i, length))) {
        #if !CYTHON_ASSUME_SAFE_MACROS
        char *asString = PyByteArray_AsString(string);
        if (unlikely(!asString)) return -1;
        asString[i] = (char)v;
        #else
        PyByteArray_AS_STRING(string)[i] = (char) v;
        #endif
        return 0;
    } else {
        __Pyx_SetStringIndexingError("bytearray index out of range", has_gil);
        return -1;
    }
}
static CYTHON_INLINE int __Pyx_SetItemInt_ByteArray_Fast(PyObject* string, Py_ssize_t i, unsigned char v,
                                                         int wraparound, int boundscheck, int has_gil, int unsafe_shared) {
    CYTHON_MAYBE_UNUSED_VAR(unsafe_shared);
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    wraparound = wraparound && i<0;
#endif
    if (wraparound | boundscheck) {
        int result;
        __Pyx_PyCriticalSection cs;
        int lock = CYTHON_COMPILING_IN_CPYTHON_FREETHREADING && has_gil && !__Pyx_IS_UNIQUELY_REFERENCED(string, unsafe_shared);
        if (lock) { __Pyx_PyCriticalSection_Begin(&cs, string); }
        result = __Pyx_SetItemInt_ByteArray_Fast_Locked(string, i, v, wraparound, boundscheck, has_gil);
        if (lock) { __Pyx_PyCriticalSection_End(&cs); }
        return result;
    } else {
        #if !CYTHON_ASSUME_SAFE_MACROS
        char *asString = PyByteArray_AsString(string);
        if (unlikely(!asString)) return -1;
        asString[i] = (char)v;
        #else
        PyByteArray_AS_STRING(string)[i] = (char) v;
        #endif
        return 0;
    }
}

/* BytesEquals (used by UnicodeEquals) */
static CYTHON_INLINE int __Pyx_PyBytes_Equals(PyObject* s1, PyObject* s2, int equals) {
#if CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API || CYTHON_COMPILING_IN_GRAAL ||\
        !(CYTHON_ASSUME_SAFE_SIZE && CYTHON_ASSUME_SAFE_MACROS)
    return PyObject_RichCompareBool(s1, s2, equals);
#else
    if (s1 == s2) {
        return (equals == Py_EQ);
    } else if (PyBytes_CheckExact(s1) & PyBytes_CheckExact(s2)) {
        const char *ps1, *ps2;
        Py_ssize_t length = PyBytes_GET_SIZE(s1);
        if (length != PyBytes_GET_SIZE(s2))
            return (equals == Py_NE);
        ps1 = PyBytes_AS_STRING(s1);
        ps2 = PyBytes_AS_STRING(s2);
        if (ps1[0] != ps2[0]) {
            return (equals == Py_NE);
        } else if (length == 1) {
            return (equals == Py_EQ);
        } else {
            int result;
#if CYTHON_USE_UNICODE_INTERNALS && (PY_VERSION_HEX < 0x030B0000)
            Py_hash_t hash1, hash2;
            hash1 = ((PyBytesObject*)s1)->ob_shash;
            hash2 = ((PyBytesObject*)s2)->ob_shash;
            if (hash1 != hash2 && hash1 != -1 && hash2 != -1) {
                return (equals == Py_NE);
            }
#endif
            result = memcmp(ps1, ps2, (size_t)length);
            return (equals == Py_EQ) ? (result == 0) : (result != 0);
        }
    } else if ((s1 == Py_None) & PyBytes_CheckExact(s2)) {
        return (equals == Py_NE);
    } else if ((s2 == Py_None) & PyBytes_CheckExact(s1)) {
        return (equals == Py_NE);
    } else {
        int result;
        PyObject* py_result = PyObject_RichCompare(s1, s2, equals);
        if (!py_result)
            return -1;
        result = __Pyx_PyObject_IsTrue(py_result);
        Py_DECREF(py_result);
        return result;
    }
#endif
}

/* UnicodeEquals */
static CYTHON_INLINE int __Pyx_PyUnicode_Equals(PyObject* s1, PyObject* s2, int equals) {
#if CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API || CYTHON_COMPILING_IN_GRAAL
    return PyObject_RichCompareBool(s1, s2, equals);
#else
    int s1_is_unicode, s2_is_unicode;
    if (s1 == s2) {
        goto return_eq;
    }
    s1_is_unicode = PyUnicode_CheckExact(s1);
    s2_is_unicode = PyUnicode_CheckExact(s2);
    if (s1_is_unicode & s2_is_unicode) {
        Py_ssize_t length, length2;
        int kind;
        void *data1, *data2;
        #if !CYTHON_COMPILING_IN_LIMITED_API
        if (unlikely(__Pyx_PyUnicode_READY(s1) < 0) || unlikely(__Pyx_PyUnicode_READY(s2) < 0))
            return -1;
        #endif
        length = __Pyx_PyUnicode_GET_LENGTH(s1);
        #if !CYTHON_ASSUME_SAFE_SIZE
        if (unlikely(length < 0)) return -1;
        #endif
        length2 = __Pyx_PyUnicode_GET_LENGTH(s2);
        #if !CYTHON_ASSUME_SAFE_SIZE
        if (unlikely(length2 < 0)) return -1;
        #endif
        if (length != length2) {
            goto return_ne;
        }
#if CYTHON_USE_UNICODE_INTERNALS
        {
            Py_hash_t hash1, hash2;
            hash1 = ((PyASCIIObject*)s1)->hash;
            hash2 = ((PyASCIIObject*)s2)->hash;
            if (hash1 != hash2 && hash1 != -1 && hash2 != -1) {
                goto return_ne;
            }
        }
#endif
        kind = __Pyx_PyUnicode_KIND(s1);
        if (kind != __Pyx_PyUnicode_KIND(s2)) {
            goto return_ne;
        }
        data1 = __Pyx_PyUnicode_DATA(s1);
        data2 = __Pyx_PyUnicode_DATA(s2);
        if (__Pyx_PyUnicode_READ(kind, data1, 0) != __Pyx_PyUnicode_READ(kind, data2, 0)) {
            goto return_ne;
        } else if (length == 1) {
            goto return_eq;
        } else {
            int result = memcmp(data1, data2, (size_t)(length * kind));
            return (equals == Py_EQ) ? (result == 0) : (result != 0);
        }
    } else if ((s1 == Py_None) & s2_is_unicode) {
        goto return_ne;
    } else if ((s2 == Py_None) & s1_is_unicode) {
        goto return_ne;
    } else {
        int result;
        PyObject* py_result = PyObject_RichCompare(s1, s2, equals);
        if (!py_result)
            return -1;
        result = __Pyx_PyObject_IsTrue(py_result);
        Py_DECREF(py_result);
        return result;
    }
return_eq:
    return (equals == Py_EQ);
return_ne:
    return (equals == Py_NE);
#endif
}

/* TupleAndListFromArray (used by fastcall) */
#if !CYTHON_COMPILING_IN_CPYTHON && CYTHON_METH_FASTCALL
static CYTHON_INLINE PyObject *
__Pyx_PyTuple_FromArray(PyObject *const *src, Py_ssize_t n)
{
    PyObject *res;
    Py_ssize_t i;
    if (n <= 0) {
        return __Pyx_NewRef(__pyx_mstate_global->__pyx_empty_tuple);
    }
    res = PyTuple_New(n);
    if (unlikely(res == NULL)) return NULL;
    for (i = 0; i < n; i++) {
        if (unlikely(__Pyx_PyTuple_SET_ITEM(res, i, src[i]) < (0))) {
            Py_DECREF(res);
            return NULL;
        }
        Py_INCREF(src[i]);
    }
    return res;
}
#elif CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE void __Pyx_copy_object_array(PyObject *const *CYTHON_RESTRICT src, PyObject** CYTHON_RESTRICT dest, Py_ssize_t length) {
    PyObject *v;
    Py_ssize_t i;
    for (i = 0; i < length; i++) {
        v = dest[i] = src[i];
        Py_INCREF(v);
    }
}
static CYTHON_INLINE PyObject *
__Pyx_PyTuple_FromArray(PyObject *const *src, Py_ssize_t n)
{
    PyObject *res;
    if (n <= 0) {
        return __Pyx_NewRef(__pyx_mstate_global->__pyx_empty_tuple);
    }
    res = PyTuple_New(n);
    if (unlikely(res == NULL)) return NULL;
    __Pyx_copy_object_array(src, ((PyTupleObject*)res)->ob_item, n);
    return res;
}
static CYTHON_INLINE PyObject *
__Pyx_PyList_FromArray(PyObject *const *src, Py_ssize_t n)
{
    PyObject *res;
    if (n <= 0) {
        return PyList_New(0);
    }
    res = PyList_New(n);
    if (unlikely(res == NULL)) return NULL;
    __Pyx_copy_object_array(src, ((PyListObject*)res)->ob_item, n);
    return res;
}
#endif

/* fastcall */
#if CYTHON_METH_FASTCALL
static CYTHON_INLINE PyObject * __Pyx_GetKwValue_FASTCALL(PyObject *kwnames, PyObject *const *kwvalues, PyObject *s)
{
    Py_ssize_t i, n = __Pyx_PyTuple_GET_SIZE(kwnames);
    #if !CYTHON_ASSUME_SAFE_SIZE
    if (unlikely(n == -1)) return NULL;
    #endif
    for (i = 0; i < n; i++)
    {
        PyObject *namei = __Pyx_PyTuple_GET_ITEM(kwnames, i);
        #if !CYTHON_ASSUME_SAFE_MACROS
        if (unlikely(!namei)) return NULL;
        #endif
        if (s == namei) return kwvalues[i];
    }
    for (i = 0; i < n; i++)
    {
        PyObject *namei = __Pyx_PyTuple_GET_ITEM(kwnames, i);
        #if !CYTHON_ASSUME_SAFE_MACROS
        if (unlikely(!namei)) return NULL;
        #endif
        int eq = __Pyx_PyUnicode_Equals(s, namei, Py_EQ);
        if (unlikely(eq != 0)) {
            if (unlikely(eq < 0)) return NULL;
            return kwvalues[i];
        }
    }
    return NULL;
}
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030d0000 || CYTHON_COMPILING_IN_LIMITED_API
CYTHON_UNUSED static PyObject *__Pyx_KwargsAsDict_FASTCALL(PyObject *kwnames, PyObject *const *kwvalues) {
    Py_ssize_t i, nkwargs;
    PyObject *dict;
#if !CYTHON_ASSUME_SAFE_SIZE
    nkwargs = PyTuple_Size(kwnames);
    if (unlikely(nkwargs < 0)) return NULL;
#else
    nkwargs = PyTuple_GET_SIZE(kwnames);
#endif
    dict = PyDict_New();
    if (unlikely(!dict))
        return NULL;
    for (i=0; i<nkwargs; i++) {
#if !CYTHON_ASSUME_SAFE_MACROS
        PyObject *key = PyTuple_GetItem(kwnames, i);
        if (!key) goto bad;
#else
        PyObject *key = PyTuple_GET_ITEM(kwnames, i);
#endif
        if (unlikely(PyDict_SetItem(dict, key, kwvalues[i]) < 0))
            goto bad;
    }
    return dict;
bad:
    Py_DECREF(dict);
    return NULL;
}
#endif
#endif

/* PyObjectCallOneArg (used by CallUnboundCMethod0) */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *args[2] = {NULL, arg};
    return __Pyx_PyObject_FastCall(func, args+1, 1 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET);
}

/* UnpackUnboundCMethod (used by CallUnboundCMethod0) */
#if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030C0000
static PyObject *__Pyx_SelflessCall(PyObject *method, PyObject *args, PyObject *kwargs) {
    PyObject *result;
    PyObject *selfless_args = PyTuple_GetSlice(args, 1, PyTuple_Size(args));
    if (unlikely(!selfless_args)) return NULL;
    result = PyObject_Call(method, selfless_args, kwargs);
    Py_DECREF(selfless_args);
    return result;
}
#elif CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x03090000
static PyObject *__Pyx_SelflessCall(PyObject *method, PyObject **args, Py_ssize_t nargs, PyObject *kwnames) {
        return _PyObject_Vectorcall
            (method, args ? args+1 : NULL, nargs ? nargs-1 : 0, kwnames);
}
#else
static PyObject *__Pyx_SelflessCall(PyObject *method, PyObject *const *args, Py_ssize_t nargs, PyObject *kwnames) {
    return
#if PY_VERSION_HEX < 0x03090000
    _PyObject_Vectorcall
#else
    PyObject_Vectorcall
#endif
        (method, args ? args+1 : NULL, nargs ? (size_t) nargs-1 : 0, kwnames);
}
#endif
static PyMethodDef __Pyx_UnboundCMethod_Def = {
     "CythonUnboundCMethod",
     __PYX_REINTERPRET_FUNCION(PyCFunction, __Pyx_SelflessCall),
#if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030C0000
     METH_VARARGS | METH_KEYWORDS,
#else
     METH_FASTCALL | METH_KEYWORDS,
#endif
     NULL
};
static int __Pyx_TryUnpackUnboundCMethod(__Pyx_CachedCFunction* target) {
    PyObject *method, *result=NULL;
    method = __Pyx_PyObject_GetAttrStr(target->type, *target->method_name);
    if (unlikely(!method))
        return -1;
    result = method;
#if CYTHON_COMPILING_IN_CPYTHON
    if (likely(__Pyx_TypeCheck(method, &PyMethodDescr_Type)))
    {
        PyMethodDescrObject *descr = (PyMethodDescrObject*) method;
        target->func = descr->d_method->ml_meth;
        target->flag = descr->d_method->ml_flags & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_STACKLESS);
    } else
#endif
#if CYTHON_COMPILING_IN_PYPY
#else
    if (PyCFunction_Check(method))
#endif
    {
        PyObject *self;
        int self_found;
#if CYTHON_COMPILING_IN_LIMITED_API || CYTHON_COMPILING_IN_PYPY
        self = PyObject_GetAttrString(method, "__self__");
        if (!self) {
            PyErr_Clear();
        }
#else
        self = PyCFunction_GET_SELF(method);
#endif
        self_found = (self && self != Py_None);
#if CYTHON_COMPILING_IN_LIMITED_API || CYTHON_COMPILING_IN_PYPY
        Py_XDECREF(self);
#endif
        if (self_found) {
            PyObject *unbound_method = PyCFunction_New(&__Pyx_UnboundCMethod_Def, method);
            if (unlikely(!unbound_method)) return -1;
            Py_DECREF(method);
            result = unbound_method;
        }
    }
#if !CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    if (unlikely(target->method)) {
        Py_DECREF(result);
    } else
#endif
    target->method = result;
    return 0;
}

/* CallUnboundCMethod0 */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_CallUnboundCMethod0(__Pyx_CachedCFunction* cfunc, PyObject* self) {
    int was_initialized = __Pyx_CachedCFunction_GetAndSetInitializing(cfunc);
    if (likely(was_initialized == 2 && cfunc->func)) {
        if (likely(cfunc->flag == METH_NOARGS))
            return __Pyx_CallCFunction(cfunc, self, NULL);
        if (likely(cfunc->flag == METH_FASTCALL))
            return __Pyx_CallCFunctionFast(cfunc, self, NULL, 0);
        if (cfunc->flag == (METH_FASTCALL | METH_KEYWORDS))
            return __Pyx_CallCFunctionFastWithKeywords(cfunc, self, NULL, 0, NULL);
        if (likely(cfunc->flag == (METH_VARARGS | METH_KEYWORDS)))
            return __Pyx_CallCFunctionWithKeywords(cfunc, self, __pyx_mstate_global->__pyx_empty_tuple, NULL);
        if (cfunc->flag == METH_VARARGS)
            return __Pyx_CallCFunction(cfunc, self, __pyx_mstate_global->__pyx_empty_tuple);
        return __Pyx__CallUnboundCMethod0(cfunc, self);
    }
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    else if (unlikely(was_initialized == 1)) {
        __Pyx_CachedCFunction tmp_cfunc = {
#ifndef __cplusplus
            0
#endif
        };
        tmp_cfunc.type = cfunc->type;
        tmp_cfunc.method_name = cfunc->method_name;
        return __Pyx__CallUnboundCMethod0(&tmp_cfunc, self);
    }
#endif
    PyObject *result = __Pyx__CallUnboundCMethod0(cfunc, self);
    __Pyx_CachedCFunction_SetFinishedInitializing(cfunc);
    return result;
}
#endif
static PyObject* __Pyx__CallUnboundCMethod0(__Pyx_CachedCFunction* cfunc, PyObject* self) {
    PyObject *result;
    if (unlikely(!cfunc->method) && unlikely(__Pyx_TryUnpackUnboundCMethod(cfunc) < 0)) return NULL;
    result = __Pyx_PyObject_CallOneArg(cfunc->method, self);
    return result;
}

/* py_dict_items (used by OwnedDictNext) */
static CYTHON_INLINE PyObject* __Pyx_PyDict_Items(PyObject* d) {
    return __Pyx_CallUnboundCMethod0(&__pyx_mstate_global->__pyx_umethod_PyDict_Type_items, d);
}

/* py_dict_values (used by OwnedDictNext) */
static CYTHON_INLINE PyObject* __Pyx_PyDict_Values(PyObject* d) {
    return __Pyx_CallUnboundCMethod0(&__pyx_mstate_global->__pyx_umethod_PyDict_Type_values, d);
}

/* OwnedDictNext (used by ParseKeywordsImpl) */
#if CYTHON_AVOID_BORROWED_REFS
static int __Pyx_PyDict_NextRef(PyObject *p, PyObject **ppos, PyObject **pkey, PyObject **pvalue) {
    PyObject *next = NULL;
    if (!*ppos) {
        if (pvalue) {
            PyObject *dictview = pkey ? __Pyx_PyDict_Items(p) : __Pyx_PyDict_Values(p);
            if (unlikely(!dictview)) goto bad;
            *ppos = PyObject_GetIter(dictview);
            Py_DECREF(dictview);
        } else {
            *ppos = PyObject_GetIter(p);
        }
        if (unlikely(!*ppos)) goto bad;
    }
    next = PyIter_Next(*ppos);
    if (!next) {
        if (PyErr_Occurred()) goto bad;
        return 0;
    }
    if (pkey && pvalue) {
        *pkey = __Pyx_PySequence_ITEM(next, 0);
        if (unlikely(*pkey)) goto bad;
        *pvalue = __Pyx_PySequence_ITEM(next, 1);
        if (unlikely(*pvalue)) goto bad;
        Py_DECREF(next);
    } else if (pkey) {
        *pkey = next;
    } else {
        assert(pvalue);
        *pvalue = next;
    }
    return 1;
  bad:
    Py_XDECREF(next);
#if !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x030d0000
    PyErr_FormatUnraisable("Exception ignored in __Pyx_PyDict_NextRef");
#else
    PyErr_WriteUnraisable(__pyx_mstate_global->__pyx_n_u_Pyx_PyDict_NextRef);
#endif
    if (pkey) *pkey = NULL;
    if (pvalue) *pvalue = NULL;
    return 0;
}
#else // !CYTHON_AVOID_BORROWED_REFS
static int __Pyx_PyDict_NextRef(PyObject *p, Py_ssize_t *ppos, PyObject **pkey, PyObject **pvalue) {
    int result = PyDict_Next(p, ppos, pkey, pvalue);
    if (likely(result == 1)) {
        if (pkey) Py_INCREF(*pkey);
        if (pvalue) Py_INCREF(*pvalue);
    }
    return result;
}
#endif

/* RaiseDoubleKeywords (used by ParseKeywordsImpl) */
static void __Pyx_RaiseDoubleKeywordsError(
    const char* func_name,
    PyObject* kw_name)
{
    PyErr_Format(PyExc_TypeError,
        "%s() got multiple values for keyword argument '%U'", func_name, kw_name);
}

/* CallUnboundCMethod2 */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject *__Pyx_CallUnboundCMethod2(__Pyx_CachedCFunction *cfunc, PyObject *self, PyObject *arg1, PyObject *arg2) {
    int was_initialized = __Pyx_CachedCFunction_GetAndSetInitializing(cfunc);
    if (likely(was_initialized == 2 && cfunc->func)) {
        PyObject *args[2] = {arg1, arg2};
        if (cfunc->flag == METH_FASTCALL) {
            return __Pyx_CallCFunctionFast(cfunc, self, args, 2);
        }
        if (cfunc->flag == (METH_FASTCALL | METH_KEYWORDS))
            return __Pyx_CallCFunctionFastWithKeywords(cfunc, self, args, 2, NULL);
    }
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    else if (unlikely(was_initialized == 1)) {
        __Pyx_CachedCFunction tmp_cfunc = {
#ifndef __cplusplus
            0
#endif
        };
        tmp_cfunc.type = cfunc->type;
        tmp_cfunc.method_name = cfunc->method_name;
        return __Pyx__CallUnboundCMethod2(&tmp_cfunc, self, arg1, arg2);
    }
#endif
    PyObject *result = __Pyx__CallUnboundCMethod2(cfunc, self, arg1, arg2);
    __Pyx_CachedCFunction_SetFinishedInitializing(cfunc);
    return result;
}
#endif
static PyObject* __Pyx__CallUnboundCMethod2(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg1, PyObject* arg2){
    if (unlikely(!cfunc->func && !cfunc->method) && unlikely(__Pyx_TryUnpackUnboundCMethod(cfunc) < 0)) return NULL;
#if CYTHON_COMPILING_IN_CPYTHON
    if (cfunc->func && (cfunc->flag & METH_VARARGS)) {
        PyObject *result = NULL;
        PyObject *args = PyTuple_New(2);
        if (unlikely(!args)) return NULL;
        Py_INCREF(arg1);
        PyTuple_SET_ITEM(args, 0, arg1);
        Py_INCREF(arg2);
        PyTuple_SET_ITEM(args, 1, arg2);
        if (cfunc->flag & METH_KEYWORDS)
            result = __Pyx_CallCFunctionWithKeywords(cfunc, self, args, NULL);
        else
            result = __Pyx_CallCFunction(cfunc, self, args);
        Py_DECREF(args);
        return result;
    }
#endif
    {
        PyObject *args[4] = {NULL, self, arg1, arg2};
        return __Pyx_PyObject_FastCall(cfunc->method, args+1, 3 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET);
    }
}

/* ParseKeywordsImpl (used by ParseKeywords) */
static int __Pyx_ValidateDuplicatePosArgs(
    PyObject *kwds,
    PyObject ** const argnames[],
    PyObject ** const *first_kw_arg,
    const char* function_name)
{
    PyObject ** const *name = argnames;
    while (name != first_kw_arg) {
        PyObject *key = **name;
        int found = PyDict_Contains(kwds, key);
        if (unlikely(found)) {
            if (found == 1) __Pyx_RaiseDoubleKeywordsError(function_name, key);
            goto bad;
        }
        name++;
    }
    return 0;
bad:
    return -1;
}
#if CYTHON_USE_UNICODE_INTERNALS
static CYTHON_INLINE int __Pyx_UnicodeKeywordsEqual(PyObject *s1, PyObject *s2) {
    int kind;
    Py_ssize_t len = PyUnicode_GET_LENGTH(s1);
    if (len != PyUnicode_GET_LENGTH(s2)) return 0;
    kind = PyUnicode_KIND(s1);
    if (kind != PyUnicode_KIND(s2)) return 0;
    const void *data1 = PyUnicode_DATA(s1);
    const void *data2 = PyUnicode_DATA(s2);
    return (memcmp(data1, data2, (size_t) len * (size_t) kind) == 0);
}
#endif
static int __Pyx_MatchKeywordArg_str(
    PyObject *key,
    PyObject ** const argnames[],
    PyObject ** const *first_kw_arg,
    size_t *index_found,
    const char *function_name)
{
    PyObject ** const *name;
    #if CYTHON_USE_UNICODE_INTERNALS
    Py_hash_t key_hash = ((PyASCIIObject*)key)->hash;
    if (unlikely(key_hash == -1)) {
        key_hash = PyObject_Hash(key);
        if (unlikely(key_hash == -1))
            goto bad;
    }
    #endif
    name = first_kw_arg;
    while (*name) {
        PyObject *name_str = **name;
        #if CYTHON_USE_UNICODE_INTERNALS
        if (key_hash == ((PyASCIIObject*)name_str)->hash && __Pyx_UnicodeKeywordsEqual(name_str, key)) {
            *index_found = (size_t) (name - argnames);
            return 1;
        }
        #else
        #if CYTHON_ASSUME_SAFE_SIZE
        if (PyUnicode_GET_LENGTH(name_str) == PyUnicode_GET_LENGTH(key))
        #endif
        {
            int cmp = PyUnicode_Compare(name_str, key);
            if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
            if (cmp == 0) {
                *index_found = (size_t) (name - argnames);
                return 1;
            }
        }
        #endif
        name++;
    }
    name = argnames;
    while (name != first_kw_arg) {
        PyObject *name_str = **name;
        #if CYTHON_USE_UNICODE_INTERNALS
        if (unlikely(key_hash == ((PyASCIIObject*)name_str)->hash)) {
            if (__Pyx_UnicodeKeywordsEqual(name_str, key))
                goto arg_passed_twice;
        }
        #else
        #if CYTHON_ASSUME_SAFE_SIZE
        if (PyUnicode_GET_LENGTH(name_str) == PyUnicode_GET_LENGTH(key))
        #endif
        {
            if (unlikely(name_str == key)) goto arg_passed_twice;
            int cmp = PyUnicode_Compare(name_str, key);
            if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
            if (cmp == 0) goto arg_passed_twice;
        }
        #endif
        name++;
    }
    return 0;
arg_passed_twice:
    __Pyx_RaiseDoubleKeywordsError(function_name, key);
    goto bad;
bad:
    return -1;
}
static int __Pyx_MatchKeywordArg_nostr(
    PyObject *key,
    PyObject ** const argnames[],
    PyObject ** const *first_kw_arg,
    size_t *index_found,
    const char *function_name)
{
    PyObject ** const *name;
    if (unlikely(!PyUnicode_Check(key))) goto invalid_keyword_type;
    name = first_kw_arg;
    while (*name) {
        int cmp = PyObject_RichCompareBool(**name, key, Py_EQ);
        if (cmp == 1) {
            *index_found = (size_t) (name - argnames);
            return 1;
        }
        if (unlikely(cmp == -1)) goto bad;
        name++;
    }
    name = argnames;
    while (name != first_kw_arg) {
        int cmp = PyObject_RichCompareBool(**name, key, Py_EQ);
        if (unlikely(cmp != 0)) {
            if (cmp == 1) goto arg_passed_twice;
            else goto bad;
        }
        name++;
    }
    return 0;
arg_passed_twice:
    __Pyx_RaiseDoubleKeywordsError(function_name, key);
    goto bad;
invalid_keyword_type:
    PyErr_Format(PyExc_TypeError,
        "%.200s() keywords must be strings", function_name);
    goto bad;
bad:
    return -1;
}
static CYTHON_INLINE int __Pyx_MatchKeywordArg(
    PyObject *key,
    PyObject ** const argnames[],
    PyObject ** const *first_kw_arg,
    size_t *index_found,
    const char *function_name)
{
    return likely(PyUnicode_CheckExact(key)) ?
        __Pyx_MatchKeywordArg_str(key, argnames, first_kw_arg, index_found, function_name) :
        __Pyx_MatchKeywordArg_nostr(key, argnames, first_kw_arg, index_found, function_name);
}
static void __Pyx_RejectUnknownKeyword(
    PyObject *kwds,
    PyObject ** const argnames[],
    PyObject ** const *first_kw_arg,
    const char *function_name)
{
    #if CYTHON_AVOID_BORROWED_REFS
    PyObject *pos = NULL;
    #else
    Py_ssize_t pos = 0;
    #endif
    PyObject *key = NULL;
    __Pyx_BEGIN_CRITICAL_SECTION(kwds);
    while (
        #if CYTHON_AVOID_BORROWED_REFS
        __Pyx_PyDict_NextRef(kwds, &pos, &key, NULL)
        #else
        PyDict_Next(kwds, &pos, &key, NULL)
        #endif
    ) {
        PyObject** const *name = first_kw_arg;
        while (*name && (**name != key)) name++;
        if (!*name) {
            size_t index_found = 0;
            int cmp = __Pyx_MatchKeywordArg(key, argnames, first_kw_arg, &index_found, function_name);
            if (cmp != 1) {
                if (cmp == 0) {
                    PyErr_Format(PyExc_TypeError,
                        "%s() got an unexpected keyword argument '%U'",
                        function_name, key);
                }
                #if CYTHON_AVOID_BORROWED_REFS
                Py_DECREF(key);
                #endif
                break;
            }
        }
        #if CYTHON_AVOID_BORROWED_REFS
        Py_DECREF(key);
        #endif
    }
    __Pyx_END_CRITICAL_SECTION();
    #if CYTHON_AVOID_BORROWED_REFS
    Py_XDECREF(pos);
    #endif
    assert(PyErr_Occurred());
}
static int __Pyx_ParseKeywordDict(
    PyObject *kwds,
    PyObject ** const argnames[],
    PyObject *values[],
    Py_ssize_t num_pos_args,
    Py_ssize_t num_kwargs,
    const char* function_name,
    int ignore_unknown_kwargs)
{
    PyObject** const *name;
    PyObject** const *first_kw_arg = argnames + num_pos_args;
    Py_ssize_t extracted = 0;
#if !CYTHON_COMPILING_IN_PYPY || defined(PyArg_ValidateKeywordArguments)
    if (unlikely(!PyArg_ValidateKeywordArguments(kwds))) return -1;
#endif
    name = first_kw_arg;
    while (*name && num_kwargs > extracted) {
        PyObject * key = **name;
        PyObject *value;
        int found = 0;
        #if __PYX_LIMITED_VERSION_HEX >= 0x030d0000
        found = PyDict_GetItemRef(kwds, key, &value);
        #else
        value = PyDict_GetItemWithError(kwds, key);
        if (value) {
            Py_INCREF(value);
            found = 1;
        } else {
            if (unlikely(PyErr_Occurred())) goto bad;
        }
        #endif
        if (found) {
            if (unlikely(found < 0)) goto bad;
            values[name-argnames] = value;
            extracted++;
        }
        name++;
    }
    if (num_kwargs > extracted) {
        if (ignore_unknown_kwargs) {
            if (unlikely(__Pyx_ValidateDuplicatePosArgs(kwds, argnames, first_kw_arg, function_name) == -1))
                goto bad;
        } else {
            __Pyx_RejectUnknownKeyword(kwds, argnames, first_kw_arg, function_name);
            goto bad;
        }
    }
    return 0;
bad:
    return -1;
}
static int __Pyx_ParseKeywordDictToDict(
    PyObject *kwds,
    PyObject ** const argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    const char* function_name)
{
    PyObject** const *name;
    PyObject** const *first_kw_arg = argnames + num_pos_args;
    Py_ssize_t len;
#if !CYTHON_COMPILING_IN_PYPY || defined(PyArg_ValidateKeywordArguments)
    if (unlikely(!PyArg_ValidateKeywordArguments(kwds))) return -1;
#endif
    if (PyDict_Update(kwds2, kwds) < 0) goto bad;
    name = first_kw_arg;
    while (*name) {
        PyObject *key = **name;
        PyObject *value;
#if !CYTHON_COMPILING_IN_LIMITED_API && (PY_VERSION_HEX >= 0x030d00A2 || defined(PyDict_Pop))
        int found = PyDict_Pop(kwds2, key, &value);
        if (found) {
            if (unlikely(found < 0)) goto bad;
            values[name-argnames] = value;
        }
#elif __PYX_LIMITED_VERSION_HEX >= 0x030d0000
        int found = PyDict_GetItemRef(kwds2, key, &value);
        if (found) {
            if (unlikely(found < 0)) goto bad;
            values[name-argnames] = value;
            if (unlikely(PyDict_DelItem(kwds2, key) < 0)) goto bad;
        }
#else
    #if CYTHON_COMPILING_IN_CPYTHON
        value = _PyDict_Pop(kwds2, key, kwds2);
    #else
        value = __Pyx_CallUnboundCMethod2(&__pyx_mstate_global->__pyx_umethod_PyDict_Type_pop, kwds2, key, kwds2);
    #endif
        if (value == kwds2) {
            Py_DECREF(value);
        } else {
            if (unlikely(!value)) goto bad;
            values[name-argnames] = value;
        }
#endif
        name++;
    }
    len = PyDict_Size(kwds2);
    if (len > 0) {
        return __Pyx_ValidateDuplicatePosArgs(kwds, argnames, first_kw_arg, function_name);
    } else if (unlikely(len == -1)) {
        goto bad;
    }
    return 0;
bad:
    return -1;
}
static int __Pyx_ParseKeywordsTuple(
    PyObject *kwds,
    PyObject * const *kwvalues,
    PyObject ** const argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    Py_ssize_t num_kwargs,
    const char* function_name,
    int ignore_unknown_kwargs)
{
    PyObject *key = NULL;
    PyObject** const * name;
    PyObject** const *first_kw_arg = argnames + num_pos_args;
    for (Py_ssize_t pos = 0; pos < num_kwargs; pos++) {
#if CYTHON_AVOID_BORROWED_REFS
        key = __Pyx_PySequence_ITEM(kwds, pos);
#else
        key = __Pyx_PyTuple_GET_ITEM(kwds, pos);
#endif
#if !CYTHON_ASSUME_SAFE_MACROS
        if (unlikely(!key)) goto bad;
#endif
        name = first_kw_arg;
        while (*name && (**name != key)) name++;
        if (*name) {
            PyObject *value = kwvalues[pos];
            values[name-argnames] = __Pyx_NewRef(value);
        } else {
            size_t index_found = 0;
            int cmp = __Pyx_MatchKeywordArg(key, argnames, first_kw_arg, &index_found, function_name);
            if (cmp == 1) {
                PyObject *value = kwvalues[pos];
                values[index_found] = __Pyx_NewRef(value);
            } else {
                if (unlikely(cmp == -1)) goto bad;
                if (kwds2) {
                    PyObject *value = kwvalues[pos];
                    if (unlikely(PyDict_SetItem(kwds2, key, value))) goto bad;
                } else if (!ignore_unknown_kwargs) {
                    goto invalid_keyword;
                }
            }
        }
        #if CYTHON_AVOID_BORROWED_REFS
        Py_DECREF(key);
        key = NULL;
        #endif
    }
    return 0;
invalid_keyword:
    PyErr_Format(PyExc_TypeError,
        "%s() got an unexpected keyword argument '%U'",
        function_name, key);
    goto bad;
bad:
    #if CYTHON_AVOID_BORROWED_REFS
    Py_XDECREF(key);
    #endif
    return -1;
}

/* ParseKeywords */
static int __Pyx_ParseKeywords(
    PyObject *kwds,
    PyObject * const *kwvalues,
    PyObject ** const argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    Py_ssize_t num_kwargs,
    const char* function_name,
    int ignore_unknown_kwargs)
{
    if (CYTHON_METH_FASTCALL && likely(PyTuple_Check(kwds)))
        return __Pyx_ParseKeywordsTuple(kwds, kwvalues, argnames, kwds2, values, num_pos_args, num_kwargs, function_name, ignore_unknown_kwargs);
    else if (kwds2)
        return __Pyx_ParseKeywordDictToDict(kwds, argnames, kwds2, values, num_pos_args, function_name);
    else
        return __Pyx_ParseKeywordDict(kwds, argnames, values, num_pos_args, num_kwargs, function_name, ignore_unknown_kwargs);
}

/* RaiseArgTupleInvalid */
static void __Pyx_RaiseArgtupleInvalid(
    const char* func_name,
    int exact,
    Py_ssize_t num_min,
    Py_ssize_t num_max,
    Py_ssize_t num_found)
{
    Py_ssize_t num_expected;
    const char *more_or_less;
    if (num_found < num_min) {
        num_expected = num_min;
        more_or_less = "at least";
    } else {
        num_expected = num_max;
        more_or_less = "at most";
    }
    if (exact) {
        more_or_less = "exactly";
    }
    PyErr_Format(PyExc_TypeError,
                 "%.200s() takes %.8s %" CYTHON_FORMAT_SSIZE_T "d positional argument%.1s (%" CYTHON_FORMAT_SSIZE_T "d given)",
                 func_name, more_or_less, num_expected,
                 (num_expected == 1) ? "" : "s", num_found);
}

/* ArgTypeTestFunc (used by ArgTypeTest) */
static int __Pyx__ArgTypeTest(PyObject *obj, PyTypeObject *type, const char *name, int exact)
{
    __Pyx_TypeName type_name;
    __Pyx_TypeName obj_type_name;
    PyObject *extra_info = __pyx_mstate_global->__pyx_empty_unicode;
    int from_annotation_subclass = 0;
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    else if (!exact) {
        if (likely(__Pyx_TypeCheck(obj, type))) return 1;
    } else if (exact == 2) {
        if (__Pyx_TypeCheck(obj, type)) {
            from_annotation_subclass = 1;
            extra_info = __pyx_mstate_global->__pyx_kp_u_Note_that_Cython_is_deliberately;
        }
    }
    type_name = __Pyx_PyType_GetFullyQualifiedName(type);
    obj_type_name = __Pyx_PyType_GetFullyQualifiedName(Py_TYPE(obj));
    PyErr_Format(PyExc_TypeError,
        "Argument '%.200s' has incorrect type (expected " __Pyx_FMT_TYPENAME
        ", got " __Pyx_FMT_TYPENAME ")"
#if __PYX_LIMITED_VERSION_HEX < 0x030C0000
        "%s%U"
#endif
        , name, type_name, obj_type_name
#if __PYX_LIMITED_VERSION_HEX < 0x030C0000
        , (from_annotation_subclass ? ". " : ""), extra_info
#endif
        );
#if __PYX_LIMITED_VERSION_HEX >= 0x030C0000
    if (exact == 2 && from_annotation_subclass) {
        PyObject *res;
        PyObject *vargs[2];
        vargs[0] = PyErr_GetRaisedException();
        vargs[1] = extra_info;
        res = PyObject_VectorcallMethod(__pyx_mstate_global->__pyx_kp_u_add_note, vargs, 2, NULL);
        Py_XDECREF(res);
        PyErr_SetRaisedException(vargs[0]);
    }
#endif
    __Pyx_DECREF_TypeName(type_name);
    __Pyx_DECREF_TypeName(obj_type_name);
    return 0;
}

/* PyObjectLookupSpecial */
#if CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx__PyObject_LookupSpecial(PyObject* obj, PyObject* attr_name, int with_error) {
    PyObject *res;
    PyTypeObject *tp = Py_TYPE(obj);
    res = _PyType_Lookup(tp, attr_name);
    if (likely(res)) {
        descrgetfunc f = Py_TYPE(res)->tp_descr_get;
        if (!f) {
            Py_INCREF(res);
        } else {
            res = f(res, obj, (PyObject *)tp);
        }
    } else if (with_error) {
        PyErr_SetObject(PyExc_AttributeError, attr_name);
    }
    return res;
}
#endif

/* GetItemInt */
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j) {
    PyObject *r;
    if (unlikely(!j)) return NULL;
    r = PyObject_GetItem(o, j);
    Py_DECREF(j);
    return r;
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck, int unsafe_shared) {
    CYTHON_MAYBE_UNUSED_VAR(unsafe_shared);
#if CYTHON_ASSUME_SAFE_SIZE
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyList_GET_SIZE(o);
    }
    if ((CYTHON_AVOID_BORROWED_REFS || CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS || !CYTHON_ASSUME_SAFE_MACROS)) {
        return __Pyx_PyList_GetItemRefFast(o, wrapped_i, unsafe_shared);
    } else
    if ((!boundscheck) || likely(__Pyx_is_valid_index(wrapped_i, PyList_GET_SIZE(o)))) {
        return __Pyx_NewRef(PyList_GET_ITEM(o, wrapped_i));
    }
    return __Pyx_GetItemInt_Generic(o, PyLong_FromSsize_t(i));
#else
    (void)wraparound;
    (void)boundscheck;
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck, int unsafe_shared) {
    CYTHON_MAYBE_UNUSED_VAR(unsafe_shared);
#if CYTHON_ASSUME_SAFE_SIZE && CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyTuple_GET_SIZE(o);
    }
    if ((!boundscheck) || likely(__Pyx_is_valid_index(wrapped_i, PyTuple_GET_SIZE(o)))) {
        return __Pyx_NewRef(PyTuple_GET_ITEM(o, wrapped_i));
    }
    return __Pyx_GetItemInt_Generic(o, PyLong_FromSsize_t(i));
#else
    (void)wraparound;
    (void)boundscheck;
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i, int is_list,
                                                     int wraparound, int boundscheck, int unsafe_shared) {
    CYTHON_MAYBE_UNUSED_VAR(unsafe_shared);
#if CYTHON_ASSUME_SAFE_MACROS && CYTHON_ASSUME_SAFE_SIZE
    if (is_list || PyList_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyList_GET_SIZE(o);
        if ((CYTHON_AVOID_BORROWED_REFS || CYTHON_AVOID_THREAD_UNSAFE_BORROWED_REFS)) {
            return __Pyx_PyList_GetItemRefFast(o, n, unsafe_shared);
        } else if ((!boundscheck) || (likely(__Pyx_is_valid_index(n, PyList_GET_SIZE(o))))) {
            return __Pyx_NewRef(PyList_GET_ITEM(o, n));
        }
    } else
    #if !CYTHON_AVOID_BORROWED_REFS
    if (PyTuple_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyTuple_GET_SIZE(o);
        if ((!boundscheck) || likely(__Pyx_is_valid_index(n, PyTuple_GET_SIZE(o)))) {
            return __Pyx_NewRef(PyTuple_GET_ITEM(o, n));
        }
    } else
    #endif
#endif
#if CYTHON_USE_TYPE_SLOTS && !CYTHON_COMPILING_IN_PYPY
    {
        PyMappingMethods *mm = Py_TYPE(o)->tp_as_mapping;
        PySequenceMethods *sm = Py_TYPE(o)->tp_as_sequence;
        if (!is_list && mm && mm->mp_subscript) {
            PyObject *r, *key = PyLong_FromSsize_t(i);
            if (unlikely(!key)) return NULL;
            r = mm->mp_subscript(o, key);
            Py_DECREF(key);
            return r;
        }
        if (is_list || likely(sm && sm->sq_item)) {
            if (wraparound && unlikely(i < 0) && likely(sm->sq_length)) {
                Py_ssize_t l = sm->sq_length(o);
                if (likely(l >= 0)) {
                    i += l;
                } else {
                    if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                        return NULL;
                    PyErr_Clear();
                }
            }
            return sm->sq_item(o, i);
        }
    }
#else
    if (is_list || !PyMapping_Check(o)) {
        return PySequence_GetItem(o, i);
    }
#endif
    (void)wraparound;
    (void)boundscheck;
    return __Pyx_GetItemInt_Generic(o, PyLong_FromSsize_t(i));
}

/* ExtTypeTest */
static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type) {
    __Pyx_TypeName obj_type_name;
    __Pyx_TypeName type_name;
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    if (likely(__Pyx_TypeCheck(obj, type)))
        return 1;
    obj_type_name = __Pyx_PyType_GetFullyQualifiedName(Py_TYPE(obj));
    type_name = __Pyx_PyType_GetFullyQualifiedName(type);
    PyErr_Format(PyExc_TypeError,
                 "Cannot convert " __Pyx_FMT_TYPENAME " to " __Pyx_FMT_TYPENAME,
                 obj_type_name, type_name);
    __Pyx_DECREF_TypeName(obj_type_name);
    __Pyx_DECREF_TypeName(type_name);
    return 0;
}

/* RejectKeywords */
static void __Pyx_RejectKeywords(const char* function_name, PyObject *kwds) {
    PyObject *key = NULL;
    if (CYTHON_METH_FASTCALL && likely(PyTuple_Check(kwds))) {
        key = __Pyx_PySequence_ITEM(kwds, 0);
    } else {
#if CYTHON_AVOID_BORROWED_REFS
        PyObject *pos = NULL;
#else
        Py_ssize_t pos = 0;
#endif
#if !CYTHON_COMPILING_IN_PYPY || defined(PyArg_ValidateKeywordArguments)
        if (unlikely(!PyArg_ValidateKeywordArguments(kwds))) return;
#endif
        __Pyx_PyDict_NextRef(kwds, &pos, &key, NULL);
#if CYTHON_AVOID_BORROWED_REFS
        Py_XDECREF(pos);
#endif
    }
    if (likely(key)) {
        PyErr_Format(PyExc_TypeError,
            "%s() got an unexpected keyword argument '%U'",
            function_name, key);
        Py_DECREF(key);
    }
}

/* WriteUnraisableException */
static void __Pyx_WriteUnraisable(const char *name, int clineno,
                                  int lineno, const char *filename,
                                  int full_traceback, int nogil) {
    PyObject *old_exc, *old_val, *old_tb;
    PyObject *ctx;
    __Pyx_PyThreadState_declare
    PyGILState_STATE state;
    if (nogil)
        state = PyGILState_Ensure();
    else state = (PyGILState_STATE)0;
    CYTHON_UNUSED_VAR(clineno);
    CYTHON_UNUSED_VAR(lineno);
    CYTHON_UNUSED_VAR(filename);
    CYTHON_MAYBE_UNUSED_VAR(nogil);
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&old_exc, &old_val, &old_tb);
    if (full_traceback) {
        Py_XINCREF(old_exc);
        Py_XINCREF(old_val);
        Py_XINCREF(old_tb);
        __Pyx_ErrRestore(old_exc, old_val, old_tb);
        PyErr_PrintEx(0);
    }
    ctx = PyUnicode_FromString(name);
    __Pyx_ErrRestore(old_exc, old_val, old_tb);
    if (!ctx) {
        PyErr_WriteUnraisable(Py_None);
    } else {
        PyErr_WriteUnraisable(ctx);
        Py_DECREF(ctx);
    }
    if (nogil)
        PyGILState_Release(state);
}

/* PyObjectFormatAndDecref */
static CYTHON_INLINE PyObject* __Pyx_PyObject_FormatSimpleAndDecref(PyObject* s, PyObject* f) {
    if (unlikely(!s)) return NULL;
    if (likely(PyUnicode_CheckExact(s))) return s;
    return __Pyx_PyObject_FormatAndDecref(s, f);
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_FormatAndDecref(PyObject* s, PyObject* f) {
    PyObject *result;
    if (unlikely(!s)) return NULL;
    result = PyObject_Format(s, f);
    Py_DECREF(s);
    return result;
}

/* PyUnicode_Unicode */
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_Unicode(PyObject *obj) {
    if (unlikely(obj == Py_None))
        obj = __pyx_mstate_global->__pyx_kp_u_None;
    return __Pyx_NewRef(obj);
}

/* JoinPyUnicode */
static PyObject* __Pyx_PyUnicode_Join(PyObject** values, Py_ssize_t value_count, Py_ssize_t result_ulength,
                                      Py_UCS4 max_char) {
#if CYTHON_USE_UNICODE_INTERNALS && CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    PyObject *result_uval;
    int result_ukind, kind_shift;
    Py_ssize_t i, char_pos;
    void *result_udata;
    if (max_char > 1114111) max_char = 1114111;
    result_uval = PyUnicode_New(result_ulength, max_char);
    if (unlikely(!result_uval)) return NULL;
    result_ukind = (max_char <= 255) ? PyUnicode_1BYTE_KIND : (max_char <= 65535) ? PyUnicode_2BYTE_KIND : PyUnicode_4BYTE_KIND;
    kind_shift = (result_ukind == PyUnicode_4BYTE_KIND) ? 2 : result_ukind - 1;
    result_udata = PyUnicode_DATA(result_uval);
    assert(kind_shift == 2 || kind_shift == 1 || kind_shift == 0);
    if (unlikely((PY_SSIZE_T_MAX >> kind_shift) - result_ulength < 0))
        goto overflow;
    char_pos = 0;
    for (i=0; i < value_count; i++) {
        int ukind;
        Py_ssize_t ulength;
        void *udata;
        PyObject *uval = values[i];
        #if !CYTHON_COMPILING_IN_LIMITED_API
        if (__Pyx_PyUnicode_READY(uval) == (-1))
            goto bad;
        #endif
        ulength = __Pyx_PyUnicode_GET_LENGTH(uval);
        #if !CYTHON_ASSUME_SAFE_SIZE
        if (unlikely(ulength < 0)) goto bad;
        #endif
        if (unlikely(!ulength))
            continue;
        if (unlikely((PY_SSIZE_T_MAX >> kind_shift) - ulength < char_pos))
            goto overflow;
        ukind = __Pyx_PyUnicode_KIND(uval);
        udata = __Pyx_PyUnicode_DATA(uval);
        if (ukind == result_ukind) {
            memcpy((char *)result_udata + (char_pos << kind_shift), udata, (size_t) (ulength << kind_shift));
        } else {
            #if PY_VERSION_HEX >= 0x030d0000
            if (unlikely(PyUnicode_CopyCharacters(result_uval, char_pos, uval, 0, ulength) < 0)) goto bad;
            #elif CYTHON_COMPILING_IN_CPYTHON || defined(_PyUnicode_FastCopyCharacters)
            _PyUnicode_FastCopyCharacters(result_uval, char_pos, uval, 0, ulength);
            #else
            Py_ssize_t j;
            for (j=0; j < ulength; j++) {
                Py_UCS4 uchar = __Pyx_PyUnicode_READ(ukind, udata, j);
                __Pyx_PyUnicode_WRITE(result_ukind, result_udata, char_pos+j, uchar);
            }
            #endif
        }
        char_pos += ulength;
    }
    return result_uval;
overflow:
    PyErr_SetString(PyExc_OverflowError, "join() result is too long for a Python string");
bad:
    Py_DECREF(result_uval);
    return NULL;
#else
    Py_ssize_t i;
    PyObject *result = NULL;
    PyObject *value_tuple = PyTuple_New(value_count);
    if (unlikely(!value_tuple)) return NULL;
    CYTHON_UNUSED_VAR(max_char);
    CYTHON_UNUSED_VAR(result_ulength);
    for (i=0; i<value_count; i++) {
        if (__Pyx_PyTuple_SET_ITEM(value_tuple, i, values[i]) != (0)) goto bad;
        Py_INCREF(values[i]);
    }
    result = PyUnicode_Join(__pyx_mstate_global->__pyx_empty_unicode, value_tuple);
bad:
    Py_DECREF(value_tuple);
    return result;
#endif
}

/* PyObjectVectorCallKwBuilder */
#if CYTHON_VECTORCALL
static int __Pyx_VectorcallBuilder_AddArg(PyObject *key, PyObject *value, PyObject *builder, PyObject **args, int n) {
    (void)__Pyx_PyObject_FastCallDict;
    if (__Pyx_PyTuple_SET_ITEM(builder, n, key) != (0)) return -1;
    Py_INCREF(key);
    args[n] = value;
    return 0;
}
CYTHON_UNUSED static int __Pyx_VectorcallBuilder_AddArg_Check(PyObject *key, PyObject *value, PyObject *builder, PyObject **args, int n) {
    (void)__Pyx_VectorcallBuilder_AddArgStr;
    if (unlikely(!PyUnicode_Check(key))) {
        PyErr_SetString(PyExc_TypeError, "keywords must be strings");
        return -1;
    }
    return __Pyx_VectorcallBuilder_AddArg(key, value, builder, args, n);
}
static int __Pyx_VectorcallBuilder_AddArgStr(const char *key, PyObject *value, PyObject *builder, PyObject **args, int n) {
    PyObject *pyKey = PyUnicode_FromString(key);
    if (!pyKey) return -1;
    return __Pyx_VectorcallBuilder_AddArg(pyKey, value, builder, args, n);
}
#else // CYTHON_VECTORCALL
CYTHON_UNUSED static int __Pyx_VectorcallBuilder_AddArg_Check(PyObject *key, PyObject *value, PyObject *builder, CYTHON_UNUSED PyObject **args, CYTHON_UNUSED int n) {
    if (unlikely(!PyUnicode_Check(key))) {
        PyErr_SetString(PyExc_TypeError, "keywords must be strings");
        return -1;
    }
    return PyDict_SetItem(builder, key, value);
}
#endif

/* RaiseTooManyValuesToUnpack */
static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected) {
    PyErr_Format(PyExc_ValueError,
                 "too many values to unpack (expected %" CYTHON_FORMAT_SSIZE_T "d)", expected);
}

/* RaiseNeedMoreValuesToUnpack */
static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index) {
    PyErr_Format(PyExc_ValueError,
                 "need more than %" CYTHON_FORMAT_SSIZE_T "d value%.1s to unpack",
                 index, (index == 1) ? "" : "s");
}

/* IterFinish */
static CYTHON_INLINE int __Pyx_IterFinish(void) {
    PyObject* exc_type;
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    exc_type = __Pyx_PyErr_CurrentExceptionType();
    if (unlikely(exc_type)) {
        if (unlikely(!__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration)))
            return -1;
        __Pyx_PyErr_Clear();
        return 0;
    }
    return 0;
}

/* UnpackItemEndCheck */
static int __Pyx_IternextUnpackEndCheck(PyObject *retval, Py_ssize_t expected) {
    if (unlikely(retval)) {
        Py_DECREF(retval);
        __Pyx_RaiseTooManyValuesError(expected);
        return -1;
    }
    return __Pyx_IterFinish();
}

/* RaiseKeywordRequired */
static void __Pyx_RaiseKeywordRequired(const char* func_name, PyObject* kw_name) {
    PyErr_Format(PyExc_TypeError,
        "%s() needs keyword-only argument %U", func_name, kw_name);
}

/* PyObjectVectorCallMethodKwBuilder */
#if !CYTHON_VECTORCALL || PY_VERSION_HEX < 0x03090000
static PyObject *__Pyx_Object_VectorcallMethod_CallFromBuilder(PyObject *name, PyObject *const *args, size_t nargsf, PyObject *kwnames) {
    PyObject *result;
    PyObject *obj = PyObject_GetAttr(args[0], name);
    if (unlikely(!obj))
        return NULL;
    result = __Pyx_Object_Vectorcall_CallFromBuilder(obj, args+1, nargsf-1, kwnames);
    Py_DECREF(obj);
    return result;
}
#endif

/* GetAttr3 */
#if __PYX_LIMITED_VERSION_HEX < 0x030d0000
static PyObject *__Pyx_GetAttr3Default(PyObject *d) {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    if (unlikely(!__Pyx_PyErr_ExceptionMatches(PyExc_AttributeError)))
        return NULL;
    __Pyx_PyErr_Clear();
    Py_INCREF(d);
    return d;
}
#endif
static CYTHON_INLINE PyObject *__Pyx_GetAttr3(PyObject *o, PyObject *n, PyObject *d) {
    PyObject *r;
#if __PYX_LIMITED_VERSION_HEX >= 0x030d0000
    int res = PyObject_GetOptionalAttr(o, n, &r);
    return (res != 0) ? r : __Pyx_NewRef(d);
#else
  #if CYTHON_USE_TYPE_SLOTS
    if (likely(PyUnicode_Check(n))) {
        r = __Pyx_PyObject_GetAttrStrNoError(o, n);
        if (unlikely(!r) && likely(!PyErr_Occurred())) {
            r = __Pyx_NewRef(d);
        }
        return r;
    }
  #endif
    r = PyObject_GetAttr(o, n);
    return (likely(r)) ? r : __Pyx_GetAttr3Default(d);
#endif
}

/* PyObjectCall2Args (used by CallUnboundCMethod1) */
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call2Args(PyObject* function, PyObject* arg1, PyObject* arg2) {
    PyObject *args[3] = {NULL, arg1, arg2};
    return __Pyx_PyObject_FastCall(function, args+1, 2 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET);
}

/* CallUnboundCMethod1 */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_CallUnboundCMethod1(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg) {
    int was_initialized =  __Pyx_CachedCFunction_GetAndSetInitializing(cfunc);
    if (likely(was_initialized == 2 && cfunc->func)) {
        int flag = cfunc->flag;
        if (flag == METH_O) {
            return __Pyx_CallCFunction(cfunc, self, arg);
        } else if (flag == METH_FASTCALL) {
            return __Pyx_CallCFunctionFast(cfunc, self, &arg, 1);
        } else if (flag == (METH_FASTCALL | METH_KEYWORDS)) {
            return __Pyx_CallCFunctionFastWithKeywords(cfunc, self, &arg, 1, NULL);
        }
    }
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    else if (unlikely(was_initialized == 1)) {
        __Pyx_CachedCFunction tmp_cfunc = {
#ifndef __cplusplus
            0
#endif
        };
        tmp_cfunc.type = cfunc->type;
        tmp_cfunc.method_name = cfunc->method_name;
        return __Pyx__CallUnboundCMethod1(&tmp_cfunc, self, arg);
    }
#endif
    PyObject* result = __Pyx__CallUnboundCMethod1(cfunc, self, arg);
    __Pyx_CachedCFunction_SetFinishedInitializing(cfunc);
    return result;
}
#endif
static PyObject* __Pyx__CallUnboundCMethod1(__Pyx_CachedCFunction* cfunc, PyObject* self, PyObject* arg){
    PyObject *result = NULL;
    if (unlikely(!cfunc->func && !cfunc->method) && unlikely(__Pyx_TryUnpackUnboundCMethod(cfunc) < 0)) return NULL;
#if CYTHON_COMPILING_IN_CPYTHON
    if (cfunc->func && (cfunc->flag & METH_VARARGS)) {
        PyObject *args = PyTuple_New(1);
        if (unlikely(!args)) return NULL;
        Py_INCREF(arg);
        PyTuple_SET_ITEM(args, 0, arg);
        if (cfunc->flag & METH_KEYWORDS)
            result = __Pyx_CallCFunctionWithKeywords(cfunc, self, args, NULL);
        else
            result = __Pyx_CallCFunction(cfunc, self, args);
        Py_DECREF(args);
    } else
#endif
    {
        result = __Pyx_PyObject_Call2Args(cfunc->method, self, arg);
    }
    return result;
}

/* py_dict_pop */
static CYTHON_INLINE PyObject *__Pyx_PyDict_Pop(PyObject *d, PyObject *key, PyObject *default_value) {
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030d00A2 || defined(PyDict_Pop)
    PyObject *value;
    if (PyDict_Pop(d, key, &value) == 0) {
        if (default_value) {
            Py_INCREF(default_value);
        } else {
            PyErr_SetObject(PyExc_KeyError, key);
        }
        value = default_value;
    }
    return value;
#elif CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030d0000
    return _PyDict_Pop(d, key, default_value);
#else
    if (default_value) {
        return __Pyx_CallUnboundCMethod2(&__pyx_mstate_global->__pyx_umethod_PyDict_Type_pop, d, key, default_value);
    } else {
        return __Pyx_CallUnboundCMethod1(&__pyx_mstate_global->__pyx_umethod_PyDict_Type_pop, d, key);
    }
#endif
}

/* RaiseClosureNameError */
static void __Pyx_RaiseClosureNameError(const char *varname) {
    PyErr_Format(PyExc_NameError, "free variable '%s' referenced before assignment in enclosing scope", varname);
}

/* dict_setdefault (used by FetchCommonType) */
static CYTHON_INLINE PyObject *__Pyx_PyDict_SetDefault(PyObject *d, PyObject *key, PyObject *default_value) {
    PyObject* value;
#if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX >= 0x030C0000
    PyObject *args[] = {d, key, default_value};
    value = PyObject_VectorcallMethod(__pyx_mstate_global->__pyx_n_u_setdefault, args, 3 | PY_VECTORCALL_ARGUMENTS_OFFSET, NULL);
#elif CYTHON_COMPILING_IN_LIMITED_API
    value = PyObject_CallMethodObjArgs(d, __pyx_mstate_global->__pyx_n_u_setdefault, key, default_value, NULL);
#elif PY_VERSION_HEX >= 0x030d0000
    PyDict_SetDefaultRef(d, key, default_value, &value);
#else
    value = PyDict_SetDefault(d, key, default_value);
    if (unlikely(!value)) return NULL;
    Py_INCREF(value);
#endif
    return value;
}

/* LimitedApiGetTypeDict (used by SetItemOnTypeDict) */
#if CYTHON_COMPILING_IN_LIMITED_API
static Py_ssize_t __Pyx_GetTypeDictOffset(void) {
    PyObject *tp_dictoffset_o;
    Py_ssize_t tp_dictoffset;
    tp_dictoffset_o = PyObject_GetAttrString((PyObject*)(&PyType_Type), "__dictoffset__");
    if (unlikely(!tp_dictoffset_o)) return -1;
    tp_dictoffset = PyLong_AsSsize_t(tp_dictoffset_o);
    Py_DECREF(tp_dictoffset_o);
    if (unlikely(tp_dictoffset == 0)) {
        PyErr_SetString(
            PyExc_TypeError,
            "'type' doesn't have a dictoffset");
        return -1;
    } else if (unlikely(tp_dictoffset < 0)) {
        PyErr_SetString(
            PyExc_TypeError,
            "'type' has an unexpected negative dictoffset. "
            "Please report this as Cython bug");
        return -1;
    }
    return tp_dictoffset;
}
static PyObject *__Pyx_GetTypeDict(PyTypeObject *tp) {
    static Py_ssize_t tp_dictoffset = 0;
    if (unlikely(tp_dictoffset == 0)) {
        tp_dictoffset = __Pyx_GetTypeDictOffset();
        if (unlikely(tp_dictoffset == -1 && PyErr_Occurred())) {
            tp_dictoffset = 0; // try again next time?
            return NULL;
        }
    }
    return *(PyObject**)((char*)tp + tp_dictoffset);
}
#endif

/* SetItemOnTypeDict (used by FixUpExtensionType) */
static int __Pyx__SetItemOnTypeDict(PyTypeObject *tp, PyObject *k, PyObject *v) {
    int result;
    PyObject *tp_dict;
#if CYTHON_COMPILING_IN_LIMITED_API
    tp_dict = __Pyx_GetTypeDict(tp);
    if (unlikely(!tp_dict)) return -1;
#else
    tp_dict = tp->tp_dict;
#endif
    result = PyDict_SetItem(tp_dict, k, v);
    if (likely(!result)) {
        PyType_Modified(tp);
        if (unlikely(PyObject_HasAttr(v, __pyx_mstate_global->__pyx_n_u_set_name))) {
            PyObject *setNameResult = PyObject_CallMethodObjArgs(v, __pyx_mstate_global->__pyx_n_u_set_name,  (PyObject *) tp, k, NULL);
            if (!setNameResult) return -1;
            Py_DECREF(setNameResult);
        }
    }
    return result;
}

/* FixUpExtensionType (used by FetchCommonType) */
static int __Pyx_fix_up_extension_type_from_spec(PyType_Spec *spec, PyTypeObject *type) {
#if __PYX_LIMITED_VERSION_HEX > 0x030900B1
    CYTHON_UNUSED_VAR(spec);
    CYTHON_UNUSED_VAR(type);
    CYTHON_UNUSED_VAR(__Pyx__SetItemOnTypeDict);
#else
    const PyType_Slot *slot = spec->slots;
    int changed = 0;
#if !CYTHON_COMPILING_IN_LIMITED_API
    while (slot && slot->slot && slot->slot != Py_tp_members)
        slot++;
    if (slot && slot->slot == Py_tp_members) {
#if !CYTHON_COMPILING_IN_CPYTHON
        const
#endif  // !CYTHON_COMPILING_IN_CPYTHON)
            PyMemberDef *memb = (PyMemberDef*) slot->pfunc;
        while (memb && memb->name) {
            if (memb->name[0] == '_' && memb->name[1] == '_') {
                if (strcmp(memb->name, "__weaklistoffset__") == 0) {
                    assert(memb->type == T_PYSSIZET);
                    assert(memb->flags == READONLY);
                    type->tp_weaklistoffset = memb->offset;
                    changed = 1;
                }
                else if (strcmp(memb->name, "__dictoffset__") == 0) {
                    assert(memb->type == T_PYSSIZET);
                    assert(memb->flags == READONLY);
                    type->tp_dictoffset = memb->offset;
                    changed = 1;
                }
#if CYTHON_METH_FASTCALL
                else if (strcmp(memb->name, "__vectorcalloffset__") == 0) {
                    assert(memb->type == T_PYSSIZET);
                    assert(memb->flags == READONLY);
                    type->tp_vectorcall_offset = memb->offset;
                    changed = 1;
                }
#endif  // CYTHON_METH_FASTCALL
#if !CYTHON_COMPILING_IN_PYPY
                else if (strcmp(memb->name, "__module__") == 0) {
                    PyObject *descr;
                    assert(memb->type == T_OBJECT);
                    assert(memb->flags == 0 || memb->flags == READONLY);
                    descr = PyDescr_NewMember(type, memb);
                    if (unlikely(!descr))
                        return -1;
                    int set_item_result = PyDict_SetItem(type->tp_dict, PyDescr_NAME(descr), descr);
                    Py_DECREF(descr);
                    if (unlikely(set_item_result < 0)) {
                        return -1;
                    }
                    changed = 1;
                }
#endif  // !CYTHON_COMPILING_IN_PYPY
            }
            memb++;
        }
    }
#endif  // !CYTHON_COMPILING_IN_LIMITED_API
#if !CYTHON_COMPILING_IN_PYPY
    slot = spec->slots;
    while (slot && slot->slot && slot->slot != Py_tp_getset)
        slot++;
    if (slot && slot->slot == Py_tp_getset) {
        PyGetSetDef *getset = (PyGetSetDef*) slot->pfunc;
        while (getset && getset->name) {
            if (getset->name[0] == '_' && getset->name[1] == '_' && strcmp(getset->name, "__module__") == 0) {
                PyObject *descr = PyDescr_NewGetSet(type, getset);
                if (unlikely(!descr))
                    return -1;
                #if CYTHON_COMPILING_IN_LIMITED_API
                PyObject *pyname = PyUnicode_FromString(getset->name);
                if (unlikely(!pyname)) {
                    Py_DECREF(descr);
                    return -1;
                }
                int set_item_result = __Pyx_SetItemOnTypeDict(type, pyname, descr);
                Py_DECREF(pyname);
                #else
                CYTHON_UNUSED_VAR(__Pyx__SetItemOnTypeDict);
                int set_item_result = PyDict_SetItem(type->tp_dict, PyDescr_NAME(descr), descr);
                #endif
                Py_DECREF(descr);
                if (unlikely(set_item_result < 0)) {
                    return -1;
                }
                changed = 1;
            }
            ++getset;
        }
    }
#else
    CYTHON_UNUSED_VAR(__Pyx__SetItemOnTypeDict);
#endif  // !CYTHON_COMPILING_IN_PYPY
    if (changed)
        PyType_Modified(type);
#endif  // PY_VERSION_HEX > 0x030900B1
    return 0;
}

/* AddModuleRef (used by FetchSharedCythonModule) */
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
  static PyObject *__Pyx_PyImport_AddModuleObjectRef(PyObject *name) {
      PyObject *module_dict = PyImport_GetModuleDict();
      PyObject *m;
      if (PyMapping_GetOptionalItem(module_dict, name, &m) < 0) {
          return NULL;
      }
      if (m != NULL && PyModule_Check(m)) {
          return m;
      }
      Py_XDECREF(m);
      m = PyModule_NewObject(name);
      if (m == NULL)
          return NULL;
      if (PyDict_CheckExact(module_dict)) {
          PyObject *new_m;
          (void)PyDict_SetDefaultRef(module_dict, name, m, &new_m);
          Py_DECREF(m);
          return new_m;
      } else {
           if (PyObject_SetItem(module_dict, name, m) != 0) {
                Py_DECREF(m);
                return NULL;
            }
            return m;
      }
  }
  static PyObject *__Pyx_PyImport_AddModuleRef(const char *name) {
      PyObject *py_name = PyUnicode_FromString(name);
      if (!py_name) return NULL;
      PyObject *module = __Pyx_PyImport_AddModuleObjectRef(py_name);
      Py_DECREF(py_name);
      return module;
  }
#elif __PYX_LIMITED_VERSION_HEX >= 0x030d0000
  #define __Pyx_PyImport_AddModuleRef(name) PyImport_AddModuleRef(name)
#else
  static PyObject *__Pyx_PyImport_AddModuleRef(const char *name) {
      PyObject *module = PyImport_AddModule(name);
      Py_XINCREF(module);
      return module;
  }
#endif

/* FetchSharedCythonModule (used by FetchCommonType) */
static PyObject *__Pyx_FetchSharedCythonABIModule(void) {
    return __Pyx_PyImport_AddModuleRef(__PYX_ABI_MODULE_NAME);
}

/* FetchCommonType (used by CommonTypesMetaclass) */
#if __PYX_LIMITED_VERSION_HEX < 0x030C0000
static PyObject* __Pyx_PyType_FromMetaclass(PyTypeObject *metaclass, PyObject *module, PyType_Spec *spec, PyObject *bases) {
    PyObject *result = __Pyx_PyType_FromModuleAndSpec(module, spec, bases);
    if (result && metaclass) {
        PyObject *old_tp = (PyObject*)Py_TYPE(result);
    Py_INCREF((PyObject*)metaclass);
#if __PYX_LIMITED_VERSION_HEX >= 0x03090000
        Py_SET_TYPE(result, metaclass);
#else
        result->ob_type = metaclass;
#endif
        Py_DECREF(old_tp);
    }
    return result;
}
#else
#define __Pyx_PyType_FromMetaclass(me, mo, s, b) PyType_FromMetaclass(me, mo, s, b)
#endif
static int __Pyx_VerifyCachedType(PyObject *cached_type,
                               const char *name,
                               Py_ssize_t expected_basicsize) {
    Py_ssize_t basicsize;
    if (!PyType_Check(cached_type)) {
        PyErr_Format(PyExc_TypeError,
            "Shared Cython type %.200s is not a type object", name);
        return -1;
    }
    if (expected_basicsize == 0) {
        return 0; // size is inherited, nothing useful to check
    }
#if CYTHON_COMPILING_IN_LIMITED_API
    PyObject *py_basicsize;
    py_basicsize = PyObject_GetAttrString(cached_type, "__basicsize__");
    if (unlikely(!py_basicsize)) return -1;
    basicsize = PyLong_AsSsize_t(py_basicsize);
    Py_DECREF(py_basicsize);
    py_basicsize = NULL;
    if (unlikely(basicsize == (Py_ssize_t)-1) && PyErr_Occurred()) return -1;
#else
    basicsize = ((PyTypeObject*) cached_type)->tp_basicsize;
#endif
    if (basicsize != expected_basicsize) {
        PyErr_Format(PyExc_TypeError,
            "Shared Cython type %.200s has the wrong size, try recompiling",
            name);
        return -1;
    }
    return 0;
}
static PyTypeObject *__Pyx_FetchCommonTypeFromSpec(PyTypeObject *metaclass, PyObject *module, PyType_Spec *spec, PyObject *bases) {
    PyObject *abi_module = NULL, *cached_type = NULL, *abi_module_dict, *new_cached_type, *py_object_name;
    int get_item_ref_result;
    const char* object_name = strrchr(spec->name, '.');
    object_name = object_name ? object_name+1 : spec->name;
    py_object_name = PyUnicode_FromString(object_name);
    if (!py_object_name) return NULL;
    abi_module = __Pyx_FetchSharedCythonABIModule();
    if (!abi_module) goto done;
    abi_module_dict = PyModule_GetDict(abi_module);
    if (!abi_module_dict) goto done;
    get_item_ref_result = __Pyx_PyDict_GetItemRef(abi_module_dict, py_object_name, &cached_type);
    if (get_item_ref_result == 1) {
        if (__Pyx_VerifyCachedType(
              cached_type,
              object_name,
              spec->basicsize) < 0) {
            goto bad;
        }
        goto done;
    } else if (unlikely(get_item_ref_result == -1)) {
        goto bad;
    }
    cached_type = __Pyx_PyType_FromMetaclass(
        metaclass,
        CYTHON_USE_MODULE_STATE ? module : abi_module,
        spec, bases);
    if (unlikely(!cached_type)) goto bad;
    if (unlikely(__Pyx_fix_up_extension_type_from_spec(spec, (PyTypeObject *) cached_type) < 0)) goto bad;
    new_cached_type = __Pyx_PyDict_SetDefault(abi_module_dict, py_object_name, cached_type);
    if (unlikely(new_cached_type != cached_type)) {
        if (unlikely(!new_cached_type)) goto bad;
        Py_DECREF(cached_type);
        cached_type = new_cached_type;
        if (__Pyx_VerifyCachedType(
                cached_type,
                object_name,
                spec->basicsize) < 0) {
            goto bad;
        }
        goto done;
    } else {
        Py_DECREF(new_cached_type);
    }
done:
    Py_XDECREF(abi_module);
    Py_DECREF(py_object_name);
    assert(cached_type == NULL || PyType_Check(cached_type));
    return (PyTypeObject *) cached_type;
bad:
    Py_XDECREF(cached_type);
    cached_type = NULL;
    goto done;
}

/* CommonTypesMetaclass (used by CythonFunctionShared) */
static PyObject* __pyx_CommonTypesMetaclass_get_module(CYTHON_UNUSED PyObject *self, CYTHON_UNUSED void* context) {
    return PyUnicode_FromString(__PYX_ABI_MODULE_NAME);
}
#if __PYX_LIMITED_VERSION_HEX < 0x030A0000
static PyObject* __pyx_CommonTypesMetaclass_call(CYTHON_UNUSED PyObject *self, CYTHON_UNUSED PyObject *args, CYTHON_UNUSED PyObject *kwds) {
    PyErr_SetString(PyExc_TypeError, "Cannot instantiate Cython internal types");
    return NULL;
}
static int __pyx_CommonTypesMetaclass_setattr(CYTHON_UNUSED PyObject *self, CYTHON_UNUSED PyObject *attr, CYTHON_UNUSED PyObject *value) {
    PyErr_SetString(PyExc_TypeError, "Cython internal types are immutable");
    return -1;
}
#endif
static PyGetSetDef __pyx_CommonTypesMetaclass_getset[] = {
    {"__module__", __pyx_CommonTypesMetaclass_get_module, NULL, NULL, NULL},
    {0, 0, 0, 0, 0}
};
static PyType_Slot __pyx_CommonTypesMetaclass_slots[] = {
    {Py_tp_getset, (void *)__pyx_CommonTypesMetaclass_getset},
    #if __PYX_LIMITED_VERSION_HEX < 0x030A0000
    {Py_tp_call, (void*)__pyx_CommonTypesMetaclass_call},
    {Py_tp_new, (void*)__pyx_CommonTypesMetaclass_call},
    {Py_tp_setattro, (void*)__pyx_CommonTypesMetaclass_setattr},
    #endif
    {0, 0}
};
static PyType_Spec __pyx_CommonTypesMetaclass_spec = {
    __PYX_TYPE_MODULE_PREFIX "_common_types_metatype",
    0,
    0,
    Py_TPFLAGS_IMMUTABLETYPE |
    Py_TPFLAGS_DISALLOW_INSTANTIATION |
    Py_TPFLAGS_DEFAULT,
    __pyx_CommonTypesMetaclass_slots
};
static int __pyx_CommonTypesMetaclass_init(PyObject *module) {
    __pyx_mstatetype *mstate = __Pyx_PyModule_GetState(module);
    PyObject *bases = PyTuple_Pack(1, &PyType_Type);
    if (unlikely(!bases)) {
        return -1;
    }
    mstate->__pyx_CommonTypesMetaclassType = __Pyx_FetchCommonTypeFromSpec(NULL, module, &__pyx_CommonTypesMetaclass_spec, bases);
    Py_DECREF(bases);
    if (unlikely(mstate->__pyx_CommonTypesMetaclassType == NULL)) {
        return -1;
    }
    return 0;
}

/* CallTypeTraverse (used by CythonFunctionShared) */
#if !CYTHON_USE_TYPE_SPECS || (!CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x03090000)
#else
static int __Pyx_call_type_traverse(PyObject *o, int always_call, visitproc visit, void *arg) {
    #if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x03090000
    if (__Pyx_get_runtime_version() < 0x03090000) return 0;
    #endif
    if (!always_call) {
        PyTypeObject *base = __Pyx_PyObject_GetSlot(o, tp_base, PyTypeObject*);
        unsigned long flags = PyType_GetFlags(base);
        if (flags & Py_TPFLAGS_HEAPTYPE) {
            return 0;
        }
    }
    Py_VISIT((PyObject*)Py_TYPE(o));
    return 0;
}
#endif

/* PyMethodNew (used by CythonFunctionShared) */
#if CYTHON_COMPILING_IN_LIMITED_API
static PyObject *__Pyx_PyMethod_New(PyObject *func, PyObject *self, PyObject *typ) {
    PyObject *result;
    CYTHON_UNUSED_VAR(typ);
    if (!self)
        return __Pyx_NewRef(func);
    #if __PYX_LIMITED_VERSION_HEX >= 0x030C0000
    {
        PyObject *args[] = {func, self};
        result = PyObject_Vectorcall(__pyx_mstate_global->__Pyx_CachedMethodType, args, 2, NULL);
    }
    #else
    result = PyObject_CallFunctionObjArgs(__pyx_mstate_global->__Pyx_CachedMethodType, func, self, NULL);
    #endif
    return result;
}
#else
static PyObject *__Pyx_PyMethod_New(PyObject *func, PyObject *self, PyObject *typ) {
    CYTHON_UNUSED_VAR(typ);
    if (!self)
        return __Pyx_NewRef(func);
    return PyMethod_New(func, self);
}
#endif

/* PyVectorcallFastCallDict (used by CythonFunctionShared) */
#if CYTHON_METH_FASTCALL && CYTHON_VECTORCALL
static PyObject *__Pyx_PyVectorcall_FastCallDict_kw(PyObject *func, __pyx_vectorcallfunc vc, PyObject *const *args, size_t nargs, PyObject *kw)
{
    PyObject *res = NULL;
    PyObject *kwnames;
    PyObject **newargs;
    PyObject **kwvalues;
    Py_ssize_t i;
    #if CYTHON_AVOID_BORROWED_REFS
    PyObject *pos;
    #else
    Py_ssize_t pos;
    #endif
    size_t j;
    PyObject *key, *value;
    unsigned long keys_are_strings;
    #if !CYTHON_ASSUME_SAFE_SIZE
    Py_ssize_t nkw = PyDict_Size(kw);
    if (unlikely(nkw == -1)) return NULL;
    #else
    Py_ssize_t nkw = PyDict_GET_SIZE(kw);
    #endif
    newargs = (PyObject **)PyMem_Malloc((nargs + (size_t)nkw) * sizeof(args[0]));
    if (unlikely(newargs == NULL)) {
        PyErr_NoMemory();
        return NULL;
    }
    for (j = 0; j < nargs; j++) newargs[j] = args[j];
    kwnames = PyTuple_New(nkw);
    if (unlikely(kwnames == NULL)) {
        PyMem_Free(newargs);
        return NULL;
    }
    kwvalues = newargs + nargs;
    pos = 0;
    i = 0;
    keys_are_strings = Py_TPFLAGS_UNICODE_SUBCLASS;
    while (__Pyx_PyDict_NextRef(kw, &pos, &key, &value)) {
        keys_are_strings &=
        #if CYTHON_COMPILING_IN_LIMITED_API
            PyType_GetFlags(Py_TYPE(key));
        #else
            Py_TYPE(key)->tp_flags;
        #endif
        #if !CYTHON_ASSUME_SAFE_MACROS
        if (unlikely(PyTuple_SetItem(kwnames, i, key) < 0)) goto cleanup;
        #else
        PyTuple_SET_ITEM(kwnames, i, key);
        #endif
        kwvalues[i] = value;
        i++;
    }
    if (unlikely(!keys_are_strings)) {
        PyErr_SetString(PyExc_TypeError, "keywords must be strings");
        goto cleanup;
    }
    res = vc(func, newargs, nargs, kwnames);
cleanup:
    #if CYTHON_AVOID_BORROWED_REFS
    Py_DECREF(pos);
    #endif
    Py_DECREF(kwnames);
    for (i = 0; i < nkw; i++)
        Py_DECREF(kwvalues[i]);
    PyMem_Free(newargs);
    return res;
}
static CYTHON_INLINE PyObject *__Pyx_PyVectorcall_FastCallDict(PyObject *func, __pyx_vectorcallfunc vc, PyObject *const *args, size_t nargs, PyObject *kw)
{
    Py_ssize_t kw_size =
        likely(kw == NULL) ?
        0 :
#if !CYTHON_ASSUME_SAFE_SIZE
        PyDict_Size(kw);
#else
        PyDict_GET_SIZE(kw);
#endif
    if (kw_size == 0) {
        return vc(func, args, nargs, NULL);
    }
#if !CYTHON_ASSUME_SAFE_SIZE
    else if (unlikely(kw_size == -1)) {
        return NULL;
    }
#endif
    return __Pyx_PyVectorcall_FastCallDict_kw(func, vc, args, nargs, kw);
}
#endif

/* CythonFunctionShared (used by CythonFunction) */
#if CYTHON_COMPILING_IN_LIMITED_API
static CYTHON_INLINE int __Pyx__IsSameCyOrCFunctionNoMethod(PyObject *func, void (*cfunc)(void)) {
    if (__Pyx_CyFunction_Check(func)) {
        return PyCFunction_GetFunction(((__pyx_CyFunctionObject*)func)->func) == (PyCFunction) cfunc;
    } else if (PyCFunction_Check(func)) {
        return PyCFunction_GetFunction(func) == (PyCFunction) cfunc;
    }
    return 0;
}
static CYTHON_INLINE int __Pyx__IsSameCyOrCFunction(PyObject *func, void (*cfunc)(void)) {
    if ((PyObject*)Py_TYPE(func) == __pyx_mstate_global->__Pyx_CachedMethodType) {
        int result;
        PyObject *newFunc = PyObject_GetAttr(func, __pyx_mstate_global->__pyx_n_u_func);
        if (unlikely(!newFunc)) {
            PyErr_Clear(); // It's only an optimization, so don't throw an error
            return 0;
        }
        result = __Pyx__IsSameCyOrCFunctionNoMethod(newFunc, cfunc);
        Py_DECREF(newFunc);
        return result;
    }
    return __Pyx__IsSameCyOrCFunctionNoMethod(func, cfunc);
}
#else
static CYTHON_INLINE int __Pyx__IsSameCyOrCFunction(PyObject *func, void (*cfunc)(void)) {
    if (PyMethod_Check(func)) {
        func = PyMethod_GET_FUNCTION(func);
    }
    return __Pyx_CyOrPyCFunction_Check(func) && __Pyx_CyOrPyCFunction_GET_FUNCTION(func) == (PyCFunction) cfunc;
}
#endif
static CYTHON_INLINE void __Pyx__CyFunction_SetClassObj(__pyx_CyFunctionObject* f, PyObject* classobj) {
#if PY_VERSION_HEX < 0x030900B1 || CYTHON_COMPILING_IN_LIMITED_API
    __Pyx_Py_XDECREF_SET(
        __Pyx_CyFunction_GetClassObj(f),
            ((classobj) ? __Pyx_NewRef(classobj) : NULL));
#else
    __Pyx_Py_XDECREF_SET(
        ((PyCMethodObject *) (f))->mm_class,
        (PyTypeObject*)((classobj) ? __Pyx_NewRef(classobj) : NULL));
#endif
}
static PyObject *
__Pyx_CyFunction_get_doc_locked(__pyx_CyFunctionObject *op)
{
    if (unlikely(op->func_doc == NULL)) {
#if CYTHON_COMPILING_IN_LIMITED_API
        op->func_doc = PyObject_GetAttrString(op->func, "__doc__");
        if (unlikely(!op->func_doc)) return NULL;
#else
        if (((PyCFunctionObject*)op)->m_ml->ml_doc) {
            op->func_doc = PyUnicode_FromString(((PyCFunctionObject*)op)->m_ml->ml_doc);
            if (unlikely(op->func_doc == NULL))
                return NULL;
        } else {
            Py_INCREF(Py_None);
            return Py_None;
        }
#endif
    }
    Py_INCREF(op->func_doc);
    return op->func_doc;
}
static PyObject *
__Pyx_CyFunction_get_doc(__pyx_CyFunctionObject *op, void *closure) {
    PyObject *result;
    CYTHON_UNUSED_VAR(closure);
    __Pyx_BEGIN_CRITICAL_SECTION(op);
    result = __Pyx_CyFunction_get_doc_locked(op);
    __Pyx_END_CRITICAL_SECTION();
    return result;
}
static int
__Pyx_CyFunction_set_doc(__pyx_CyFunctionObject *op, PyObject *value, void *context)
{
    CYTHON_UNUSED_VAR(context);
    if (value == NULL) {
        value = Py_None;
    }
    Py_INCREF(value);
    __Pyx_BEGIN_CRITICAL_SECTION(op);
    __Pyx_Py_XDECREF_SET(op->func_doc, value);
    __Pyx_END_CRITICAL_SECTION();
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_name_locked(__pyx_CyFunctionObject *op)
{
    if (unlikely(op->func_name == NULL)) {
#if CYTHON_COMPILING_IN_LIMITED_API
        op->func_name = PyObject_GetAttrString(op->func, "__name__");
#else
        op->func_name = PyUnicode_InternFromString(((PyCFunctionObject*)op)->m_ml->ml_name);
#endif
        if (unlikely(op->func_name == NULL))
            return NULL;
    }
    Py_INCREF(op->func_name);
    return op->func_name;
}
static PyObject *
__Pyx_CyFunction_get_name(__pyx_CyFunctionObject *op, void *context)
{
    PyObject *result = NULL;
    CYTHON_UNUSED_VAR(context);
    __Pyx_BEGIN_CRITICAL_SECTION(op);
    result = __Pyx_CyFunction_get_name_locked(op);
    __Pyx_END_CRITICAL_SECTION();
    return result;
}
static int
__Pyx_CyFunction_set_name(__pyx_CyFunctionObject *op, PyObject *value, void *context)
{
    CYTHON_UNUSED_VAR(context);
    if (unlikely(value == NULL || !PyUnicode_Check(value))) {
        PyErr_SetString(PyExc_TypeError,
                        "__name__ must be set to a string object");
        return -1;
    }
    Py_INCREF(value);
    __Pyx_BEGIN_CRITICAL_SECTION(op);
    __Pyx_Py_XDECREF_SET(op->func_name, value);
    __Pyx_END_CRITICAL_SECTION();
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_qualname(__pyx_CyFunctionObject *op, void *context)
{
    CYTHON_UNUSED_VAR(context);
    PyObject *result;
    __Pyx_BEGIN_CRITICAL_SECTION(op);
    Py_INCREF(op->func_qualname);
    result = op->func_qualname;
    __Pyx_END_CRITICAL_SECTION();
    return result;
}
static int
__Pyx_CyFunction_set_qualname(__pyx_CyFunctionObject *op, PyObject *value, void *context)
{
    CYTHON_UNUSED_VAR(context);
    if (unlikely(value == NULL || !PyUnicode_Check(value))) {
        PyErr_SetString(PyExc_TypeError,
                        "__qualname__ must be set to a string object");
        return -1;
    }
    Py_INCREF(value);
    __Pyx_BEGIN_CRITICAL_SECTION(op);
    __Pyx_Py_XDECREF_SET(op->func_qualname, value);
    __Pyx_END_CRITICAL_SECTION();
    return 0;
}
#if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030A0000
static PyObject *
__Pyx_CyFunction_get_dict(__pyx_CyFunctionObject *op, void *context)
{
    CYTHON_UNUSED_VAR(context);
    if (unlikely(op->func_dict == NULL)) {
        op->func_dict = PyDict_New();
        if (unlikely(op->func_dict == NULL))
            return NULL;
    }
    Py_INCREF(op->func_dict);
    return op->func_dict;
}
#endif
static PyObject *
__Pyx_CyFunction_get_globals(__pyx_CyFunctionObject *op, void *context)
{
    CYTHON_UNUSED_VAR(context);
    Py_INCREF(op->func_globals);
    return op->func_globals;
}
static PyObject *
__Pyx_CyFunction_get_closure(__pyx_CyFunctionObject *op, void *context)
{
    CYTHON_UNUSED_VAR(op);
    CYTHON_UNUSED_VAR(context);
    Py_INCREF(Py_None);
    return Py_None;
}
static PyObject *
__Pyx_CyFunction_get_code(__pyx_CyFunctionObject *op, void *context)
{
    PyObject* result = (op->func_code) ? op->func_code : Py_None;
    CYTHON_UNUSED_VAR(context);
    Py_INCREF(result);
    return result;
}
static int
__Pyx_CyFunction_init_defaults(__pyx_CyFunctionObject *op) {
    int result = 0;
    PyObject *res = op->defaults_getter((PyObject *) op);
    if (unlikely(!res))
        return -1;
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    op->defaults_tuple = PyTuple_GET_ITEM(res, 0);
    Py_INCREF(op->defaults_tuple);
    op->defaults_kwdict = PyTuple_GET_ITEM(res, 1);
    Py_INCREF(op->defaults_kwdict);
    #else
    op->defaults_tuple = __Pyx_PySequence_ITEM(res, 0);
    if (unlikely(!op->defaults_tuple)) result = -1;
    else {
        op->defaults_kwdict = __Pyx_PySequence_ITEM(res, 1);
        if (unlikely(!op->defaults_kwdict)) result = -1;
    }
    #endif
    Py_DECREF(res);
    return result;
}
static int
__Pyx_CyFunction_set_defaults(__pyx_CyFunctionObject *op, PyObject* value, void *context) {
    CYTHON_UNUSED_VAR(context);
    if (!value) {
        value = Py_None;
    } else if (unlikely(value != Py_None && !PyTuple_Check(value))) {
        PyErr_SetString(PyExc_TypeError,
                        "__defaults__ must be set to a tuple object");
        return -1;
    }
    PyErr_WarnEx(PyExc_RuntimeWarning, "changes to cyfunction.__defaults__ will not "
                 "currently affect the values used in function calls", 1);
    Py_INCREF(value);
    __Pyx_BEGIN_CRITICAL_SECTION(op);
    __Pyx_Py_XDECREF_SET(op->defaults_tuple, value);
    __Pyx_END_CRITICAL_SECTION();
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_defaults_locked(__pyx_CyFunctionObject *op) {
    PyObject* result = op->defaults_tuple;
    if (unlikely(!result)) {
        if (op->defaults_getter) {
            if (unlikely(__Pyx_CyFunction_init_defaults(op) < 0)) return NULL;
            result = op->defaults_tuple;
        } else {
            result = Py_None;
        }
    }
    Py_INCREF(result);
    return result;
}
static PyObject *
__Pyx_CyFunction_get_defaults(__pyx_CyFunctionObject *op, void *context) {
    PyObject* result = NULL;
    CYTHON_UNUSED_VAR(context);
    __Pyx_BEGIN_CRITICAL_SECTION(op);
    result = __Pyx_CyFunction_get_defaults_locked(op);
    __Pyx_END_CRITICAL_SECTION();
    return result;
}
static int
__Pyx_CyFunction_set_kwdefaults(__pyx_CyFunctionObject *op, PyObject* value, void *context) {
    CYTHON_UNUSED_VAR(context);
    if (!value) {
        value = Py_None;
    } else if (unlikely(value != Py_None && !PyDict_Check(value))) {
        PyErr_SetString(PyExc_TypeError,
                        "__kwdefaults__ must be set to a dict object");
        return -1;
    }
    PyErr_WarnEx(PyExc_RuntimeWarning, "changes to cyfunction.__kwdefaults__ will not "
                 "currently affect the values used in function calls", 1);
    Py_INCREF(value);
    __Pyx_BEGIN_CRITICAL_SECTION(op);
    __Pyx_Py_XDECREF_SET(op->defaults_kwdict, value);
    __Pyx_END_CRITICAL_SECTION();
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_kwdefaults_locked(__pyx_CyFunctionObject *op) {
    PyObject* result = op->defaults_kwdict;
    if (unlikely(!result)) {
        if (op->defaults_getter) {
            if (unlikely(__Pyx_CyFunction_init_defaults(op) < 0)) return NULL;
            result = op->defaults_kwdict;
        } else {
            result = Py_None;
        }
    }
    Py_INCREF(result);
    return result;
}
static PyObject *
__Pyx_CyFunction_get_kwdefaults(__pyx_CyFunctionObject *op, void *context) {
    PyObject* result;
    CYTHON_UNUSED_VAR(context);
    __Pyx_BEGIN_CRITICAL_SECTION(op);
    result = __Pyx_CyFunction_get_kwdefaults_locked(op);
    __Pyx_END_CRITICAL_SECTION();
    return result;
}
static int
__Pyx_CyFunction_set_annotations(__pyx_CyFunctionObject *op, PyObject* value, void *context) {
    CYTHON_UNUSED_VAR(context);
    if (!value || value == Py_None) {
        value = NULL;
    } else if (unlikely(!PyDict_Check(value))) {
        PyErr_SetString(PyExc_TypeError,
                        "__annotations__ must be set to a dict object");
        return -1;
    }
    Py_XINCREF(value);
    __Pyx_BEGIN_CRITICAL_SECTION(op);
    __Pyx_Py_XDECREF_SET(op->func_annotations, value);
    __Pyx_END_CRITICAL_SECTION();
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_annotations_locked(__pyx_CyFunctionObject *op) {
    PyObject* result = op->func_annotations;
    if (unlikely(!result)) {
        result = PyDict_New();
        if (unlikely(!result)) return NULL;
        op->func_annotations = result;
    }
    Py_INCREF(result);
    return result;
}
static PyObject *
__Pyx_CyFunction_get_annotations(__pyx_CyFunctionObject *op, void *context) {
    PyObject *result;
    CYTHON_UNUSED_VAR(context);
    __Pyx_BEGIN_CRITICAL_SECTION(op);
    result = __Pyx_CyFunction_get_annotations_locked(op);
    __Pyx_END_CRITICAL_SECTION();
    return result;
}
static PyObject *
__Pyx_CyFunction_get_is_coroutine_value(__pyx_CyFunctionObject *op) {
    int is_coroutine = op->flags & __Pyx_CYFUNCTION_COROUTINE;
    if (is_coroutine) {
        PyObject *is_coroutine_value, *module, *fromlist, *marker = __pyx_mstate_global->__pyx_n_u_is_coroutine;
        fromlist = PyList_New(1);
        if (unlikely(!fromlist)) return NULL;
        Py_INCREF(marker);
#if CYTHON_ASSUME_SAFE_MACROS
        PyList_SET_ITEM(fromlist, 0, marker);
#else
        if (unlikely(PyList_SetItem(fromlist, 0, marker) < 0)) {
            Py_DECREF(marker);
            Py_DECREF(fromlist);
            return NULL;
        }
#endif
        module = PyImport_ImportModuleLevelObject(__pyx_mstate_global->__pyx_n_u_asyncio_coroutines, NULL, NULL, fromlist, 0);
        Py_DECREF(fromlist);
        if (unlikely(!module)) goto ignore;
        is_coroutine_value = __Pyx_PyObject_GetAttrStr(module, marker);
        Py_DECREF(module);
        if (likely(is_coroutine_value)) {
            return is_coroutine_value;
        }
ignore:
        PyErr_Clear();
    }
    return __Pyx_PyBool_FromLong(is_coroutine);
}
static PyObject *
__Pyx_CyFunction_get_is_coroutine(__pyx_CyFunctionObject *op, void *context) {
    PyObject *result;
    CYTHON_UNUSED_VAR(context);
    if (op->func_is_coroutine) {
        return __Pyx_NewRef(op->func_is_coroutine);
    }
    result = __Pyx_CyFunction_get_is_coroutine_value(op);
    if (unlikely(!result))
        return NULL;
    __Pyx_BEGIN_CRITICAL_SECTION(op);
    if (op->func_is_coroutine) {
        Py_DECREF(result);
        result = __Pyx_NewRef(op->func_is_coroutine);
    } else {
        op->func_is_coroutine = __Pyx_NewRef(result);
    }
    __Pyx_END_CRITICAL_SECTION();
    return result;
}
static void __Pyx_CyFunction_raise_argument_count_error(__pyx_CyFunctionObject *func, const char* message, Py_ssize_t size) {
#if CYTHON_COMPILING_IN_LIMITED_API
    PyObject *py_name = __Pyx_CyFunction_get_name(func, NULL);
    if (!py_name) return;
    PyErr_Format(PyExc_TypeError,
        "%.200S() %s (%" CYTHON_FORMAT_SSIZE_T "d given)",
        py_name, message, size);
    Py_DECREF(py_name);
#else
    const char* name = ((PyCFunctionObject*)func)->m_ml->ml_name;
    PyErr_Format(PyExc_TypeError,
        "%.200s() %s (%" CYTHON_FORMAT_SSIZE_T "d given)",
        name, message, size);
#endif
}
static void __Pyx_CyFunction_raise_type_error(__pyx_CyFunctionObject *func, const char* message) {
#if CYTHON_COMPILING_IN_LIMITED_API
    PyObject *py_name = __Pyx_CyFunction_get_name(func, NULL);
    if (!py_name) return;
    PyErr_Format(PyExc_TypeError,
        "%.200S() %s",
        py_name, message);
    Py_DECREF(py_name);
#else
    const char* name = ((PyCFunctionObject*)func)->m_ml->ml_name;
    PyErr_Format(PyExc_TypeError,
        "%.200s() %s",
        name, message);
#endif
}
#if CYTHON_COMPILING_IN_LIMITED_API
static PyObject *
__Pyx_CyFunction_get_module(__pyx_CyFunctionObject *op, void *context) {
    CYTHON_UNUSED_VAR(context);
    return PyObject_GetAttrString(op->func, "__module__");
}
static int
__Pyx_CyFunction_set_module(__pyx_CyFunctionObject *op, PyObject* value, void *context) {
    CYTHON_UNUSED_VAR(context);
    return PyObject_SetAttrString(op->func, "__module__", value);
}
#endif
static PyGetSetDef __pyx_CyFunction_getsets[] = {
    {"func_doc", (getter)__Pyx_CyFunction_get_doc, (setter)__Pyx_CyFunction_set_doc, 0, 0},
    {"__doc__",  (getter)__Pyx_CyFunction_get_doc, (setter)__Pyx_CyFunction_set_doc, 0, 0},
    {"func_name", (getter)__Pyx_CyFunction_get_name, (setter)__Pyx_CyFunction_set_name, 0, 0},
    {"__name__", (getter)__Pyx_CyFunction_get_name, (setter)__Pyx_CyFunction_set_name, 0, 0},
    {"__qualname__", (getter)__Pyx_CyFunction_get_qualname, (setter)__Pyx_CyFunction_set_qualname, 0, 0},
#if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030A0000
    {"func_dict", (getter)__Pyx_CyFunction_get_dict, (setter)PyObject_GenericSetDict, 0, 0},
    {"__dict__", (getter)__Pyx_CyFunction_get_dict, (setter)PyObject_GenericSetDict, 0, 0},
#else
    {"func_dict", (getter)PyObject_GenericGetDict, (setter)PyObject_GenericSetDict, 0, 0},
    {"__dict__", (getter)PyObject_GenericGetDict, (setter)PyObject_GenericSetDict, 0, 0},
#endif
    {"func_globals", (getter)__Pyx_CyFunction_get_globals, 0, 0, 0},
    {"__globals__", (getter)__Pyx_CyFunction_get_globals, 0, 0, 0},
    {"func_closure", (getter)__Pyx_CyFunction_get_closure, 0, 0, 0},
    {"__closure__", (getter)__Pyx_CyFunction_get_closure, 0, 0, 0},
    {"func_code", (getter)__Pyx_CyFunction_get_code, 0, 0, 0},
    {"__code__", (getter)__Pyx_CyFunction_get_code, 0, 0, 0},
    {"func_defaults", (getter)__Pyx_CyFunction_get_defaults, (setter)__Pyx_CyFunction_set_defaults, 0, 0},
    {"__defaults__", (getter)__Pyx_CyFunction_get_defaults, (setter)__Pyx_CyFunction_set_defaults, 0, 0},
    {"__kwdefaults__", (getter)__Pyx_CyFunction_get_kwdefaults, (setter)__Pyx_CyFunction_set_kwdefaults, 0, 0},
    {"__annotations__", (getter)__Pyx_CyFunction_get_annotations, (setter)__Pyx_CyFunction_set_annotations, 0, 0},
    {"_is_coroutine", (getter)__Pyx_CyFunction_get_is_coroutine, 0, 0, 0},
#if CYTHON_COMPILING_IN_LIMITED_API
    {"__module__", (getter)__Pyx_CyFunction_get_module, (setter)__Pyx_CyFunction_set_module, 0, 0},
#endif
    {0, 0, 0, 0, 0}
};
static PyMemberDef __pyx_CyFunction_members[] = {
#if !CYTHON_COMPILING_IN_LIMITED_API
    {"__module__", T_OBJECT, offsetof(PyCFunctionObject, m_module), 0, 0},
#endif
#if PY_VERSION_HEX < 0x030C0000 || CYTHON_COMPILING_IN_LIMITED_API
    {"__dictoffset__", T_PYSSIZET, offsetof(__pyx_CyFunctionObject, func_dict), READONLY, 0},
#endif
#if CYTHON_METH_FASTCALL
#if CYTHON_COMPILING_IN_LIMITED_API
    {"__vectorcalloffset__", T_PYSSIZET, offsetof(__pyx_CyFunctionObject, func_vectorcall), READONLY, 0},
#else
    {"__vectorcalloffset__", T_PYSSIZET, offsetof(PyCFunctionObject, vectorcall), READONLY, 0},
#endif
#if CYTHON_COMPILING_IN_LIMITED_API
    {"__weaklistoffset__", T_PYSSIZET, offsetof(__pyx_CyFunctionObject, func_weakreflist), READONLY, 0},
#else
    {"__weaklistoffset__", T_PYSSIZET, offsetof(PyCFunctionObject, m_weakreflist), READONLY, 0},
#endif
#endif
    {0, 0, 0,  0, 0}
};
static PyObject *
__Pyx_CyFunction_reduce(__pyx_CyFunctionObject *m, PyObject *args)
{
    PyObject *result = NULL;
    CYTHON_UNUSED_VAR(args);
    __Pyx_BEGIN_CRITICAL_SECTION(m);
    Py_INCREF(m->func_qualname);
    result = m->func_qualname;
    __Pyx_END_CRITICAL_SECTION();
    return result;
}
static PyMethodDef __pyx_CyFunction_methods[] = {
    {"__reduce__", (PyCFunction)__Pyx_CyFunction_reduce, METH_VARARGS, 0},
    {0, 0, 0, 0}
};
#if CYTHON_COMPILING_IN_LIMITED_API
#define __Pyx_CyFunction_weakreflist(cyfunc) ((cyfunc)->func_weakreflist)
#else
#define __Pyx_CyFunction_weakreflist(cyfunc) (((PyCFunctionObject*)cyfunc)->m_weakreflist)
#endif
static PyObject *__Pyx_CyFunction_Init(__pyx_CyFunctionObject *op, PyMethodDef *ml, int flags, PyObject* qualname,
                                       PyObject *closure, PyObject *module, PyObject* globals, PyObject* code) {
#if !CYTHON_COMPILING_IN_LIMITED_API
    PyCFunctionObject *cf = (PyCFunctionObject*) op;
#endif
    if (unlikely(op == NULL))
        return NULL;
#if CYTHON_COMPILING_IN_LIMITED_API
    op->func = PyCFunction_NewEx(ml, (PyObject*)op, module);
    if (unlikely(!op->func)) return NULL;
#endif
    op->flags = flags;
    __Pyx_CyFunction_weakreflist(op) = NULL;
#if !CYTHON_COMPILING_IN_LIMITED_API
    cf->m_ml = ml;
    cf->m_self = (PyObject *) op;
#endif
    Py_XINCREF(closure);
    op->func_closure = closure;
#if !CYTHON_COMPILING_IN_LIMITED_API
    Py_XINCREF(module);
    cf->m_module = module;
#endif
#if PY_VERSION_HEX < 0x030C0000 || CYTHON_COMPILING_IN_LIMITED_API
    op->func_dict = NULL;
#endif
    op->func_name = NULL;
    Py_INCREF(qualname);
    op->func_qualname = qualname;
    op->func_doc = NULL;
#if PY_VERSION_HEX < 0x030900B1 || CYTHON_COMPILING_IN_LIMITED_API
    op->func_classobj = NULL;
#else
    ((PyCMethodObject*)op)->mm_class = NULL;
#endif
    op->func_globals = globals;
    Py_INCREF(op->func_globals);
    Py_XINCREF(code);
    op->func_code = code;
    op->defaults = NULL;
    op->defaults_tuple = NULL;
    op->defaults_kwdict = NULL;
    op->defaults_getter = NULL;
    op->func_annotations = NULL;
    op->func_is_coroutine = NULL;
#if CYTHON_METH_FASTCALL
    switch (ml->ml_flags & (METH_VARARGS | METH_FASTCALL | METH_NOARGS | METH_O | METH_KEYWORDS | METH_METHOD)) {
    case METH_NOARGS:
        __Pyx_CyFunction_func_vectorcall(op) = __Pyx_CyFunction_Vectorcall_NOARGS;
        break;
    case METH_O:
        __Pyx_CyFunction_func_vectorcall(op) = __Pyx_CyFunction_Vectorcall_O;
        break;
    case METH_METHOD | METH_FASTCALL | METH_KEYWORDS:
        __Pyx_CyFunction_func_vectorcall(op) = __Pyx_CyFunction_Vectorcall_FASTCALL_KEYWORDS_METHOD;
        break;
    case METH_FASTCALL | METH_KEYWORDS:
        __Pyx_CyFunction_func_vectorcall(op) = __Pyx_CyFunction_Vectorcall_FASTCALL_KEYWORDS;
        break;
    case METH_VARARGS | METH_KEYWORDS:
        __Pyx_CyFunction_func_vectorcall(op) = NULL;
        break;
    default:
        PyErr_SetString(PyExc_SystemError, "Bad call flags for CyFunction");
        Py_DECREF(op);
        return NULL;
    }
#endif
    return (PyObject *) op;
}
static int
__Pyx_CyFunction_clear(__pyx_CyFunctionObject *m)
{
    Py_CLEAR(m->func_closure);
#if CYTHON_COMPILING_IN_LIMITED_API
    Py_CLEAR(m->func);
#else
    Py_CLEAR(((PyCFunctionObject*)m)->m_module);
#endif
#if PY_VERSION_HEX < 0x030C0000 || CYTHON_COMPILING_IN_LIMITED_API
    Py_CLEAR(m->func_dict);
#elif PY_VERSION_HEX < 0x030d0000
    _PyObject_ClearManagedDict((PyObject*)m);
#else
    PyObject_ClearManagedDict((PyObject*)m);
#endif
    Py_CLEAR(m->func_name);
    Py_CLEAR(m->func_qualname);
    Py_CLEAR(m->func_doc);
    Py_CLEAR(m->func_globals);
    Py_CLEAR(m->func_code);
#if !CYTHON_COMPILING_IN_LIMITED_API
#if PY_VERSION_HEX < 0x030900B1
    Py_CLEAR(__Pyx_CyFunction_GetClassObj(m));
#else
    {
        PyObject *cls = (PyObject*) ((PyCMethodObject *) (m))->mm_class;
        ((PyCMethodObject *) (m))->mm_class = NULL;
        Py_XDECREF(cls);
    }
#endif
#endif
    Py_CLEAR(m->defaults_tuple);
    Py_CLEAR(m->defaults_kwdict);
    Py_CLEAR(m->func_annotations);
    Py_CLEAR(m->func_is_coroutine);
    Py_CLEAR(m->defaults);
    return 0;
}
static void __Pyx__CyFunction_dealloc(__pyx_CyFunctionObject *m)
{
    if (__Pyx_CyFunction_weakreflist(m) != NULL)
        PyObject_ClearWeakRefs((PyObject *) m);
    __Pyx_CyFunction_clear(m);
    __Pyx_PyHeapTypeObject_GC_Del(m);
}
static void __Pyx_CyFunction_dealloc(__pyx_CyFunctionObject *m)
{
    PyObject_GC_UnTrack(m);
    __Pyx__CyFunction_dealloc(m);
}
static int __Pyx_CyFunction_traverse(__pyx_CyFunctionObject *m, visitproc visit, void *arg)
{
    {
        int e = __Pyx_call_type_traverse((PyObject*)m, 1, visit, arg);
        if (e) return e;
    }
    Py_VISIT(m->func_closure);
#if CYTHON_COMPILING_IN_LIMITED_API
    Py_VISIT(m->func);
#else
    Py_VISIT(((PyCFunctionObject*)m)->m_module);
#endif
#if PY_VERSION_HEX < 0x030C0000 || CYTHON_COMPILING_IN_LIMITED_API
    Py_VISIT(m->func_dict);
#else
    {
        int e =
#if PY_VERSION_HEX < 0x030d0000
            _PyObject_VisitManagedDict
#else
            PyObject_VisitManagedDict
#endif
                ((PyObject*)m, visit, arg);
        if (e != 0) return e;
    }
#endif
    __Pyx_VISIT_CONST(m->func_name);
    __Pyx_VISIT_CONST(m->func_qualname);
    Py_VISIT(m->func_doc);
    Py_VISIT(m->func_globals);
    __Pyx_VISIT_CONST(m->func_code);
#if !CYTHON_COMPILING_IN_LIMITED_API
    Py_VISIT(__Pyx_CyFunction_GetClassObj(m));
#endif
    Py_VISIT(m->defaults_tuple);
    Py_VISIT(m->defaults_kwdict);
    Py_VISIT(m->func_is_coroutine);
    Py_VISIT(m->defaults);
    return 0;
}
static PyObject*
__Pyx_CyFunction_repr(__pyx_CyFunctionObject *op)
{
    PyObject *repr;
    __Pyx_BEGIN_CRITICAL_SECTION(op);
    repr = PyUnicode_FromFormat("<cyfunction %U at %p>",
                                op->func_qualname, (void *)op);
    __Pyx_END_CRITICAL_SECTION();
    return repr;
}
static PyObject * __Pyx_CyFunction_CallMethod(PyObject *func, PyObject *self, PyObject *arg, PyObject *kw) {
#if CYTHON_COMPILING_IN_LIMITED_API
    PyObject *f = ((__pyx_CyFunctionObject*)func)->func;
    PyCFunction meth;
    int flags;
    meth = PyCFunction_GetFunction(f);
    if (unlikely(!meth)) return NULL;
    flags = PyCFunction_GetFlags(f);
    if (unlikely(flags < 0)) return NULL;
#else
    PyCFunctionObject* f = (PyCFunctionObject*)func;
    PyCFunction meth = f->m_ml->ml_meth;
    int flags = f->m_ml->ml_flags;
#endif
    Py_ssize_t size;
    switch (flags & (METH_VARARGS | METH_KEYWORDS | METH_NOARGS | METH_O)) {
    case METH_VARARGS:
        if (likely(kw == NULL || PyDict_Size(kw) == 0))
            return (*meth)(self, arg);
        break;
    case METH_VARARGS | METH_KEYWORDS:
        return (*(PyCFunctionWithKeywords)(void(*)(void))meth)(self, arg, kw);
    case METH_NOARGS:
        if (likely(kw == NULL || PyDict_Size(kw) == 0)) {
#if CYTHON_ASSUME_SAFE_SIZE
            size = PyTuple_GET_SIZE(arg);
#else
            size = PyTuple_Size(arg);
            if (unlikely(size < 0)) return NULL;
#endif
            if (likely(size == 0))
                return (*meth)(self, NULL);
            __Pyx_CyFunction_raise_argument_count_error(
                (__pyx_CyFunctionObject*)func,
                "takes no arguments", size);
            return NULL;
        }
        break;
    case METH_O:
        if (likely(kw == NULL || PyDict_Size(kw) == 0)) {
#if CYTHON_ASSUME_SAFE_SIZE
            size = PyTuple_GET_SIZE(arg);
#else
            size = PyTuple_Size(arg);
            if (unlikely(size < 0)) return NULL;
#endif
            if (likely(size == 1)) {
                PyObject *result, *arg0;
                #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
                arg0 = PyTuple_GET_ITEM(arg, 0);
                #else
                arg0 = __Pyx_PySequence_ITEM(arg, 0); if (unlikely(!arg0)) return NULL;
                #endif
                result = (*meth)(self, arg0);
                #if !(CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS)
                Py_DECREF(arg0);
                #endif
                return result;
            }
            __Pyx_CyFunction_raise_argument_count_error(
                (__pyx_CyFunctionObject*)func,
                "takes exactly one argument", size);
            return NULL;
        }
        break;
    default:
        PyErr_SetString(PyExc_SystemError, "Bad call flags for CyFunction");
        return NULL;
    }
    __Pyx_CyFunction_raise_type_error(
        (__pyx_CyFunctionObject*)func, "takes no keyword arguments");
    return NULL;
}
static CYTHON_INLINE PyObject *__Pyx_CyFunction_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    PyObject *self, *result;
#if CYTHON_COMPILING_IN_LIMITED_API
    self = PyCFunction_GetSelf(((__pyx_CyFunctionObject*)func)->func);
    if (unlikely(!self) && PyErr_Occurred()) return NULL;
#else
    self = ((PyCFunctionObject*)func)->m_self;
#endif
    result = __Pyx_CyFunction_CallMethod(func, self, arg, kw);
    return result;
}
static PyObject *__Pyx_CyFunction_CallAsMethod(PyObject *func, PyObject *args, PyObject *kw) {
    PyObject *result;
    __pyx_CyFunctionObject *cyfunc = (__pyx_CyFunctionObject *) func;
#if CYTHON_METH_FASTCALL && CYTHON_VECTORCALL
     __pyx_vectorcallfunc vc = __Pyx_CyFunction_func_vectorcall(cyfunc);
    if (vc) {
#if CYTHON_ASSUME_SAFE_MACROS && CYTHON_ASSUME_SAFE_SIZE
        return __Pyx_PyVectorcall_FastCallDict(func, vc, &PyTuple_GET_ITEM(args, 0), (size_t)PyTuple_GET_SIZE(args), kw);
#else
        (void) &__Pyx_PyVectorcall_FastCallDict;
        return PyVectorcall_Call(func, args, kw);
#endif
    }
#endif
    if ((cyfunc->flags & __Pyx_CYFUNCTION_CCLASS) && !(cyfunc->flags & __Pyx_CYFUNCTION_STATICMETHOD)) {
        Py_ssize_t argc;
        PyObject *new_args;
        PyObject *self;
#if CYTHON_ASSUME_SAFE_SIZE
        argc = PyTuple_GET_SIZE(args);
#else
        argc = PyTuple_Size(args);
        if (unlikely(argc < 0)) return NULL;
#endif
        new_args = PyTuple_GetSlice(args, 1, argc);
        if (unlikely(!new_args))
            return NULL;
        self = PyTuple_GetItem(args, 0);
        if (unlikely(!self)) {
            Py_DECREF(new_args);
            PyErr_Format(PyExc_TypeError,
                         "unbound method %.200S() needs an argument",
                         cyfunc->func_qualname);
            return NULL;
        }
        result = __Pyx_CyFunction_CallMethod(func, self, new_args, kw);
        Py_DECREF(new_args);
    } else {
        result = __Pyx_CyFunction_Call(func, args, kw);
    }
    return result;
}
#if CYTHON_METH_FASTCALL && CYTHON_VECTORCALL
static CYTHON_INLINE int __Pyx_CyFunction_Vectorcall_CheckArgs(__pyx_CyFunctionObject *cyfunc, Py_ssize_t nargs, PyObject *kwnames)
{
    int ret = 0;
    if ((cyfunc->flags & __Pyx_CYFUNCTION_CCLASS) && !(cyfunc->flags & __Pyx_CYFUNCTION_STATICMETHOD)) {
        if (unlikely(nargs < 1)) {
            __Pyx_CyFunction_raise_type_error(
                cyfunc, "needs an argument");
            return -1;
        }
        ret = 1;
    }
    if (unlikely(kwnames) && unlikely(__Pyx_PyTuple_GET_SIZE(kwnames))) {
        __Pyx_CyFunction_raise_type_error(
            cyfunc, "takes no keyword arguments");
        return -1;
    }
    return ret;
}
static PyObject * __Pyx_CyFunction_Vectorcall_NOARGS(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames)
{
    __pyx_CyFunctionObject *cyfunc = (__pyx_CyFunctionObject *)func;
    Py_ssize_t nargs = PyVectorcall_NARGS(nargsf);
    PyObject *self;
#if CYTHON_COMPILING_IN_LIMITED_API
    PyCFunction meth = PyCFunction_GetFunction(cyfunc->func);
    if (unlikely(!meth)) return NULL;
#else
    PyCFunction meth = ((PyCFunctionObject*)cyfunc)->m_ml->ml_meth;
#endif
    switch (__Pyx_CyFunction_Vectorcall_CheckArgs(cyfunc, nargs, kwnames)) {
    case 1:
        self = args[0];
        args += 1;
        nargs -= 1;
        break;
    case 0:
#if CYTHON_COMPILING_IN_LIMITED_API
        self = PyCFunction_GetSelf(((__pyx_CyFunctionObject*)cyfunc)->func);
        if (unlikely(!self) && PyErr_Occurred()) return NULL;
#else
        self = ((PyCFunctionObject*)cyfunc)->m_self;
#endif
        break;
    default:
        return NULL;
    }
    if (unlikely(nargs != 0)) {
        __Pyx_CyFunction_raise_argument_count_error(
            cyfunc, "takes no arguments", nargs);
        return NULL;
    }
    return meth(self, NULL);
}
static PyObject * __Pyx_CyFunction_Vectorcall_O(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames)
{
    __pyx_CyFunctionObject *cyfunc = (__pyx_CyFunctionObject *)func;
    Py_ssize_t nargs = PyVectorcall_NARGS(nargsf);
    PyObject *self;
#if CYTHON_COMPILING_IN_LIMITED_API
    PyCFunction meth = PyCFunction_GetFunction(cyfunc->func);
    if (unlikely(!meth)) return NULL;
#else
    PyCFunction meth = ((PyCFunctionObject*)cyfunc)->m_ml->ml_meth;
#endif
    switch (__Pyx_CyFunction_Vectorcall_CheckArgs(cyfunc, nargs, kwnames)) {
    case 1:
        self = args[0];
        args += 1;
        nargs -= 1;
        break;
    case 0:
#if CYTHON_COMPILING_IN_LIMITED_API
        self = PyCFunction_GetSelf(((__pyx_CyFunctionObject*)cyfunc)->func);
        if (unlikely(!self) && PyErr_Occurred()) return NULL;
#else
        self = ((PyCFunctionObject*)cyfunc)->m_self;
#endif
        break;
    default:
        return NULL;
    }
    if (unlikely(nargs != 1)) {
        __Pyx_CyFunction_raise_argument_count_error(
            cyfunc, "takes exactly one argument", nargs);
        return NULL;
    }
    return meth(self, args[0]);
}
static PyObject * __Pyx_CyFunction_Vectorcall_FASTCALL_KEYWORDS(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames)
{
    __pyx_CyFunctionObject *cyfunc = (__pyx_CyFunctionObject *)func;
    Py_ssize_t nargs = PyVectorcall_NARGS(nargsf);
    PyObject *self;
#if CYTHON_COMPILING_IN_LIMITED_API
    PyCFunction meth = PyCFunction_GetFunction(cyfunc->func);
    if (unlikely(!meth)) return NULL;
#else
    PyCFunction meth = ((PyCFunctionObject*)cyfunc)->m_ml->ml_meth;
#endif
    switch (__Pyx_CyFunction_Vectorcall_CheckArgs(cyfunc, nargs, NULL)) {
    case 1:
        self = args[0];
        args += 1;
        nargs -= 1;
        break;
    case 0:
#if CYTHON_COMPILING_IN_LIMITED_API
        self = PyCFunction_GetSelf(((__pyx_CyFunctionObject*)cyfunc)->func);
        if (unlikely(!self) && PyErr_Occurred()) return NULL;
#else
        self = ((PyCFunctionObject*)cyfunc)->m_self;
#endif
        break;
    default:
        return NULL;
    }
    return ((__Pyx_PyCFunctionFastWithKeywords)(void(*)(void))meth)(self, args, nargs, kwnames);
}
static PyObject * __Pyx_CyFunction_Vectorcall_FASTCALL_KEYWORDS_METHOD(PyObject *func, PyObject *const *args, size_t nargsf, PyObject *kwnames)
{
    __pyx_CyFunctionObject *cyfunc = (__pyx_CyFunctionObject *)func;
    PyTypeObject *cls = (PyTypeObject *) __Pyx_CyFunction_GetClassObj(cyfunc);
    Py_ssize_t nargs = PyVectorcall_NARGS(nargsf);
    PyObject *self;
#if CYTHON_COMPILING_IN_LIMITED_API
    PyCFunction meth = PyCFunction_GetFunction(cyfunc->func);
    if (unlikely(!meth)) return NULL;
#else
    PyCFunction meth = ((PyCFunctionObject*)cyfunc)->m_ml->ml_meth;
#endif
    switch (__Pyx_CyFunction_Vectorcall_CheckArgs(cyfunc, nargs, NULL)) {
    case 1:
        self = args[0];
        args += 1;
        nargs -= 1;
        break;
    case 0:
#if CYTHON_COMPILING_IN_LIMITED_API
        self = PyCFunction_GetSelf(((__pyx_CyFunctionObject*)cyfunc)->func);
        if (unlikely(!self) && PyErr_Occurred()) return NULL;
#else
        self = ((PyCFunctionObject*)cyfunc)->m_self;
#endif
        break;
    default:
        return NULL;
    }
    #if PY_VERSION_HEX < 0x030e00A6
    size_t nargs_value = (size_t) nargs;
    #else
    Py_ssize_t nargs_value = nargs;
    #endif
    return ((__Pyx_PyCMethod)(void(*)(void))meth)(self, cls, args, nargs_value, kwnames);
}
#endif
static PyType_Slot __pyx_CyFunctionType_slots[] = {
    {Py_tp_dealloc, (void *)__Pyx_CyFunction_dealloc},
    {Py_tp_repr, (void *)__Pyx_CyFunction_repr},
    {Py_tp_call, (void *)__Pyx_CyFunction_CallAsMethod},
    {Py_tp_traverse, (void *)__Pyx_CyFunction_traverse},
    {Py_tp_clear, (void *)__Pyx_CyFunction_clear},
    {Py_tp_methods, (void *)__pyx_CyFunction_methods},
    {Py_tp_members, (void *)__pyx_CyFunction_members},
    {Py_tp_getset, (void *)__pyx_CyFunction_getsets},
    {Py_tp_descr_get, (void *)__Pyx_PyMethod_New},
    {0, 0},
};
static PyType_Spec __pyx_CyFunctionType_spec = {
    __PYX_TYPE_MODULE_PREFIX "cython_function_or_method",
    sizeof(__pyx_CyFunctionObject),
    0,
#ifdef Py_TPFLAGS_METHOD_DESCRIPTOR
    Py_TPFLAGS_METHOD_DESCRIPTOR |
#endif
#if CYTHON_METH_FASTCALL
#if defined(Py_TPFLAGS_HAVE_VECTORCALL)
    Py_TPFLAGS_HAVE_VECTORCALL |
#elif defined(_Py_TPFLAGS_HAVE_VECTORCALL)
    _Py_TPFLAGS_HAVE_VECTORCALL |
#endif
#endif // CYTHON_METH_FASTCALL
#if PY_VERSION_HEX >= 0x030C0000 && !CYTHON_COMPILING_IN_LIMITED_API
    Py_TPFLAGS_MANAGED_DICT |
#endif
    Py_TPFLAGS_IMMUTABLETYPE | Py_TPFLAGS_DISALLOW_INSTANTIATION |
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC | Py_TPFLAGS_BASETYPE,
    __pyx_CyFunctionType_slots
};
static int __pyx_CyFunction_init(PyObject *module) {
    __pyx_mstatetype *mstate = __Pyx_PyModule_GetState(module);
    mstate->__pyx_CyFunctionType = __Pyx_FetchCommonTypeFromSpec(
        mstate->__pyx_CommonTypesMetaclassType, module, &__pyx_CyFunctionType_spec, NULL);
    if (unlikely(mstate->__pyx_CyFunctionType == NULL)) {
        return -1;
    }
    return 0;
}
static CYTHON_INLINE PyObject *__Pyx_CyFunction_InitDefaults(PyObject *func, PyTypeObject *defaults_type) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults = PyObject_CallObject((PyObject*)defaults_type, NULL); // _PyObject_New(defaults_type);
    if (unlikely(!m->defaults))
        return NULL;
    return m->defaults;
}
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsTuple(PyObject *func, PyObject *tuple) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults_tuple = tuple;
    Py_INCREF(tuple);
}
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsKwDict(PyObject *func, PyObject *dict) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults_kwdict = dict;
    Py_INCREF(dict);
}
static CYTHON_INLINE void __Pyx_CyFunction_SetAnnotationsDict(PyObject *func, PyObject *dict) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->func_annotations = dict;
    Py_INCREF(dict);
}

/* CythonFunction */
static PyObject *__Pyx_CyFunction_New(PyMethodDef *ml, int flags, PyObject* qualname,
                                      PyObject *closure, PyObject *module, PyObject* globals, PyObject* code) {
    PyObject *op = __Pyx_CyFunction_Init(
        PyObject_GC_New(__pyx_CyFunctionObject, __pyx_mstate_global->__pyx_CyFunctionType),
        ml, flags, qualname, closure, module, globals, code
    );
    if (likely(op)) {
        PyObject_GC_Track(op);
    }
    return op;
}

/* AllocateExtensionType */
static PyObject *__Pyx_AllocateExtensionType(PyTypeObject *t, int is_final) {
    if (is_final || likely(!__Pyx_PyType_HasFeature(t, Py_TPFLAGS_IS_ABSTRACT))) {
        allocfunc alloc_func = __Pyx_PyType_GetSlot(t, tp_alloc, allocfunc);
        return alloc_func(t, 0);
    } else {
        newfunc tp_new = __Pyx_PyType_TryGetSlot(&PyBaseObject_Type, tp_new, newfunc);
    #if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030A0000
        if (!tp_new) {
            PyObject *new_str = PyUnicode_FromString("__new__");
            if (likely(new_str)) {
                PyObject *o = PyObject_CallMethodObjArgs((PyObject *)&PyBaseObject_Type, new_str, t, NULL);
                Py_DECREF(new_str);
                return o;
            } else
                return NULL;
        } else
    #endif
        return tp_new(t, __pyx_mstate_global->__pyx_empty_tuple, 0);
    }
}

/* PyObjectCallNoArg (used by PyObjectCallMethod0) */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func) {
    PyObject *arg[2] = {NULL, NULL};
    return __Pyx_PyObject_FastCall(func, arg + 1, 0 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET);
}

/* PyObjectGetMethod (used by PyObjectCallMethod0) */
#if !(CYTHON_VECTORCALL && (__PYX_LIMITED_VERSION_HEX >= 0x030C0000 || (!CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x03090000)))
static int __Pyx_PyObject_GetMethod(PyObject *obj, PyObject *name, PyObject **method) {
    PyObject *attr;
#if CYTHON_UNPACK_METHODS && CYTHON_COMPILING_IN_CPYTHON && CYTHON_USE_PYTYPE_LOOKUP
    __Pyx_TypeName type_name;
    PyTypeObject *tp = Py_TYPE(obj);
    PyObject *descr;
    descrgetfunc f = NULL;
    PyObject **dictptr, *dict;
    int meth_found = 0;
    assert (*method == NULL);
    if (unlikely(tp->tp_getattro != PyObject_GenericGetAttr)) {
        attr = __Pyx_PyObject_GetAttrStr(obj, name);
        goto try_unpack;
    }
    if (unlikely(tp->tp_dict == NULL) && unlikely(PyType_Ready(tp) < 0)) {
        return 0;
    }
    descr = _PyType_Lookup(tp, name);
    if (likely(descr != NULL)) {
        Py_INCREF(descr);
#if defined(Py_TPFLAGS_METHOD_DESCRIPTOR) && Py_TPFLAGS_METHOD_DESCRIPTOR
        if (__Pyx_PyType_HasFeature(Py_TYPE(descr), Py_TPFLAGS_METHOD_DESCRIPTOR))
#else
        #ifdef __Pyx_CyFunction_USED
        if (likely(PyFunction_Check(descr) || __Pyx_IS_TYPE(descr, &PyMethodDescr_Type) || __Pyx_CyFunction_Check(descr)))
        #else
        if (likely(PyFunction_Check(descr) || __Pyx_IS_TYPE(descr, &PyMethodDescr_Type)))
        #endif
#endif
        {
            meth_found = 1;
        } else {
            f = Py_TYPE(descr)->tp_descr_get;
            if (f != NULL && PyDescr_IsData(descr)) {
                attr = f(descr, obj, (PyObject *)Py_TYPE(obj));
                Py_DECREF(descr);
                goto try_unpack;
            }
        }
    }
    dictptr = _PyObject_GetDictPtr(obj);
    if (dictptr != NULL && (dict = *dictptr) != NULL) {
        Py_INCREF(dict);
        attr = __Pyx_PyDict_GetItemStr(dict, name);
        if (attr != NULL) {
            Py_INCREF(attr);
            Py_DECREF(dict);
            Py_XDECREF(descr);
            goto try_unpack;
        }
        Py_DECREF(dict);
    }
    if (meth_found) {
        *method = descr;
        return 1;
    }
    if (f != NULL) {
        attr = f(descr, obj, (PyObject *)Py_TYPE(obj));
        Py_DECREF(descr);
        goto try_unpack;
    }
    if (likely(descr != NULL)) {
        *method = descr;
        return 0;
    }
    type_name = __Pyx_PyType_GetFullyQualifiedName(tp);
    PyErr_Format(PyExc_AttributeError,
                 "'" __Pyx_FMT_TYPENAME "' object has no attribute '%U'",
                 type_name, name);
    __Pyx_DECREF_TypeName(type_name);
    return 0;
#else
    attr = __Pyx_PyObject_GetAttrStr(obj, name);
    goto try_unpack;
#endif
try_unpack:
#if CYTHON_UNPACK_METHODS
    if (likely(attr) && PyMethod_Check(attr) && likely(PyMethod_GET_SELF(attr) == obj)) {
        PyObject *function = PyMethod_GET_FUNCTION(attr);
        Py_INCREF(function);
        Py_DECREF(attr);
        *method = function;
        return 1;
    }
#endif
    *method = attr;
    return 0;
}
#endif

/* PyObjectCallMethod0 (used by PyType_Ready) */
static PyObject* __Pyx_PyObject_CallMethod0(PyObject* obj, PyObject* method_name) {
#if CYTHON_VECTORCALL && (__PYX_LIMITED_VERSION_HEX >= 0x030C0000 || (!CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x03090000))
    PyObject *args[1] = {obj};
    (void) __Pyx_PyObject_CallOneArg;
    (void) __Pyx_PyObject_CallNoArg;
    return PyObject_VectorcallMethod(method_name, args, 1 | PY_VECTORCALL_ARGUMENTS_OFFSET, NULL);
#else
    PyObject *method = NULL, *result = NULL;
    int is_method = __Pyx_PyObject_GetMethod(obj, method_name, &method);
    if (likely(is_method)) {
        result = __Pyx_PyObject_CallOneArg(method, obj);
        Py_DECREF(method);
        return result;
    }
    if (unlikely(!method)) goto bad;
    result = __Pyx_PyObject_CallNoArg(method);
    Py_DECREF(method);
bad:
    return result;
#endif
}

/* ValidateBasesTuple (used by PyType_Ready) */
#if CYTHON_COMPILING_IN_CPYTHON || CYTHON_COMPILING_IN_LIMITED_API || CYTHON_USE_TYPE_SPECS
static int __Pyx_validate_bases_tuple(const char *type_name, Py_ssize_t dictoffset, PyObject *bases) {
    Py_ssize_t i, n;
#if CYTHON_ASSUME_SAFE_SIZE
    n = PyTuple_GET_SIZE(bases);
#else
    n = PyTuple_Size(bases);
    if (unlikely(n < 0)) return -1;
#endif
    for (i = 1; i < n; i++)
    {
        PyTypeObject *b;
#if CYTHON_AVOID_BORROWED_REFS
        PyObject *b0 = PySequence_GetItem(bases, i);
        if (!b0) return -1;
#elif CYTHON_ASSUME_SAFE_MACROS
        PyObject *b0 = PyTuple_GET_ITEM(bases, i);
#else
        PyObject *b0 = PyTuple_GetItem(bases, i);
        if (!b0) return -1;
#endif
        b = (PyTypeObject*) b0;
        if (!__Pyx_PyType_HasFeature(b, Py_TPFLAGS_HEAPTYPE))
        {
            __Pyx_TypeName b_name = __Pyx_PyType_GetFullyQualifiedName(b);
            PyErr_Format(PyExc_TypeError,
                "base class '" __Pyx_FMT_TYPENAME "' is not a heap type", b_name);
            __Pyx_DECREF_TypeName(b_name);
#if CYTHON_AVOID_BORROWED_REFS
            Py_DECREF(b0);
#endif
            return -1;
        }
        if (dictoffset == 0)
        {
            Py_ssize_t b_dictoffset = 0;
#if CYTHON_USE_TYPE_SLOTS
            b_dictoffset = b->tp_dictoffset;
#else
            PyObject *py_b_dictoffset = PyObject_GetAttrString((PyObject*)b, "__dictoffset__");
            if (!py_b_dictoffset) goto dictoffset_return;
            b_dictoffset = PyLong_AsSsize_t(py_b_dictoffset);
            Py_DECREF(py_b_dictoffset);
            if (b_dictoffset == -1 && PyErr_Occurred()) goto dictoffset_return;
#endif
            if (b_dictoffset) {
                {
                    __Pyx_TypeName b_name = __Pyx_PyType_GetFullyQualifiedName(b);
                    PyErr_Format(PyExc_TypeError,
                        "extension type '%.200s' has no __dict__ slot, "
                        "but base type '" __Pyx_FMT_TYPENAME "' has: "
                        "either add 'cdef dict __dict__' to the extension type "
                        "or add '__slots__ = [...]' to the base type",
                        type_name, b_name);
                    __Pyx_DECREF_TypeName(b_name);
                }
#if !CYTHON_USE_TYPE_SLOTS
              dictoffset_return:
#endif
#if CYTHON_AVOID_BORROWED_REFS
                Py_DECREF(b0);
#endif
                return -1;
            }
        }
#if CYTHON_AVOID_BORROWED_REFS
        Py_DECREF(b0);
#endif
    }
    return 0;
}
#endif

/* PyType_Ready */
CYTHON_UNUSED static int __Pyx_PyType_HasMultipleInheritance(PyTypeObject *t) {
    while (t) {
        PyObject *bases = __Pyx_PyType_GetSlot(t, tp_bases, PyObject*);
        if (bases) {
            return 1;
        }
        t = __Pyx_PyType_GetSlot(t, tp_base, PyTypeObject*);
    }
    return 0;
}
static int __Pyx_PyType_Ready(PyTypeObject *t) {
#if CYTHON_USE_TYPE_SPECS || !CYTHON_COMPILING_IN_CPYTHON || defined(PYSTON_MAJOR_VERSION)
    (void)__Pyx_PyObject_CallMethod0;
#if CYTHON_USE_TYPE_SPECS
    (void)__Pyx_validate_bases_tuple;
#endif
    return PyType_Ready(t);
#else
    int r;
    if (!__Pyx_PyType_HasMultipleInheritance(t)) {
        return PyType_Ready(t);
    }
    PyObject *bases = __Pyx_PyType_GetSlot(t, tp_bases, PyObject*);
    if (bases && unlikely(__Pyx_validate_bases_tuple(t->tp_name, t->tp_dictoffset, bases) == -1))
        return -1;
#if !defined(PYSTON_MAJOR_VERSION)
    {
        int gc_was_enabled;
    #if PY_VERSION_HEX >= 0x030A00b1
        gc_was_enabled = PyGC_Disable();
        (void)__Pyx_PyObject_CallMethod0;
    #else
        PyObject *ret, *py_status;
        PyObject *gc = NULL;
        #if (!CYTHON_COMPILING_IN_PYPY || PYPY_VERSION_NUM+0 >= 0x07030400) &&\
                !CYTHON_COMPILING_IN_GRAAL
        gc = PyImport_GetModule(__pyx_mstate_global->__pyx_kp_u_gc);
        #endif
        if (unlikely(!gc)) gc = PyImport_Import(__pyx_mstate_global->__pyx_kp_u_gc);
        if (unlikely(!gc)) return -1;
        py_status = __Pyx_PyObject_CallMethod0(gc, __pyx_mstate_global->__pyx_kp_u_isenabled);
        if (unlikely(!py_status)) {
            Py_DECREF(gc);
            return -1;
        }
        gc_was_enabled = __Pyx_PyObject_IsTrue(py_status);
        Py_DECREF(py_status);
        if (gc_was_enabled > 0) {
            ret = __Pyx_PyObject_CallMethod0(gc, __pyx_mstate_global->__pyx_kp_u_disable);
            if (unlikely(!ret)) {
                Py_DECREF(gc);
                return -1;
            }
            Py_DECREF(ret);
        } else if (unlikely(gc_was_enabled == -1)) {
            Py_DECREF(gc);
            return -1;
        }
    #endif
        t->tp_flags |= Py_TPFLAGS_HEAPTYPE;
#if PY_VERSION_HEX >= 0x030A0000
        t->tp_flags |= Py_TPFLAGS_IMMUTABLETYPE;
#endif
#else
        (void)__Pyx_PyObject_CallMethod0;
#endif
    r = PyType_Ready(t);
#if !defined(PYSTON_MAJOR_VERSION)
        t->tp_flags &= ~Py_TPFLAGS_HEAPTYPE;
    #if PY_VERSION_HEX >= 0x030A00b1
        if (gc_was_enabled)
            PyGC_Enable();
    #else
        if (gc_was_enabled) {
            PyObject *tp, *v, *tb;
            PyErr_Fetch(&tp, &v, &tb);
            ret = __Pyx_PyObject_CallMethod0(gc, __pyx_mstate_global->__pyx_kp_u_enable);
            if (likely(ret || r == -1)) {
                Py_XDECREF(ret);
                PyErr_Restore(tp, v, tb);
            } else {
                Py_XDECREF(tp);
                Py_XDECREF(v);
                Py_XDECREF(tb);
                r = -1;
            }
        }
        Py_DECREF(gc);
    #endif
    }
#endif
    return r;
#endif
}

/* SetVTable */
static int __Pyx_SetVtable(PyTypeObject *type, void *vtable) {
    PyObject *ob = PyCapsule_New(vtable, 0, 0);
    if (unlikely(!ob))
        goto bad;
#if CYTHON_COMPILING_IN_LIMITED_API
    if (unlikely(PyObject_SetAttr((PyObject *) type, __pyx_mstate_global->__pyx_n_u_pyx_vtable, ob) < 0))
#else
    if (unlikely(PyDict_SetItem(type->tp_dict, __pyx_mstate_global->__pyx_n_u_pyx_vtable, ob) < 0))
#endif
        goto bad;
    Py_DECREF(ob);
    return 0;
bad:
    Py_XDECREF(ob);
    return -1;
}

/* GetVTable (used by MergeVTables) */
static void* __Pyx_GetVtable(PyTypeObject *type) {
    void* ptr;
#if CYTHON_COMPILING_IN_LIMITED_API
    PyObject *ob = PyObject_GetAttr((PyObject *)type, __pyx_mstate_global->__pyx_n_u_pyx_vtable);
#else
    PyObject *ob = PyObject_GetItem(type->tp_dict, __pyx_mstate_global->__pyx_n_u_pyx_vtable);
#endif
    if (!ob)
        goto bad;
    ptr = PyCapsule_GetPointer(ob, 0);
    if (!ptr && !PyErr_Occurred())
        PyErr_SetString(PyExc_RuntimeError, "invalid vtable found for imported type");
    Py_DECREF(ob);
    return ptr;
bad:
    Py_XDECREF(ob);
    return NULL;
}

/* MergeVTables */
static int __Pyx_MergeVtables(PyTypeObject *type) {
    int i=0;
    Py_ssize_t size;
    void** base_vtables;
    __Pyx_TypeName tp_base_name = NULL;
    __Pyx_TypeName base_name = NULL;
    void* unknown = (void*)-1;
    PyObject* bases = __Pyx_PyType_GetSlot(type, tp_bases, PyObject*);
    int base_depth = 0;
    {
        PyTypeObject* base = __Pyx_PyType_GetSlot(type, tp_base, PyTypeObject*);
        while (base) {
            base_depth += 1;
            base = __Pyx_PyType_GetSlot(base, tp_base, PyTypeObject*);
        }
    }
    base_vtables = (void**) PyMem_Malloc(sizeof(void*) * (size_t)(base_depth + 1));
    base_vtables[0] = unknown;
#if CYTHON_COMPILING_IN_LIMITED_API
    size = PyTuple_Size(bases);
    if (size < 0) goto other_failure;
#else
    size = PyTuple_GET_SIZE(bases);
#endif
    for (i = 1; i < size; i++) {
        PyObject *basei;
        void* base_vtable;
#if CYTHON_AVOID_BORROWED_REFS
        basei = PySequence_GetItem(bases, i);
        if (unlikely(!basei)) goto other_failure;
#elif !CYTHON_ASSUME_SAFE_MACROS
        basei = PyTuple_GetItem(bases, i);
        if (unlikely(!basei)) goto other_failure;
#else
        basei = PyTuple_GET_ITEM(bases, i);
#endif
        base_vtable = __Pyx_GetVtable((PyTypeObject*)basei);
#if CYTHON_AVOID_BORROWED_REFS
        Py_DECREF(basei);
#endif
        if (base_vtable != NULL) {
            int j;
            PyTypeObject* base = __Pyx_PyType_GetSlot(type, tp_base, PyTypeObject*);
            for (j = 0; j < base_depth; j++) {
                if (base_vtables[j] == unknown) {
                    base_vtables[j] = __Pyx_GetVtable(base);
                    base_vtables[j + 1] = unknown;
                }
                if (base_vtables[j] == base_vtable) {
                    break;
                } else if (base_vtables[j] == NULL) {
                    goto bad;
                }
                base = __Pyx_PyType_GetSlot(base, tp_base, PyTypeObject*);
            }
        }
    }
    PyErr_Clear();
    PyMem_Free(base_vtables);
    return 0;
bad:
    {
        PyTypeObject* basei = NULL;
        PyTypeObject* tp_base = __Pyx_PyType_GetSlot(type, tp_base, PyTypeObject*);
        tp_base_name = __Pyx_PyType_GetFullyQualifiedName(tp_base);
#if CYTHON_AVOID_BORROWED_REFS
        basei = (PyTypeObject*)PySequence_GetItem(bases, i);
        if (unlikely(!basei)) goto really_bad;
#elif !CYTHON_ASSUME_SAFE_MACROS
        basei = (PyTypeObject*)PyTuple_GetItem(bases, i);
        if (unlikely(!basei)) goto really_bad;
#else
        basei = (PyTypeObject*)PyTuple_GET_ITEM(bases, i);
#endif
        base_name = __Pyx_PyType_GetFullyQualifiedName(basei);
#if CYTHON_AVOID_BORROWED_REFS
        Py_DECREF(basei);
#endif
    }
    PyErr_Format(PyExc_TypeError,
        "multiple bases have vtable conflict: '" __Pyx_FMT_TYPENAME "' and '" __Pyx_FMT_TYPENAME "'", tp_base_name, base_name);
#if CYTHON_AVOID_BORROWED_REFS || !CYTHON_ASSUME_SAFE_MACROS
really_bad: // bad has failed!
#endif
    __Pyx_DECREF_TypeName(tp_base_name);
    __Pyx_DECREF_TypeName(base_name);
#if CYTHON_COMPILING_IN_LIMITED_API || CYTHON_AVOID_BORROWED_REFS || !CYTHON_ASSUME_SAFE_MACROS
other_failure:
#endif
    PyMem_Free(base_vtables);
    return -1;
}

/* DelItemOnTypeDict (used by SetupReduce) */
static int __Pyx__DelItemOnTypeDict(PyTypeObject *tp, PyObject *k) {
    int result;
    PyObject *tp_dict;
#if CYTHON_COMPILING_IN_LIMITED_API
    tp_dict = __Pyx_GetTypeDict(tp);
    if (unlikely(!tp_dict)) return -1;
#else
    tp_dict = tp->tp_dict;
#endif
    result = PyDict_DelItem(tp_dict, k);
    if (likely(!result)) PyType_Modified(tp);
    return result;
}

/* SetupReduce */
static int __Pyx_setup_reduce_is_named(PyObject* meth, PyObject* name) {
  int ret;
  PyObject *name_attr;
  name_attr = __Pyx_PyObject_GetAttrStrNoError(meth, __pyx_mstate_global->__pyx_n_u_name_2);
  if (likely(name_attr)) {
      ret = PyObject_RichCompareBool(name_attr, name, Py_EQ);
  } else {
      ret = -1;
  }
  if (unlikely(ret < 0)) {
      PyErr_Clear();
      ret = 0;
  }
  Py_XDECREF(name_attr);
  return ret;
}
static int __Pyx_setup_reduce(PyObject* type_obj) {
    int ret = 0;
    PyObject *object_reduce = NULL;
    PyObject *object_getstate = NULL;
    PyObject *object_reduce_ex = NULL;
    PyObject *reduce = NULL;
    PyObject *reduce_ex = NULL;
    PyObject *reduce_cython = NULL;
    PyObject *setstate = NULL;
    PyObject *setstate_cython = NULL;
    PyObject *getstate = NULL;
#if CYTHON_USE_PYTYPE_LOOKUP
    getstate = _PyType_Lookup((PyTypeObject*)type_obj, __pyx_mstate_global->__pyx_n_u_getstate);
#else
    getstate = __Pyx_PyObject_GetAttrStrNoError(type_obj, __pyx_mstate_global->__pyx_n_u_getstate);
    if (!getstate && PyErr_Occurred()) {
        goto __PYX_BAD;
    }
#endif
    if (getstate) {
#if CYTHON_USE_PYTYPE_LOOKUP
        object_getstate = _PyType_Lookup(&PyBaseObject_Type, __pyx_mstate_global->__pyx_n_u_getstate);
#else
        object_getstate = __Pyx_PyObject_GetAttrStrNoError((PyObject*)&PyBaseObject_Type, __pyx_mstate_global->__pyx_n_u_getstate);
        if (!object_getstate && PyErr_Occurred()) {
            goto __PYX_BAD;
        }
#endif
        if (object_getstate != getstate) {
            goto __PYX_GOOD;
        }
    }
#if CYTHON_USE_PYTYPE_LOOKUP
    object_reduce_ex = _PyType_Lookup(&PyBaseObject_Type, __pyx_mstate_global->__pyx_n_u_reduce_ex); if (!object_reduce_ex) goto __PYX_BAD;
#else
    object_reduce_ex = __Pyx_PyObject_GetAttrStr((PyObject*)&PyBaseObject_Type, __pyx_mstate_global->__pyx_n_u_reduce_ex); if (!object_reduce_ex) goto __PYX_BAD;
#endif
    reduce_ex = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_mstate_global->__pyx_n_u_reduce_ex); if (unlikely(!reduce_ex)) goto __PYX_BAD;
    if (reduce_ex == object_reduce_ex) {
#if CYTHON_USE_PYTYPE_LOOKUP
        object_reduce = _PyType_Lookup(&PyBaseObject_Type, __pyx_mstate_global->__pyx_n_u_reduce); if (!object_reduce) goto __PYX_BAD;
#else
        object_reduce = __Pyx_PyObject_GetAttrStr((PyObject*)&PyBaseObject_Type, __pyx_mstate_global->__pyx_n_u_reduce); if (!object_reduce) goto __PYX_BAD;
#endif
        reduce = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_mstate_global->__pyx_n_u_reduce); if (unlikely(!reduce)) goto __PYX_BAD;
        if (reduce == object_reduce || __Pyx_setup_reduce_is_named(reduce, __pyx_mstate_global->__pyx_n_u_reduce_cython)) {
            reduce_cython = __Pyx_PyObject_GetAttrStrNoError(type_obj, __pyx_mstate_global->__pyx_n_u_reduce_cython);
            if (likely(reduce_cython)) {
                ret = __Pyx_SetItemOnTypeDict((PyTypeObject*)type_obj, __pyx_mstate_global->__pyx_n_u_reduce, reduce_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
                ret = __Pyx_DelItemOnTypeDict((PyTypeObject*)type_obj, __pyx_mstate_global->__pyx_n_u_reduce_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
            } else if (reduce == object_reduce || PyErr_Occurred()) {
                goto __PYX_BAD;
            }
            setstate = __Pyx_PyObject_GetAttrStrNoError(type_obj, __pyx_mstate_global->__pyx_n_u_setstate);
            if (!setstate) PyErr_Clear();
            if (!setstate || __Pyx_setup_reduce_is_named(setstate, __pyx_mstate_global->__pyx_n_u_setstate_cython)) {
                setstate_cython = __Pyx_PyObject_GetAttrStrNoError(type_obj, __pyx_mstate_global->__pyx_n_u_setstate_cython);
                if (likely(setstate_cython)) {
                    ret = __Pyx_SetItemOnTypeDict((PyTypeObject*)type_obj, __pyx_mstate_global->__pyx_n_u_setstate, setstate_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
                    ret = __Pyx_DelItemOnTypeDict((PyTypeObject*)type_obj, __pyx_mstate_global->__pyx_n_u_setstate_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
                } else if (!setstate || PyErr_Occurred()) {
                    goto __PYX_BAD;
                }
            }
            PyType_Modified((PyTypeObject*)type_obj);
        }
    }
    goto __PYX_GOOD;
__PYX_BAD:
    if (!PyErr_Occurred()) {
        __Pyx_TypeName type_obj_name =
            __Pyx_PyType_GetFullyQualifiedName((PyTypeObject*)type_obj);
        PyErr_Format(PyExc_RuntimeError,
            "Unable to initialize pickling for " __Pyx_FMT_TYPENAME, type_obj_name);
        __Pyx_DECREF_TypeName(type_obj_name);
    }
    ret = -1;
__PYX_GOOD:
#if !CYTHON_USE_PYTYPE_LOOKUP
    Py_XDECREF(object_reduce);
    Py_XDECREF(object_reduce_ex);
    Py_XDECREF(object_getstate);
    Py_XDECREF(getstate);
#endif
    Py_XDECREF(reduce);
    Py_XDECREF(reduce_ex);
    Py_XDECREF(reduce_cython);
    Py_XDECREF(setstate);
    Py_XDECREF(setstate_cython);
    return ret;
}

/* TypeImport */
#ifndef __PYX_HAVE_RT_ImportType_3_2_3
#define __PYX_HAVE_RT_ImportType_3_2_3
static PyTypeObject *__Pyx_ImportType_3_2_3(PyObject *module, const char *module_name, const char *class_name,
    size_t size, size_t alignment, enum __Pyx_ImportType_CheckSize_3_2_3 check_size)
{
    PyObject *result = 0;
    Py_ssize_t basicsize;
    Py_ssize_t itemsize;
#if defined(Py_LIMITED_API) || (defined(CYTHON_COMPILING_IN_LIMITED_API) && CYTHON_COMPILING_IN_LIMITED_API)
    PyObject *py_basicsize;
    PyObject *py_itemsize;
#endif
    result = PyObject_GetAttrString(module, class_name);
    if (!result)
        goto bad;
    if (!PyType_Check(result)) {
        PyErr_Format(PyExc_TypeError,
            "%.200s.%.200s is not a type object",
            module_name, class_name);
        goto bad;
    }
#if !( defined(Py_LIMITED_API) || (defined(CYTHON_COMPILING_IN_LIMITED_API) && CYTHON_COMPILING_IN_LIMITED_API) )
    basicsize = ((PyTypeObject *)result)->tp_basicsize;
    itemsize = ((PyTypeObject *)result)->tp_itemsize;
#else
    if (size == 0) {
        return (PyTypeObject *)result;
    }
    py_basicsize = PyObject_GetAttrString(result, "__basicsize__");
    if (!py_basicsize)
        goto bad;
    basicsize = PyLong_AsSsize_t(py_basicsize);
    Py_DECREF(py_basicsize);
    py_basicsize = 0;
    if (basicsize == (Py_ssize_t)-1 && PyErr_Occurred())
        goto bad;
    py_itemsize = PyObject_GetAttrString(result, "__itemsize__");
    if (!py_itemsize)
        goto bad;
    itemsize = PyLong_AsSsize_t(py_itemsize);
    Py_DECREF(py_itemsize);
    py_itemsize = 0;
    if (itemsize == (Py_ssize_t)-1 && PyErr_Occurred())
        goto bad;
#endif
    if (itemsize) {
        if (size % alignment) {
            alignment = size % alignment;
        }
        if (itemsize < (Py_ssize_t)alignment)
            itemsize = (Py_ssize_t)alignment;
    }
    if ((size_t)(basicsize + itemsize) < size) {
        PyErr_Format(PyExc_ValueError,
            "%.200s.%.200s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd from PyObject",
            module_name, class_name, size, basicsize+itemsize);
        goto bad;
    }
    if (check_size == __Pyx_ImportType_CheckSize_Error_3_2_3 &&
            ((size_t)basicsize > size || (size_t)(basicsize + itemsize) < size)) {
        PyErr_Format(PyExc_ValueError,
            "%.200s.%.200s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd-%zd from PyObject",
            module_name, class_name, size, basicsize, basicsize+itemsize);
        goto bad;
    }
    else if (check_size == __Pyx_ImportType_CheckSize_Warn_3_2_3 && (size_t)basicsize > size) {
        if (PyErr_WarnFormat(NULL, 0,
                "%.200s.%.200s size changed, may indicate binary incompatibility. "
                "Expected %zd from C header, got %zd from PyObject",
                module_name, class_name, size, basicsize) < 0) {
            goto bad;
        }
    }
    return (PyTypeObject *)result;
bad:
    Py_XDECREF(result);
    return NULL;
}
#endif

/* HasAttr (used by ImportImpl) */
#if __PYX_LIMITED_VERSION_HEX < 0x030d0000
static CYTHON_INLINE int __Pyx_HasAttr(PyObject *o, PyObject *n) {
    PyObject *r;
    if (unlikely(!PyUnicode_Check(n))) {
        PyErr_SetString(PyExc_TypeError,
                        "hasattr(): attribute name must be string");
        return -1;
    }
    r = __Pyx_PyObject_GetAttrStrNoError(o, n);
    if (!r) {
        return (unlikely(PyErr_Occurred())) ? -1 : 0;
    } else {
        Py_DECREF(r);
        return 1;
    }
}
#endif

/* ImportImpl (used by Import) */
static int __Pyx__Import_GetModule(PyObject *qualname, PyObject **module) {
    PyObject *imported_module = PyImport_GetModule(qualname);
    if (unlikely(!imported_module)) {
        *module = NULL;
        if (PyErr_Occurred()) {
            return -1;
        }
        return 0;
    }
    *module = imported_module;
    return 1;
}
static int __Pyx__Import_Lookup(PyObject *qualname, PyObject *const *imported_names, Py_ssize_t len_imported_names, PyObject **module) {
    PyObject *imported_module;
    PyObject *top_level_package_name;
    Py_ssize_t i;
    int status, module_found;
    Py_ssize_t dot_index;
    module_found = __Pyx__Import_GetModule(qualname, &imported_module);
    if (unlikely(!module_found || module_found == -1)) {
        *module = NULL;
        return module_found;
    }
    if (imported_names) {
        for (i = 0; i < len_imported_names; i++) {
            PyObject *imported_name = imported_names[i];
#if __PYX_LIMITED_VERSION_HEX < 0x030d0000
            int has_imported_attribute = PyObject_HasAttr(imported_module, imported_name);
#else
            int has_imported_attribute = PyObject_HasAttrWithError(imported_module, imported_name);
            if (unlikely(has_imported_attribute == -1)) goto error;
#endif
            if (!has_imported_attribute) {
                goto not_found;
            }
        }
        *module = imported_module;
        return 1;
    }
    dot_index = PyUnicode_FindChar(qualname, '.', 0, PY_SSIZE_T_MAX, 1);
    if (dot_index == -1) {
        *module = imported_module;
        return 1;
    }
    if (unlikely(dot_index == -2)) goto error;
    top_level_package_name = PyUnicode_Substring(qualname, 0, dot_index);
    if (unlikely(!top_level_package_name)) goto error;
    Py_DECREF(imported_module);
    status = __Pyx__Import_GetModule(top_level_package_name, module);
    Py_DECREF(top_level_package_name);
    return status;
error:
    Py_DECREF(imported_module);
    *module = NULL;
    return -1;
not_found:
    Py_DECREF(imported_module);
    *module = NULL;
    return 0;
}
static PyObject *__Pyx__Import(PyObject *name, PyObject *const *imported_names, Py_ssize_t len_imported_names, PyObject *qualname, PyObject *moddict, int level) {
    PyObject *module = 0;
    PyObject *empty_dict = 0;
    PyObject *from_list = 0;
    int module_found;
    if (!qualname) {
        qualname = name;
    }
    module_found = __Pyx__Import_Lookup(qualname, imported_names, len_imported_names, &module);
    if (likely(module_found == 1)) {
        return module;
    } else if (unlikely(module_found == -1)) {
        return NULL;
    }
    empty_dict = PyDict_New();
    if (unlikely(!empty_dict))
        goto bad;
    if (imported_names) {
#if CYTHON_COMPILING_IN_CPYTHON
        from_list = __Pyx_PyList_FromArray(imported_names, len_imported_names);
        if (unlikely(!from_list))
            goto bad;
#else
        from_list = PyList_New(len_imported_names);
        if (unlikely(!from_list)) goto bad;
        for (Py_ssize_t i=0; i<len_imported_names; ++i) {
            if (PyList_SetItem(from_list, i, __Pyx_NewRef(imported_names[i])) < 0) goto bad;
        }
#endif
    }
    if (level == -1) {
        const char* package_sep = strchr(__Pyx_MODULE_NAME, '.');
        if (package_sep != (0)) {
            module = PyImport_ImportModuleLevelObject(
                name, moddict, empty_dict, from_list, 1);
            if (unlikely(!module)) {
                if (unlikely(!PyErr_ExceptionMatches(PyExc_ImportError)))
                    goto bad;
                PyErr_Clear();
            }
        }
        level = 0;
    }
    if (!module) {
        module = PyImport_ImportModuleLevelObject(
            name, moddict, empty_dict, from_list, level);
    }
bad:
    Py_XDECREF(from_list);
    Py_XDECREF(empty_dict);
    return module;
}

/* Import */
static PyObject *__Pyx_Import(PyObject *name, PyObject *const *imported_names, Py_ssize_t len_imported_names, PyObject *qualname, int level) {
    return __Pyx__Import(name, imported_names, len_imported_names, qualname, __pyx_mstate_global->__pyx_d, level);
}

/* ImportFrom */
static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name) {
    PyObject* value = __Pyx_PyObject_GetAttrStr(module, name);
    if (unlikely(!value) && PyErr_ExceptionMatches(PyExc_AttributeError)) {
        const char* module_name_str = 0;
        PyObject* module_name = 0;
        PyObject* module_dot = 0;
        PyObject* full_name = 0;
        PyErr_Clear();
        module_name_str = PyModule_GetName(module);
        if (unlikely(!module_name_str)) { goto modbad; }
        module_name = PyUnicode_FromString(module_name_str);
        if (unlikely(!module_name)) { goto modbad; }
        module_dot = PyUnicode_Concat(module_name, __pyx_mstate_global->__pyx_kp_u__3);
        if (unlikely(!module_dot)) { goto modbad; }
        full_name = PyUnicode_Concat(module_dot, name);
        if (unlikely(!full_name)) { goto modbad; }
        #if (CYTHON_COMPILING_IN_PYPY && PYPY_VERSION_NUM  < 0x07030400) ||\
                CYTHON_COMPILING_IN_GRAAL
        {
            PyObject *modules = PyImport_GetModuleDict();
            if (unlikely(!modules))
                goto modbad;
            value = PyObject_GetItem(modules, full_name);
        }
        #else
        value = PyImport_GetModule(full_name);
        #endif
      modbad:
        Py_XDECREF(full_name);
        Py_XDECREF(module_dot);
        Py_XDECREF(module_name);
    }
    if (unlikely(!value)) {
        PyErr_Format(PyExc_ImportError, "cannot import name %S", name);
    }
    return value;
}

/* CIntToDigits (used by CIntToPyUnicode) */
static const char DIGIT_PAIRS_10[2*10*10+1] = {
    "00010203040506070809"
    "10111213141516171819"
    "20212223242526272829"
    "30313233343536373839"
    "40414243444546474849"
    "50515253545556575859"
    "60616263646566676869"
    "70717273747576777879"
    "80818283848586878889"
    "90919293949596979899"
};
static const char DIGIT_PAIRS_8[2*8*8+1] = {
    "0001020304050607"
    "1011121314151617"
    "2021222324252627"
    "3031323334353637"
    "4041424344454647"
    "5051525354555657"
    "6061626364656667"
    "7071727374757677"
};
static const char DIGITS_HEX[2*16+1] = {
    "0123456789abcdef"
    "0123456789ABCDEF"
};

/* BuildPyUnicode (used by COrdinalToPyUnicode) */
static PyObject* __Pyx_PyUnicode_BuildFromAscii(Py_ssize_t ulength, const char* chars, int clength,
                                                int prepend_sign, char padding_char) {
    PyObject *uval;
    Py_ssize_t uoffset = ulength - clength;
#if CYTHON_USE_UNICODE_INTERNALS
    Py_ssize_t i;
    void *udata;
    uval = PyUnicode_New(ulength, 127);
    if (unlikely(!uval)) return NULL;
    udata = PyUnicode_DATA(uval);
    if (uoffset > 0) {
        i = 0;
        if (prepend_sign) {
            __Pyx_PyUnicode_WRITE(PyUnicode_1BYTE_KIND, udata, 0, '-');
            i++;
        }
        for (; i < uoffset; i++) {
            __Pyx_PyUnicode_WRITE(PyUnicode_1BYTE_KIND, udata, i, padding_char);
        }
    }
    for (i=0; i < clength; i++) {
        __Pyx_PyUnicode_WRITE(PyUnicode_1BYTE_KIND, udata, uoffset+i, chars[i]);
    }
#else
    {
        PyObject *sign = NULL, *padding = NULL;
        uval = NULL;
        if (uoffset > 0) {
            prepend_sign = !!prepend_sign;
            if (uoffset > prepend_sign) {
                padding = PyUnicode_FromOrdinal(padding_char);
                if (likely(padding) && uoffset > prepend_sign + 1) {
                    PyObject *tmp = PySequence_Repeat(padding, uoffset - prepend_sign);
                    Py_DECREF(padding);
                    padding = tmp;
                }
                if (unlikely(!padding)) goto done_or_error;
            }
            if (prepend_sign) {
                sign = PyUnicode_FromOrdinal('-');
                if (unlikely(!sign)) goto done_or_error;
            }
        }
        uval = PyUnicode_DecodeASCII(chars, clength, NULL);
        if (likely(uval) && padding) {
            PyObject *tmp = PyUnicode_Concat(padding, uval);
            Py_DECREF(uval);
            uval = tmp;
        }
        if (likely(uval) && sign) {
            PyObject *tmp = PyUnicode_Concat(sign, uval);
            Py_DECREF(uval);
            uval = tmp;
        }
done_or_error:
        Py_XDECREF(padding);
        Py_XDECREF(sign);
    }
#endif
    return uval;
}

/* COrdinalToPyUnicode (used by CIntToPyUnicode) */
static CYTHON_INLINE int __Pyx_CheckUnicodeValue(int value) {
    return value <= 1114111;
}
static PyObject* __Pyx_PyUnicode_FromOrdinal_Padded(int value, Py_ssize_t ulength, char padding_char) {
    Py_ssize_t padding_length = ulength - 1;
    if (likely((padding_length <= 250) && (value < 0xD800 || value > 0xDFFF))) {
        char chars[256];
        if (value <= 255) {
            memset(chars, padding_char, (size_t) padding_length);
            chars[ulength-1] = (char) value;
            return PyUnicode_DecodeLatin1(chars, ulength, NULL);
        }
        char *cpos = chars + sizeof(chars);
        if (value < 0x800) {
            *--cpos = (char) (0x80 | (value & 0x3f));
            value >>= 6;
            *--cpos = (char) (0xc0 | (value & 0x1f));
        } else if (value < 0x10000) {
            *--cpos = (char) (0x80 | (value & 0x3f));
            value >>= 6;
            *--cpos = (char) (0x80 | (value & 0x3f));
            value >>= 6;
            *--cpos = (char) (0xe0 | (value & 0x0f));
        } else {
            *--cpos = (char) (0x80 | (value & 0x3f));
            value >>= 6;
            *--cpos = (char) (0x80 | (value & 0x3f));
            value >>= 6;
            *--cpos = (char) (0x80 | (value & 0x3f));
            value >>= 6;
            *--cpos = (char) (0xf0 | (value & 0x07));
        }
        cpos -= padding_length;
        memset(cpos, padding_char, (size_t) padding_length);
        return PyUnicode_DecodeUTF8(cpos, chars + sizeof(chars) - cpos, NULL);
    }
    if (value <= 127 && CYTHON_USE_UNICODE_INTERNALS) {
        const char chars[1] = {(char) value};
        return __Pyx_PyUnicode_BuildFromAscii(ulength, chars, 1, 0, padding_char);
    }
    {
        PyObject *uchar, *padding_uchar, *padding, *result;
        padding_uchar = PyUnicode_FromOrdinal(padding_char);
        if (unlikely(!padding_uchar)) return NULL;
        padding = PySequence_Repeat(padding_uchar, padding_length);
        Py_DECREF(padding_uchar);
        if (unlikely(!padding)) return NULL;
        uchar = PyUnicode_FromOrdinal(value);
        if (unlikely(!uchar)) {
            Py_DECREF(padding);
            return NULL;
        }
        result = PyUnicode_Concat(padding, uchar);
        Py_DECREF(padding);
        Py_DECREF(uchar);
        return result;
    }
}

/* CIntToPyUnicode */
static CYTHON_INLINE PyObject* __Pyx_uchar___Pyx_PyUnicode_From_int(int value, Py_ssize_t width, char padding_char) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const int neg_one = (int) -1, const_zero = (int) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (unlikely(!(is_unsigned || value == 0 || value > 0) ||
                    !(sizeof(value) <= 2 || value & ~ (int) 0x01fffff || __Pyx_CheckUnicodeValue((int) value)))) {
        PyErr_SetString(PyExc_OverflowError, "%c arg not in range(0x110000)");
        return NULL;
    }
    if (width <= 1) {
        return PyUnicode_FromOrdinal((int) value);
    }
    return __Pyx_PyUnicode_FromOrdinal_Padded((int) value, width, padding_char);
}
static CYTHON_INLINE PyObject* __Pyx____Pyx_PyUnicode_From_int(int value, Py_ssize_t width, char padding_char, char format_char) {
    char digits[sizeof(int)*3+2];
    char *dpos, *end = digits + sizeof(int)*3+2;
    const char *hex_digits = DIGITS_HEX;
    Py_ssize_t length, ulength;
    int prepend_sign, last_one_off;
    int remaining;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const int neg_one = (int) -1, const_zero = (int) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (format_char == 'X') {
        hex_digits += 16;
        format_char = 'x';
    }
    remaining = value;
    last_one_off = 0;
    dpos = end;
    do {
        int digit_pos;
        switch (format_char) {
        case 'o':
            digit_pos = abs((int)(remaining % (8*8)));
            remaining = (int) (remaining / (8*8));
            dpos -= 2;
            memcpy(dpos, DIGIT_PAIRS_8 + digit_pos * 2, 2);
            last_one_off = (digit_pos < 8);
            break;
        case 'd':
            digit_pos = abs((int)(remaining % (10*10)));
            remaining = (int) (remaining / (10*10));
            dpos -= 2;
            memcpy(dpos, DIGIT_PAIRS_10 + digit_pos * 2, 2);
            last_one_off = (digit_pos < 10);
            break;
        case 'x':
            *(--dpos) = hex_digits[abs((int)(remaining % 16))];
            remaining = (int) (remaining / 16);
            break;
        default:
            assert(0);
            break;
        }
    } while (unlikely(remaining != 0));
    assert(!last_one_off || *dpos == '0');
    dpos += last_one_off;
    length = end - dpos;
    ulength = length;
    prepend_sign = 0;
    if (!is_unsigned && value <= neg_one) {
        if (padding_char == ' ' || width <= length + 1) {
            *(--dpos) = '-';
            ++length;
        } else {
            prepend_sign = 1;
        }
        ++ulength;
    }
    if (width > ulength) {
        ulength = width;
    }
    if (ulength == 1) {
        return PyUnicode_FromOrdinal(*dpos);
    }
    return __Pyx_PyUnicode_BuildFromAscii(ulength, dpos, (int) length, prepend_sign, padding_char);
}

/* Py3UpdateBases */
static PyObject*
__Pyx_PEP560_update_bases(PyObject *bases)
{
    Py_ssize_t i, j, size_bases;
    PyObject *base = NULL, *meth, *new_base, *result, *new_bases = NULL;
#if CYTHON_ASSUME_SAFE_SIZE
    size_bases = PyTuple_GET_SIZE(bases);
#else
    size_bases = PyTuple_Size(bases);
    if (size_bases < 0) return NULL;
#endif
    for (i = 0; i < size_bases; i++) {
#if CYTHON_AVOID_BORROWED_REFS
        Py_CLEAR(base);
#endif
#if CYTHON_ASSUME_SAFE_MACROS
        base = PyTuple_GET_ITEM(bases, i);
#else
        base = PyTuple_GetItem(bases, i);
        if (!base) goto error;
#endif
#if CYTHON_AVOID_BORROWED_REFS
        Py_INCREF(base);
#endif
        if (PyType_Check(base)) {
            if (new_bases) {
                if (PyList_Append(new_bases, base) < 0) {
                    goto error;
                }
            }
            continue;
        }
        meth = __Pyx_PyObject_GetAttrStrNoError(base, __pyx_mstate_global->__pyx_n_u_mro_entries);
        if (!meth && PyErr_Occurred()) {
            goto error;
        }
        if (!meth) {
            if (new_bases) {
                if (PyList_Append(new_bases, base) < 0) {
                    goto error;
                }
            }
            continue;
        }
        new_base = __Pyx_PyObject_CallOneArg(meth, bases);
        Py_DECREF(meth);
        if (!new_base) {
            goto error;
        }
        if (!PyTuple_Check(new_base)) {
            PyErr_SetString(PyExc_TypeError,
                            "__mro_entries__ must return a tuple");
            Py_DECREF(new_base);
            goto error;
        }
        if (!new_bases) {
            if (!(new_bases = PyList_New(i))) {
                goto error;
            }
            for (j = 0; j < i; j++) {
                PyObject *base_from_list;
#if CYTHON_ASSUME_SAFE_MACROS
                base_from_list = PyTuple_GET_ITEM(bases, j);
                PyList_SET_ITEM(new_bases, j, base_from_list);
                Py_INCREF(base_from_list);
#else
                base_from_list = PyTuple_GetItem(bases, j);
                if (!base_from_list) goto error;
                Py_INCREF(base_from_list);
                if (PyList_SetItem(new_bases, j, base_from_list) < 0) goto error;
#endif
            }
        }
#if CYTHON_ASSUME_SAFE_SIZE
        j = PyList_GET_SIZE(new_bases);
#else
        j = PyList_Size(new_bases);
        if (j < 0) goto error;
#endif
        if (PyList_SetSlice(new_bases, j, j, new_base) < 0) {
            goto error;
        }
        Py_DECREF(new_base);
    }
    if (!new_bases) {
        Py_INCREF(bases);
        return bases;
    }
    result = PyList_AsTuple(new_bases);
    Py_DECREF(new_bases);
#if CYTHON_AVOID_BORROWED_REFS
    Py_XDECREF(base);
#endif
    return result;
error:
    Py_XDECREF(new_bases);
#if CYTHON_AVOID_BORROWED_REFS
    Py_XDECREF(base);
#endif
    return NULL;
}

/* CalculateMetaclass */
static PyObject *__Pyx_CalculateMetaclass(PyTypeObject *metaclass, PyObject *bases) {
    Py_ssize_t i, nbases;
#if CYTHON_ASSUME_SAFE_SIZE
    nbases = PyTuple_GET_SIZE(bases);
#else
    nbases = PyTuple_Size(bases);
    if (nbases < 0) return NULL;
#endif
    for (i=0; i < nbases; i++) {
        PyTypeObject *tmptype;
#if CYTHON_ASSUME_SAFE_MACROS
        PyObject *tmp = PyTuple_GET_ITEM(bases, i);
#else
        PyObject *tmp = PyTuple_GetItem(bases, i);
        if (!tmp) return NULL;
#endif
        tmptype = Py_TYPE(tmp);
        if (!metaclass) {
            metaclass = tmptype;
            continue;
        }
        if (PyType_IsSubtype(metaclass, tmptype))
            continue;
        if (PyType_IsSubtype(tmptype, metaclass)) {
            metaclass = tmptype;
            continue;
        }
        PyErr_SetString(PyExc_TypeError,
                        "metaclass conflict: "
                        "the metaclass of a derived class "
                        "must be a (non-strict) subclass "
                        "of the metaclasses of all its bases");
        return NULL;
    }
    if (!metaclass) {
        metaclass = &PyType_Type;
    }
    Py_INCREF((PyObject*) metaclass);
    return (PyObject*) metaclass;
}

/* Py3ClassCreate */
static PyObject *__Pyx_Py3MetaclassPrepare(PyObject *metaclass, PyObject *bases, PyObject *name,
                                           PyObject *qualname, PyObject *mkw, PyObject *modname, PyObject *doc) {
    PyObject *ns;
    if (metaclass) {
        PyObject *prep = __Pyx_PyObject_GetAttrStrNoError(metaclass, __pyx_mstate_global->__pyx_n_u_prepare);
        if (prep) {
            PyObject *pargs[3] = {NULL, name, bases};
            ns = __Pyx_PyObject_FastCallDict(prep, pargs+1, 2 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET, mkw);
            Py_DECREF(prep);
        } else {
            if (unlikely(PyErr_Occurred()))
                return NULL;
            ns = PyDict_New();
        }
    } else {
        ns = PyDict_New();
    }
    if (unlikely(!ns))
        return NULL;
    if (unlikely(PyObject_SetItem(ns, __pyx_mstate_global->__pyx_n_u_module, modname) < 0)) goto bad;
    if (unlikely(PyObject_SetItem(ns, __pyx_mstate_global->__pyx_n_u_qualname, qualname) < 0)) goto bad;
    if (unlikely(doc && PyObject_SetItem(ns, __pyx_mstate_global->__pyx_n_u_doc, doc) < 0)) goto bad;
    return ns;
bad:
    Py_DECREF(ns);
    return NULL;
}
static PyObject *__Pyx_Py3ClassCreate(PyObject *metaclass, PyObject *name, PyObject *bases,
                                      PyObject *dict, PyObject *mkw,
                                      int calculate_metaclass, int allow_py2_metaclass) {
    PyObject *result;
    PyObject *owned_metaclass = NULL;
    PyObject *margs[4] = {NULL, name, bases, dict};
    if (allow_py2_metaclass) {
        owned_metaclass = PyObject_GetItem(dict, __pyx_mstate_global->__pyx_n_u_metaclass);
        if (owned_metaclass) {
            metaclass = owned_metaclass;
        } else if (likely(PyErr_ExceptionMatches(PyExc_KeyError))) {
            PyErr_Clear();
        } else {
            return NULL;
        }
    }
    if (calculate_metaclass && (!metaclass || PyType_Check(metaclass))) {
        metaclass = __Pyx_CalculateMetaclass((PyTypeObject*) metaclass, bases);
        Py_XDECREF(owned_metaclass);
        if (unlikely(!metaclass))
            return NULL;
        owned_metaclass = metaclass;
    }
    result = __Pyx_PyObject_FastCallDict(metaclass, margs+1, 3 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET, mkw);
    Py_XDECREF(owned_metaclass);
    return result;
}

/* CLineInTraceback (used by AddTraceback) */
#if CYTHON_CLINE_IN_TRACEBACK && CYTHON_CLINE_IN_TRACEBACK_RUNTIME
#if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030A0000
#define __Pyx_PyProbablyModule_GetDict(o) __Pyx_XNewRef(PyModule_GetDict(o))
#elif !CYTHON_COMPILING_IN_CPYTHON || CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
#define __Pyx_PyProbablyModule_GetDict(o) PyObject_GenericGetDict(o, NULL);
#else
PyObject* __Pyx_PyProbablyModule_GetDict(PyObject *o) {
    PyObject **dict_ptr = _PyObject_GetDictPtr(o);
    return dict_ptr ? __Pyx_XNewRef(*dict_ptr) : NULL;
}
#endif
static int __Pyx_CLineForTraceback(PyThreadState *tstate, int c_line) {
    PyObject *use_cline = NULL;
    PyObject *ptype, *pvalue, *ptraceback;
    PyObject *cython_runtime_dict;
    CYTHON_MAYBE_UNUSED_VAR(tstate);
    if (unlikely(!__pyx_mstate_global->__pyx_cython_runtime)) {
        return c_line;
    }
    __Pyx_ErrFetchInState(tstate, &ptype, &pvalue, &ptraceback);
    cython_runtime_dict = __Pyx_PyProbablyModule_GetDict(__pyx_mstate_global->__pyx_cython_runtime);
    if (likely(cython_runtime_dict)) {
        __PYX_PY_DICT_LOOKUP_IF_MODIFIED(
            use_cline, cython_runtime_dict,
            __Pyx_PyDict_SetDefault(cython_runtime_dict, __pyx_mstate_global->__pyx_n_u_cline_in_traceback, Py_False))
    }
    if (use_cline == NULL || use_cline == Py_False || (use_cline != Py_True && PyObject_Not(use_cline) != 0)) {
        c_line = 0;
    }
    Py_XDECREF(use_cline);
    Py_XDECREF(cython_runtime_dict);
    __Pyx_ErrRestoreInState(tstate, ptype, pvalue, ptraceback);
    return c_line;
}
#endif

/* CodeObjectCache (used by AddTraceback) */
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line) {
    int start = 0, mid = 0, end = count - 1;
    if (end >= 0 && code_line > entries[end].code_line) {
        return count;
    }
    while (start < end) {
        mid = start + (end - start) / 2;
        if (code_line < entries[mid].code_line) {
            end = mid;
        } else if (code_line > entries[mid].code_line) {
             start = mid + 1;
        } else {
            return mid;
        }
    }
    if (code_line <= entries[mid].code_line) {
        return mid;
    } else {
        return mid + 1;
    }
}
static __Pyx_CachedCodeObjectType *__pyx__find_code_object(struct __Pyx_CodeObjectCache *code_cache, int code_line) {
    __Pyx_CachedCodeObjectType* code_object;
    int pos;
    if (unlikely(!code_line) || unlikely(!code_cache->entries)) {
        return NULL;
    }
    pos = __pyx_bisect_code_objects(code_cache->entries, code_cache->count, code_line);
    if (unlikely(pos >= code_cache->count) || unlikely(code_cache->entries[pos].code_line != code_line)) {
        return NULL;
    }
    code_object = code_cache->entries[pos].code_object;
    Py_INCREF(code_object);
    return code_object;
}
static __Pyx_CachedCodeObjectType *__pyx_find_code_object(int code_line) {
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING && !CYTHON_ATOMICS
    (void)__pyx__find_code_object;
    return NULL; // Most implementation should have atomics. But otherwise, don't make it thread-safe, just miss.
#else
    struct __Pyx_CodeObjectCache *code_cache = &__pyx_mstate_global->__pyx_code_cache;
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    __pyx_nonatomic_int_type old_count = __pyx_atomic_incr_acq_rel(&code_cache->accessor_count);
    if (old_count < 0) {
        __pyx_atomic_decr_acq_rel(&code_cache->accessor_count);
        return NULL;
    }
#endif
    __Pyx_CachedCodeObjectType *result = __pyx__find_code_object(code_cache, code_line);
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    __pyx_atomic_decr_acq_rel(&code_cache->accessor_count);
#endif
    return result;
#endif
}
static void __pyx__insert_code_object(struct __Pyx_CodeObjectCache *code_cache, int code_line, __Pyx_CachedCodeObjectType* code_object)
{
    int pos, i;
    __Pyx_CodeObjectCacheEntry* entries = code_cache->entries;
    if (unlikely(!code_line)) {
        return;
    }
    if (unlikely(!entries)) {
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Malloc(64*sizeof(__Pyx_CodeObjectCacheEntry));
        if (likely(entries)) {
            code_cache->entries = entries;
            code_cache->max_count = 64;
            code_cache->count = 1;
            entries[0].code_line = code_line;
            entries[0].code_object = code_object;
            Py_INCREF(code_object);
        }
        return;
    }
    pos = __pyx_bisect_code_objects(code_cache->entries, code_cache->count, code_line);
    if ((pos < code_cache->count) && unlikely(code_cache->entries[pos].code_line == code_line)) {
        __Pyx_CachedCodeObjectType* tmp = entries[pos].code_object;
        entries[pos].code_object = code_object;
        Py_INCREF(code_object);
        Py_DECREF(tmp);
        return;
    }
    if (code_cache->count == code_cache->max_count) {
        int new_max = code_cache->max_count + 64;
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Realloc(
            code_cache->entries, ((size_t)new_max) * sizeof(__Pyx_CodeObjectCacheEntry));
        if (unlikely(!entries)) {
            return;
        }
        code_cache->entries = entries;
        code_cache->max_count = new_max;
    }
    for (i=code_cache->count; i>pos; i--) {
        entries[i] = entries[i-1];
    }
    entries[pos].code_line = code_line;
    entries[pos].code_object = code_object;
    code_cache->count++;
    Py_INCREF(code_object);
}
static void __pyx_insert_code_object(int code_line, __Pyx_CachedCodeObjectType* code_object) {
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING && !CYTHON_ATOMICS
    (void)__pyx__insert_code_object;
    return; // Most implementation should have atomics. But otherwise, don't make it thread-safe, just fail.
#else
    struct __Pyx_CodeObjectCache *code_cache = &__pyx_mstate_global->__pyx_code_cache;
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    __pyx_nonatomic_int_type expected = 0;
    if (!__pyx_atomic_int_cmp_exchange(&code_cache->accessor_count, &expected, INT_MIN)) {
        return;
    }
#endif
    __pyx__insert_code_object(code_cache, code_line, code_object);
#if CYTHON_COMPILING_IN_CPYTHON_FREETHREADING
    __pyx_atomic_sub(&code_cache->accessor_count, INT_MIN);
#endif
#endif
}

/* AddTraceback */
#include "compile.h"
#include "frameobject.h"
#include "traceback.h"
#if PY_VERSION_HEX >= 0x030b00a6 && !CYTHON_COMPILING_IN_LIMITED_API && !defined(PYPY_VERSION)
  #ifndef Py_BUILD_CORE
    #define Py_BUILD_CORE 1
  #endif
  #include "internal/pycore_frame.h"
#endif
#if CYTHON_COMPILING_IN_LIMITED_API
static PyObject *__Pyx_PyCode_Replace_For_AddTraceback(PyObject *code, PyObject *scratch_dict,
                                                       PyObject *firstlineno, PyObject *name) {
    PyObject *replace = NULL;
    if (unlikely(PyDict_SetItemString(scratch_dict, "co_firstlineno", firstlineno))) return NULL;
    if (unlikely(PyDict_SetItemString(scratch_dict, "co_name", name))) return NULL;
    replace = PyObject_GetAttrString(code, "replace");
    if (likely(replace)) {
        PyObject *result = PyObject_Call(replace, __pyx_mstate_global->__pyx_empty_tuple, scratch_dict);
        Py_DECREF(replace);
        return result;
    }
    PyErr_Clear();
    return NULL;
}
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename) {
    PyObject *code_object = NULL, *py_py_line = NULL, *py_funcname = NULL, *dict = NULL;
    PyObject *replace = NULL, *getframe = NULL, *frame = NULL;
    PyObject *exc_type, *exc_value, *exc_traceback;
    int success = 0;
    if (c_line) {
        c_line = __Pyx_CLineForTraceback(__Pyx_PyThreadState_Current, c_line);
    }
    PyErr_Fetch(&exc_type, &exc_value, &exc_traceback);
    code_object = __pyx_find_code_object(c_line ? -c_line : py_line);
    if (!code_object) {
        code_object = Py_CompileString("_getframe()", filename, Py_eval_input);
        if (unlikely(!code_object)) goto bad;
        py_py_line = PyLong_FromLong(py_line);
        if (unlikely(!py_py_line)) goto bad;
        if (c_line) {
            py_funcname = PyUnicode_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        } else {
            py_funcname = PyUnicode_FromString(funcname);
        }
        if (unlikely(!py_funcname)) goto bad;
        dict = PyDict_New();
        if (unlikely(!dict)) goto bad;
        {
            PyObject *old_code_object = code_object;
            code_object = __Pyx_PyCode_Replace_For_AddTraceback(code_object, dict, py_py_line, py_funcname);
            Py_DECREF(old_code_object);
        }
        if (unlikely(!code_object)) goto bad;
        __pyx_insert_code_object(c_line ? -c_line : py_line, code_object);
    } else {
        dict = PyDict_New();
    }
    getframe = PySys_GetObject("_getframe");
    if (unlikely(!getframe)) goto bad;
    if (unlikely(PyDict_SetItemString(dict, "_getframe", getframe))) goto bad;
    frame = PyEval_EvalCode(code_object, dict, dict);
    if (unlikely(!frame) || frame == Py_None) goto bad;
    success = 1;
  bad:
    PyErr_Restore(exc_type, exc_value, exc_traceback);
    Py_XDECREF(code_object);
    Py_XDECREF(py_py_line);
    Py_XDECREF(py_funcname);
    Py_XDECREF(dict);
    Py_XDECREF(replace);
    if (success) {
        PyTraceBack_Here(
            (struct _frame*)frame);
    }
    Py_XDECREF(frame);
}
#else
static PyCodeObject* __Pyx_CreateCodeObjectForTraceback(
            const char *funcname, int c_line,
            int py_line, const char *filename) {
    PyCodeObject *py_code = NULL;
    PyObject *py_funcname = NULL;
    if (c_line) {
        py_funcname = PyUnicode_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        if (!py_funcname) goto bad;
        funcname = PyUnicode_AsUTF8(py_funcname);
        if (!funcname) goto bad;
    }
    py_code = PyCode_NewEmpty(filename, funcname, py_line);
    Py_XDECREF(py_funcname);
    return py_code;
bad:
    Py_XDECREF(py_funcname);
    return NULL;
}
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyFrameObject *py_frame = 0;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject *ptype, *pvalue, *ptraceback;
    if (c_line) {
        c_line = __Pyx_CLineForTraceback(tstate, c_line);
    }
    py_code = __pyx_find_code_object(c_line ? -c_line : py_line);
    if (!py_code) {
        __Pyx_ErrFetchInState(tstate, &ptype, &pvalue, &ptraceback);
        py_code = __Pyx_CreateCodeObjectForTraceback(
            funcname, c_line, py_line, filename);
        if (!py_code) {
            /* If the code object creation fails, then we should clear the
               fetched exception references and propagate the new exception */
            Py_XDECREF(ptype);
            Py_XDECREF(pvalue);
            Py_XDECREF(ptraceback);
            goto bad;
        }
        __Pyx_ErrRestoreInState(tstate, ptype, pvalue, ptraceback);
        __pyx_insert_code_object(c_line ? -c_line : py_line, py_code);
    }
    py_frame = PyFrame_New(
        tstate,            /*PyThreadState *tstate,*/
        py_code,           /*PyCodeObject *code,*/
        __pyx_mstate_global->__pyx_d,    /*PyObject *globals,*/
        0                  /*PyObject *locals*/
    );
    if (!py_frame) goto bad;
    __Pyx_PyFrame_SetLineNumber(py_frame, py_line);
    PyTraceBack_Here(py_frame);
bad:
    Py_XDECREF(py_code);
    Py_XDECREF(py_frame);
}
#endif

/* CheckUnpickleChecksum */
static void __Pyx_RaiseUnpickleChecksumError(long checksum, long checksum1, long checksum2, long checksum3, const char *members) {
    PyObject *pickle_module = PyImport_ImportModule("pickle");
    if (unlikely(!pickle_module)) return;
    PyObject *pickle_error = PyObject_GetAttrString(pickle_module, "PickleError");
    Py_DECREF(pickle_module);
    if (unlikely(!pickle_error)) return;
    if (checksum2 == checksum1) {
        PyErr_Format(pickle_error, "Incompatible checksums (0x%x vs (0x%x) = (%s))",
            checksum, checksum1, members);
    } else if (checksum3 == checksum2) {
        PyErr_Format(pickle_error, "Incompatible checksums (0x%x vs (0x%x, 0x%x) = (%s))",
            checksum, checksum1, checksum2, members);
    } else {
        PyErr_Format(pickle_error, "Incompatible checksums (0x%x vs (0x%x, 0x%x, 0x%x) = (%s))",
            checksum, checksum1, checksum2, checksum3, members);
    }
    Py_DECREF(pickle_error);
}
static int __Pyx_CheckUnpickleChecksum(long checksum, long checksum1, long checksum2, long checksum3, const char *members) {
    int found = 0;
    found |= checksum1 == checksum;
    found |= checksum2 == checksum;
    found |= checksum3 == checksum;
    if (likely(found))
        return 0;
    __Pyx_RaiseUnpickleChecksumError(checksum, checksum1, checksum2, checksum3, members);
    return -1;
}

/* CIntFromPyVerify */
#define __PYX_VERIFY_RETURN_INT(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 0)
#define __PYX_VERIFY_RETURN_INT_EXC(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 1)
#define __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, exc)\
    {\
        func_type value = func_value;\
        if (sizeof(target_type) < sizeof(func_type)) {\
            if (unlikely(value != (func_type) (target_type) value)) {\
                func_type zero = 0;\
                if (exc && unlikely(value == (func_type)-1 && PyErr_Occurred()))\
                    return (target_type) -1;\
                if (is_unsigned && unlikely(value < zero))\
                    goto raise_neg_overflow;\
                else\
                    goto raise_overflow;\
            }\
        }\
        return (target_type) value;\
    }

/* CIntFromPy */
static CYTHON_INLINE int __Pyx_PyLong_As_int(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const int neg_one = (int) -1, const_zero = (int) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (unlikely(!PyLong_Check(x))) {
        int val;
        PyObject *tmp = __Pyx_PyNumber_Long(x);
        if (!tmp) return (int) -1;
        val = __Pyx_PyLong_As_int(tmp);
        Py_DECREF(tmp);
        return val;
    }
    if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
        if (unlikely(__Pyx_PyLong_IsNeg(x))) {
            goto raise_neg_overflow;
        } else if (__Pyx_PyLong_IsCompact(x)) {
            __PYX_VERIFY_RETURN_INT(int, __Pyx_compact_upylong, __Pyx_PyLong_CompactValueUnsigned(x))
        } else {
            const digit* digits = __Pyx_PyLong_Digits(x);
            assert(__Pyx_PyLong_DigitCount(x) > 1);
            switch (__Pyx_PyLong_DigitCount(x)) {
                case 2:
                    if ((8 * sizeof(int) > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) >= 2 * PyLong_SHIFT)) {
                            return (int) (((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if ((8 * sizeof(int) > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) >= 3 * PyLong_SHIFT)) {
                            return (int) (((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if ((8 * sizeof(int) > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) >= 4 * PyLong_SHIFT)) {
                            return (int) (((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
            }
        }
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030C00A7
        if (unlikely(Py_SIZE(x) < 0)) {
            goto raise_neg_overflow;
        }
#else
        {
            int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
            if (unlikely(result < 0))
                return (int) -1;
            if (unlikely(result == 1))
                goto raise_neg_overflow;
        }
#endif
        if ((sizeof(int) <= sizeof(unsigned long))) {
            __PYX_VERIFY_RETURN_INT_EXC(int, unsigned long, PyLong_AsUnsignedLong(x))
        } else if ((sizeof(int) <= sizeof(unsigned PY_LONG_LONG))) {
            __PYX_VERIFY_RETURN_INT_EXC(int, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
        }
    } else {
#if CYTHON_USE_PYLONG_INTERNALS
        if (__Pyx_PyLong_IsCompact(x)) {
            __PYX_VERIFY_RETURN_INT(int, __Pyx_compact_pylong, __Pyx_PyLong_CompactValue(x))
        } else {
            const digit* digits = __Pyx_PyLong_Digits(x);
            assert(__Pyx_PyLong_DigitCount(x) > 1);
            switch (__Pyx_PyLong_SignedDigitCount(x)) {
                case -2:
                    if ((8 * sizeof(int) - 1 > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) - 1 > 2 * PyLong_SHIFT)) {
                            return (int) (((int)-1)*(((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if ((8 * sizeof(int) > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) - 1 > 2 * PyLong_SHIFT)) {
                            return (int) ((((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if ((8 * sizeof(int) - 1 > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) - 1 > 3 * PyLong_SHIFT)) {
                            return (int) (((int)-1)*(((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if ((8 * sizeof(int) > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) - 1 > 3 * PyLong_SHIFT)) {
                            return (int) ((((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if ((8 * sizeof(int) - 1 > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) - 1 > 4 * PyLong_SHIFT)) {
                            return (int) (((int)-1)*(((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if ((8 * sizeof(int) > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(int) - 1 > 4 * PyLong_SHIFT)) {
                            return (int) ((((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
            }
        }
#endif
        if ((sizeof(int) <= sizeof(long))) {
            __PYX_VERIFY_RETURN_INT_EXC(int, long, PyLong_AsLong(x))
        } else if ((sizeof(int) <= sizeof(PY_LONG_LONG))) {
            __PYX_VERIFY_RETURN_INT_EXC(int, PY_LONG_LONG, PyLong_AsLongLong(x))
        }
    }
    {
        int val;
        int ret = -1;
#if PY_VERSION_HEX >= 0x030d00A6 && !CYTHON_COMPILING_IN_LIMITED_API
        Py_ssize_t bytes_copied = PyLong_AsNativeBytes(
            x, &val, sizeof(val), Py_ASNATIVEBYTES_NATIVE_ENDIAN | (is_unsigned ? Py_ASNATIVEBYTES_UNSIGNED_BUFFER | Py_ASNATIVEBYTES_REJECT_NEGATIVE : 0));
        if (unlikely(bytes_copied == -1)) {
        } else if (unlikely(bytes_copied > (Py_ssize_t) sizeof(val))) {
            goto raise_overflow;
        } else {
            ret = 0;
        }
#elif PY_VERSION_HEX < 0x030d0000 && !(CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API) || defined(_PyLong_AsByteArray)
        int one = 1; int is_little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&val;
        ret = _PyLong_AsByteArray((PyLongObject *)x,
                                    bytes, sizeof(val),
                                    is_little, !is_unsigned);
#else
        PyObject *v;
        PyObject *stepval = NULL, *mask = NULL, *shift = NULL;
        int bits, remaining_bits, is_negative = 0;
        int chunk_size = (sizeof(long) < 8) ? 30 : 62;
        if (likely(PyLong_CheckExact(x))) {
            v = __Pyx_NewRef(x);
        } else {
            v = PyNumber_Long(x);
            if (unlikely(!v)) return (int) -1;
            assert(PyLong_CheckExact(v));
        }
        {
            int result = PyObject_RichCompareBool(v, Py_False, Py_LT);
            if (unlikely(result < 0)) {
                Py_DECREF(v);
                return (int) -1;
            }
            is_negative = result == 1;
        }
        if (is_unsigned && unlikely(is_negative)) {
            Py_DECREF(v);
            goto raise_neg_overflow;
        } else if (is_negative) {
            stepval = PyNumber_Invert(v);
            Py_DECREF(v);
            if (unlikely(!stepval))
                return (int) -1;
        } else {
            stepval = v;
        }
        v = NULL;
        val = (int) 0;
        mask = PyLong_FromLong((1L << chunk_size) - 1); if (unlikely(!mask)) goto done;
        shift = PyLong_FromLong(chunk_size); if (unlikely(!shift)) goto done;
        for (bits = 0; bits < (int) sizeof(int) * 8 - chunk_size; bits += chunk_size) {
            PyObject *tmp, *digit;
            long idigit;
            digit = PyNumber_And(stepval, mask);
            if (unlikely(!digit)) goto done;
            idigit = PyLong_AsLong(digit);
            Py_DECREF(digit);
            if (unlikely(idigit < 0)) goto done;
            val |= ((int) idigit) << bits;
            tmp = PyNumber_Rshift(stepval, shift);
            if (unlikely(!tmp)) goto done;
            Py_DECREF(stepval); stepval = tmp;
        }
        Py_DECREF(shift); shift = NULL;
        Py_DECREF(mask); mask = NULL;
        {
            long idigit = PyLong_AsLong(stepval);
            if (unlikely(idigit < 0)) goto done;
            remaining_bits = ((int) sizeof(int) * 8) - bits - (is_unsigned ? 0 : 1);
            if (unlikely(idigit >= (1L << remaining_bits)))
                goto raise_overflow;
            val |= ((int) idigit) << bits;
        }
        if (!is_unsigned) {
            if (unlikely(val & (((int) 1) << (sizeof(int) * 8 - 1))))
                goto raise_overflow;
            if (is_negative)
                val = ~val;
        }
        ret = 0;
    done:
        Py_XDECREF(shift);
        Py_XDECREF(mask);
        Py_XDECREF(stepval);
#endif
        if (unlikely(ret))
            return (int) -1;
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to int");
    return (int) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to int");
    return (int) -1;
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyLong_From_int(int value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const int neg_one = (int) -1, const_zero = (int) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(int) < sizeof(long)) {
            return PyLong_FromLong((long) value);
        } else if (sizeof(int) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#if !CYTHON_COMPILING_IN_PYPY
        } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(int) <= sizeof(long)) {
            return PyLong_FromLong((long) value);
        } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
        }
    }
    {
        unsigned char *bytes = (unsigned char *)&value;
#if !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x030d00A4
        if (is_unsigned) {
            return PyLong_FromUnsignedNativeBytes(bytes, sizeof(value), -1);
        } else {
            return PyLong_FromNativeBytes(bytes, sizeof(value), -1);
        }
#elif !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030d0000
        int one = 1; int little = (int)*(unsigned char *)&one;
        return _PyLong_FromByteArray(bytes, sizeof(int),
                                     little, !is_unsigned);
#else
        int one = 1; int little = (int)*(unsigned char *)&one;
        PyObject *from_bytes, *result = NULL, *kwds = NULL;
        PyObject *py_bytes = NULL, *order_str = NULL;
        from_bytes = PyObject_GetAttrString((PyObject*)&PyLong_Type, "from_bytes");
        if (!from_bytes) return NULL;
        py_bytes = PyBytes_FromStringAndSize((char*)bytes, sizeof(int));
        if (!py_bytes) goto limited_bad;
        order_str = PyUnicode_FromString(little ? "little" : "big");
        if (!order_str) goto limited_bad;
        {
            PyObject *args[3+(CYTHON_VECTORCALL ? 1 : 0)] = { NULL, py_bytes, order_str };
            if (!is_unsigned) {
                kwds = __Pyx_MakeVectorcallBuilderKwds(1);
                if (!kwds) goto limited_bad;
                if (__Pyx_VectorcallBuilder_AddArgStr("signed", __Pyx_NewRef(Py_True), kwds, args+3, 0) < 0) goto limited_bad;
            }
            result = __Pyx_Object_Vectorcall_CallFromBuilder(from_bytes, args+1, 2 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET, kwds);
        }
        limited_bad:
        Py_XDECREF(kwds);
        Py_XDECREF(order_str);
        Py_XDECREF(py_bytes);
        Py_XDECREF(from_bytes);
        return result;
#endif
    }
}

/* CIntFromPy */
static CYTHON_INLINE long __Pyx_PyLong_As_long(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const long neg_one = (long) -1, const_zero = (long) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (unlikely(!PyLong_Check(x))) {
        long val;
        PyObject *tmp = __Pyx_PyNumber_Long(x);
        if (!tmp) return (long) -1;
        val = __Pyx_PyLong_As_long(tmp);
        Py_DECREF(tmp);
        return val;
    }
    if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
        if (unlikely(__Pyx_PyLong_IsNeg(x))) {
            goto raise_neg_overflow;
        } else if (__Pyx_PyLong_IsCompact(x)) {
            __PYX_VERIFY_RETURN_INT(long, __Pyx_compact_upylong, __Pyx_PyLong_CompactValueUnsigned(x))
        } else {
            const digit* digits = __Pyx_PyLong_Digits(x);
            assert(__Pyx_PyLong_DigitCount(x) > 1);
            switch (__Pyx_PyLong_DigitCount(x)) {
                case 2:
                    if ((8 * sizeof(long) > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) >= 2 * PyLong_SHIFT)) {
                            return (long) (((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if ((8 * sizeof(long) > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) >= 3 * PyLong_SHIFT)) {
                            return (long) (((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if ((8 * sizeof(long) > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) >= 4 * PyLong_SHIFT)) {
                            return (long) (((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
            }
        }
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030C00A7
        if (unlikely(Py_SIZE(x) < 0)) {
            goto raise_neg_overflow;
        }
#else
        {
            int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
            if (unlikely(result < 0))
                return (long) -1;
            if (unlikely(result == 1))
                goto raise_neg_overflow;
        }
#endif
        if ((sizeof(long) <= sizeof(unsigned long))) {
            __PYX_VERIFY_RETURN_INT_EXC(long, unsigned long, PyLong_AsUnsignedLong(x))
        } else if ((sizeof(long) <= sizeof(unsigned PY_LONG_LONG))) {
            __PYX_VERIFY_RETURN_INT_EXC(long, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
        }
    } else {
#if CYTHON_USE_PYLONG_INTERNALS
        if (__Pyx_PyLong_IsCompact(x)) {
            __PYX_VERIFY_RETURN_INT(long, __Pyx_compact_pylong, __Pyx_PyLong_CompactValue(x))
        } else {
            const digit* digits = __Pyx_PyLong_Digits(x);
            assert(__Pyx_PyLong_DigitCount(x) > 1);
            switch (__Pyx_PyLong_SignedDigitCount(x)) {
                case -2:
                    if ((8 * sizeof(long) - 1 > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) - 1 > 2 * PyLong_SHIFT)) {
                            return (long) (((long)-1)*(((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if ((8 * sizeof(long) > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) - 1 > 2 * PyLong_SHIFT)) {
                            return (long) ((((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if ((8 * sizeof(long) - 1 > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) - 1 > 3 * PyLong_SHIFT)) {
                            return (long) (((long)-1)*(((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if ((8 * sizeof(long) > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) - 1 > 3 * PyLong_SHIFT)) {
                            return (long) ((((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if ((8 * sizeof(long) - 1 > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) - 1 > 4 * PyLong_SHIFT)) {
                            return (long) (((long)-1)*(((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if ((8 * sizeof(long) > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(long) - 1 > 4 * PyLong_SHIFT)) {
                            return (long) ((((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
            }
        }
#endif
        if ((sizeof(long) <= sizeof(long))) {
            __PYX_VERIFY_RETURN_INT_EXC(long, long, PyLong_AsLong(x))
        } else if ((sizeof(long) <= sizeof(PY_LONG_LONG))) {
            __PYX_VERIFY_RETURN_INT_EXC(long, PY_LONG_LONG, PyLong_AsLongLong(x))
        }
    }
    {
        long val;
        int ret = -1;
#if PY_VERSION_HEX >= 0x030d00A6 && !CYTHON_COMPILING_IN_LIMITED_API
        Py_ssize_t bytes_copied = PyLong_AsNativeBytes(
            x, &val, sizeof(val), Py_ASNATIVEBYTES_NATIVE_ENDIAN | (is_unsigned ? Py_ASNATIVEBYTES_UNSIGNED_BUFFER | Py_ASNATIVEBYTES_REJECT_NEGATIVE : 0));
        if (unlikely(bytes_copied == -1)) {
        } else if (unlikely(bytes_copied > (Py_ssize_t) sizeof(val))) {
            goto raise_overflow;
        } else {
            ret = 0;
        }
#elif PY_VERSION_HEX < 0x030d0000 && !(CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API) || defined(_PyLong_AsByteArray)
        int one = 1; int is_little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&val;
        ret = _PyLong_AsByteArray((PyLongObject *)x,
                                    bytes, sizeof(val),
                                    is_little, !is_unsigned);
#else
        PyObject *v;
        PyObject *stepval = NULL, *mask = NULL, *shift = NULL;
        int bits, remaining_bits, is_negative = 0;
        int chunk_size = (sizeof(long) < 8) ? 30 : 62;
        if (likely(PyLong_CheckExact(x))) {
            v = __Pyx_NewRef(x);
        } else {
            v = PyNumber_Long(x);
            if (unlikely(!v)) return (long) -1;
            assert(PyLong_CheckExact(v));
        }
        {
            int result = PyObject_RichCompareBool(v, Py_False, Py_LT);
            if (unlikely(result < 0)) {
                Py_DECREF(v);
                return (long) -1;
            }
            is_negative = result == 1;
        }
        if (is_unsigned && unlikely(is_negative)) {
            Py_DECREF(v);
            goto raise_neg_overflow;
        } else if (is_negative) {
            stepval = PyNumber_Invert(v);
            Py_DECREF(v);
            if (unlikely(!stepval))
                return (long) -1;
        } else {
            stepval = v;
        }
        v = NULL;
        val = (long) 0;
        mask = PyLong_FromLong((1L << chunk_size) - 1); if (unlikely(!mask)) goto done;
        shift = PyLong_FromLong(chunk_size); if (unlikely(!shift)) goto done;
        for (bits = 0; bits < (int) sizeof(long) * 8 - chunk_size; bits += chunk_size) {
            PyObject *tmp, *digit;
            long idigit;
            digit = PyNumber_And(stepval, mask);
            if (unlikely(!digit)) goto done;
            idigit = PyLong_AsLong(digit);
            Py_DECREF(digit);
            if (unlikely(idigit < 0)) goto done;
            val |= ((long) idigit) << bits;
            tmp = PyNumber_Rshift(stepval, shift);
            if (unlikely(!tmp)) goto done;
            Py_DECREF(stepval); stepval = tmp;
        }
        Py_DECREF(shift); shift = NULL;
        Py_DECREF(mask); mask = NULL;
        {
            long idigit = PyLong_AsLong(stepval);
            if (unlikely(idigit < 0)) goto done;
            remaining_bits = ((int) sizeof(long) * 8) - bits - (is_unsigned ? 0 : 1);
            if (unlikely(idigit >= (1L << remaining_bits)))
                goto raise_overflow;
            val |= ((long) idigit) << bits;
        }
        if (!is_unsigned) {
            if (unlikely(val & (((long) 1) << (sizeof(long) * 8 - 1))))
                goto raise_overflow;
            if (is_negative)
                val = ~val;
        }
        ret = 0;
    done:
        Py_XDECREF(shift);
        Py_XDECREF(mask);
        Py_XDECREF(stepval);
#endif
        if (unlikely(ret))
            return (long) -1;
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to long");
    return (long) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to long");
    return (long) -1;
}

/* CIntFromPy */
static CYTHON_INLINE size_t __Pyx_PyLong_As_size_t(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const size_t neg_one = (size_t) -1, const_zero = (size_t) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (unlikely(!PyLong_Check(x))) {
        size_t val;
        PyObject *tmp = __Pyx_PyNumber_Long(x);
        if (!tmp) return (size_t) -1;
        val = __Pyx_PyLong_As_size_t(tmp);
        Py_DECREF(tmp);
        return val;
    }
    if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
        if (unlikely(__Pyx_PyLong_IsNeg(x))) {
            goto raise_neg_overflow;
        } else if (__Pyx_PyLong_IsCompact(x)) {
            __PYX_VERIFY_RETURN_INT(size_t, __Pyx_compact_upylong, __Pyx_PyLong_CompactValueUnsigned(x))
        } else {
            const digit* digits = __Pyx_PyLong_Digits(x);
            assert(__Pyx_PyLong_DigitCount(x) > 1);
            switch (__Pyx_PyLong_DigitCount(x)) {
                case 2:
                    if ((8 * sizeof(size_t) > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(size_t) >= 2 * PyLong_SHIFT)) {
                            return (size_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if ((8 * sizeof(size_t) > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(size_t) >= 3 * PyLong_SHIFT)) {
                            return (size_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if ((8 * sizeof(size_t) > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(size_t) >= 4 * PyLong_SHIFT)) {
                            return (size_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
                        }
                    }
                    break;
            }
        }
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030C00A7
        if (unlikely(Py_SIZE(x) < 0)) {
            goto raise_neg_overflow;
        }
#else
        {
            int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
            if (unlikely(result < 0))
                return (size_t) -1;
            if (unlikely(result == 1))
                goto raise_neg_overflow;
        }
#endif
        if ((sizeof(size_t) <= sizeof(unsigned long))) {
            __PYX_VERIFY_RETURN_INT_EXC(size_t, unsigned long, PyLong_AsUnsignedLong(x))
        } else if ((sizeof(size_t) <= sizeof(unsigned PY_LONG_LONG))) {
            __PYX_VERIFY_RETURN_INT_EXC(size_t, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
        }
    } else {
#if CYTHON_USE_PYLONG_INTERNALS
        if (__Pyx_PyLong_IsCompact(x)) {
            __PYX_VERIFY_RETURN_INT(size_t, __Pyx_compact_pylong, __Pyx_PyLong_CompactValue(x))
        } else {
            const digit* digits = __Pyx_PyLong_Digits(x);
            assert(__Pyx_PyLong_DigitCount(x) > 1);
            switch (__Pyx_PyLong_SignedDigitCount(x)) {
                case -2:
                    if ((8 * sizeof(size_t) - 1 > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(size_t, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(size_t) - 1 > 2 * PyLong_SHIFT)) {
                            return (size_t) (((size_t)-1)*(((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if ((8 * sizeof(size_t) > 1 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 2 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(size_t) - 1 > 2 * PyLong_SHIFT)) {
                            return (size_t) ((((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if ((8 * sizeof(size_t) - 1 > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(size_t, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(size_t) - 1 > 3 * PyLong_SHIFT)) {
                            return (size_t) (((size_t)-1)*(((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if ((8 * sizeof(size_t) > 2 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 3 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(size_t) - 1 > 3 * PyLong_SHIFT)) {
                            return (size_t) ((((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if ((8 * sizeof(size_t) - 1 > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(size_t, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(size_t) - 1 > 4 * PyLong_SHIFT)) {
                            return (size_t) (((size_t)-1)*(((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if ((8 * sizeof(size_t) > 3 * PyLong_SHIFT)) {
                        if ((8 * sizeof(unsigned long) > 4 * PyLong_SHIFT)) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if ((8 * sizeof(size_t) - 1 > 4 * PyLong_SHIFT)) {
                            return (size_t) ((((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
            }
        }
#endif
        if ((sizeof(size_t) <= sizeof(long))) {
            __PYX_VERIFY_RETURN_INT_EXC(size_t, long, PyLong_AsLong(x))
        } else if ((sizeof(size_t) <= sizeof(PY_LONG_LONG))) {
            __PYX_VERIFY_RETURN_INT_EXC(size_t, PY_LONG_LONG, PyLong_AsLongLong(x))
        }
    }
    {
        size_t val;
        int ret = -1;
#if PY_VERSION_HEX >= 0x030d00A6 && !CYTHON_COMPILING_IN_LIMITED_API
        Py_ssize_t bytes_copied = PyLong_AsNativeBytes(
            x, &val, sizeof(val), Py_ASNATIVEBYTES_NATIVE_ENDIAN | (is_unsigned ? Py_ASNATIVEBYTES_UNSIGNED_BUFFER | Py_ASNATIVEBYTES_REJECT_NEGATIVE : 0));
        if (unlikely(bytes_copied == -1)) {
        } else if (unlikely(bytes_copied > (Py_ssize_t) sizeof(val))) {
            goto raise_overflow;
        } else {
            ret = 0;
        }
#elif PY_VERSION_HEX < 0x030d0000 && !(CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_LIMITED_API) || defined(_PyLong_AsByteArray)
        int one = 1; int is_little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&val;
        ret = _PyLong_AsByteArray((PyLongObject *)x,
                                    bytes, sizeof(val),
                                    is_little, !is_unsigned);
#else
        PyObject *v;
        PyObject *stepval = NULL, *mask = NULL, *shift = NULL;
        int bits, remaining_bits, is_negative = 0;
        int chunk_size = (sizeof(long) < 8) ? 30 : 62;
        if (likely(PyLong_CheckExact(x))) {
            v = __Pyx_NewRef(x);
        } else {
            v = PyNumber_Long(x);
            if (unlikely(!v)) return (size_t) -1;
            assert(PyLong_CheckExact(v));
        }
        {
            int result = PyObject_RichCompareBool(v, Py_False, Py_LT);
            if (unlikely(result < 0)) {
                Py_DECREF(v);
                return (size_t) -1;
            }
            is_negative = result == 1;
        }
        if (is_unsigned && unlikely(is_negative)) {
            Py_DECREF(v);
            goto raise_neg_overflow;
        } else if (is_negative) {
            stepval = PyNumber_Invert(v);
            Py_DECREF(v);
            if (unlikely(!stepval))
                return (size_t) -1;
        } else {
            stepval = v;
        }
        v = NULL;
        val = (size_t) 0;
        mask = PyLong_FromLong((1L << chunk_size) - 1); if (unlikely(!mask)) goto done;
        shift = PyLong_FromLong(chunk_size); if (unlikely(!shift)) goto done;
        for (bits = 0; bits < (int) sizeof(size_t) * 8 - chunk_size; bits += chunk_size) {
            PyObject *tmp, *digit;
            long idigit;
            digit = PyNumber_And(stepval, mask);
            if (unlikely(!digit)) goto done;
            idigit = PyLong_AsLong(digit);
            Py_DECREF(digit);
            if (unlikely(idigit < 0)) goto done;
            val |= ((size_t) idigit) << bits;
            tmp = PyNumber_Rshift(stepval, shift);
            if (unlikely(!tmp)) goto done;
            Py_DECREF(stepval); stepval = tmp;
        }
        Py_DECREF(shift); shift = NULL;
        Py_DECREF(mask); mask = NULL;
        {
            long idigit = PyLong_AsLong(stepval);
            if (unlikely(idigit < 0)) goto done;
            remaining_bits = ((int) sizeof(size_t) * 8) - bits - (is_unsigned ? 0 : 1);
            if (unlikely(idigit >= (1L << remaining_bits)))
                goto raise_overflow;
            val |= ((size_t) idigit) << bits;
        }
        if (!is_unsigned) {
            if (unlikely(val & (((size_t) 1) << (sizeof(size_t) * 8 - 1))))
                goto raise_overflow;
            if (is_negative)
                val = ~val;
        }
        ret = 0;
    done:
        Py_XDECREF(shift);
        Py_XDECREF(mask);
        Py_XDECREF(stepval);
#endif
        if (unlikely(ret))
            return (size_t) -1;
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to size_t");
    return (size_t) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to size_t");
    return (size_t) -1;
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyLong_From_long(long value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const long neg_one = (long) -1, const_zero = (long) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(long) < sizeof(long)) {
            return PyLong_FromLong((long) value);
        } else if (sizeof(long) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#if !CYTHON_COMPILING_IN_PYPY
        } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(long) <= sizeof(long)) {
            return PyLong_FromLong((long) value);
        } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
        }
    }
    {
        unsigned char *bytes = (unsigned char *)&value;
#if !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x030d00A4
        if (is_unsigned) {
            return PyLong_FromUnsignedNativeBytes(bytes, sizeof(value), -1);
        } else {
            return PyLong_FromNativeBytes(bytes, sizeof(value), -1);
        }
#elif !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030d0000
        int one = 1; int little = (int)*(unsigned char *)&one;
        return _PyLong_FromByteArray(bytes, sizeof(long),
                                     little, !is_unsigned);
#else
        int one = 1; int little = (int)*(unsigned char *)&one;
        PyObject *from_bytes, *result = NULL, *kwds = NULL;
        PyObject *py_bytes = NULL, *order_str = NULL;
        from_bytes = PyObject_GetAttrString((PyObject*)&PyLong_Type, "from_bytes");
        if (!from_bytes) return NULL;
        py_bytes = PyBytes_FromStringAndSize((char*)bytes, sizeof(long));
        if (!py_bytes) goto limited_bad;
        order_str = PyUnicode_FromString(little ? "little" : "big");
        if (!order_str) goto limited_bad;
        {
            PyObject *args[3+(CYTHON_VECTORCALL ? 1 : 0)] = { NULL, py_bytes, order_str };
            if (!is_unsigned) {
                kwds = __Pyx_MakeVectorcallBuilderKwds(1);
                if (!kwds) goto limited_bad;
                if (__Pyx_VectorcallBuilder_AddArgStr("signed", __Pyx_NewRef(Py_True), kwds, args+3, 0) < 0) goto limited_bad;
            }
            result = __Pyx_Object_Vectorcall_CallFromBuilder(from_bytes, args+1, 2 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET, kwds);
        }
        limited_bad:
        Py_XDECREF(kwds);
        Py_XDECREF(order_str);
        Py_XDECREF(py_bytes);
        Py_XDECREF(from_bytes);
        return result;
#endif
    }
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyLong_From_uint64_t(uint64_t value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const uint64_t neg_one = (uint64_t) -1, const_zero = (uint64_t) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(uint64_t) < sizeof(long)) {
            return PyLong_FromLong((long) value);
        } else if (sizeof(uint64_t) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#if !CYTHON_COMPILING_IN_PYPY
        } else if (sizeof(uint64_t) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(uint64_t) <= sizeof(long)) {
            return PyLong_FromLong((long) value);
        } else if (sizeof(uint64_t) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
        }
    }
    {
        unsigned char *bytes = (unsigned char *)&value;
#if !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x030d00A4
        if (is_unsigned) {
            return PyLong_FromUnsignedNativeBytes(bytes, sizeof(value), -1);
        } else {
            return PyLong_FromNativeBytes(bytes, sizeof(value), -1);
        }
#elif !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX < 0x030d0000
        int one = 1; int little = (int)*(unsigned char *)&one;
        return _PyLong_FromByteArray(bytes, sizeof(uint64_t),
                                     little, !is_unsigned);
#else
        int one = 1; int little = (int)*(unsigned char *)&one;
        PyObject *from_bytes, *result = NULL, *kwds = NULL;
        PyObject *py_bytes = NULL, *order_str = NULL;
        from_bytes = PyObject_GetAttrString((PyObject*)&PyLong_Type, "from_bytes");
        if (!from_bytes) return NULL;
        py_bytes = PyBytes_FromStringAndSize((char*)bytes, sizeof(uint64_t));
        if (!py_bytes) goto limited_bad;
        order_str = PyUnicode_FromString(little ? "little" : "big");
        if (!order_str) goto limited_bad;
        {
            PyObject *args[3+(CYTHON_VECTORCALL ? 1 : 0)] = { NULL, py_bytes, order_str };
            if (!is_unsigned) {
                kwds = __Pyx_MakeVectorcallBuilderKwds(1);
                if (!kwds) goto limited_bad;
                if (__Pyx_VectorcallBuilder_AddArgStr("signed", __Pyx_NewRef(Py_True), kwds, args+3, 0) < 0) goto limited_bad;
            }
            result = __Pyx_Object_Vectorcall_CallFromBuilder(from_bytes, args+1, 2 | __Pyx_PY_VECTORCALL_ARGUMENTS_OFFSET, kwds);
        }
        limited_bad:
        Py_XDECREF(kwds);
        Py_XDECREF(order_str);
        Py_XDECREF(py_bytes);
        Py_XDECREF(from_bytes);
        return result;
#endif
    }
}

/* PyObjectCallMethod1 */
#if !(CYTHON_VECTORCALL && (__PYX_LIMITED_VERSION_HEX >= 0x030C0000 || (!CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x03090000)))
static PyObject* __Pyx__PyObject_CallMethod1(PyObject* method, PyObject* arg) {
    PyObject *result = __Pyx_PyObject_CallOneArg(method, arg);
    Py_DECREF(method);
    return result;
}
#endif
static PyObject* __Pyx_PyObject_CallMethod1(PyObject* obj, PyObject* method_name, PyObject* arg) {
#if CYTHON_VECTORCALL && (__PYX_LIMITED_VERSION_HEX >= 0x030C0000 || (!CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x03090000))
    PyObject *args[2] = {obj, arg};
    (void) __Pyx_PyObject_CallOneArg;
    (void) __Pyx_PyObject_Call2Args;
    return PyObject_VectorcallMethod(method_name, args, 2 | PY_VECTORCALL_ARGUMENTS_OFFSET, NULL);
#else
    PyObject *method = NULL, *result;
    int is_method = __Pyx_PyObject_GetMethod(obj, method_name, &method);
    if (likely(is_method)) {
        result = __Pyx_PyObject_Call2Args(method, obj, arg);
        Py_DECREF(method);
        return result;
    }
    if (unlikely(!method)) return NULL;
    return __Pyx__PyObject_CallMethod1(method, arg);
#endif
}

/* UpdateUnpickledDict */
static int __Pyx__UpdateUnpickledDict(PyObject *obj, PyObject *state, Py_ssize_t index) {
    PyObject *state_dict = __Pyx_PySequence_ITEM(state, index);
    if (unlikely(!state_dict)) {
        return -1;
    }
    int non_empty = PyObject_IsTrue(state_dict);
    if (non_empty == 0) {
        Py_DECREF(state_dict);
        return 0;
    } else if (unlikely(non_empty == -1)) {
        return -1;
    }
    PyObject *dict;
    #if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030A0000
    dict = PyObject_GetAttrString(obj, "__dict__");
    #else
    dict = PyObject_GenericGetDict(obj, NULL);
    #endif
    if (unlikely(!dict)) {
        Py_DECREF(state_dict);
        return -1;
    }
    int result;
    if (likely(PyDict_CheckExact(dict))) {
        result = PyDict_Update(dict, state_dict);
    } else {
        PyObject *obj_result = __Pyx_PyObject_CallMethod1(dict, __pyx_mstate_global->__pyx_n_u_update, state_dict);
        if (likely(obj_result)) {
            Py_DECREF(obj_result);
            result = 0;
        } else {
            result = -1;
        }
    }
    Py_DECREF(state_dict);
    Py_DECREF(dict);
    return result;
}
static int __Pyx_UpdateUnpickledDict(PyObject *obj, PyObject *state, Py_ssize_t index) {
    Py_ssize_t state_size = __Pyx_PyTuple_GET_SIZE(state);
    #if !CYTHON_ASSUME_SAFE_SIZE
    if (unlikely(state_size == -1)) return -1;
    #endif
    if (state_size <= index) {
        return 0;
    }
    return __Pyx__UpdateUnpickledDict(obj, state, index);
}

/* FormatTypeName */
#if CYTHON_COMPILING_IN_LIMITED_API && __PYX_LIMITED_VERSION_HEX < 0x030d0000
static __Pyx_TypeName
__Pyx_PyType_GetFullyQualifiedName(PyTypeObject* tp)
{
    PyObject *module = NULL, *name = NULL, *result = NULL;
    #if __PYX_LIMITED_VERSION_HEX < 0x030b0000
    name = __Pyx_PyObject_GetAttrStr((PyObject *)tp,
                                               __pyx_mstate_global->__pyx_n_u_qualname);
    #else
    name = PyType_GetQualName(tp);
    #endif
    if (unlikely(name == NULL) || unlikely(!PyUnicode_Check(name))) goto bad;
    module = __Pyx_PyObject_GetAttrStr((PyObject *)tp,
                                               __pyx_mstate_global->__pyx_n_u_module);
    if (unlikely(module == NULL) || unlikely(!PyUnicode_Check(module))) goto bad;
    if (PyUnicode_CompareWithASCIIString(module, "builtins") == 0) {
        result = name;
        name = NULL;
        goto done;
    }
    result = PyUnicode_FromFormat("%U.%U", module, name);
    if (unlikely(result == NULL)) goto bad;
  done:
    Py_XDECREF(name);
    Py_XDECREF(module);
    return result;
  bad:
    PyErr_Clear();
    if (name) {
        result = name;
        name = NULL;
    } else {
        result = __Pyx_NewRef(__pyx_mstate_global->__pyx_kp_u__4);
    }
    goto done;
}
#endif

/* FastTypeChecks */
#if CYTHON_COMPILING_IN_CPYTHON
static int __Pyx_InBases(PyTypeObject *a, PyTypeObject *b) {
    while (a) {
        a = __Pyx_PyType_GetSlot(a, tp_base, PyTypeObject*);
        if (a == b)
            return 1;
    }
    return b == &PyBaseObject_Type;
}
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b) {
    PyObject *mro;
    if (a == b) return 1;
    mro = a->tp_mro;
    if (likely(mro)) {
        Py_ssize_t i, n;
        n = PyTuple_GET_SIZE(mro);
        for (i = 0; i < n; i++) {
            if (PyTuple_GET_ITEM(mro, i) == (PyObject *)b)
                return 1;
        }
        return 0;
    }
    return __Pyx_InBases(a, b);
}
static CYTHON_INLINE int __Pyx_IsAnySubtype2(PyTypeObject *cls, PyTypeObject *a, PyTypeObject *b) {
    PyObject *mro;
    if (cls == a || cls == b) return 1;
    mro = cls->tp_mro;
    if (likely(mro)) {
        Py_ssize_t i, n;
        n = PyTuple_GET_SIZE(mro);
        for (i = 0; i < n; i++) {
            PyObject *base = PyTuple_GET_ITEM(mro, i);
            if (base == (PyObject *)a || base == (PyObject *)b)
                return 1;
        }
        return 0;
    }
    return __Pyx_InBases(cls, a) || __Pyx_InBases(cls, b);
}
static CYTHON_INLINE int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject *exc_type2) {
    if (exc_type1) {
        return __Pyx_IsAnySubtype2((PyTypeObject*)err, (PyTypeObject*)exc_type1, (PyTypeObject*)exc_type2);
    } else {
        return __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type2);
    }
}
static int __Pyx_PyErr_GivenExceptionMatchesTuple(PyObject *exc_type, PyObject *tuple) {
    Py_ssize_t i, n;
    assert(PyExceptionClass_Check(exc_type));
    n = PyTuple_GET_SIZE(tuple);
    for (i=0; i<n; i++) {
        if (exc_type == PyTuple_GET_ITEM(tuple, i)) return 1;
    }
    for (i=0; i<n; i++) {
        PyObject *t = PyTuple_GET_ITEM(tuple, i);
        if (likely(PyExceptionClass_Check(t))) {
            if (__Pyx_inner_PyErr_GivenExceptionMatches2(exc_type, NULL, t)) return 1;
        } else {
        }
    }
    return 0;
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject* exc_type) {
    if (likely(err == exc_type)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        if (likely(PyExceptionClass_Check(exc_type))) {
            return __Pyx_inner_PyErr_GivenExceptionMatches2(err, NULL, exc_type);
        } else if (likely(PyTuple_Check(exc_type))) {
            return __Pyx_PyErr_GivenExceptionMatchesTuple(err, exc_type);
        } else {
        }
    }
    return PyErr_GivenExceptionMatches(err, exc_type);
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *exc_type1, PyObject *exc_type2) {
    assert(PyExceptionClass_Check(exc_type1));
    assert(PyExceptionClass_Check(exc_type2));
    if (likely(err == exc_type1 || err == exc_type2)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        return __Pyx_inner_PyErr_GivenExceptionMatches2(err, exc_type1, exc_type2);
    }
    return (PyErr_GivenExceptionMatches(err, exc_type1) || PyErr_GivenExceptionMatches(err, exc_type2));
}
#endif

/* GetRuntimeVersion */
#if __PYX_LIMITED_VERSION_HEX < 0x030b0000
void __Pyx_init_runtime_version(void) {
    if (__Pyx_cached_runtime_version == 0) {
        const char* rt_version = Py_GetVersion();
        unsigned long version = 0;
        unsigned long factor = 0x01000000UL;
        unsigned int digit = 0;
        int i = 0;
        while (factor) {
            while ('0' <= rt_version[i] && rt_version[i] <= '9') {
                digit = digit * 10 + (unsigned int) (rt_version[i] - '0');
                ++i;
            }
            version += factor * digit;
            if (rt_version[i] != '.')
                break;
            digit = 0;
            factor >>= 8;
            ++i;
        }
        __Pyx_cached_runtime_version = version;
    }
}
#endif
static unsigned long __Pyx_get_runtime_version(void) {
#if __PYX_LIMITED_VERSION_HEX >= 0x030b0000
    return Py_Version & ~0xFFUL;
#else
    return __Pyx_cached_runtime_version;
#endif
}

/* CheckBinaryVersion */
static int __Pyx_check_binary_version(unsigned long ct_version, unsigned long rt_version, int allow_newer) {
    const unsigned long MAJOR_MINOR = 0xFFFF0000UL;
    if ((rt_version & MAJOR_MINOR) == (ct_version & MAJOR_MINOR))
        return 0;
    if (likely(allow_newer && (rt_version & MAJOR_MINOR) > (ct_version & MAJOR_MINOR)))
        return 1;
    {
        char message[200];
        PyOS_snprintf(message, sizeof(message),
                      "compile time Python version %d.%d "
                      "of module '%.100s' "
                      "%s "
                      "runtime version %d.%d",
                       (int) (ct_version >> 24), (int) ((ct_version >> 16) & 0xFF),
                       __Pyx_MODULE_NAME,
                       (allow_newer) ? "was newer than" : "does not match",
                       (int) (rt_version >> 24), (int) ((rt_version >> 16) & 0xFF)
       );
        return PyErr_WarnEx(NULL, message, 1);
    }
}

/* NewCodeObj */
#if CYTHON_COMPILING_IN_LIMITED_API
    static PyObject* __Pyx__PyCode_New(int a, int p, int k, int l, int s, int f,
                                       PyObject *code, PyObject *c, PyObject* n, PyObject *v,
                                       PyObject *fv, PyObject *cell, PyObject* fn,
                                       PyObject *name, int fline, PyObject *lnos) {
        PyObject *exception_table = NULL;
        PyObject *types_module=NULL, *code_type=NULL, *result=NULL;
        #if __PYX_LIMITED_VERSION_HEX < 0x030b0000
        PyObject *version_info;
        PyObject *py_minor_version = NULL;
        #endif
        long minor_version = 0;
        PyObject *type, *value, *traceback;
        PyErr_Fetch(&type, &value, &traceback);
        #if __PYX_LIMITED_VERSION_HEX >= 0x030b0000
        minor_version = 11;
        #else
        if (!(version_info = PySys_GetObject("version_info"))) goto end;
        if (!(py_minor_version = PySequence_GetItem(version_info, 1))) goto end;
        minor_version = PyLong_AsLong(py_minor_version);
        Py_DECREF(py_minor_version);
        if (minor_version == -1 && PyErr_Occurred()) goto end;
        #endif
        if (!(types_module = PyImport_ImportModule("types"))) goto end;
        if (!(code_type = PyObject_GetAttrString(types_module, "CodeType"))) goto end;
        if (minor_version <= 7) {
            (void)p;
            result = PyObject_CallFunction(code_type, "iiiiiOOOOOOiOOO", a, k, l, s, f, code,
                          c, n, v, fn, name, fline, lnos, fv, cell);
        } else if (minor_version <= 10) {
            result = PyObject_CallFunction(code_type, "iiiiiiOOOOOOiOOO", a,p, k, l, s, f, code,
                          c, n, v, fn, name, fline, lnos, fv, cell);
        } else {
            if (!(exception_table = PyBytes_FromStringAndSize(NULL, 0))) goto end;
            result = PyObject_CallFunction(code_type, "iiiiiiOOOOOOOiOOOO", a,p, k, l, s, f, code,
                          c, n, v, fn, name, name, fline, lnos, exception_table, fv, cell);
        }
    end:
        Py_XDECREF(code_type);
        Py_XDECREF(exception_table);
        Py_XDECREF(types_module);
        if (type) {
            PyErr_Restore(type, value, traceback);
        }
        return result;
    }
#elif PY_VERSION_HEX >= 0x030B0000
  static PyCodeObject* __Pyx__PyCode_New(int a, int p, int k, int l, int s, int f,
                                         PyObject *code, PyObject *c, PyObject* n, PyObject *v,
                                         PyObject *fv, PyObject *cell, PyObject* fn,
                                         PyObject *name, int fline, PyObject *lnos) {
    PyCodeObject *result;
    result =
      #if PY_VERSION_HEX >= 0x030C0000
        PyUnstable_Code_NewWithPosOnlyArgs
      #else
        PyCode_NewWithPosOnlyArgs
      #endif
        (a, p, k, l, s, f, code, c, n, v, fv, cell, fn, name, name, fline, lnos, __pyx_mstate_global->__pyx_empty_bytes);
    #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030c00A1
    if (likely(result))
        result->_co_firsttraceable = 0;
    #endif
    return result;
  }
#elif !CYTHON_COMPILING_IN_PYPY
  #define __Pyx__PyCode_New(a, p, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_NewWithPosOnlyArgs(a, p, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
#else
  #define __Pyx__PyCode_New(a, p, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
#endif
static PyObject* __Pyx_PyCode_New(
        const __Pyx_PyCode_New_function_description descr,
        PyObject * const *varnames,
        PyObject *filename,
        PyObject *funcname,
        PyObject *line_table,
        PyObject *tuple_dedup_map
) {
    PyObject *code_obj = NULL, *varnames_tuple_dedup = NULL, *code_bytes = NULL;
    Py_ssize_t var_count = (Py_ssize_t) descr.nlocals;
    PyObject *varnames_tuple = PyTuple_New(var_count);
    if (unlikely(!varnames_tuple)) return NULL;
    for (Py_ssize_t i=0; i < var_count; i++) {
        Py_INCREF(varnames[i]);
        if (__Pyx_PyTuple_SET_ITEM(varnames_tuple, i, varnames[i]) != (0)) goto done;
    }
    #if CYTHON_COMPILING_IN_LIMITED_API
    varnames_tuple_dedup = PyDict_GetItem(tuple_dedup_map, varnames_tuple);
    if (!varnames_tuple_dedup) {
        if (unlikely(PyDict_SetItem(tuple_dedup_map, varnames_tuple, varnames_tuple) < 0)) goto done;
        varnames_tuple_dedup = varnames_tuple;
    }
    #else
    varnames_tuple_dedup = PyDict_SetDefault(tuple_dedup_map, varnames_tuple, varnames_tuple);
    if (unlikely(!varnames_tuple_dedup)) goto done;
    #endif
    #if CYTHON_AVOID_BORROWED_REFS
    Py_INCREF(varnames_tuple_dedup);
    #endif
    if (__PYX_LIMITED_VERSION_HEX >= (0x030b0000) && line_table != NULL && !CYTHON_COMPILING_IN_GRAAL) {
        Py_ssize_t line_table_length = __Pyx_PyBytes_GET_SIZE(line_table);
        #if !CYTHON_ASSUME_SAFE_SIZE
        if (unlikely(line_table_length == -1)) goto done;
        #endif
        Py_ssize_t code_len = (line_table_length * 2 + 4) & ~3LL;
        code_bytes = PyBytes_FromStringAndSize(NULL, code_len);
        if (unlikely(!code_bytes)) goto done;
        char* c_code_bytes = PyBytes_AsString(code_bytes);
        if (unlikely(!c_code_bytes)) goto done;
        memset(c_code_bytes, 0, (size_t) code_len);
    }
    code_obj = (PyObject*) __Pyx__PyCode_New(
        (int) descr.argcount,
        (int) descr.num_posonly_args,
        (int) descr.num_kwonly_args,
        (int) descr.nlocals,
        0,
        (int) descr.flags,
        code_bytes ? code_bytes : __pyx_mstate_global->__pyx_empty_bytes,
        __pyx_mstate_global->__pyx_empty_tuple,
        __pyx_mstate_global->__pyx_empty_tuple,
        varnames_tuple_dedup,
        __pyx_mstate_global->__pyx_empty_tuple,
        __pyx_mstate_global->__pyx_empty_tuple,
        filename,
        funcname,
        (int) descr.first_line,
        (__PYX_LIMITED_VERSION_HEX >= (0x030b0000) && line_table) ? line_table : __pyx_mstate_global->__pyx_empty_bytes
    );
done:
    Py_XDECREF(code_bytes);
    #if CYTHON_AVOID_BORROWED_REFS
    Py_XDECREF(varnames_tuple_dedup);
    #endif
    Py_DECREF(varnames_tuple);
    return code_obj;
}

/* DecompressString */
static PyObject *__Pyx_DecompressString(const char *s, Py_ssize_t length, int algo) {
    PyObject *module, *decompress, *compressed_bytes, *decompressed;
    const char* module_name = algo == 3 ? "compression.zstd" : algo == 2 ? "bz2" : "zlib";
    PyObject *methodname = PyUnicode_FromString("decompress");
    if (unlikely(!methodname)) return NULL;
    #if __PYX_LIMITED_VERSION_HEX >= 0x030e0000
    if (algo == 3) {
        PyObject *fromlist = Py_BuildValue("[O]", methodname);
        if (unlikely(!fromlist)) return NULL;
        module = PyImport_ImportModuleLevel("compression.zstd", NULL, NULL, fromlist, 0);
        Py_DECREF(fromlist);
    } else
    #endif
        module = PyImport_ImportModule(module_name);
    if (unlikely(!module)) goto import_failed;
    decompress = PyObject_GetAttr(module, methodname);
    if (unlikely(!decompress)) goto import_failed;
    {
        #ifdef __cplusplus
            char *memview_bytes = const_cast<char*>(s);
        #else
            #if defined(__clang__)
              #pragma clang diagnostic push
              #pragma clang diagnostic ignored "-Wcast-qual"
            #elif !defined(__INTEL_COMPILER) && defined(__GNUC__)
              #pragma GCC diagnostic push
              #pragma GCC diagnostic ignored "-Wcast-qual"
            #endif
            char *memview_bytes = (char*) s;
            #if defined(__clang__)
              #pragma clang diagnostic pop
            #elif !defined(__INTEL_COMPILER) && defined(__GNUC__)
              #pragma GCC diagnostic pop
            #endif
        #endif
        #if CYTHON_COMPILING_IN_LIMITED_API && !defined(PyBUF_READ)
        int memview_flags = 0x100;
        #else
        int memview_flags = PyBUF_READ;
        #endif
        compressed_bytes = PyMemoryView_FromMemory(memview_bytes, length, memview_flags);
    }
    if (unlikely(!compressed_bytes)) {
        Py_DECREF(decompress);
        goto bad;
    }
    decompressed = PyObject_CallFunctionObjArgs(decompress, compressed_bytes, NULL);
    Py_DECREF(compressed_bytes);
    Py_DECREF(decompress);
    Py_DECREF(module);
    Py_DECREF(methodname);
    return decompressed;
import_failed:
    PyErr_Format(PyExc_ImportError,
        "Failed to import '%.20s.decompress' - cannot initialise module strings. "
        "String compression was configured with the C macro 'CYTHON_COMPRESS_STRINGS=%d'.",
        module_name, algo);
bad:
    Py_XDECREF(module);
    Py_DECREF(methodname);
    return NULL;
}

#include <string.h>
static CYTHON_INLINE Py_ssize_t __Pyx_ssize_strlen(const char *s) {
    size_t len = strlen(s);
    if (unlikely(len > (size_t) PY_SSIZE_T_MAX)) {
        PyErr_SetString(PyExc_OverflowError, "byte string is too long");
        return -1;
    }
    return (Py_ssize_t) len;
}
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char* c_str) {
    Py_ssize_t len = __Pyx_ssize_strlen(c_str);
    if (unlikely(len < 0)) return NULL;
    return __Pyx_PyUnicode_FromStringAndSize(c_str, len);
}
static CYTHON_INLINE PyObject* __Pyx_PyByteArray_FromString(const char* c_str) {
    Py_ssize_t len = __Pyx_ssize_strlen(c_str);
    if (unlikely(len < 0)) return NULL;
    return PyByteArray_FromStringAndSize(c_str, len);
}
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject* o) {
    Py_ssize_t ignore;
    return __Pyx_PyObject_AsStringAndSize(o, &ignore);
}
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_UTF8
static CYTHON_INLINE const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    if (unlikely(__Pyx_PyUnicode_READY(o) == -1)) return NULL;
#if CYTHON_COMPILING_IN_LIMITED_API
    {
        const char* result;
        Py_ssize_t unicode_length;
        CYTHON_MAYBE_UNUSED_VAR(unicode_length); // only for __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
        #if __PYX_LIMITED_VERSION_HEX < 0x030A0000
        if (unlikely(PyArg_Parse(o, "s#", &result, length) < 0)) return NULL;
        #else
        result = PyUnicode_AsUTF8AndSize(o, length);
        #endif
        #if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
        unicode_length = PyUnicode_GetLength(o);
        if (unlikely(unicode_length < 0)) return NULL;
        if (unlikely(unicode_length != *length)) {
            PyUnicode_AsASCIIString(o);
            return NULL;
        }
        #endif
        return result;
    }
#else
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    if (likely(PyUnicode_IS_ASCII(o))) {
        *length = PyUnicode_GET_LENGTH(o);
        return PyUnicode_AsUTF8(o);
    } else {
        PyUnicode_AsASCIIString(o);
        return NULL;
    }
#else
    return PyUnicode_AsUTF8AndSize(o, length);
#endif
#endif
}
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_UTF8
    if (PyUnicode_Check(o)) {
        return __Pyx_PyUnicode_AsStringAndSize(o, length);
    } else
#endif
    if (PyByteArray_Check(o)) {
#if (CYTHON_ASSUME_SAFE_SIZE && CYTHON_ASSUME_SAFE_MACROS) || (CYTHON_COMPILING_IN_PYPY && (defined(PyByteArray_AS_STRING) && defined(PyByteArray_GET_SIZE)))
        *length = PyByteArray_GET_SIZE(o);
        return PyByteArray_AS_STRING(o);
#else
        *length = PyByteArray_Size(o);
        if (*length == -1) return NULL;
        return PyByteArray_AsString(o);
#endif
    } else
    {
        char* result;
        int r = PyBytes_AsStringAndSize(o, &result, length);
        if (unlikely(r < 0)) {
            return NULL;
        } else {
            return result;
        }
    }
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject* x) {
   int is_true = x == Py_True;
   if (is_true | (x == Py_False) | (x == Py_None)) return is_true;
   else return PyObject_IsTrue(x);
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrueAndDecref(PyObject* x) {
    int retval;
    if (unlikely(!x)) return -1;
    retval = __Pyx_PyObject_IsTrue(x);
    Py_DECREF(x);
    return retval;
}
static PyObject* __Pyx_PyNumber_LongWrongResultType(PyObject* result) {
    __Pyx_TypeName result_type_name = __Pyx_PyType_GetFullyQualifiedName(Py_TYPE(result));
    if (PyLong_Check(result)) {
        if (PyErr_WarnFormat(PyExc_DeprecationWarning, 1,
                "__int__ returned non-int (type " __Pyx_FMT_TYPENAME ").  "
                "The ability to return an instance of a strict subclass of int is deprecated, "
                "and may be removed in a future version of Python.",
                result_type_name)) {
            __Pyx_DECREF_TypeName(result_type_name);
            Py_DECREF(result);
            return NULL;
        }
        __Pyx_DECREF_TypeName(result_type_name);
        return result;
    }
    PyErr_Format(PyExc_TypeError,
                 "__int__ returned non-int (type " __Pyx_FMT_TYPENAME ")",
                 result_type_name);
    __Pyx_DECREF_TypeName(result_type_name);
    Py_DECREF(result);
    return NULL;
}
static CYTHON_INLINE PyObject* __Pyx_PyNumber_Long(PyObject* x) {
#if CYTHON_USE_TYPE_SLOTS
  PyNumberMethods *m;
#endif
  PyObject *res = NULL;
  if (likely(PyLong_Check(x)))
      return __Pyx_NewRef(x);
#if CYTHON_USE_TYPE_SLOTS
  m = Py_TYPE(x)->tp_as_number;
  if (likely(m && m->nb_int)) {
      res = m->nb_int(x);
  }
#else
  if (!PyBytes_CheckExact(x) && !PyUnicode_CheckExact(x)) {
      res = PyNumber_Long(x);
  }
#endif
  if (likely(res)) {
      if (unlikely(!PyLong_CheckExact(res))) {
          return __Pyx_PyNumber_LongWrongResultType(res);
      }
  }
  else if (!PyErr_Occurred()) {
      PyErr_SetString(PyExc_TypeError,
                      "an integer is required");
  }
  return res;
}
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject* b) {
  Py_ssize_t ival;
  PyObject *x;
  if (likely(PyLong_CheckExact(b))) {
    #if CYTHON_USE_PYLONG_INTERNALS
    if (likely(__Pyx_PyLong_IsCompact(b))) {
        return __Pyx_PyLong_CompactValue(b);
    } else {
      const digit* digits = __Pyx_PyLong_Digits(b);
      const Py_ssize_t size = __Pyx_PyLong_SignedDigitCount(b);
      switch (size) {
         case 2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
      }
    }
    #endif
    return PyLong_AsSsize_t(b);
  }
  x = PyNumber_Index(b);
  if (!x) return -1;
  ival = PyLong_AsSsize_t(x);
  Py_DECREF(x);
  return ival;
}
static CYTHON_INLINE Py_hash_t __Pyx_PyIndex_AsHash_t(PyObject* o) {
  if (sizeof(Py_hash_t) == sizeof(Py_ssize_t)) {
    return (Py_hash_t) __Pyx_PyIndex_AsSsize_t(o);
  } else {
    Py_ssize_t ival;
    PyObject *x;
    x = PyNumber_Index(o);
    if (!x) return -1;
    ival = PyLong_AsLong(x);
    Py_DECREF(x);
    return ival;
  }
}
static CYTHON_INLINE PyObject *__Pyx_Owned_Py_None(int b) {
    CYTHON_UNUSED_VAR(b);
    return __Pyx_NewRef(Py_None);
}
static CYTHON_INLINE PyObject * __Pyx_PyBool_FromLong(long b) {
  return __Pyx_NewRef(b ? Py_True: Py_False);
}
static CYTHON_INLINE PyObject * __Pyx_PyLong_FromSize_t(size_t ival) {
    return PyLong_FromSize_t(ival);
}


/* MultiPhaseInitModuleState */
#if CYTHON_PEP489_MULTI_PHASE_INIT && CYTHON_USE_MODULE_STATE
#ifndef CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE
#if (CYTHON_COMPILING_IN_LIMITED_API || PY_VERSION_HEX >= 0x030C0000)
  #define CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE 1
#else
  #define CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE 0
#endif
#endif
#if CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE && !CYTHON_ATOMICS
#error "Module state with PEP489 requires atomics. Currently that's one of\
 C11, C++11, gcc atomic intrinsics or MSVC atomic intrinsics"
#endif
#if !CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE
#define __Pyx_ModuleStateLookup_Lock()
#define __Pyx_ModuleStateLookup_Unlock()
#elif !CYTHON_COMPILING_IN_LIMITED_API && PY_VERSION_HEX >= 0x030d0000
static PyMutex __Pyx_ModuleStateLookup_mutex = {0};
#define __Pyx_ModuleStateLookup_Lock() PyMutex_Lock(&__Pyx_ModuleStateLookup_mutex)
#define __Pyx_ModuleStateLookup_Unlock() PyMutex_Unlock(&__Pyx_ModuleStateLookup_mutex)
#elif defined(__cplusplus) && __cplusplus >= 201103L
#include <mutex>
static std::mutex __Pyx_ModuleStateLookup_mutex;
#define __Pyx_ModuleStateLookup_Lock() __Pyx_ModuleStateLookup_mutex.lock()
#define __Pyx_ModuleStateLookup_Unlock() __Pyx_ModuleStateLookup_mutex.unlock()
#elif defined(__STDC_VERSION__) && (__STDC_VERSION__ > 201112L) && !defined(__STDC_NO_THREADS__)
#include <threads.h>
static mtx_t __Pyx_ModuleStateLookup_mutex;
static once_flag __Pyx_ModuleStateLookup_mutex_once_flag = ONCE_FLAG_INIT;
static void __Pyx_ModuleStateLookup_initialize_mutex(void) {
    mtx_init(&__Pyx_ModuleStateLookup_mutex, mtx_plain);
}
#define __Pyx_ModuleStateLookup_Lock()\
  call_once(&__Pyx_ModuleStateLookup_mutex_once_flag, __Pyx_ModuleStateLookup_initialize_mutex);\
  mtx_lock(&__Pyx_ModuleStateLookup_mutex)
#define __Pyx_ModuleStateLookup_Unlock() mtx_unlock(&__Pyx_ModuleStateLookup_mutex)
#elif defined(HAVE_PTHREAD_H)
#include <pthread.h>
static pthread_mutex_t __Pyx_ModuleStateLookup_mutex = PTHREAD_MUTEX_INITIALIZER;
#define __Pyx_ModuleStateLookup_Lock() pthread_mutex_lock(&__Pyx_ModuleStateLookup_mutex)
#define __Pyx_ModuleStateLookup_Unlock() pthread_mutex_unlock(&__Pyx_ModuleStateLookup_mutex)
#elif defined(_WIN32)
#include <Windows.h>  // synchapi.h on its own doesn't work
static SRWLOCK __Pyx_ModuleStateLookup_mutex = SRWLOCK_INIT;
#define __Pyx_ModuleStateLookup_Lock() AcquireSRWLockExclusive(&__Pyx_ModuleStateLookup_mutex)
#define __Pyx_ModuleStateLookup_Unlock() ReleaseSRWLockExclusive(&__Pyx_ModuleStateLookup_mutex)
#else
#error "No suitable lock available for CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE.\
 Requires C standard >= C11, or C++ standard >= C++11,\
 or pthreads, or the Windows 32 API, or Python >= 3.13."
#endif
typedef struct {
    int64_t id;
    PyObject *module;
} __Pyx_InterpreterIdAndModule;
typedef struct {
    char interpreter_id_as_index;
    Py_ssize_t count;
    Py_ssize_t allocated;
    __Pyx_InterpreterIdAndModule table[1];
} __Pyx_ModuleStateLookupData;
#define __PYX_MODULE_STATE_LOOKUP_SMALL_SIZE 32
#if CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE
static __pyx_atomic_int_type __Pyx_ModuleStateLookup_read_counter = 0;
#endif
#if CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE
static __pyx_atomic_ptr_type __Pyx_ModuleStateLookup_data = 0;
#else
static __Pyx_ModuleStateLookupData* __Pyx_ModuleStateLookup_data = NULL;
#endif
static __Pyx_InterpreterIdAndModule* __Pyx_State_FindModuleStateLookupTableLowerBound(
        __Pyx_InterpreterIdAndModule* table,
        Py_ssize_t count,
        int64_t interpreterId) {
    __Pyx_InterpreterIdAndModule* begin = table;
    __Pyx_InterpreterIdAndModule* end = begin + count;
    if (begin->id == interpreterId) {
        return begin;
    }
    while ((end - begin) > __PYX_MODULE_STATE_LOOKUP_SMALL_SIZE) {
        __Pyx_InterpreterIdAndModule* halfway = begin + (end - begin)/2;
        if (halfway->id == interpreterId) {
            return halfway;
        }
        if (halfway->id < interpreterId) {
            begin = halfway;
        } else {
            end = halfway;
        }
    }
    for (; begin < end; ++begin) {
        if (begin->id >= interpreterId) return begin;
    }
    return begin;
}
static PyObject *__Pyx_State_FindModule(CYTHON_UNUSED void* dummy) {
    int64_t interpreter_id = PyInterpreterState_GetID(__Pyx_PyInterpreterState_Get());
    if (interpreter_id == -1) return NULL;
#if CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE
    __Pyx_ModuleStateLookupData* data = (__Pyx_ModuleStateLookupData*)__pyx_atomic_pointer_load_relaxed(&__Pyx_ModuleStateLookup_data);
    {
        __pyx_atomic_incr_acq_rel(&__Pyx_ModuleStateLookup_read_counter);
        if (likely(data)) {
            __Pyx_ModuleStateLookupData* new_data = (__Pyx_ModuleStateLookupData*)__pyx_atomic_pointer_load_acquire(&__Pyx_ModuleStateLookup_data);
            if (likely(data == new_data)) {
                goto read_finished;
            }
        }
        __pyx_atomic_decr_acq_rel(&__Pyx_ModuleStateLookup_read_counter);
        __Pyx_ModuleStateLookup_Lock();
        __pyx_atomic_incr_relaxed(&__Pyx_ModuleStateLookup_read_counter);
        data = (__Pyx_ModuleStateLookupData*)__pyx_atomic_pointer_load_relaxed(&__Pyx_ModuleStateLookup_data);
        __Pyx_ModuleStateLookup_Unlock();
    }
  read_finished:;
#else
    __Pyx_ModuleStateLookupData* data = __Pyx_ModuleStateLookup_data;
#endif
    __Pyx_InterpreterIdAndModule* found = NULL;
    if (unlikely(!data)) goto end;
    if (data->interpreter_id_as_index) {
        if (interpreter_id < data->count) {
            found = data->table+interpreter_id;
        }
    } else {
        found = __Pyx_State_FindModuleStateLookupTableLowerBound(
            data->table, data->count, interpreter_id);
    }
  end:
    {
        PyObject *result=NULL;
        if (found && found->id == interpreter_id) {
            result = found->module;
        }
#if CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE
        __pyx_atomic_decr_acq_rel(&__Pyx_ModuleStateLookup_read_counter);
#endif
        return result;
    }
}
#if CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE
static void __Pyx_ModuleStateLookup_wait_until_no_readers(void) {
    while (__pyx_atomic_load(&__Pyx_ModuleStateLookup_read_counter) != 0);
}
#else
#define __Pyx_ModuleStateLookup_wait_until_no_readers()
#endif
static int __Pyx_State_AddModuleInterpIdAsIndex(__Pyx_ModuleStateLookupData **old_data, PyObject* module, int64_t interpreter_id) {
    Py_ssize_t to_allocate = (*old_data)->allocated;
    while (to_allocate <= interpreter_id) {
        if (to_allocate == 0) to_allocate = 1;
        else to_allocate *= 2;
    }
    __Pyx_ModuleStateLookupData *new_data = *old_data;
    if (to_allocate != (*old_data)->allocated) {
         new_data = (__Pyx_ModuleStateLookupData *)realloc(
            *old_data,
            sizeof(__Pyx_ModuleStateLookupData)+(to_allocate-1)*sizeof(__Pyx_InterpreterIdAndModule));
        if (!new_data) {
            PyErr_NoMemory();
            return -1;
        }
        for (Py_ssize_t i = new_data->allocated; i < to_allocate; ++i) {
            new_data->table[i].id = i;
            new_data->table[i].module = NULL;
        }
        new_data->allocated = to_allocate;
    }
    new_data->table[interpreter_id].module = module;
    if (new_data->count < interpreter_id+1) {
        new_data->count = interpreter_id+1;
    }
    *old_data = new_data;
    return 0;
}
static void __Pyx_State_ConvertFromInterpIdAsIndex(__Pyx_ModuleStateLookupData *data) {
    __Pyx_InterpreterIdAndModule *read = data->table;
    __Pyx_InterpreterIdAndModule *write = data->table;
    __Pyx_InterpreterIdAndModule *end = read + data->count;
    for (; read<end; ++read) {
        if (read->module) {
            write->id = read->id;
            write->module = read->module;
            ++write;
        }
    }
    data->count = write - data->table;
    for (; write<end; ++write) {
        write->id = 0;
        write->module = NULL;
    }
    data->interpreter_id_as_index = 0;
}
static int __Pyx_State_AddModule(PyObject* module, CYTHON_UNUSED void* dummy) {
    int64_t interpreter_id = PyInterpreterState_GetID(__Pyx_PyInterpreterState_Get());
    if (interpreter_id == -1) return -1;
    int result = 0;
    __Pyx_ModuleStateLookup_Lock();
#if CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE
    __Pyx_ModuleStateLookupData *old_data = (__Pyx_ModuleStateLookupData *)
            __pyx_atomic_pointer_exchange(&__Pyx_ModuleStateLookup_data, 0);
#else
    __Pyx_ModuleStateLookupData *old_data = __Pyx_ModuleStateLookup_data;
#endif
    __Pyx_ModuleStateLookupData *new_data = old_data;
    if (!new_data) {
        new_data = (__Pyx_ModuleStateLookupData *)calloc(1, sizeof(__Pyx_ModuleStateLookupData));
        if (!new_data) {
            result = -1;
            PyErr_NoMemory();
            goto end;
        }
        new_data->allocated = 1;
        new_data->interpreter_id_as_index = 1;
    }
    __Pyx_ModuleStateLookup_wait_until_no_readers();
    if (new_data->interpreter_id_as_index) {
        if (interpreter_id < __PYX_MODULE_STATE_LOOKUP_SMALL_SIZE) {
            result = __Pyx_State_AddModuleInterpIdAsIndex(&new_data, module, interpreter_id);
            goto end;
        }
        __Pyx_State_ConvertFromInterpIdAsIndex(new_data);
    }
    {
        Py_ssize_t insert_at = 0;
        {
            __Pyx_InterpreterIdAndModule* lower_bound = __Pyx_State_FindModuleStateLookupTableLowerBound(
                new_data->table, new_data->count, interpreter_id);
            assert(lower_bound);
            insert_at = lower_bound - new_data->table;
            if (unlikely(insert_at < new_data->count && lower_bound->id == interpreter_id)) {
                lower_bound->module = module;
                goto end;  // already in table, nothing more to do
            }
        }
        if (new_data->count+1 >= new_data->allocated) {
            Py_ssize_t to_allocate = (new_data->count+1)*2;
            new_data =
                (__Pyx_ModuleStateLookupData*)realloc(
                    new_data,
                    sizeof(__Pyx_ModuleStateLookupData) +
                    (to_allocate-1)*sizeof(__Pyx_InterpreterIdAndModule));
            if (!new_data) {
                result = -1;
                new_data = old_data;
                PyErr_NoMemory();
                goto end;
            }
            new_data->allocated = to_allocate;
        }
        ++new_data->count;
        int64_t last_id = interpreter_id;
        PyObject *last_module = module;
        for (Py_ssize_t i=insert_at; i<new_data->count; ++i) {
            int64_t current_id = new_data->table[i].id;
            new_data->table[i].id = last_id;
            last_id = current_id;
            PyObject *current_module = new_data->table[i].module;
            new_data->table[i].module = last_module;
            last_module = current_module;
        }
    }
  end:
#if CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE
    __pyx_atomic_pointer_exchange(&__Pyx_ModuleStateLookup_data, new_data);
#else
    __Pyx_ModuleStateLookup_data = new_data;
#endif
    __Pyx_ModuleStateLookup_Unlock();
    return result;
}
static int __Pyx_State_RemoveModule(CYTHON_UNUSED void* dummy) {
    int64_t interpreter_id = PyInterpreterState_GetID(__Pyx_PyInterpreterState_Get());
    if (interpreter_id == -1) return -1;
    __Pyx_ModuleStateLookup_Lock();
#if CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE
    __Pyx_ModuleStateLookupData *data = (__Pyx_ModuleStateLookupData *)
            __pyx_atomic_pointer_exchange(&__Pyx_ModuleStateLookup_data, 0);
#else
    __Pyx_ModuleStateLookupData *data = __Pyx_ModuleStateLookup_data;
#endif
    if (data->interpreter_id_as_index) {
        if (interpreter_id < data->count) {
            data->table[interpreter_id].module = NULL;
        }
        goto done;
    }
    {
        __Pyx_ModuleStateLookup_wait_until_no_readers();
        __Pyx_InterpreterIdAndModule* lower_bound = __Pyx_State_FindModuleStateLookupTableLowerBound(
            data->table, data->count, interpreter_id);
        if (!lower_bound) goto done;
        if (lower_bound->id != interpreter_id) goto done;
        __Pyx_InterpreterIdAndModule *end = data->table+data->count;
        for (;lower_bound<end-1; ++lower_bound) {
            lower_bound->id = (lower_bound+1)->id;
            lower_bound->module = (lower_bound+1)->module;
        }
    }
    --data->count;
    if (data->count == 0) {
        free(data);
        data = NULL;
    }
  done:
#if CYTHON_MODULE_STATE_LOOKUP_THREAD_SAFE
    __pyx_atomic_pointer_exchange(&__Pyx_ModuleStateLookup_data, data);
#else
    __Pyx_ModuleStateLookup_data = data;
#endif
    __Pyx_ModuleStateLookup_Unlock();
    return 0;
}
#endif

/* #### Code section: utility_code_pragmas_end ### */
#ifdef _MSC_VER
#pragma warning( pop )
#endif



/* #### Code section: end ### */
#endif /* Py_PYTHON_H */
